/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: a_bone.cc,v 1.17 2009/01/01 07:46:43 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/a_bone.h"

#include "libochusha/bulletin_board.h"
#include "libochusha/bbs_thread.h"
#include "libochusha/hashtable.h"
#include "libochusha/misc_utils.h"

#include <string>

#include <zlib.h>


using namespace std;
namespace Ochusha
{

ABone::ABone()
  : a_bone_by_name(false),
    a_bone_by_name_pattern(NULL),
    a_bone_by_mail(false),
    a_bone_by_mail_pattern(NULL),
    a_bone_by_id(false),
    a_bone_by_id_pattern(NULL),
    a_bone_by_content(false),
    a_bone_by_content_pattern(NULL)
{
}


ABone::~ABone()
{
  if (a_bone_by_name_pattern != NULL)
    free(a_bone_by_name_pattern);
  if (a_bone_by_mail_pattern != NULL)
    free(a_bone_by_mail_pattern);
  if (a_bone_by_id_pattern != NULL)
    free(a_bone_by_id_pattern);
  if (a_bone_by_content_pattern != NULL)
    free(a_bone_by_content_pattern);
}


void
ABone::set_name_pattern(const char *pattern)
{
  if (pattern != NULL && *pattern != '\0'
      && !a_bone_by_name_regexp.compile(pattern))
    {
      if (a_bone_by_name_pattern != NULL)
	a_bone_by_name_regexp.compile(a_bone_by_name_pattern);
      return;
    }

  if (a_bone_by_name_pattern != NULL)
    free(a_bone_by_name_pattern);
  a_bone_by_name_pattern
    = (pattern != NULL && *pattern != '\0') ? strdup(pattern) : NULL;
}


void
ABone::set_mail_pattern(const char *pattern)
{
  if (pattern != NULL && *pattern != '\0'
      && !a_bone_by_mail_regexp.compile(pattern))
    {
      if (a_bone_by_mail_pattern != NULL)
	a_bone_by_mail_regexp.compile(a_bone_by_mail_pattern);
      return;
    }

  if (a_bone_by_mail_pattern != NULL)
    free(a_bone_by_mail_pattern);
  a_bone_by_mail_pattern
    = (pattern != NULL && *pattern != '\0') ? strdup(pattern) : NULL;
}


void
ABone::set_id_pattern(const char *pattern)
{
  if (pattern != NULL && *pattern != '\0'
      && !a_bone_by_id_regexp.compile(pattern))
    {
      if (a_bone_by_id_pattern != NULL)
	a_bone_by_id_regexp.compile(a_bone_by_id_pattern);
      return;
    }

  if (a_bone_by_id_pattern != NULL)
    free(a_bone_by_id_pattern);
  a_bone_by_id_pattern
    = (pattern != NULL && *pattern != '\0') ? strdup(pattern) : NULL;
}


void
ABone::set_content_pattern(const char *pattern)
{
  if (pattern != NULL && *pattern != '\0'
      && !a_bone_by_content_regexp.compile(pattern))
    {
      if (a_bone_by_content_pattern != NULL)
	a_bone_by_content_regexp.compile(a_bone_by_content_pattern);
      return;
    }

  if (a_bone_by_content_pattern != NULL)
    free(a_bone_by_content_pattern);
  a_bone_by_content_pattern
    = (pattern != NULL && *pattern != '\0') ? strdup(pattern) : NULL;
}


bool
ABone::is_a_boned(const Response &response)
{
  return (a_bone_by_name && a_bone_by_name_regexp.match(response.get_name(), -1))
    || (a_bone_by_mail && a_bone_by_mail_regexp.match(response.get_mailto(), -1))
    || (a_bone_by_id && a_bone_by_id_regexp.match(response.get_id(), -1))
    || (a_bone_by_content && a_bone_by_content_regexp.match(response.get_message(), -1));
}


#define OUTPUT_ATTRIBUTE_BOOLEAN(gzfile, attribute, ind)		\
  do									\
    {									\
      if (attribute)							\
	gzprintf(gzfile,						\
		 "%s<attribute name=\"" #attribute "\">\n"		\
		 "%s  <boolean val=\"true\"/>\n"			\
		 "%s</attribute>\n", ind, ind, ind);			\
    }									\
  while (0)

#define OUTPUT_ATTRIBUTE_INT(gzfile, attribute, ind)			\
  do									\
    {									\
      if (attribute)							\
	gzprintf(gzfile,						\
		 "%s<attribute name=\"" #attribute "\">\n"		\
		 "%s  <int val=\"%d\"/>\n"				\
		 "%s</attribute>\n", ind, ind, attribute, ind);		\
    }									\
  while (0)

#define OUTPUT_ATTRIBUTE_STRING(gzfile, attribute, ind)			\
  do									\
    {									\
      if (attribute != NULL)						\
	{								\
	  string text;							\
	  gzprintf(gzfile,						\
		   "%s<attribute name=\"" #attribute "\">\n"		\
		   "%s  <string>%s</string>\n"				\
		   "%s</attribute>\n", ind, ind,			\
		   append_markup_escaped_text(text, attribute).c_str(),	\
		   ind);						\
	}								\
    }									\
  while (0)


void
ABone::write_xml_elements(gzFile file, const char *indent) const
{
  OUTPUT_ATTRIBUTE_BOOLEAN(file, a_bone_by_name, indent);
  OUTPUT_ATTRIBUTE_STRING(file, a_bone_by_name_pattern, indent);
  OUTPUT_ATTRIBUTE_BOOLEAN(file, a_bone_by_mail, indent);
  OUTPUT_ATTRIBUTE_STRING(file, a_bone_by_mail_pattern, indent);
  OUTPUT_ATTRIBUTE_BOOLEAN(file, a_bone_by_id, indent);
  OUTPUT_ATTRIBUTE_STRING(file, a_bone_by_id_pattern, indent);
  OUTPUT_ATTRIBUTE_BOOLEAN(file, a_bone_by_content, indent);
  OUTPUT_ATTRIBUTE_STRING(file, a_bone_by_content_pattern, indent);
}


void
ABone::read_xml_elements(AttributeTable &attributes)
{
  a_bone_by_name = attributes.get_bool("a_bone_by_name");
  set_name_pattern(attributes.get_string("a_bone_by_name_pattern"));

  a_bone_by_mail = attributes.get_bool("a_bone_by_mail");
  set_mail_pattern(attributes.get_string("a_bone_by_mail_pattern"));

  a_bone_by_id = attributes.get_bool("a_bone_by_id");
  set_id_pattern(attributes.get_string("a_bone_by_id_pattern"));

  a_bone_by_content = attributes.get_bool("a_bone_by_content");
  set_content_pattern(attributes.get_string("a_bone_by_content_pattern"));
}

}
