/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: base_utils.h,v 1.7 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BASE_UTILS_H_
#define _LIBOCHUSHA_BASE_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/buffer.h"
#include "libochusha/ref_count.h"


#include <cstddef>


using namespace std;
namespace Ochusha
{

  class Base64Encoder : public Buffer
  {
  public:
    static const char *base64;
    static const char *base64url;


  private:
    const char *alphabet;
    BufferHandle result_buffer;
    char pending_buffer[2];
    size_t pending_length;

    bool append_result(const char *data, size_t length, int lock_id);


  public:
    Base64Encoder(size_t buffer_len = DEFAULT_BUFFER_LENGTH,
		  const char *alphabet = base64);
    Base64Encoder(int fd, bool gzip, const char *alphabet = base64);
    Base64Encoder(Buffer *buffer, const char *alphabet = base64);
    virtual ~Base64Encoder();

    virtual bool fix(int lock_id);
    virtual bool clear(int lock_id);
    virtual bool append_data(const char *data, size_t length, int lock_id);

    static char *encode_text_to_base64(const char *text);
    static char *encode_text_to_base64url(const char *text);
  };


  class Base64Decoder : public Buffer
  {
  public:
    static const unsigned char base64[];
    static const unsigned char base64url[];


  private:
    const unsigned char *dictionary;
    BufferHandle result_buffer;
    char pending_buffer[3];
    size_t pending_length;
    bool ignore_garbage;

    bool append_result(const char *data, size_t length, int lock_id);
    int collect_word(const unsigned char *src, const unsigned char *tail,
		     unsigned char dst[]);


  public:
    Base64Decoder(size_t buffer_len = DEFAULT_BUFFER_LENGTH,
		  const unsigned char *dictionary = base64);
    Base64Decoder(int fd, bool gzip, const unsigned char *dictionary = base64);
    Base64Decoder(Buffer *buffer, const unsigned char *alphabet = base64);
    virtual ~Base64Decoder();

    inline void set_ignore_garbage(bool value) { ignore_garbage = value; }
    inline bool get_ignore_garbage() { return ignore_garbage; }

    virtual bool fix(int lock_id);
    virtual bool clear(int lock_id);
    virtual bool append_data(const char *data, size_t length, int lock_id);

    static char *decode_text_from_base64(const char *text);
    static char *decode_text_from_base64url(const char *text);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BASE_UTILS_H_ */
