/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_2ch_utils.cc,v 1.61 2010/08/15 15:36:11 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/bbs_2ch_utils.h"

#include "libochusha/bbs_thread.h"
#include "libochusha/buffer.h"
#include "libochusha/bulletin_board.h"
#include "libochusha/configuration.h"
#include "libochusha/constants.h"
#include "libochusha/html_utils.h"
#include "libochusha/http_cookies.h"
#include "libochusha/http_headers.h"
#include "libochusha/misc_utils.h"
#include "libochusha/network_agent.h"
#include "libochusha/regexp.h"
#include "libochusha/text_utils.h"
#include "libochusha/thread_pool.h"
#include "libochusha/uri_utils.h"

#include <cassert>
#include <climits>
#include <cstring>

#ifdef HAVE_TIME_H
#include <time.h>
#endif


//#define SAVE_SUBJECT_TXT	1
//#define ENABLE_BENCHMARKING	1
//#define VERBOSE_DESTRUCTOR	1


#ifdef ENABLE_BENCHMARKING
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  undef ENABLE_BENCHMARKING
# endif
#endif


namespace Ochusha
{


SubjectTxtParser::SubjectTxtParser(BulletinBoard *board, Configuration &config,
				   Repository &repository, NetworkAgent *agent,
				   const char *encoding, const char *regexp)
  : ThreadlistUpdater(board, config, repository, agent),
    iconv_buffer(IconvBuffer::UTF8_ENCODING, encoding),
    regexp(regexp, encoding),
    buffer(NULL),
    status_code(0),
    offset(0),
    number_of_threads(0)
{
  agent->prepare_request.connect(this, &SubjectTxtParser::prepare_request_cb);
  agent->response_header_received
    .connect(this, &SubjectTxtParser::response_header_received_cb);
  agent->access_progressed
    .connect(this, &SubjectTxtParser::access_progressed_cb);
  agent->access_finished.connect(this, &SubjectTxtParser::access_finished_cb);
  agent->access_failed.connect(this, &SubjectTxtParser::access_failed_cb);
  agent->access_terminated
    .connect(this, &SubjectTxtParser::access_terminated_cb);

  char url[PATH_MAX];
  if (base_uri.is_valid()
      && (snprintf(url, PATH_MAX, "%ssubject.txt", board->get_base_url())
	  < PATH_MAX))
    {
      buffer = new Buffer();
      agent->get(url, buffer, this);
    }
  else
    {
      // ɤˤʤ̤
      recover_threadlist();
      finalize();
    }
}


SubjectTxtParser::~SubjectTxtParser()
{
#ifndef NDEBUG
  NetworkAgent *ag = agent;
  assert(ag == NULL);
#endif
#ifdef VERBOSE_DESTRUCTOR
  cerr << "SubjectTxtParser destructed\n";
#endif
}


void
SubjectTxtParser::prepare_request_cb(NetworkAgent *agent, RefCount *data)
{
  if (board->get_last_modified() != NULL)
    {
      // If-Modified-Sinceդǥ
      agent->set_header(HTTPHeader::If_Modified_Since,
			board->get_last_modified());
    }
  agent->set_header(HTTPHeader::Accept_Encoding, "deflate, gzip");
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_USER_AGENT);

  if (config.get_enable_proxy_to_read())
    agent->set_proxy_uri(config.get_proxy_uri_to_read());
}


void
SubjectTxtParser::response_header_received_cb(NetworkAgent *agent,
					      RefCount *data)
{
  status_code = agent->get_status_code();
  // status_code301,302,403,404äꡢ
  // Content-Lengthϰžθ롣
  // PollEventManagerNetworkAgentο񤤤ȤŬʤȤޤ
  // ³ͥåȥǤڤΤݤʤΤǡ
  // žкϥĤޤԤġ
}


void
SubjectTxtParser::access_progressed_cb(NetworkAgent *agent, size_t data_read,
				       uintmax_t total_len, RefCount *data)
{
  if (status_code != 200)
    return;	// žäݤȤɤ᤿Τ̵뤷Ƥ

  // Υߥ󥰤subject.txtϤ롣
  // PollEventManagerpollingåɤƤФ뤳ȤꤵΤǡ
  // ;ʬʤȤϤʤ
  // buffer³鸽ɤϰϤΥǤƼ̤
  // board->lookup_bbs_thread_by_id()֤äƤBBSThread֥
  // threadˤĤthread->alive = trueȤNULLξˤϿ
  // BBSThread֥Ȥ롣
  // ˡupdated_list.push_back(thread)Ȥ롣
  // ɤʬβϤλġupdated_listǤˤϡ
  // state_changedʥȯԤ롣
  int last_number_of_threads = number_of_threads;
  Buffer *buffer = agent->get_buffer();
  RWLock *rwlock = buffer->get_lock();
  int buf_lock_id = rwlock->rdlock();	// Ф˥֥åʤ

  int list_lock_id = lock->wrlock();	// ֥å뤫ġ
  const char *buf_top = buffer->get_buffer(buf_lock_id);
  if (buf_top != NULL)
    {
      TextCanonicalizer canonicalizer(iconv_buffer);
      const char *cur_pos = buf_top + offset;
      size_t rest_len = data_read - offset;
      const char *eol_pos;
      char id_buf[PATH_MAX];

      while (rest_len > 0
	     && (eol_pos = static_cast<const char *>(memchr(cur_pos, '\n',
							    rest_len))) != NULL)
	{
	  if (regexp.match(cur_pos, eol_pos - cur_pos))
	    {
	      char *id;
	      if (regexp.match_len(1) < PATH_MAX)
		{
		  memcpy(id_buf, regexp.match_begin(1), regexp.match_len(1));
		  id_buf[regexp.match_len(1)] = '\0';
		  id = id_buf;
		}
	      else
		id = strndup(regexp.match_begin(1), regexp.match_len(1));

	      BBSThread *thread = board->lookup_bbs_thread_by_id(id);
	      if (thread == NULL)
		{
		  canonicalizer.clear();
		  const char *title
		    = canonicalizer.canon(regexp.match_begin(2),
					  regexp.match_len(2));
		  thread = board->thread_new(id, title);
		}
	      else
		{
		  const char *title = thread->get_title();
		  if (title == NULL || title[0] == '\0')
		    {
		      canonicalizer.clear();
		      title = canonicalizer.canon(regexp.match_begin(2),
						  regexp.match_len(2));
		      thread->set_title(title);
		    }
		}

	      if (!get_alive(thread))
		{
		  int n_res = parse_int(regexp.match_begin(3),
					regexp.match_len(3));
		  thread->set_number_of_responses_on_server(n_res);
		  thread->set_rank(++number_of_threads);
		  make_alive(thread);
		  thread->set_dat_dropped(false);
		  thread->set_grave(NULL);
		  updated_list.push_back(thread);
		}

	      if (id != id_buf)
		free(id);
	    }
	  offset = eol_pos + 1 - buf_top;
	  cur_pos = eol_pos + 1;
	  rest_len = data_read - offset;
	}
    }
  lock->unlock(list_lock_id);
  rwlock->unlock(buf_lock_id);

  if (last_number_of_threads < number_of_threads)
    state_changed();
}


void
SubjectTxtParser::access_finished_cb(NetworkAgent *agent, RefCount *data)
{
  if (status_code == 304)
    {
      // ʤ
      recover_threadlist();
      finalize();
      state_changed();
      return;
    }

  if (status_code == 200 && buffer->get_length0() > 0)
    {
      // λ
      update_threadlist();
      finalize();
      state_changed();

      board->set_last_modified(agent->get_header(HTTPHeader::Last_Modified));
      board->set_date(agent->get_header(HTTPHeader::Date));

#ifdef SAVE_SUBJECT_TXT
      char tmp_buf[PATH_MAX];
      if (snprintf(tmp_buf, PATH_MAX, "%ssubject.txt", board->get_base_url())
	  < PATH_MAX)
	{
	  // å¸ǤϤ⤦ɤळȤϤʤġġ
	  int fd = repository.get_bbs_data().open_file(tmp_buf, O_WRONLY | O_TRUNC | O_CREAT);
	  if (fd >= 0)
	    agent->save_buffer(fd);
	}
      // ĤǤ˼¸
      board->write_threadlist_xml(repository);
#endif
      return;
    }

  if (status_code == 200 && buffer->get_length0() == 0)
    {
      // ǶΣͤǤϻž˰žλsubject.txtä
      // Υե뤬֤Ƥ롣
      // ¿ʬapacheΥ顼404ޤ뤳ȤƤΤȤ

      // ʤΤǡʾϻžȸȤˤ롣
      board_moved();
    }

  // Ȥꤢݻ
  recover_threadlist();
  finalize();
  state_changed();

  return;
}


void
SubjectTxtParser::access_failed_cb(NetworkAgent *agent, const char *reason,
				   RefCount *data)
{
  switch (status_code)
    {
    case 301:	// Moved Permanently
    case 302:	// Found
    case 303:	// See Other
    case 307:	// Temporary Redirect
    case 403:	// Forbidden
    case 404:	// Not Found
      board_moved();
      break;
    }

  error_occurred();

  recover_threadlist();
  finalize();
  state_changed();
}


void
SubjectTxtParser::access_terminated_cb(NetworkAgent *agent, RefCount *data)
{
  error_occurred();

  recover_threadlist();
  finalize();
  state_changed();
}


static const char *
get_bbs_encoding(int bbs_type)
{
  switch (bbs_type)
    {
    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
    case OCHUSHA_BBS_TYPE_2CH_BE:
      return IconvBuffer::EUCJP_ENCODING;

    default:
      return IconvBuffer::CP932_ENCODING;
    }
}


ResponseCursor2chDAT::ResponseCursor2chDAT(BBSThread *thread,
					   Configuration &config,
					   Repository &repository,
					   NetworkAgent *agent)
  : ResponseCursor(thread, config, repository, new Buffer(), agent,
		   get_bbs_encoding(thread->get_bbs_type())),
    dirty(false)
{
  // ե뤫ɤǲ᤹Τˤɤٻ֤ݤΤפäƤߤ褦
  // ΡPCΥǥǤ⤻15ms餤ʤΤǡե뤫
  //   ɤ߹ߤϥåɲʤɤ
#ifdef ENABLE_BENCHMARKING
  struct timeval start_time;
  gettimeofday(&start_time, NULL);
#endif

  if (!restore_hints())
    {
      thread->set_last_modified(NULL);	// ̵̣ʤΤǾõ
      clear_hints();
    }

#ifdef ENABLE_BENCHMARKING
  struct timeval end_time;
  gettimeofday(&end_time, NULL);

  time_t sec = end_time.tv_sec - start_time.tv_sec;
  suseconds_t usec = end_time.tv_usec - start_time.tv_usec;
  while (usec < 0)
    {
      sec--;
      usec += 1000000;
    }
  printf("reading and early parsing of DAT consumes: %ld sec, %ld usec\n",
	 static_cast<long>(sec), static_cast<long>(usec));
#endif

  // ͥåȥγϤˤupdate()᥽åɤȤȤˤΤ
  // ǰöbuffer->fix()Ƥ
  buffer->fix();

  if (agent == NULL)
    return;

  agent->prepare_request
    .connect(this, &ResponseCursor2chDAT::prepare_request_cb);
  agent->response_header_received
    .connect(this, &ResponseCursor2chDAT::response_header_received_cb);
  agent->cache_is_dirty.connect(this, &ResponseCursor2chDAT::cache_is_dirty_cb);
  agent->access_progressed
    .connect(this, &ResponseCursor2chDAT::access_progressed_cb);
  agent->access_finished
    .connect(this, &ResponseCursor2chDAT::access_finished_cb);
  agent->access_failed.connect(this, &ResponseCursor2chDAT::access_failed_cb);
  agent->access_terminated
    .connect(this, &ResponseCursor2chDAT::access_terminated_cb);
}


ResponseCursor2chDAT::~ResponseCursor2chDAT()
{
}


void
ResponseCursor2chDAT::prepare_request_cb(NetworkAgent *agent, RefCount *data)
{
  if (thread->get_last_modified() != NULL && buffer->get_length0() > 0)
    {
      // If-Modified-Sinceդǥ
      agent->set_header(HTTPHeader::If_Modified_Since,
			thread->get_last_modified());
      // ʤ󤬡gzipȤRangeإåդΥ򤹤
      // 416ˤʤΤǻߤᡣ
      agent->set_header(HTTPHeader::Accept_Encoding, "deflate");
    }
  else
    agent->set_header(HTTPHeader::Accept_Encoding, "deflate, gzip");
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_USER_AGENT);

  if (config.get_enable_proxy_to_read())
    agent->set_proxy_uri(config.get_proxy_uri_to_read());
}


bool
ResponseCursor2chDAT::restore_hints()
{
  clear_hints();
  int fd = thread->open_dat_file(repository, O_RDONLY);
  if (fd >= 0)
    {
      if (buffer->read_file(fd))
	{
	  // bufferƬbuffer->get_length()ʬˤʤơLF򸫤Ĥ
	  // ٤set_hints(res_num, offset, length)롣
	  char *buf_top = buffer->get_buffer0();
	  size_t buf_len = buffer->get_length0();

	  char *buf_pos = buf_top;
	  size_t rest_len = buf_len;
	  char *tmp_pos;
	  unsigned int res_num = 1;
	  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n',
						       rest_len))) != NULL)
	    {
	      size_t len = tmp_pos - buf_pos + 1;
	      int offset = buf_pos - buf_top;
	      set_hints(res_num++, offset, len);
	      buf_pos = tmp_pos + 1;
	      rest_len -= len;
	    }
	  return true;
	}
      // ()Buffer::read_file()ѤȤfdclose()Ƥ롣
    }
  return false;
}


void
ResponseCursor2chDAT::response_header_received_cb(NetworkAgent *agent,
						  RefCount *data)
{
  int status_code = agent->get_status_code();
  if (status_code == 200 || status_code == 206 || status_code == 304)
    return;

  // ǶΣͤǤϡDATˤĤstatus_code == 203
  // ȶˡ>>1εDATե֤⤢餷

  // žΤΥ
#warning "Revisit Here!"
#if 0
  // ̿(200Ȥ)ΣͤλΥ쥹ݥ󥹥إåˤ
  // Date, Last-Modified, Etag, Accept-Ranges, Content-Length,
  // "Connection: close"
  // "Content-Type: text/plain"
  // 餤ޤޤƤʤ
  // ConnectionContent-TypeϾͤʤä
  const HTTPHeaders *headers = agent->get_headers();
  if (headers == NULL)
    return;

  vector<HTTPHeader>::const_iterator iter = headers->begin();
  vector<HTTPHeader>::const_iterator end_iter = headers->end();

  while (iter != end_iter)
    {
      const HTTPHeader &header = *iter++;
      fprintf(stderr, "%s: %s\n",
	      header.get_name(), header.get_value());
    }
#endif
}


void
ResponseCursor2chDAT::access_progressed_cb(NetworkAgent *agent,
					   size_t data_read,
					   uintmax_t total_len,
					   RefCount *data)
{
  // ͥåȥ()̤Buffer˽񤭹åɤ
  // ¹ԤƤΤǡδؿμ¹BufferΥΰ褬ư
  // Ϥʤ
  unsigned int res_num0 = get_next_res_num();
  int offset = get_next_offset();
  char *buf_top = buffer->get_buffer0();
  size_t buf_len = buffer->get_length0();

#if 0
  printf("res_num0=%d, offset=%d", res_num0, offset);
#endif

  unsigned int res_num = res_num0;
  char *buf_pos = buf_top + offset;
  size_t rest_len = buf_len - offset;
  char *tmp_pos;
  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n',
					       rest_len))) != NULL)
    {
      size_t len = tmp_pos - buf_pos + 1;
      offset = buf_pos - buf_top;
      set_hints(res_num++, offset, len);
      buf_pos = tmp_pos + 1;
      rest_len -= len;
    }

#if 0
  printf(" res_num=%d\n", res_num);
#endif

  if (res_num > res_num0)
    state_changed();
}


void
ResponseCursor2chDAT::cache_is_dirty_cb(NetworkAgent *agent, RefCount *data)
{
  dirty = true;
  clear_hints();
}


void
ResponseCursor2chDAT::access_finished_cb(NetworkAgent *agent, RefCount *data)
{
  buffer->fix();
  state_changed();
  const char *last_modified = agent->get_header(HTTPHeader::Last_Modified);
  if (last_modified != NULL)	// 304֤äƤʤ
    thread->set_last_modified(last_modified);
  thread->set_date(agent->get_header(HTTPHeader::Date));
  if (agent->get_status_code() != 304)
    {
      char buf[PATH_MAX];
      if (thread->find_dat_file(repository, buf, PATH_MAX, true) != NULL)
	agent->save_buffer(buf, last_modified);
    }
}


void
ResponseCursor2chDAT::access_failed_cb(NetworkAgent *agent, const char *reason,
				       RefCount *data)
{
  if (dirty)
    {
      // åԸλ˥ͥåȥ顼äΤǥե뤫
      // ɤľ
      restore_hints();
    }
  buffer->fix();
  state_changed();
}


void
ResponseCursor2chDAT::access_terminated_cb(NetworkAgent *agent, RefCount *data)
{
  buffer->fix();
  state_changed();
}


bool
ResponseCursor2chDAT::update()
{
  if (static_cast<NetworkAgent *>(agent) == NULL)
    return false;

  LockBlock lock_obj(libochusha_giant);

  int lock_id = lock();
  if (lock_id == 0)
    return false;		// ̿Ūġġ

  if (config.get_offline())
    {
      unlock(lock_id);
      return false;
    }

  if (!buffer->is_fixed())	// ¾ΥƥȤǹ
    {
      unlock(lock_id);
      return false;
    }

  if (!thread->is_alive())
    {
      unlock(lock_id);
      return false;		// ⤦Ϥʤ
    }

  char url_buffer[PATH_MAX];
  char *url = thread->get_dat_url(url_buffer, PATH_MAX);
  if (url == NULL)
    {
      unlock(lock_id);
      return false;
    }

  dirty = false;
  buffer->unfix0();
  unlock(lock_id);
  agent->update(url, buffer, this);
  return true;
}


bool
ResponseCursor2chDAT::parse_response(unsigned int res_num,
				     const char *start_pos, size_t length)
{
  assert(res_num > 0);
  assert(start_pos[length - 1] == '\n');
  assert(memchr(start_pos, '\n', length - 1) == NULL);

  char *clone_buffer = NULL;
  bool result = true;
  size_t i;

  // DATǡƤ뤳Ȥ⤢ΤǺǽ˥å
  const char *tmp_pos = static_cast<const char *>(memchr(start_pos, '\0',
							 length));
  if (tmp_pos != NULL)
    {
      clone_buffer = static_cast<char *>(malloc(length));
      if (clone_buffer == NULL)
	return false;
      for (i = 0; i < length; i++)
	clone_buffer[i] = start_pos[i] != '\0' ? start_pos[i] : '*';
      start_pos = clone_buffer;
    }

  const char *cur_pos = start_pos;
  assert(cur_pos[length - 1] == '\n');

  // ǽDATեηǧ롣
  // ̾()DATե
  // ̾<>ᥤ<>[ID][ۥ]<>ʸ<>[쥿]

  // ̾,ᥤ,[ID][ۥ],ʸ<>[쥿]
  // ߤѤʤȤˤʤäƤäꤹ餷
  // ä¾ˤ⤢ΤȤʤΤǥݡȤǤʤ

#define DAT_RES_NAME	0
#define DAT_RES_MAIL	1
#define DAT_RES_DATEID	2
#define DAT_RES_BODY	3
#define DAT_RES_TITLE	4

  // ǽ"<>"ο롣
  const char *field[5] = { NULL, NULL, NULL, NULL, NULL };
  size_t field_len[5] = { 0, 0, 0, 0, 0 };

  field[0] = cur_pos;
  for (i = 0; i < 4; i++)
    {
      tmp_pos = strnstr(cur_pos, "<>", length - (cur_pos - start_pos));
      if (tmp_pos != NULL)
	{
	  field_len[i] = tmp_pos - cur_pos;
	  cur_pos = tmp_pos + 2;
	  field[i + 1] = cur_pos;
	  continue;
	}
      break;
    }

  // : Ǥ⡢λfield_len[4]̤׻

  if (i == 0)
    {
      result = false;
      goto finish;
    }

  if (i == 1)
    {
      field[4] = field[1];
      // ޤο
      size_t tmp_len = field[4] - start_pos;
      cur_pos = start_pos;
      for (i = 0; i < 3; i++)
	{
	  tmp_pos
	    = static_cast<const char *>(memchr(cur_pos, ',',
					       tmp_len - (cur_pos - start_pos)));
	  if (tmp_pos != NULL)
	    {
	      field_len[i] = tmp_pos - cur_pos;
	      cur_pos = tmp_pos + 1;
	      field[i + 1] = cur_pos;
	      continue;
	    }
	  break;
	}

      if (i == 3)
	field_len[DAT_RES_BODY]
	  = field[DAT_RES_TITLE] - field[DAT_RES_BODY] - 2; // "<>"ʬ򸺻
    }

  // : Ǥ⡢λfield_len[4]̤׻

  if (i != 4)
    {
      result = false;
      goto finish;
    }

  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_NAME], field_len[DAT_RES_NAME]);
  set_name(iconv_buffer.get_string());

  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_MAIL], field_len[DAT_RES_MAIL]);
  set_mailto(iconv_buffer.get_string());


  // " ID:"⤷" ȯ:"⤷"BE:"dateȤ褦
  // ͤξ硢ν˷ǤǤ褤Τʡ
  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_DATEID], field_len[DAT_RES_DATEID]);

  tmp_pos = strstr(iconv_buffer.get_buffer(), " ID:");
  if (tmp_pos == NULL)
    {
      tmp_pos = strstr(iconv_buffer.get_string(), HASSHINMOTO_UTF8);
      if (tmp_pos == NULL)
	tmp_pos = strstr(iconv_buffer.get_string(), " BE:");
    }

  if (tmp_pos != NULL)
    {
      assert(*tmp_pos == ' ');
      char *space_pos = const_cast<char *>(tmp_pos);
      *space_pos = '\0';
      set_id(tmp_pos + 1);
    }
  else
    set_id("");

  set_date(iconv_buffer.get_string());
  if (res_num > 1000
      && strcmp(iconv_buffer.get_string(), "Over 1000 Thread") == 0)
    thread->set_stopped(true);


  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_BODY], field_len[DAT_RES_BODY]);
  set_message(iconv_buffer.get_string());

  if (res_num == 1)
    {
      // : Ǥ⡢λfield_len[4]̤׻
      field_len[DAT_RES_TITLE]
	= length - (field[DAT_RES_TITLE] - start_pos) - 1;
      assert(field_len[DAT_RES_TITLE] >= 0);
      iconv_buffer.clear();
      iconv_buffer.append(field[DAT_RES_TITLE], field_len[DAT_RES_TITLE]);
      set_title(iconv_buffer.get_string());
    }

 finish:
  if (clone_buffer != NULL)
    free(clone_buffer);
  return result;
}


PostResponseAgent2ch::PostResponseAgent2ch(BulletinBoard *board,
					   Configuration &config,
					   NetworkAgent *agent)
  : PostResponseAgent(board, config, agent),
    to_bbs_converter(get_bbs_encoding(board->get_bbs_type()),
		     IconvBuffer::UTF8_ENCODING),
    from_bbs_converter(IconvBuffer::UTF8_ENCODING,
		       get_bbs_encoding(board->get_bbs_type())),
    result_buffer(new Buffer()),
    referer_url(strdup(board->get_base_url())),
    title(NULL),
    name(NULL),
    mailto(NULL),
    message(NULL),
    be_id(NULL),
    viewer_id(NULL),
    hidden_keywords(NULL)
{
  time_t time_val = -1;
  const char *date = board->get_date();
  if (date != NULL)
    time_val = get_utc_from_date(date);

  if (time_val == -1)
    time_val = time(NULL);

  utc_time = time_val;

  agent->prepare_request.connect(this,
				 &PostResponseAgent2ch::prepare_request_cb);
  agent->access_finished.connect(this,
				 &PostResponseAgent2ch::access_finished_cb);
  agent->access_failed.connect(this, &PostResponseAgent2ch::access_failed_cb);
  agent->access_terminated.connect(this,
				   &PostResponseAgent2ch::access_terminated_cb);
}


PostResponseAgent2ch::PostResponseAgent2ch(BBSThread *thread,
					   Configuration &config,
					   NetworkAgent *agent)
  : PostResponseAgent(thread, config, agent),
    to_bbs_converter(get_bbs_encoding(thread->get_bbs_type()),
		     IconvBuffer::UTF8_ENCODING),
    from_bbs_converter(IconvBuffer::UTF8_ENCODING,
		       get_bbs_encoding(thread->get_bbs_type())),
    result_buffer(new Buffer()),
    referer_url(thread->get_url()),
    title(NULL),
    name(NULL),
    mailto(NULL),
    message(NULL),
    be_id(NULL),
    viewer_id(NULL),
    hidden_keywords(NULL)
{
  time_t time_val = -1;
  const char *date = thread->get_date();
  if (date != NULL)
    time_val = get_utc_from_date(date);

  if (time_val == -1)
    time_val = time(NULL);

  utc_time = time_val;

  agent->prepare_request.connect(this,
				 &PostResponseAgent2ch::prepare_request_cb);
  agent->access_finished.connect(this,
				 &PostResponseAgent2ch::access_finished_cb);
  agent->access_failed.connect(this, &PostResponseAgent2ch::access_failed_cb);
  agent->access_terminated.connect(this,
				   &PostResponseAgent2ch::access_terminated_cb);
}


PostResponseAgent2ch::~PostResponseAgent2ch()
{
  if (referer_url != NULL)
    free(referer_url);

  if (title != NULL)
    free(title);

  if (name != NULL)
    free(name);

  if (mailto != NULL)
    free(mailto);

  if (message != NULL)
    free(message);

  if (be_id != NULL)
    free(be_id);

  if (viewer_id != NULL)
    free(viewer_id);

  if (hidden_keywords != NULL)
    free(hidden_keywords);
}


void
PostResponseAgent2ch::prepare_request_cb(NetworkAgent *agent, RefCount *data)
{
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_USER_AGENT);
  agent->set_header(HTTPHeader::Accept_Charset,
		    "Shift_JIS,EUC-JP,utf-8;q=0.66,*;q=0.66");
  agent->set_header(HTTPHeader::Content_Type,
		    "application/x-www-form-urlencoded");
  agent->set_header(HTTPHeader::Referer, referer_url);

  if (be_id != NULL && be_id[0] != '\0')
    agent->set_header(HTTPHeader::Cookie, be_id);

  if (config.get_enable_proxy_to_post())
    agent->set_proxy_uri(config.get_proxy_uri_to_post());
}


class KeywordsExtractor : public HTMLSAXParser
{
private:
  string keywords;
  bool in_form_element;


protected:
  virtual void start_element(const char *name, const char *const attrs[]);
  virtual void end_element(const char *name);
  virtual void characters(const char *ch, size_t len);


public:
  KeywordsExtractor() : in_form_element(false) {}
  virtual ~KeywordsExtractor() {}

  inline const char *get_keywords() const { return keywords.c_str(); }
};


void
KeywordsExtractor::start_element(const char *name, const char *const attrs[])
{
  if (strcasecmp(name, "form") == 0)
    {
      in_form_element = true;
      return;
    }

  if (!in_form_element)
    return;

  if (strcasecmp(name, "input") != 0)
    return;

  const char *attr_type = NULL;
  const char *attr_name = NULL;
  const char *attr_value = NULL;
  int i = 0;
  while (attrs[i * 2] != NULL)
    {
      if (strcasecmp(attrs[i * 2], "type") == 0)
	attr_type = attrs[i * 2 + 1];
      else if (strcasecmp(attrs[i * 2], "name") == 0)
	attr_name = attrs[i * 2 + 1];
      else if (strcasecmp(attrs[i * 2], "value") == 0)
	attr_value = attrs[i * 2 + 1];
      i++;
    }

  if (attr_name == NULL
      || attr_type == NULL || strcasecmp(attr_type, "hidden") != 0)
    return;

  if (strcasecmp(attr_name, "subject") == 0
      || strcasecmp(attr_name, "from") == 0
      || strcasecmp(attr_name, "mail") == 0
      || strcasecmp(attr_name, "message") == 0
      || strcasecmp(attr_name, "bbs") == 0
      || strcasecmp(attr_name, "time") == 0
      || strcasecmp(attr_name, "key") == 0
      || strcasecmp(attr_name, "submit") == 0)
    return;

  keywords.append("&").append(attr_name).append("=");
  if (attr_value != NULL)
    keywords.append(attr_value);
}


void
KeywordsExtractor::end_element(const char *name)
{
  if (strcasecmp(name, "form") == 0)
    in_form_element = false;
}


void
KeywordsExtractor::characters(const char *ch, size_t len)
{
}


void
PostResponseAgent2ch::access_finished_cb(NetworkAgent *agent, RefCount *data)
{
  smart_ptr<PostResponseAgent2ch> tmp_this = this;

  from_bbs_converter.clear();

  RWLock *lock = result_buffer->get_lock();
  int lock_id = lock->rdlock();	// ̵̣ġ

  from_bbs_converter.append(result_buffer->get_buffer(lock_id),
			    result_buffer->get_length(lock_id));
  lock->unlock(lock_id);

  const char *buf = from_bbs_converter.get_buffer();

  if (agent->get_status_code() == 200)
    {
      time_t time_val = -1;
      const char *date = agent->get_header(HTTPHeader::Date);
      if (date != NULL)
	time_val = get_utc_from_date(date);

      if (time_val == -1)
	time_val = time(NULL);

      utc_time = time_val;

      const char *x_2ch = strstr(buf, "2ch_X:");
      if (x_2ch == NULL
	  && (strstr(buf, COOKIE_KAKUNIN_UTF8) != NULL
	      || strstr(buf, TOKO_KAKUNIN_UTF8) != NULL))
	x_2ch = "2ch_X:cookie";	// 

      if (x_2ch != NULL)
	{
	  if (strncmp(x_2ch, "2ch_X:cookie", 12) == 0)
	    {
	      KeywordsExtractor extractor;
	      extractor.parse_text(buf, strlen(buf));
	      if (hidden_keywords != NULL)
		{
		  free(hidden_keywords);
		  hidden_keywords = NULL;
		}

	      const char *keywords = extractor.get_keywords();
	      if (keywords != NULL && keywords[0] != '\0')
		hidden_keywords = strdup(keywords);
# if 0
	      fprintf(stderr, "hidden_keywords = %s\n", extractor.get_keywords());
	      fprintf(stderr, "2ch_X:cookie\n%s\n", buf);
# endif
	      check_cookie(this, buf);
	      return;
	    }
	  post_failed(this, buf);
	  return;
	}

      post_finished(this);
      return;
    }
  post_failed(this, buf);
}


void
PostResponseAgent2ch::access_failed_cb(NetworkAgent *agent, const char *reason,
				       RefCount *data)
{
  smart_ptr<PostResponseAgent2ch> tmp_this = this;

  post_failed(this, reason);
}


void
PostResponseAgent2ch::access_terminated_cb(NetworkAgent *agent, RefCount *data)
{
  smart_ptr<PostResponseAgent2ch> tmp_this = this;

  post_failed(this,
	      LIB_("Post terminated.  It isn't clear whether posting accepted or not."));
}


void
PostResponseAgent2ch::do_post(const char *kakikomu)
{
  assert(message != NULL);
  assert(*message != '\0');

  const char *name = this->name;
  if (name == NULL)
    name = "";

  const char *mailto = this->mailto;
  if (mailto == NULL)
    mailto = "";

  BulletinBoard *board = get_board();
  char cgi_url[PATH_MAX];
  char *bbs_root = board->get_bbs_root();
  if (bbs_root == NULL)
    {
      post_failed(this, LIB_("Out of memory"));
      return;
    }
  if (snprintf(cgi_url, PATH_MAX, "%stest/bbs.cgi", bbs_root) >= PATH_MAX)
    {
      post_failed(this, LIB_("Too long URL"));
      free(bbs_root);
      return;
    }
  free(bbs_root);

  char *bbs = board->get_id();
  if (bbs == NULL)
    {
      post_failed(this, LIB_("Out of memory"));
      return;
    }

  string key;
  string subject;

  if (get_thread() != NULL)
    key.append("&key=").append(get_thread()->get_id());
  else if (title != NULL)
    subject.append("&subject=").append(title);

  assert(key.c_str()[0] != '\0' || subject.c_str()[0] != '\0');

  int query_len = 2048;	// 2ܤƤȤΤǼ¼4096ХȤĩ
  char *query = NULL;
  int result;
  do
    {
      query_len *= 2;
      char *tmp_buf = static_cast<char *>(realloc(query, query_len));
      if (tmp_buf == NULL)
	{
	  post_failed(this, LIB_("Out of memory"));
	  if (query != NULL)
	    free(query);
	  return;
	}
      query = tmp_buf;
      if (viewer_id != NULL)
	result = snprintf(query, query_len,
			  "submit=%s%s&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s%s"
			  "&sid=%s&time=%jd%s",
			  kakikomu, subject.c_str(), name, mailto, message, bbs,
			  key.c_str(),
			  viewer_id, static_cast<intmax_t>(utc_time),
			  hidden_keywords != NULL ? hidden_keywords : "");
      else
	result = snprintf(query, query_len,
			  "submit=%s%s&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s%s"
			  "&time=%jd%s",
			  kakikomu, subject.c_str(), name, mailto, message, bbs,
			  key.c_str(),
			  static_cast<intmax_t>(utc_time),
			  hidden_keywords != NULL ? hidden_keywords : "");
    }
  while (result >= query_len);

#if 0
  fprintf(stderr, "POST query: %s\n", query);
#endif

  result_buffer->clear(0);
  result_buffer->unfix0();

  smart_ptr<PostResponseAgent> tmp_ref = this;
  get_agent()->post(cgi_url, query, result_buffer, this);
  free(query);
  free(bbs);
}


void
PostResponseAgent2ch::try_new_thread(const char *title, const char *name,
				     const char *mailto, const char *message)
{
  if (hidden_keywords != NULL)
    {
      free(hidden_keywords);
      hidden_keywords = NULL;
    }

  if (title == NULL || *title == '\0')
    {
      post_failed(this, LIB_("Title is empty"));
      return;
    }

  if (message == NULL || *message == '\0')
    {
      post_failed(this, LIB_("Message is empty"));
      return;
    }

  if (this->title != NULL)
    {
      free(this->title);
      this->title = NULL;
    }

  to_bbs_converter.clear();
  to_bbs_converter.append(title, strlen(title));
  this->title = URI::encode_string(to_bbs_converter.get_string());

  if (this->name != NULL)
    {
      free(this->name);
      this->name = NULL;
    }

  if (name != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(name, strlen(name));
      this->name = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->mailto != NULL)
    {
      free(this->mailto);
      this->mailto = NULL;
    }

  if (mailto != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(mailto, strlen(mailto));
      this->mailto = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->message != NULL)
    {
      free(this->message);
      this->message = NULL;
    }

  to_bbs_converter.clear();
  to_bbs_converter.append(message, strlen(message));
  this->message = URI::encode_string(to_bbs_converter.get_string());

  switch (get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
    case OCHUSHA_BBS_TYPE_2CH_BE:
      do_post(SHINKI_THREAD_SAKUSEI_EUCJP_URIENC);
      return;

    default:
      do_post(SHINKI_THREAD_SAKUSEI_CP932_URIENC);
      return;
    }
}


void
PostResponseAgent2ch::try_post(const char *name, const char *mailto,
			       const char *message)
{
  if (hidden_keywords != NULL)
    {
      free(hidden_keywords);
      hidden_keywords = NULL;
    }

  if (message == NULL || *message == '\0')
    {
      post_failed(this, LIB_("Message is empty"));
      return;
    }

  if (this->name != NULL)
    {
      free(this->name);
      this->name = NULL;
    }

  if (name != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(name, strlen(name));
      this->name = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->mailto != NULL)
    {
      free(this->mailto);
      this->mailto = NULL;
    }

  if (mailto != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(mailto, strlen(mailto));
      this->mailto = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->message != NULL)
    {
      free(this->message);
      this->message = NULL;
    }

  if (message != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(message, strlen(message));
      this->message = URI::encode_string(to_bbs_converter.get_string());
    }

  switch (get_thread()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
    case OCHUSHA_BBS_TYPE_2CH_BE:
      do_post(KAKIKOMU_EUCJP_URIENC);
      return;

    default:
      do_post(KAKIKOMU_CP932_URIENC);
      return;
    }
}


void
PostResponseAgent2ch::retry_post(const char *submit, time_t time_val)
{
  if (message == NULL || *message == '\0')
    {
      post_failed(this, LIB_("Message is empty"));
      return;
    }

  if (time_val != -1)
    utc_time = time_val;

  char *kakikomu;
  if (submit != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(submit, strlen(submit));
      kakikomu = URI::encode_string(to_bbs_converter.get_string());

      do_post(kakikomu);

      free(kakikomu);
      return;
    }

  switch (get_thread()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
    case OCHUSHA_BBS_TYPE_2CH_BE:
      do_post(SHODAKU_KAKIKOMU_EUCJP_URIENC);
      return;

    default:
      do_post(SHODAKU_KAKIKOMU_CP932_URIENC);
      return;
    }
}


void
PostResponseAgent2ch::set_2ch_be_session_id(const char *id)
{
  if (be_id != NULL)
    free(be_id);

  switch (get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_2CH:
    case OCHUSHA_BBS_TYPE_2CH_BE:
    case OCHUSHA_BBS_TYPE_BBSPINK:
      be_id = id != NULL ? strdup(id) : NULL;
      return;

    default:
      be_id = NULL;
      return;
    }
}


void
PostResponseAgent2ch::set_2ch_viewer_session_id(const char *id)
{
  if (viewer_id != NULL)
    free(viewer_id);

  switch (get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_2CH:
    case OCHUSHA_BBS_TYPE_2CH_BE:
    case OCHUSHA_BBS_TYPE_BBSPINK:
      viewer_id = id != NULL ? strdup(id) : NULL;
      return;

    default:
      viewer_id = NULL;
      return;
    }
}


BBS2ch::BBS2ch(Configuration &config)
  : config(config),
    login_2ch_be_lock(false),
    session_id_2ch_be(NULL),
    login_2ch_viewer_lock(false),
    session_id_2ch_viewer(NULL)
{
}


BBS2ch::~BBS2ch()
{
  if (session_id_2ch_be != NULL)
    free(session_id_2ch_be);

  if (session_id_2ch_viewer != NULL)
    free(session_id_2ch_viewer);
}


class Login2chBeBuffer : public Buffer
{
private:
  BBS2ch &bbs;

public:
  Login2chBeBuffer(BBS2ch &bbs) : bbs(bbs) {}
  virtual ~Login2chBeBuffer()
  {
#ifdef VERBOSE_DESTRUCTOR
    cerr << "Login2chBeBuffer deleted\n";
#endif
  }
  virtual void fix()
  {
    Buffer::fix();
    char *result_buf = get_buffer0();
    size_t len = get_length0();
    char *session_id = NULL;
    if (len > 0)
      {
#if 0
	IconvBuffer utf8_buf(IconvBuffer::UTF8_ENCODING,
			     IconvBuffer::EUCJP_ENCODING);
	utf8_buf.append(result_buf, len);
	fprintf(stderr, "Be-Login response body:\n%s\n", utf8_buf.get_string());
#endif
	char *mdmd = strnstr(result_buf, "MDMD=", len);
	char *dmdm = strnstr(result_buf, "DMDM=", len);
	if (mdmd != NULL && dmdm != NULL)
	  {
	    char *mdmd_end
	      = strnstr(mdmd, ";", len - (mdmd - result_buf));
	    char *dmdm_end = NULL;
	    size_t dmdm_left = len - (dmdm - result_buf);
	    char *dmdm_value = strnstr(dmdm, "escape('", dmdm_left);
	    if (dmdm_value != NULL)
	      dmdm = dmdm_value + 8;
	    else
	      {
		dmdm_value = strnstr(dmdm, "' + '", dmdm_left);
		if (dmdm_value != NULL)
		  dmdm = dmdm_value + 5;
	      }
	    if (dmdm != NULL)
	      dmdm_end = strnstr(dmdm, "'", dmdm_left);
	    if (mdmd_end != NULL && dmdm_end != NULL)
	      {
		*mdmd_end = '\0';
		*dmdm_end = '\0';
		string cookie;
		cookie.append(mdmd).append("; DMDM=").append(dmdm);
		session_id = strdup(cookie.c_str());
	      }
	  }

	if (session_id == NULL)
	  {
	    IconvBuffer utf8_buf(IconvBuffer::UTF8_ENCODING,
				 IconvBuffer::EUCJP_ENCODING);
	    utf8_buf.append(result_buf, len);
	    bbs.login_2ch_be_failed(utf8_buf.get_string());
	  }
      }
    else
      bbs.login_2ch_be_failed(LIB_("Unknown reason"));

    LockBlock lock_obj(libochusha_giant);
    assert(bbs.session_id_2ch_be == NULL);
    assert(bbs.login_2ch_be_lock);

    bbs.session_id_2ch_be = session_id;
    bbs.login_2ch_be_lock = false;
    bbs.login_2ch_be_finished(session_id);
  }
};


void
BBS2ch::login_2ch_be(const char *id, const char *passwd,
		     ThreadPool &pool, NetworkAgentBroker &broker)
{
  if (config.get_offline())
    {
      login_2ch_be_failed(LIB_("It is in offline mode."));
      login_2ch_be_finished(NULL);
      return;
    }

  LockBlock lock_obj(libochusha_giant);
  if (login_2ch_be_lock)
    return;	// 

  if (id == NULL)
    {
      login_2ch_be_failed(LIB_("User-ID is empty."));
      login_2ch_be_finished(NULL);
      return;
    }
  if (passwd == NULL)
    {
      login_2ch_be_failed(LIB_("Password is empty."));
      login_2ch_be_finished(NULL);
      return;
    }

  char *enc_id = URI::encode_string(id);
  if (enc_id == NULL)
    {
      login_2ch_be_failed(LIB_("Out of memory?"));
      login_2ch_be_finished(NULL);
      return;	// Out of memory?
    }
  char *enc_pw = URI::encode_string(passwd);
  if (enc_pw == NULL)
    {
      login_2ch_be_failed(LIB_("Out of memory?"));
      login_2ch_be_finished(NULL);
      free(enc_id);
      return;	// Out of memory?
    }
  char body[4096];
  snprintf(body, 4096, "submit=%%C5%%D0%%CF%%BF&m=%s&p=%s", enc_id, enc_pw);

  free(enc_id);
  free(enc_pw);

  login_2ch_be_lock = true;

  NetworkAgent *agent = broker.employ_agent();
  Buffer *buf = new Login2chBeBuffer(*this);
  agent->prepare_request.connect(this, &BBS2ch::prepare_be_headers);
  agent->post(config.get_login_be_uri().get_uri(), body, buf);
}


void
BBS2ch::prepare_be_headers(NetworkAgent *agent, RefCount *data)
{
  agent->set_header(HTTPHeader::Accept_Encoding, "deflate, gzip");
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_USER_AGENT);
  agent->set_header(HTTPHeader::Accept_Charset,
		    "Shift_JIS,EUC-JP,utf-8;q=0.66,*;q=0.66");
  agent->set_header(HTTPHeader::Content_Type,
		    "application/x-www-form-urlencoded");
  agent->set_header(HTTPHeader::Referer, OCHUSHA_2CH_BE_LOGIN_URL);

  if (config.get_enable_proxy_to_post())
    agent->set_proxy_uri(config.get_proxy_uri_to_post());
}


void
BBS2ch::logout_2ch_be()
{
  LockBlock lock_obj(libochusha_giant);
  if (session_id_2ch_be != NULL)
    {
      free(session_id_2ch_be);
      session_id_2ch_be = NULL;
    }
}


class Login2chViewerBuffer : public Buffer
{
private:
  BBS2ch &bbs;

public:
  Login2chViewerBuffer(BBS2ch &bbs) : bbs(bbs) {}
  virtual ~Login2chViewerBuffer()
  {
#ifdef VERBOSE_DESTRUCTOR
    cerr << "Login2chViewerBuffer deleted\n";
#endif
  }
  virtual void fix()
  {
    Buffer::fix();
    char *result_buf = get_buffer0();
    size_t len = get_length0();
    char *session_id = NULL;
    if (len > 0)
      {
	const char *sid = strstr(result_buf, "SESSION-ID=");
	if (sid != NULL)
	  {
	    sid += 11;	// "SESSION-ID="ɤФ
	    if (strncmp(sid, "ERROR:", 6) != 0)
	      {
		char *eol_pos = const_cast<char *>(strchr(sid, '\n'));
		if (eol_pos != NULL)
		  *eol_pos = '\0';
		session_id = URI::encode_string(sid);
	      }
	  }

	if (session_id == NULL)
	  {
	    if (sid != NULL)
	      bbs.login_2ch_viewer_failed("SESSION-ID=ERROR:ppp...");
	    else
	      {
		IconvBuffer utf8_buf(IconvBuffer::UTF8_ENCODING,
				     IconvBuffer::CP932_ENCODING);
		utf8_buf.append(result_buf, len);
#if 0
		fprintf(stderr, "Login2chViewerBuffer::fix(): server response\n%s\n",
			utf8_buf.get_string());
#endif
		bbs.login_2ch_viewer_failed(utf8_buf.get_string());
	      }
	  }
      }
    else
      bbs.login_2ch_viewer_failed(LIB_("Unknown reason"));

    LockBlock lock_obj(libochusha_giant);
    assert(bbs.session_id_2ch_viewer == NULL);
    assert(bbs.login_2ch_viewer_lock);

    bbs.session_id_2ch_viewer = session_id;
    bbs.login_2ch_viewer_lock = false;
    bbs.login_2ch_viewer_finished(session_id);
  }
};


void
BBS2ch::login_2ch_viewer(const char *id, const char *passwd,
			 ThreadPool &pool, NetworkAgentBroker &broker)
{
  if (config.get_offline())
    {
      login_2ch_viewer_failed(LIB_("It is in offline mode."));
      login_2ch_viewer_finished(NULL);
      return;
    }

  LockBlock lock_obj(libochusha_giant);
  if (login_2ch_viewer_lock)
    return;	// 

  if (id == NULL)
    {
      login_2ch_viewer_failed(LIB_("User-ID is empty."));
      login_2ch_viewer_finished(NULL);
      return;
    }
  if (passwd == NULL)
    {
      login_2ch_viewer_failed(LIB_("Password is empty."));
      login_2ch_viewer_finished(NULL);
      return;
    }

  char *enc_id = URI::encode_string(id);
  if (enc_id == NULL)
    {
      login_2ch_viewer_failed(LIB_("Out of memory?"));
      login_2ch_viewer_finished(NULL);
      return;	// Out of memory?
    }
  char *enc_pw = URI::encode_string(passwd);
  if (enc_pw == NULL)
    {
      login_2ch_viewer_failed(LIB_("Out of memory?"));
      login_2ch_viewer_finished(NULL);
      free(enc_id);
      return;	// Out of memory?
    }
  char body[4096];
  snprintf(body, 4096, "ID=%s&PW=%s", enc_id, enc_pw);

  free(enc_id);
  free(enc_pw);

  login_2ch_viewer_lock = true;
  NetworkAgent *agent = broker.employ_agent();
  Buffer *buf = new Login2chViewerBuffer(*this);
  agent->prepare_request.connect(this, &BBS2ch::prepare_viewer_headers);
  agent->post(config.get_login_viewer_uri().get_uri(), body, buf);
}


void
BBS2ch::prepare_viewer_headers(NetworkAgent *agent, RefCount *data)
{
  agent->set_header(HTTPHeader::Accept_Encoding, "deflate, gzip");
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_DOLIB_USER_AGENT);
  agent->set_header("X-2ch-UA", OCHUSHA_X_2CH_UA);
  agent->set_header(HTTPHeader::Accept_Charset,
		    "Shift_JIS,EUC-JP,utf-8;q=0.66,*;q=0.66");
  agent->set_header(HTTPHeader::Content_Type,
		    "application/x-www-form-urlencoded");

#if 0
  if (config.get_enable_proxy_to_post())
    agent->set_proxy_uri(config.get_proxy_uri_to_post());
#endif
}


void
BBS2ch::logout_2ch_viewer()
{
  LockBlock lock_obj(libochusha_giant);
  if (session_id_2ch_viewer != NULL)
    {
      free(session_id_2ch_viewer);
      session_id_2ch_viewer = NULL;
    }
}


RefCountHashtable BBS2ch::table(101);


void
BBS2ch::register_board(BulletinBoard *board)
{
  URI base_uri(board->get_base_url());
  if (!base_uri.is_valid())
    return;

  int type = guess_board_type(base_uri);
  if (type != OCHUSHA_BBS_TYPE_2CH
      && type != OCHUSHA_BBS_TYPE_2CH_HEADLINE
      && type != OCHUSHA_BBS_TYPE_2CH_BE
      && type != OCHUSHA_BBS_TYPE_BBSPINK
      && type != OCHUSHA_BBS_TYPE_BBSPINK_HEADLINE)
    {
      type = board->get_bbs_type();
      if (type == OCHUSHA_BBS_TYPE_2CH
	  || type == OCHUSHA_BBS_TYPE_2CH_HEADLINE
	  || type == OCHUSHA_BBS_TYPE_2CH_BE
	  || type == OCHUSHA_BBS_TYPE_BBSPINK
	  || type == OCHUSHA_BBS_TYPE_BBSPINK_HEADLINE)
	board->set_bbs_type(OCHUSHA_BBS_TYPE_2CH_COMPATIBLE);
      return;
    }

  const char *path = base_uri.get_path();
  if (path == NULL || *path != '/')
    return;	// ͤĤˤƤϲ

  if (type != board->get_bbs_type())
    {
      board->set_bbs_type(type);	// Ǽļ̤
      return;	// ˺ƤΤǤμ¹Ԥס
    }

  char *canonical_base_url = guess_board_url(base_uri);
  if (canonical_base_url == NULL)
    return;	// äĤʤ

  BBSURLInfo info(canonical_base_url, 1);
  const char *board_id = info.get_board_id();

  if (board_id == NULL)
    return;	// äĤʤ

  BulletinBoard *entry = dynamic_cast<BulletinBoard *>(table.lookup(board_id));
  if (entry == NULL)
    {
      if (strcmp(board->get_base_url(), canonical_base_url) == 0)
	table.insert(board_id, board);
      else
	board->set_base_url(canonical_base_url);
    }
#if 0
  else
    {
      if (entry != board)
	fprintf(stderr, "multiple instance of board identified by %s found.\n",
		board_id);
    }
#endif
  free(canonical_base_url);
}


BulletinBoard *
BBS2ch::guess_board(const URI &uri)
{
  assert(uri.is_valid());

  int board_type = guess_board_type(uri);
  if (board_type != OCHUSHA_BBS_TYPE_2CH
      && board_type != OCHUSHA_BBS_TYPE_BBSPINK
      && board_type != OCHUSHA_BBS_TYPE_BBSPINK_HEADLINE
      && board_type != OCHUSHA_BBS_TYPE_2CH_HEADLINE
      && board_type != OCHUSHA_BBS_TYPE_2CH_BE)
    return NULL;

  BBSURLInfo info(uri, 1);
  const char *board_id = info.get_board_id();
  if (board_id != NULL)
    return dynamic_cast<BulletinBoard *>(table.lookup(board_id));
  return NULL;
}


int
BBS2ch::guess_board_type(const URI &uri)
{
  assert(uri.is_valid());

  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return OCHUSHA_BBS_TYPE_UNKNOWN;

  const char *server = uri.get_server();
  if (server == NULL)
    return OCHUSHA_BBS_TYPE_UNKNOWN;

  const char *tmp_pos = strcasestr(server, ".2ch.net");
  if (tmp_pos != NULL && tmp_pos[8] == '\0')
    {
      // bbqˤϥإåɥ饤ϤʤϤ
      if (strcasecmp(server, "headline.2ch.net") == 0
	  || strcasecmp(server, "bbq.2ch.net") == 0)
	return OCHUSHA_BBS_TYPE_2CH_HEADLINE;

      if (strcasecmp(server, "info.2ch.net") == 0)
	return OCHUSHA_BBS_TYPE_UNKNOWN;	// infoˤĤϤʤ

      if (strcasecmp(server, "be.2ch.net") == 0)
	return OCHUSHA_BBS_TYPE_2CH_BE;		// ΰʪ

      return OCHUSHA_BBS_TYPE_2CH;
    }

  if (strcasecmp(server, "headline.bbspink.com") == 0)
    return OCHUSHA_BBS_TYPE_BBSPINK_HEADLINE;

  tmp_pos = strcasestr(server, ".bbspink.com");
  if (tmp_pos != NULL && tmp_pos[12] == '\0')
    return OCHUSHA_BBS_TYPE_BBSPINK;

  return OCHUSHA_BBS_TYPE_UNKNOWN;
}


char *
BBS2ch::guess_board_url(const URI &uri)
{
  assert(uri.is_valid());

  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return NULL;

  const char *server = uri.get_server();
  if (server == NULL)
    return NULL;

  const char *tmp_pos = strcasestr(server, "2ch.net");
  if (tmp_pos != NULL && tmp_pos[7] == '\0')
    {
      if (strcasecmp(server, "info.2ch.net") == 0)
	return NULL;		// infoˤĤϤʤ
    }
  else
    {
      tmp_pos = strcasestr(server, "bbspink.com");
      if (tmp_pos == NULL || tmp_pos[11] != '\0')
	return NULL;
    }

  const char *path = uri.get_path();
  if (*path != '/')
    return NULL;

  BBSURLInfo info(uri, 1);
  if (info.get_board_id() == NULL)
    return NULL;

  string board_url;
  board_url.append(scheme).append("://").append(server).append("/")
    .append(info.get_board_id()).append("/");
  return strdup(board_url.c_str());
}


char *
BBS2ch::get_board_id(const BulletinBoard *board)
{
  URI uri(board->get_base_url());
  if (!uri.is_valid() || uri.get_path() == NULL)
    return NULL;

  char id_buf[4096];
  snprintf(id_buf, 4096, "%s", uri.get_path());
  size_t len = strlen(id_buf);
  if (*id_buf != '/' || id_buf[len - 1] != '/')
    return NULL;

  id_buf[len - 1] = '\0';
  char *slash_pos = strrchr(id_buf, '/');
  // ͤȸߴǼĤǤbase_urlκǿΥ֥ǥ쥯ȥ̾ID
  if (slash_pos != NULL)
    return strdup(slash_pos + 1);
  return NULL;
}


char *
BBS2ch::get_bbs_root(const BulletinBoard *board)
{
  URI uri(board->get_base_url());
  if (!uri.is_valid() || uri.get_path() == NULL)
    return NULL;

  char url_buf[4096];
  snprintf(url_buf, 4096, "%s", board->get_base_url());
  size_t len = strlen(url_buf);
  char *path_pos = strstr(url_buf, uri.get_path());
  if (path_pos == NULL || *path_pos != '/' || url_buf[len - 1] != '/')
    return NULL;

  url_buf[len - 1] = '\0';
  char *slash_pos = strrchr(url_buf, '/');
  // ͤȸߴǼĤǤbase_urlκǿΥ֥ǥ쥯ȥ̾ID
  if (slash_pos != NULL)
    {
      slash_pos[1] = '\0';
      return strdup(url_buf);
    }
  return NULL;
}


char *
BBS2ch::get_url(BBSThread *thread, int from, int to)
{
  assert(thread != NULL);

  URI board_uri(thread->get_board()->get_base_url());
  if (!board_uri.is_valid()
      || board_uri.get_server() == NULL || board_uri.get_path() == NULL)
    return NULL;

  BBSURLInfo info(board_uri, 1);
  if (info.get_bbs_root() == NULL || info.get_board_id() == NULL)
    return NULL;

  char url_buffer[PATH_MAX];
  if (from == 0)
    snprintf(url_buffer, PATH_MAX, "http://%s%stest/read.cgi/%s/%s/",
	     board_uri.get_server(), info.get_bbs_root(), info.get_board_id(),
	     thread->get_id());
  else if (from < 0)
    snprintf(url_buffer, PATH_MAX, "http://%s%stest/read.cgi/%s/%s/l%d",
	     board_uri.get_server(), info.get_bbs_root(), info.get_board_id(),
	     thread->get_id(), -from);
  else if (from < to)
    snprintf(url_buffer, PATH_MAX, "http://%s%stest/read.cgi/%s/%s/%d-%d",
	     board_uri.get_server(), info.get_bbs_root(), info.get_board_id(),
	     thread->get_id(), from, to);
  else
    snprintf(url_buffer, PATH_MAX, "http://%s%stest/read.cgi/%s/%s/%d",
	     board_uri.get_server(), info.get_bbs_root(), info.get_board_id(),
	     thread->get_id(), from);

  return strdup(url_buffer);
}


char *
BBS2ch::parse_thread_url(const char *url, int *from_p, int *to_p)
{
  BBSURLInfo info(url, 1);
  if (info.get_thread_id() != NULL)
    {
      if (from_p != NULL)
	*from_p = info.get_from();
      if (to_p != NULL)
	*to_p = info.get_to();
      return strdup(info.get_thread_id());
    }
  return NULL;
}


char *
BBS2ch::get_trackback_url(BBSThread *thread, int res_num)
{
  char *res_url = get_url(thread, res_num);
  if (res_url == NULL)
    return NULL;	// Out of memory?

  string buf;
  buf.append("http://info.2ch.net/test/tb.cgi?__mode=list&tb_id=")
    .append(res_url);
  free(res_url);

  return strdup(buf.c_str());
}


char *
BBS2chCompatible::guess_board_url(const URI &uri)
{
  assert(uri.is_valid());

  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return NULL;

  const char *server = uri.get_server();
  if (server == NULL)
    return NULL;

  BBSURLInfo info(uri, 1);
  if (info.get_board_id() == NULL || info.get_bbs_root() == NULL)
    return NULL;

  string board_url;
  board_url.append(scheme).append("://").append(server)
    .append(info.get_bbs_root()).append(info.get_board_id()).append("/");
  return strdup(board_url.c_str());
}

}
