/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_2ch_utils.h,v 1.24 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BBS_2CH_UTILS_H_
#define _LIBOCHUSHA_BBS_2CH_UTILS_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/buffer.h"
#include "libochusha/bulletin_board.h"
#include "libochusha/bbs_utils.h"
#include "libochusha/hashtable.h"
#include "libochusha/lock.h"
#include "libochusha/sigslot.h"
#include "libochusha/regexp.h"
#include "libochusha/text_utils.h"


#if defined(HAVE_STDINT_H) || !defined(HAVE_CONFIG_H)
#include <stdint.h>
#else
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif
#endif


using namespace std;
namespace Ochusha
{
  class BBSThread;
  class Configuration;
  class NetworkAgent;
  class NetworkAgentBroker;
  class Repository;
  class ThreadPool;
  class URI;

  class SubjectTxtParser : public ThreadlistUpdater
  {
  private:
    IconvBuffer iconv_buffer;
    Regexp regexp;

    BufferHandle buffer;

    int status_code;

    size_t offset;
    int number_of_threads;


    void prepare_request_cb(NetworkAgent *agent, RefCount *data);
    void response_header_received_cb(NetworkAgent *agent, RefCount *data);
    void access_progressed_cb(NetworkAgent *agent, size_t data_read,
			      uintmax_t total_len, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  public:
    SubjectTxtParser(BulletinBoard *board, Configuration &config,
		     Repository &repository, NetworkAgent *agent,
		     const char *encoding, const char *regexp);
    virtual ~SubjectTxtParser();
  };


  class ResponseCursor2chDAT : public ResponseCursor
  {
  private:
    bool dirty;

    bool restore_hints();

    void prepare_request_cb(NetworkAgent *agent, RefCount *data);
    void response_header_received_cb(NetworkAgent *agent, RefCount *data);
    void cache_is_dirty_cb(NetworkAgent *agent, RefCount *data);
    void access_progressed_cb(NetworkAgent *agent, size_t data_read,
			      uintmax_t total_len, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  protected:
    virtual bool parse_response(unsigned int res_num,
				const char *start, size_t length);


  public:
    ResponseCursor2chDAT(BBSThread *thread, Configuration &config,
			 Repository &repository, NetworkAgent *agent);
    virtual ~ResponseCursor2chDAT();

    virtual bool update();
  };


  class PostResponseAgent2ch : public PostResponseAgent, public has_slots
  {
  protected:
    IconvBuffer to_bbs_converter;
    IconvBuffer from_bbs_converter;
    BufferHandle result_buffer;
    char *referer_url;
    char *title;
    char *name;
    char *mailto;
    char *message;
    time_t utc_time;
    char *be_id;
    char *viewer_id;

    char *hidden_keywords;
 
    void do_post(const char *kakikomu);

    void prepare_request_cb(NetworkAgent *agent, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  public:
    PostResponseAgent2ch(BulletinBoard *board, Configuration &config,
			 NetworkAgent *agent);
    PostResponseAgent2ch(BBSThread *thread, Configuration &config,
			 NetworkAgent *agent);
    virtual ~PostResponseAgent2ch();

    virtual void try_new_thread(const char *title, const char *name,
				const char *mailto, const char *message);
    virtual void try_post(const char *name, const char *mailto,
			  const char *message);
    virtual void retry_post(const char *submit, time_t time_val);

    virtual void set_2ch_be_session_id(const char *id);
    virtual void set_2ch_viewer_session_id(const char *id);
  };


  class BBS2ch : public has_slots
  {
    friend class Login2chBeBuffer;
    friend class Login2chViewerBuffer;

  private:
    Configuration &config;

    bool login_2ch_be_lock;
    char *session_id_2ch_be;

    bool login_2ch_viewer_lock;
    char *session_id_2ch_viewer;


    void prepare_be_headers(NetworkAgent *agent, RefCount *data);
    void prepare_viewer_headers(NetworkAgent *agent, RefCount *data);


  public:
    BBS2ch(Configuration &config);
    virtual ~BBS2ch();


    // ̥åɤǹԤδؿϤ롣
    void login_2ch_be(const char *id, const char *passwd,
		      ThreadPool &pool, NetworkAgentBroker &broker);
    void logout_2ch_be();
    const char *get_session_id_2ch_be() const { return session_id_2ch_be; }


    // 2ch BeؤΥԻWorkerThreadȯԤ롣
    // void login_2ch_be_failed(const char *reason);
    signal1<const char *> login_2ch_be_failed;

    // 2ch BeؤΥλWorkerThreadȯԤ롣
    // void login_2ch_be_finished(const char *session_id);
    // Իˤsession_idNULL
    signal1<const char *> login_2ch_be_finished;


    // ̥åɤǹԤδؿϤ롣
    void login_2ch_viewer(const char *id, const char *passwd,
			  ThreadPool &pool, NetworkAgentBroker &broker);
    void logout_2ch_viewer();
    const char *get_session_id_2ch_viewer() const
    { return session_id_2ch_viewer; }


    // 2ch塼()ؤΥԻWorkerThreadȯԤ롣
    // void login_2ch_viewer_failed(const char *reason);
    signal1<const char *> login_2ch_viewer_failed;

    // 2ch塼()ؤΥλWorkerThread
    // ȯԤ롣
    // void login_2ch_viewer_finished(const char *session_id);
    // Իˤsession_idNULL
    signal1<const char *> login_2ch_viewer_finished;


  private:
    static RefCountHashtable table;	// "/ID/"BulletinBoard *ʥơ֥롣


  public:
    static void register_board(BulletinBoard *board);
    static BulletinBoard *guess_board(const URI &uri);
    static int guess_board_type(const URI &uri);
    static char *guess_board_url(const URI &uri);
    static char *get_board_id(const BulletinBoard *board);
    static char *get_bbs_root(const BulletinBoard *board);
    static char *get_url(BBSThread *thread, int from = 0, int to = 0);
    static char *parse_thread_url(const char *url, int *from_p, int *to_p);
    static char *get_trackback_url(BBSThread *thread, int res_num);
  };


  class BBS2chCompatible
  {
  public:
    static char *guess_board_url(const URI &uri);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BBS_2CH_UTILS_H_ */
