/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_data_manager.cc,v 1.14 2009/01/01 07:46:43 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/bbs_data_manager.h"

#include "libochusha/cache_manager.h"
#include "libochusha/misc_utils.h"
#include "libochusha/repository.h"


#include <climits>
#include <list>


namespace Ochusha
{

// ̰å̵ʵŪ¸
BBSDataManager::BBSDataManager(Repository &repository)
  : CacheManager(repository, NULL, "cache", 0, 0)
{
}


BBSDataManager::~BBSDataManager()
{
}


char *
BBSDataManager::get_pathname(const char *url) const
{
  char buf[PATH_MAX];
  if (get_pathname(url, buf, PATH_MAX) != NULL)
    return strdup(buf);

  return NULL;
}


char *
BBSDataManager::get_pathname(const char *url, char *buf, size_t buf_len) const
{
  char pathname[PATH_MAX];
  if (get_tenured_file_path(url, pathname, PATH_MAX) == NULL)
    return NULL;

  return get_repository().expand_path(pathname, buf, buf_len);
}


char *
BBSDataManager::find_file(const char *url) const
{
  return find_cache(url);
}


char *
BBSDataManager::find_file(const char *url, char *buf, size_t buf_len) const
{
  return find_cache(url, buf, buf_len);
}


char *
BBSDataManager::find_directory(const char *url) const
{
  return find_cache_directory(url);
}


char *
BBSDataManager::find_directory(const char *url, char *buf, size_t buf_len) const
{
  return find_cache_directory(url, buf, buf_len);
}


int
BBSDataManager::open_file(const char *url, int flags) const
{
  return open_cache(url, flags);
}


bool
BBSDataManager::move_file(const char *filename, const char *url)
{
  return recache(filename, url);
}


void
BBSDataManager::remove_file(const char *url)
{
  invalidate(url);
}


static void
foreach_server(const char *fullpath, const char *filename,
	       struct stat *sb, void *user_data)
{
  list<string> &result = *static_cast<list<string> *>(user_data);
  result.push_front(string(filename));
}


void
BBSDataManager::enumerate_servers(list<string> &result) const
{
  Repository &repository = get_repository();
  repository.foreach_dir(get_tenured_cache_directory(), foreach_server,
			 &result);
}

}
