/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_jbbs_utils.cc,v 1.36 2010/08/15 15:36:11 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/bbs_jbbs_utils.h"

#include "libochusha/bbs_2ch_utils.h"
#include "libochusha/bbs_thread.h"
#include "libochusha/bulletin_board.h"
#include "libochusha/configuration.h"
#include "libochusha/constants.h"
#include "libochusha/http_headers.h"
#include "libochusha/misc_utils.h"
#include "libochusha/regexp.h"
#include "libochusha/uri_utils.h"

#include <cassert>
#include <climits>
#include <cstring>
#include <string>


namespace Ochusha
{


static const char *
get_bbs_encoding(int bbs_type)
{
  switch (bbs_type)
    {
    case OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR:
      return IconvBuffer::EUCJP_ENCODING;
    }
  return IconvBuffer::CP932_ENCODING;
}


ResponseCursorJbbs::ResponseCursorJbbs(BBSThread *thread, Configuration &config,
				       Repository &repository, Buffer *buffer,
				       NetworkAgent *agent)
  : ResponseCursor(thread, config, repository, buffer, agent,
		   get_bbs_encoding(thread->get_bbs_type())),
    number_of_responses_got(0)
{
  if (agent == NULL)
    return;

  agent->prepare_request.connect(this,
				 &ResponseCursorJbbs::prepare_request_cb);
  agent->response_header_received.connect(this,
					  &ResponseCursorJbbs::response_header_received_cb);
  agent->access_progressed.connect(this,
				   &ResponseCursorJbbs::access_progressed_cb);
  agent->access_finished.connect(this, &ResponseCursorJbbs::access_finished_cb);
  agent->access_failed.connect(this, &ResponseCursorJbbs::access_failed_cb);
  agent->access_terminated.connect(this,
				   &ResponseCursorJbbs::access_terminated_cb);
}


ResponseCursorJbbs::~ResponseCursorJbbs()
{
}


bool
ResponseCursorJbbs::restore_hints()
{
  clear_hints();

  int fd = thread->open_dat_file(repository, O_RDONLY);
  if (fd >= 0)
    {
      if (buffer->read_file(fd))
	{
	  // bufferƬbuffer->get_length()ʬˤʤơLF򸫤Ĥ
	  // ٤set_hints(res_num, offset, length)롣
	  // ֹ椬Ǥ餽Υ쥹ˤĤset_deleted(res_num, offset);
	  char *buf_top = buffer->get_buffer0();
	  size_t buf_len = buffer->get_length0();

	  // 1쥹ܤäǰ˵Ǥʤå롣
	  char *tmp_pos = static_cast<char *>(memchr(buf_top, '\n', buf_len));
	  if (tmp_pos == NULL)
	    return false;	// äݤǤ

	  size_t len = tmp_pos - buf_top + 1;
	  // <>ο롣Ǥ6ĸ롣
	  int i = 0;
	  tmp_pos = buf_top;
	  while (true)
	    {
	      char *delim = strnstr(tmp_pos, "<>", len - (tmp_pos - buf_top));
	      if (delim == NULL)
		break;
	      tmp_pos = delim + 2;
	      ++i;
	    }

	  if (i == 4)
	    return convert_dat();	// ǻ

	  if (i != 6)
	    return false;		// Τʤǻ

	  char *buf_pos = buf_top;
	  size_t rest_len = buf_len;
	  unsigned int res_num = 1;
	  char num_buf[64];

	  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n',
						       rest_len))) != NULL)
	    {
	      len = tmp_pos - buf_pos + 1;
	      int offset = buf_pos - buf_top;

	      char *delim = strnstr(buf_pos, "<>", len);
	      if (delim != NULL)
		{
		  size_t num_len = delim - buf_pos;
		  if (num_len > 0 && num_len < 63)
		    {
		      memcpy(num_buf, buf_pos, num_len);
		      num_buf[num_len] = '\0';
		      unsigned int num;
		      if (sscanf(num_buf, "%u", &num) == 1)
			{
			  while (res_num < num)
			    set_deleted(res_num++, offset);
			  if (res_num == num)
			    set_hints(res_num++, offset, len);
			  else
			    return convert_dat();	// ʡ
			}
		      else
			return convert_dat();	// 
		    }
		}
	      else
		{
		  // ǤϺ줿쥹϶ԤˤƤʤΤǤ
		  // Ѵ
		  return convert_dat();
		}
	      buf_pos = tmp_pos + 1;
	      rest_len -= len;
	    }
	  return true;
	}
      // ()Buffer::read_file()ѤȤfdclose()Ƥ롣
    }
  return false;
}


bool
ResponseCursorJbbs::parse_response(unsigned int res_num,
				   const char *start_pos, size_t length)
{
  assert(res_num > 0);
  assert(start_pos[length - 1] == '\n');
  assert(memchr(start_pos, '\n', length - 1) == NULL);

  char *clone_buffer = NULL;
  bool result = true;
  size_t i;

  // DATǡƤ뤳Ȥ⤢ΤǺǽ˥å
  const char *tmp_pos
    = static_cast<const char *>(memchr(start_pos, '\0', length));
  if (tmp_pos != NULL)
    {
      clone_buffer = static_cast<char *>(malloc(length));
      if (clone_buffer == NULL)
	return false;
      for (i = 0; i < length; i++)
	clone_buffer[i] = start_pos[i] != '\0' ? start_pos[i] : '*';
      start_pos = clone_buffer;
    }

  const char *cur_pos = start_pos;
  assert(cur_pos[length - 1] == '\n');

  // JBBSDATեϸߤJBBS饤֥ɥrawmode.cgiνϷ
  // 碌Ƥ
  // ֹ<>̾<>᡼륢ɥ쥹<><>ʸ<>[쥿]<>[ۥ/ID]
  // 0.5ϰΤϣͤDAT˹碌Ƥ
  // rawmode.cgiȤʤ餽ä˹碌ƤڤʤΤѹ
  // (¸)DATեɤ߹ߤ˴ؤƤϡ
  // restore_hints()Ѵ롣

#define DAT_RES_NUMBER	0
#define DAT_RES_NAME	1
#define DAT_RES_MAIL	2
#define DAT_RES_DATE	3
#define DAT_RES_BODY	4
#define DAT_RES_TITLE	5
#define DAT_RES_ID	6

  const char *field[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
  size_t field_len[7] = { 0, 0, 0, 0, 0, 0, 0 };

  field[0] = cur_pos;
  for (i = 0; i < 6; i++)
    {
      tmp_pos = strnstr(cur_pos, "<>", length - (cur_pos - start_pos));
      if (tmp_pos != NULL)
	{
	  field_len[i] = tmp_pos - cur_pos;
	  cur_pos = tmp_pos + 2;
	  field[i + 1] = cur_pos;
	  continue;
	}
      break;
    }

#ifndef NDEBUG
  unsigned int num = 0;
#endif

  // : Ǥ⡢λfield_len[6]̤׻
  if (i != 6)
    {
      result = false;
      goto finish;
    }

#ifndef NDEBUG
  assert(sscanf(field[0], "%u", &num) == 1);
  assert(num == res_num);
#endif

  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_NAME], field_len[DAT_RES_NAME]);
  set_name(iconv_buffer.get_string());

  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_MAIL], field_len[DAT_RES_MAIL]);
  set_mailto(iconv_buffer.get_string());

  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_DATE], field_len[DAT_RES_DATE]);
  set_date(iconv_buffer.get_string());

  iconv_buffer.clear();
  iconv_buffer.append(field[DAT_RES_BODY], field_len[DAT_RES_BODY]);
  set_message(iconv_buffer.get_string());

  if (res_num == 1)
    {
      iconv_buffer.clear();
      iconv_buffer.append(field[DAT_RES_TITLE], field_len[DAT_RES_TITLE]);
      set_title(iconv_buffer.get_string());
    }

  field_len[DAT_RES_ID] = length - (field[DAT_RES_ID] - start_pos) - 1;
  assert(field_len[DAT_RES_ID] >= 0);
  iconv_buffer.clear();

  if (thread->get_bbs_type() == OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR)
    {
      size_t id_len = field_len[DAT_RES_ID];
      if (id_len > 0)
	{
	  if (id_len == 8)
	    {
	      iconv_buffer.append("ID:", 3);
	      iconv_buffer.append(field[DAT_RES_ID], id_len);
	    }
	  else if (id_len > 8)
	    {
	      iconv_buffer.append("HOST:", 5);
	      iconv_buffer.append(field[DAT_RES_ID], id_len);
	    }
	  else if (id_len == 3 && strncmp(field[DAT_RES_ID], "???", 3) == 0)
	    {
	      iconv_buffer.append("ID:???", 6);
	    }
	  else
	    {
	      iconv_buffer.append("[ ", 2);
	      iconv_buffer.append(field[DAT_RES_ID], id_len);
	      iconv_buffer.append(" ]", 2);
	    }
	}
    }
  else
    iconv_buffer.append(field[DAT_RES_ID], field_len[DAT_RES_ID]);
  set_id(iconv_buffer.get_string());


 finish:
  if (clone_buffer != NULL)
    free(clone_buffer);
  return result;
}


void
ResponseCursorJbbs::prepare_request_cb(NetworkAgent *agent, RefCount *data)
{
  agent->set_header(HTTPHeader::Accept_Encoding, "deflate, gzip");
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_USER_AGENT);

  if (config.get_enable_proxy_to_read())
    agent->set_proxy_uri(config.get_proxy_uri_to_read());
}


void
ResponseCursorJbbs::response_header_received_cb(NetworkAgent *agent,
						RefCount *data)
{
#ifndef NDEBUG	// Ĵ
  // ʻͤäȤΤʤä
  // http://info.2ch.net/wiki/index.php?monazilla%2Fdevelop%2Fshitaraba

  const char *value = agent->get_header("Error");
  if (value != NULL)
    {
      fprintf(stderr,
	      "ResponseCursorJbbs::response_header_received_cb(): \"Error: %s\" status_code=%d\n",
	      value, agent->get_status_code());
    }
#endif
}


void
ResponseCursorJbbs::access_progressed_cb(NetworkAgent *agent,
					 size_t data_read, uintmax_t total_len,
					 RefCount *data)
{
  // ͥåȥ()̤Buffer˽񤭹åɤ
  // ¹ԤƤΤǡδؿμ¹BufferΥΰ褬ư
  // Ϥʤ
  unsigned int res_num0 = get_next_res_num();
  int offset = get_next_offset();
  char *buf_top = buffer->get_buffer0();
  size_t buf_len = buffer->get_length0();

#if 0
  fprintf(stderr, "res_num0=%d, offset=%d\n", res_num0, offset);
#endif

  unsigned int res_num = res_num0;
  char *buf_pos = buf_top + offset;
  size_t rest_len = buf_len - offset;
  char *tmp_pos;
  char num_buf[64];
  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n',
					       rest_len))) != NULL)
    {
      size_t len = tmp_pos - buf_pos + 1;
      offset = buf_pos - buf_top;
#if 0
      char *line = strndup(buf_pos, len);
      fprintf(stderr, "res_num=%u: \"%s\"\n",
	      static_cast<unsigned int>(res_num), line);
      free(line);
#endif
      char *delim = strnstr(buf_pos, "<>", len);
      if (delim != NULL)
	{
	  size_t num_len = delim - buf_pos;
	  if (num_len > 0 && num_len < 63)
	    {
	      memcpy(num_buf, buf_pos, num_len);
	      num_buf[num_len] = '\0';
	      unsigned int num;
	      if (sscanf(num_buf, "%u", &num) == 1)
		{
		  while (res_num < num)
		    set_deleted(res_num++, offset);
		  if (res_num == num)
		    set_hints(res_num++, offset, len);
		}
	    }
	}
      else
	set_broken(res_num++, offset, len);
      buf_pos = tmp_pos + 1;
      rest_len -= len;
    }
}


void
ResponseCursorJbbs::access_finished_cb(NetworkAgent *agent, RefCount *data)
{
  buffer->fix();
  state_changed();
  const char *last_modified = agent->get_header(HTTPHeader::Last_Modified);
  if (last_modified != NULL)
    thread->set_last_modified(last_modified);
  const char *date = agent->get_header(HTTPHeader::Date);
  if (date != NULL)
    thread->set_date(date);

  if (number_of_responses_got < thread->get_number_of_responses_got())
    {
      char buf[PATH_MAX];
      if (thread->find_dat_file(repository, buf, PATH_MAX, true) != NULL)
	agent->save_buffer(buf, last_modified);
    }
}


void
ResponseCursorJbbs::access_failed_cb(NetworkAgent *agent,
				     const char *reason, RefCount *data)
{
  buffer->fix();
  state_changed();
}


void
ResponseCursorJbbs::access_terminated_cb(NetworkAgent *agent, RefCount *data)
{
  buffer->fix();
  state_changed();
}


class MachiBBSBuffer : public Buffer
{
private:
  ResponseCursorMachiBBS *cursor;
  CStringBuffer pending_buffer;
  CStringBuffer buffer;
  Regexp title;
  Regexp res;
  char *title_text;


public:
  MachiBBSBuffer(ResponseCursorMachiBBS *cursor);
  virtual ~MachiBBSBuffer();

  virtual bool append_data(const char *data, size_t length, int lock_id);
};


MachiBBSBuffer::MachiBBSBuffer(ResponseCursorMachiBBS *cursor)
  : cursor(cursor),
    title("<title>(.*)</title>", IconvBuffer::CP932_ENCODING),
    res("(?i:<dt>)([1-9][0-9]*) " NAME_CP932
	"[^<]*(?:<(?i:font)[^>]*|<(?i:a) href=\"mailto:([^\"]*)\"[^>]*)>"
	"<[bB]> (.*) </[bB]>.*" DATE_CP932
	" ([^<I\\[]*)"
	"(?: ((?:ID:[^ ]+)?"
#if 0
	" ?(?:<font[^>]*>)?(?:\\[ [^\\]]+  ?\\])?(?:</font>)?))?"
#else
	" ?(?:<font[^>]*>)?[^<]*?(?:</font>)?))?"
#endif
	"(?i:<br><dd>)(.*) (?i:<br><br>)$",
	IconvBuffer::CP932_ENCODING),
    title_text(NULL)
{
  // 0: 
  // 1: 쥹ֹ
  // 2: ᥤ
  // 3: ̾
  // 4: 
  // 5: ID/host
  // 6: ʸ
  assert(title.is_valid());
  assert(res.is_valid());
}


MachiBBSBuffer::~MachiBBSBuffer()
{
  if (title_text != NULL)
    free(title_text);
}


bool
MachiBBSBuffer::append_data(const char *data, size_t length, int lock_id)
{
  buffer.clear();
  buffer.append(pending_buffer.get_string(),
		pending_buffer.get_string_length());
  buffer.append(data, length);

  char *buf_top = buffer.get_buffer();
  size_t buf_len = buffer.get_string_length();

  char *buf_pos = buf_top;
  size_t rest_len = buf_len;
  unsigned int res_num = cursor->get_next_res_num();
  if (res_num == 1 && title_text == NULL)
    {
      char *tmp_pos;
      while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n', rest_len)))
	     != NULL)
	{
	  size_t len = tmp_pos - buf_pos + 1;
	  if (title.match(buf_pos, len))
	    {
	      title_text = strndup(title.match_begin(1), title.match_len(1));
	      buf_pos = tmp_pos + 1;
	      rest_len -= len;
	      break;
	    }
	  buf_pos = tmp_pos + 1;
	  rest_len -= len;
	}
    }

  char *tmp_pos;
  bool two_line = false;
  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n', rest_len)))
	 != NULL)
    {
      size_t len = tmp_pos - buf_pos + 1;
      unsigned int num;
      if (res.match(buf_pos, len)
	  && sscanf(res.match_begin(1), "%u", &num) == 1 && res_num <= num)
	{
	  if (res_num < num)
	    res_num = num;
	  Buffer::append_data(res.match_begin(1), res.match_len(1), lock_id);
	  Buffer::append_data("<>", 2, lock_id);
	  Buffer::append_data(res.match_begin(3), res.match_len(3), lock_id);
	  Buffer::append_data("<>", 2, lock_id);
	  Buffer::append_data(res.match_begin(2), res.match_len(2), lock_id);
	  Buffer::append_data("<>", 2, lock_id);
	  Buffer::append_data(res.match_begin(4), res.match_len(4), lock_id);
	  Buffer::append_data("<>", 2, lock_id);
	  Buffer::append_data(res.match_begin(6), res.match_len(6), lock_id);
	  Buffer::append_data("<>", 2, lock_id);
	  if (res_num == 1 && title_text != NULL)
	    Buffer::append_data(title_text, strlen(title_text), lock_id);
	  Buffer::append_data("<>", 2, lock_id);
	  Buffer::append_data(res.match_begin(5), res.match_len(5), lock_id);
	  Buffer::append_data("\n", 1, lock_id);
	  ++res_num;
	  two_line = false;
	}
      else
	{
	  if (strncasecmp(buf_pos, "<dt>", 4) == 0)
	    {
	      if (!two_line)
		{
		  *tmp_pos = ' ';	// ιԤȷҤǺĩ
		  two_line = true;
		  continue;
		}
#ifndef NDEBUG
	      if (strncasestr(buf_pos, "thread_maintenance.cgi", len) == NULL)
		{
		  char *line = strndup(buf_pos, len);
		  fprintf(stderr, "res_num(%u): \"%s\"\n",
			  static_cast<unsigned int>(res_num), line);
		  free(line);
		}
#endif
	    }
	}
      buf_pos = tmp_pos + 1;
      rest_len -= len;
    }

  pending_buffer.clear();
  pending_buffer.append(buf_pos, rest_len);

  return true;
}


ResponseCursorMachiBBS::ResponseCursorMachiBBS(BBSThread *thread,
					       Configuration &config,
					       Repository &repository,
					       NetworkAgent *agent)
  : ResponseCursorJbbs(thread, config, repository, new MachiBBSBuffer(this),
		       agent)
{
  if (!restore_hints())
    {
      thread->set_last_modified(NULL);	// ̵̣ʤΤǾõ
      clear_hints();
    }

  buffer->fix();
}


ResponseCursorMachiBBS::~ResponseCursorMachiBBS()
{
}


// (ͤDATߴ)JBBS饤֥ɥrawmode.cgi
// Ѵ롣
bool
ResponseCursorMachiBBS::convert_dat()
{
  buffer->clear0();
  clear_hints();

  bool conversion_failed = false;
  int fd = thread->open_dat_file(repository, O_RDONLY);
  if (fd >= 0)
    {
      BufferHandle old_dat = new Buffer();
      if (old_dat->read_file(fd))
	{
	  // 0: 
	  // 1: ̾<>ᥤ<>
	  // 2: ID/host
	  // 3: <>ʸ
	  // 4: <>ȥ
	  Regexp pattern("(.*<>[^<]*<>[^<I\\[]*)"
			 "(?: ((?:ID:[^ ]+)? ?"
			 "(?:<font[^>]*>)?(?:\\[ [^\\]]+ \\])?(?:</font>)?))?"
			 "(<>.*) (<>.*)$",
			 get_bbs_encoding(thread->get_bbs_type()));
	  assert(pattern.is_valid());

	  char *buf_top = old_dat->get_buffer0();
	  size_t buf_len = old_dat->get_length0();

	  char *buf_pos = buf_top;
	  size_t rest_len = buf_len;
	  char *tmp_pos;
	  unsigned int res_num = 1;
	  char num_buf[64];
	  int lock_id = buffer->get_lock()->wrlock();
	  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n',
						       rest_len))) != NULL)
	    {
	      size_t len = tmp_pos - buf_pos + 1;
	      int offset = buffer->get_length0();
	      
	      if (tmp_pos != buf_pos && pattern.match(buf_pos, len))
		{
		  snprintf(num_buf, 64, "%u", res_num);
		  buffer->Buffer::append_data(num_buf, strlen(num_buf),
					      lock_id);
		  buffer->Buffer::append_data("<>", 2, lock_id);
		  buffer->Buffer::append_data(pattern.match_begin(1),
					      pattern.match_len(1), lock_id);
		  buffer->Buffer::append_data(pattern.match_begin(3),
					      pattern.match_len(3), lock_id);
		  buffer->Buffer::append_data(pattern.match_begin(4),
					      pattern.match_len(4), lock_id);
		  buffer->Buffer::append_data("<>", 2, lock_id);
		  buffer->Buffer::append_data(pattern.match_begin(2),
					      pattern.match_len(2), lock_id);
		  buffer->Buffer::append_data("\n", 1, lock_id);
		  set_hints(res_num++, offset, buffer->get_length0() - offset);
		}
	      else
		{
		  if (tmp_pos != buf_pos)
		    {
		      // ޥå͡
#ifndef NDEBUG
		      char *line = strndup(buf_pos, len);
		      fprintf(stderr, "Unmatched line: \"%s\"\n", line);
		      free(line);
#endif
		      conversion_failed = true;
		    }
		  set_deleted(res_num++, offset);
		}

	      buf_pos = tmp_pos + 1;
	      rest_len -= len;
	    }
	  buffer->get_lock()->unlock(lock_id);

	  // ѴΤ¸
	  buffer->fix();

	  if (conversion_failed)
	    {
	      const char *thread_title = thread->get_title();
	      if (thread_title == NULL)
		thread_title = _("Anonymous Thread");

	      const char *board_name = thread->get_board()->get_name();
	      if (board_name == NULL)
		board_name = _("Anonymous Board");

	      char *thread_url = thread->get_url();
	      if (thread_url == NULL)
		{
		  fprintf(stderr, LIB_("Conversion of old-fashioned pseudo DAT file for: \"%s\" thread @ \"%s\" board failed: url=NULL!!!  fatal failure.  Out of memory?\n"),
			  thread_title, board_name);
		  exit(EXIT_FAILURE);
		}

	      char dat_path[PATH_MAX];
	      if (thread->find_dat_file(repository, dat_path, PATH_MAX, false)
		  != NULL)
		fprintf(stderr, LIB_("Conversion of old-fashioned pseudo DAT file for: \"%s\" thread @ \"%s\" board failed: url=%s\nPlease report if you can show us the backup file (%s.bak)\n"),
			thread_title, board_name, thread_url, dat_path);
	      else
		fprintf(stderr, LIB_("Conversion of old-fashioned pseudo DAT file for: \"%s\" thread @ \"%s\" board failed: url=%s\n"),
			thread_title, board_name, thread_url);
	      free(thread_url);

	      // Хååפ
	      thread->rename_dat_file_for_backup(repository);
	    }

	  fd = thread->open_dat_file(repository, O_WRONLY | O_CREAT | O_TRUNC);
	  if (fd >= 0)
	    buffer->write_file(fd);

	  return true;
	}
    }
  return false;
}


bool
ResponseCursorMachiBBS::update()
{
  if (static_cast<NetworkAgent *>(agent) == NULL)
    return false;

  LockBlock lock_obj(libochusha_giant);

  int lock_id = lock();
  if (lock_id == 0)
    return false;		// ̿Ūġġ

  if (config.get_offline())
    {
      unlock(lock_id);
      return false;
    }

  if (!buffer->is_fixed())	// ¾ΥƥȤǹ
    {
      unlock(lock_id);
      return false;
    }

  if (!thread->is_alive())
    {
      unlock(lock_id);
      return false;		// ⤦Ϥʤ
    }

  number_of_responses_got = thread->get_number_of_responses_got();

  char *url = thread->get_url(thread->get_number_of_responses_got() + 1);
  if (url != NULL)
    {
      buffer->unfix0();
      unlock(lock_id);
      agent->get(url, buffer, this);
      free(url);
      return true;
    }

  unlock(lock_id);
  return false;
}


PostResponseAgentJbbs::PostResponseAgentJbbs(BulletinBoard *board,
					     Configuration &config,
					     NetworkAgent *agent)
  : PostResponseAgent(board, config, agent),
    to_bbs_converter(get_bbs_encoding(board->get_bbs_type()),
		     IconvBuffer::UTF8_ENCODING),
    from_bbs_converter(IconvBuffer::UTF8_ENCODING,
		       get_bbs_encoding(board->get_bbs_type())),
    result_buffer(new Buffer()),
    referer_url(strdup(board->get_base_url())),
    title(NULL),
    name(NULL),
    mailto(NULL),
    message(NULL)
{
  time_t time_val = -1;
  const char *date = board->get_date();
  if (date != NULL)
    time_val = get_utc_from_date(date);

  if (time_val == -1)
    time_val = time(NULL);

  utc_time = time_val;

  agent->prepare_request.connect(this,
				 &PostResponseAgentJbbs::prepare_request_cb);
  agent->access_finished.connect(this,
				 &PostResponseAgentJbbs::access_finished_cb);
  agent->access_failed.connect(this, &PostResponseAgentJbbs::access_failed_cb);
  agent->access_terminated.connect(this,
				   &PostResponseAgentJbbs::access_terminated_cb);
}


PostResponseAgentJbbs::PostResponseAgentJbbs(BBSThread *thread,
					     Configuration &config,
					     NetworkAgent *agent)
  : PostResponseAgent(thread, config, agent),
    to_bbs_converter(get_bbs_encoding(thread->get_bbs_type()),
		     IconvBuffer::UTF8_ENCODING),
    from_bbs_converter(IconvBuffer::UTF8_ENCODING,
		       get_bbs_encoding(thread->get_bbs_type())),
    result_buffer(new Buffer()),
    referer_url(thread->get_url()),
    title(NULL),
    name(NULL),
    mailto(NULL),
    message(NULL)
{
  time_t time_val = -1;
  const char *date = thread->get_date();
  if (date != NULL)
    time_val = get_utc_from_date(date);

  if (time_val == -1)
    time_val = time(NULL);

  utc_time = time_val;

  agent->prepare_request.connect(this,
				 &PostResponseAgentJbbs::prepare_request_cb);
  agent->access_finished.connect(this,
				 &PostResponseAgentJbbs::access_finished_cb);
  agent->access_failed.connect(this, &PostResponseAgentJbbs::access_failed_cb);
  agent->access_terminated.connect(this,
				   &PostResponseAgentJbbs::access_terminated_cb);
}


PostResponseAgentJbbs::~PostResponseAgentJbbs()
{
  if (referer_url != NULL)
    free(referer_url);

  if (title != NULL)
    free(title);

  if (name != NULL)
    free(name);

  if (mailto != NULL)
    free(mailto);

  if (message != NULL)
    free(message);
}


void
PostResponseAgentJbbs::prepare_request_cb(NetworkAgent *agent, RefCount *data)
{
  agent->set_header(HTTPHeader::User_Agent, OCHUSHA_USER_AGENT);
  agent->set_header(HTTPHeader::Accept_Charset,
		    "Shift_JIS,EUC-JP,utf-8;q=0.66,*;q=0.66");
  agent->set_header(HTTPHeader::Content_Type,
		    "application/x-www-form-urlencoded");
  agent->set_header(HTTPHeader::Referer, referer_url);

  if (config.get_enable_proxy_to_post())
    agent->set_proxy_uri(config.get_proxy_uri_to_post());
}


void
PostResponseAgentJbbs::access_finished_cb(NetworkAgent *agent, RefCount *data)
{
  from_bbs_converter.clear();

  RWLock *lock = result_buffer->get_lock();
  int lock_id = lock->rdlock();	// ̵̣ġ

  from_bbs_converter.append(result_buffer->get_buffer(lock_id),
			    result_buffer->get_length(lock_id));
  lock->unlock(lock_id);

  const char *buf = from_bbs_converter.get_buffer();

  if (agent->get_status_code() == 200)
    {
      time_t time_val = -1;
      const char *date = agent->get_header(HTTPHeader::Date);
      if (date != NULL)
	time_val = get_utc_from_date(date);

      if (time_val == -1)
	time_val = time(NULL);

      utc_time = time_val;
#if 0
      fprintf(stderr, "Post may be succeeded: %s\n", buf);
#endif

      post_finished(this);
      return;
    }
  post_failed(this, buf);
}


void
PostResponseAgentJbbs::access_failed_cb(NetworkAgent *agent, const char *reason,
					RefCount *data)
{
  if (agent->get_status_code() == 302)
    {
      int bbs_type = get_board()->get_bbs_type();
      if (bbs_type == OCHUSHA_BBS_TYPE_MACHIBBS)
	{
	  // ޤ££Ӥξ¿ʬ񤭹
	  post_finished(this);
	  return;
	}
    }
  post_failed(this, reason);
}


void
PostResponseAgentJbbs::access_terminated_cb(NetworkAgent *agent, RefCount *data)
{
  post_failed(this,
	      LIB_("Post terminated.  It isn't clear whether posting accepted or not."));
}


void
PostResponseAgentJbbs::do_post(const char *kakikomu)
{
  assert(message != NULL);
  assert(*message != '\0');

  const char *name = this->name;
  if (name == NULL)
    name = "";

  const char *mailto = this->mailto;
  if (mailto == NULL)
    mailto = "";

  BulletinBoard *board = get_board();
  char cgi_url[PATH_MAX];
  char *bbs_root = board->get_bbs_root();
  if (bbs_root == NULL)
    {
      post_failed(this, LIB_("Out of memory"));
      return;
    }


  char *bbs = board->get_id();
  if (bbs == NULL)
    {
      free(bbs_root);
      post_failed(this, LIB_("Out of memory"));
      return;
    }

  string key;
  string subject;
  if (get_thread() != NULL)
    key.append("&KEY=").append(get_thread()->get_id());
  else if (title != NULL)
    subject.append("&SUBJECT=").append(title);

  assert(key.c_str()[0] != '\0' || subject.c_str()[0] != '\0');

  int len;
  const char *dir = NULL;
  const char *bbs2 = NULL;
  switch (board->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_MACHIBBS:
      len = snprintf(cgi_url, PATH_MAX, "%sbbs/write.cgi", bbs_root);
      break;

    case OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR:
      {
	const char *id;
	if (get_thread() != NULL)
	  id = get_thread()->get_id();
	else
	  id = "new";

	len = snprintf(cgi_url, PATH_MAX, "%sbbs/write.cgi/%s/%s/",
		       bbs_root, bbs, id);
	dir = bbs;
	char *slash_pos = strchr(bbs, '/');
	assert(slash_pos != NULL);
	assert(slash_pos != dir);
	*slash_pos = '\0';
	bbs2 = slash_pos + 1;
      }
      break;

    default:
      {
	free(bbs_root);
	free(bbs);
	post_failed(this, LIB_("Wrong Way!"));
	return;
      }
    }

  if (len >= PATH_MAX)
    {
      post_failed(this, LIB_("Too long URL"));
      free(bbs_root);
      return;
    }
  free(bbs_root);

  int query_len = 2048;	// 2ܤƤȤΤǼ¼4096ХȤĩ
  char *query = NULL;
  do
    {
      query_len *= 2;
      char *tmp_buf = static_cast<char *>(realloc(query, query_len));
      if (tmp_buf == NULL)
	{
	  post_failed(this, LIB_("Out of memory"));
	  if (query != NULL)
	    free(query);
	  return;
	}
      query = tmp_buf;

      switch (board->get_bbs_type())
	{
	case OCHUSHA_BBS_TYPE_MACHIBBS:
	  len = snprintf(query, query_len,
			 "submit=%s%s&NAME=%s&MAIL=%s&MESSAGE=%s&BBS=%s"
			 "%s&TIME=%jd",
			 kakikomu, subject.c_str(), name, mailto, message, bbs,
			 key.c_str(), static_cast<intmax_t>(utc_time));
	  break;

	case OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR:
	  len = snprintf(query, query_len,
			 "submit=%s%s&NAME=%s&MAIL=%s&MESSAGE=%s&BBS=%s"
			 "%s&DIR=%s&TIME=%jd",
			 kakikomu, subject.c_str(), name, mailto, message, bbs2,
			 key.c_str(), dir, static_cast<intmax_t>(utc_time));
	  break;
	}
    }
  while (len >= query_len);

  result_buffer->clear(0);
  result_buffer->unfix0();

  smart_ptr<PostResponseAgent> tmp_ref = this;
  get_agent()->post(cgi_url, query, result_buffer, this);
  free(query);
  free(bbs);
}


void
PostResponseAgentJbbs::try_new_thread(const char *title, const char *name,
				      const char *mailto, const char *message)
{
  if (title == NULL || *title == '\0')
    {
      post_failed(this, LIB_("Title is empty"));
      return;
    }

  if (message == NULL || *message == '\0')
    {
      post_failed(this, LIB_("Message is empty"));
      return;
    }

  if (this->title != NULL)
    {
      free(this->title);
      this->title = NULL;
    }

  to_bbs_converter.clear();
  to_bbs_converter.append(title, strlen(title));
  this->title = URI::encode_string(to_bbs_converter.get_string());

  if (this->name != NULL)
    {
      free(this->name);
      this->name = NULL;
    }

  if (name != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(name, strlen(name));
      this->name = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->mailto != NULL)
    {
      free(this->mailto);
      this->mailto = NULL;
    }

  if (mailto != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(mailto, strlen(mailto));
      this->mailto = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->message != NULL)
    {
      free(this->message);
      this->message = NULL;
    }

  if (message != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(message, strlen(message));
      this->message = URI::encode_string(to_bbs_converter.get_string());
    }

  switch (get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR:
      do_post(SHINKI_KAKIKOMI_EUCJP_URIENC);
      return;

    default:
      do_post(SHINKI_KAKIKOMI_CP932_URIENC);
      return;
    }
}


void
PostResponseAgentJbbs::try_post(const char *name, const char *mailto,
				const char *message)
{
  if (message == NULL || *message == '\0')
    {
      post_failed(this, LIB_("Message is empty"));
      return;
    }

  if (this->name != NULL)
    {
      free(this->name);
      this->name = NULL;
    }

  if (name != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(name, strlen(name));
      this->name = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->mailto != NULL)
    {
      free(this->mailto);
      this->mailto = NULL;
    }

  if (mailto != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(mailto, strlen(mailto));
      this->mailto = URI::encode_string(to_bbs_converter.get_string());
    }

  if (this->message != NULL)
    {
      free(this->message);
      this->message = NULL;
    }

  if (message != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(message, strlen(message));
      this->message = URI::encode_string(to_bbs_converter.get_string());
    }

  switch (get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR:
      do_post(KAKIKOMU_EUCJP_URIENC);
      return;

    default:
      do_post(KAKIKOMU_CP932_URIENC);
      return;
    }
}


void
PostResponseAgentJbbs::retry_post(const char *submit, time_t time_val)
{
  if (message == NULL || *message == '\0')
    {
      post_failed(this, LIB_("Message is empty"));
      return;
    }

  if (time_val != -1)
    utc_time = time_val;

  char *kakikomu;
  if (submit != NULL)
    {
      to_bbs_converter.clear();
      to_bbs_converter.append(submit, strlen(submit));
      kakikomu = URI::encode_string(to_bbs_converter.get_string());

      do_post(kakikomu);

      free(kakikomu);
      return;
    }

  switch (get_board()->get_bbs_type())
    {
    case OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR:
      do_post(SHODAKU_KAKIKOMU_EUCJP_URIENC);
      return;

    default:
      do_post(SHODAKU_KAKIKOMU_CP932_URIENC);
      return;
    }
}


RefCountHashtable BBSMachiBBS::table(101);

void
BBSMachiBBS::register_board(BulletinBoard *board)
{
  URI base_uri(board->get_base_url());
  if (!base_uri.is_valid())
    return;

  int type = guess_board_type(base_uri);
  if (type != OCHUSHA_BBS_TYPE_MACHIBBS)
    {
      board->set_bbs_type(OCHUSHA_BBS_TYPE_UNKNOWN);
      return;
    }

  const char *path = base_uri.get_path();
  if (path == NULL || *path != '/')
    return;	// ޤ££ӤĤˤƤϲ

  BulletinBoard *entry = dynamic_cast<BulletinBoard *>(table.lookup(path));
  if (entry != NULL)
    {
      if (entry != board)
	fprintf(stderr,
		"multiple instance of board identified by %s found: \"%s\" v.s. \"%s\"\n",
		path, entry->get_name(), board->get_name());
      return;
    }

  char *canonical_base_url = guess_board_url(base_uri);
  if (canonical_base_url == NULL)
    return;	// äȲδְ㤤

  if (strcmp(board->get_base_url(), canonical_base_url) == 0)
    table.insert(path, board);
  else
    board->set_base_url(canonical_base_url);
  free(canonical_base_url);
  return;
}


BulletinBoard *
BBSMachiBBS::guess_board(const URI &uri)
{
  assert(uri.is_valid());

  int board_type = guess_board_type(uri);
  if (board_type != OCHUSHA_BBS_TYPE_MACHIBBS)
    return NULL;

  BBSURLInfo info(uri, 1);
  const char *board_id = info.get_board_id();
  if (board_id != NULL)
    return dynamic_cast<BulletinBoard *>(table.lookup(board_id));
  return NULL;
}


int
BBSMachiBBS::guess_board_type(const URI &uri)
{
  assert(uri.is_valid());

  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return OCHUSHA_BBS_TYPE_UNKNOWN;

  const char *server = uri.get_server();
  if (server == NULL)
    return OCHUSHA_BBS_TYPE_UNKNOWN;

  const char *tmp_pos = strcasestr(server, ".machi.to");
  if (tmp_pos != NULL && tmp_pos[9] == '\0')
    return OCHUSHA_BBS_TYPE_MACHIBBS;

  tmp_pos = strcasestr(server, ".machibbs.com");
  if (tmp_pos != NULL && tmp_pos[13] == '\0')
    return OCHUSHA_BBS_TYPE_MACHIBBS;

  return OCHUSHA_BBS_TYPE_UNKNOWN;
}


char *
BBSMachiBBS::guess_board_url(const URI &uri)
{
  assert(uri.is_valid());
  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return NULL;

  const char *server = uri.get_server();
  if (server == NULL)
    return NULL;

  const char *tmp_pos = strcasestr(server, ".machi.to");
  if (tmp_pos == NULL || tmp_pos[9] != '\0')
    {
      tmp_pos = strcasestr(server, ".machibbs.com");
      if (tmp_pos == NULL || tmp_pos[13] != '\0')
	return NULL;
    }

  const char *path = uri.get_path();
  if (*path != '/')
    return NULL;

  BBSURLInfo info(uri, 1);
  if (info.get_board_id() == NULL)
    return NULL;

  string board_url;
  board_url.append(scheme).append("://").append(server).append("/")
    .append(info.get_board_id()).append("/");
  return strdup(board_url.c_str());
}


char *
BBSMachiBBS::get_board_id(const BulletinBoard *board)
{
  return BBS2ch::get_board_id(board);	// ͤƱɤ
}


char *
BBSMachiBBS::get_bbs_root(const BulletinBoard *board)
{
  return BBS2ch::get_bbs_root(board);	// ͤƱɤ
}


char *
BBSMachiBBS::get_url(BBSThread *thread, int from, int to)
{
  assert(thread != NULL);
  URI board_uri(thread->get_board()->get_base_url());
  if (!board_uri.is_valid()
      || board_uri.get_server() == NULL || board_uri.get_path() == NULL)
    return NULL;

  char board_id[PATH_MAX];
  if (snprintf(board_id, PATH_MAX, "%s", board_uri.get_path()) >= PATH_MAX)
    return NULL;	// Ūʤ

  assert(*board_id == '/');

  char *tmp_pos = strchr(board_id + 1, '/');
  if (tmp_pos == NULL || tmp_pos[1] != '\0')
    return NULL;	// ͡

  *tmp_pos = '\0';
  const char *bbs = board_id + 1;

  char url_buffer[PATH_MAX];
  if (from == 0)
    snprintf(url_buffer, PATH_MAX, "http://%s/bbs/read.cgi?BBS=%s&KEY=%s",
	     board_uri.get_server(), bbs, thread->get_id());
  else if (from < 0)
    snprintf(url_buffer, PATH_MAX,
	     "http://%s/bbs/read.cgi?BBS=%s&KEY=%s&LAST=%d",
	     board_uri.get_server(), bbs, thread->get_id(), -from);
  else if (from < to)
    snprintf(url_buffer, PATH_MAX,
	     "http://%s/bbs/read.cgi?BBS=%s&KEY=%s&START=%d&END=%d&NOFIRST=TRUE",
	     board_uri.get_server(), bbs, thread->get_id(), from, to);
  else if (from != 1)
    snprintf(url_buffer, PATH_MAX,
	     "http://%s/bbs/read.cgi?BBS=%s&KEY=%s&START=%d&NOFIRST=TRUE",
	     board_uri.get_server(), bbs, thread->get_id(), from);
  else
    snprintf(url_buffer, PATH_MAX,
	     "http://%s/bbs/read.cgi?BBS=%s&KEY=%s&START=%d",
	     board_uri.get_server(), bbs, thread->get_id(), from);

  return strdup(url_buffer);
}


char *
BBSMachiBBS::parse_thread_url(const char *url, int *from_p, int *to_p)
{
  BBSURLInfo info(url, 1);
  if (info.get_thread_id() != NULL)
    {
      if (from_p != NULL)
	*from_p = info.get_from();
      if (to_p != NULL)
	*to_p = info.get_to();
      return strdup(info.get_thread_id());
    }
  return NULL;
}


ResponseCursorJbbsLivedoor::ResponseCursorJbbsLivedoor(BBSThread *thread,
						       Configuration &config,
						       Repository &repository,
						       NetworkAgent *agent)
  : ResponseCursorJbbs(thread, config, repository, new Buffer(), agent)
{
  if (!restore_hints())
    {
      thread->set_last_modified(NULL);	// ̵̣ʤΤǾõ
      clear_hints();
    }

  buffer->fix();
}


ResponseCursorJbbsLivedoor::~ResponseCursorJbbsLivedoor()
{
}


// (ͤDATߴ)JBBS饤֥ɥrawmode.cgi
// Ѵ롣
bool
ResponseCursorJbbsLivedoor::convert_dat()
{
  buffer->clear0();
  clear_hints();

  bool conversion_failed = false;
  int fd = thread->open_dat_file(repository, O_RDONLY);
  if (fd >= 0)
    {
      BufferHandle old_dat = new Buffer();
      if (old_dat->read_file(fd))
	{
	  // 0: 
	  // 1: ̾<>ᥤ<>
	  // 2: ID/host
	  // 3: <>ʸ
	  // 4: <>ȥ
	  Regexp pattern("(.*<>[^<]*<>[^<\\[]*)(?: (?:<font[^>]*>)?\\[ (.+) \\](?:</font>)?)?(<>.*) (<>.*)$",
			 get_bbs_encoding(thread->get_bbs_type()));
	  assert(pattern.is_valid());

	  char *buf_top = old_dat->get_buffer0();
	  size_t buf_len = old_dat->get_length0();

	  char *buf_pos = buf_top;
	  size_t rest_len = buf_len;
	  char *tmp_pos;
	  unsigned int res_num = 1;
	  char num_buf[64];
	  int lock_id = buffer->get_lock()->wrlock();
	  while ((tmp_pos = static_cast<char *>(memchr(buf_pos, '\n',
						       rest_len))) != NULL)
	    {
	      size_t len = tmp_pos - buf_pos + 1;
	      int offset = buffer->get_length0();
	      
	      if (tmp_pos != buf_pos && pattern.match(buf_pos, len))
		{
		  snprintf(num_buf, 64, "%u", res_num);
		  buffer->append_data(num_buf, strlen(num_buf), lock_id);
		  buffer->append_data("<>", 2, lock_id);
		  buffer->append_data(pattern.match_begin(1),
				      pattern.match_len(1), lock_id);
		  buffer->append_data(pattern.match_begin(3),
				      pattern.match_len(3), lock_id);
		  buffer->append_data(pattern.match_begin(4),
				      pattern.match_len(4), lock_id);
		  buffer->append_data("<>", 2, lock_id);
		  buffer->append_data(pattern.match_begin(2),
				      pattern.match_len(2), lock_id);
		  buffer->append_data("\n", 1, lock_id);
		  set_hints(res_num++, offset, buffer->get_length0() - offset);
		}
	      else
		{
		  if (tmp_pos != buf_pos)
		    {
#ifndef NDEBUG
		      // ޥå͡
		      char *line = strndup(buf_pos, len);
		      fprintf(stderr, "Unmatched line: \"%s\"\n", line);
		      free(line);
#endif
		      conversion_failed = true;
		    }
		  set_deleted(res_num++, offset);
		}

	      buf_pos = tmp_pos + 1;
	      rest_len -= len;
	    }
	  buffer->get_lock()->unlock(lock_id);

	  // ѴΤ¸
	  buffer->fix();

	  if (conversion_failed)
	    {
	      const char *thread_title = thread->get_title();
	      if (thread_title == NULL)
		thread_title = _("Anonymous Thread");

	      const char *board_name = thread->get_board()->get_name();
	      if (board_name == NULL)
		board_name = _("Anonymous Board");

	      char *thread_url = thread->get_url();
	      if (thread_url == NULL)
		{
		  fprintf(stderr, LIB_("Conversion of old-fashioned pseudo DAT file for: \"%s\" thread @ \"%s\" board failed: url=NULL!!!  fatal failure.  Out of memory?\n"),
			  thread_title, board_name);
		  exit(EXIT_FAILURE);
		}

	      char dat_path[PATH_MAX];
	      if (thread->find_dat_file(repository, dat_path, PATH_MAX, false)
		  != NULL)
		fprintf(stderr, LIB_("Conversion of old-fashioned pseudo DAT file for: \"%s\" thread @ \"%s\" board failed: url=%s\nPlease report if you can show us the backup file (%s.bak)\n"),
			thread_title, board_name, thread_url, dat_path);
	      else
		fprintf(stderr, LIB_("Conversion of old-fashioned pseudo DAT file for: \"%s\" thread @ \"%s\" board failed: url=%s\n"),
			thread_title, board_name, thread_url);
	      free(thread_url);

	      // Хååפ
	      thread->rename_dat_file_for_backup(repository);
	    }

	  fd = thread->open_dat_file(repository, O_WRONLY | O_CREAT | O_TRUNC);
	  if (fd >= 0)
	    buffer->write_file(fd);

	  return true;
	}
    }
  return false;
}


bool
ResponseCursorJbbsLivedoor::update()
{
  if (static_cast<NetworkAgent *>(agent) == NULL)
    return false;

  LockBlock lock_obj(libochusha_giant);

  int lock_id = lock();
  if (lock_id == 0)
    return false;		// ̿Ūġġ

  if (config.get_offline())
    {
      unlock(lock_id);
      return false;
    }

  if (!buffer->is_fixed())	// ¾ΥƥȤǹ
    {
      unlock(lock_id);
      return false;
    }

  if (!thread->is_alive())
    {
      unlock(lock_id);
      return false;		// ⤦Ϥʤ
    }

  URI uri(thread->get_board()->get_base_url());
  assert(uri.is_valid());

  number_of_responses_got = thread->get_number_of_responses_got();

  char rawmode_url[PATH_MAX];
  if (snprintf(rawmode_url, PATH_MAX, "http://%s/bbs/rawmode.cgi%s%s/%u-",
	       uri.get_server(), uri.get_path(), thread->get_id(),
	       thread->get_number_of_responses_got() + 1) < PATH_MAX)
    {
      buffer->unfix0();
      unlock(lock_id);
      agent->get(rawmode_url, buffer, this);
      return true;
    }

  unlock(lock_id);
  return false;
}


RefCountHashtable BBSJbbsLivedoor::table(101);


void
BBSJbbsLivedoor::register_board(BulletinBoard *board)
{
  URI base_uri(board->get_base_url());
  if (!base_uri.is_valid())
    return;

  int type = guess_board_type(base_uri);
  if (type != OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR)
    {
      board->set_bbs_type(OCHUSHA_BBS_TYPE_UNKNOWN);
      return;
    }

  const char *path = base_uri.get_path();
  if (path == NULL || *path != '/')
    return;	// JBBS饤֥ɥĤˤƤϲ

  BulletinBoard *entry = dynamic_cast<BulletinBoard *>(table.lookup(path));
  if (entry != NULL)
    {
      if (entry != board)
	fprintf(stderr,
		"multiple instance of board identified by %s found: \"%s\" v.s. \"%s\"\n",
		path, entry->get_name(), board->get_name());
      return;
    }

  char *canonical_base_url = guess_board_url(base_uri);
  if (canonical_base_url == NULL)
    return;	// äȲ

  if (strcmp(board->get_base_url(), canonical_base_url) == 0)
    table.insert(path, board);
  else
    board->set_base_url(canonical_base_url);
  free(canonical_base_url);
}


BulletinBoard *
BBSJbbsLivedoor::guess_board(const URI &uri)
{
  assert(uri.is_valid());

  int board_type = guess_board_type(uri);
  if (board_type != OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR)
    return NULL;

  BBSURLInfo info(uri, 2);
  const char *board_id = info.get_board_id();
  if (board_id != NULL)
    return dynamic_cast<BulletinBoard *>(table.lookup(board_id));
  return NULL;
}


int
BBSJbbsLivedoor::guess_board_type(const URI &uri)
{
  assert(uri.is_valid());

  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return OCHUSHA_BBS_TYPE_UNKNOWN;

  const char *server = uri.get_server();
  if (server == NULL)
    return OCHUSHA_BBS_TYPE_UNKNOWN;

  if (strcasecmp(server, "jbbs.livedoor.jp") == 0
      || strcasecmp(server, "jbbs.livedoor.com") == 0
      || strcasecmp(server, "jbbs.shitaraba.com") == 0)
    return OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR;

  return OCHUSHA_BBS_TYPE_UNKNOWN;
}


char *
BBSJbbsLivedoor::guess_board_url(const URI &uri)
{
  assert(uri.is_valid());
  const char *scheme = uri.get_scheme();
  if (scheme == NULL || strcmp(scheme, "http") != 0)
    return NULL;

  const char *server = uri.get_server();
  if (server == NULL)
    return NULL;

  if (strcasecmp(server, "jbbs.livedoor.jp") != 0
      && strcasecmp(server, "jbbs.livedoor.com") != 0
      && strcasecmp(server, "jbbs.shitaraba.com") != 0)
    return NULL;

  const char *path = uri.get_path();
  if (*path != '/')
    return NULL;

  BBSURLInfo info(uri, 2);
  if (info.get_board_id() == NULL)
    return NULL;

  string board_url;
  board_url.append(scheme).append("://").append(server).append("/")
    .append(info.get_board_id()).append("/");
  return strdup(board_url.c_str());
}


char *
BBSJbbsLivedoor::get_board_id(const BulletinBoard *board)
{
  URI uri(board->get_base_url());
  if (!uri.is_valid() || uri.get_path() == NULL)
    return NULL;

  char id_buf[4096];
  snprintf(id_buf, 4096, "%s", uri.get_path());
  size_t len = strlen(id_buf);
  if (*id_buf != '/' || id_buf[len - 1] != '/')
    return NULL;

  id_buf[len - 1] = '\0';
  char *slash_pos = strrchr(id_buf, '/');

  // JBBS饤֥ɥǼĤǤbase_urlκǿΥ֥ǥ쥯ȥ̾ID
  // ΰľ夬ƥˤʤ롣get_board_id()"ƥ/ID"Ȥ
  // ʸ֤
  if (slash_pos == NULL || slash_pos == id_buf)
    return NULL;

#ifdef NDEBUG
  while (*--slash_pos != '/')
    ;
  if (slash_pos != id_buf)
    return NULL;
#else
  slash_pos = id_buf;
#endif
  return strdup(slash_pos + 1);
}


char *
BBSJbbsLivedoor::get_bbs_root(const BulletinBoard *board)
{
  URI uri(board->get_base_url());
  if (!uri.is_valid() || uri.get_path() == NULL)
    return NULL;

  char url_buf[4096];
  snprintf(url_buf, 4096, "%s", board->get_base_url());
  size_t len = strlen(url_buf);
  char *path_pos = strstr(url_buf, uri.get_path());
  if (path_pos == NULL || *path_pos != '/' || url_buf[len - 1] != '/')
    return NULL;

  // JBBS LivedoorʤURLpathʬκǽΥåޤǤɤ
  path_pos[1] = '\0';
  return strdup(url_buf);
}


char *
BBSJbbsLivedoor::get_url(BBSThread *thread, int from, int to)
{
  assert(thread != NULL);
  URI board_uri(thread->get_board()->get_base_url());
  if (!board_uri.is_valid()
      || board_uri.get_server() == NULL || board_uri.get_path() == NULL)
    return NULL;

  char board_id[PATH_MAX];
  if (snprintf(board_id, PATH_MAX, "%s", board_uri.get_path()) >= PATH_MAX)
    return NULL;	// Ūʤ

  assert(*board_id == '/');

  char *tmp_pos = strchr(board_id + 1, '/');
  if (tmp_pos == NULL || tmp_pos[1] == '\0' || tmp_pos[1] == '/')
    return NULL;	// ͡

  *tmp_pos = '\0';
  const char *genre = board_id + 1;
  const char *bbs = tmp_pos + 1;

  tmp_pos = strchr(tmp_pos + 1, '/');
  if (tmp_pos == NULL || tmp_pos[1] != '\0')
    return NULL;	// ͡
  *tmp_pos = '\0';

  char url_buffer[PATH_MAX];
  if (from == 0)
    snprintf(url_buffer, PATH_MAX, "http://%s/bbs/read.cgi/%s/%s/%s/",
	     board_uri.get_server(), genre, bbs, thread->get_id());
  else if (from < 0)
    snprintf(url_buffer, PATH_MAX, "http://%s/bbs/read.cgi/%s/%s/%s/l%d",
	     board_uri.get_server(), genre, bbs, thread->get_id(), -from);
  else if (from < to)
    snprintf(url_buffer, PATH_MAX, "http://%s/bbs/read.cgi/%s/%s/%s/%d-%d",
	     board_uri.get_server(), genre, bbs, thread->get_id(), from, to);
  else
    snprintf(url_buffer, PATH_MAX, "http://%s/bbs/read.cgi/%s/%s/%s/%d",
	     board_uri.get_server(), genre, bbs, thread->get_id(), from);

  return strdup(url_buffer);
}


char *
BBSJbbsLivedoor::parse_thread_url(const char *url, int *from_p, int *to_p)
{
  BBSURLInfo info(url, 2);
  if (info.get_thread_id() != NULL)
    {
      if (from_p != NULL)
	*from_p = info.get_from();
      if (to_p != NULL)
	*to_p = info.get_to();
      return strdup(info.get_thread_id());
    }
  return NULL;
}


char *
BBSJbbsLivedoor::get_trackback_url(BBSThread *thread, int res_num)
{
  assert(thread != NULL);
  assert(thread->get_bbs_type() == OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR);

  char *res_url = get_url(thread, res_num);
  if (res_url == NULL)
    return NULL;	// Out of memory?

  string buf;
  buf.append("http://jbbs.livedoor.jp/bbs/tb.cgi/");

  BBSURLInfo info(res_url, 2);
  free(res_url);

  char board_id[PATH_MAX];
  assert(info.get_board_id() != NULL);
  if (snprintf(board_id, PATH_MAX, "%s", info.get_board_id()) < PATH_MAX)
    {
      for (char *slash_pos = board_id; *slash_pos != '\0'; slash_pos++)
	{
	  if (*slash_pos == '/')
	    *slash_pos = '_';
	}
      buf.append(board_id).append("_");

      assert(info.get_thread_id() != NULL);
      buf.append(info.get_thread_id()).append("?__mode=list");

      return strdup(buf.c_str());
    }

  return NULL;
}


}

