/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_jbbs_utils.h,v 1.18 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BBS_JBBS_UTILS_H_
#define _LIBOCHUSHA_BBS_JBBS_UTILS_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bulletin_board.h"
#include "libochusha/bbs_utils.h"
#include "libochusha/hashtable.h"
#include "libochusha/sigslot.h"
#include "libochusha/uri_utils.h"


#if defined(HAVE_STDINT_H) || !defined(HAVE_CONFIG_H)
#include <stdint.h>
#else
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif
#endif


using namespace std;
namespace Ochusha
{
  class BBSThread;
  class BulletinBoard;
  class Configuration;
  class NetworkAgent;
  class RefCount;
  class Repository;
  class URI;


  class ResponseCursorJbbs : public ResponseCursor
  {
  private:
    void prepare_request_cb(NetworkAgent *agent, RefCount *data);
    void response_header_received_cb(NetworkAgent *agent, RefCount *data);
    void access_progressed_cb(NetworkAgent *agent, size_t data_read,
			      uintmax_t total_len, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  protected:
    int number_of_responses_got;

    bool restore_hints();
    virtual bool convert_dat() = 0;
    virtual bool parse_response(unsigned int res_num,
				const char *start, size_t length);


  public:
    ResponseCursorJbbs(BBSThread *thread, Configuration &config,
		       Repository &repository, Buffer *buffer,
		       NetworkAgent *agent);
    virtual ~ResponseCursorJbbs();
  };


  class PostResponseAgentJbbs : public PostResponseAgent, public has_slots
  {
  protected:
    IconvBuffer to_bbs_converter;
    IconvBuffer from_bbs_converter;
    BufferHandle result_buffer;
    char *referer_url;
    char *title;
    char *name;
    char *mailto;
    char *message;
    time_t utc_time;

 
    void do_post(const char *kakikomu);

    void prepare_request_cb(NetworkAgent *agent, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  public:
    PostResponseAgentJbbs(BulletinBoard *board, Configuration &config,
			  NetworkAgent *agent);
    PostResponseAgentJbbs(BBSThread *thread, Configuration &config,
			  NetworkAgent *agent);
    virtual ~PostResponseAgentJbbs();

    virtual void try_new_thread(const char *title, const char *name,
				const char *mailto, const char *message);
    virtual void try_post(const char *name, const char *mailto,
			  const char *message);
    virtual void retry_post(const char *submit, time_t time_val);
  };


  class ResponseCursorMachiBBS : public ResponseCursorJbbs
  {
  protected:
    virtual bool convert_dat();


  public:
    ResponseCursorMachiBBS(BBSThread *thread, Configuration &config,
			   Repository &repository, NetworkAgent *agent);
    virtual ~ResponseCursorMachiBBS();
    inline BBSThread *get_thread() { return thread; }
    inline unsigned int get_next_res_num()
    { return ResponseCursor::get_next_res_num(); }

    virtual bool update();
  };


  class BBSMachiBBS
  {
  private:
    static RefCountHashtable table;	// IDBulletinBoard *ʥơ֥롣


  public:
    static void register_board(BulletinBoard *board);
    static BulletinBoard *guess_board(const URI &uri);
    static int guess_board_type(const URI &uri);
    static char *guess_board_url(const URI &uri);
    static char *get_board_id(const BulletinBoard *board);
    static char *get_bbs_root(const BulletinBoard *board);
    static char *get_url(BBSThread *thread, int from = 0, int to = 0);
    static char *parse_thread_url(const char *url, int *from_p, int *to_p);
  };


  class ResponseCursorJbbsLivedoor : public ResponseCursorJbbs
  {
  protected:
    virtual bool convert_dat();


  public:
    ResponseCursorJbbsLivedoor(BBSThread *thread, Configuration &config,
			       Repository &repository, NetworkAgent *agent);
    virtual ~ResponseCursorJbbsLivedoor();

    virtual bool update();
  };


  class BBSJbbsLivedoor
  {
  private:
    static RefCountHashtable table;	// IDBulletinBoard *ʥơ֥롣


  public:
    static void register_board(BulletinBoard *board);
    static BulletinBoard *guess_board(const URI &uri);
    static int guess_board_type(const URI &uri);
    static char *guess_board_url(const URI &uri);
    static char *get_board_id(const BulletinBoard *board);
    static char *get_bbs_root(const BulletinBoard *board);
    static char *get_url(BBSThread *thread, int from = 0, int to = 0);
    static char *parse_thread_url(const char *url, int *from_p, int *to_p);
    static char *get_trackback_url(BBSThread *thread, int res_num);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BBS_JBBS_UTILS_H_ */
