/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_table.h,v 1.28 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BBS_TABLE_H_
#define _LIBOCHUSHA_BBS_TABLE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "libochusha/sigslot.h"
#include "libochusha/tree.h"

#include <list>
#include <vector>

#include <zlib.h>


using namespace std;
namespace Ochusha
{
  class BulletinBoard;
  class Configuration;
  class NetworkAgentBroker;
  class Repository;
  class ThreadPool;
  class URI;

  typedef vector<TreeElementHandle> TreeElementVector;

  class BBSTable : public RootFolder
  {
  private:
    RefCountHashtable category_table;	/* bbsmenu.htmlͳΥեФ
					 * ƥ̾=>(Folder *)ʥơ֥
					 */

    RefCountHashtable board_table;	/* base_url=>(BulletinBoard *) */

    // öäBulletinBoard֥ȤϽλޤƤ
    // İԽRootFolderéĥ꡼¸ߤʤʤäƤ
    // board_tableͳǻȤ롣ɬפ˱board_table
    // äƤΤϼ֤ݤ㤤ʤ
    TreeElementVector all_boards;


  public:
    BBSTable();
    virtual ~BBSTable();

    inline TreeElementVector::iterator all_boards_begin()
    { return all_boards.begin(); }
    inline TreeElementVector::iterator all_boards_end()
    { return all_boards.end(); }

    bool read_boardlist_xml(Repository &repository);
    bool write_boardlist_xml(Repository &repository);

    virtual void prepend(TreeElement *element,
			 TreeElement *next = NULL);
    virtual void append(TreeElement *element,
			TreeElement *prev = NULL);
    virtual void remove(TreeElement *element);

    // RootFolderpure virtualμ
    virtual TreeElement *copy(const TreeElement *element);

    virtual const char *get_root_element_name() const;
    virtual const char *get_leaf_element_name() const;


  protected:
    virtual Folder *folder_new(time_t tc, time_t tlm, unsigned int sn);
    virtual Folder *folder_new(const Folder *folder, unsigned int sn);
    virtual TreeElement *leaf_element_new(AttributeTable &attributes,
					  time_t tc, time_t tlm);


  public:
    // Ϳ줿URLǼĤ¬бĤBulletinBoard *
    // Ф֤
    BulletinBoard *guess_board(const char *url);

    Folder *lookup_category(const char *name) const;
    void register_board(BulletinBoard *board, const char *base_url);
    void unregister_board(BulletinBoard *board, const char *base_url);

    BulletinBoard *lookup_board_by_url(const char *url) const;

    void update_boardlist(Configuration &config, ThreadPool &pool,
			  NetworkAgentBroker &broker);


    // update_boardlist()İžΤWorkerThreadȯԤ롣
    // void board_moved(this, board, new_url);
    signal3<BBSTable *, BulletinBoard *, const char *> board_moved;


    // ɲ
    static char *guess_board_url(const char *url);
    static int guess_board_type(const char *url);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BBS_TABLE_H_ */
