/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_thread.h,v 1.45 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BBS_THREAD_H_
#define _LIBOCHUSHA_BBS_THREAD_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/a_bone.h"
#include "libochusha/buffer.h"
#include "libochusha/bulletin_board.h"
#include "libochusha/network_agent.h"
#include "libochusha/ref_count.h"
#include "libochusha/text_utils.h"
#include "libochusha/tree.h"

#include <list>
#include <vector>

#include <cstddef>
#include <time.h>


using namespace std;
namespace Ochusha
{
  class BulletinBoard;
  class Configuration;
  class NetworkAgentBroker;
  class PostResponseAgent;
  class Repository;
  class ResponseCursor;
  class ThreadPool;

  class BBSThread : public RefCount
  {
    friend class BulletinBoard;
    friend class PostResponseAgent;
    friend class ResponseCursor;
    friend class ThreadlistUpdater;

  private:
    enum {
      DAT_DROPPED = 1 << 0,	// DAT
      STOPPED = 1 << 1,		// 쥹
      KAKO_THREAD = 1 << 2,	// HTMLѲ
      DAT_VIEWER = 1 << 3,	// Ǽ
      FAKE_DAT = 1 << 4,	// DAT֤줿DATե
      TMP_THREAD = 1 << 5	// IDΤߤȽƤ륹ɤ
    };

    enum {
      MARKED = 1 << 0,
      HIDDEN = 1 << 1,
      LAST_POST_SAGE = 1 << 2,
      LAST_POST_BE = 1 << 3,
      LAST_POST_VIEWER = 1 << 4
    };


    BulletinBoard *board;

    char *id;
    char *title;
    char *last_modified;	// DATեɤ߹߻Last-Modified
    char *date;			// DATեɤ߹߻Date

    // ʲĤϼ¼ͤѤˤʤ
    char *kako_html;		// HTMLѲξURL
    char *grave;		// DATURL

    char *trigger_url;		// lookup_thread_by_urlǸĤʤä
				// ˡɤurlõȤΤϿ
				// threadlist.xmlˤ¸ʤ

    int rank;				// ǿΥ
    int previous_rank;			// Υν
    					// 줬0ʤΤϿ줫DAT

    int number_of_responses_on_server;	// ɤ߹ȴԤǤ쥹
    int number_of_responses_got;	// DATե˴ޤޤ쥹
    int number_of_responses_read;	// Υɤ߹߻Υ쥹

    int number_of_new_responses;	// ΥΥ쥹ʬ

    int bookmark_position;		// Ѥɽ(0̵)
    int last_view_response;		// ɽƤ쥹(0)

    int shown_count;			// ˥ɽ줿

    int flags;
    int ui_flags;

    list<unsigned int> a_bone_list;	// 쥹֤Ǥܡ
    ABone a_bone;			// Ѥܡ

    char *last_name;
    char *last_mail;


    // DATեϣͤȤθߴǼѤ
    // skinΥִǻȤ
    size_t dat_filesize;


    // Ū˻Ȥ
    // ľfalseꤷsubject.txt˴ޤޤƤǤ
    // Ƚtrueꤹ롣λ塢DATɥξ
    // ̤trueꤵ롣ʳDATBulletinBoardΥꥹ
    // HashtableΤǡ̤refcountƤʤ¤ꡢ
    // destruct롣줬DATƤʤɤ¸ŪȽ
    // 뤿ˤϡis_alive()᥽åɤȤ
    bool alive;


  protected:
    virtual int get_flags() const;
    virtual void set_flags(int flags);

    virtual void read_extended_threadlist_element(AttributeTable &attributes);
    virtual void write_extended_threadlist_element(gzFile threadlist_xml) const;

    virtual char *excavate_dat_file(Repository &repository,
				    char *buf, size_t buf_len);


  public:
    BBSThread(BulletinBoard *board, const char *id, const char *title = NULL);
    virtual ~BBSThread();

    ResponseCursor *get_responses(Configuration &config,
				  Repository &repository,
				  NetworkAgentBroker &broker);

    PostResponseAgent *get_post_agent(Configuration &config,
				      NetworkAgentBroker &broker);

    inline BulletinBoard *get_board() const { return board; }
    int get_bbs_type() const;

    inline const char *get_id() const { return id; }
    void set_id(const char *value);

    inline const char *get_title() const { return title; }
    void set_title(const char *value);

    inline const char *get_last_modified() const { return last_modified; }
    void set_last_modified(const char *value);

    inline const char *get_date() const { return date; }
    void set_date(const char *value);

    inline const char *get_kako_html() const { return kako_html; }
    void set_kako_html(const char *value);

    inline const char *get_grave() const { return grave; }
    void set_grave(const char *value);

    inline const char *get_trigger_url() const { return trigger_url; }
    void set_trigger_url(const char *value);

    inline int get_rank() const { return rank; }
    inline void set_rank(int value)
    { previous_rank = rank; rank = value; }

    inline int get_previous_rank() const { return previous_rank; }

    inline int get_number_of_responses_on_server() const
    { return number_of_responses_on_server; }
    void set_number_of_responses_on_server(int value);

    inline int get_number_of_responses_got() const
    { return number_of_responses_got; }
    void set_number_of_responses_got(int value);

    inline int get_number_of_responses_read() const
    {
      return bookmark_position > 0
	? bookmark_position : number_of_responses_read;
    }
    void set_number_of_responses_read(int value);

    inline int get_number_of_new_responses() const
    { return number_of_new_responses; }
    inline void set_number_of_new_responses(int value)
    { number_of_new_responses = value; }

    inline int get_number_of_responses_available() const
    {
      int num_res_server = get_number_of_responses_on_server();
      int num_res_got = number_of_responses_got;
      return (num_res_server > num_res_got ? num_res_server : num_res_got);
    }
    inline int get_number_of_responses_to_get() const
    {
      int diff = get_number_of_responses_available()
	- number_of_responses_got;
      return diff >= 0 ? diff : 0;
    }
    inline int get_number_of_responses_to_read() const
    {
      int diff = get_number_of_responses_available()
	- get_number_of_responses_read();
      return diff >= 0 ? diff : 0;
    }

    inline int get_bookmark_position() const { return bookmark_position; }
    void set_bookmark_position(int value);

    inline int get_last_view_response() const { return last_view_response; }
    void set_last_view_response(int value);

    inline int get_shown_count() const { return shown_count; }
    void set_shown_count(int value);

    inline size_t get_dat_filesize() const { return dat_filesize; }
    size_t get_dat_filesize(Repository &repository);
    inline void set_dat_filesize(size_t value) { dat_filesize = value; }

    inline bool is_alive() const
    { return (get_flags() & DAT_DROPPED) == 0; }
    void set_dat_dropped(bool value);

    inline bool is_stopped() const
    { return (get_flags() & STOPPED) != 0; }
    void set_stopped(bool value);

    inline bool is_kako_thread() const
    { return (get_flags() & KAKO_THREAD) != 0; }
    void set_kako_thread(bool value);

    inline bool is_viewer_used() const
    { return (get_flags() & DAT_VIEWER) != 0; }
    void set_viewer_used(bool value);

    inline bool is_fake_dat() const
    { return (get_flags() & FAKE_DAT) != 0; }
    void set_fake_dat(bool value);

    inline bool is_tmp_thread() const
    { return (get_flags() & TMP_THREAD) != 0; }
    void set_tmp_thread(bool value);

    inline bool is_marked() const
    { return (ui_flags & MARKED) != 0; }
    void set_marked(bool value);

    inline bool is_hidden() const
    { return (ui_flags & HIDDEN) != 0; }
    void set_hidden(bool value);

    inline bool has_last_post_sage() const
    { return (ui_flags & LAST_POST_SAGE) != 0; }
    void set_last_post_sage(bool value);

    inline bool has_last_post_be() const
    { return (ui_flags & LAST_POST_BE) != 0; }
    void set_last_post_be(bool value);

    inline bool has_last_post_viewer() const
    { return (ui_flags & LAST_POST_VIEWER) != 0; }
    void set_last_post_viewer(bool value);


    inline const char *get_last_name() const { return last_name; }
    void set_last_name(const char *value);

    inline const char *get_last_mail() const { return last_mail; }
    void set_last_mail(const char *value);

    bool is_a_boned(unsigned int res_num);
    void register_a_bone(unsigned int res_num);
    void unregister_a_bone(unsigned int res_num);
    inline list<unsigned int>::const_iterator a_bone_list_begin()
    { return a_bone_list.begin(); }
    inline list<unsigned int>::const_iterator a_bone_list_end()
    { return a_bone_list.end(); }

    inline ABone &get_a_bone() { return a_bone; }

    virtual time_t get_last_modified_utc();
    virtual time_t get_birthtime();

    char *get_dat_url(char *buf, size_t buf_len) const;

    void read_threadlist_element(AttributeTable &attributes);
    void write_threadlist_element(gzFile threadlist_xml);


    char *find_dat_file(Repository &repository, char *buf, size_t buf_len,
			bool to_create);
    int open_dat_file(Repository &repository, int flags);
    void remove_dat_file(Repository &repository);
    void rename_dat_file_for_backup(Repository &repository);

    void recover_information(Configuration &config, Repository &repository);


    // Ϳ줿urlᤷthisΥΤΤɤȽꤹ롣
    // thisΥΤΤǤtrue򡢤ʳξˤfalse֤
    // thisΥΤΤǤꡢġfromtoNULLξˤϡ
    // urlΥ쥹ֻʬβfromto褦Ѵ֤
    // url쥹ֻʤˤfromtoˤ0֤
    // urlthisΥΤΤǤʤˤfromtoˤϿʤ
    bool parse_url(const char *url, int *from_p = NULL, int *to_p = NULL);
    /*
     * URL֤fromtoͿ줿ˤϥ쥹ֻURLˤʤ롣
     * ɬ˳Ƥ줿ʸ֤ΤǡcallerfreeǤ
     * 餦
     */
    virtual char *get_url(int from = 0, int to = 0);

    virtual bool is_trackback_supported();
    virtual char *get_trackback_url(int res_num = 0);
    virtual void reset(Repository &repository);
  };

  typedef smart_ptr<BBSThread> BBSThreadHandle;


  class Response
  {
    friend class ResponseCursor;

  private:

    enum {
      RESPONSE_BROKEN = 1 << 0,
      RESPONSE_DELETED = 1 << 1,
      RESPONSE_NOT_READY = 1 << 2
    };

    char *name;
    char *mailto;
    char *date;
    char *id;
    char *message;
    char *title;
    int flags;

    Response();
    ~Response();

    void set_name(const char *value);
    void set_mailto(const char *value);
    void set_date(const char *value);
    void set_id(const char *value);
    void set_message(const char *value);
    void set_title(const char *value);

    inline void clear_flags() { flags = 0; }
    inline void set_broken() { flags |= RESPONSE_BROKEN; }
    inline void set_deleted() { flags |= RESPONSE_DELETED; }
    inline void set_not_ready() { flags |= RESPONSE_NOT_READY; }


  public:
    inline const char *get_name() const { return name; }
    inline const char *get_mailto() const { return mailto; }
    inline const char *get_date() const { return date; }
    inline const char *get_id() const { return id; }
    inline const char *get_message() const { return message; }
    inline const char *get_title() const { return title; }

    // Ȥͭɤ
    inline bool is_valid() const { return flags == 0; }

    // 줿쥹ʤtrue
    inline bool is_broken() const { return (flags & RESPONSE_BROKEN); }
    // 줿쥹ʤtrue(e.g.ĮBBSƩ)
    inline bool is_deleted() const { return (flags & RESPONSE_DELETED); }
    // ¸ߤʤ쥹ʤtrue
    inline bool is_not_ready() const { return (flags & RESPONSE_NOT_READY); }
  };


  struct ResponseHint
  {
    int offset;
    size_t length;
    ResponseHint() : offset(0), length(0) {}
    ResponseHint(int offset, size_t length) : offset(offset), length(length) {}
    ResponseHint(const ResponseHint &hint)
      : offset(hint.offset), length(hint.length) {}
  };


  class ResponseCursor : public RefCount, public has_slots
  {
    friend class BBSThread;

  private:
    Response response;

    enum {
      RESPONSE_HINT_BROKEN = 0x80000000,
      RESPONSE_HINT_DELETED = 0x40000000,
      RESPONSE_HINT_MASK = (RESPONSE_HINT_BROKEN | RESPONSE_HINT_DELETED)
    };
    vector<ResponseHint> hints;


  protected:
    BBSThreadHandle thread;
    Configuration &config;
    Repository &repository;
    IconvBuffer iconv_buffer;

    BufferHandle buffer;
    NetworkAgentHandle agent;


    inline void set_name(const char *name) { response.set_name(name); }
    inline void set_mailto(const char *mailto) { response.set_mailto(mailto); }
    inline void set_date(const char *date) { response.set_date(date); }
    inline void set_id(const char *id) { response.set_id(id); }
    inline void set_message(const char *message)
    { response.set_message(message); }
    inline void set_title(const char *title) { response.set_title(title); }



    // 顢֥饹ǼDATեΥѡAPI
    // ()ͥåȥɤ߽ФˤϲƤΤϸΤǤʤ
    // 㤨УͤDATեɤ߹߻ˤϥ쥹ڤLF
    // 򸫤ĤhintꤹȤꤷƤ롣
    inline unsigned int get_next_res_num() { return hints.size() + 1; }
    int get_next_offset();


    // ܡλDATեΥå夬Ĥʤäɤ
    // ähints̵ˤ롣
    void clear_hints();

    // res_num֤Υ쥹bufferoffsetĹlengthʬǤ뤳ȤϿ
    void set_hints(unsigned int res_num, int offset, size_t length);

    // res_num֤Υ쥹bufferoffsetĹlengthƤƲ
    // 뤳ȤϿ
    void set_broken(unsigned int res_num, int offset, size_t length);

    // bufferoffsetϤޤϤres_num֤Υ쥹Ƥ뤳ȤϿ
    void set_deleted(unsigned int res_num, int offset);


    // ResponseCursorget_response()ƤФ줿ǸƤӽФtemplate
    // method֥饹startlengthʬres_numܤΥ쥹ȸʤƲ
    // Ϥ̾󤫤ƤޤǤꤹ롣쥹Ƥfalse֤
    virtual bool parse_response(unsigned int res_num,
				const char *start, size_t length) = 0;


    ResponseCursor(BBSThread *thread,
		   Configuration &config, Repository &repository,
		   Buffer *buffer, NetworkAgent *agent,
		   const char *from_encoding);


  public:
    virtual ~ResponseCursor();

    // ιǤˤϥιߤ롣DATʤ
    // βǽʤˤfalse֤
    virtual bool update() = 0;

    // ʲreaderå
    inline int lock() { return buffer->get_lock()->rdlock(); }
    inline int trylock() { return buffer->get_lock()->tryrdlock(); }
    inline int unlock(int lock_id)
    { return buffer->get_lock()->unlock(lock_id); }


    // λǥ쥹res_numΥ쥹Ǥtrue֤ͥåȥ
    // DATեμǤ⥭å䤽ޤɤ߹ʬ
    // б쥹ޤޤƤtrue֤롣դˡԤäƤФ
    // ǽˤʤʬǤ⡢λǼǽǤʤfalse
    // ܡΤˤƼʤɤΤǡƱres_numξǤ
    // Υ᥽åɤη̤lock()ƤָͭȤʤ롣
    virtual bool has_response(unsigned int res_num, int lock_id);

    // 쥹res_numΥ쥹Ƥ֤
    Response &get_response(unsigned int res_num, int lock_id);

    // DATեβϤλƤ뤫ɤ֤finished()
    // trueʻhas_response()falseʥ쥹¸ߤʤΤƱ
    virtual bool finished(int lock_id);

    NetworkAgent::NetworkAgentState get_agent_state(int lock_id);


    // DATեβϤʹԤfinished()η̤Ȥtrue
    // ֤郎äȤȯԤsigslotʥ롣
    Signal0 state_changed;


    // ʲDATեι˥ͥåȥȼäˡ
    // ˱ȯԤ륷ʥ롣̩ƱΤ
    // ݤʳ㤤ʤΤǡŬ˽
    // void access_started();
    Signal0 access_started;

    // void access_progressed();
    Signal0 access_progressed;

    // void access_finished();
    Signal0 access_finished;

    // ܡλȯԤ롣
    // void cache_is_dirty();
    Signal0 cache_is_dirty;

    // void access_failed(const char *reason);
    Signal0 access_failed;

    // void access_terminated();
    Signal0 access_terminated;


  private:
    void access_started_cb(NetworkAgent *agent, RefCount *data);
    void access_progressed_cb(NetworkAgent *agent, size_t data_read,
			      uintmax_t total_len, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void cache_is_dirty_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);
  };

  typedef smart_ptr<ResponseCursor> ResponseCursorHandle;

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BBS_THREAD_H_ */
