/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_utils.h,v 1.4 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BBS_UTILS_H_
#define _LIBOCHUSHA_BBS_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/network_agent.h"
#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"


using namespace std;
namespace Ochusha
{

  class BBSThread;
  class BulletinBoard;
  class Configuration;

  class PostResponseAgent : public RefCount
  {
  private:
    RefCountHandle board;
    RefCountHandle thread;
    NetworkAgentHandle agent;


  protected:
    Configuration &config;


  public:
    PostResponseAgent(BulletinBoard *board, Configuration &config,
		      NetworkAgent *agent);
    PostResponseAgent(BBSThread *thread, Configuration &config,
		      NetworkAgent *agent);
    virtual ~PostResponseAgent();

    BulletinBoard *get_board();
    BBSThread *get_thread();
    NetworkAgent *get_agent();


    // åɤκԤ
    virtual void try_new_thread(const char *title, const char *name,
				const char *mailto, const char *message) = 0;
    // 񤭹ߤԤ
    virtual void try_post(const char *name, const char *mailto,
			  const char *message) = 0;
    // ƤǺƤӽ񤭹ߤߤ롣
    virtual void retry_post(const char *submit, time_t utc_time) = 0;

    // ͤѤʤΤ¾Ǥϴ̵
    virtual void set_2ch_be_session_id(const char *id);
    virtual void set_2ch_viewer_session_id(const char *id);


    // 񤭹ȯԤsigslotʥ롣
    // void post_finished(PostResponseAgent *);
    signal1<PostResponseAgent *> post_finished;

    // 񤭹߼ԻȯԤsigslotʥ롣
    // void post_failed(PostResponseAgent *, const char *reason);
    signal2<PostResponseAgent *, const char *> post_failed;

    // ͤΥåǧѡ
    // void check_cookie(PostResponseAgent *, const char *message);
    signal2<PostResponseAgent *, const char *> check_cookie;
  };


  class BoardURLSeeker : public RefCount, public has_slots
  {
  private:
    RefCountHandle board;
    NetworkAgentHandle agent;

    char *board_url;


  protected:
    Configuration &config;


  private:
    void prepare_request_cb(NetworkAgent *agent, RefCount *data);
    void response_header_received_cb(NetworkAgent *agent, RefCount *data);
    void response_body_received_cb(NetworkAgent *agent, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  public:
    BoardURLSeeker(BulletinBoard *board, Configuration &config,
		   NetworkAgent *agent);
    virtual ~BoardURLSeeker();

    BulletinBoard *get_board();
    NetworkAgent *get_agent();
    inline const char *get_board_url() { return board_url; }
    void set_board_url(const char *url);


    virtual void seek();


    // žURLȯȯԤsigslotʥ
    // void url_found(BoardURLSeeker *, const char *url);
    signal2<BoardURLSeeker *, const char *> url_found;

    // žURLȯԻȯԤsigslotʥ
    // void seek_failed(BoardURLSeeker *);
    signal1<BoardURLSeeker *> seek_failed;
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BBS_UTILS_H_ */
