/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bookmarks.h,v 1.15 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BOOKMARKS_H_
#define _LIBOCHUSHA_BOOKMARKS_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "libochusha/ref_count.h"
#include "libochusha/repository.h"
#include "libochusha/tree.h"

#include <list>
#include <string>

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include <zlib.h>


using namespace std;
namespace Ochusha
{

  class BookmarkEntry : public TreeElement
  {
  private:
    char *url;


  public:
    BookmarkEntry(const char *name = NULL, const char *url = NULL,
		  time_t tc = 0, time_t last_mod = 0);
    BookmarkEntry(AttributeTable &attributes, time_t tc, time_t last_mod);
    virtual ~BookmarkEntry();

    inline const char *get_url() const { return url; }
    void set_url(const char *url);

    virtual bool is_folder() const;
    virtual const char *get_element_name() const;
    virtual void read_extended_tree_element(AttributeTable &attributes);
    virtual void write_extended_tree_element(WriteTree &args);
  };


  class Bookmarks : public RootFolder
  {
  protected:
    virtual Folder *folder_new(time_t tc, time_t last_mod, unsigned int sn);
    virtual Folder *folder_new(const Folder *folder, unsigned int sn);
    virtual TreeElement *leaf_element_new(AttributeTable &attributes,
					  time_t tc, time_t last_mod);


  public:
    Bookmarks();

    virtual TreeElement *copy(const TreeElement *element);

    bool read_bookmarks_xml(Repository &repository);
    bool write_bookmarks_xml(Repository &repository);

    virtual const char *get_root_element_name() const;
    virtual const char *get_leaf_element_name() const;
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BOOKMARKS_H_ */
