/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: buffer.h,v 1.18 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BUFFER_H_
#define _LIBOCHUSHA_BUFFER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/lock.h"
#include "libochusha/ref_count.h"

#include <cstddef>

#include <zlib.h>

#define DEFAULT_BUFFER_LENGTH		0x10000		// 64K


using namespace std;
namespace Ochusha
{
  class Buffer : public RefCount
  {
  private:
    char *buffer;
    size_t buffer_length;

    size_t data_length;

    volatile bool fixed;
    bool mmapped;


    // 񤭹ѥХåեѡ
    int fd;
    gzFile gzfile;

    smart_ptr<RWLock> lock;


  public:
    // buffer_lenʥХåե롣
    Buffer(size_t buffer_len = DEFAULT_BUFFER_LENGTH);

    // 񤭹ѥХåե롣
    Buffer(int fd, bool gzip);

    // gzipʽ񤭹ѥХåե롣
    Buffer(gzFile gzfile);

    virtual ~Buffer();

    bool is_fixed() { return fixed; }
    RWLock *get_lock() { return lock; }

    // Хåեؤν񤭹ߤνλ롣
    virtual void fix();

    // Хåե˥եǥץfdͿ줿եɤ߹ࡣ
    // ե뤬gzipإåäƤ硢Ĺ롣
    // λ塢եclose()롣
    bool read_file(int fd);

    // Хåե˥եޥåפ롣mmapΤĶǤmmapȤ
    // ¸ߤʤĶǤϥեɤ߹ࡣޥå׸塢Хåեread
    // onlyȤ롣
    // λ塢եclose()롣
    bool map_file(int fd);

    // ХåեƤեǥץfdͿ줿ե˽
    // Ф
    // λ塢եclose()롣
    bool write_file(int fd, bool gzip = false);

    // ХåեƤgzipʥե˽񤭽Ф
    // λ塢եclose()롣
    bool write_file(gzFile gzfile);


    /*
     * reader APIμ
     */
    // ХåեreaderŪͭʥǡĹ֤
    size_t get_length(int lock_id);

    // ХåեƬؤΥݥ󥿤֤
    const char *get_buffer(int lock_id);


    /*
     * writer APIμ
     */
    // Хåեˤ롣
    // wrlockȤƼlock_idꤵ줿硢wrlockݤƤ
    // ΤȤƿ񤦡0ξɬפ˱Ūwrlock롣
    virtual bool clear(int lock_id);

    // Хåե˥ǡɵ롣
    // wrlockȤƼlock_idꤵ줿硢wrlockݤƤ
    // ΤȤƿ񤦡0ξɬפ˱Ūwrlock롣
    virtual bool append_data(const char *data, size_t length, int lock_id);

    // Хåեΰ̣ͭǡĹ򥻥åȤ롣
    // wrlockȤƼlock_idꤵ줿硢wrlockݤƤ
    // ΤȤƿ񤦡0ξɬפ˱Ūwrlock롣
    virtual bool set_length(size_t length, int lock_id);

    // ХåեĹlengthɵѶ֤ݤºݤ˳ݤƤ
    // ɵѶ֤Ĺ֤
    // wrlockȤƼlock_idꤵ줿硢wrlockݤƤ
    // ΤȤƿ񤦡0ξɬפ˱Ūwrlock롣
    virtual size_t ensure_free_space(size_t length, int lock_id);


    // writerѡ
    inline size_t get_length0() { return data_length; }
    inline void clear0() { data_length = 0; }
    inline char *get_buffer0() { return buffer; }
    inline void unfix0() { fixed = false; }
  };

  typedef smart_ptr<Buffer> BufferHandle;
}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BUFFER_H_ */
