/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bulletin_board.h,v 1.54 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_BULLETIN_BOARD_H_
#define _LIBOCHUSHA_BULLETIN_BOARD_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/a_bone.h"
#include "libochusha/buffer.h"
#include "libochusha/bbs_thread.h"
#include "libochusha/constants.h"
#include "libochusha/hashtable.h"
#include "libochusha/network_agent.h"
#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"
#include "libochusha/tree.h"
#include "libochusha/uri_utils.h"


#include <list>
#include <set>
#include <vector>


#ifdef HAVE_TIME_H
#include <time.h>
#endif


using namespace std;
namespace Ochusha
{
  class BulletinBoard;
  class Configuration;
  class PostResponseAgent;
  class Repository;
  class ThreadPool;


  typedef vector<BBSThreadHandle> BBSThreadVector;

  class ThreadlistUpdater : public RefCount, public has_slots
  {
    friend class BulletinBoard;
    friend class ThreadlistIterator;

  protected:
    BulletinBoard *board;
    URI base_uri;

    Configuration &config;
    Repository &repository;
    NetworkAgentHandle agent;

    // updated_listιˤϤlockwrlock()٤
    // updated_listλȻˤϤlockrdlock()٤
    smart_ptr<RWLock> lock;

    BBSThreadVector updated_list;


    // ļΤϹȤ򲿤Ƥʤñ
    // BulletinBoardThreadlistIteratorȤĤλԴ֤Ʊʬ
    // Ƥ롣SubjectTxtParserʤɤȤΤ(֥饹)
    ThreadlistUpdater(BulletinBoard *board, Configuration &config,
		      Repository &repository, NetworkAgent *agent);

    inline void make_alive(BBSThread *thread) { thread->alive = true; }
    inline bool get_alive(BBSThread *thread) { return thread->alive; }
    void recover_threadlist();
    void update_threadlist();

    virtual void finalize();	// ()has_slots::disconnect_all()롣

    Signal0 state_changed;
    Signal0 board_moved;
    Signal0 error_occurred;


  public:
    virtual ~ThreadlistUpdater();
  };


  class ThreadlistIterator;
  typedef list<BulletinBoard *> BulletinBoardList;

  class InterboardThreadlistUpdater : public ThreadlistUpdater
  {
    friend class ThreadEnumeratorJob;

  private:
    ThreadPool &pool;
    smart_ptr<WorkerJob> job;
    bool update;

    BulletinBoardList boards_to_be_updated;
    set<BulletinBoard *> listed_boards;

    set<BBSThread *> enumerated_threads;

    bool enumerate_threadlist(ThreadlistIterator *iterator);


    ThreadlistIterator *get_threadlist(BulletinBoard *board, bool no_update);

    void notify_started(BulletinBoard *board);
    void notify_finished(bool succeeded);


  protected:
    void add_board_to_be_updated(BulletinBoard *board_to_be_updated);
    void start();

    // ꥹȤ˸٤threadɤȽꤹ롣
    // ThreadlistUpdater򻲾ȤThreadlistIteratorϡ
    // Υ᥽åɤtrue֤threadΤߤ롣
    virtual bool should_be_enumerated(BBSThread *thread) = 0;


  public:
    InterboardThreadlistUpdater(BulletinBoard *board, Configuration &config,
				Repository &repository, ThreadPool &pool,
				NetworkAgent *agent, bool update);
    virtual ~InterboardThreadlistUpdater();
  };


  class ThreadlistIterator : public RefCount, public has_slots
  {
    friend class UpdaterActivityWatcher;

  private:
    smart_ptr<ThreadlistUpdater> updater;
    NetworkActivityNotifier *notifier;
    BBSThreadVector &updated_list;
    int index;
    bool error_occurred;

    void state_changed_cb();
    void board_moved_cb();
    void error_occurred_cb();


  protected:
    inline BulletinBoard *get_board() { return updater->board; }


  public:
    ThreadlistIterator(ThreadlistUpdater *updater);
    ~ThreadlistIterator();

    inline bool has_error_occurred() const { return error_occurred; }

    // ̾reader lockɬפǡiterationŪlock/unlock
    // ֤ƱϢϴ˱äǤ뤬pthreadǤƱϹ
    // ʤΤǡäʤˡThreadlistIteratorΥ桼ϡ
    // has_more_threads()next_thread()finished()θƤӽФ
    // ĤǥåԤ٤
    inline int lock() { return updater->lock->rdlock(); }
    inline int trylock() { return updater->lock->tryrdlock(); }
    inline int unlock(int lock_id) { return updater->lock->unlock(lock_id); }


    /* ̯semantics*/

    // has_more_threads()ϤλiterateǤBBSThread¸ߤ
    // true֤ͥåȥsubject.txtɤ߹Ǥʤɡ
    // ³Ǥ⤽λɤ߹ƤʬiterateǤ
    // BBSThread¸ߤʤˤfalse֤
    bool has_more_threads(int lock_id);

    // next_thread()ϤλiterateǤBBSThread֤
    // iterateǤBBSThread̵ˤNULL֤
    BBSThread *next_thread(int lock_id);

    // finished()ƤΥФiterationνλƤ
    // true֤(finished() && !has_more_threads())trueξ硢
    // ƤBBSThreaditerationλȤ̣롣
    bool finished(int lock_id);

    NetworkAgent::NetworkAgentState get_agent_state(int lock_id);


    // iterationǽˤʤäfinished()η̤Ȥtrue
    // ֤郎äȤȯԤ롣
    Signal0 state_changed;


    // ʲϥι˥ͥåȥȼäˡ
    // ˱ȯԤ륷ʥ롣̩ƱΤ
    // ݤʳ˼꤬ʤΤǡŬ˽
    Signal0 access_started;
    Signal0 access_progressed;
    Signal0 access_finished;
    Signal0 access_failed;
    Signal0 access_terminated;


    // ʲĤΰžΤ줿ȯԤ륷ʥ롣
    Signal0 board_moved;
  };


  typedef smart_ptr<ThreadlistIterator> ThreadlistIteratorHandle;


  enum {
    OCHUSHA_BBS_TYPE_PSEUDO_FOLDER = -5,
    OCHUSHA_BBS_TYPE_PASSIVE_FOLDER = -4,
    OCHUSHA_BBS_TYPE_ACTIVE_FOLDER = -3,
    OCHUSHA_BBS_TYPE_VIRTUAL_BOARD = -2,	// Τ̾Ĥ(ѻ)
    OCHUSHA_BBS_TYPE_UNKNOWN = -1,
    OCHUSHA_BBS_TYPE_2CH = 0,
    OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP,
    OCHUSHA_BBS_TYPE_JBBS,		// ¸ߤΤ
    OCHUSHA_BBS_TYPE_MACHIBBS,
    OCHUSHA_BBS_TYPE_JBBS_LIVEDOOR,
    OCHUSHA_BBS_TYPE_MITINOKU,		// ΤбƤ̤б
    OCHUSHA_BBS_TYPE_2CH_COMPATIBLE,
    OCHUSHA_BBS_TYPE_2CH_HEADLINE,	// ̰ͽ
    OCHUSHA_BBS_TYPE_2CH_BE,		// ΰʪ(BEĤüä)
    OCHUSHA_BBS_TYPE_2CH_RESERVED,
    OCHUSHA_BBS_TYPE_BBSPINK,
    OCHUSHA_BBS_TYPE_2CH_COMPATIBLE_RO,	// ɤ߹
    OCHUSHA_BBS_TYPE_BBSPINK_HEADLINE
  };


  enum {
    NOT_DIRTY = 0,
    MINOR_MODIFICATION = 1,
    URGENT_SAVE_RECOMMENDED = 10000
  };


  class DATFileExplorer;

  class BulletinBoard : public TreeElement
  {
    friend class BBSTable;
    friend class InterboardThreadlistUpdater;
    friend class ThreadlistUpdater;
    friend class ThreadlistIterator;

  private:
    // ιƱ˹⡹1åɤޤǤ˸ꡣ
    // Ĥν񤭴bulletin_board_giantåǼ롣
    // ĤNULLʾϹǤʤȤ̣롣
    smart_ptr<ThreadlistUpdater> updater;

    time_t previous_access_time;	// InterboardThreadlistUpdater
    FolderHandle correspond_folder;	// ActiveFolder


    enum {
      MAILTO_MODE_MASK = ((1 << 0) | (1 << 1)),
      LAST_POST_SAGE = 1 << 2,
      LAST_POST_BE = 1 << 3,
      LAST_POST_VIEWER = 1 << 4
    };

    // updater->updated_listthreadlist˥ԡ()
    void update_threadlist(ThreadlistUpdater *updater);
    // threadlistupdater->updated_list˥ԡ(ʤӼԻ)
    void recover_threadlist(ThreadlistUpdater *updater);


    void set_base_url(const char *base_url, bool internal);
    void set_bbs_type(int type, bool internal);
    void set_last_modified(const char *value, bool internal);
    void set_date(const char *value, bool internal);
    void set_short_name(const char *value, bool internal);
    void set_rule(const char *value, bool internal);
    void set_mailto_mode(MailtoMode mode, bool internal);
    void set_last_post_sage(bool value, bool internal);
    void set_last_post_be(bool value, bool internal);
    void set_last_post_viewer(bool value, bool internal);
    void set_last_name(const char *value, bool internal);
    void set_last_mail(const char *value, bool internal);


    ThreadlistIterator *get_threadlist(Configuration &config,
				       Repository &repository, ThreadPool &pool,
				       NetworkAgent *agent, bool update);


  protected:
    int bbs_type;		// ǼĤμ
    char *base_url;		// 㤨 "http://pc5.2ch.net/unix/"
    char *last_modified;	// subject.txtɤ߹߻Last-Modified
    char *date;			// subject.txtɤ߹߻Date

    char *short_name;		// ֤ΤȤϻפʤġ
    char *rule;			// ֤ΤȤϻפʤġ
    int ui_flags;		// ֤ΤȤϻפʤġ
    char *last_name;		// ֤ΤȤϻפʤġ
    char *last_mail;		// ֤ΤȤϻפʤġ

    char *previous_server;	// žȯľλϿ(žõ)

    list<char *> movement_history;	// ž

    ABone a_bone;		// Ѥܡ

    BBSThreadVector threadlist;	// ɤ뤳ȤԤǤ륹Υꥹ
    RefCountHashtable thread_table;	// IDʸBBSThread *ʥơ֥

    int dirtyness;		// threadlist.xml¸ɬפɤ
    bool threadlist_read;	// threadlist.xmlɤ߹ߺѤɤ


    bool in_boardlist;		// BBSTableѤñ㲽Τˤˤ

    inline void set_in_boardlist() { in_boardlist = true; }
    inline bool is_in_boardlist() const { return in_boardlist; }


  public:
    BulletinBoard(time_t tc = 0, time_t tlm = 0);
    BulletinBoard(AttributeTable &attributes, time_t tc, time_t tlm);
    ~BulletinBoard();

    PostResponseAgent *get_new_thread_agent(Configuration &config,
					    NetworkAgentBroker &broker);

    ThreadlistIterator *get_threadlist(Configuration &config,
				       Repository &repository, ThreadPool &pool,
				       NetworkAgentBroker &broker, bool update);
    DATFileExplorer *get_dat_file_explorer(Configuration &config,
					   Repository &repository);

    BBSThread *lookup_bbs_thread_by_id(const char *id);
    BBSThread *lookup_bbs_thread_by_url(const char *url,
					int *from_p = NULL, int *to_p = NULL);

    bool read_threadlist_xml(Repository &repository);
    bool write_threadlist_xml(Repository &repository);

    inline bool has_threadlist_read() const { return threadlist_read; }

    int how_dirty() const;
    inline int get_dirtyness() { return dirtyness; }
    inline void notify_minor_modification()
    { dirtyness += MINOR_MODIFICATION; }
    inline void notify_major_modification()
    { dirtyness += URGENT_SAVE_RECOMMENDED; }

    BBSThread *thread_new(const char *id, const char *title);

    inline const char *get_base_url() const { return base_url; }
    inline void set_base_url(const char *base_url)
    { set_base_url(base_url, false); }

    inline int get_bbs_type() const { return bbs_type; }
    inline void set_bbs_type(int type) { set_bbs_type(type, false); }

    inline list<char *>::const_iterator movement_history_begin() const
    { return movement_history.begin(); }
    inline list<char *>::const_iterator movement_history_end() const
    { return movement_history.end(); }
    // ϲʤAPIʡ
    inline list<char *> &get_movement_history() { return movement_history; }

    inline ABone &get_a_bone() { return a_bone; }

    char *get_id() const;
    char *get_bbs_root() const;

    char *get_url(BBSThread *thread, int from = 0, int to = 0) const;

    inline const char *get_last_modified() const { return last_modified; }
    inline void set_last_modified(const char *value)
    { set_last_modified(value, false); }

    inline const char *get_date() const { return date; }
    inline void set_date(const char *value) { set_date(value, false); }

    inline const char *get_short_name() const
    { return short_name != NULL ? short_name : get_name(); }
    inline void set_short_name(const char *value)
    { set_short_name(value, false); }

    inline const char *get_rule() const { return rule; }
    inline void set_rule(const char *value) { set_rule(value, false); }

    inline MailtoMode get_mailto_mode() const
    { return static_cast<MailtoMode>(ui_flags & MAILTO_MODE_MASK); }
    inline void set_mailto_mode(MailtoMode mode)
    { set_mailto_mode(mode, false); }

    inline bool has_last_post_sage() const
    { return (ui_flags & LAST_POST_SAGE) != 0; }
    inline void set_last_post_sage(bool value)
    { set_last_post_sage(value, false); }

    inline bool has_last_post_be() const
    { return (ui_flags & LAST_POST_BE) != 0; }
    inline void set_last_post_be(bool value)
    { set_last_post_be(value, false); }

    inline bool has_last_post_viewer() const
    { return (ui_flags & LAST_POST_VIEWER) != 0; }
    inline void set_last_post_viewer(bool value)
    { set_last_post_viewer(value, false); }

    inline const char *get_last_name() const { return last_name; }
    inline void set_last_name(const char *value)
    { set_last_name(value, false); }

    inline const char *get_last_mail() const { return last_mail; }
    inline void set_last_mail(const char *value)
    { set_last_mail(value, false); }

    inline const char *get_previous_server() const { return previous_server; }
    void set_previous_server(const char *value);


    // ActiveFolderѤñ㲽Τᤳˤ
    inline Folder *get_correspond_folder() { return correspond_folder; }
    inline void set_correspond_folder(Folder *folder)
    { correspond_folder = folder; }


    // TreeElementvirtualؿ
    virtual bool is_folder() const { return false; }
    virtual const char *get_element_name() const;
    virtual void read_extended_tree_element(AttributeTable &attributes);
    virtual void write_extended_tree_element(WriteTree &args);



    static void get_all_bbs_type_list(vector<int> &bbs_type_list);
    static const char *get_bbs_type_description(int type);

    static char *synthesize_active_folder_url(unsigned int sn);
    static unsigned int get_serial_number_from_active_folder_url(const char *url);
    static bool is_active_folder_url(const char *url);

    static char *synthesize_pseudo_folder_url(const char *algorithm);
    static char *get_algorithm_from_pseudo_folder_url(const char *url);
    static const char *get_description_about_pseudo_folder_url(const char *url);
    static bool is_pseudo_folder_url(const char *url);
  };

  typedef smart_ptr<BulletinBoard> BulletinBoardHandle;


  class DATFileExplorer : public ThreadlistUpdater
  {
  public:
    DATFileExplorer(BulletinBoard *board, Configuration &config,
		    Repository &repository);
    virtual ~DATFileExplorer();

    virtual bool explore(ThreadPool &thread_pool) = 0;


    // »ҤDATե򸫤Ĥ٤ȯԤ
    // void dat_file_found(BulletinBoard *board, BBSThread *thread);
    signal2<BulletinBoard *, BBSThread *> dat_file_found;

    // ΤΥξDATե뤫ιȯԤ
    // void information_recovered(BulletinBoard *board, BBSThread *thread);
    signal2<BulletinBoard *, BBSThread *> information_recovered;

    // ȯλȯԤ
    // void finished(BulletinBoard *board);
    signal1<BulletinBoard *> finished;
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_BULLETIN_BOARD_H_ */
