/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: configuration.h,v 1.22 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_CONFIGURATION_H_
#define _LIBOCHUSHA_CONFIGURATION_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/a_bone.h"
#include "libochusha/uri_utils.h"

#include <cstdlib>
#include <cstring>

#include <zlib.h>


#define OCHUSHA_CURRENT_CONFIG_VERSION		6


using namespace std;
namespace Ochusha
{
  class Repository;
  class Configuration
  {
  private:
    int config_version;

    URI bbsmenu_uri;
    URI login_be_uri;
    URI login_viewer_uri;

    bool offline;

    bool enable_proxy_to_read;
    URI proxy_uri_to_read;

    bool enable_proxy_to_post;
    URI proxy_uri_to_post;


    ABone a_bone;	/* ̤ܡ */


    // config.xmlΥѥ᡼ȯ
    AttributeTable attributes;


  public:
    Configuration();
    virtual ~Configuration();


    inline int get_config_version() const { return config_version; }


    inline const URI &get_bbsmenu_uri() const { return bbsmenu_uri; }
    void set_bbsmenu_uri(const char *url);

    inline const URI &get_login_be_uri() const { return login_be_uri; }
    void set_login_be_uri(const char *url);

    inline const URI &get_login_viewer_uri() const { return login_viewer_uri; }
    void set_login_viewer_uri(const char *url);

    inline bool get_offline() const { return offline; }
    inline void set_offline(bool value) { offline = value; }


    inline bool get_enable_proxy_to_read() const
    { return enable_proxy_to_read; }
    inline void set_enable_proxy_to_read(bool value)
    { enable_proxy_to_read = value; }

    inline const URI &get_proxy_uri_to_read() const
    { return proxy_uri_to_read; }
    inline bool set_proxy_uri_to_read(const char *uri)
    { return proxy_uri_to_read.parse_uri(uri); }


    inline bool get_enable_proxy_to_post() const
    { return enable_proxy_to_post; }
    inline void set_enable_proxy_to_post(bool value)
    { enable_proxy_to_post = value; }

    inline const URI &get_proxy_uri_to_post() const
    { return proxy_uri_to_post; }
    inline bool set_proxy_uri_to_post(const char *uri)
    { return proxy_uri_to_post.parse_uri(uri); }


    inline ABone &get_a_bone() { return a_bone; }


    virtual bool write_config_xml(Repository &repository);
    virtual void read_config_xml(Repository &repository);


    // config.xmlΥѥ᡼ȯ
    inline bool get_config_bool(const char *name) const
    { return attributes.get_bool(name); }
    inline int get_config_int(const char *name) const
    { return attributes.get_int(name); }
    inline long get_config_long(const char *name) const
    { return attributes.get_long(name); }
    inline const char *get_config_string(const char *name) const
    { return attributes.get_string(name); }


  protected:
    virtual void write_extended_elements(gzFile config_xml) {}
    virtual void read_extended_elements(AttributeTable &attributes) {}
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_CONFIGURATION_H_ */
