/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: history_manager.h,v 1.3 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_HISTORY_MANAGER_H_
#define _LIBOCHUSHA_HISTORY_MANAGER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/lock.h"
#include "libochusha/sqlite_utils.h"
#include "libochusha/uri_utils.h"

#include <vector>

#include <sqlite3.h>

#ifdef HAVE_TIME_H
#include <time.h>
#endif


using namespace std;
namespace Ochusha
{


  class History
  {
    friend class HistoryManager;
  private:
    sqlite3_int64 id;
    char *url;
    char *comment;
    time_t first_accessed;
    time_t last_accessed;
    int number_of_accesses;


    History(sqlite3_int64 id, const char *url, const char *comment,
	    time_t first_accessed, time_t last_accessed,
	    int number_of_accesses);


  public:
    History(const char *url, time_t access_time, const char *comment = NULL);
    History(const History &history);
    virtual ~History();

    inline sqlite3_int64 get_id() const { return id; }
    inline const char *get_url() const { return url; }
    inline const char *get_comment() const { return comment; }
    inline time_t get_first_accessed() const { return first_accessed; }
    inline time_t get_last_accessed() const { return last_accessed; }
    inline int get_number_of_accesses() const { return number_of_accesses; }

    // commentNULLʤѹ̵
    void set_comment(const char *comment);
  };


  class Histories
  {
  private:
    typedef vector<History> HistoryVector;
    HistoryVector histories;


  public:
    inline HistoryVector::iterator begin() { return histories.begin(); }
    inline HistoryVector::iterator end() { return histories.end(); }
  };


  class HistoryManager
  {
  private:
    Lock lock;
    SQLite3 *db_handle;

    SQLite3Statement *select_all_histories;
    SQLite3Statement *select_matched_url;
    SQLite3Statement *insert_new_row;
    SQLite3Statement *update_matched_id;
    SQLite3Statement *delete_matched_id;

    // allow_duplicated_recordstrueξ硢ƱURLؤΥǤäƤ⡢
    // ʪȤƵϿ롣ŪˤϽ񤭹εϿ˻Ȥ
    // ȤꤷƤ롣ξdbΥȥϾ
    // (first_accessed == last_accessed && number_of_accesses == 1)
    // Ȥʤ롣
    bool allow_duplicated_records;


  public:
    HistoryManager(const char *db_filename, bool allow_duplicated_records);
    ~HistoryManager();

    // urlؤΥ¸ߤtrue֤
    bool has_history(const char *url);

    // urlؤΥ̵֤NULL֤
    History *get_history(const char *url);

    // urlؤΥäȤϿ롣
    bool record(const char *url, const char *comment = NULL,
		time_t access_time = static_cast<time_t>(-1));

    // historyǻꤵ줿dbΥξ򹹿롣
    bool update(const History &history);

    // historyǻꤵ줿ξdbõ롣
    void remove(const History &history);

    // ƾõ롣
    void remove_all();

    // ޤȤ᤿HistoriesؤΥݥ󥿤֤
    // ̤Ȥ֤Histories֥ȤdeletecallerǤ
    Histories *get_histories();
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_HISTORY_MANAGER_H_ */
