/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: html_utils.h,v 1.22 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_HTML_UTILS_H_
#define _LIBOCHUSHA_HTML_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/text_utils.h"

#include <iconv.h>


using namespace std;
namespace Ochusha
{

  /* SAXʤäHTMLѡɬ׺Ǿ¤ʤ */
  class HTMLSAXParser
  {
  private:
    CStringBuffer tmp_buffer;

    bool in_anchor;

    const char *process_tag(const char *head, size_t len);
    const char *process_entity_ref(const char *head, size_t len);
    const char *fake_anchor_tag(const char *head, size_t len);


  protected:
    virtual void start_element(const char *name, const char *const attrs[]);
    virtual void end_element(const char *name);
    virtual void entity_reference(const char *name, size_t len);
    virtual void characters(const char *ch, size_t len) = 0;
    virtual void markup_declaration(const char *ch, size_t len);


  public:
    HTMLSAXParser() {}
    virtual ~HTMLSAXParser() {}

    const char *parse_text(const char *text,
			   size_t len, bool fake_anchor = false);
  };


  /* λȤޤʸ󤫤饿õλȤΤѴ
   * ʸ뤿Υإѥ饹
   */
  class TextCanonicalizer : private HTMLSAXParser
  {
  private:
    CStringBuffer &buffer;

  protected:
    virtual void characters(const char *ch, size_t len);


  public:
    TextCanonicalizer(CStringBuffer &buffer)
      : buffer(buffer) {}

    inline void clear() { buffer.clear(); }
    const char *canon(const char *src, int len);
    const char *canon_url(const char *src, int len);
  };
}

extern "C" {
#endif

#define UTF8_BUFFER_SIZE	5
/*
 * UTF8_BUFFER_SIZEʾĹbuffercharacterUTF-8encode
 * ʸǼ롣NUL terminate롣
 * ˤbuffer򡢼Ի(0xfffff礭character̤б)ˤ
 * NULL֤
 */
char *encode_unicode_to_utf8(unsigned int character, char *buffer);

#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHA_HTML_UTILS_H_ */
