/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: http_cookies.h,v 1.7 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_HTTP_COOKIES_H_
#define _LIBOCHUSHA_HTTP_COOKIES_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/http_enums.h"
#include "libochusha/uri_utils.h"

#include <string>
#include <vector>

#include <time.h>


using namespace std;
namespace Ochusha
{

  class HTTPCookies;
  class URI;

  class HTTPCookie
  {
    friend class HTTPCookies;

  public:
  enum HTTPCookieSpec {
    SPEC_UNKNOWN = -1,
    SPEC_NETSCAPE = 0,
    SPEC_RFC2109 = 1,
    SPEC_RFC2965 = 2
  };


  private:
    HTTPCookieSpec spec;
    char *name;
    char *value;
    char *domain;
    char *path;
    time_t expires;	// the cookie should be discarded at `expires' in UTC.
			// expires == 0 means, the cookie should be discarded
			// at the end of current session.
    char *comment;
    char *comment_url;
    char *port;
    bool secure;

    inline void set_spec(HTTPCookieSpec spec) { this->spec = spec; }
    void set_name(const char *name_start, size_t length);
    void set_value(const char *value_start, size_t length);
    void set_domain(const char *domain_start, size_t length);
    void set_path(const char *path_start, size_t length);
    inline void set_expires(time_t expires) { this->expires = expires; }
    void set_comment(const char *comment_start, size_t length);
    void set_comment_url(const char *comment_url_start, size_t length);
    void set_port(const char *port_start, size_t length);
    inline void set_secure(bool secure) { this->secure = secure; }


  public:
    HTTPCookie(HTTPCookieSpec spec, const char *name, const char *value,
	       const char *domain, const char *path,
	       time_t expires = static_cast<time_t>(-1),
	       const char *comment = NULL, const char *comment_url = NULL,
	       const char *port = NULL, bool secure = false);
    HTTPCookie(HTTPCookieSpec spec, const string *name, const string *value,
	       const string *domain, const string *path,
	       time_t expires = static_cast<time_t>(-1),
	       const string *comment = NULL, const string *comment_url = NULL,
	       const string *port = NULL, bool secure = false);
    HTTPCookie(const HTTPCookie &cookie);
    ~HTTPCookie();

    inline HTTPCookieSpec get_spec() const { return spec; }
    inline const char *get_name() const { return name; }
    inline const char *get_value() const { return value; }
    inline const char *get_domain() const { return domain; }
    inline const char *get_path() const { return path; }
    inline time_t get_expires() const { return expires; }
    inline const char *get_comment() const { return comment; }
    inline const char *get_comment_url() const { return comment_url; }
    inline const char *get_port() const { return port; }
    inline bool get_secure() const { return secure; }
  };


  class HTTPCookies
  {
  private:
    typedef vector<HTTPCookie> HTTPCookieVector;
    HTTPCookieVector cookies;


    bool parse_set_cookie_value(const URI &uri, const char *value,
				bool is_set_cookie2);


  public:
    HTTPCookies() {}
    ~HTTPCookies() {}

    inline HTTPCookieVector::const_iterator begin() const
    { return cookies.begin(); }
    inline HTTPCookieVector::const_iterator end() const
    { return cookies.end(); }

    bool parse_set_cookie_value(const URI &uri, const char *value);
    bool parse_set_cookie2_value(const URI &uri, const char *value);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_HTTP_COOKIES_H_ */
