/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: http_headers.h,v 1.6 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_HTTP_HEADERS_H_
#define _LIBOCHUSHA_HTTP_HEADERS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <cstdlib>
#include <vector>


using namespace std;
namespace Ochusha
{
  class HTTPHeaders;

  class HTTPHeader
  {
    friend class HTTPHeaders;

  private:
    char *name;
    char *value;

    void set_value(const char *value);
    void set_value(const char *value, size_t value_length);


  public:
    HTTPHeader(const char *name, const char *value);
    HTTPHeader(const char *name_start, size_t name_length,
	       const char *value_start, size_t value_length);
    HTTPHeader(const HTTPHeader &header);
    ~HTTPHeader();

    inline const char *get_name() const { return name; }
    inline const char *get_value() const { return value; }

    // general headers (from RFC2616 Section 4.5)
    static const char * const Cache_Control;
    static const char * const Connection;
    static const char * const Date;
    static const char * const Pragma;
    static const char * const Transfer_Encoding;
    static const char * const Update;
    static const char * const Trailer;
    static const char * const Via;
    static const char * const Warning;

    // request headers (from RFC2616 Section 5.3)
    static const char * const Accept;
    static const char * const Accept_Charset;
    static const char * const Accept_Encoding;
    static const char * const Accept_Language;
    static const char * const Authorization;
    static const char * const Expect;
    static const char * const From;
    static const char * const Host;
    static const char * const If_Match;
    static const char * const If_Modified_Since;
    static const char * const If_None_Match;
    static const char * const If_Range;
    static const char * const If_Unmodified_Since;
    static const char * const Max_Forwards;
    static const char * const Proxy_Authorization;
    static const char * const Range;
    static const char * const Referer;
    static const char * const TE;
    static const char * const User_Agent;

    // response headers (from RFC2616 Section 6.2)
    static const char * const Accept_Ranges;
    static const char * const Age;
    static const char * const ETag;
    static const char * const Location;
    static const char * const Proxy_Authenticate;
    static const char * const Retry_After;
    static const char * const Server;
    static const char * const Vary;
    static const char * const WWW_Authenticate;

    // entity headers (from RFC2616 Section 7.1)
    static const char * const Allow;
    static const char * const Content_Encoding;
    static const char * const Content_Language;
    static const char * const Content_Length;
    static const char * const Content_Location;
    static const char * const Content_MD5;
    static const char * const Content_Range;
    static const char * const Content_Type;
    static const char * const Expires;
    static const char * const Last_Modified;

    // other headers (from RFC2109 and RFC2965)
    static const char * const Cookie;
    static const char * const Set_Cookie;
    static const char * const Cookie2;
    static const char * const Set_Cookie2;
  };


  class HTTPHeaders
  {
  private:
    typedef vector<HTTPHeader> HTTPHeaderVector;
    HTTPHeaderVector headers;

    HTTPHeader *lookup_header(const char *name);
    HTTPHeader *lookup_header(const char *name_start, size_t name_length);


  public:
    HTTPHeaders() {}
    ~HTTPHeaders() {}

    bool set_value(const char *name, const char *value);
    bool set_value(const char *name_start, size_t name_length,
		   const char *value_start, size_t value_length);
    void clear_value(const char *name);
    const char *get_value(const char *name) const;
    const HTTPHeader *get_header(const char *name) const;

    void clean();

    inline HTTPHeaderVector::const_iterator begin() const
    { return headers.begin(); }
    inline HTTPHeaderVector::const_iterator end() const
    { return headers.end(); }
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_HTTP_HEADERS_H_ */
