/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: http_request.h,v 1.10 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_HTTP_REQUEST_H_
#define _LIBOCHUSHA_HTTP_REQUEST_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/http_enums.h"
#include "libochusha/http_headers.h"


using namespace std;
namespace Ochusha
{

  class HTTPTransport;
  class URI;

  class HTTPRequest
  {
  private:
    const URI *uri;
    const URI *proxy_uri;

    HTTPRequestMethod method;
    unsigned int major_version;
    unsigned int minor_version;

    HTTPHeaders headers;

    char *body;
    size_t body_length;

    enum {
      STATE_INITIAL = 0,
      STATE_SENDING_REQUEST,
      STATE_SENDING_HEADERS,
      STATE_SENDING_BODY,
      STATE_DONE,
      STATE_ERROR
    } state;

    HTTPStatus prepare_request(HTTPTransport &connection);
    HTTPStatus prepare_headers(HTTPTransport &connection);
    HTTPStatus prepare_body(HTTPTransport &connection);

    static const char * const http_request_method_strings[];
    inline const char *get_request_method_string() const
    { return http_request_method_strings[method]; }


  public:
    HTTPRequest(const URI *uri = NULL, const URI *proxy_uri = NULL);
    ~HTTPRequest();

    bool set_uri(const URI *uri);
    bool set_proxy_uri(const URI *proxy_uri);
    bool set_request_method(HTTPRequestMethod method);
    HTTPRequestMethod get_request_method() const { return method; }
    bool set_body(const char *body, size_t body_length);
    inline void set_header(const char *name, const char *value)
    { headers.set_value(name, value); }
    inline const char *get_header(const char *name) const
    { return headers.get_value(name); }
    inline void clear_header(const char *name)
    { headers.clear_value(name); }
    void set_version(unsigned int major, unsigned int minor)
    { major_version = major; minor_version = minor; }
    inline unsigned int get_major_version() const { return major_version; }
    inline unsigned int get_minor_version() const { return minor_version; }

    bool prepare();
    HTTPStatus send(HTTPTransport &connection);
    void clean();

    static inline bool is_method_supported(HTTPRequestMethod method)
    {
      return method >= HTTP_REQUEST_METHOD_OPTIONS
	&& method <= HTTP_REQUEST_METHOD_CONNECT;
    }
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_HTTP_REQUEST_H_ */
