/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: misc_utils.h,v 1.12 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_MISC_UTILS_H_
#define _LIBOCHUSHA_MISC_UTILS_H_

#ifdef __cplusplus

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <string>

#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <list>


using namespace std;
namespace Ochusha
{

#define ERROR_MESSAGE_BUFLEN	512

  class ErrorMessage
  {
  private:
    int error_number;
    char buf[ERROR_MESSAGE_BUFLEN];


  public:
    ErrorMessage();
    ErrorMessage(int errnum);
    inline int get_errno() const { return error_number; }
    inline const char *get_message() const { return buf; }

    void set_errno(int errnum);

    // 餷Ȥˡ
    void set_unknown_errno(int errnum);
    inline char *get_buf() { return buf; }
  };


  class CStringEqual
  {
  private:
    const char *with;

  public:
    inline CStringEqual(const char *with) : with(with) {}
    inline bool operator()(const char *element)
    { return strcmp(with, element) == 0; }
  };

  string &append_markup_escaped_text(string &str, const char *text);

}

extern "C" {
#endif

#include <time.h>


int mkdir_p(const char *path);
char *markup_escape_text(const char *text);
char *shell_escape_text(const char *text);

time_t get_utc_from_date(const char *date_string);
char *get_gmt_date_from_utc(time_t utc_time, char *buf, size_t buf_len);
char *mempbrk(const char *string, const char *charset, size_t len);
char *reverse_strpbrk(const char *string, const char *charset);


#ifndef MIN
# define MIN(x, y)	((x) < (y) ? (x) : (y))
#endif

#ifndef MAX
# define MAX(x, y)	((x) < (y) ? (y) : (x))
#endif

#ifdef HAVE_CONFIG_H

# ifndef HAVE_TIMEGM
time_t timegm(struct tm *tm);
# endif

# ifndef HAVE_STRNSTR
char *strnstr(const char *big, const char *little, size_t len);
# endif

# ifndef HAVE_STRDUP
char *strdup(const char *src);
# endif

# ifndef HAVE_STRNDUP
char *strndup(const char *src, size_t len);
# endif

# ifndef HAVE_STRCASECMP
int strcasecmp(const char *s1, const char *s2);
# endif

# ifndef HAVE_STRNCASECMP
int strncasecmp(const char *s1, const char *s2, size_t len);
# endif

# ifndef HAVE_STRCASESTR
char *strcasestr(const char *big, const char *little);
# endif

# ifndef HAVE_STRNCASESTR
char *strncasestr(const char *big, const char *little, size_t len);
# endif

/* ήФˤϤʤʡ*/
char *strnrstr(const char *big, const char *little, size_t len);
char *strncaserstr(const char *big, const char *little, size_t len);

#endif


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHA_MISC_UTILS_H_ */
