/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: network_agent.cc,v 1.62 2009/01/01 07:46:43 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/network_agent.h"

#include "libochusha/cookie_manager.h"
#include "libochusha/http_headers.h"
#include "libochusha/misc_utils.h"
#include "libochusha/uri_utils.h"

#include <vector>

#include <cstdlib>
#include <cstring>

#include <poll.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_UTIME_H
#include <utime.h>
#endif


namespace Ochusha
{


#define LOG_MESSAGE_LENGTH		1024
#undef VERBOSE_DESTRUCTOR


// POSTꥯȤȤΥ쥹ݥ󥹤Cookieɤߡ񤭤롣
#define LIMIT_USE_OF_COOKIES	1


NetworkAgentBroker::NetworkAgentBroker(CookieManager &cookie_manager,
				       PollEventManager &manager,
				       ThreadPool *pool)
  : cookie_manager(cookie_manager), manager(manager), thread_pool(pool)
{
}


NetworkAgent *
NetworkAgentBroker::employ_agent()
{
  return new NetworkAgent(*this);
}


class EstablishConnectionJob : public WorkerJob
{
private:
  NetworkAgentHandle agent;

  char *errstr;
  void notify_error();


public:
  EstablishConnectionJob(NetworkAgent *agent) : agent(agent) {}

  virtual void work(const WorkerThread *employee);
  virtual const char *description() const
  { return "establish network connection"; }
};


void
EstablishConnectionJob::work(const WorkerThread *employee)
{
  if (agent->handle == NULL)
    {
      // ˴ؤΥͻΤߤʤ롣
      const char *errstr = LIB_("Voluntary Restricted for Dead Server");

      NetworkAgentHandle tmp = agent;

      // XXX: access_failedʥڤóݤthisκѤͤusesite
      //̥åɤthisκѤߤ礬롣Υ᥽åɤ
      //λ˺ѤϤȡhandleuser_dataʤɤδ
      //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
      //Ѳǽ֤ԤAPIɬפ
      //षget(...)ȤǾԤƤɤ
      //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
      //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
      //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
      //ɤġġ
      LockBlock lock_obj(libochusha_giant);

      agent->access_failed(agent, errstr, agent->user_data);

      char message[LOG_MESSAGE_LENGTH];
      snprintf(message, LOG_MESSAGE_LENGTH,
	       LIB_("Access Failed(%s) due to: %s\n"), agent->url, errstr);
      agent->broker.output_log(NON_FATAL_ERROR_LOG, message);

      agent->user_data = NULL;

      return;
    }

  if (employee->get_state() == WorkerThread::DIE_NOW)
    {
      NetworkAgentHandle tmp = agent;

      // XXX: access_terminatedʥڤóݤthisκѤͤusesite
      //̥åɤthisκѤߤ礬롣Υ᥽åɤ
      //λ˺ѤϤȡhandleuser_dataʤɤδ
      //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
      //Ѳǽ֤ԤAPIɬפ
      //षget(...)ȤǾԤƤɤ
      //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
      //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
      //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
      //ɤġġ
      LockBlock lock_obj(libochusha_giant);

      agent->access_terminated(agent, agent->user_data);
      // ¿ʬץཪλʤΤǡϽФʤǤ꽪λ

      HTTPHandle *handle = agent->handle;
      agent->handle = NULL;
      agent->broker.connection_pool.release_http_handle(agent->url, handle,
							ACCESS_SUCCEEDED);
      agent->user_data = NULL;

      return;
    }

  agent->start_access();
}


class ProcessResponseBodyJob : public WorkerJob
{
private:
  NetworkAgentHandle agent;

public:
  ProcessResponseBodyJob(NetworkAgent *agent) : agent(agent) {}
  virtual void work(const WorkerThread *employee);
  virtual const char *description() const
  { return "process response body"; }
};


void
ProcessResponseBodyJob::work(const WorkerThread *employee)
{
  if (employee->get_state() == WorkerThread::DIE_NOW)
    {
      NetworkAgentHandle tmp = agent;

      // XXX: access_terminatedʥڤóݤthisκѤͤusesite
      //̥åɤthisκѤߤ礬롣Υ᥽åɤ
      //λ˺ѤϤȡhandleuser_dataʤɤδ
      //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
      //Ѳǽ֤ԤAPIɬפ
      //षget(...)ȤǾԤƤɤ
      //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
      //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
      //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
      //ɤġġ
      LockBlock lock_obj(libochusha_giant);

      agent->access_terminated(agent, agent->user_data);
      // ¿ʬץཪλʤΤǡϽФʤǤ꽪λ

      HTTPHandle *handle = agent->handle;
      agent->handle = NULL;
      agent->broker.connection_pool.release_http_handle(agent->url, handle,
							ACCESS_SUCCEEDED);
      agent->user_data = NULL;

      return;
    }

  agent->process_response_body_can_block();
}


NetworkAgent::NetworkAgent(NetworkAgentBroker &broker)
  : broker(broker),
    mode(NOT_SPECIFIED),
    state(STATE_INITIAL),
    url(NULL),
    post_body(NULL),
    handle(NULL),
    buffer(NULL),
    user_data(NULL),
    cache_comparison_buffer_length(0),
    cache_check_required(false),
    bytes_total(0)
{
}


NetworkAgent::~NetworkAgent()
{
  if (url != NULL)
    free(url);

  assert(handle == NULL);
  assert(!(user_data != NULL));
#ifdef VERBOSE_DESTRUCTOR
  cerr << "NetworkAgent destructed.\n";
#endif
}


void
NetworkAgent::notify_allowed(HTTPHandle *handle)
{
  assert(this->handle == NULL);

  this->handle = handle;
  broker.thread_pool->commit(new EstablishConnectionJob(this));
}


bool
NetworkAgent::notify_busy()
{
  assert(this->handle == NULL);

  return true;
}


void
NetworkAgent::notify_denied()
{
  assert(this->handle == NULL);

  // ԤǤ뤳ȤȽäƤΤˡƥåɤ˻ŻꤲΤ
  // 餫̵̤åطμԹˤꡢͥåȥ
  // ϢsigslotʥϲǽʤƱΥåɤȯ
  // ڤʤΤǡƤ
  broker.thread_pool->commit(new EstablishConnectionJob(this));
}


const char *
NetworkAgent::get_url()
{
  return url;
}


void
NetworkAgent::establish_connection()
{
  assert(handle == NULL);

  broker.connection_pool.request_connection(this);
}


void
NetworkAgent::refresh_dirty_cache()
{
  char message[LOG_MESSAGE_LENGTH];

  cache_check_required = false;
  cache_comparison_buffer_length = 0;
  buffer->clear0();
  mode = GET_REQUEST;
  state = STATE_INITIAL;
  cache_is_dirty(this, user_data);

  snprintf(message, LOG_MESSAGE_LENGTH, LIB_("Cache is Dirty: %s\n"), url);
  broker.output_log(INFORMATION_LOG, message);

  HTTPHandle *hdl = handle;
  handle = NULL;
  broker.connection_pool.release_http_handle(url, hdl, ACCESS_SUCCEEDED);
  establish_connection();
}


bool
NetworkAgent::process_response_body_real(int lock_id)
{
  assert(state == STATE_BODY_RECEIVING || state == STATE_BODY_SKIPPING);
  assert(lock_id != 0);

  if (handle->get_status_code() == 416)
    {
      // Requested range not satisfiableʤΤǥåԡ
      buffer->get_lock()->unlock(lock_id);
      refresh_dirty_cache();
      return false;
    }

  if (cache_check_required && handle->get_status_code() == 200)
    {
      // Range̵꤬뤵ƤƤ硣
      // HTTPŪ˵ƤơProxyФʤɤαƶǸ¤ͭ餷
      buffer->clear(lock_id);
      cache_check_required = false;
      cache_comparison_buffer_length = 0;
    }

  short event_mask;
  do
    {
      HTTPStatus http_result = handle->process();

      if (http_result == HTTP_ERROR)
	{
	  buffer->get_lock()->unlock(lock_id);
	  notify_error();
	  // thisdestructƤǽꡣ
	  return false;
	}

      if (state == STATE_BODY_RECEIVING)
	{
	  size_t body_len = handle->get_body_length();
	  if (body_len > 0)
	    {
	      const char *body = handle->get_body();
	      assert(body != NULL);

	      if (cache_check_required)
		{
		  size_t cmp_buf_left
		    = CACHE_COMPARISON_SIZE - cache_comparison_buffer_length;
		  assert(cmp_buf_left > 0);

		  size_t len = MIN(body_len, cmp_buf_left);
		  memcpy((cache_comparison_buffer
			  + cache_comparison_buffer_length), body, len);
		  body_len -= len;
		  body += len;
		  cache_comparison_buffer_length += len;

		  if (cache_comparison_buffer_length == CACHE_COMPARISON_SIZE)
		    {
		      cache_check_required = false;
		      if (memcmp(cache_comparison_buffer,
				 (buffer->get_buffer0() + buffer->get_length0()
				  - CACHE_COMPARISON_SIZE),
				 CACHE_COMPARISON_SIZE) != 0)
			{
			  // å夬äȤˤĤơ
			  buffer->get_lock()->unlock(lock_id);
			  refresh_dirty_cache();
			  return false;
			}
		    }
		  else
		    {
		      // ޤåԤĴ٤ۤɥǡʤ
		      assert(body_len == 0);
		      assert(cache_comparison_buffer_length
			     < CACHE_COMPARISON_SIZE);
		    }
		}

	      if (body_len > 0)
		{
		  if (buffer->append_data(body, body_len, lock_id))
		    handle->flush_response_body_buffer();
		  else
		    {
		      // ¿ʬOut of memoryΤ褦̿Ūʥ顼
		      buffer->get_lock()->unlock(lock_id);
		      NetworkAgentHandle tmp = this;

		      // XXX: access_failedʥڤóݤthisκ
		      //ͤusesite̥åɤthisκѤߤ
		      //礬롣Υ᥽åɤλ˺Ѥ
		      //ȡhandleuser_dataʤɤδíܤˤʤ
		      //Ƥޤ괺giantǼäƤܼŪˤ
		      //Ѳǽ֤ԤAPIɬפ
		      //षget(...)ȤǾԤƤɤ
		      //ȤϤäƤ⡢ǤNetworkAgentsigslot
		      //ʥlibochusha_giantå˻ȤäƤΤǡ
		      //ΤޤޥʥȯԸ塢Ѳǽˤʤޤ
		      //libochusha_giantΥåʤɤġġ
		      LockBlock lock_obj(libochusha_giant);

		      access_failed(this, LIB_("Out of memory"), user_data);
		      broker.notify_access_status(url, ACCESS_FINISHED);

		      HTTPHandle *hdl = handle;
		      handle = NULL;
		      broker.connection_pool.release_http_handle(url, hdl,
								 ACCESS_BUSY);
		      user_data = NULL;

		      return false;
		    }
		}
	    }
	}	// if (state == BODY_RECEIVING)Υ֥å

      if (http_result == HTTP_DONE)
	{
	  if (state == STATE_BODY_RECEIVING)
	    state = STATE_BODY_RECEIVED;
	  else
	    {
	      assert(state == STATE_BODY_SKIPPING);

	      state = STATE_BODY_SKIPPED;
	    }
	  event_mask = 0;
	  break;
	}
      assert(http_result == HTTP_NOT_DONE);

      event_mask = static_cast<short>(handle->get_poll_event_mask());
    }
  while (event_mask == 0);

  buffer->get_lock()->unlock(lock_id);

  size_t len = buffer->get_length0();
  access_progressed(this, len, bytes_total, user_data);

  if (state == STATE_BODY_RECEIVING || state == STATE_BODY_SKIPPING)
    {
      assert(event_mask != 0);

      return true;
    }

  if (!cache_check_required)
    {
      if (state == STATE_BODY_RECEIVED)
	{
	  buffer->fix();
	  NetworkAgentHandle tmp = this;
	  response_body_received(this, user_data);
	  notify_finished();
	}
      else
	{
	  assert(state == STATE_BODY_SKIPPED);

	  if (handle->get_status_code() == 304)
	    {
	      buffer->fix();
	      NetworkAgentHandle tmp = this;
	      response_body_received(this, user_data);
	      notify_finished();
	    }
	  else
	    notify_failure();
	}

      return false;
    }

  // ɤ߹߽äΤ˥ǡ­ʤΤǡåԡ
  refresh_dirty_cache();
  return false;
}


void
NetworkAgent::process_response_body_can_block()
{
  int lock_id = buffer->get_lock()->wrlock();
  if (!process_response_body_real(lock_id))
    {
      /*
       * Τϡ
       * (1) Ƥresponse body
       * (2) åԤȯ줿ɤľϥꥯȺѡ
       * (3) ʥ顼ȯ
       * ʾ壳ĤξΤ줫Ǥꡢʾ夹뤳ȤϤʤ
       */
      return;
    }

  assert(state == STATE_BODY_RECEIVING || state == STATE_BODY_SKIPPING);

  broker.notify_access_status(url, WAIT_FOR_SERVER);

  short event_mask = static_cast<short>(handle->get_poll_event_mask());
  assert(event_mask != 0);

  broker.manager.request_poll(this, handle->get_socket(),
			      event_mask,
			      PollEventManager::PRIORITY_NORMAL,
			      PollEventManager::TIMEOUT_IO);
}


int
NetworkAgent::process_response_body_nonblock()
{
  int lock_id = buffer->get_lock()->trywrlock();
  if (lock_id == 0)
    {
      // contentionäϡbodyɤɤǽ
      // åɤư
      broker.thread_pool->commit(new ProcessResponseBodyJob(this));
      return EVENT_RESULT_CONTENTION;
    }

  if (!process_response_body_real(lock_id))
    {
      /*
       * Τϡ
       * (1) Ƥresponse body
       * (2) åԤȯ줿ɤľϥꥯȺѡ
       * (3) ʥ顼ȯ
       * ʾ壳ĤξΤ줫Ǥꡢʾ夹뤳ȤϤʤ
       */
      return EVENT_RESULT_DONE;
    }

  assert(state == STATE_BODY_RECEIVING || state == STATE_BODY_SKIPPING);

  broker.notify_access_status(url, WAIT_FOR_SERVER);

  short event_mask = static_cast<short>(handle->get_poll_event_mask());
  assert(event_mask != 0);

  int interval = handle->get_prefered_polling_interval_millisec();
  return EVENT_RESULT_CONTINUE(event_mask, interval);
}


NetworkAgent::NetworkAgentState
NetworkAgent::update_state()
{
  assert (state != STATE_INITIAL);

  NetworkAgentState previous_state = state;

  if (state == STATE_STARTED)
    {
      if (handle->is_connected())
	{
	  state = STATE_CONNECTED;
	  connection_established(this, user_data);
	}
    }

  if (state == STATE_CONNECTED)
    {
      if (handle->has_request_done())
	{
	  state = STATE_REQUEST_DONE;
	  request_sended(this, user_data);
	}
    }

  if (state == STATE_REQUEST_DONE)
    {
      if (handle->is_status_available())
	{
	  state = STATE_STATUS_RECEIVED;
	  status_received(this, user_data);
	}
    }

  if (state == STATE_STATUS_RECEIVED)
    {
      if (handle->is_headers_available())
	{
	  state = STATE_HEADERS_RECEIVED;

	  if (handle->get_content_length() != 0)
	    bytes_total = handle->get_content_length();

#ifdef LIMIT_USE_OF_COOKIES
	  if (mode == POST_REQUEST)
	    {
#endif
	      const HTTPHeaders &headers = handle->get_headers();
	      vector<HTTPHeader>::const_iterator iter = headers.begin();
	      vector<HTTPHeader>::const_iterator end_iter = headers.end();

	      while (iter != end_iter)
		{
		  const HTTPHeader &header = *iter++;
		  if (strcasecmp(header.get_name(),
				 HTTPHeader::Set_Cookie) == 0)
		    broker.cookie_manager.set_cookie(*handle->get_uri(),
						     header.get_value());
		  else if (strcasecmp(header.get_name(),
				      HTTPHeader::Set_Cookie2) == 0)
		    broker.cookie_manager.set_cookie2(*handle->get_uri(),
						      header.get_value());
		}
#ifdef LIMIT_USE_OF_COOKIES
	    }
#endif

	  response_header_received(this, user_data);
	}
    }

  if (state == STATE_HEADERS_RECEIVED)
    {
      int status_code = handle->get_status_code();

      switch (status_code)
	{
	case 200:
	case 206:
	  state = STATE_BODY_RECEIVING;
	  break;

	case 304:
	  cache_check_required = false;
	  /* FALLTHROUGH */

	default:
	  state = STATE_BODY_SKIPPING;
	  break;
	}

      char message[LOG_MESSAGE_LENGTH];
      const char *reason = handle->get_reason_phrase();
      if (reason != NULL)
	snprintf(message, LOG_MESSAGE_LENGTH,
		 "%s: %d %s\n", url, status_code, reason);
      else
	snprintf(message, LOG_MESSAGE_LENGTH,
		 "%s: %d\n", url, status_code);
      broker.output_log(INFORMATION_LOG, message);
    }

  return previous_state;
}


void
NetworkAgent::notify_finished()
{
  char message[LOG_MESSAGE_LENGTH];
  snprintf(message, LOG_MESSAGE_LENGTH, LIB_("Access Finished(%s): %d\n"),
	   url, handle->get_status_code());

  NetworkAgentHandle tmp = this;

  // XXX: access_finishedʥڤóݤthisκѤͤusesite
  //̥åɤthisκѤߤ礬롣Υ᥽åɤ
  //λ˺ѤϤȡhandleuser_dataʤɤδ
  //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
  //Ѳǽ֤ԤAPIɬפ
  //षget(...)ȤǾԤƤɤ
  //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
  //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
  //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
  //ɤġġ
  LockBlock lock_obj(libochusha_giant);

  access_finished(this, user_data);

  broker.output_log(DEBUG_LOG, message);
  broker.notify_access_status(url, ACCESS_FINISHED);

  HTTPHandle *hdl = handle;
  handle = NULL;
  broker.connection_pool.release_http_handle(url, hdl, ACCESS_SUCCEEDED);
  user_data = NULL;
}


// ŪHTTPꥯȤ̤ʤäȤΡ(404ʤ)
void
NetworkAgent::notify_failure()
{
  const char *errstr;
  char status_buf[16];

  if (handle != NULL)
    {
      snprintf(status_buf, 16, "%d", handle->get_status_code());
      errstr = status_buf;
    }
  else
    errstr = LIB_("Unknown reason, may be terminated at signal handler.");

  if (errstr == NULL)
    errstr = LIB_("Unknown error");

  // access_failedȯԤȡϥɥthis(ʤNetworkAgent)
  // ΤƤƤޤǽ⤤
  NetworkAgentHandle tmp = this;

  // XXX: access_failedʥڤóݤthisκѤͤusesite
  //̥åɤthisκѤߤ礬롣Υ᥽åɤ
  //λ˺ѤϤȡhandleuser_dataʤɤδ
  //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
  //Ѳǽ֤ԤAPIɬפ
  //षget(...)ȤǾԤƤɤ
  //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
  //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
  //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
  //ɤġġ
  LockBlock lock_obj(libochusha_giant);

  access_failed(this, errstr, user_data);

  char message[LOG_MESSAGE_LENGTH];
  snprintf(message, LOG_MESSAGE_LENGTH,
	   LIB_("Access Failed(%s): due to %s\n"), url, errstr);
  broker.output_log(NON_FATAL_ERROR_LOG, message);

  broker.notify_access_status(url, ACCESS_FINISHED);

  HTTPHandle *hdl = handle;
  handle = NULL;
  broker.connection_pool.release_http_handle(url, hdl, ACCESS_SUCCEEDED);
  user_data = NULL;
}


// HTTPⲼؤǤμԤΡ(̿顼)
void
NetworkAgent::notify_error()
{
  const char *errstr;

  if (handle != NULL)
    errstr = handle->get_error_string();
  else
    errstr = LIB_("Terminated at signal handler?");

  if (errstr == NULL)
    errstr = LIB_("Unknown error");

  // access_failedȯԤȡϥɥthis(ʤNetworkAgent)
  // ΤƤƤޤǽ⤤
  NetworkAgentHandle tmp = this;

  // XXX: access_failedʥڤóݤthisκѤͤusesite
  //̥åɤthisκѤߤ礬롣Υ᥽åɤ
  //λ˺ѤϤȡhandleuser_dataʤɤδ
  //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
  //Ѳǽ֤ԤAPIɬפ
  //षget(...)ȤǾԤƤɤ
  //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
  //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
  //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
  //ɤġġ
  LockBlock lock_obj(libochusha_giant);

  access_failed(this, errstr, user_data);

  char message[LOG_MESSAGE_LENGTH];
  snprintf(message, LOG_MESSAGE_LENGTH,
	   LIB_("Access Failed(%s): due to %s\n"), url, errstr);
  broker.output_log(NON_FATAL_ERROR_LOG, message);

  broker.notify_access_status(url, ACCESS_FINISHED);

  HTTPHandle *hdl = handle;
  handle = NULL;
  broker.connection_pool.release_http_handle(url, hdl, ACCESS_FAILED);
  user_data = NULL;
}


int
NetworkAgent::handle_event(int fd, short events, short revents)
{
  // PollEventManagerevent loopåɤƤФ롣
  // ХåեΥåԤǥ֥åƤϤʤʤ

  if ((events & revents) == 0)
    {
      const char *errstr;
      if (state == STATE_INITIAL)
	errstr = LIB_("Connection Timeout");
      else
	errstr = LIB_("I/O Timeout");

      NetworkAgentHandle tmp = this;

      // XXX: access_failedʥڤóݤthisκѤͤusesite
      //̥åɤthisκѤߤ礬롣Υ᥽åɤ
      //λ˺ѤϤȡhandleuser_dataʤɤδ
      //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
      //Ѳǽ֤ԤAPIɬפ
      //षget(...)ȤǾԤƤɤ
      //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
      //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
      //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
      //ɤġġ
      LockBlock lock_obj(libochusha_giant);

      access_failed(this, errstr, user_data);

      char message[LOG_MESSAGE_LENGTH];
      snprintf(message, LOG_MESSAGE_LENGTH,
	       LIB_("Access Failed(%s): due to %s\n"), url, errstr);
      broker.output_log(NON_FATAL_ERROR_LOG, message);

      broker.notify_access_status(url, ACCESS_FINISHED);

      HTTPHandle *hdl = handle;
      handle = NULL;
      broker.connection_pool.release_http_handle(url, hdl, ACCESS_BUSY);
      user_data = NULL;

      return EVENT_RESULT_DONE;
    }

  if ((revents & (POLLIN | POLLPRI)) != 0)
    broker.notify_access_status(url, TRANSFER_DATA);

  HTTPStatus http_result;
  short event_mask;
  do
    {
      if (state == STATE_BODY_RECEIVING || state == STATE_BODY_SKIPPING)
	return process_response_body_nonblock();

      http_result = handle->process();

      if (http_result == HTTP_ERROR)
	{
	  notify_error();
	  // thisdestructƤǽꡣ
	  return EVENT_RESULT_DONE;
	}

#ifndef NDEBUG
      NetworkAgentState previous_state = update_state();
#else
      update_state();
#endif

      if (state == STATE_BODY_RECEIVING)
	{
	  assert(previous_state != STATE_BODY_RECEIVING);

	  return process_response_body_nonblock();
	}

      if (state == STATE_BODY_SKIPPING)
	{
	  assert(previous_state != STATE_BODY_SKIPPING);

	  return process_response_body_nonblock();
	}

      if (state == STATE_TERMINATED)
	return EVENT_RESULT_DONE;

      if (http_result == HTTP_DONE)
	{
	  // HEADꥯȤξʳϤ뤳ȤϤʤ
	  // ơΤȤNetworkAgentHEADꥯȤȤäƤʤġġ
	  assert(handle != NULL);

	  notify_finished();

	  return EVENT_RESULT_DONE;	// ǤΤ
	}
      assert(http_result == HTTP_NOT_DONE);

      event_mask = static_cast<short>(handle->get_poll_event_mask());
    }
  while (event_mask == 0);

  assert(http_result == HTTP_NOT_DONE);
  assert(event_mask != 0);

  int interval = handle->get_prefered_polling_interval_millisec();
  switch (state)
    {
    case STATE_CONNECTED:
      broker.notify_access_status(url, SEND_REQUEST);
      return EVENT_RESULT_CONTINUE(event_mask, interval);

    case STATE_REQUEST_DONE:
    case STATE_STATUS_RECEIVED:
    case STATE_HEADERS_RECEIVED:
      broker.notify_access_status(url, WAIT_FOR_SERVER);
      return EVENT_RESULT_CONTINUE(event_mask, interval);

    case STATE_INITIAL:
    case STATE_STARTED:
    case STATE_BODY_SKIPPING:
    case STATE_BODY_SKIPPED:
    case STATE_BODY_RECEIVING:
    case STATE_BODY_RECEIVED:
    case STATE_TERMINATED:
      // should never reached
      break;
    }

  // should never reached
  assert(state != STATE_INITIAL);
  assert(state != STATE_STARTED);
  assert(state != STATE_BODY_SKIPPING);
  assert(state != STATE_BODY_SKIPPED);
  assert(state != STATE_BODY_RECEIVING);
  assert(state != STATE_BODY_RECEIVED);
  assert(state != STATE_TERMINATED);

  abort();
}


void
NetworkAgent::terminate(int fd)
{
  // 
  state = STATE_TERMINATED;

  NetworkAgentHandle tmp = this;

  // XXX: access_failedʥڤóݤthisκѤͤusesite
  //̥åɤthisκѤߤ礬롣Υ᥽åɤ
  //λ˺ѤϤȡhandleuser_dataʤɤδ
  //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
  //Ѳǽ֤ԤAPIɬפ
  //षget(...)ȤǾԤƤɤ
  //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
  //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
  //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
  //ɤġġ
  LockBlock lock_obj(libochusha_giant);

  access_terminated(this, user_data);

  char message[LOG_MESSAGE_LENGTH];
  snprintf(message, LOG_MESSAGE_LENGTH,
	   LIB_("Access Terminated(%s)\n"), url);
  broker.output_log(INFORMATION_LOG, message);

  broker.notify_access_status(url, ACCESS_FINISHED);

  HTTPHandle *hdl = handle;
  handle = NULL;
  broker.connection_pool.release_http_handle(url, hdl, ACCESS_TERMINATED);
  user_data = NULL;
}


// connectionΩ뤿WorkerThreadǥꥯ˰٤ƤФ롣
void
NetworkAgent::start_access()
{
  assert(state == STATE_INITIAL);

  char message[LOG_MESSAGE_LENGTH];

  // EstablishConnectionJob꿶줿WorkerThreadǼ¹Ԥ졢
  // δؿreturnWorkerThreadϽλ롣
  handle->set_uri(url);
  if (mode == GET_REQUEST || mode == UPDATE_REQUEST)
    {
      snprintf(message, LOG_MESSAGE_LENGTH,
	       LIB_("HTTP Request: GET %s\n"), url);
      handle->set_method(HTTP_REQUEST_METHOD_GET);
    }
  else if (mode == POST_REQUEST)
    {
      snprintf(message, LOG_MESSAGE_LENGTH,
	       LIB_("HTTP Request: POST %s\n"), url);
      handle->set_method(HTTP_REQUEST_METHOD_POST);
      handle->set_body(post_body, strlen(post_body));
    }
  else
    {
      // EstablishConnectionJobreferenceäƤΤǤס
      // NetworkAgentHandle tmp = this;

      // XXX: access_failedʥڤóݤthisκѤͤusesite
      //̥åɤthisκѤߤ礬롣Υ᥽åɤ
      //λ˺ѤϤȡhandleuser_dataʤɤδ
      //íܤˤʤäƤޤ괺giantǼäƤܼŪˤ
      //Ѳǽ֤ԤAPIɬפ
      //षget(...)ȤǾԤƤɤ
      //ȤϤäƤ⡢ǤNetworkAgentsigslotʥ
      //libochusha_giantå˻ȤäƤΤǡΤޤޥʥ
      //ȯԸ塢Ѳǽˤʤޤlibochusha_giantΥå
      //ɤġġ
      LockBlock lock_obj(libochusha_giant);

      access_terminated(this, user_data);

      HTTPHandle *hdl = handle;
      handle = NULL;
      broker.connection_pool.release_http_handle(url, hdl, ACCESS_BUSY);
      user_data = NULL;

      return;
    }

  broker.pre_prepare_request(this);
  prepare_request(this, user_data);
  broker.post_prepare_request(this);

  if (mode == UPDATE_REQUEST)
    {
      if (handle->get_request_header(HTTPHeader::If_Modified_Since) != NULL)
	{
	  char range_value[64];	// size_t64bitsǤ28;͵ġġ
	  snprintf(range_value, 64, "bytes=%lu-",
		   static_cast<unsigned long>(buffer->get_length0())
		   - CACHE_COMPARISON_SIZE);
	  handle->set_request_header(HTTPHeader::Range, range_value);
	}
      else
	{
	  // ɤIf-Modified-SinceʤRangeGETꥯȤФ
	  // ɤʤȤ餷
	  mode = GET_REQUEST;
	  cache_check_required = false;
	  cache_comparison_buffer_length = 0;
	  buffer->clear(0);
	  cache_is_dirty(this, user_data);
	}
    }
#ifdef LIMIT_USE_OF_COOKIES
  if (mode == POST_REQUEST)
    {
#endif
      char *cookie = broker.cookie_manager.prepare_cookie(*handle->get_uri());
      if (cookie != NULL)
	{
	  assert(cookie[0] != '\0');

	  string cookie_buf(cookie);
	  free(cookie);

	  const char *user_cookies
	    = handle->get_request_header(HTTPHeader::Cookie);
	  if (user_cookies != NULL)
	    {
	      cookie_buf.append("; ");
	      cookie_buf.append(user_cookies);
	      handle->clear_request_header(HTTPHeader::Cookie);
	    }

	  handle->set_request_header(HTTPHeader::Cookie, cookie_buf.c_str());
	}
#ifdef LIMIT_USE_OF_COOKIES
    }
#endif

  handle->set_request_header(HTTPHeader::Connection, "close");
  handle->set_asynchronous(true);
  handle->set_nonblock(true);
  handle->prepare();

  state = STATE_STARTED;

  access_started(this, user_data);
  broker.output_log(DEBUG_LOG, message);

  URI uri(url);
  assert(uri.is_valid());
  assert(uri.get_server() != NULL);

  broker.notify_access_status(url, LOOK_UP_SERVER);

  HTTPStatus http_result;
  short event_mask;
  do
    {
      if (state == STATE_BODY_RECEIVING || state == STATE_BODY_SKIPPING)
	{
	  process_response_body_can_block();
	  // ʤpollingɬפʾrequest_poll()
	  // process_response_body_can_block()˷פ餦
	  return;
	}

      http_result = handle->process();

      if (http_result == HTTP_ERROR)
	{
	  notify_error();
	  // thisdestructƤǽꡣ
	  return;
	}

#ifndef NDEBUG
      NetworkAgentState previous_state = update_state();
#else
      update_state();
#endif

      /*
       * O_NONBLOCKʥʤΤǡ̾
       * previous_state == state == STATE_INITIAL
       * http_result == HTTP_NOT_DONE
       * event_mask == POLLOUT
       */

      if (state == STATE_BODY_RECEIVING)
	{
	  assert(previous_state != STATE_BODY_RECEIVING);

	  process_response_body_can_block();
	  // ʤpollingɬפʾrequest_poll()
	  // process_response_body_can_block()˷פ餦
	  return;
	}

      if (state == STATE_BODY_SKIPPING)
	{
	  assert(previous_state != STATE_BODY_SKIPPING);

	  process_response_body_can_block();
	  // ʤpollingɬפʾrequest_poll()
	  // process_response_body_can_block()˷פ餦
	  return;
	}

      if (state == STATE_TERMINATED)
	return;

      if (http_result == HTTP_DONE)
	{
	  // HEADꥯȤξʳϤ뤳ȤϤʤ
	  // ơΤȤNetworkAgentHEADꥯȤȤäƤʤġġ
	  assert(handle != NULL);

	  notify_finished();
	  return;
	}
      assert(http_result == HTTP_NOT_DONE);

      event_mask = static_cast<short>(handle->get_poll_event_mask());
    }
  while (event_mask == 0);

  assert(http_result == HTTP_NOT_DONE);

  int fd = handle->get_socket();
  assert(fd > 0);

  switch (state)
    {
    case STATE_STARTED:
      broker.notify_access_status(url, CONNECT_TO_SERVER);
      broker.manager.request_poll(this, fd, event_mask,
				  PollEventManager::PRIORITY_NORMAL,
				  PollEventManager::TIMEOUT_CONNECTION);
      return;


      /*****************************************************************
       * O_NONBLOCKդǥƤΤǰʲϸŪǤʤġġ*
       *****************************************************************/
    case STATE_CONNECTED:
      broker.notify_access_status(url, SEND_REQUEST);
      broker.manager.request_poll(this, fd, event_mask,
				  PollEventManager::PRIORITY_NORMAL,
				  PollEventManager::TIMEOUT_IO);
      return;


    case STATE_REQUEST_DONE:
    case STATE_STATUS_RECEIVED:
    case STATE_HEADERS_RECEIVED:
      broker.notify_access_status(url, WAIT_FOR_SERVER);
      broker.manager.request_poll(this, fd, event_mask,
				  PollEventManager::PRIORITY_NORMAL,
				  PollEventManager::TIMEOUT_IO);
      return;


    case STATE_INITIAL:
    case STATE_BODY_SKIPPING:
    case STATE_BODY_SKIPPED:
    case STATE_BODY_RECEIVING:
    case STATE_BODY_RECEIVED:
    case STATE_TERMINATED:
      // should never reached
      break;
    }

  // should never reached
  assert(state != STATE_INITIAL);
  assert(state != STATE_BODY_SKIPPING);
  assert(state != STATE_BODY_SKIPPED);
  assert(state != STATE_BODY_RECEIVING);
  assert(state != STATE_BODY_RECEIVED);
  assert(state != STATE_TERMINATED);

  abort();
}


void
NetworkAgent::get(const char *url, Buffer *buf, RefCount *data)
{
  assert(buf != NULL);

  if (this->url != NULL)
    free(this->url);
  this->url = strdup(url);

  if (post_body != NULL)
    {
      free(post_body);
      post_body = NULL;
    }

  mode = GET_REQUEST;
  state = STATE_INITIAL;

  buffer = buf;
  cache_check_required = false;
  cache_comparison_buffer_length = 0;
  bytes_total = 0;
  user_data = data;

  establish_connection();
}


void
NetworkAgent::update(const char *url, Buffer *buf, RefCount *data)
{
  assert(buf != NULL);

  if (this->url != NULL)
    free(this->url);

  this->url = strdup(url);
  if (post_body != NULL)
    {
      free(post_body);
      post_body = NULL;
    }

  if (buf->get_length0() < CACHE_COMPARISON_SIZE)
    {
      // åοĴ٤ۤɥǡʤΤñGET
      mode = GET_REQUEST;
      cache_check_required = false;
      buf->clear(0);
    }
  else
    {
      mode = UPDATE_REQUEST;
      cache_check_required = true;
    }

  state = STATE_INITIAL;

  buffer = buf;
  cache_comparison_buffer_length = 0;
  bytes_total = 0;
  user_data = data;

  establish_connection();
}


void
NetworkAgent::post(const char *url, const char *body, Buffer *buf,
		   RefCount *data)
{
  assert(buf != NULL);

  if (this->url != NULL)
    free(this->url);
  this->url = strdup(url);

  if (post_body != NULL)
    free(post_body);

  post_body = strdup(body);

  mode = POST_REQUEST;
  state = STATE_INITIAL;

  buffer = buf;
  cache_check_required = false;
  cache_comparison_buffer_length = 0;
  bytes_total = 0;
  user_data = data;

  establish_connection();
}


bool
NetworkAgent::terminate_access()
{
  if (handle != NULL)
    return handle->close();
  return false;
}


class SaveBufferJob : public WorkerJob
{
private:
  NetworkAgentHandle agent;
  string pathname;
  time_t last_modified_utc;
  int fd;

public:
  SaveBufferJob(NetworkAgent *agent, int fd);
  SaveBufferJob(NetworkAgent *agent, const char *pathname,
		const char *last_modified);
  virtual void work(const WorkerThread *employee);
  virtual const char *description() const
  { return "saving buffer to file"; }
};


SaveBufferJob::SaveBufferJob(NetworkAgent *agent, int fd)
  : agent(agent),
    last_modified_utc(-1),
    fd(fd)
{
}


SaveBufferJob::SaveBufferJob(NetworkAgent *agent, const char *pathname,
			     const char *last_modified)
  : agent(agent),
    pathname(pathname),
    last_modified_utc(get_utc_from_date(last_modified)),
    fd(open(pathname, O_WRONLY | O_TRUNC | O_CREAT, 0x180))
{
  if (fd < 0)
    {
      // ǥ쥯ȥ򷡤äƤĩ
      char *dirname = strdup(pathname);
      if (dirname != NULL)
	{
	  size_t len = strlen(dirname);
	  while (true)
	    {
	      if (dirname[len] == '/')
		break;
	      len--;
	    }
	  dirname[len] = '\0';

	  if (mkdir_p(dirname) == 0)
	    fd = open(pathname, O_WRONLY | O_TRUNC | O_CREAT, 0x180);
	}
    }
}


void
SaveBufferJob::work(const WorkerThread *employee)
{
  if (fd < 0)
    return;

  agent->get_buffer()->write_file(fd);

  if (last_modified_utc != -1 && !pathname.empty())
    {
      struct utimbuf buf = { last_modified_utc, last_modified_utc };
      utime(pathname.c_str(), &buf);
    }
}


void
NetworkAgent::save_buffer(int fd)
{
  broker.thread_pool->commit(new SaveBufferJob(this, fd));
}


void
NetworkAgent::save_buffer(const char *pathname, const char *last_modified)
{
  assert(pathname != NULL);
  assert(*pathname == '/');	// Хѥ

  broker.thread_pool->commit(new SaveBufferJob(this, pathname, last_modified));
}


void
NetworkAgent::set_proxy_url(const char *url)
{
  handle->set_proxy(url);
}


void
NetworkAgent::set_proxy_uri(const URI &uri)
{
  handle->set_proxy(uri);
}


void
NetworkAgent::set_proxy_authinfo(const char *user, const char *password)
{
  handle->set_proxy_authinfo(user, password);
}


void
NetworkAgent::set_header(const char *name, const char *value)
{
  handle->set_request_header(name, value);
}


void
NetworkAgent::set_non_interactive_mode(bool value)
{
  handle->set_policy(value
		     ? HTTP_POLICY_HIGH_THROUGHPUT : HTTP_POLICY_LOW_LATENCY);
}


const char *
NetworkAgent::get_header(const char *name) const
{
  if (handle == NULL)
    return NULL;

  return handle->get_response_header(name);
}


const HTTPHeaders *
NetworkAgent::get_headers() const
{
  if (handle == NULL)
    return NULL;

  return &handle->get_headers();
}


int
NetworkAgent::get_status_code() const
{
  if (handle == NULL)
    return 0;

  return handle->get_status_code();
}


const char *
NetworkAgent::get_reason_phrase() const
{
  const char *reason;
  if (handle != NULL)
    reason = handle->get_reason_phrase();
  else
    reason = NULL;

  return reason != NULL ? reason : "";
}


NetworkActivityNotifier::NetworkActivityNotifier(NetworkAgent *agent)
  : magic(0x22360679)
{
  if (agent != NULL)
    {
      LockBlock lock_obj(libochusha_giant);

      agent_state = agent->get_state();

      agent->access_started
	.connect(this, &NetworkActivityNotifier::access_started_cb);
      agent->connection_established
	.connect(this, &NetworkActivityNotifier::connection_established_cb);
      agent->request_sended
	.connect(this, &NetworkActivityNotifier::request_sended_cb);
      agent->status_received
	.connect(this, &NetworkActivityNotifier::status_received_cb);
      agent->response_header_received
	.connect(this, &NetworkActivityNotifier::response_header_received_cb);
      agent->response_body_received
	.connect(this, &NetworkActivityNotifier::response_body_received_cb);
      agent->access_progressed
	.connect(this, &NetworkActivityNotifier::access_progressed_cb);
      agent->access_finished
	.connect(this, &NetworkActivityNotifier::access_finished_cb);
      agent->cache_is_dirty
	.connect(this, &NetworkActivityNotifier::cache_is_dirty_cb);
      agent->access_failed
	.connect(this, &NetworkActivityNotifier::access_failed_cb);
      agent->access_terminated
	.connect(this, &NetworkActivityNotifier::access_terminated_cb);
    }
  else
    agent_state = NetworkAgent::STATE_INITIAL;
}


NetworkActivityNotifier::~NetworkActivityNotifier()
{
  magic = 0;
}


void
NetworkActivityNotifier::access_started_cb(NetworkAgent *agent, RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  access_started(agent);
}


void
NetworkActivityNotifier::connection_established_cb(NetworkAgent *agent,
						   RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  connection_established(agent);
}


void
NetworkActivityNotifier::request_sended_cb(NetworkAgent *agent, RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  request_sended(agent);
}


void
NetworkActivityNotifier::status_received_cb(NetworkAgent *agent, RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  status_received(agent);
}


void
NetworkActivityNotifier::response_header_received_cb(NetworkAgent *agent,
						     RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  response_header_received(agent);
}


void
NetworkActivityNotifier::response_body_received_cb(NetworkAgent *agent,
						   RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  response_body_received(agent);
}


void
NetworkActivityNotifier::access_progressed_cb(NetworkAgent *agent,
					      size_t data_read,
					      uintmax_t total_len,
					      RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  access_progressed(agent, data_read, total_len);
}


void
NetworkActivityNotifier::access_finished_cb(NetworkAgent *agent, RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  access_finished(agent);
}


void
NetworkActivityNotifier::cache_is_dirty_cb(NetworkAgent *agent, RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  cache_is_dirty(agent);
}


void
NetworkActivityNotifier::access_failed_cb(NetworkAgent *agent,
					  const char *reason, RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  access_failed(agent, reason);
}


void
NetworkActivityNotifier::access_terminated_cb(NetworkAgent *agent,
					      RefCount *data)
{
  assert(magic == 0x22360679);

  agent_state = agent->get_state();
  access_terminated(agent);
}


void
NetworkActivityNotifier::access_started(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::connection_established(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::request_sended(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::status_received(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::response_header_received(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::response_body_received(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::access_progressed(NetworkAgent *agent, size_t data_read, uintmax_t total_len)
{
}


void
NetworkActivityNotifier::access_finished(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::cache_is_dirty(NetworkAgent *agent)
{
}


void
NetworkActivityNotifier::access_failed(NetworkAgent *agent, const char *reason)
{
}


void
NetworkActivityNotifier::access_terminated(NetworkAgent *agent)
{
}


}

