/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: network_agent.h,v 1.35 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_NETWORK_AGENT_H_
#define _LIBOCHUSHA_NETWORK_AGENT_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/buffer.h"
#include "libochusha/constants.h"
#include "libochusha/http_handle.h"
#include "libochusha/network_connection_pool.h"
#include "libochusha/poll_event_manager.h"
#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"
#include "libochusha/thread_pool.h"

#if defined(HAVE_STDINT_H) || !defined(HAVE_CONFIG_H)
#include <stdint.h>
#else
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif
#endif


using namespace std;
namespace Ochusha
{
  class CookieManager;
  class NetworkAgent;

  enum NetworkAccessStatus {
    LOOK_UP_SERVER,
    CONNECT_TO_SERVER,
    SEND_REQUEST,
    WAIT_FOR_SERVER,
    TRANSFER_DATA,
    ACCESS_FINISHED
  };


  class NetworkAgentBroker
  {
  private:
    friend class NetworkAgent;
    friend class EstablishConnectionJob;
    friend class ProcessResponseBodyJob;
    CookieManager &cookie_manager;
    PollEventManager &manager;
    NetworkConnectionPool connection_pool;
    smart_ptr<ThreadPool> thread_pool;


  public:
    NetworkAgentBroker(CookieManager &cookie_manager, PollEventManager &manager,
		       ThreadPool *pool);
    NetworkAgent *employ_agent();


    // WorkerThread⤷PollEventMangerΥåɾȯ
    // void output_log(LogCategory category, const char *message);
    signal2<LogCategory, const char *> output_log;


    // ʲĤNetworkAgent̿ľWorkerThreadȯ

    // NetworkAgentprepare_requestʥȯľȯ
    // void pre_prepare_request(NetworkAgent *);
    signal1<NetworkAgent *> pre_prepare_request;

    // NetworkAgentprepare_requestʥȯľȯ
    // void post_prepare_request(NetworkAgent *);
    signal1<NetworkAgent *> post_prepare_request;


    // void notify_access_status(const char *url, NetworkAccessStatus status);
    signal2<const char *, NetworkAccessStatus> notify_access_status;
  };


  class NetworkAgent : public PollEventListener, public ConnectionRequest
  {
    friend class NetworkAgentBroker;
    friend class EstablishConnectionJob;
    friend class ProcessResponseBodyJob;

  public:
    enum NetworkAgentMode {
      NOT_SPECIFIED = 0,
      GET_REQUEST,
      UPDATE_REQUEST,
      POST_REQUEST
    };

    enum NetworkAgentState {
      STATE_INITIAL = 0,
      STATE_STARTED,
      STATE_CONNECTED,
      STATE_REQUEST_DONE,
      STATE_STATUS_RECEIVED,
      STATE_HEADERS_RECEIVED,
      STATE_BODY_SKIPPING,
      STATE_BODY_SKIPPED,
      STATE_BODY_RECEIVING,
      STATE_BODY_RECEIVED,
      STATE_TERMINATED
    };


  private:
    static const size_t CACHE_COMPARISON_SIZE = 64;


    NetworkAgentBroker &broker;
    NetworkAgentMode mode;

    NetworkAgentState state;

    char *url;
    char *post_body;
    HTTPHandle *handle;

    BufferHandle buffer;
    RefCountHandle user_data;

    char cache_comparison_buffer[CACHE_COMPARISON_SIZE];
    size_t cache_comparison_buffer_length;
    bool cache_check_required;

    uintmax_t bytes_total;


    NetworkAgent(NetworkAgentBroker &broker);

    bool process_response_body_real(int lock_id);
    void process_response_body_can_block();
    int process_response_body_nonblock();
    int handle_event(int fd, short events, short revents);
    void terminate(int fd);
    void start_access();


    // ʲhandle->process()η̤˱ɬפʽԤ
    NetworkAgentState update_state();

    void notify_finished();
    void notify_failure();
    void notify_error();

    void refresh_dirty_cache();


  public:
    ~NetworkAgent();
    inline NetworkAgentState get_state() const { return state; }

    inline Buffer *get_buffer() { return buffer; }
    inline RefCount *get_user_data() { return user_data; }

    virtual void notify_allowed(HTTPHandle *handle);
    virtual bool notify_busy();
    virtual void notify_denied();
    virtual const char *get_url();

    void establish_connection();


    // NetworkAgentư˱ƶͿ᥽å
    /*
     * ʲĶǽWorkerThreadꤽξǥͥåȥ
     * Ϥ롣DNSresolveconnect()ƥॳȯԤ
     * poll٥ȥϥɥPollEventManagerϿǽ
     * WorkerThreadǤνϽλ롣
     * θϡŪPollEventManagerpoll롼פ¹Ԥ륹åɤ
     * ɤ߹ǡBuffer˽񤭹बBufferwriter lock¨
     * ǤʤäˤϤλΥ٥ȽΤWorkerThread
     * Ȥ
     *
     * θƤӽФϡ̿ݤˤ餺¨¤롣
     */
    void get(const char *url, Buffer *buf, RefCount *data = NULL);
    void update(const char *url, Buffer *buf, RefCount *data = NULL);
    void post(const char *url, const char *body, Buffer *buf = NULL,
	      RefCount *data = NULL);

    bool terminate_access();

    void save_buffer(int fd);	// ̥åɤbufferƤfd¸
    void save_buffer(const char *pathname, const char *last_modified);


    /*
     * οʹԤ˹碌NetworkAgentȯ륷ʥ
     */
    // ̿ľ˸ƤФ롣Ƽrequest headerproxyʤɤ꤬ǽ
    // WorkerThreadȯ롣
    // void prepare_request(this, user_data);
    signal2<NetworkAgent *, RefCount *> prepare_request;


    inline NetworkAgentMode get_agent_mode() const { return mode; }
    inline const char *get_url() const { return url; }


    void set_proxy_url(const char *url);
    void set_proxy_uri(const URI &uri);
    void set_proxy_authinfo(const char *user, const char *password);


    void set_header(const char *name, const char *value);

    // ̿ݥꥷѹ롣
    // trueꤹpolling intervalĹ뤳Ȥthrough put
    // 
    void set_non_interactive_mode(bool value);


    // ̿ѥåɤưФǸƤФ롣
    // WorkerThreadȯ
    // void access_started(this, user_data);
    signal2<NetworkAgent *, RefCount *> access_started;


    // connectƥॳ뤬λǸƤФ롣
    // PollEventMangerΥåɾȯ
    // void connection_established(this, user_data);
    signal2<NetworkAgent *, RefCount *> connection_established;


    // HTTP requestإåλǸƤФ롣
    // PollEventManagerΥåɾȯ
    // void request_sended(this, user_data);
    signal2<NetworkAgent *, RefCount *> request_sended;


    // HTTP responseStatus-LineμλǸƤФ롣
    // ʹߡstatus_codereason phraseμǽ
    // void status_received(this, user_data);
    signal2<NetworkAgent *, RefCount *> status_received;
    int get_status_code() const;
    const char *get_reason_phrase() const;


    // HTTP responseƤheaderɤ߼괰λ˸ƤФ롣
    // ʹߡƼresponse headerμǽ
    // PollEventMangerΥåɾȯ
    // void response_header_received(this, user_data);
    signal2<NetworkAgent *, RefCount *> response_header_received;


    const char *get_header(const char *name) const;
    const HTTPHeaders *get_headers() const;


    // HTTP responseƤbodyɤ߼괰λ˸ƤФ롣
    // PollEventMangerΥåɤ⤷WorkerThreadȯ
    // void response_body_received(this, user_data);
    signal2<NetworkAgent *, RefCount *> response_body_received;


    // bufferؤΥǡ񤭹ľ˸ƤФ롣
    // PollEventManagerΥåɤ⤷WorkerThreadȯ롣
    // λwriter lockϲƤ롣
    // Content-LengthͿƤʤ̿ξ硢total_len0ˤʤ뤳Ȥ
    // ա
    // void access_progressed(this, data_read, total_len, user_data);
    signal4<NetworkAgent *, size_t, uintmax_t, RefCount *> access_progressed;


    // PollEventMangerΥåɤ⤷WorkerThreadȯ
    // void access_finished(this, user_data);
    signal2<NetworkAgent *, RefCount *> access_finished;


    // PollEventMangerΥåɤ⤷WorkerThreadȯ
    // void cache_is_dirty(this, user_data);
    signal2<NetworkAgent *, RefCount *> cache_is_dirty;


    // PollEventMangerΥåɤ⤷WorkerThreadȯ
    // void access_failed(this, reason, user_data);
    signal3<NetworkAgent *, const char *, RefCount *> access_failed;


    // PollEventMangerΥåɤ⤷WorkerThreadȯ
    // void access_terminated(this, user_data);
    signal2<NetworkAgent *, RefCount *> access_terminated;
  };

  typedef smart_ptr<NetworkAgent> NetworkAgentHandle;


  class NetworkActivityNotifier : public has_slots
  {
  private:
    int magic;	// ǥХå
    NetworkAgent::NetworkAgentState agent_state;

    void access_started_cb(NetworkAgent *agent, RefCount *data);
    void connection_established_cb(NetworkAgent *agent, RefCount *data);
    void request_sended_cb(NetworkAgent *agent, RefCount *data);
    void status_received_cb(NetworkAgent *agent, RefCount *data);
    void response_header_received_cb(NetworkAgent *agent, RefCount *data);
    void response_body_received_cb(NetworkAgent *agent, RefCount *data);
    void access_progressed_cb(NetworkAgent *agent, size_t data_read,
			      uintmax_t total_len, RefCount *data);
    void access_finished_cb(NetworkAgent *agent, RefCount *data);
    void cache_is_dirty_cb(NetworkAgent *agent, RefCount *data);
    void access_failed_cb(NetworkAgent *agent, const char *reason,
			  RefCount *data);
    void access_terminated_cb(NetworkAgent *agent, RefCount *data);


  protected:
    virtual void access_started(NetworkAgent *agent);
    virtual void connection_established(NetworkAgent *agent);
    virtual void request_sended(NetworkAgent *agent);
    virtual void status_received(NetworkAgent *agent);
    virtual void response_header_received(NetworkAgent *agent);
    virtual void response_body_received(NetworkAgent *agent);
    virtual void access_progressed(NetworkAgent *agent, size_t data_read,
				   uintmax_t total_len);
    virtual void access_finished(NetworkAgent *agent);
    virtual void cache_is_dirty(NetworkAgent *agent);
    virtual void access_failed(NetworkAgent *agent, const char *reason);
    virtual void access_terminated(NetworkAgent *agent);


  public:
    NetworkActivityNotifier(NetworkAgent *agent);
    virtual ~NetworkActivityNotifier();

    inline NetworkAgent::NetworkAgentState get_agent_state() const
    { return agent_state; }
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_NETWORK_AGENT_H_ */
