/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: network_connection_pool.h,v 1.19 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_NETWORK_CONNECTION_POOL_H_
#define _LIBOCHUSHA_NETWORK_CONNECTION_POOL_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "libochusha/http_handle.h"
#include "libochusha/lock.h"


#define DEFAULT_MAXIMUM_NUMBER_OF_CONNECTIONS	8


using namespace std;
namespace Ochusha
{


  enum ConnectionState {
    ACCESS_SUCCEEDED,
    ACCESS_BUSY,
    ACCESS_FAILED,
    ACCESS_TERMINATED
  };


  /*
   * NetworkConnectionPoolrequest_connection()ΰȤƻȤ롣
   *
   * request_connection()θƤӽФ
   * ľ̿򳫻ϤɤǤrequest_allowed()
   * Ʊ³¤ԤĤ٤Ǥrequest_busy()
   * ˴äݤξǤrequest_denied()
   * 줾쥳ХåȤƸƤФ롣
   *
   * request_busy()bool֤᥽åɤǡtrue֤ˤϸ
   * ̿򳫻Ϥɤˤʤäǲrequest_allowed()
   * ƤФ褦ConnectionRequestΤ塼롣false
   * ֤ˤϤΤޤ޲ʤäȤˤ롣
   */
  class ConnectionRequest
  {
  public:
    ConnectionRequest();
    virtual ~ConnectionRequest();

    
    virtual void notify_allowed(HTTPHandle *handle) = 0;
    virtual bool notify_busy() = 0;
    virtual void notify_denied() = 0;

    virtual const char *get_url() = 0;
  };


  class NetworkConnectionPool
  {
  private:
    RefCountHashtable server_info_table;	// server̾->ServerInfo
    Lock lock;


  public:
    NetworkConnectionPool(int max_connections
			  = DEFAULT_MAXIMUM_NUMBER_OF_CONNECTIONS);
    ~NetworkConnectionPool();

    void request_connection(ConnectionRequest *request);

    void release_http_handle(const char *url, HTTPHandle *handle,
			     ConnectionState state);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_NETWORK_CONNECTION_POOL_H_ */
