/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ochusha_procedure.h,v 1.22 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_OCHUSHA_PROCEDURE_H_
#define _LIBOCHUSHA_OCHUSHA_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/tscore.h"

#include <vector>


using namespace std;
using namespace LibTSCore;
namespace Ochusha
{
  class OchushaProcedure;

  class ThreadlistRuleInfo
  {
    friend class OchushaProcedure;

  private:
    char *name;
    char *label;
    char *description;
    CellHandle procedure;

    void set_name(Cell *name_cell);
    void set_label(Cell *label_cell);
    void set_description(Cell *desc_cell);
    void set_procedure(Cell *proc_cell);


  public:
    ThreadlistRuleInfo(Cell *name_cell, Cell *label_cell, Cell *desc_cell,
		       Cell *proc_cell, TSCore &core);
    ~ThreadlistRuleInfo();

    inline const char *get_name() const { return name; }
    inline const char *get_label() const { return label; }
    inline const char *get_description() const { return description; }
    inline Cell *get_procedure() const { return procedure.get_cell(); }
  };


  class ExternalToolInfo
  {
    friend class OchushaProcedure;

  public:
    enum ToolType {
      UNKNOWN_TOOL = 0,
      LINK_TOOL,
      IMAGE_TOOL,
      TEXT_TOOL
    };


  private:
    char *name;
    char *label;
    char *description;
    ToolType tool_type;
    CellHandle command_line_template;

    void set_name(Cell *name_cell);
    void set_label(Cell *label_cell);
    void set_description(Cell *desc_cell);
    void set_tool_type(Cell *tool_type_cell);
    void set_template(Cell *template_cell);


  public:
    ExternalToolInfo(Cell *name_cell, Cell *label_cell, Cell *desc_cell,
		     Cell *tool_type_cell, Cell *template_cell, TSCore &core);
    ~ExternalToolInfo();

    inline const char *get_name() const { return name; }
    inline const char *get_label() const { return label; }
    inline const char *get_description() const { return description; }
    inline ToolType get_tool_type() const { return tool_type; }
    inline Cell *get_template() const
    { return command_line_template.get_cell(); }
  };


  class OchushaProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];

    typedef vector<ThreadlistRuleInfo *> ThreadlistRuleInfoVector;
    ThreadlistRuleInfoVector threadlist_rule_list;

    typedef vector<ExternalToolInfo *> ExternalToolInfoVector;
    ExternalToolInfoVector external_tool_list;


    void register_threadlist_rule(Cell *name, Cell *label, Cell *desc,
				  Cell *proc);
    void register_external_tool(Cell *name, Cell *label, Cell *desc,
				Cell *tool_type, Cell *command_line_template);


  public:
    OchushaProcedure(TSCore &core);
    virtual ~OchushaProcedure();

    const ThreadlistRuleInfo *lookup_threadlist_rule_info(const char *name) const;
    inline ThreadlistRuleInfoVector::const_iterator threadlist_rule_list_begin() const { return threadlist_rule_list.begin(); }
    inline ThreadlistRuleInfoVector::const_iterator threadlist_rule_list_end() const { return threadlist_rule_list.end(); }
    void threadlist_rule_list_clear();


    const ExternalToolInfo *lookup_external_tool_info(const char *name) const;
    inline ExternalToolInfoVector::const_iterator external_tool_list_begin() const { return external_tool_list.begin(); }
    inline ExternalToolInfoVector::const_iterator external_tool_list_end() const { return external_tool_list.end(); }
    void external_tool_list_clear();


    Cell *assign_procedures(Cell *environment, Cell *frame);


    // (url-encode-string STRING)
    static Cell *url_encode_string(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data);
    // (url-decode-string STRING)
    static Cell *url_decode_string(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data);
    // (board-get-bbs-type BOARD)
    static Cell *board_get_bbs_type(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    // (board-get-url BOARD)
    static Cell *board_get_url(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    // (board-get-name BOARD)
    static Cell *board_get_name(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);


    // (thread-get-bbs-type THREAD)
    static Cell *thread_get_bbs_type(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data);
    // (thread-get-board THREAD)
    static Cell *thread_get_board(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    // (thread-get-id THREAD)
    static Cell *thread_get_id(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    // (thread-get-title THREAD)
    static Cell *thread_get_title(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    // (thread-get-url THREAD)
    static Cell *thread_get_url(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
    // (thread-get-rank THREAD)
    static Cell *thread_get_rank(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    // (thread-get-previous-rank THREAD)
    static Cell *thread_get_previous_rank(VirtualMachine &vm,
					  Register &context,
					  unsigned long argument_pointer,
					  unsigned long nargs, void *data);
    // (thread-get-number-of-responses-on-server THREAD)
    static Cell *thread_get_number_of_responses_on_server(
					VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (thread-get-number-of-responses-read THREAD)
    static Cell *thread_get_number_of_responses_read(
					VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (thread-get-number-of-responses-got THREAD)
    static Cell *thread_get_number_of_responses_got(
					VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (thread-get-number-of-new-responses THREAD)
    static Cell *thread_get_number_of_new_responses(
					VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (thread-get-number-of-responses-available THREAD)
    static Cell *thread_get_number_of_responses_available(
					VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (thread-get-number-of-responses-to-read THREAD)
    static Cell *thread_get_number_of_responses_to_read(
					VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (thread-get-birthtime THREAD)
    static Cell *thread_get_birthtime(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);
    // (thread-new? THREAD)
    static Cell *is_thread_new(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    // (thread-alive? THREAD)
    static Cell *is_thread_alive(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    // (thread-stopped? THREAD)
    static Cell *is_thread_stopped(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data);
    // (thread-marked? THREAD)
    static Cell *is_thread_marked(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    // (thread-hidden? THREAD)
    static Cell *is_thread_hidden(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);

    // (register-threadlist-rule NAME LABEL DESCRIPTION PROCEDURE)
    static Cell *register_threadlist_rule(VirtualMachine &vm,
					  Register &context,
					  unsigned long argument_pointer,
					  unsigned long nargs, void *data);

    // (register-external-tool NAME LABEL DESCRIPTION TOOLTYPE TEMPLorPROC)
    static Cell *register_external_tool(VirtualMachine &vm,
					Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);

    // (network-agent-get-mode agent)
    static Cell *network_agent_get_mode(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    // (network-agent-get-url agent)
    static Cell *network_agent_get_url(VirtualMachine &vm, Register &context,
				       unsigned long argument_pointer,
				       unsigned long nargs, void *data);
    // (network-agent-set-header agent name value)
    static Cell *network_agent_set_header(VirtualMachine &vm, Register &context,
					  unsigned long argument_pointer,
					  unsigned long nargs, void *data);
    // (network-agent-get-header agent name)
    static Cell *network_agent_get_header(VirtualMachine &vm, Register &context,
					  unsigned long argument_pointer,
					  unsigned long nargs, void *data);
    // (network-agent-get-headers agent)
    static Cell *network_agent_get_headers(VirtualMachine &vm,
					   Register &context,
					   unsigned long argument_pointer,
					   unsigned long nargs, void *data);
    // (network-agent-get-status-code agent)
    static Cell *network_agent_get_status_code(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data);
    // (network-agent-get-reason-phrase agent)
    static Cell *network_agent_get_reason_phrase(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data);
    // (network-agent-get-response-body agent)
    static Cell *network_agent_get_response_body(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data);
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_OCHUSHA_PROCEDURE_H_ */
