/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: poll_event_manager.h,v 1.14 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_POLL_EVENT_MANAGER_H_
#define _LIBOCHUSHA_POLL_EVENT_MANAGER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "libochusha/lock.h"
#include "libochusha/ref_count.h"
#include "libochusha/thread_pool.h"

#include <pthread.h>


using namespace std;
namespace Ochusha
{


#define EVENT_RESULT_FLAG_CONTENTION		(static_cast<int>(0x80000000))

#define EVENT_RESULT_POLL_EVENT_MASK		0xffff
#define EVENT_RESULT_POLL_INTERVAL_SHIFT	16
#define EVENT_RESULT_POLL_INTERVAL_MASK		0x7fff0000


#define EVENT_RESULT_CONTINUE(req, interval)	(req | ((interval << EVENT_RESULT_POLL_INTERVAL_SHIFT) & EVENT_RESULT_POLL_INTERVAL_MASK))
#define EVENT_RESULT_CONTENTION			(EVENT_RESULT_FLAG_CONTENTION)
#define EVENT_RESULT_DONE			(0)


#define CHECK_WAIT_REQUESTED(result)		((result & EVENT_RESULT_POLL_INTERVAL_MASK) != 0)
#define GET_NEXT_EVENT_REQUEST(result)		(static_cast<short>(result & EVENT_RESULT_POLL_EVENT_MASK))
#define GET_NEXT_EVENT_POLL_INTERVAL(result)	(static_cast<int>((result >> 16) & 0x7fff))


  class PollEventListener : public RefCount
  {
    friend class PollEventManager;

  protected:
    // EVENT_RESULT_CONTINUE(new_event_mask, interval)
    // ֤ͤ硢new_event_maskͤ򿷤ʥ٥ȥޥȤ
    // intervalߥø夫polling³롣

    // EVENT_RESULT_CONTENTION
    // EVENT_RESULT_DONE
    // ֤ͤ硢PollEventManagerоݤȤʤäfdƻоݤ鳰
    // Ĥͤϡñפ뤿˻ȤʬƤ롣

    // fd: poll׵ᤷfd
    // events: ԤäƤevents
    // revents: äevents
    virtual int handle_event(int fd, short events, short revents) = 0;

    // ̿ߤ줿˸ƤФ롣
    virtual void terminate(int fd) = 0;
  };


  class PollEventManager
  {
  public:
    PollEventManager();
    ~PollEventManager();


    enum Priority {
      PRIORITY_HIGH,
      PRIORITY_NORMAL,
      PRIORITY_LOW
    };


    enum TimeoutType {
      TIMEOUT_CONNECTION = -7474,
      TIMEOUT_IO = -7272,
      TIMEOUT_NEVER = -7171
    };


    class Event;


  private:
    int command_pipe[2];
    Lock lock;
    pthread_t event_loop_thread_id;

    typedef smart_ptr<Event> EventHandle;
    typedef list<EventHandle> EventList;
    EventList event_list;		// priority˥
    unsigned int number_of_events;

    unsigned long total_number_of_events;
    unsigned long total_number_of_contention;

    int timeout_connection;
    int timeout_io;

    void event_loop();
    static void *event_loop_thread_main(void *arg);


  public:
    void *request_poll(PollEventListener *listener, int fd, short events,
		       Priority priority, int timeout);
    void terminate_all();

    inline int get_timeout_connection() const { return timeout_connection; }
    inline void set_timeout_connection(int msec) { timeout_connection = msec; }
    inline int get_timeout_io() const { return timeout_io; }
    inline void set_timeout_io(int msec) { timeout_io = msec; }
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_POLL_EVENT_MANAGER_H_ */
