/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: ref_count.h,v 1.19 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_REF_COUNT_H_
#define _LIBOCHUSHA_REF_COUNT_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <cassert>
#include <cstdlib>


using namespace std;
namespace Ochusha
{
  class RefCount
  {
  private:
    long count;

  public:
    RefCount() : count(0) {}
    virtual ~RefCount() { assert(count == 0); }
    inline void ref() { ++count; }
    inline void unref() { if (--count == 0) delete this; }
    // for debugging
    inline long get_count() { return count; }
  };


  template <class T>
  class smart_ptr
  {
  private:
    T *pointer;

  public:
    inline smart_ptr(T *obj = 0) : pointer(obj)
    { if (obj != 0) obj->ref(); }

    inline smart_ptr(const smart_ptr &sp) : pointer(sp.pointer)
    { if (pointer != 0) pointer->ref(); }

    inline ~smart_ptr() { if (pointer != 0) pointer->unref(); }

    inline operator T *() const { return pointer; }

    inline T *operator->() const { return pointer; }

    inline smart_ptr &operator=(T *obj)
    {
      if (obj != 0) obj->ref();
      if (pointer != 0) pointer->unref();
      pointer = obj;
      return *this;
    }

    inline smart_ptr &operator=(const smart_ptr &sp)
    {
      if (sp.pointer != 0) sp.pointer->ref();
      if (pointer != 0) pointer->unref();
      pointer = sp.pointer;
      return *this;
    }

    inline bool operator==(smart_ptr &sp) const
    { return pointer == sp.pointer; }

    inline bool operator==(const smart_ptr &sp) const
    { return pointer == sp.pointer; }

    inline bool operator==(T *obj) const
    { return pointer == obj; }

    inline bool operator==(const T *obj) const
    { return pointer == obj; }
  };

  typedef smart_ptr<RefCount> RefCountHandle;

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_REF_COUNT_H_ */
