/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: repository.h,v 1.21 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_REPOSITORY_H_
#define _LIBOCHUSHA_REPOSITORY_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/cache_manager.h"
#include "libochusha/bbs_data_manager.h"
#include "libochusha/ref_count.h"

#include <cstddef>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/stat.h>


using namespace std;
namespace Ochusha
{

  class Repository;

  class Tmpfile : public RefCount
  {
  private:
    friend class Repository;
    char *fullpath;
    int fd;
    struct stat buf;

    Tmpfile(const char *fullpath, int fd);


  public:
    ~Tmpfile();

    const char *get_fullpath() const { return fullpath; }
    int get_fd() const { return fd; }
    void close();	// ŪĤʳϥեϾõ

    const struct stat *stat();
  };

  typedef smart_ptr<Tmpfile> TmpfileHandle;


  class Repository
  {
  private:
    char *home_path;

    BBSDataManager bbs_data;

    char *prepare_home_path(const char *path);


  public:
    Repository(const char *path);
    virtual ~Repository();

    const char *get_home_path();

    char *expand_path(const char *path, char *buf, size_t buf_size) const;
    bool file_exist(const char *filename) const;
    bool directory_exist(const char *dirname) const;
    char *find_file(const char *filename) const;
    char *find_file(const char *filename, char *buf, size_t buf_size) const;
    char *find_directory(const char *dirname) const;
    char *find_directory(const char *dirname, char *buf, size_t buf_size) const;
    int open_file(const char *filename, int flags) const;
    FILE *fopen_file(const char *filename, const char *mode) const;
    Tmpfile *create_tmpfile(int flags) const;
    bool unlink_file(const char *filename) const;
    bool unlink_directory(const char *dirname) const;
    bool rename(const char *fromname, const char *toname) const;
    bool touch(const char *filename) const;
    bool stat(const char *filename, struct stat &sb) const;

    typedef void (*ForeachFunc)(const char *fullpath, const char *filename,
				struct stat *sb, void *user_data);

    void foreach_file(const char *dir, ForeachFunc func, void *user_data) const;
    void foreach_dir(const char *dir, ForeachFunc func, void *user_data) const;


    BBSDataManager &get_bbs_data() { return bbs_data; }
  };


}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_REPOSITORY_H_ */
