/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sax_context.cc,v 1.14 2009/01/01 07:46:43 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/sax_context.h"

#include <cstring>
#include <iostream>

#include <libxml/parser.h>


namespace Ochusha
{

void
SAXContext::start_element_func(void *context,
			       const xmlChar *name, const xmlChar **attrs)
{
  SAXContext *sax_context = static_cast<SAXContext *>(context);
  sax_context->start_element_handler(reinterpret_cast<const char *>(name),
				     reinterpret_cast<const char **>(attrs));
}


void
SAXContext::end_element_func(void *context, const xmlChar *name)
{
  SAXContext *sax_context = static_cast<SAXContext *>(context);
  sax_context->end_element_handler(reinterpret_cast<const char *>(name));
}


void
SAXContext::characters_func(void *context, const xmlChar *ch, int len)
{
  SAXContext *sax_context = static_cast<SAXContext *>(context);
  sax_context->characters_handler(reinterpret_cast<const char *>(ch),
				  static_cast<size_t>(len));
}


static void
nop_func(void *context)
{
}


static xmlEntityPtr
get_entity_func(void *context, const xmlChar *name)
{
  return NULL;
}


void
SAXContext::parse_xml_file(const char *pathname)
{
  xmlSAXHandler sax_handler;
  memset(&sax_handler, 0, sizeof(xmlSAXHandler));

  xmlSAX2InitDefaultSAXHandler(&sax_handler, 1);

  sax_handler.getEntity = get_entity_func;
  sax_handler.startDocument = nop_func;
  sax_handler.endDocument = nop_func;
  sax_handler.startElement = start_element_func;
  sax_handler.endElement = end_element_func;
  sax_handler.startElementNs = NULL;
  sax_handler.endElementNs = NULL;
  sax_handler.characters = characters_func;

  xmlSAXUserParseFile(&sax_handler, this, pathname);
}

}
