/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sigslot.cc,v 1.24 2009/01/01 07:46:43 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libochusha/sigslot.h"

#include <cassert>


namespace Ochusha
{


static Monitor sigslot_giant("sigslot");


has_slots::has_slots(const has_slots &hs)
  : lock(hs.lock)
{
  MonitorBlock giant(sigslot_giant);

  while (true)
    {
      if (hs.senders.empty())
	{
	  assert(lock == NULL);

	  return;	// connectƤsignal̵硣
	}

      assert(lock != NULL);

      if (lock->trylock())
	break;

      giant.wait(10);
    }

  assert(lock != NULL);
  assert(lock->is_locked_by_self());
  assert(sigslot_giant.is_locked_by_self());

  set<_signal_base *>::const_iterator iter = hs.senders.begin();
  set<_signal_base *>::const_iterator end_iter = hs.senders.end();
  while (iter != end_iter)
    {
      (*iter)->slot_duplicate(&hs, this);
      senders.insert(*iter++);
    }

  lock->unlock();

  assert(!senders.empty());
}


has_slots::~has_slots()
{
  disconnect_all();
}


void
has_slots::signal_connect(_signal_base *sender)
{
  assert(sender->lock->is_locked_by_self());


  MonitorBlock giant(sigslot_giant);

  // connecthas_slots_signal_baseƱåǥɤƤʤ
  // Фʤʤ

  // MEMO: connect줿has_slots_signal_baseΥǥȥ饯̥å
  // Ʊ˼¹Ԥ줿ˡ줾ξ֤ݤĤˤϥåɬ
  // ߤ˼ʬǥȥ饯Ȥ뤳Ȥ礦ᡢ줾
  // ¾椬̤Ȥʤꡢ̤ΥåѤƤˤɬ
  // ؤ롣
  assert(lock == NULL || lock == sender->lock);

  lock = sender->lock;
  senders.insert(sender);
}


void
has_slots::signal_disconnect(_signal_base *sender)
{
  assert(sender->lock->is_locked_by_self());


  MonitorBlock giant(sigslot_giant);

  senders.erase(sender);

  if (senders.empty())
    lock = NULL;
}


void
has_slots::disconnect_all()
{
  MonitorBlock giant(sigslot_giant);

  while (true)
    {
      if (senders.empty())
	{
	  assert(lock == NULL);

	  return;
	}

      assert(lock != NULL);

      if (lock->trylock())
	break;

      giant.wait(10);
    }

  assert(lock != NULL);
  assert(lock->is_locked_by_self());
  assert(sigslot_giant.is_locked_by_self());

  set<_signal_base *>::const_iterator iter = senders.begin();
  set<_signal_base *>::const_iterator end_iter = senders.end();
  while (iter != end_iter)
    (*iter++)->slot_disconnect(this);

  senders.clear();

  lock->unlock();
  lock = NULL;
}


_signal_emitter_context::~_signal_emitter_context()
{
  if (signal_base != NULL)
    {
      assert(signal_base->emitter_context == this);

      signal_base->emitter_context = NULL;
    }
}


_connection_base *
_signal_emitter_context::next_slot()
{
  assert(!pending_slots.empty());

  _connection_base *slot = pending_slots.front();
  pending_slots.pop_front();

  return slot;
}


_signal_base::_signal_base(Lock *lock)
  : emitter_context(NULL)
{
  if (lock == NULL)
    this->lock = &libochusha_giant;
  else
    this->lock = lock;
}


_signal_base::~_signal_base()
{
  disconnect_all();

  assert(emitter_context == NULL);
}


void
_signal_base::signal_connect(_connection_base *conn, has_slots *target)
{
  assert(target != NULL);

  LockBlock lock_obj(lock);

  assert(find(connected_slots.begin(), connected_slots.end(), conn)
	 == connected_slots.end());

  connected_slots.push_back(conn);
  target->signal_connect(this);
}


void
_signal_base::disconnect_all()
{
  LockBlock lock_obj(lock);

  if (connected_slots.empty())
    return;


  if (emitter_context != NULL)
    {
      assert(emitter_context->signal_base == this);

      emitter_context->remove_all();
      emitter_context->signal_base = NULL;
      emitter_context = NULL;
    }


  MonitorBlock giant(sigslot_giant);

  while (!connected_slots.empty())
    {
      _connection_base *base = connected_slots.front();
      base->get_dest()->signal_disconnect(this);
      connected_slots.pop_front();

      delete base;
    }
}


void
_signal_base::disconnect(has_slots *slot_holder)
{
  LockBlock lock_obj(lock);

  if (connected_slots.empty())
    return;


  MonitorBlock giant(sigslot_giant);

  list<_connection_base *>::iterator iter = connected_slots.begin();
  list<_connection_base *>::iterator end_iter = connected_slots.end();
  while (iter != end_iter)
    {
      if ((*iter)->get_dest() == slot_holder)
	{
	  delete *iter;
	  connected_slots.erase(iter);
	  slot_holder->signal_disconnect(this);

	  return;
	}
      ++iter;
    }
}


// has_slots::disconnect_all()θƤӽФѡ
void
_signal_base::slot_disconnect(has_slots *slot)
{
  assert(lock->is_locked_by_self());

  _connection_base *base = NULL;
  list<_connection_base *>::iterator iter = connected_slots.begin();
  list<_connection_base *>::iterator end_iter = connected_slots.end();
  while (iter != end_iter)
    {
      _connection_base *base_at_iter = *iter;
      if (base_at_iter->get_dest() == slot)
	{
	  connected_slots.erase(iter);
	  base = base_at_iter;
	  break;
	}

      iter++;
    }

  assert(base != NULL);		// connectƤʤhas_slots׵ᡩ

  if (emitter_context != NULL)
    emitter_context->remove_slot(base);

  delete base;
}


// has_slots::has_slots(const has_slots &hs)θƤӽФѡ
void
_signal_base::slot_duplicate(const has_slots *oldtarget, has_slots *newtarget)
{
  assert(lock->is_locked_by_self());

  list<_connection_base *>::iterator iter = connected_slots.begin();
  list<_connection_base *>::iterator end_iter = connected_slots.end();
  while (iter != end_iter)
    {
      _connection_base *base = *iter;
      if (base->get_dest() == oldtarget)
	connected_slots.push_back(base->duplicate(newtarget));

      ++iter;
    }
}


void
_signal_base::prepare_for_emission(_signal_emitter_context &context)
{
  // 괺recursiveʥʥȯԤ󥵥ݡȤȤȤǡġ
  assert(emitter_context == NULL);
  assert(lock->is_locked_by_self());

  context.prepare_for_emission(connected_slots, *this);
  emitter_context = &context;
}


_connection_base::~_connection_base()
{
}


void
Signal0::emit()
{
  LockBlock lock_obj(lock);
  _signal_emitter_context context;

  prepare_for_emission(context);
  while (context.has_more_slots())
    {
      _connection_base *slot = context.next_slot();
      dynamic_cast<_connection_base0 *>(slot)->emit();
    }
}


}


#ifdef SIG_TEST
#include <iostream>

using namespace Ochusha;

class Switch
{
public:
  Signal0 clicked;
};


class Counter : public has_slots
{
private:
  int count;

public:
  Counter() { count = 0; }
  void increment()
  {
    cerr << "count=" << ++count << endl;
    incremented(this);
  }

  signal1<Counter *> incremented;
};


class CounterWatcher : public has_slots
{
public:
  void show_incremented(Counter *counter)
  {
    cerr << "counter(" << counter << ") incremented.\n";
  }
};


int
main(int argc, char *argv[])
{
  Switch sw;
  Counter counter;
  CounterWatcher watcher;
  sw.clicked.connect(&counter, &Counter::increment);
  sw.clicked();

  counter.incremented.connect(&watcher, &CounterWatcher::show_incremented);
  sw.clicked();
  return 0;
}
#endif
