/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sqlite_utils.h,v 1.5 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_SQLITE_UTILS_H_
#define _LIBOCHUSHA_SQLITE_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <sqlite3.h>

#if SQLITE_VERSION_NUMBER < 3005000
typedef sqlite_int64 sqlite3_int64;
typedef sqlite_uint64 sqlite3_uint64;
#endif


using namespace std;
namespace Ochusha
{

  class SQLite3;

  class SQLite3Statement
  {
    friend class SQLite3;

  protected:
    sqlite3 *db_handle;
    sqlite3_stmt *statement;


  public:
    SQLite3Statement(SQLite3 &db_handle, const char *sql_templ);
    ~SQLite3Statement();

    bool bind(int index, int value);
    bool bind(int index, sqlite3_int64 value);
    bool bind(int index, const char *value);
    bool bind_static(int index, const char *value);

    inline int get_column_int(int column)
    { return sqlite3_column_int(statement, column); }
    inline sqlite3_int64 get_column_int64(int column)
    { return sqlite3_column_int64(statement, column); }
    inline const char *get_column_text(int column)
    {
      return reinterpret_cast<const char *>(sqlite3_column_text(statement,
								column));
    }

    inline bool reset()
    { return sqlite3_reset(statement) == SQLITE_OK; }

    inline bool clear_bindings()
    { return sqlite3_clear_bindings(statement) == SQLITE_OK; }

    int step();
  };


  class SQLite3
  {
  protected:
    sqlite3 *db_handle;

    char **query_result;
    int number_of_rows;
    int number_of_columns;
    char *error_message;


  public:
    SQLite3(const char *db_filename);
    ~SQLite3();

    inline sqlite3 *get_db_handle() { return db_handle; }
    inline const char * const *get_query_result() { return query_result; }
    inline int get_number_of_rows() { return number_of_rows; }
    inline int get_number_of_columns() { return number_of_columns; }
    inline const char *get_error_message() { return error_message; }

    void reset_table();
    bool get_table(const char *sql);

    inline SQLite3Statement *statement_new(const char *sql_templ)
    { return new SQLite3Statement(*this, sql_templ); }
  };


}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_SQLITE_UTILS_H_ */
