/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: text_utils.h,v 1.22 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_TEXT_UTILS_H_
#define _LIBOCHUSHA_TEXT_UTILS_H_

#ifdef __cplusplus

#include <iconv.h>

#include <cstdlib>
#include <cstring>

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

extern "C" {
typedef size_t (*IconvHelperFunc)(const char **inbuf,
				  size_t *inbytesleft,
				  char **outbuf, size_t *outbytesleft);
}

using namespace std;
namespace Ochusha
{

  /* Ĺchar󡣴Ūˤ򥹥å˳ݤ­
   * ҡפѤ롽ȤΤΡ˳饹
   */
  class CStringBuffer
  {
  protected:
    char default_buffer[4096];
    size_t buffer_length;
    char *buffer;
    char *tail;


  public:
    CStringBuffer()
    {
      buffer_length = sizeof(default_buffer);
      buffer = default_buffer;
      tail = buffer;
    }

    virtual ~CStringBuffer()
    {
      if (buffer != default_buffer && buffer != NULL)
	free(buffer);
    }

    inline size_t get_buffer_length() const { return buffer_length; }
    inline const char *get_string() const { return buffer; }
    inline size_t get_string_length() const { return tail - buffer; }

    // ñʤĹcharȤƻȤ桼ˡ
    inline char *get_buffer() { return buffer; }

    // Хåե2ܤ˳ĥʬ֤
    size_t expand();

    // Хåեrequired_len礭ʤޤǳĥtrue֤
    bool ensure_capacity(size_t required_len);

    inline void append(const char *text) { append(text, strlen(text)); }
    virtual void append(const char *text, size_t length);
    virtual void clear() { tail = buffer; *tail = '\0'; }
  };


  class IconvBuffer : public CStringBuffer
  {
  private:
    iconv_t converter;
    IconvHelperFunc helper;


  public:
    static const char * const CP932_ENCODING;
    static const char * const EUCJP_ENCODING;
    static const char * const UTF8_ENCODING;
    static const char * const UTF8_ENCODING_IGNORE;

    IconvBuffer(const char *to_encoding, const char *from_encoding,
		IconvHelperFunc helper);
    IconvBuffer(const char *to_encoding, const char *from_encoding);
    virtual ~IconvBuffer();

    virtual void append(const char *text, size_t length);
  };

}


extern "C" {
#endif

const char *get_canonical_encoding_name(const char *encoding);
const char *get_encoding_from_content_type(const char *content_type);
char *wipe_string(const char *str);
int parse_int(const char *start, size_t len);

#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHA_TEXT_UTILS_H_ */
