/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: thread_pool.h,v 1.17 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_THREAD_POOL_H_
#define _LIBOCHUSHA_THREAD_POOL_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/lock.h"
#include "libochusha/ref_count.h"

#include <list>

#ifdef HAVE_PTHREAD_H
#include <pthread.h>
#endif


using namespace std;
namespace Ochusha
{

  class WorkerThread;

  class WorkerJob : public RefCount
  {
  public:
    virtual void work(const WorkerThread *employee) = 0;
    virtual const char *description() const = 0;	// ǥХå
  };


  class ThreadPool : public RefCount
  {
    friend class WorkerThread;

  private:
    Monitor monitor;

    list<WorkerThread *> worker_threads;
    typedef smart_ptr<WorkerJob> PendingJob;
    list<PendingJob> pending_jobs;

    volatile unsigned int number_of_threads;
    volatile unsigned int number_of_idle_threads;
    unsigned int maximum_number_of_threads;

    bool employ_new_worker();


  public:
    ThreadPool(unsigned int initial_num_threads = 0,
	       unsigned int maximum_num_threads = 32);
    ~ThreadPool();

    // jobμ¹Ԥꤹ롣
    // forcetrueξ硢åɿ¤̵뤷ƿWorkerThread롣
    bool commit(WorkerJob *job, bool force = false);

    // Ƥpendingjobcancel롣
    void cancel_all();

    // ƤΥåɤߤ褦Ȼߤ롣
    void terminate_workers();
  };


#ifdef CHECK_LOCK_ORDER
  class WorkerThreadInfo;
#endif

  class WorkerThread
  {
    friend class ThreadPool;
    friend class JoinThreadFunc;
    friend class SetDieNowFunc;

  public:
    enum WorkerState { RUNNABLE, BUSY, DIE_NOW, DEAD, DETACHED };


  private:
    ThreadPool &pool;
    pthread_t id;
    WorkerState state;
    smart_ptr<WorkerJob> current_job;


    static void *worker_thread_main(void *arg);

    WorkerThread(ThreadPool &pool, WorkerJob *job = NULL);
    bool join();


  public:
    inline WorkerState get_state() const volatile { return state; }
    inline const WorkerJob *get_current_job() const { return current_job; }
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_THREAD_POOL_H_ */
