/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: uri_utils.h,v 1.21 2009/01/01 07:46:43 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_URI_UTILS_H_
#define _LIBOCHUSHA_URI_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <libxml/uri.h>


using namespace std;
namespace Ochusha
{

  class URI
  {
  private:
    char *full_uri;
    xmlURIPtr xml_uri;


  public:
    URI(const char *uri = NULL);
    URI(const URI &uri);
    ~URI();

    bool parse_uri(const char *uri);

    inline bool is_valid() const { return full_uri != NULL; }
    inline const char *get_uri() const { return full_uri; }
    inline const char *get_scheme() const
    { return xml_uri != NULL ? xml_uri->scheme : NULL; }
    inline const char *get_opaque() const
    { return xml_uri != NULL ? xml_uri->opaque : NULL; }
    inline const char *get_authority() const
    { return xml_uri != NULL ? xml_uri->authority : NULL; }
    inline const char *get_server() const
    { return xml_uri != NULL ? xml_uri->server : NULL; }
    inline const char *get_user() const
    { return xml_uri != NULL ? xml_uri->user : NULL; }
    inline int get_port() const
    { return xml_uri != NULL ? xml_uri->port : 0; }
    inline const char *get_path() const
    {
      return xml_uri != NULL
	? ((xml_uri->path != NULL && *xml_uri->path != '\0')
	   ? xml_uri->path : "/")
	: NULL;
    }
    inline const char *get_query() const
    { return xml_uri != NULL ? xml_uri->query : NULL; }
    inline const char *get_fragment() const
    { return xml_uri != NULL ? xml_uri->fragment : NULL; }


    // class function
    static char *encode_string(const char *src);
    static char *decode_string(const char *src);
  };


  class QueryKeyTuple
  {
  private:
    friend class QueryParser;
    const char *name;
    size_t name_len;
    const char *value;
    size_t value_len;


  public:
    char *get_name() const;
    char *get_value() const;

    inline const char *get_name_raw() const { return name; }
    inline size_t get_name_len() const { return name_len; }
    inline const char *get_value_raw() const { return value; }
    inline size_t get_value_len() const { return value_len; }
  };


  class QueryParser
  {
  private:
    const char *query_pos;
    QueryKeyTuple query_key;


  public:
    QueryParser(const char *query);
    QueryParser(const URI &uri);
    QueryParser(const URI *uri);

    inline bool has_more_keys() const
    { return query_pos != NULL && *query_pos != '\0'; };
    const QueryKeyTuple *next_key();
  };


  class BBSURLInfo
  {
  private:
    char *server;
    char *bbs_root;	// "/test/read.cgi"Ȥ"/cgi-bin/12ch/test/read.cgi"
			// "/""/cgi-bin/12ch/"
    char *bbs_dir;	// "/test/read.cgi"Ȥ"/bbs/read.pl""test/""bbs/"
    char *board_id;
    char *thread_id;
    int from;
    int to;

    void set_server(const char *value, size_t len);
    void set_bbs_root(const char *value, size_t len);
    void set_bbs_dir(const char *value, size_t len);
    void set_board_id(const char *value, size_t len);
    void set_thread_id(const char *value, size_t len);

    void parse(const URI &uri, int board_id_depth);


  public:
    BBSURLInfo(const char *url, int board_id_depth);
    BBSURLInfo(const URI &uri, int board_id_depth);
    ~BBSURLInfo();

    const char *get_server() const { return server; }
    const char *get_bbs_root() const { return bbs_root; }
    const char *get_bbs_dir() const { return bbs_dir; }
    const char *get_board_id() const { return board_id; }
    const char *get_thread_id() const { return thread_id; }
    int get_from() const { return from; }
    int get_to() const { return to; }
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBOCHUSHA_URI_UTILS_H_ */
