/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: arrow_action.c,v 1.5 2009/01/01 07:46:51 fuyu Exp $
 */

#include "config.h"

#include "libochushagtk/arrow_action.h"

#include <gtk/gtk.h>


static void ochusha_arrow_action_class_init(OchushaArrowActionClass *klass);
static void ochusha_arrow_action_init(OchushaArrowAction *action);
static void ochusha_arrow_action_finalize(GObject *object);

static GtkWidget *create_tool_item(GtkAction *action);


GType
ochusha_arrow_action_get_type(void)
{
  static GType arrow_action_type = 0;

  if (arrow_action_type == 0)
    {
      static const GTypeInfo arrow_action_info =
	{
	  sizeof(OchushaArrowActionClass),
	  NULL,	/* base_init */
	  NULL, /* base_finalize */
	  (GClassInitFunc)ochusha_arrow_action_class_init,
	  NULL,	/* class_finalize */
	  NULL,	/* class_data */
	  sizeof(OchushaArrowAction),
	  0,	/* n_preallocs */
	  (GInstanceInitFunc)ochusha_arrow_action_init,
	};

      arrow_action_type = g_type_register_static(GTK_TYPE_ACTION,
						 "OchushaArrowAction",
						 &arrow_action_info, 0);
    }

  return arrow_action_type;
}


static GObjectClass *parent_class = NULL;


static void
ochusha_arrow_action_class_init(OchushaArrowActionClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
  GtkActionClass *action_class = GTK_ACTION_CLASS(klass);

  parent_class = g_type_class_peek_parent(klass);

  /* as a GObject */
  gobject_class->finalize = ochusha_arrow_action_finalize;

  /* as a GtkAction */
  action_class->menu_item_type = GTK_TYPE_IMAGE_MENU_ITEM;
  action_class->toolbar_item_type = GTK_TYPE_TOOL_BUTTON;

  action_class->create_tool_item = create_tool_item;
}


static void
ochusha_arrow_action_init(OchushaArrowAction *action)
{
#if GTK_CHECK_VERSION(2,10,0)
  action->arrow_type = GTK_ARROW_NONE;
#else
  action->arrow_type = GTK_ARROW_RIGHT;
#endif
  action->shadow_type = GTK_SHADOW_NONE;

  action->tool_item_list = NULL;
}


static void
ochusha_arrow_action_finalize(GObject *object)
{
  OchushaArrowAction *action;
  g_return_if_fail(OCHUSHA_IS_ARROW_ACTION(object));

  action = OCHUSHA_ARROW_ACTION(object);

  g_return_if_fail(action->tool_item_list == NULL);

  if (G_OBJECT_CLASS(parent_class)->finalize != NULL)
    (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}


OchushaArrowAction *
ochusha_arrow_action_new(const gchar *name, const gchar *label,
			 const gchar *tooltip,
			 GtkArrowType arrow_type, GtkShadowType shadow_type)
{
  OchushaArrowAction *action = g_object_new(OCHUSHA_TYPE_ARROW_ACTION,
					    "name", name,
					    "label", label,
					    "tooltip", tooltip,
					    "stock_id", NULL,
					    NULL);

  action->arrow_type = arrow_type;
  action->shadow_type = shadow_type;

  return action;
}


static gboolean
arrow_set_helper(GtkToolButton *button, OchushaArrowAction *action)
{
  GtkWidget *widget = gtk_tool_button_get_icon_widget(button);
  g_return_val_if_fail(widget != NULL, FALSE);

  gtk_arrow_set(GTK_ARROW(widget), action->arrow_type, action->shadow_type);

  return FALSE;
}


void
ochusha_arrow_action_set(OchushaArrowAction *action,
			 GtkArrowType arrow_type, GtkShadowType shadow_type)
{
  g_return_if_fail(OCHUSHA_IS_ARROW_ACTION(action));

  if (action->arrow_type == arrow_type && action->shadow_type == shadow_type)
    return;

  action->arrow_type = arrow_type;
  action->shadow_type = shadow_type;

  g_list_foreach(action->tool_item_list, (GFunc)arrow_set_helper, action);
}


static GtkWidget *
create_tool_item(GtkAction *action)
{
  OchushaArrowAction *arrow_action = OCHUSHA_ARROW_ACTION(action);
  GtkWidget *arrow = gtk_arrow_new(arrow_action->arrow_type,
				   arrow_action->shadow_type);
  GtkWidget *tool_button;
  gtk_widget_show(arrow);

  tool_button = GTK_WIDGET(gtk_tool_button_new(arrow, NULL));
  gtk_widget_show(tool_button);

  return tool_button;
}

