/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: combo_action.h,v 1.6 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_COMBO_ACTION_H_
#define _LIBOCHUSHAGTK_COMBO_ACTION_H_

#include "libochushagtk/tree_menu_helper.h"

#include <gtk/gtk.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif


#define OCHUSHA_TYPE_COMBO_ACTION		(ochusha_combo_action_get_type())
#define OCHUSHA_COMBO_ACTION(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_COMBO_ACTION, OchushaComboAction))
#define OCHUSHA_COMBO_ACTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_COMBO_ACTION, OchushaComboActionClass))
#define OCHUSHA_IS_COMBO_ACTION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_COMBO_ACTION))
#define OCHUSHA_IS_COMBO_ACTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_COMBO_ACTION))
#define OCHUSHA_COMBO_ACTION_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_COMBO_ACTION, OchushaComboActionClass))


typedef struct _OchushaComboAction OchushaComboAction;
typedef struct _OchushaComboActionClass OchushaComboActionClass;

typedef void (*OchushaTooltipDataFunc)(GtkTooltip *tooltip,
				       GtkTreeModel *model,
				       GtkTreeIter *iter,
				       gpointer data);


struct _OchushaComboAction
{
  GtkAction parent;

  OchushaTreeMenuHelper *helper;

  GtkTreeModel *model;
  GtkTreeRowReference *active_row;

  GtkCellLayoutDataFunc layout_data_func;
  gpointer layout_data_func_data;
  GDestroyNotify layout_data_func_data_destroy;

  OchushaTooltipDataFunc tooltip_data_func;
  gpointer tooltip_data_func_data;
  GDestroyNotify tooltip_data_func_data_destroy;

  GList *combo_list;
};


struct _OchushaComboActionClass
{
  GtkActionClass parent_class;

  void (*changed)(OchushaComboAction *action, GtkTreeModel *model,
		  GtkTreeIter *iter, GtkTreePath *path);
};


GType ochusha_combo_action_get_type(void) G_GNUC_CONST;
OchushaComboAction *ochusha_combo_action_new(const gchar *name,
					     const gchar *label,
					     const gchar *tooltip,
					     const gchar *stock_id);
OchushaComboAction *ochusha_combo_action_new_with_model(const gchar *name,
							const gchar *label,
							const gchar *tooltip,
							const gchar *stock_id,
							GtkTreeModel *model);
GtkWidget *ochusha_combo_action_create_combo_box(OchushaComboAction *action);
void ochusha_combo_action_set_model(OchushaComboAction *action,
				    GtkTreeModel *model);
GtkTreeModel *ochusha_combo_action_get_model(OchushaComboAction *action);

void ochusha_combo_action_set_menu_data_func(OchushaComboAction *action,
					     OchushaTreeMenuDataFunc func,
					     gpointer func_data,
					     GDestroyNotify destroy);

void ochusha_combo_action_set_cell_data_func(OchushaComboAction *action,
					     GtkCellLayoutDataFunc func,
					     gpointer func_data,
					     GDestroyNotify destroy);

void ochusha_combo_action_set_tooltip_data_func(OchushaComboAction *action,
						OchushaTooltipDataFunc func,
						gpointer func_data,
						GDestroyNotify destroy);

gboolean ochusha_combo_action_get_active_iter(OchushaComboAction *action,
					      GtkTreeIter *iter);
void ochusha_combo_action_set_active_iter(OchushaComboAction *action,
					  GtkTreeIter *iter);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_COMBO_ACTION_H_ */
