/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: entry_action.h,v 1.5 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_ENTRY_ACTION_H_
#define _LIBOCHUSHAGTK_ENTRY_ACTION_H_

#include <gtk/gtk.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif


#define OCHUSHA_TYPE_ENTRY_ACTION		(ochusha_entry_action_get_type())
#define OCHUSHA_ENTRY_ACTION(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_ENTRY_ACTION, OchushaEntryAction))
#define OCHUSHA_ENTRY_ACTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_ENTRY_ACTION, OchushaEntryActionClass))
#define OCHUSHA_IS_ENTRY_ACTION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_ENTRY_ACTION))
#define OCHUSHA_IS_ENTRY_ACTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_ENTRY_ACTION))
#define OCHUSHA_ENTRY_ACTION_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_ENTRY_ACTION, OchushaEntryActionClass))


typedef struct _OchushaEntryAction OchushaEntryAction;
typedef struct _OchushaEntryActionClass OchushaEntryActionClass;


struct _OchushaEntryAction
{
  GtkAction parent;

  gint width_chars;

  GtkTreeModel *model;

  GList *widget_list;

  gboolean enable_completion;
};


struct _OchushaEntryActionClass
{
  GtkActionClass parent_class;

  void (*entry_activate)(OchushaEntryAction *action, const gchar *text);
};


GType ochusha_entry_action_get_type(void) G_GNUC_CONST;
OchushaEntryAction *ochusha_entry_action_new(const gchar *name,
					     const gchar *label,
					     const gchar *tooltip,
					     const gchar *stock_id);
gint ochusha_entry_action_get_default_width_chars(OchushaEntryAction *action);
void ochusha_entry_action_set_default_width_chars(OchushaEntryAction *action,
						  gint n_chars);
gboolean ochusha_entry_action_get_enable_completion(OchushaEntryAction *action);
void ochusha_entry_action_set_enable_completion(OchushaEntryAction *action,
						gboolean value);
void ochusha_entry_action_set_prefix(OchushaEntryAction *action,
				     const gchar *prefix);
void ochusha_entry_action_insert_completion_text(OchushaEntryAction *action,
						 gint index,
						 const gchar *text);

GtkWidget *ochusha_entry_action_create_entry_widget(OchushaEntryAction *action);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_ENTRY_ACTION_H_ */
