/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: hypertext_buffer.h,v 1.18 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_HYPERTEXT_BUFFER_H_
#define _LIBOCHUSHAGTK_HYPERTEXT_BUFFER_H_

#include <gtk/gtk.h>

#include "libochushagtk/hypertext_tag_table.h"

#ifdef __cplusplus
extern "C" {
#endif

#define OCHUSHA_TYPE_HYPERTEXT_BUFFER		(ochusha_hypertext_buffer_get_type())
#define OCHUSHA_HYPERTEXT_BUFFER(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_HYPERTEXT_BUFFER, OchushaHypertextBuffer))
#define OCHUSHA_HYPERTEXT_BUFFER_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_HYPERTEXT_BUFFER, OchushaHypertextBufferClass))
#define OCHUSHA_IS_HYPERTEXT_BUFFER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_HYPERTEXT_BUFFER))
#define OCHUSHA_IS_HYPERTEXT_BUFFER_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_HYPERTEXT_BUFFER))
#define OCHUSHA_HYPERTEXT_BUFFER_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_HYPERTEXT_BUFFER, OchushaHypertextBufferClass))


typedef struct _OchushaHypertextBuffer OchushaHypertextBuffer;
typedef struct _OchushaHypertextBufferClass OchushaHypertextBufferClass;
typedef struct _OchushaHypertextBufferPrivate OchushaHypertextBufferPrivate;


struct _OchushaHypertextBuffer
{
  GtkTextBuffer text_buffer;

  GtkTextIter iter;
  GSList *tags;

  GtkTextTag *link_tag;
  int link_start_offset;

#if !GLIB_CHECK_VERSION(2,10,0)
  GMemChunk *link_info_chunk;
#endif
  GTree *link_info;

#if !GLIB_CHECK_VERSION(2,10,0)
  GMemChunk *child_info_chunk;
#endif
  GTree *child_info;
};


struct _OchushaHypertextBufferClass
{
  GtkTextBufferClass parent_class;
};


GType ochusha_hypertext_buffer_get_type(void) G_GNUC_CONST;

OchushaHypertextTagTable *ochusha_hypertext_buffer_get_default_tag_table(void);

OchushaHypertextBuffer *ochusha_hypertext_buffer_new(
					OchushaHypertextTagTable *table);
void ochusha_hypertext_buffer_clear(OchushaHypertextBuffer *buffer);

gint ochusha_hypertext_buffer_get_link_offset(OchushaHypertextBuffer *buffer,
					      GtkTextIter *iter);
const char *ochusha_hypertext_buffer_get_link_at_offset(
						OchushaHypertextBuffer *buffer,
						gint offset);

void ochusha_hypertext_buffer_place_cursor_at_offset(
						OchushaHypertextBuffer *buffer,
						gint offset);

gint ochusha_hypertext_buffer_get_current_offset(
						OchushaHypertextBuffer *buffer);
GtkTextMark *ochusha_hypertext_buffer_create_mark(
						OchushaHypertextBuffer *buffer,
						const gchar *mark_name);
GtkTextMark *ochusha_hypertext_buffer_create_mark_at_offset(
						OchushaHypertextBuffer *buffer,
						const gchar *mark_name,
						gint offset);
void ochusha_hypertext_buffer_delete_mark(OchushaHypertextBuffer *buffer,
					  GtkTextMark *mark);
void ochusha_hypertext_buffer_append_text(OchushaHypertextBuffer *buffer,
					  const gchar *text, gint len);
void ochusha_hypertext_buffer_append_text_with_tag(OchushaHypertextBuffer *buffer,
						   const gchar *text, gint len,
						   GtkTextTag *tag);
gint ochusha_hypertext_buffer_insert_text_at_offset(OchushaHypertextBuffer *buffer,
						    const gchar *text, gint len);
gint ochusha_hypertext_buffer_remove_range(OchushaHypertextBuffer *buffer,
					   gint start_offset, gint end_offset);
gboolean ochusha_hypertext_buffer_is_at_start_of_line(OchushaHypertextBuffer *buffer);

GtkTextChildAnchor *ochusha_hypertext_buffer_append_child(
						OchushaHypertextBuffer *buffer,
						gpointer key);

GtkTextChildAnchor *ochusha_hypertext_buffer_insert_child_at_offset(
						OchushaHypertextBuffer *buffer,
						gpointer key, gint offset);

gint ochusha_hypertext_buffer_remove_child(OchushaHypertextBuffer *buffer,
					   gpointer key);

GtkTextTag *ochusha_hypertext_buffer_get_tag_by_name(
						OchushaHypertextBuffer *buffer,
						const gchar *name);

void ochusha_hypertext_buffer_push_tag_by_name(OchushaHypertextBuffer *buffer,
					       const gchar *name);
void ochusha_hypertext_buffer_push_tag(OchushaHypertextBuffer *buffer,
				       GtkTextTag *tag);
GtkTextTag *ochusha_hypertext_buffer_pop_tag(OchushaHypertextBuffer *buffer);
void ochusha_hypertext_buffer_pop_tags(OchushaHypertextBuffer *buffer,
				       GtkTextTag *tag);

void ochusha_hypertext_buffer_push_link_tag(OchushaHypertextBuffer *buffer,
					    GtkTextTag *tag, const gchar *link);
void ochusha_hypertext_buffer_pop_link_tag(OchushaHypertextBuffer *buffer,
					   GtkTextTag *tag);

#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_HYPERTEXT_BUFFER_H_ */
