/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: hypertext_tag_table.c,v 1.9 2009/01/01 07:46:51 fuyu Exp $
 */

#include "config.h"

#include "libochushagtk/hypertext_tag_table.h"


static void ochusha_hypertext_tag_table_class_init(OchushaHypertextTagTableClass *klass);
static void ochusha_hypertext_tag_table_init(OchushaHypertextTagTable *table);
static void ochusha_hypertext_tag_table_finalize(GObject *object);


static GtkTextTagTableClass *parent_class = NULL;


GType
ochusha_hypertext_tag_table_get_type(void)
{
  static GType hypertext_tag_table_type = 0;

  if (hypertext_tag_table_type == 0)
    {
      static const GTypeInfo hypertext_tag_table_info =
	{
	  sizeof(OchushaHypertextTagTableClass),
	  NULL, /* base_init */
	  NULL, /* base_finalize */
	  (GClassInitFunc)ochusha_hypertext_tag_table_class_init,
	  NULL, /* class_finalize */
	  NULL, /* class_data */
	  sizeof(OchushaHypertextTagTable),
	  0,	/* n_preallocs */
	  (GInstanceInitFunc)ochusha_hypertext_tag_table_init,
	};

      hypertext_tag_table_type
	= g_type_register_static(GTK_TYPE_TEXT_TAG_TABLE,
				 "OchushaHypertextTagTable",
				 &hypertext_tag_table_info, 0);
    }
  
  return hypertext_tag_table_type;
}


static void
ochusha_hypertext_tag_table_class_init(OchushaHypertextTagTableClass *klass)
{
  GObjectClass *o_class = (GObjectClass *)klass;

  parent_class = g_type_class_peek_parent(klass);

  /* GObject signals */
  o_class->finalize = ochusha_hypertext_tag_table_finalize;
}


static void
ochusha_hypertext_tag_table_init(OchushaHypertextTagTable *table)
{
  table->link_tag_list = NULL;
}


static void
ochusha_hypertext_tag_table_finalize(GObject *object)
{
  OchushaHypertextTagTable *table = OCHUSHA_HYPERTEXT_TAG_TABLE(object);

  if (table->link_tag_list != NULL)
    {
      g_slist_free(table->link_tag_list);
      table->link_tag_list = NULL;
    }

  if (G_OBJECT_CLASS(parent_class)->finalize != NULL)
    (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}


OchushaHypertextTagTable *
ochusha_hypertext_tag_table_new(void)
{
  return g_object_new(OCHUSHA_TYPE_HYPERTEXT_TAG_TABLE, NULL);
}


void
ochusha_hypertext_tag_table_add_link_tag(OchushaHypertextTagTable *table,
					 GtkTextTag *tag)
{
  g_return_if_fail(OCHUSHA_IS_HYPERTEXT_TAG_TABLE(table));
  g_return_if_fail(GTK_IS_TEXT_TAG(tag));

  gtk_text_tag_table_add(&table->tag_table, tag);
  if (g_slist_find(table->link_tag_list, tag) == NULL)
    table->link_tag_list = g_slist_append(table->link_tag_list, tag);
}


void
ochusha_hypertext_tag_table_remove_tag(OchushaHypertextTagTable *table,
				       GtkTextTag *tag)
{
  g_return_if_fail(OCHUSHA_IS_HYPERTEXT_TAG_TABLE(table));
  g_return_if_fail(GTK_IS_TEXT_TAG(tag));

  gtk_text_tag_table_remove(&table->tag_table, tag);
  table->link_tag_list = g_slist_remove(table->link_tag_list, tag);
}


void
ochusha_hypertext_tag_table_foreach_link_tag(OchushaHypertextTagTable *table,
					     GtkTextTagTableForeach func,
					     gpointer data)
{
  g_return_if_fail(OCHUSHA_IS_HYPERTEXT_TAG_TABLE(table));
  g_return_if_fail(func != NULL);

  g_slist_foreach(table->link_tag_list, (GFunc)func, data);
}


GtkTextTag *
ochusha_hypertext_tag_table_find_link_tag_at_iter(OchushaHypertextTagTable *table,
						  GtkTextIter *iter)
{
  GSList *link_tag_list;

  g_return_val_if_fail(OCHUSHA_IS_HYPERTEXT_TAG_TABLE(table), NULL);
  g_return_val_if_fail(iter != NULL, NULL);

  link_tag_list = table->link_tag_list;

  while (link_tag_list != NULL)
    {
      GtkTextTag *tag = link_tag_list->data;

      if (gtk_text_iter_has_tag(iter, tag))
	return tag;

      link_tag_list = link_tag_list->next;
    }

  return NULL;
}
