/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: hypertext_view.h,v 1.19 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_HYPERTEXT_VIEW_H_
#define _LIBOCHUSHAGTK_HYPERTEXT_VIEW_H_

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/searchable.h"

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif


#define OCHUSHA_TYPE_HYPERTEXT_VIEW		(ochusha_hypertext_view_get_type())
#define OCHUSHA_HYPERTEXT_VIEW(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_HYPERTEXT_VIEW, OchushaHypertextView))
#define OCHUSHA_HYPERTEXT_VIEW_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_HYPERTEXT_VIEW, OchushaHypertextViewClass))
#define OCHUSHA_IS_HYPERTEXT_VIEW(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_HYPERTEXT_VIEW))
#define OCHUSHA_IS_HYPERTEXT_VIEW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_HYPERTEXT_VIEW))
#define OCHUSHA_HYPERTEXT_VIEW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_HYPERTEXT_VIEW, OchushaHypertextViewClass))


typedef struct _OchushaHypertextView OchushaHypertextView;
typedef struct _OchushaHypertextViewClass OchushaHypertextViewClass;
typedef struct _OchushaHypertextViewPrivate OchushaHypertextViewPrivate;


struct _OchushaHypertextView
{
  GtkTextView text_view;

  GdkCursor *ibeam_cursor;
  GdkCursor *hand_cursor;

  /* ʥطǻȤ */
  gint last_event_offset;


  /* طǻȤ */
  gint last_match_offset;
  gboolean search_wrapped;

  OchushaSearchKey *last_key;
  guint last_key_id;
};


struct _OchushaHypertextViewClass
{
  GtkTextViewClass parent_class;

  void (*link_mouse_over)(OchushaHypertextView *view,
			  GdkEventMotion *event, const gchar *link);
  void (*link_mouse_out)(OchushaHypertextView *view, GdkEventAny *event);
  gboolean (*link_mouse_press)(OchushaHypertextView *view,
			       GdkEventButton *event, const gchar *link);
  gboolean (*link_mouse_release)(OchushaHypertextView *view,
				 GdkEventButton *event, const gchar *link);

  gboolean (*start_interactive_search)(OchushaHypertextView *view,
				       OchushaSearchActionType action);
};


GType ochusha_hypertext_view_get_type(void) G_GNUC_CONST;

GtkWidget *ochusha_hypertext_view_new(OchushaHypertextBuffer *buffer);

gint ochusha_hypertext_view_append_widget(OchushaHypertextView *view,
					  GtkWidget *widget,
					  gint hborder, gint vborder);

gint ochusha_hypertext_view_insert_widget_at_offset(OchushaHypertextView *view,
						    GtkWidget *widget,
						    gint hborder, gint vborder,
						    gint offset);
gint ochusha_hypertext_view_remove_widget(OchushaHypertextView *view,
					  GtkWidget *widget);

void ochusha_hypertext_view_scroll_to_start(OchushaHypertextView *view);
void ochusha_hypertext_view_scroll_to_end(OchushaHypertextView *view);
void ochusha_hypertext_view_scroll_to_mark(OchushaHypertextView *view,
					   GtkTextMark *mark);
gint ochusha_hypertext_view_scroll_to_offset(OchushaHypertextView *view,
					     gint offset);

gint ochusha_hypertext_view_get_visible_offset(OchushaHypertextView *view);
void ochusha_hypertext_view_make_offset_visible(OchushaHypertextView *view,
						gint offset);

gchar *ochusha_hypertext_view_get_selected_text(OchushaHypertextView *view);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_HYPERTEXT_VIEW_H_ */
