/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: icon_label.h,v 1.13 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_ICON_LABEL_H_
#define _LIBOCHUSHAGTK_ICON_LABEL_H_

#include <gtk/gtk.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif


#define OCHUSHA_TYPE_ICON_LABEL			(ochusha_icon_label_get_type())
#define OCHUSHA_ICON_LABEL(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_ICON_LABEL, OchushaIconLabel))
#define OCHUSHA_ICON_LABEL_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_ICON_LABEL, OchushaIconLabelClass))
#define OCHUSHA_IS_ICON_LABEL(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_ICON_LABEL))
#define OCHUSHA_IS_ICON_LABEL_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_ICON_LABEL))
#define OCHUSHA_ICON_LABEL_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_ICON_LABEL, OchushaIconLabelClass))


typedef struct _OchushaIconLabel OchushaIconLabel;
typedef struct _OchushaIconLabelClass OchushaIconLabelClass;

struct _OchushaIconLabel
{
  GtkLabel label;

  GtkAllocation label_allocation;

  GdkPixbuf *pixbuf;
  GdkPixbufAnimation *animation;
  GdkPixbufAnimationIter *animation_iter;
  gchar *stock_id;

  GdkPixbuf *current_pixbuf;

  guint frame_timeout_id;
};


struct _OchushaIconLabelClass
{
  GtkLabelClass parent_class;
};


GType ochusha_icon_label_get_type(void) G_GNUC_CONST;

GtkWidget *ochusha_icon_label_new(const gchar *label_text);
GtkWidget *ochusha_icon_label_new_from_stock(const gchar *label_text,
					     const gchar *stock_id);
GtkWidget *ochusha_icon_label_new_with_mnemonic(const gchar *mnemonic);
GtkWidget *ochusha_icon_label_new_with_mnemonic_from_stock(const gchar *mnemonic,
							   const gchar *stock_id);

void ochusha_icon_label_set_icon_from_pixbuf(OchushaIconLabel *icon_label,
					     GdkPixbuf *pixbuf);
void ochusha_icon_label_set_icon_from_animation(OchushaIconLabel *icon_label,
						GdkPixbufAnimation *animation);
void ochusha_icon_label_set_icon_from_stock(OchushaIconLabel *icon_label,
					    const gchar *stock_id);
GdkPixbuf *ochusha_icon_label_get_icon_pixbuf(OchushaIconLabel *icon_label);
GdkPixbufAnimation *ochusha_icon_label_get_icon_animation(OchushaIconLabel *icon_label);
const gchar *ochusha_icon_label_get_icon_stock(OchushaIconLabel *icon_label);

GdkPixbuf *ochusha_icon_label_get_current_pixbuf(OchushaIconLabel *icon_label);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_ICON_LABEL_H_ */
