/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: menu_action.h,v 1.5 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_MENU_ACTION_H_
#define _LIBOCHUSHAGTK_MENU_ACTION_H_

#include "libochushagtk/tree_menu_helper.h"

#include <gtk/gtk.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif


#define OCHUSHA_TYPE_MENU_ACTION		(ochusha_menu_action_get_type())
#define OCHUSHA_MENU_ACTION(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_MENU_ACTION, OchushaMenuAction))
#define OCHUSHA_MENU_ACTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_MENU_ACTION, OchushaMenuActionClass))
#define OCHUSHA_IS_MENU_ACTION(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_MENU_ACTION))
#define OCHUSHA_IS_MENU_ACTION_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_MENU_ACTION))
#define OCHUSHA_MENU_ACTION_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_MENU_ACTION, OchushaMenuActionClass))


typedef struct _OchushaMenuAction OchushaMenuAction;
typedef struct _OchushaMenuActionClass OchushaMenuActionClass;


struct _OchushaMenuAction
{
  GtkAction parent;

  OchushaTreeMenuHelper *helper;
};


struct _OchushaMenuActionClass
{
  GtkActionClass parent_class;

  void (*menu_activate)(OchushaMenuAction *action, GtkTreeModel *model,
			GtkTreeIter *iter, GtkTreePath *path);
};


GType ochusha_menu_action_get_type(void) G_GNUC_CONST;
OchushaMenuAction *ochusha_menu_action_new(const gchar *name,
					   const gchar *label,
					   const gchar *tooltip,
					   const gchar *stock_id);
OchushaMenuAction *ochusha_menu_action_new_with_model(const gchar *name,
						      const gchar *label,
						      const gchar *tooltip,
						      const gchar *stock_id,
						      GtkTreeModel *model);
void ochusha_menu_action_set_model(OchushaMenuAction *action,
				   GtkTreeModel *model);
GtkTreeModel *ochusha_menu_action_get_model(OchushaMenuAction *action);

void ochusha_menu_action_set_menu_data_func(OchushaMenuAction *action,
					    OchushaTreeMenuDataFunc func,
					    gpointer func_data,
					    GDestroyNotify destroy);

void ochusha_menu_action_set_row_separator_func(OchushaMenuAction *action,
						GtkTreeViewRowSeparatorFunc func,
						gpointer func_data,
						GDestroyNotify destroy);

void ochusha_menu_action_set_row_sensitive_func(OchushaMenuAction *action,
						OchushaTreeMenuRowSensitiveFunc func,
						gpointer func_data,
						GDestroyNotify destroy);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_MENU_ACTION_H_ */
