/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: response_editor.c,v 1.18 2009/01/01 07:46:51 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"
#include "libochushagtk/response_editor.h"

#include "libochushagtk/marshal.h"
#include "libochushagtk/stock.h"

#include <gdk/gdkkeysyms.h>
#include <glib.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


static void ochusha_response_editor_class_init(OchushaResponseEditorClass *klass);
static void ochusha_response_editor_init(OchushaResponseEditor *editor);
static void ochusha_response_editor_finalize(GObject *object);
static void ochusha_response_editor_destroy(GtkObject *object);


GType
ochusha_response_editor_get_type(void)
{
  static GType response_editor_type = 0;

  if (response_editor_type == 0)
    {
      static const GTypeInfo response_editor_info =
	{
	  sizeof(OchushaResponseEditorClass),
	  NULL,	/* base_init */
	  NULL,	/* base_finalize */
	  (GClassInitFunc)ochusha_response_editor_class_init,
	  NULL,	/* class_finalize */
	  NULL,	/* class_data */
	  sizeof(OchushaResponseEditor),
	  0,	/* n_preallocs */
	  (GInstanceInitFunc)ochusha_response_editor_init,
	};

      response_editor_type = g_type_register_static(GTK_TYPE_VBOX,
						    "OchushaResponseEditor",
						    &response_editor_info, 0);
    }

  return response_editor_type;
}


enum {
  NOP_SIGNAL,
  POST_SIGNAL,
  CLEAR_SIGNAL,
  CLOSE_SIGNAL,
  LAST_SIGNAL
};


static GtkVBoxClass *parent_class;
static int response_editor_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };


static void
ochusha_response_editor_class_init(OchushaResponseEditorClass *klass)
{
  GObjectClass *o_class = (GObjectClass *)klass;
  GtkObjectClass *object_class = (GtkObjectClass *)klass;
  GtkBindingSet *binding_set;

  parent_class = g_type_class_peek_parent(klass);
  binding_set = gtk_binding_set_by_class(klass);

  /* GObject signals */
  o_class->finalize = ochusha_response_editor_finalize;

  /* GtkObject signals */
  object_class->destroy = ochusha_response_editor_destroy;

  response_editor_signals[NOP_SIGNAL] =
    g_signal_new("nop",
		 G_TYPE_FROM_CLASS(klass),
		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
		 0,
		 NULL, NULL,
		 libochushagtk_marshal_VOID__VOID,
		 G_TYPE_NONE, 0);

  response_editor_signals[POST_SIGNAL] =
    g_signal_new("post",
		 G_TYPE_FROM_CLASS(klass),
		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
		 0,
		 NULL, NULL,
		 libochushagtk_marshal_VOID__VOID,
		 G_TYPE_NONE, 0);

  response_editor_signals[CLEAR_SIGNAL] =
    g_signal_new("clear",
		 G_TYPE_FROM_CLASS(klass),
		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
		 0,
		 NULL, NULL,
		 libochushagtk_marshal_VOID__VOID,
		 G_TYPE_NONE, 0);

  response_editor_signals[CLOSE_SIGNAL] =
    g_signal_new("close",
		 G_TYPE_FROM_CLASS(klass),
		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
		 0,
		 NULL, NULL,
		 libochushagtk_marshal_VOID__VOID,
		 G_TYPE_NONE, 0);

  gtk_binding_entry_add_signal(binding_set, GDK_Escape, 0, "nop", 0);
}


static void
ochusha_response_editor_insert_aa_button_cb(gpointer unused,
					    OchushaResponseEditor *editor)
{
  gchar *aa = ochusha_response_editor_get_current_aa(editor);

  if (aa == NULL)
    return;

  ochusha_response_editor_insert_response(editor, aa);
  g_free(aa);

  return;
}


static void
ochusha_response_editor_prepare_aa_support(OchushaResponseEditor *editor,
					   gboolean enable)
{
  if (enable)
    gtk_widget_show(GTK_WIDGET(editor->aa_hbox));
  else
    gtk_widget_hide(GTK_WIDGET(editor->aa_hbox));

  editor->aa_support = enable;
}


static void
show_aa_helper_toggled_cb(GtkToggleButton *button,
			  OchushaResponseEditor *editor)
{
  gboolean enabled = gtk_toggle_button_get_active(button);
  const gchar *stock_id;
  if (enabled)
    stock_id = OCHUSHA_STOCK_AA_HELPER_ENABLED;
  else
    stock_id = OCHUSHA_STOCK_AA_HELPER_DISABLED;

  gtk_button_set_image(GTK_BUTTON(button),
		       gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_MENU));
  ochusha_response_editor_prepare_aa_support(editor, enabled);
}


static void
ochusha_response_editor_init(OchushaResponseEditor *editor)
{
  GtkWidget *title_hbox;
  GtkWidget *title_label;
  GtkWidget *title_entry;

  GtkWidget *name_mail_hbox;
  GtkWidget *name_label;
  GtkWidget *name_combo;
  GtkWidget *mail_label;
  GtkWidget *mail_combo;
  GtkWidget *show_aa_helper;

  GtkWidget *scrolled_window;
  GtkWidget *text_view;

  GtkWidget *aa_hbox;
  GtkWidget *aa_combo;
  GtkWidget *aa_button;

  /* title_hboxν */
  title_hbox = gtk_hbox_new(FALSE, 5);
  editor->title_hbox = GTK_BOX(title_hbox);
  gtk_widget_show(title_hbox);
  gtk_box_pack_start(GTK_BOX(&editor->container), title_hbox, FALSE, FALSE, 0);

  title_label = gtk_label_new(LIB_("Title:"));
  gtk_widget_show(title_label);
  gtk_box_pack_start(editor->title_hbox, title_label, FALSE, FALSE, 0);

  title_entry = gtk_entry_new();
  editor->title_entry = GTK_ENTRY(title_entry);
  gtk_widget_show(title_entry);
  gtk_box_pack_start(editor->title_hbox, title_entry, TRUE, TRUE, 0);


  /* name_mail_hboxν */
  name_mail_hbox = gtk_hbox_new(FALSE, 5);
  editor->name_mail_hbox = GTK_BOX(name_mail_hbox);
  gtk_widget_show(name_mail_hbox);
  gtk_box_pack_start(GTK_BOX(&editor->container),
		     name_mail_hbox, FALSE, FALSE, 0);

  name_label = gtk_label_new(LIB_("Name:"));
  gtk_widget_show(name_label);
  gtk_box_pack_start(editor->name_mail_hbox, name_label, FALSE, FALSE, 0);

  name_combo = gtk_combo_box_entry_new_text();
  editor->name_combo = GTK_COMBO_BOX(name_combo);
  gtk_combo_box_set_focus_on_click(editor->name_combo, FALSE);

  gtk_widget_show(name_combo);
  gtk_box_pack_start(editor->name_mail_hbox, name_combo, TRUE, TRUE, 0);


  mail_label = gtk_label_new(LIB_("E-mail:"));
  gtk_widget_show(mail_label);
  gtk_box_pack_start(editor->name_mail_hbox, mail_label, FALSE, FALSE, 0);

  mail_combo = gtk_combo_box_entry_new_text();
  editor->mail_combo = GTK_COMBO_BOX(mail_combo);
  gtk_combo_box_set_focus_on_click(editor->mail_combo, FALSE);

  gtk_widget_show(mail_combo);
  gtk_box_pack_start(editor->name_mail_hbox, mail_combo, TRUE, TRUE, 0);

  show_aa_helper = gtk_toggle_button_new();
  editor->show_aa_helper = GTK_TOGGLE_BUTTON(show_aa_helper);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_aa_helper), TRUE);
  gtk_button_set_image(GTK_BUTTON(show_aa_helper),
		       gtk_image_new_from_stock(OCHUSHA_STOCK_AA_HELPER_ENABLED,
						GTK_ICON_SIZE_MENU));
  g_signal_connect(show_aa_helper, "toggled",
		   G_CALLBACK(show_aa_helper_toggled_cb), editor);
#if GTK_CHECK_VERSION(2,12,0)
  gtk_widget_set_tooltip_text(show_aa_helper, LIB_("AA Input Helper"));
#endif
  gtk_widget_show(show_aa_helper);
  gtk_box_pack_end(editor->name_mail_hbox, show_aa_helper, FALSE, FALSE, 0);


  /* GtkTextEditorν */
  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_show(scrolled_window);
  gtk_box_pack_start(GTK_BOX(&editor->container),
		     scrolled_window, TRUE, TRUE, 0);

  text_view = gtk_text_view_new();
  gtk_widget_set_name(text_view, "responseeditor");
  editor->text_view = GTK_TEXT_VIEW(text_view);
  gtk_text_view_set_wrap_mode(editor->text_view, GTK_WRAP_CHAR);
  gtk_widget_show(text_view);
  gtk_container_add(GTK_CONTAINER(scrolled_window), text_view);

  aa_hbox = gtk_hbox_new(FALSE, 5);
  editor->aa_hbox = GTK_BOX(aa_hbox);
  gtk_widget_show(aa_hbox);
  gtk_box_pack_start(GTK_BOX(&editor->container), aa_hbox, FALSE, FALSE, 0);

  aa_combo = gtk_combo_box_new_text();
  editor->aa_combo = GTK_COMBO_BOX(aa_combo);
  gtk_combo_box_set_focus_on_click(editor->aa_combo, FALSE);
  gtk_widget_show(aa_combo);
  gtk_box_pack_start(GTK_BOX(aa_hbox), aa_combo, TRUE, TRUE, 0);

  aa_button = gtk_button_new_with_label(LIB_("Insert"));
  g_signal_connect(aa_button, "clicked",
		   G_CALLBACK(ochusha_response_editor_insert_aa_button_cb),
		   editor);
  gtk_button_set_focus_on_click(GTK_BUTTON(aa_button), FALSE);
#if GTK_CHECK_VERSION(2,12,0)
  gtk_widget_set_tooltip_text(aa_button, LIB_("Insert this AA"));
#endif
  gtk_widget_show(aa_button);

  editor->number_of_aa = 0;

  gtk_box_pack_start(GTK_BOX(aa_hbox), aa_button, FALSE, FALSE, 0);
}


static void
ochusha_response_editor_finalize(GObject *object)
{
  if (G_OBJECT_CLASS(parent_class)->finalize != NULL)
    (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}


static void
ochusha_response_editor_destroy(GtkObject *object)
{
  OchushaResponseEditor *editor = OCHUSHA_RESPONSE_EDITOR(object);

  if (editor->name_list != NULL)
    {
      g_list_foreach(editor->name_list, (GFunc)g_free, NULL);
      g_list_free(editor->name_list);
      editor->name_list = NULL;
    }

  if (editor->mail_list != NULL)
    {
      g_list_foreach(editor->mail_list, (GFunc)g_free, NULL);
      g_list_free(editor->mail_list);
      editor->mail_list = NULL;
    }

  if (GTK_OBJECT_CLASS(parent_class)->destroy != NULL)
    (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}


GtkWidget *
ochusha_response_editor_new(void)
{
  return g_object_new(OCHUSHA_TYPE_RESPONSE_EDITOR,
		      NULL);
}


void
ochusha_response_editor_show_title_entry(OchushaResponseEditor *editor)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  gtk_widget_show(GTK_WIDGET(editor->title_hbox));
}


void
ochusha_response_editor_hide_title_entry(OchushaResponseEditor *editor)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  gtk_widget_hide(GTK_WIDGET(editor->title_hbox));
}


const gchar *
ochusha_response_editor_get_title(OchushaResponseEditor *editor)
{
  g_return_val_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor), NULL);

  return gtk_entry_get_text(editor->title_entry);
}


void
ochusha_response_editor_set_title(OchushaResponseEditor *editor,
				  const gchar *title)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  gtk_entry_set_text(editor->title_entry, title);
}


const gchar *
ochusha_response_editor_get_name(OchushaResponseEditor *editor)
{
  g_return_val_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor), NULL);

#if GTK_CHECK_VERSION(2,8,0)
  return gtk_combo_box_get_active_text(editor->name_combo);
#else
  return gtk_entry_get_text(GTK_ENTRY(GTK_BIN(editor->name_combo)->child));
#endif
}


void
ochusha_response_editor_set_name(OchushaResponseEditor *editor,
				 const gchar *name)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  gtk_entry_set_text(GTK_ENTRY(GTK_BIN(editor->name_combo)->child), name);
}


void
ochusha_response_editor_add_name(OchushaResponseEditor *editor,
				 const gchar *name)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  if (name == NULL)
    return;

  if (g_list_find_custom(editor->name_list, name, (GCompareFunc)strcmp)
      != NULL)
    return;

  editor->name_list = g_list_append(editor->name_list, g_strdup(name));

  gtk_combo_box_append_text(editor->name_combo, name);
}


const gchar *
ochusha_response_editor_get_mail(OchushaResponseEditor *editor)
{
  g_return_val_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor), NULL);

#if GTK_CHECK_VERSION(2,8,0)
  return gtk_combo_box_get_active_text(editor->mail_combo);
#else
  return gtk_entry_get_text(GTK_ENTRY(GTK_BIN(editor->mail_combo)->child));
#endif
}


void
ochusha_response_editor_set_mail(OchushaResponseEditor *editor,
				 const gchar *mail)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  gtk_entry_set_text(GTK_ENTRY(GTK_BIN(editor->mail_combo)->child), mail);
  return;
}


void
ochusha_response_editor_add_mail(OchushaResponseEditor *editor,
				 const gchar *mail)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  if (mail == NULL)
    return;

  if (g_list_find_custom(editor->mail_list, mail, (GCompareFunc)strcmp)
      != NULL)
    return;

  editor->mail_list = g_list_append(editor->mail_list, g_strdup(mail));

  gtk_combo_box_append_text(editor->mail_combo, mail);
}


void
ochusha_response_editor_clear_response(OchushaResponseEditor *editor)
{
  GtkTextIter start_iter;
  GtkTextIter end_iter;
  GtkTextBuffer *buffer;

  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  buffer = gtk_text_view_get_buffer(editor->text_view);
  gtk_text_buffer_get_start_iter(buffer, &start_iter);
  gtk_text_buffer_get_end_iter(buffer, &end_iter);
  gtk_text_buffer_delete(buffer, &start_iter, &end_iter);
}


gchar *
ochusha_response_editor_get_response(OchushaResponseEditor *editor)
{
  GtkTextIter start_iter;
  GtkTextIter end_iter;
  GtkTextBuffer *buffer;

  g_return_val_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor), NULL);

  buffer = gtk_text_view_get_buffer(editor->text_view);
  gtk_text_buffer_get_start_iter(buffer, &start_iter);
  gtk_text_buffer_get_end_iter(buffer, &end_iter);
  return gtk_text_buffer_get_text(buffer, &start_iter, &end_iter, FALSE);
}


void
ochusha_response_editor_append_response(OchushaResponseEditor *editor,
					const gchar *response)
{
  GtkTextIter end_iter;
  GtkTextBuffer *buffer;

  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));
  g_return_if_fail(response != NULL);

  buffer = gtk_text_view_get_buffer(editor->text_view);
  gtk_text_buffer_get_end_iter(buffer, &end_iter);
  if (gtk_text_buffer_get_char_count(buffer) > 0)
    gtk_text_buffer_insert(buffer, &end_iter, "\n", 1);
  gtk_text_buffer_insert(buffer, &end_iter, response, -1);
}


void
ochusha_response_editor_insert_response(OchushaResponseEditor *editor,
					const gchar *response)
{
  GtkTextBuffer *buffer;

  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));
  g_return_if_fail(response != NULL);

  buffer = gtk_text_view_get_buffer(editor->text_view);
  gtk_text_buffer_insert_interactive_at_cursor(buffer, response, -1, TRUE);
}


void
ochusha_response_editor_set_response(OchushaResponseEditor *editor,
				     const gchar *response)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));
  ochusha_response_editor_clear_response(editor);
  ochusha_response_editor_append_response(editor, response);
}


void
ochusha_response_editor_set_aa_support(OchushaResponseEditor *editor,
				       gboolean enable)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  if (editor->aa_support == enable)
    return;

  gtk_toggle_button_set_active(editor->show_aa_helper, enable);
}


gboolean
ochusha_response_editor_get_aa_support(OchushaResponseEditor *editor)
{
  g_return_val_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor), FALSE);

  return editor->aa_support;
}


void
ochusha_response_editor_reset_aa_list(OchushaResponseEditor *editor)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  while (editor->number_of_aa > 0)
    gtk_combo_box_remove_text(editor->aa_combo, --editor->number_of_aa);
}


void
ochusha_response_editor_add_aa(OchushaResponseEditor *editor, const gchar *aa)
{
  g_return_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor));

  if (aa == NULL)
    return;

  gtk_combo_box_append_text(editor->aa_combo, aa);
  editor->number_of_aa++;
}


gchar *
ochusha_response_editor_get_current_aa(OchushaResponseEditor *editor)
{
  g_return_val_if_fail(OCHUSHA_IS_RESPONSE_EDITOR(editor), NULL);
  return gtk_combo_box_get_active_text(editor->aa_combo);
}
