/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: search_window.h,v 1.9 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_SEARCH_WINDOW_H_
#define _LIBOCHUSHAGTK_SEARCH_WINDOW_H_

#include "libochushagtk/searchable.h"

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

#define OCHUSHA_TYPE_SEARCH_WINDOW		(ochusha_search_window_get_type())
#define OCHUSHA_SEARCH_WINDOW(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_SEARCH_WINDOW, OchushaSearchWindow))
#define OCHUSHA_SEARCH_WINDOW_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_SEARCH_WINDOW, OchushaSearchWindowClass))
#define OCHUSHA_IS_SEARCH_WINDOW(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_SEARCH_WINDOW))
#define OCHUSHA_IS_SEARCH_WINDOW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_SEARCH_WINDOW))
#define OCHUSHA_SEARCH_WINDOW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_SEARCH_WINDOW, OchushaSearchWindowClass))


typedef struct _OchushaSearchWindow OchushaSearchWindow;
typedef struct _OchushaSearchWindowClass OchushaSearchWindowClass;


struct _OchushaSearchWindow
{
  GtkWindow parent_object;

  /* main_box */
  GtkEntry *entry;

  /* search_option_box */
  GtkToggleButton *match_case_button;
  GtkToggleButton *use_regexp_button;

  /* direction_box */
  GtkToggleButton *direction_up_button;
  GtkToggleButton *direction_down_button;

  GtkStatusbar *statusbar;
  guint statusbar_context_id;
  guint statusbar_message_id;


  OchushaSearchKey *key;
  gulong key_changed_id;

  OchushaSearchable *last_search_target;
  gulong last_search_target_destroy_id;

  guint enable_incremental_search	: 1;
};


struct _OchushaSearchWindowClass
{
  GtkWindowClass parent_class;

  void (*response)(OchushaSearchWindow *window, gint response_id);
  OchushaSearchable *(*search_request)(OchushaSearchWindow *window);
  void (*find_previous)(OchushaSearchWindow *bar);
  void (*find_next)(OchushaSearchWindow *bar);
};


GType ochusha_search_window_get_type(void) G_GNUC_CONST;

GtkWidget *ochusha_search_window_new(void);
GtkWidget *ochusha_search_window_new_with_key(OchushaSearchKey *key);

void ochusha_search_window_set_key(OchushaSearchWindow *window,
				   OchushaSearchKey *key);
OchushaSearchKey *ochusha_search_window_get_key(OchushaSearchWindow *window);

void ochusha_search_window_set_enable_incremental_search(
						OchushaSearchWindow *window,
						gboolean enable);
gboolean ochusha_search_window_get_enable_incremental_search(
						OchushaSearchWindow *window);

void ochusha_search_window_response(OchushaSearchWindow *window,
				    gint response_id);
void ochusha_search_window_set_key_selected(OchushaSearchWindow *window);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_SEARCH_WINDOW_H_ */
