/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: searchable.h,v 1.10 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_SEARCHABLE_H_
#define _LIBOCHUSHAGTK_SEARCHABLE_H_

#include <glib-object.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
  OCHUSHA_SEARCH_DIRECTION_FORWARD = 0,
  OCHUSHA_SEARCH_DIRECTION_BACKWARD = 1,
} OchushaSearchDirectionType;


#define OCHUSHA_TYPE_SEARCH_DIRECTION	(ochusha_search_direction_get_type())
GType ochusha_search_direction_get_type(void) G_GNUC_CONST;


typedef enum {
  OCHUSHA_SEARCH_ACTION_FORWARD,
  OCHUSHA_SEARCH_ACTION_BACKWARD,
  OCHUSHA_SEARCH_ACTION_REGEXP_FORWARD,
  OCHUSHA_SEARCH_ACTION_REGEXP_BACKWARD,
  OCHUSHA_SEARCH_ACTION_TERMINATE
} OchushaSearchActionType;


#define OCHUSHA_TYPE_SEARCH_ACTION	(ochusha_search_action_get_type())
GType ochusha_search_action_get_type(void) G_GNUC_CONST;



#define OCHUSHA_TYPE_SEARCH_KEY			(ochusha_search_key_get_type())
#define OCHUSHA_SEARCH_KEY(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_SEARCH_KEY, OchushaSearchKey))
#define OCHUSHA_SEARCH_KEY_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_SEARCH_KEY, OchushaSearchKeyClass))
#define OCHUSHA_IS_SEARCH_KEY(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_SEARCH_KEY))
#define OCHUSHA_IS_SEARCH_KEY_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_SEARCH_KEY))
#define OCHUSHA_SEARCH_KEY_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_SEARCH_KEY, OchushaSearchKeyClass))


/*
 * 虜虜GObjectʥץƥȤäƤ̣Ϥʤ⤷줺
 *
 * ¤Τsearchable_private.h⡣
 * OchushaSearchable󥿥ե륯饹μΤߤ˸
 */
typedef struct _OchushaSearchKey OchushaSearchKey;
typedef struct _OchushaSearchKeyClass OchushaSearchKeyClass;


GType ochusha_search_key_get_type(void) G_GNUC_CONST;
OchushaSearchKey *ochusha_search_key_new(void);
OchushaSearchKey *ochusha_search_key_new_with_initial_key(const gchar *key_str,
					OchushaSearchDirectionType direction,
					gboolean match_case,
					gboolean use_regexp);

void ochusha_search_key_set_full(OchushaSearchKey *key, const gchar *key_str,
				 OchushaSearchDirectionType direction,
				 gboolean match_case, gboolean use_regexp);

void ochusha_search_key_set_key(OchushaSearchKey *key, const gchar *key_str);
const gchar *ochusha_search_key_get_key(OchushaSearchKey *key);

void ochusha_search_key_set_direction(OchushaSearchKey *key,
				      OchushaSearchDirectionType direction);
OchushaSearchDirectionType ochusha_search_key_get_direction(OchushaSearchKey *key);

void ochusha_search_key_set_match_case(OchushaSearchKey *key,
				       gboolean match_case);
gboolean ochusha_search_key_get_match_case(OchushaSearchKey *key);

void ochusha_search_key_set_use_regexp(OchushaSearchKey *key,
				       gboolean use_regexp);
gboolean ochusha_search_key_get_use_regexp(OchushaSearchKey *key);

gboolean ochusha_search_key_match_text(OchushaSearchKey *key,
				       const gchar *text, int len);
const gchar *ochusha_search_key_last_match_begin(OchushaSearchKey *key);
const gchar *ochusha_search_key_last_match_end(OchushaSearchKey *key);
size_t ochusha_search_key_last_match_len(OchushaSearchKey *key);



#define OCHUSHA_TYPE_SEARCHABLE			(ochusha_searchable_get_type())
#define OCHUSHA_SEARCHABLE(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_SEARCHABLE, OchushaSearchable))
#define OCHUSHA_IS_SEARCHABLE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_SEARCHABLE))
#define OCHUSHA_SEARCHABLE_GET_IFACE(obj)	(G_TYPE_INSTANCE_GET_INTERFACE((obj), OCHUSHA_TYPE_SEARCHABLE, OchushaSearchableIface))


typedef struct _OchushaSearchable OchushaSearchable;	/* Dummy */
typedef struct _OchushaSearchableIface OchushaSearchableIface;

struct _OchushaSearchableIface
{
  GTypeInterface g_iface;

  /* Signals */

  /* Virtual Function Table */
  gboolean (*find_next)(OchushaSearchable *searchable, OchushaSearchKey *key);
  void (*terminate_search)(OchushaSearchable *searchable,
			   OchushaSearchKey *key);
};


GType ochusha_searchable_get_type(void) G_GNUC_CONST;
gboolean ochusha_searchable_find_next(OchushaSearchable *searchable,
				      OchushaSearchKey *key);
void ochusha_searchable_terminate_search(OchushaSearchable *searchable,
					 OchushaSearchKey *key);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_SEARCHABLE_H_ */
