/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: searchbar.h,v 1.8 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_SEARCHBAR_H_
#define _LIBOCHUSHAGTK_SEARCHBAR_H_

#include "libochushagtk/searchable.h"

#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

#define OCHUSHA_TYPE_SEARCHBAR			(ochusha_searchbar_get_type())
#define OCHUSHA_SEARCHBAR(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_SEARCHBAR, OchushaSearchbar))
#define OCHUSHA_SEARCHBAR_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_SEARCHBAR, OchushaSearchbarClass))
#define OCHUSHA_IS_SEARCHBAR(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_SEARCHBAR))
#define OCHUSHA_IS_SEARCHBAR_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), OCHUSHA_TYPE_SEARCHBAR))
#define OCHUSHA_SEARCHBAR_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_SEARCHBAR, OchushaSearchbarClass))


typedef struct _OchushaSearchbar OchushaSearchbar;
typedef struct _OchushaSearchbarClass OchushaSearchbarClass;


struct _OchushaSearchbar
{
  GtkHBox parent_object;

  GtkEntry *entry;

  GtkToggleButton *match_case_button;
  GtkToggleButton *use_regexp_button;

  GtkWidget *find_next_button;
  GtkWidget *find_prev_button;

  GtkStatusbar *statusbar;
  guint statusbar_context_id;
  guint statusbar_message_id;


  OchushaSearchKey *key;
  gulong key_changed_id;

  OchushaSearchable *last_search_target;
  gulong last_search_target_destroy_id;

  guint enable_incremental_search	: 1;
};


struct _OchushaSearchbarClass
{
  GtkHBoxClass parent_class;

  void (*response)(OchushaSearchbar *bar, int response_id);
  OchushaSearchable *(*search_request)(OchushaSearchbar *bar);
  void (*find_previous)(OchushaSearchbar *bar);
  void (*find_next)(OchushaSearchbar *bar);
};


GType ochusha_searchbar_get_type(void) G_GNUC_CONST;

GtkWidget *ochusha_searchbar_new(void);
GtkWidget *ochusha_searchbar_new_with_key(OchushaSearchKey *key);

void ochusha_searchbar_set_key(OchushaSearchbar *bar, OchushaSearchKey *key);
OchushaSearchKey *ochusha_searchbar_get_key(OchushaSearchbar *bar);

void ochusha_searchbar_set_enable_incremental_search(OchushaSearchbar *bar,
						     gboolean enable);
gboolean ochusha_searchbar_get_enable_incremental_search(OchushaSearchbar *bar);

void ochusha_searchbar_response(OchushaSearchbar *bar, int response_id);
void ochusha_searchbar_set_key_selected(OchushaSearchbar *bar);

void ochusha_searchbar_grab_focus(OchushaSearchbar *bar);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_SEARCH_BAR_H_ */
