/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: stock.c,v 1.22 2009/01/01 07:46:51 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#include "libochushagtk/stock.h"

#include <gtk/gtk.h>


#include "libochushagtk/aa_helper_disabled_pixdata.h"
#include "libochushagtk/aa_helper_enabled_pixdata.h"
#include "libochushagtk/active_folder_pixdata.h"
#include "libochushagtk/add_board_pixdata.h"
#include "libochushagtk/add_bookmark_pixdata.h"
#include "libochushagtk/board_bookmark_pixdata.h"
#include "libochushagtk/board_pixdata.h"
#include "libochushagtk/bookmark_pixdata.h"
#include "libochushagtk/bookmark_folder_pixdata.h"
#include "libochushagtk/edit_disabled_pixdata.h"
#include "libochushagtk/edit_enabled_pixdata.h"
#include "libochushagtk/embeded_editor_pixdata.h"
#include "libochushagtk/folder_closed_pixdata.h"
#include "libochushagtk/folder_open_pixdata.h"
#include "libochushagtk/missing_image_pixdata.h"
#include "libochushagtk/post_pixdata.h"
#include "libochushagtk/pseudo_folder_pixdata.h"
#include "libochushagtk/thread_bookmark_pixdata.h"
#include "libochushagtk/thread_pixdata.h"

#include "libochushagtk/net_done_pixdata.h"
#include "libochushagtk/net_error_pixdata.h"
#include "libochushagtk/net_notice_pixdata.h"
#include "libochushagtk/net_preparation_pixdata.h"
#include "libochushagtk/net_wait_pixdata.h"
#include "libochushagtk/net_progress0_pixdata.h"
#include "libochushagtk/net_progress1_pixdata.h"
#include "libochushagtk/net_progress2_pixdata.h"
#include "libochushagtk/net_progress3_pixdata.h"
#include "libochushagtk/net_progress4_pixdata.h"
#include "libochushagtk/net_progress5_pixdata.h"
#include "libochushagtk/net_progress6_pixdata.h"
#include "libochushagtk/net_progress7_pixdata.h"


#include <stdio.h>


#define add_stock_icons(factory, stock_id, basename)			\
  do									\
    {									\
      GtkIconSource *source = gtk_icon_source_new();			\
      GtkIconSet *set = gtk_icon_set_new();				\
      GdkPixbuf *pixbuf16 = gdk_pixbuf_new_from_inline(-1, basename ## _16, \
						       FALSE, NULL);	\
      GdkPixbuf *pixbuf18 = gdk_pixbuf_new_from_inline(-1, basename ## _18, \
						       FALSE, NULL);	\
      GdkPixbuf *pixbuf20 = gdk_pixbuf_new_from_inline(-1, basename ## _20, \
						       FALSE, NULL);	\
      GdkPixbuf *pixbuf24 = gdk_pixbuf_new_from_inline(-1, basename ## _24, \
						       FALSE, NULL);	\
      GdkPixbuf *pixbuf32 = gdk_pixbuf_new_from_inline(-1, basename ## _32, \
						       FALSE, NULL);	\
      GdkPixbuf *pixbuf48 = gdk_pixbuf_new_from_inline(-1, basename ## _48, \
						       FALSE, NULL);	\
      gtk_icon_source_set_icon_name(source, stock_id);			\
      gtk_icon_set_add_source(set, source);				\
      gtk_icon_factory_add(factory, stock_id, set);			\
      gtk_icon_theme_add_builtin_icon(stock_id, 16, pixbuf16);		\
      gtk_icon_theme_add_builtin_icon(stock_id, 18, pixbuf18);		\
      gtk_icon_theme_add_builtin_icon(stock_id, 20, pixbuf20);		\
      gtk_icon_theme_add_builtin_icon(stock_id, 24, pixbuf24);		\
      gtk_icon_theme_add_builtin_icon(stock_id, 32, pixbuf32);		\
      gtk_icon_theme_add_builtin_icon(stock_id, 48, pixbuf48);		\
      g_object_unref(pixbuf48);						\
      g_object_unref(pixbuf32);						\
      g_object_unref(pixbuf24);						\
      g_object_unref(pixbuf20);						\
      g_object_unref(pixbuf18);						\
      g_object_unref(pixbuf16);						\
      gtk_icon_source_free(source);					\
      gtk_icon_set_unref(set);						\
    }									\
  while(0)


static void
ochusha_initialize_icons(void)
{
  GtkIconFactory *factory = gtk_icon_factory_new();

  add_stock_icons(factory, OCHUSHA_STOCK_AA_HELPER_DISABLED, aa_helper_disabled);
  add_stock_icons(factory, OCHUSHA_STOCK_AA_HELPER_ENABLED, aa_helper_enabled);
  add_stock_icons(factory, OCHUSHA_STOCK_ACTIVE_FOLDER, active_folder);
  add_stock_icons(factory, OCHUSHA_STOCK_ADD_BOARD, add_board);
  add_stock_icons(factory, OCHUSHA_STOCK_ADD_BOOKMARK, add_bookmark);
  add_stock_icons(factory, OCHUSHA_STOCK_BOARD, board);
  add_stock_icons(factory, OCHUSHA_STOCK_BOARD_BOOKMARK, board_bookmark);
  add_stock_icons(factory, OCHUSHA_STOCK_BOOKMARK, bookmark);
  add_stock_icons(factory, OCHUSHA_STOCK_BOOKMARK_FOLDER, bookmark_folder);
  add_stock_icons(factory, OCHUSHA_STOCK_EDIT_DISABLED, edit_disabled);
  add_stock_icons(factory, OCHUSHA_STOCK_EDIT_ENABLED, edit_enabled);
  add_stock_icons(factory, OCHUSHA_STOCK_EMBEDED_EDITOR, embeded_editor);
  add_stock_icons(factory, OCHUSHA_STOCK_FOLDER_CLOSED, folder_closed);
  add_stock_icons(factory, OCHUSHA_STOCK_FOLDER_OPEN, folder_open);
  add_stock_icons(factory, OCHUSHA_STOCK_MISSING_IMAGE, missing_image);
  add_stock_icons(factory, OCHUSHA_STOCK_POST, post);
  add_stock_icons(factory, OCHUSHA_STOCK_PSEUDO_FOLDER, pseudo_folder);
  add_stock_icons(factory, OCHUSHA_STOCK_THREAD, thread);
  add_stock_icons(factory, OCHUSHA_STOCK_THREAD_BOOKMARK, thread_bookmark);

  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON0, net_progress0);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON1, net_progress1);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON2, net_progress2);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON3, net_progress3);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON4, net_progress4);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON5, net_progress5);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON6, net_progress6);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_PROGRESS_ADDON7, net_progress7);

  add_stock_icons(factory, OCHUSHA_STOCK_NET_PREPARATION_ADDON, net_preparation);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_DONE_ADDON, net_done);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_NOTICE_ADDON, net_notice);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_ERROR_ADDON, net_error);
  add_stock_icons(factory, OCHUSHA_STOCK_NET_WAIT_ADDON, net_wait);

  gtk_icon_factory_add_default(factory);

  g_object_unref(factory);
}


void
ochusha_initialize_stock_items(void)
{
#if 0
  static const GtkStockItem ochusha_stock_items[] = {
    {
      (char *)OCHUSHA_STOCK_AA_HELPER_DISABLED,
      (char *)N_("AA Input Helper Disabled"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_AA_HELPER_ENABLED,
      (char *)N_("AA Input Helper Enabled"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_ACTIVE_FOLDER,
      (char *)N_("Active Folder"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_ADD_BOARD,
      (char *)N_("Add Board"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_ADD_BOOKMARK,
      (char *)N_("Add Bookmark"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_BOARD,
      (char *)N_("Board"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_BOOKMARK,
      (char *)N_("Bookmark"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_BOOKMARK_FOLDER,
      (char *)N_("Bookmark Folder"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_EDIT_DISABLED,
      (char *)N_("Edit Disabled"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_EDIT_ENABLED,
      (char *)N_("Edit Enabled"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_EMBEDED_EDITOR,
      (char *)N_("Embeded Editor"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_FOLDER_CLOSED,
      (char *)N_("Folder Closed"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_FOLDER_OPEN,
      (char *)N_("Folder Open"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_MISSING_IMAGE,
      (char *)N_("Missing Image"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_PSEUDO_FOLDER,
      (char *)N_("Pseudo Folder"),
      0, 0,
      (char *)PACKAGE
    },
    {
      (char *)OCHUSHA_STOCK_THREAD,
      (char *)N_("Thread"),
      0, 0,
      (char *)PACKAGE
    }
  };
#endif

  ochusha_initialize_icons();

#if 0
  /* ɬפʤʡġġ */
  gtk_stock_add_static(ochusha_stock_items, G_N_ELEMENTS(ochusha_stock_items));
#endif
}
