/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree_menu_helper.c,v 1.14 2009/01/01 07:46:51 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"
#include "libochushagtk/tree_menu_helper.h"
#include "libochushagtk/marshal.h"

#include <glib.h>
#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


static void ochusha_tree_menu_helper_class_init(OchushaTreeMenuHelperClass *klass);
static void ochusha_tree_menu_helper_init(OchushaTreeMenuHelper *helper);
static void ochusha_tree_menu_helper_finalize(GObject *object);

static gboolean ochusha_tree_menu_helper_menu_item_new_accumulator(
						GSignalInvocationHint *ihint,
						GValue *return_accu,
						const GValue *handler_return,
						gpointer data);

static GtkMenuItem *ochusha_tree_menu_helper_menu_item_new(
						OchushaTreeMenuHelper *helper,
						GtkMenuShell *shell,
						GtkTreeModel *model,
						GtkTreeIter *iter);

static void construct_menu(OchushaTreeMenuHelper *helper, GtkMenuShell *shell,
			   GtkTreeModel *model, GtkTreeIter *iter);


GType
ochusha_tree_menu_helper_get_type(void)
{
  static GType tree_menu_helper_type = 0;

  if (tree_menu_helper_type == 0)
    {
      static const GTypeInfo tree_menu_helper_info =
	{
	  sizeof(OchushaTreeMenuHelperClass),
	  NULL,	/* base_init */
	  NULL,	/* base_finalize */
	  (GClassInitFunc)ochusha_tree_menu_helper_class_init,
	  NULL,	/* class_finalize */
	  NULL,	/* class_data */
	  sizeof(OchushaTreeMenuHelper),
	  0,	/* n_preallocs */
	  (GInstanceInitFunc)ochusha_tree_menu_helper_init
	};

      tree_menu_helper_type = g_type_register_static(G_TYPE_OBJECT,
						     "OchushaTreeMenuHelper",
						     &tree_menu_helper_info,
						     0);
    }

  return tree_menu_helper_type;
}


enum {
  ACTIVATE_SIGNAL,
  MENU_ITEM_NEW_SIGNAL,
  LAST_SIGNAL
};


static GObjectClass *parent_class = NULL;
static int tree_menu_helper_signals[LAST_SIGNAL] = { 0, 0 };

static GQuark menu_info_id;


static void
ochusha_tree_menu_helper_class_init(OchushaTreeMenuHelperClass *klass)
{
  GObjectClass *o_class = (GObjectClass *)klass;

  parent_class = g_type_class_peek_parent(klass);

  /* GObject signals */
  o_class->finalize = ochusha_tree_menu_helper_finalize;

  tree_menu_helper_signals[ACTIVATE_SIGNAL]
    = g_signal_new("activate",
		   G_TYPE_FROM_CLASS(klass),
		   G_SIGNAL_RUN_LAST,
		   G_STRUCT_OFFSET(OchushaTreeMenuHelperClass, activate),
		   NULL, NULL,
		   libochushagtk_marshal_VOID__OBJECT_BOXED_BOXED_UINT,
		   G_TYPE_NONE, 4,
		   GTK_TYPE_TREE_MODEL,
		   GTK_TYPE_TREE_ITER | G_SIGNAL_TYPE_STATIC_SCOPE,
		   GTK_TYPE_TREE_PATH | G_SIGNAL_TYPE_STATIC_SCOPE,
		   G_TYPE_UINT);

  tree_menu_helper_signals[MENU_ITEM_NEW_SIGNAL]
    = g_signal_new("menu_item_new",
		   G_TYPE_FROM_CLASS(klass),
		   G_SIGNAL_RUN_LAST,
		   G_STRUCT_OFFSET(OchushaTreeMenuHelperClass, menu_item_new),
		   ochusha_tree_menu_helper_menu_item_new_accumulator, NULL,
		   libochushagtk_marshal_OBJECT__OBJECT_OBJECT_BOXED,
		   GTK_TYPE_MENU_ITEM, 3,
		   GTK_TYPE_MENU_SHELL,
		   GTK_TYPE_TREE_MODEL,
		   GTK_TYPE_TREE_ITER | G_SIGNAL_TYPE_STATIC_SCOPE);


  klass->activate = NULL;
  klass->menu_item_new = ochusha_tree_menu_helper_menu_item_new;


  menu_info_id = g_quark_from_static_string("OchushaTreeMenuHelper::MenuInfo");
}


static void
ochusha_tree_menu_helper_init(OchushaTreeMenuHelper *helper)
{
  helper->model = NULL;

  helper->menu_data_func = NULL;
  helper->menu_data_func_data = NULL;
  helper->menu_data_func_data_destroy = NULL;

  helper->row_separator_func = NULL;
  helper->row_separator_func_data = NULL;
  helper->row_separator_func_data_destroy = NULL;

  helper->row_sensitive_func = NULL;
  helper->row_sensitive_func_data = NULL;
  helper->row_sensitive_func_data_destroy = NULL;

  helper->menu_shell_list = NULL;

  helper->button = 0;
}


static void
ochusha_tree_menu_helper_finalize(GObject *object)
{
  OchushaTreeMenuHelper *helper = OCHUSHA_TREE_MENU_HELPER(object);

  if (helper->menu_data_func_data_destroy != NULL)
    {
      (*helper->menu_data_func_data_destroy)(helper->menu_data_func_data);
      helper->menu_data_func = NULL;
      helper->menu_data_func_data = NULL;
      helper->menu_data_func_data_destroy = NULL;
    }

  if (helper->row_separator_func_data_destroy != NULL)
    {
      (*helper->row_separator_func_data_destroy)(helper->row_separator_func_data);
      helper->row_separator_func = NULL;
      helper->row_separator_func_data = NULL;
      helper->row_separator_func_data_destroy = NULL;
    }

  if (helper->row_sensitive_func_data_destroy != NULL)
    {
      (*helper->row_sensitive_func_data_destroy)(helper->row_sensitive_func_data);
      helper->row_sensitive_func = NULL;
      helper->row_sensitive_func_data = NULL;
      helper->row_sensitive_func_data_destroy = NULL;
    }

  if (helper->model != NULL)
    {
      g_object_unref(helper->model);
      helper->model = NULL;
    }

  if (G_OBJECT_CLASS(parent_class)->finalize != NULL)
    (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}


static gboolean
ochusha_tree_menu_helper_menu_item_new_accumulator(GSignalInvocationHint *ihint,
						   GValue *return_accu,
						   const GValue *handler_return,
						   gpointer data)
{
  gpointer value = g_value_get_object(handler_return);
  if (value != NULL)
    {
      g_value_set_object(return_accu, value);
      return FALSE;
    }

  return TRUE;
}


static GtkMenuItem *
ochusha_tree_menu_helper_menu_item_new(OchushaTreeMenuHelper *helper,
				       GtkMenuShell *shell,
				       GtkTreeModel *model, GtkTreeIter *iter)
{
  return g_object_new(GTK_TYPE_IMAGE_MENU_ITEM, NULL);
}


typedef struct _TreeMenuItemInfo
{
  OchushaTreeMenuHelper *helper;
  GtkTreeRowReference *reference;
} TreeMenuItemInfo;


static TreeMenuItemInfo *
menu_item_info_new(OchushaTreeMenuHelper *helper, GtkTreePath *path)
{
  TreeMenuItemInfo *info = g_new(TreeMenuItemInfo, 1);
  info->helper = helper;
  g_object_ref(helper);
  info->reference = gtk_tree_row_reference_new(helper->model, path);

  return info;
}


static void
menu_item_info_free(TreeMenuItemInfo *info)
{
  g_object_unref(info->helper);
  info->helper = NULL;
  gtk_tree_row_reference_free(info->reference);
  info->reference = NULL;
  g_free(info);
}


static inline TreeMenuItemInfo *
get_tree_menu_item_info(OchushaTreeMenuHelper *helper, GtkMenuItem *item)
{
  TreeMenuItemInfo *info = g_object_get_qdata(G_OBJECT(item), menu_info_id);
  if (info != NULL && info->helper == helper)
    return info;
  return NULL;
}


static void
menu_item_activate_cb(GtkMenuItem *menu_item, OchushaTreeMenuHelper *helper)
{
  TreeMenuItemInfo *info = get_tree_menu_item_info(helper, menu_item);
  GtkTreePath *path;
  GtkTreeIter iter;
  guint button;

  g_return_if_fail(info != NULL);
  g_return_if_fail(gtk_tree_row_reference_valid(info->reference));

  if (GTK_IS_RADIO_MENU_ITEM(menu_item)
      && !gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item)))
    return;

  path = gtk_tree_row_reference_get_path(info->reference);
  if (gtk_tree_model_get_iter(helper->model, &iter, path))
    {
      if (helper->button != 0)
	{
	  button = helper->button;
	  helper->button = 0;
	}
      else
	button = 1;	/* may be activated by key-events, fake as button 1 */

      g_signal_emit(helper, tree_menu_helper_signals[ACTIVATE_SIGNAL],
		    0,
		    helper->model, &iter, path, button);
    }

  gtk_tree_path_free(path);
}


static gboolean
menu_item_button_release_cb(GtkMenuItem *item, GdkEventButton *event,
			    OchushaTreeMenuHelper *helper)
{
  helper->button = event->button;
  return FALSE;
}


static GtkWidget *
create_menu_item(OchushaTreeMenuHelper *helper, GtkMenuShell *shell,
		 GtkTreeModel *model, GtkTreeIter *iter, GtkTreePath *path)
{
  GtkMenuItem *menu_item = NULL;
  TreeMenuItemInfo *info;
  gboolean is_separator;

  if (helper->row_separator_func != NULL)
    is_separator
      = (*helper->row_separator_func)(model, iter,
				      helper->row_separator_func_data);
  else
    is_separator = FALSE;

  if (!is_separator)
    g_signal_emit(helper, tree_menu_helper_signals[MENU_ITEM_NEW_SIGNAL],
		  0,
		  shell, model, iter,
		  &menu_item);
  else
    menu_item = GTK_MENU_ITEM(gtk_separator_menu_item_new());

  if (menu_item != NULL && !GTK_IS_MENU_ITEM(menu_item))
    {
#if GTK_CHECK_VERSION(2,10,0)
      g_object_ref_sink(menu_item);
      g_object_unref(menu_item);
#else
      gtk_object_sink(GTK_OBJECT(menu_item));
#endif
      menu_item = NULL;
    }

  if (menu_item == NULL)
    menu_item = ochusha_tree_menu_helper_menu_item_new(helper, shell,
						       model, iter);

  g_return_val_if_fail(menu_item != NULL, NULL);

  info = menu_item_info_new(helper, path);
  g_object_set_qdata_full(G_OBJECT(menu_item), menu_info_id, info,
			  (GDestroyNotify)menu_item_info_free);

  if (!is_separator)
    {
      GtkWidget *label;

      g_signal_connect(menu_item, "activate",
		       G_CALLBACK(menu_item_activate_cb), helper);
      g_signal_connect(menu_item, "button_release_event",
		       G_CALLBACK(menu_item_button_release_cb), helper);

      label = gtk_bin_get_child(GTK_BIN(menu_item));
      if (label == NULL)
	{
	  label = gtk_label_new(NULL);
	  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	  gtk_widget_show(label);
	  gtk_container_add(GTK_CONTAINER(menu_item), label);
	}

      if (helper->menu_data_func != NULL)
	(*helper->menu_data_func)(menu_item, GTK_LABEL(label), model, iter,
				  helper->menu_data_func_data);
      if (helper->row_sensitive_func != NULL)
	{
	  gboolean sensitive
	    = (*helper->row_sensitive_func)(model, iter,
					    helper->row_sensitive_func_data);
	  gtk_widget_set_sensitive(GTK_WIDGET(menu_item), sensitive);
	}
    }

  return GTK_WIDGET(menu_item);
}


static void
update_menu_item(OchushaTreeMenuHelper *helper, GtkMenuItem *menu_item,
		 GtkTreeModel *model, GtkTreeIter *iter)
{
  GtkWidget *label;

  /* Revisit Here!
   * We don't care about state change of non separator to separator and
   * its opposite at the time being.
   */
  if (helper->row_separator_func != NULL)
    {
      gboolean is_separator
	= (*helper->row_separator_func)(model, iter,
					helper->row_separator_func_data);
      g_return_if_fail(is_separator == GTK_IS_SEPARATOR_MENU_ITEM(menu_item));
      if (is_separator)
	return;
    }

  g_return_if_fail(helper->menu_data_func != NULL);
  g_return_if_fail(GTK_IS_MENU_ITEM(menu_item));

  label = gtk_bin_get_child(GTK_BIN(menu_item));

  if (label != NULL && !GTK_IS_LABEL(label))
    {
      gtk_container_remove(GTK_CONTAINER(menu_item), label);
      label = NULL;
    }

  if (label == NULL)
    {
      label = gtk_label_new(NULL);
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
      gtk_widget_show(label);
      gtk_container_add(GTK_CONTAINER(menu_item), label);
    }

  (*helper->menu_data_func)(menu_item, GTK_LABEL(label), model, iter,
			    helper->menu_data_func_data);

  if (helper->row_sensitive_func != NULL)
    {
      gboolean sensitive
	= (*helper->row_sensitive_func)(model, iter,
					helper->row_sensitive_func_data);
      gtk_widget_set_sensitive(GTK_WIDGET(menu_item), sensitive);
    }
}


typedef struct _LookupNthMenuItemArgs
{
  OchushaTreeMenuHelper *helper;
  gint n;
  gint count;
  gint position;
  GtkMenuItem *item;
  TreeMenuItemInfo *info;
} LookupNthMenuItemArgs;


static void
find_nth_child(GtkWidget *widget, gpointer data)
{
  LookupNthMenuItemArgs *args = data;

  if (args->n < args->count)
    return;

  if (GTK_IS_MENU_ITEM(widget))
    {
      GtkMenuItem *item = GTK_MENU_ITEM(widget);
      TreeMenuItemInfo *info = get_tree_menu_item_info(args->helper, item);
      if (info != NULL && args->n == args->count++)
	{
	  args->item = item;
	  args->info = info;
	  return;
	}
    }

  args->position++;
}


static inline GtkMenuItem *
lookup_nth_menu_item_from_shell(OchushaTreeMenuHelper *helper,
				GtkMenuShell *shell, gint n)
{
  LookupNthMenuItemArgs args = { helper, n, 0, 0, NULL, NULL };
  gtk_container_foreach(GTK_CONTAINER(shell), find_nth_child, &args);
  return args.item;
}


GtkMenuItem *
ochusha_tree_menu_helper_get_nth_menu_item(OchushaTreeMenuHelper *helper,
					   GtkMenuShell *shell, gint n)
{
  g_return_val_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper), NULL);
  g_return_val_if_fail(GTK_IS_MENU_SHELL(shell), NULL);

  return lookup_nth_menu_item_from_shell(helper, shell, n);
}


/*
 * lookups a GtkMenuItem corresponds to path from a tree menu rooted by
 * root.
 */
static GtkMenuItem *
lookup_menu_item_from_shell(OchushaTreeMenuHelper *helper, GtkMenuShell *root,
			    GtkTreePath *path)
{
  gint depth = gtk_tree_path_get_depth(path);
  gint *indices = gtk_tree_path_get_indices(path);
  gint current_depth;
  GtkMenuShell *shell;

  g_return_val_if_fail(depth > 0, NULL);

  current_depth = 0;
  shell = root;
  while (TRUE)
    {
      GtkMenuItem *item
	= lookup_nth_menu_item_from_shell(helper, shell,
					  indices[current_depth++]);
      GtkWidget *widget;
      g_return_val_if_fail(item != NULL, NULL);

      if (current_depth == depth)
	{
	  TreeMenuItemInfo *info = get_tree_menu_item_info(helper, item);
	  if (info != NULL && gtk_tree_row_reference_valid(info->reference))
	    {
	      GtkTreePath *item_path
		= gtk_tree_row_reference_get_path(info->reference);
	      gint compare_result = gtk_tree_path_compare(item_path, path);
	      gtk_tree_path_free(item_path);

	      if (compare_result == 0)
		return item;
	    }
	  // should never reached!
	  g_return_val_if_fail(FALSE, NULL);
	}

      widget = gtk_menu_item_get_submenu(item);
      shell = GTK_MENU_SHELL(widget);
      g_return_val_if_fail(shell != NULL, NULL);
    }
}


GList *
ochusha_tree_menu_helper_collect_menu_items(OchushaTreeMenuHelper *helper,
					    GtkTreePath *path)
{
  GList *shell_list;
  GList *result;

  g_return_val_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper), NULL);

  shell_list = helper->menu_shell_list;
  result = NULL;
  while (shell_list != NULL)
    {
      GtkMenuItem *item
	= lookup_menu_item_from_shell(helper, shell_list->data, path);
      if (item != NULL)
	result = g_list_prepend(result, item);
      shell_list = g_list_next(shell_list);
    }

  return result;
}


static inline void
update_menu_entry(OchushaTreeMenuHelper *helper,
		  GtkTreeModel *model, GtkTreePath *path,
		  GtkTreeIter *iter, GtkMenuShell *shell)
{
  GtkMenuItem *menu_item = lookup_menu_item_from_shell(helper, shell, path);
  g_return_if_fail(menu_item != NULL);

  update_menu_item(helper, menu_item, model, iter);
}


static void
row_changed_cb(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
	       OchushaTreeMenuHelper *helper)
{
  GList *list = helper->menu_shell_list;
  while (list != NULL)
    {
      update_menu_entry(helper, model, path, iter, list->data);
      list = g_list_next(list);
    }
}


/*
 * returns the index of `position'th GtkMenuItem under the control of
 * the helper.
 */
static inline gint
calculate_effective_position(OchushaTreeMenuHelper *helper,
			     GtkMenuShell *shell, gint position)
{
  LookupNthMenuItemArgs args = { helper, position, 0, 0, NULL, NULL };
  gtk_container_foreach(GTK_CONTAINER(shell), find_nth_child, &args);
  return args.position;
}


static void
insert_menu_entry_into_shell(OchushaTreeMenuHelper *helper,
			     GtkTreeModel *model,
			     GtkTreeIter *iter, GtkTreePath *path,
			     GtkMenuShell *shell)
{
  gint depth = gtk_tree_path_get_depth(path);
  GtkMenuShell *parent_shell;
  GtkWidget *item;
  gint *indices;
  gint position;

  if (depth == 1)
    parent_shell = shell;
  else
    {
      GtkTreePath *parent_path = gtk_tree_path_copy(path);
      gboolean result = gtk_tree_path_up(parent_path);
      if (result)
	{
	  GtkMenuItem *parent_item
	    = lookup_menu_item_from_shell(helper, shell, parent_path);
	  gtk_tree_path_free(parent_path);

	  if (parent_item != NULL)
	    {
	      GtkMenuItem *parent_menu_item = GTK_MENU_ITEM(parent_item);
	      GtkWidget *submenu = gtk_menu_item_get_submenu(parent_menu_item);
	      if (submenu != NULL)
		parent_shell = GTK_MENU_SHELL(submenu);
	      else
		{
		  submenu = gtk_menu_new();
		  parent_shell = GTK_MENU_SHELL(submenu);
		  gtk_widget_show(submenu);
		  gtk_menu_item_set_submenu(parent_menu_item, submenu);
		}
	    }
	  else
	    parent_shell = NULL;
	}
      else
	{
	  gtk_tree_path_free(parent_path);
	  parent_shell = NULL;
	}
    }

  g_return_if_fail(parent_shell != NULL);

  indices = gtk_tree_path_get_indices(path);
  position = calculate_effective_position(helper, parent_shell,
					  indices[depth - 1]);
  item = create_menu_item(helper, parent_shell, model, iter, path);
  gtk_widget_show(item);
  gtk_menu_shell_insert(parent_shell, item, position);

  if (gtk_tree_model_iter_has_child(model, iter))
    {
      GtkWidget *submenu = gtk_menu_new();
      GtkTreeIter children_iter;

      gtk_widget_show(submenu);
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);

      if (gtk_tree_model_iter_children(model, &children_iter, iter))
	construct_menu(helper, GTK_MENU_SHELL(submenu), model, &children_iter);
    }
}


static void
row_inserted_cb(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
		OchushaTreeMenuHelper *helper)
{
  GList *list = helper->menu_shell_list;
  while (list != NULL)
    {
      insert_menu_entry_into_shell(helper, model, iter, path, list->data);
      list = g_list_next(list);
    }
}


/*
 * lookups the GtkMenuShell that should have the GtkMenuItem corresponds
 * to path.
 */
static GtkMenuShell *
lookup_parent_shell_from_shell(OchushaTreeMenuHelper *helper,
			       GtkMenuShell *root, GtkTreePath *path)
{
  gint depth = gtk_tree_path_get_depth(path) - 1;
  gint *indices = gtk_tree_path_get_indices(path);
  gint current_depth;
  GtkMenuShell *shell;

  g_return_val_if_fail(depth >= 0, NULL);

  if (depth == 0)
    return root;

  current_depth = 0;
  shell = root;
  while (TRUE)
    {
      GtkWidget *widget;
      GtkMenuItem *item
	= lookup_nth_menu_item_from_shell(helper, shell,
					  indices[current_depth++]);
      g_return_val_if_fail(item != NULL, NULL);

      widget = gtk_menu_item_get_submenu(item);
      shell = GTK_MENU_SHELL(widget);
      g_return_val_if_fail(shell != NULL, NULL);

      if (current_depth == depth)
	return shell;
    }
}


static void
remove_menu_entry_from_shell(OchushaTreeMenuHelper *helper,
			     GtkTreeModel *model, GtkTreePath *path,
			     GtkMenuShell *shell)
{
  GtkMenuShell *parent_shell;
  GtkMenuItem *item;
  gint *indices;
  gint depth;

  parent_shell = lookup_parent_shell_from_shell(helper, shell, path);
  g_return_if_fail(parent_shell != NULL);

  depth = gtk_tree_path_get_depth(path);
  indices = gtk_tree_path_get_indices(path);

  item = lookup_nth_menu_item_from_shell(helper, parent_shell,
					 indices[depth - 1]);
  g_return_if_fail(item != NULL);

  gtk_container_remove(GTK_CONTAINER(parent_shell), GTK_WIDGET(item));
}


static void
row_deleted_cb(GtkTreeModel *model, GtkTreePath *path,
	       OchushaTreeMenuHelper *helper)
{
  GList *list = helper->menu_shell_list;
  while (list != NULL)
    {
      remove_menu_entry_from_shell(helper, model, path, list->data);
      list = g_list_next(list);
    }
}


static void
reorder_menus(OchushaTreeMenuHelper *helper, GtkMenuShell *shell,
	      gint n_children, gint *new_order, GtkWidget **menus)
{
  gint i;
  gint position;
  GtkContainer *container = GTK_CONTAINER(shell);

  for (i = 0; i < n_children; i++)
    {
      GtkMenuItem *item
	= lookup_nth_menu_item_from_shell(helper, shell, new_order[i]);
      menus[i] = GTK_WIDGET(item);
    }

  position = calculate_effective_position(helper, shell, 0);

  for (i = 0; i < n_children; i++)
    {
      GtkWidget *widget = menus[i];
      if (widget != NULL)
	{
	  g_object_ref(widget);
	  gtk_container_remove(container, widget);
	}
    }

  for (i = 0; i < n_children; i++)
    {
      GtkWidget *widget = menus[i];
      if (widget != NULL)
	{
	  gtk_menu_shell_insert(shell, widget, position++);
	  g_object_unref(widget);
	  menus[i] = NULL;
	}
    }
}


static void
rows_reordered_cb(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
		  gint *new_order, OchushaTreeMenuHelper *helper)
{
  gint n_children = gtk_tree_model_iter_n_children(model, iter);
  GList *list = helper->menu_shell_list;
  GtkWidget **menus;

  if (list == NULL)
    return;

  menus = malloc(sizeof(GtkWidget *) * n_children);

  while (list != NULL)
    {
      GtkMenuShell *root = list->data;
      GtkMenuItem *item = lookup_menu_item_from_shell(helper, root, path);
      if (item != NULL)
	{
	  GtkWidget *submenu = gtk_menu_item_get_submenu(item);
	  if (submenu != NULL && GTK_IS_MENU_SHELL(submenu))
	    reorder_menus(helper, GTK_MENU_SHELL(submenu), n_children,
			  new_order, menus);
	}

      list = g_list_next(list);
    }

  free(menus);
}


static void
menu_shell_destroy_cb(GtkWidget *shell, OchushaTreeMenuHelper *helper)
{
  g_return_if_fail(g_list_find(helper->menu_shell_list, shell) != NULL);
  helper->menu_shell_list = g_list_remove(helper->menu_shell_list, shell);
}


/*
 * Insert all menu items corresponds to iter and its siblings into
 * the shell.
 *
 * `iter' will be destroyed!
 */
static void
construct_menu(OchushaTreeMenuHelper *helper, GtkMenuShell *shell,
	       GtkTreeModel *model, GtkTreeIter *iter)
{
  gint position = calculate_effective_position(helper, shell, 0);
  do
    {
      GtkTreePath *path = gtk_tree_model_get_path(model, iter);
      GtkWidget *item = create_menu_item(helper, shell, model, iter, path);

      gtk_widget_show(item);
      gtk_menu_shell_insert(shell, item, position++);
      gtk_tree_path_free(path);

      if (gtk_tree_model_iter_has_child(model, iter))
	{
	  GtkWidget *submenu = gtk_menu_new();
	  GtkTreeIter children_iter;

	  gtk_widget_show(submenu);
	  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);

	  if (gtk_tree_model_iter_children(model, &children_iter, iter))
	    construct_menu(helper, GTK_MENU_SHELL(submenu),
			   model, &children_iter);
	}
    }
  while (gtk_tree_model_iter_next(model, iter));
}


void
ochusha_tree_menu_helper_attach_tree_menu(OchushaTreeMenuHelper *helper,
					  GtkMenuShell *shell)
{
  g_return_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper));
  g_return_if_fail(GTK_IS_MENU_SHELL(shell));
  g_return_if_fail(g_list_find(helper->menu_shell_list, shell) == NULL);

  helper->menu_shell_list = g_list_prepend(helper->menu_shell_list, shell);
  g_signal_connect(shell, "destroy",
		   G_CALLBACK(menu_shell_destroy_cb), helper);

  if (helper->model != NULL)
    {
      GtkTreeIter iter;
      if (gtk_tree_model_get_iter_first(helper->model, &iter))
	construct_menu(helper, shell, helper->model, &iter);
    }
}


static void
remove_menus(OchushaTreeMenuHelper *helper, GtkMenuShell *shell)
{
  GtkContainer *container = GTK_CONTAINER(shell);
  GList *children = gtk_container_get_children(container);
  GList *list = children;

  while (list != NULL)
    {
      TreeMenuItemInfo *info = get_tree_menu_item_info(helper, list->data);
      if (info != NULL)
	gtk_container_remove(container, list->data);
      list = g_list_next(list);
    }

  g_list_free(children);
}


void
ochusha_tree_menu_helper_detach_tree_menu(OchushaTreeMenuHelper *helper,
					  GtkMenuShell *shell)
{
  g_return_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper));
  g_return_if_fail(GTK_IS_MENU_SHELL(shell));
  g_return_if_fail(g_list_find(helper->menu_shell_list, shell) != NULL);

  g_signal_handlers_disconnect_by_func(shell, menu_shell_destroy_cb, helper);
  helper->menu_shell_list = g_list_remove(helper->menu_shell_list, shell);

  remove_menus(helper, shell);
}


OchushaTreeMenuHelper *
ochusha_tree_menu_helper_new(void)
{
  return g_object_new(OCHUSHA_TYPE_TREE_MENU_HELPER, NULL);
}


OchushaTreeMenuHelper *
ochusha_tree_menu_helper_new_with_model(GtkTreeModel *model)
{
  OchushaTreeMenuHelper *helper;
  g_return_val_if_fail(GTK_IS_TREE_MODEL(model), NULL);

  helper = g_object_new(OCHUSHA_TYPE_TREE_MENU_HELPER, NULL);
  ochusha_tree_menu_helper_set_model(helper, model);

  return helper;
}


GtkTreeModel *
ochusha_tree_menu_helper_get_model(OchushaTreeMenuHelper *helper)
{
  g_return_val_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper), NULL);
  return helper->model;
}


static inline void
reconstruct_menu(OchushaTreeMenuHelper *helper, GtkMenuShell *shell)
{
  GtkTreeIter iter;

  remove_menus(helper, shell);

  if (gtk_tree_model_get_iter_first(helper->model, &iter))
    construct_menu(helper, shell, helper->model, &iter);
}


void
ochusha_tree_menu_helper_set_model(OchushaTreeMenuHelper *helper,
				   GtkTreeModel *model)
{
  g_return_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper));
  g_return_if_fail(GTK_IS_TREE_MODEL(model));

  g_object_ref(model);

  if (helper->model != NULL)
    {
      g_signal_handlers_disconnect_by_func(helper->model,
					   row_changed_cb, helper);
      g_signal_handlers_disconnect_by_func(helper->model,
					   row_inserted_cb, helper);
      g_signal_handlers_disconnect_by_func(helper->model,
					   row_deleted_cb, helper);
      g_signal_handlers_disconnect_by_func(helper->model,
					   rows_reordered_cb, helper);
      g_object_unref(helper->model);
    }

  g_signal_connect(model, "row_changed",
		   G_CALLBACK(row_changed_cb), helper);
  g_signal_connect(model, "row_inserted",
		   G_CALLBACK(row_inserted_cb), helper);
  g_signal_connect(model, "row_deleted",
		   G_CALLBACK(row_deleted_cb), helper);
  g_signal_connect(model, "rows_reordered",
		   G_CALLBACK(rows_reordered_cb), helper);

  helper->model = model;

  if (helper->menu_shell_list != NULL)
    {
      GList *list = helper->menu_shell_list;
      while (list != NULL)
	{
	  reconstruct_menu(helper, list->data);
	  list = g_list_next(list);
	}
    }
}


void
ochusha_tree_menu_helper_set_menu_data_func(OchushaTreeMenuHelper *helper,
					    OchushaTreeMenuDataFunc func,
					    gpointer func_data,
					    GDestroyNotify destroy)
{
  g_return_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper));

  if (helper->menu_data_func_data_destroy != NULL)
    (*helper->menu_data_func_data_destroy)(helper->menu_data_func_data);

  helper->menu_data_func = func;
  helper->menu_data_func_data = func_data;
  helper->menu_data_func_data_destroy = destroy;

  if (helper->menu_shell_list != NULL)
    {
      GList *list = helper->menu_shell_list;
      while (list != NULL)
	{
	  reconstruct_menu(helper, list->data);
	  list = g_list_next(list);
	}
    }
}


void
ochusha_tree_menu_helper_set_row_separator_func(OchushaTreeMenuHelper *helper,
						GtkTreeViewRowSeparatorFunc func,
						gpointer data,
						GDestroyNotify destroy)
{
  g_return_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper));

  if (helper->row_separator_func_data_destroy != NULL)
    (*helper->row_separator_func_data_destroy)(helper->row_separator_func_data);

  helper->row_separator_func = func;
  helper->row_separator_func_data = data;
  helper->row_separator_func_data_destroy = destroy;

  if (helper->menu_shell_list != NULL)
    {
      GList *list = helper->menu_shell_list;
      while (list != NULL)
	{
	  reconstruct_menu(helper, list->data);
	  list = g_list_next(list);
	}
    }
}


void
ochusha_tree_menu_helper_set_row_sensitive_func(OchushaTreeMenuHelper *helper,
						OchushaTreeMenuRowSensitiveFunc func,
						gpointer data,
						GDestroyNotify destroy)
{
  g_return_if_fail(OCHUSHA_IS_TREE_MENU_HELPER(helper));

  if (helper->row_sensitive_func_data_destroy != NULL)
    (*helper->row_sensitive_func_data_destroy)(helper->row_sensitive_func_data);

  helper->row_sensitive_func = func;
  helper->row_sensitive_func_data = data;
  helper->row_sensitive_func_data_destroy = destroy;

  if (helper->menu_shell_list != NULL)
    {
      GList *list = helper->menu_shell_list;
      while (list != NULL)
	{
	  reconstruct_menu(helper, list->data);
	  list = g_list_next(list);
	}
    }
}
