/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree_menu_helper.h,v 1.8 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_TREE_MENU_HELPER_H_
#define _LIBOCHUSHAGTK_TREE_MENU_HELPER_H_

#include <gtk/gtk.h>


#ifdef __cplusplus
extern "C" {
#endif

#define OCHUSHA_TYPE_TREE_MENU_HELPER			(ochusha_tree_menu_helper_get_type())
#define OCHUSHA_TREE_MENU_HELPER(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_TREE_MENU_HELPER, OchushaTreeMenuHelper))
#define OCHUSHA_TREE_MENU_HELPER_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_TREE_MENU_HELPER, OchushaTreeMenuHelperClass))
#define OCHUSHA_IS_TREE_MENU_HELPER(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_TREE_MENU_HELPER))
#define OCHUSHA_IS_TREE_MENU_HELPER_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE(klass), OCHUSHA_TYPE_TREE_MENU_HELPER)
#define OCHUSHA_TREE_MENU_HELPER_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_TREE_MENU_HELPER, OchushaTreeMenuHelperClass))


typedef struct _OchushaTreeMenuHelper OchushaTreeMenuHelper;
typedef struct _OchushaTreeMenuHelperClass OchushaTreeMenuHelperClass;

typedef void (*OchushaTreeMenuDataFunc)(GtkMenuItem *menu_item,
					GtkLabel *menu_label,
					GtkTreeModel *model,
					GtkTreeIter *iter,
					gpointer data);
typedef gboolean (*OchushaTreeMenuRowSensitiveFunc)(GtkTreeModel *model,
						    GtkTreeIter *iter,
						    gpointer data);


struct _OchushaTreeMenuHelper
{
  GObject object;

  GtkTreeModel *model;

  OchushaTreeMenuDataFunc menu_data_func;
  gpointer menu_data_func_data;
  GDestroyNotify menu_data_func_data_destroy;

  GtkTreeViewRowSeparatorFunc row_separator_func;
  gpointer row_separator_func_data;
  GDestroyNotify row_separator_func_data_destroy;

  OchushaTreeMenuRowSensitiveFunc row_sensitive_func;
  gpointer row_sensitive_func_data;
  GDestroyNotify row_sensitive_func_data_destroy;

  GList *menu_shell_list;

  guint button;
};


struct _OchushaTreeMenuHelperClass
{
  GObjectClass parent_class;

  void (*activate)(OchushaTreeMenuHelper *helper, GtkTreeModel *model,
		   GtkTreeIter *iter, GtkTreePath *path, guint button);

  GtkMenuItem *(*menu_item_new)(OchushaTreeMenuHelper *helper,
				GtkMenuShell *shell,
				GtkTreeModel *model, GtkTreeIter *iter);
};


GType ochusha_tree_menu_helper_get_type(void) G_GNUC_CONST;

OchushaTreeMenuHelper *ochusha_tree_menu_helper_new(void);
OchushaTreeMenuHelper *ochusha_tree_menu_helper_new_with_model(GtkTreeModel *model);

GtkTreeModel *ochusha_tree_menu_helper_get_model(OchushaTreeMenuHelper *helper);
void ochusha_tree_menu_helper_set_model(OchushaTreeMenuHelper *helper,
					GtkTreeModel *model);

void ochusha_tree_menu_helper_set_menu_data_func(OchushaTreeMenuHelper *helper,
						 OchushaTreeMenuDataFunc func,
						 gpointer func_data,
						 GDestroyNotify destroy);

/* separator related parts haven't implemented yet */
void ochusha_tree_menu_helper_set_row_separator_func(OchushaTreeMenuHelper *helper,
						     GtkTreeViewRowSeparatorFunc func,
						     gpointer data,
						     GDestroyNotify destroy);

void ochusha_tree_menu_helper_set_row_sensitive_func(OchushaTreeMenuHelper *helper,
						     OchushaTreeMenuRowSensitiveFunc func,
						     gpointer data,
						     GDestroyNotify destroy);

void ochusha_tree_menu_helper_attach_tree_menu(OchushaTreeMenuHelper *helper,
					       GtkMenuShell *shell);
void ochusha_tree_menu_helper_detach_tree_menu(OchushaTreeMenuHelper *helper,
					       GtkMenuShell *shell);


/* not for users */
GtkMenuItem *ochusha_tree_menu_helper_get_nth_menu_item(OchushaTreeMenuHelper *helper,
							GtkMenuShell *shell,
							gint n);
GList *ochusha_tree_menu_helper_collect_menu_items(OchushaTreeMenuHelper *helper,
						   GtkTreePath *path);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_TREE_MENU_HELPER_H_ */
