/*
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree_selector_view.h,v 1.13 2009/01/01 07:46:51 fuyu Exp $
 */

#ifndef _LIBOCHUSHAGTK_TREE_SELECTOR_VIEW_H_
#define _LIBOCHUSHAGTK_TREE_SELECTOR_VIEW_H_

#include "libochushagtk/searchable.h"
#include <gtk/gtk.h>

#ifdef __cplusplus
extern "C" {
#endif

#define OCHUSHA_TYPE_TREE_SELECTOR_VIEW			(ochusha_tree_selector_view_get_type())
#define OCHUSHA_TREE_SELECTOR_VIEW(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), OCHUSHA_TYPE_TREE_SELECTOR_VIEW, OchushaTreeSelectorView))
#define OCHUSHA_TREE_SELECTOR_VIEW_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass), OCHUSHA_TYPE_TREE_SELECTOR_VIEW, OchushaTreeSelectorViewClass))
#define OCHUSHA_IS_TREE_SELECTOR_VIEW(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), OCHUSHA_TYPE_TREE_SELECTOR_VIEW))
#define OCHUSHA_IS_TREE_SELECTOR_VIEW_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE(klass), OCHUSHA_TYPE_TREE_SELECTOR_VIEW)
#define OCHUSHA_TREE_SELECTOR_VIEW_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), OCHUSHA_TYPE_TREE_SELECTOR_VIEW, OchushaTreeSelectorViewClass))


typedef struct _OchushaTreeSelectorView OchushaTreeSelectorView;
typedef struct _OchushaTreeSelectorViewClass OchushaTreeSelectorViewClass;

typedef gboolean (*OchushaTreeSelectorViewSearchEqualFunc)(OchushaTreeSelectorView *view, GtkTreeModel *model, GtkTreeIter *iter, OchushaSearchKey *key, gpointer search_data);



struct _OchushaTreeSelectorView
{
  GtkTreeView tree_view;

  GtkTreePath *last_event_path;


  /* Ϣ */
  GtkTreeRowReference *last_search_pos;

  OchushaSearchKey *last_key;
  guint last_key_id;

  gboolean search_wrapped;

  OchushaTreeSelectorViewSearchEqualFunc search_func;
  gpointer search_data;
  GDestroyNotify search_destroy;
};


struct _OchushaTreeSelectorViewClass
{
  GtkTreeViewClass parent_class;

  gboolean (*row_mouse_over)(OchushaTreeSelectorView *view, GtkTreePath *path,
			     GtkTreeViewColumn *column, GdkEventMotion *event);
  gboolean (*row_mouse_out)(OchushaTreeSelectorView *view, GtkTreePath *path);
  gboolean (*row_button_press)(OchushaTreeSelectorView *view,
			       GtkTreePath *path, GtkTreeViewColumn *column,
			       GdkEventButton *event);
  gboolean (*row_button_release)(OchushaTreeSelectorView *view,
				 GtkTreePath *path, GtkTreeViewColumn *column,
				 GdkEventButton *event);
  void (*column_resized)(OchushaTreeSelectorView *view,
			 GtkTreeViewColumn *column, gboolean use_resized_width,
			 gint resized_width);
  void (*center_cursor_on_screen)(GtkTreeView *view);
};


GType ochusha_tree_selector_view_get_type(void) G_GNUC_CONST;


GtkWidget *ochusha_tree_selector_view_new(void);
GtkWidget *ochusha_tree_selector_view_new_with_model(GtkTreeModel *model);

void ochusha_tree_selector_view_set_search_equal_func(OchushaTreeSelectorView *view,
						      OchushaTreeSelectorViewSearchEqualFunc search_func,
						      gpointer search_data,
						      GDestroyNotify search_destroy);


#ifdef __cplusplus
} // extern "C"
#endif

#endif	/* _LIBOCHUSHAGTK_TREE_SELECTOR_VIEW_H_ */
