/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
/* GdkPixbuf library - Simple frame-based animations
 *
 * Copyright (C) Dom Lachowicz
 *
 * Authors: Dom Lachowicz <cinamod@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Based on code originally by:
 *          Jonathan Blandford <jrb@redhat.com>
 *          Havoc Pennington <hp@redhat.com>
 */

/*
 * This is a modified version of GdkPixbufSimpleAnim from GTK+-2.10.6 by
 * the Ochusha Project.  See the AUTHORS file for a list of people on the
 * Ochusha Project.  See the ChangeLog files for a list of changes.
 * These files are distributed with ochusha at
 * http://ochusha.sourceforge.jp/.
 *
 * Since libochushagtk_lgpl only has limited set of modified widgets and
 * is intended to be used with the original GTK+, I don't want to cause name
 * space conflictions with original widget sets from GTK+.  Therefore,
 * ALL PUBLICLY AVAILABLE NAMES ARE RENAMED.
 *
 * $Id: mod_pixbuf_simple_anim.c,v 1.1 2006/12/22 01:57:37 fuyu Exp $
 */


#define GDK_PIXBUF_ENABLE_BACKEND	1

#include <gtk/gtk.h>

#include "mod_pixbuf_simple_anim.h"

#include <stdio.h>


struct _ModPixbufSimpleAnimClass
{
        GdkPixbufAnimationClass parent_class;
};

/* Private part of the ModPixbufSimpleAnim structure */
struct _ModPixbufSimpleAnim
{
        GdkPixbufAnimation parent_instance;
        
        gint n_frames;
        
        gfloat rate;
        gint total_time;
        
        GList *frames;
        
        gint width;
        gint height;
        
        gboolean loop;
};


typedef struct _ModPixbufSimpleAnimIter ModPixbufSimpleAnimIter;
typedef struct _ModPixbufSimpleAnimIterClass ModPixbufSimpleAnimIterClass;

#define MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER              (mod_pixbuf_simple_anim_iter_get_type ())
#define MOD_PIXBUF_SIMPLE_ANIM_ITER(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER, ModPixbufSimpleAnimIter))
#define MOD_IS_PIXBUF_SIMPLE_ANIM_ITER(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER))

#define MOD_PIXBUF_SIMPLE_ANIM_ITER_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER, ModPixbufSimpleAnimIterClass))
#define MOD_IS_PIXBUF_SIMPLE_ANIM_ITER_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER))
#define MOD_PIXBUF_SIMPLE_ANIM_ITER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER, ModPixbufSimpleAnimIterClass))


GType mod_pixbuf_simple_anim_iter_get_type (void) G_GNUC_CONST;


struct _ModPixbufSimpleAnimIterClass
{
        GdkPixbufAnimationIterClass parent_class;
};

struct _ModPixbufSimpleAnimIter
{
        GdkPixbufAnimationIter parent_instance;
        
        ModPixbufSimpleAnim *simple_anim;
        
        GTimeVal start_time;
        GTimeVal current_time;
        
        gint position;
        
        GList *current_frame;
};

typedef struct _ModPixbufFrame ModPixbufFrame;
struct _ModPixbufFrame
{
        GdkPixbuf *pixbuf;
        gint delay_time;
        gint elapsed;
};

static void mod_pixbuf_simple_anim_finalize (GObject *object);

static gboolean   is_static_image  (GdkPixbufAnimation *animation);
static GdkPixbuf *get_static_image (GdkPixbufAnimation *animation);

static void       get_size         (GdkPixbufAnimation *anim,
                                    gint               *width, 
                                    gint               *height);
static GdkPixbufAnimationIter *get_iter (GdkPixbufAnimation *anim,
                                         const GTimeVal     *start_time);


G_DEFINE_TYPE (ModPixbufSimpleAnim, mod_pixbuf_simple_anim, GDK_TYPE_PIXBUF_ANIMATION)

static void
mod_pixbuf_simple_anim_init (ModPixbufSimpleAnim *anim)
{
}

static void
mod_pixbuf_simple_anim_class_init (ModPixbufSimpleAnimClass *klass)
{
        GObjectClass *object_class;
        GdkPixbufAnimationClass *anim_class;

        object_class = G_OBJECT_CLASS (klass);
        anim_class = GDK_PIXBUF_ANIMATION_CLASS (klass);
        
        object_class->finalize = mod_pixbuf_simple_anim_finalize;
        
        anim_class->is_static_image = is_static_image;
        anim_class->get_static_image = get_static_image;
        anim_class->get_size = get_size;
        anim_class->get_iter = get_iter;
}

static void
mod_pixbuf_simple_anim_finalize (GObject *object)
{
        ModPixbufSimpleAnim *anim;
        GList *l;
        ModPixbufFrame *frame;
        
        anim = MOD_PIXBUF_SIMPLE_ANIM (object);        
        
        for (l = anim->frames; l; l = l->next) {
                frame = l->data;
                g_object_unref (frame->pixbuf);
                g_free (frame);
        }
        
        g_list_free (anim->frames);
        
        G_OBJECT_CLASS (mod_pixbuf_simple_anim_parent_class)->finalize (object);
}

static gboolean
is_static_image (GdkPixbufAnimation *animation)
{
        ModPixbufSimpleAnim *anim;
        
        anim = MOD_PIXBUF_SIMPLE_ANIM (animation);

        return (anim->frames != NULL && anim->frames->next == NULL);
}

static GdkPixbuf *
get_static_image (GdkPixbufAnimation *animation)
{
        ModPixbufSimpleAnim *anim;
        
        anim = MOD_PIXBUF_SIMPLE_ANIM (animation);
        
        if (anim->frames == NULL)
                return NULL;
        else
                return ((ModPixbufFrame *)anim->frames->data)->pixbuf;
}

static void
get_size (GdkPixbufAnimation *animation,
          gint               *width, 
          gint               *height)
{
        ModPixbufSimpleAnim *anim;

        anim = MOD_PIXBUF_SIMPLE_ANIM (animation);
        
        if (width)
                *width = anim->width;
        
        if (height)
                *height = anim->height;
}

static void
iter_clear (ModPixbufSimpleAnimIter *iter)
{
        iter->current_frame = NULL;
}

static void
iter_restart (ModPixbufSimpleAnimIter *iter)
{
        iter_clear (iter);
        
        iter->current_frame = iter->simple_anim->frames;
}

static GdkPixbufAnimationIter *
get_iter (GdkPixbufAnimation *anim,
          const GTimeVal    *start_time)
{
        ModPixbufSimpleAnimIter *iter;
        
        iter = g_object_new (MOD_TYPE_PIXBUF_SIMPLE_ANIM_ITER, NULL);

        iter->simple_anim = MOD_PIXBUF_SIMPLE_ANIM (anim);

        g_object_ref (iter->simple_anim);
        
        iter_restart (iter);
        
        iter->start_time = *start_time;
        iter->current_time = *start_time;
        
        return GDK_PIXBUF_ANIMATION_ITER (iter);
}

static void mod_pixbuf_simple_anim_iter_finalize (GObject *object);

static gint       get_delay_time             (GdkPixbufAnimationIter *iter);
static GdkPixbuf *get_pixbuf                 (GdkPixbufAnimationIter *iter);
static gboolean   on_currently_loading_frame (GdkPixbufAnimationIter *iter);
static gboolean   advance                    (GdkPixbufAnimationIter *iter,
                                              const GTimeVal         *current_time);

G_DEFINE_TYPE (ModPixbufSimpleAnimIter, mod_pixbuf_simple_anim_iter, GDK_TYPE_PIXBUF_ANIMATION_ITER)

static void
mod_pixbuf_simple_anim_iter_init (ModPixbufSimpleAnimIter *iter)
{
}

static void
mod_pixbuf_simple_anim_iter_class_init (ModPixbufSimpleAnimIterClass *klass)
{
        GObjectClass *object_class;
        GdkPixbufAnimationIterClass *anim_iter_class;

        object_class = G_OBJECT_CLASS (klass);
        anim_iter_class = GDK_PIXBUF_ANIMATION_ITER_CLASS (klass);
        
        object_class->finalize = mod_pixbuf_simple_anim_iter_finalize;
        
        anim_iter_class->get_delay_time = get_delay_time;
        anim_iter_class->get_pixbuf = get_pixbuf;
        anim_iter_class->on_currently_loading_frame = on_currently_loading_frame;
        anim_iter_class->advance = advance;
}

static void
mod_pixbuf_simple_anim_iter_finalize (GObject *object)
{
        ModPixbufSimpleAnimIter *iter;
        
        iter = MOD_PIXBUF_SIMPLE_ANIM_ITER (object);
        iter_clear (iter);
        
        g_object_unref (iter->simple_anim);
        
        G_OBJECT_CLASS (mod_pixbuf_simple_anim_iter_parent_class)->finalize (object);
}

static gboolean
advance (GdkPixbufAnimationIter *anim_iter,
         const GTimeVal         *current_time)
{
        ModPixbufSimpleAnimIter *iter;
        gint elapsed;
        gint loop;
        GList *tmp;
        GList *old;
        
        iter = MOD_PIXBUF_SIMPLE_ANIM_ITER (anim_iter);
        
        iter->current_time = *current_time;
        
        /* We use milliseconds for all times */
        elapsed = (((iter->current_time.tv_sec - iter->start_time.tv_sec) * G_USEC_PER_SEC +
                    iter->current_time.tv_usec - iter->start_time.tv_usec)) / 1000;
        
        if (elapsed < 0) {
                /* Try to compensate; probably the system clock
                 * was set backwards
                 */
                iter->start_time = iter->current_time;
                elapsed = 0;
        }
        
        g_assert (iter->simple_anim->total_time > 0);
        
        /* See how many times we've already played the full animation,
         * and subtract time for that.
         */
        loop = elapsed / iter->simple_anim->total_time;
        elapsed = elapsed % iter->simple_anim->total_time;
        
        iter->position = elapsed;

#if 0
        fprintf(stderr, "loop=%d, elapsed=%d\n", loop, elapsed);
#endif
        
        /* Now move to the proper frame */
        if (loop < 1 || iter->simple_anim->loop)
                tmp = iter->simple_anim->frames;
        else
                tmp = NULL;
        
        while (tmp != NULL) {
                ModPixbufFrame *frame = tmp->data;

#if 0                
                if (iter->position >= frame->elapsed &&
                    iter->position < (frame->elapsed + frame->delay_time))
                        break;
#else
                if (iter->position > frame->elapsed &&
                    iter->position <= (frame->elapsed + frame->delay_time))
                        break;
#endif
                
                tmp = tmp->next;
        }
        
        old = iter->current_frame;
        
        iter->current_frame = tmp;

        if (tmp == NULL && iter->simple_anim->loop && loop >= 1) {
                iter_restart(iter);
                iter->position = ((ModPixbufFrame *)iter->current_frame->data)->elapsed;
        }
        
        return iter->current_frame != old;
}

static gint
get_delay_time (GdkPixbufAnimationIter *anim_iter)
{
        ModPixbufFrame *frame;
        ModPixbufSimpleAnimIter *iter;

        iter = MOD_PIXBUF_SIMPLE_ANIM_ITER (anim_iter);
        
        if (iter->current_frame) {
                frame = iter->current_frame->data;
#if 0
                fprintf(stderr, "delay: delay=%d, position=%d, elapsed=%d\n",
                        frame->delay_time - (iter->position - frame->elapsed),
                        iter->position, frame->elapsed);
#endif
                return frame->delay_time - (iter->position - frame->elapsed);
        }
        else {
                return -1;		/* show last frame forever */
        }
}

static GdkPixbuf *
get_pixbuf (GdkPixbufAnimationIter *anim_iter)
{
        ModPixbufSimpleAnimIter *iter;
        ModPixbufFrame *frame;
        
        iter = MOD_PIXBUF_SIMPLE_ANIM_ITER (anim_iter);
        
        if (iter->current_frame)
                frame = iter->current_frame->data;
        else if (g_list_length (iter->simple_anim->frames) > 0)
                frame = g_list_last (iter->simple_anim->frames)->data;
        else
                frame = NULL;

        if (frame == NULL)
                return NULL;
        
        return frame->pixbuf;
}

static gboolean
on_currently_loading_frame (GdkPixbufAnimationIter *anim_iter)
{
  ModPixbufSimpleAnimIter *iter;

  iter = MOD_PIXBUF_SIMPLE_ANIM_ITER (anim_iter);

  return iter->current_frame == NULL || iter->current_frame->next == NULL;
}

/**
 * mod_pixbuf_simple_anim_new:
 * @width: the width of the animation
 * @height: the height of the animation
 * @rate: the speed of the animation, in frames per second
 *
 * Creates a new, empty animation.
 *
 * Returns: a newly allocated #ModPixbufSimpleAnim 
 *
 * Since: 2.8
 */
ModPixbufSimpleAnim *
mod_pixbuf_simple_anim_new (gint   width, 
                            gint   height, 
                            gfloat rate)
{
  ModPixbufSimpleAnim *anim;

  anim = g_object_new (MOD_TYPE_PIXBUF_SIMPLE_ANIM, NULL);
  anim->width = width;
  anim->height = height;
  anim->rate = rate;
  anim->n_frames = 0;

  return anim;
}

/**
 * mod_pixbuf_simple_anim_add_frame:
 * @animation: a #ModPixbufSimpleAnim
 * @pixbuf: the pixbuf to add 
 *
 * Adds a new frame to @animation. The @pixbuf must
 * have the dimensions specified when the animation 
 * was constructed.
 *
 * Since: 2.8
 */
void
mod_pixbuf_simple_anim_add_frame (ModPixbufSimpleAnim *animation,
                                  GdkPixbuf           *pixbuf)
{
  ModPixbufFrame *frame;
  
  g_return_if_fail (animation != NULL);
  g_return_if_fail (pixbuf != NULL);
  
  frame = g_new0 (ModPixbufFrame, 1);
  frame->delay_time = (gint) (1000 / animation->rate);
  frame->elapsed = (gint) (frame->delay_time * animation->n_frames++);
  animation->total_time += frame->delay_time;
  frame->pixbuf = GDK_PIXBUF (g_object_ref (pixbuf));

  animation->frames = g_list_append (animation->frames, frame);
}


void
mod_pixbuf_simple_anim_set_loop (ModPixbufSimpleAnim *animation,
                                 gboolean value)
{
        g_return_if_fail (MOD_IS_PIXBUF_SIMPLE_ANIM(animation));
        animation->loop = value;
}


gboolean
mod_pixbuf_simple_anim_get_loop (ModPixbufSimpleAnim *animation)
{
        g_return_val_if_fail (MOD_IS_PIXBUF_SIMPLE_ANIM(animation), FALSE);
        return animation->loop;
}
