/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bytecode_buffer.cc,v 1.10 2009/01/01 07:47:00 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/bytecode_buffer.h"

#include <cstdlib>
#include <cstring>


namespace LibTSCore
{

BytecodeBuffer::BytecodeBuffer()
  : buffer(NULL),
    buffer_length(0),
    offset(0),
    include_forward_branch(true)
{
}


BytecodeBuffer::~BytecodeBuffer()
{
  if (buffer != NULL)
    free(buffer);
}


void
BytecodeBuffer::optimize_bytecode()
{
  // Here we perform peephole optimization.

  // We can perform transformation of branch folding
  // i.e. branch to unconditional jump => direct branch to the final
  // offset at here.
  if (!include_forward_branch)
    return;

  // to be implemented!
  return;
}


void
BytecodeBuffer::enlarge_buffer(size_t capacity)
{
  size_t new_capacity;
  if (buffer == NULL)
    new_capacity = INITIAL_BYTECODE_BUFFER_CAPACITY;
  else
    new_capacity = buffer_length;

  while (new_capacity < capacity)
    new_capacity *= 2;

  char *old_buffer = buffer;
  buffer = static_cast<char *>(realloc(old_buffer, new_capacity));
  if (buffer != NULL)
    {
      buffer_length = new_capacity;
      return;
    }

  fprintf(stderr, "Cannot allocate enough room for a BytecodeBuffer\n");
  free(old_buffer);
  exit(EXIT_FAILURE);
}


size_t
BytecodeBuffer::append_unsigned(unsigned long value)
{
  if (value <= 0xff)
    return append_unsigned_8(value);

  if (value <= 0xffff)
    return append_unsigned_16(value);

#if SIZEOF_LONG > 4
  if (value <= 0xffffffff)
    return append_unsigned_32(value);

  return append_unsigned_64(value);
#else
  return append_unsigned_32(value);
#endif
}


size_t
BytecodeBuffer::append_signed(long value)
{
  if (value >= 0)
    {
      if (value <= 0x7f)
	return append_signed_8(value);

      if (value <= 0x7fff)
	return append_signed_16(value);

#if SIZEOF_LONG > 4
      if (value <= 0x7fffffff)
	return append_signed_32(value);

      return append_signed_64(value);
#else
      return append_signed_32(value);
#endif
    }

  if (value >= -128)
    return append_signed_8(value);

  if (value >= -32768)
    return append_signed_16(value);

#if SIZEOF_LONG > 4
  if (value >= -2147483648)
    return append_signed_32(value);

  return append_signed_64(value);
#else
  return append_signed_32(value);
#endif
}

}
