/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: macro_syntax.h,v 1.12 2009/01/01 07:47:00 fuyu Exp $
 */

#ifdef _SYNTAX_DEFINED_
#ifndef _LIBTSCORE_MACRO_SYNTAX_H_
#define _LIBTSCORE_MACRO_SYNTAX_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/syntax.h"

#include <vector>


using namespace std;
namespace LibTSCore
{

  class MacroSyntax : public Syntax
  {
  private:
    static bool match_list_pattern(VirtualMachine &vm,
				   Cell *expression, Cell *pattern,
				   vector<const Cell *> &valid_literals,
				   vector<const Cell *> &invalid_literals);
    static bool match_vector_pattern(VirtualMachine &vm,
				     Cell *expression, Cell *pattern,
				     vector<const Cell *> &valid_literals,
				     vector<const Cell *> &invalid_literals);

    static bool match_pattern(VirtualMachine &vm,
			      Cell *expression, Cell *pattern,
			      vector<const Cell *> &valid_literals,
			      vector<const Cell *> &invalid_literals);
    static Cell *match_syntax_rules(VirtualMachine &vm,
				    Cell *expression, Cell *environment,
				    const Cell *transformer);

    static const Cell *match_sub_form(VirtualMachine &vm,
				      const Cell *expression,
				      const Cell *pattern,
				      const Cell *sub_pattern);

    static const Cell *match_ellipsis_form(VirtualMachine &vm,
					   const Cell *expression,
					   const Cell *pattern,
					   const Cell *sub_pattern);

    static Cell *transform_ellipsis(VirtualMachine &vm,
				    const Cell *expression,
				    Cell *&literals, Cell *&template_alist,
				    const Cell *pattern, const Cell *templ,
				    Cell *&environment,
				    Cell *&macro_environment);
    static Cell *transform_list(VirtualMachine &vm,
				const Cell *expression,
				Cell *&literals, Cell *&template_alist,
				const Cell *pattern, const Cell *templ,
				Cell *&environment,
				Cell *&macro_environment);
    static Cell *transform_vector(VirtualMachine &vm,
				  const Cell *expression,
				  Cell *&literals, Cell *&template_alist,
				  const Cell *pattern, const Cell *templ,
				  Cell *&environment,
				  Cell *&macro_environment);
    static Cell *transform_expression(VirtualMachine &vm,
				      const Cell *expression,
				      Cell *&literals, Cell *&template_alist,
				      const Cell *pattern, const Cell *templ,
				      Cell *&environment,
				      Cell *&macro_environment);

    static Cell *expand_macro(VirtualMachine &vm,
			      Cell *expression,
			      Cell *&literals, const Cell *syntax_rule,
			      Cell *&environment,
			      Cell *&macro_environment);


  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    MacroSyntax() {}
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_MACRO_SYNTAX_H_ */

#endif	/* _SYNTAX_DEFINED_ */
