/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: number.cc,v 1.13 2009/01/01 07:47:00 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/number.h"

#include "libtscore/cell.h"


namespace LibTSCore
{


Number::Number(Cell *cell)
{
  fixnum = cell->is_integer();
  if (fixnum)
    value.ivalue = cell->get_integer0();
  else
    value.rvalue = cell->get_real();
}


Number &
Number::operator=(Cell *cell)
{
  fixnum = cell->is_integer();
  if (fixnum)
    value.ivalue = cell->get_integer0();
  else
    value.rvalue = cell->get_real();

  return *this;
}


void
Number::add(Cell *cell)
{
  bool new_fixnum = fixnum & cell->is_integer();
  if (new_fixnum)
    value.ivalue += cell->get_integer0();
  else
    value.rvalue = get_rvalue() + cell->get_real();

  fixnum = new_fixnum;
}


void
Number::mul(Cell *cell)
{
  bool new_fixnum = fixnum & cell->is_integer();
  if (new_fixnum)
    value.ivalue *= cell->get_integer0();
  else
    value.rvalue = get_rvalue() * cell->get_real();

  fixnum = new_fixnum;
}


void
Number::div(Cell *cell)
{
  bool new_fixnum = fixnum && cell->is_integer() && cell->get_integer0() != 0;
  if (new_fixnum)
    value.ivalue /= cell->get_integer0();
  else
    value.rvalue = get_rvalue() / cell->get_real();

  fixnum = new_fixnum;
}


void
Number::intdiv(Cell *cell)
{
  bool new_fixnum = fixnum & cell->is_integer();
  if (new_fixnum)
    value.ivalue /= cell->get_integer0();
  else
    value.rvalue = get_rvalue() / cell->get_real();

  fixnum = new_fixnum;
}


void
Number::sub(Cell *cell)
{
  bool new_fixnum = fixnum & cell->is_integer();
  if (new_fixnum)
    value.ivalue -= cell->get_integer0();
  else
    value.rvalue = get_rvalue() - cell->get_real();

  fixnum = new_fixnum;
}


void
Number::rem(Cell *cell)
{
  bool new_fixnum = fixnum & cell->is_integer();
  long e1 = get_ivalue();
  long e2 = cell->get_integer();
  long res = e1 % e2;
  if (res * e1 < 0)
    {
      e2 = labs(e2);
      if (res > 0)
	res -= e2;
      else
	res += e2;
    }

  if (new_fixnum)
    value.ivalue = res;
  else
    value.rvalue = static_cast<double>(res);

  fixnum = new_fixnum;
}


void
Number::mod(Cell *cell)
{
  bool new_fixnum = fixnum & cell->is_integer();
  long e1 = get_ivalue();
  long e2 = cell->get_integer();
  long res = e1 % e2;
  if (res * e2 < 0)
    {
      e2 = labs(e2);
      if (res > 0)
	res -= e2;
      else
	res += e2;
    }

  if (new_fixnum)
    value.ivalue = res;
  else
    value.rvalue = static_cast<double>(res);

  fixnum = new_fixnum;
}


bool
Number::eq(Cell *cell)
{
  if (fixnum && cell->is_integer())
    return value.ivalue == cell->get_integer0();

  return get_rvalue() == cell->get_real();
}


bool
Number::gt(Cell *cell)
{
  if (fixnum && cell->is_integer())
    return value.ivalue > cell->get_integer0();

  return get_rvalue() > cell->get_real();
}


bool
Number::lt(Cell *cell)
{
  if (fixnum && cell->is_integer())
    return value.ivalue < cell->get_integer0();

  return get_rvalue() < cell->get_real();
}


}
