/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_char_procedure.cc,v 1.12 2009/01/01 07:47:00 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/r5rs_char_procedure.h"

#include "libtscore/tscore.h"

#include <cctype>


namespace LibTSCore
{

NativeProcedure::Info R5RSCharProcedure::procedures[] = {
  { &R5RSCharProcedure::is_char,
    "char?", TST_NONE, 1, 1 },
  { &R5RSCharProcedure::is_char_eq,
    "char=?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_lt,
    "char<?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_gt,
    "char>?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_le,
    "char<=?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_ge,
    "char>=?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_ci_eq,
    "char-ci=?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_ci_lt,
    "char-ci<?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_ci_gt,
    "char-ci>?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_ci_le,
    "char-ci<=?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_ci_ge,
    "char-ci>=?", TST_CHAR, 2, 2 },
  { &R5RSCharProcedure::is_char_alphabetic,
    "char-alphabetic?", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::is_char_numeric,
    "char-numeric?", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::is_char_whitespace,
    "char-whitespace?", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::is_char_upper_case,
    "char-upper-case?", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::is_char_lower_case,
    "char-lower-case?", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::char2integer,
    "char->integer", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::integer2char,
    "integer->char", TST_INTEGER, 1, 1 },
  { &R5RSCharProcedure::char_upcase,
    "char-upcase", TST_CHAR, 1, 1 },
  { &R5RSCharProcedure::char_downcase,
    "char-downcase", TST_CHAR, 1, 1 }
};


Cell *
R5RSCharProcedure::assign_procedures(Cell *environment, Cell *frame)
{
  TSStackRoot env(core, environment);
  TSStackRoot frm(core, frame);
  size_t n_procs = sizeof(procedures) / sizeof(Info);
  for (size_t i = 0; i < n_procs; i++)
    assign_procedure(env, frm, &procedures[i], this);

  return env;
}


#define ARG(n)	context.frame->load_variable(argument_pointer + n)


Cell *
R5RSCharProcedure::is_char(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
  return Cell::ts_bool(ARG(0)->is_character());
}


Cell *
R5RSCharProcedure::is_char_eq(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
  // we know characters are represented as pseudo Cell!
  return Cell::ts_bool(ARG(0) == ARG(1));
}


Cell *
R5RSCharProcedure::is_char_lt(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
  // we know characters are represented as pseudo Cell!
  return Cell::ts_bool(ARG(0) < ARG(1));
}


Cell *
R5RSCharProcedure::is_char_gt(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
  // we know characters are represented as pseudo Cell!
  return Cell::ts_bool(ARG(0) > ARG(1));
}


Cell *
R5RSCharProcedure::is_char_le(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
  // we know characters are represented as pseudo Cell!
  return Cell::ts_bool(ARG(0) <= ARG(1));
}


Cell *
R5RSCharProcedure::is_char_ge(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
  // we know characters are represented as pseudo Cell!
  return Cell::ts_bool(ARG(0) >= ARG(1));
}


Cell *
R5RSCharProcedure::is_char_ci_eq(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  int cl = toupper(static_cast<int>(ARG(0)->get_character()));
  int cr = toupper(static_cast<int>(ARG(1)->get_character()));
  return Cell::ts_bool(cl == cr);
}


Cell *
R5RSCharProcedure::is_char_ci_lt(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  int cl = toupper(static_cast<int>(ARG(0)->get_character()));
  int cr = toupper(static_cast<int>(ARG(1)->get_character()));
  return Cell::ts_bool(cl < cr);
}


Cell *
R5RSCharProcedure::is_char_ci_gt(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  int cl = toupper(static_cast<int>(ARG(0)->get_character()));
  int cr = toupper(static_cast<int>(ARG(1)->get_character()));
  return Cell::ts_bool(cl > cr);
}


Cell *
R5RSCharProcedure::is_char_ci_le(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  int cl = toupper(static_cast<int>(ARG(0)->get_character()));
  int cr = toupper(static_cast<int>(ARG(1)->get_character()));
  return Cell::ts_bool(cl <= cr);
}


Cell *
R5RSCharProcedure::is_char_ci_ge(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  int cl = toupper(static_cast<int>(ARG(0)->get_character()));
  int cr = toupper(static_cast<int>(ARG(1)->get_character()));
  return Cell::ts_bool(cl >= cr);
}


Cell *
R5RSCharProcedure::is_char_alphabetic(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  return Cell::ts_bool(isalpha(static_cast<int>(ARG(0)->get_character())) != 0);
}


Cell *
R5RSCharProcedure::is_char_numeric(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data)
{
  return Cell::ts_bool(isdigit(static_cast<int>(ARG(0)->get_character())) != 0);
}


Cell *
R5RSCharProcedure::is_char_whitespace(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  return Cell::ts_bool(isspace(static_cast<int>(ARG(0)->get_character())) != 0);
}


Cell *
R5RSCharProcedure::is_char_upper_case(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  return Cell::ts_bool(isupper(static_cast<int>(ARG(0)->get_character())) != 0);
}


Cell *
R5RSCharProcedure::is_char_lower_case(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  return Cell::ts_bool(islower(static_cast<int>(ARG(0)->get_character())) != 0);
}


Cell *
R5RSCharProcedure::char2integer(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data)
{
  return vm.mk_integer(ARG(0)->get_character());
}


Cell *
R5RSCharProcedure::integer2char(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data)
{
  return vm.mk_character(ARG(0)->get_integer0());
}


Cell *
R5RSCharProcedure::char_upcase(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
  return vm.mk_character(toupper(static_cast<int>(ARG(0)->get_character())));
}


Cell *
R5RSCharProcedure::char_downcase(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  return vm.mk_character(tolower(static_cast<int>(ARG(0)->get_character())));
}

}
