/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_char_procedure.h,v 1.12 2009/01/01 07:47:00 fuyu Exp $
 */

#ifndef _LIBTSCORE_R5RS_CHAR_PROCEDURE_H_
#define _LIBTSCORE_R5RS_CHAR_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{
  /*
   * R5RSCharProcedure implements character related procedures defined in R5RS.
   */
  class R5RSCharProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];


  public:
    R5RSCharProcedure(TSCore &tscore) : NativeProcedure(tscore) {}
    Cell *assign_procedures(Cell *environment, Cell *frame);


    // 6.3.4 Characters
    static Cell *is_char(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *is_char_eq(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_char_lt(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_char_gt(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_char_le(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_char_ge(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_char_ci_eq(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_char_ci_lt(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_char_ci_gt(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_char_ci_le(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_char_ci_ge(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_char_alphabetic(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *is_char_numeric(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *is_char_whitespace(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *is_char_upper_case(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *is_char_lower_case(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *char2integer(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *integer2char(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *char_upcase(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *char_downcase(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
  };
}

#endif

#endif	/* _LIBTSCORE_R5RS_CHAR_PROCEDURE_H_ */
