/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_derived_syntax.h,v 1.13 2009/01/01 07:47:00 fuyu Exp $
 */

#ifdef _SYNTAX_DEFINED_
#ifndef _LIBTSCORE_R5RS_DERIVED_SYNTAX_H_
#define _LIBTSCORE_R5RS_DERIVED_SYNTAX_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/syntax.h"


using namespace std;
namespace LibTSCore
{

  class And : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    And() {}
  };


  class Or : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Or() {}
  };


  class LetAsterisk : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    LetAsterisk() {}
  };


  class Letrec : public Syntax
  {
    friend class LetAsterisk;
    friend class LetSyntax;
    friend class LetrecSyntax;
    friend class Syntax;

  private:
    static size_t emit_letrec(VirtualMachine &vm, Register &context,
			      BytecodeBuffer *buffer,
			      Cell *expression, Cell *environment,
			      Cell *constant_pool, Cell *hint,
			      Cell *letrec_bindings, Cell *letrec_body,
			      bool is_tail_context);
    static bool check_bindings(VirtualMachine &vm, Cell *bindings,
			       Cell *hint);


  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Letrec() {}
  };


  class Begin : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Begin() {}
  };


  class Delay : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Delay() {}
  };


  class Quasiquote : public Syntax
  {
    friend class Syntax;
    friend class Unquote;
    friend class UnquoteSplicing;

  private:
    static bool find_unquote(VirtualMachine &vm, Register &context,
			     Cell *environment,
			     Cell *expression, long nesting);


  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);

    size_t emit_qq_template(VirtualMachine &vm, Register &context,
			    BytecodeBuffer *buffer,
			    Cell *qq_template, long nesting,
			    Cell *environment, Cell *constant_pool,
			    Cell *hint, bool list_qq);

    bool is_quasiquote_syntax();


  public:
    Quasiquote() {}
  };


  // Syntaxes below are `internal' syntaxes, not bound to toplevel!
  class Unquote : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);

    size_t emit_qq_template(VirtualMachine &vm, Register &context,
			    BytecodeBuffer *buffer,
			    Cell *qq_template, long nesting,
			    Cell *environment, Cell *constant_pool,
			    Cell *hint, bool list_qq);

    bool is_quasiquote_syntax();


  public:
    Unquote() {}
  };


  class UnquoteSplicing : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);

    size_t emit_qq_template(VirtualMachine &vm, Register &context,
			    BytecodeBuffer *buffer,
			    Cell *qq_template, long nesting,
			    Cell *environment, Cell *constant_pool,
			    Cell *hint, bool list_qq);

    bool is_quasiquote_syntax();


  public:
    UnquoteSplicing() {}
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_R5RS_DERIVED_SYNTAX_H_ */
#endif	/* _SYNTAX_DEFINED_ */
