/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_io_procedure.cc,v 1.16 2009/01/01 07:47:00 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/r5rs_io_procedure.h"

#include "libtscore/tscore.h"
#include "libtscore/virtual_machine.h"
#include "libtscore/writer.h"


namespace LibTSCore
{

NativeProcedure::Info R5RSIOProcedure::procedures[] = {
  { &R5RSIOProcedure::call_with_input_file,
    "call-with-input-file", TST_STRING TST_PROCEDURE, 2, 2 },
  { &R5RSIOProcedure::call_with_output_file,
    "call-with-output-file", TST_STRING TST_PROCEDURE, 2, 2 },
  { &R5RSIOProcedure::is_input_port,
    "input-port?", TST_ANY, 1, 1 },
  { &R5RSIOProcedure::is_output_port,
    "output-port?", TST_ANY, 1, 1 },
  { &R5RSIOProcedure::current_input_port,
    "current-input-port", TST_NONE, 0, 0 },
  { &R5RSIOProcedure::current_output_port,
    "current-output-port", TST_NONE, 0, 0 },
  { &R5RSIOProcedure::with_input_from_file,
    "with-input-from-file", TST_STRING TST_PROCEDURE, 2, 2 },
  { &R5RSIOProcedure::with_output_to_file,
    "with-output-to-file", TST_STRING TST_PROCEDURE, 2, 2 },
  { &R5RSIOProcedure::open_input_file,
    "open-input-file", TST_STRING, 1, 1 },
  { &R5RSIOProcedure::open_output_file,
    "open-output-file", TST_STRING, 1, 1 },
  { &R5RSIOProcedure::close_input_port,
    "close-input-port", TST_INPORT, 1, 1 },
  { &R5RSIOProcedure::close_output_port,
    "close-output-port", TST_OUTPORT, 1, 1 },
  { &R5RSIOProcedure::read,
    "read", TST_INPORT, 0, 1 },
  { &R5RSIOProcedure::read_char,
    "read-char", TST_INPORT, 0, 1 },
  { &R5RSIOProcedure::peek_char,
    "peek-char", TST_INPORT, 0, 1 },
  { &R5RSIOProcedure::is_eof_object,
    "eof-object?", TST_ANY, 1, 1 },
  { &R5RSIOProcedure::is_char_ready,
    "char-ready?", TST_INPORT, 0, 1 },
  { &R5RSIOProcedure::write,
    "write", TST_ANY TST_OUTPORT, 1, 2 },
  { &R5RSIOProcedure::display,
    "display", TST_ANY TST_OUTPORT, 1, 2 },
  { &R5RSIOProcedure::newline,
    "newline", TST_OUTPORT, 0, 1 },
  { &R5RSIOProcedure::write_char,
    "write-char", TST_CHAR TST_OUTPORT, 1, 2 },
  { &R5RSIOProcedure::load,
    "load", TST_STRING, 1, 1 },
  { &R5RSIOProcedure::transcript_on,
    "transcript-on", TST_STRING, 1, 1 },
  { &R5RSIOProcedure::transcript_off,
    "transcript-off", TST_NONE, 0, 0 }
};


R5RSIOProcedure::~R5RSIOProcedure()
{
  if (transcript_port != NULL)
    delete transcript_port;
}


Cell *
R5RSIOProcedure::assign_procedures(Cell *environment, Cell *frame)
{
  TSStackRoot env(core, environment);
  TSStackRoot frm(core, frame);
  size_t n_procs = sizeof(procedures) / sizeof(Info);
  for (size_t i = 0; i < n_procs; i++)
    assign_procedure(env, frm, &procedures[i], this);

  return env;
}


#define ARG(n)	context.frame->load_variable(argument_pointer + n)


Cell *
R5RSIOProcedure::call_with_input_file(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
#warning "Implement this!: call-with-input-file"
  return signal_error(vm, "call-with-input-file: not implemented yet");
}


Cell *
R5RSIOProcedure::call_with_output_file(VirtualMachine &vm, Register &context,
				       unsigned long argument_pointer,
				       unsigned long nargs, void *data)
{
#warning "Implement this!: call-with-output-file"
  return signal_error(vm, "call-with-output-file: not implemented yet");
}


Cell *
R5RSIOProcedure::is_input_port(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
  return Cell::ts_bool(ARG(0)->is_input_port());
}


Cell *
R5RSIOProcedure::is_output_port(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data)
{
  return Cell::ts_bool(ARG(0)->is_output_port());
}


Cell *
R5RSIOProcedure::current_input_port(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data)
{
  return context.input_port;
}


Cell *
R5RSIOProcedure::current_output_port(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data)
{
  return context.output_port;
}


Cell *
R5RSIOProcedure::with_input_from_file(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
#warning "Implement this!: with-input-from-file"
  return signal_error(vm, "with-input-from-file: not implemented yet");
}


Cell *
R5RSIOProcedure::with_output_to_file(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data)
{
#warning "Implement this!: with-output-to-file"
  return signal_error(vm, "with-output-to-file: not implemented yet");
}


Cell *
R5RSIOProcedure::open_input_file(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  Cell *port = vm.mk_input_file_port(ARG(0)->get_string());
  if (port != Cell::nil())
    return port;

  return Cell::f();
}


Cell *
R5RSIOProcedure::open_output_file(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
  Cell *port = vm.mk_output_file_port(ARG(0)->get_string());
  if (port != Cell::nil())
    return port;

  return Cell::f();
}


Cell *
R5RSIOProcedure::close_input_port(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
  ARG(0)->close_port();
  return Cell::unspecified();
}


Cell *
R5RSIOProcedure::close_output_port(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data)
{
  ARG(0)->close_port();
  return Cell::unspecified();
}


Cell *
R5RSIOProcedure::read(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data)
{
  Port *input_port;
  if (nargs == 0)
    input_port = context.input_port->get_port();
  else
    input_port = ARG(0)->get_port();

  Cell *datum = vm.get_reader()->read_datum(input_port);
  if (datum != NULL)
    return datum;

  return signal_error(vm, vm.get_reader()->get_error_message());
}


Cell *
R5RSIOProcedure::read_char(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
  Port *input_port;
  if (nargs == 0)
    input_port = context.input_port->get_port();
  else
    input_port = ARG(0)->get_port();

  long c = input_port->get_char();
  if (c == EOF)
    return Cell::eof();

  return vm.mk_character(c);
}


Cell *
R5RSIOProcedure::peek_char(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
  Port *input_port;
  if (nargs == 0)
    input_port = context.input_port->get_port();
  else
    input_port = ARG(0)->get_port();

  long c = input_port->get_char();
  if (c == EOF)
    return Cell::eof();

  input_port->unget_char(c);
  return vm.mk_character(c);
}


Cell *
R5RSIOProcedure::is_eof_object(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
  return Cell::ts_bool(ARG(0) == Cell::eof());
}


Cell *
R5RSIOProcedure::is_char_ready(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
  Port *input_port;
  if (nargs == 0)
    input_port = context.input_port->get_port();
  else
    input_port = ARG(0)->get_port();

  return Cell::ts_bool(input_port->char_ready());
}


Cell *
R5RSIOProcedure::write(VirtualMachine &vm, Register &context,
		       unsigned long argument_pointer,
		       unsigned long nargs, void *data)
{
  Port *output_port;
  if (nargs == 1)
    output_port = context.output_port->get_port();
  else
    output_port = ARG(1)->get_port();

  const Cell *result = Writer::write_datum(ARG(0), output_port, true);
  if (result == Cell::nil())
    return Cell::unspecified();

  return signal_error(vm, "write: cannot represent in external form:", result);
}


Cell *
R5RSIOProcedure::display(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data)
{
  Port *output_port;
  if (nargs == 1)
    output_port = context.output_port->get_port();
  else
    output_port = ARG(1)->get_port();

#ifndef NDEBUG
  const Cell *result = Writer::write_datum(ARG(0), output_port, false);
  assert(result == Cell::nil());
#else
  Writer::write_datum(ARG(0), output_port, false);
#endif
  return Cell::unspecified();
}


Cell *
R5RSIOProcedure::newline(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data)
{
  Port *output_port;
  if (nargs == 0)
    output_port = context.output_port->get_port();
  else
    output_port = ARG(0)->get_port();

  output_port->append('\n');
  return Cell::unspecified();
}


Cell *
R5RSIOProcedure::write_char(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
  Port *output_port;
  if (nargs == 1)
    output_port = context.output_port->get_port();
  else
    output_port = ARG(1)->get_port();

  Cell *character = ARG(0);
  output_port->append(character->get_character());
  return Cell::unspecified();
}


Cell *
R5RSIOProcedure::load(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data)
{
  /*
   * XXX: This implementation isn't properly tail-recursive.
   */
  Cell *port = vm.mk_input_file_port(ARG(0)->get_string());
  if (port == Cell::nil())
    return signal_error(vm, "load: file not found: ", ARG(0));

  bool result = vm.read_eval_print(port, Cell::nil(), NULL);
  if (port->is_input_port())
    port->get_port()->close();

  if (result)
    return Cell::unspecified();

  char message[512];
  snprintf(message, 512, "load: Error: %s",
	   vm.get_error_buffer()->get_output_string());

  return signal_error(vm, message);
}


Cell *
R5RSIOProcedure::transcript_on(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
  OutputStringPort string_buf;
  Cell *name = ARG(0);
  string_buf.append(name->get_string(), name->get_strlen());
  FILE *file = fopen(string_buf.get_output_string(), "w");
  if (file == NULL)
    return signal_error(vm, "transcript-on: cannot open file:", name);

  R5RSIOProcedure *proc = static_cast<R5RSIOProcedure *>(data);
  TranscriptPort *prev_port = proc->transcript_port;
  proc->transcript_port = new TranscriptPort(file);

  Cell *input_port = vm.core.get_interaction_input_port();
  if (input_port->is_input_port())
    input_port->get_port()->set_transcript_port(proc->transcript_port);

  Cell *output_port = vm.core.get_interaction_output_port();
  if (output_port->is_output_port())
    output_port->get_port()->set_transcript_port(proc->transcript_port);

  if (prev_port != NULL)
    delete prev_port;

  return Cell::unspecified();
}


Cell *
R5RSIOProcedure::transcript_off(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data)
{
  R5RSIOProcedure *proc = static_cast<R5RSIOProcedure *>(data);
  TranscriptPort *prev_port = proc->transcript_port;
  proc->transcript_port = NULL;

  Cell *input_port = vm.core.get_interaction_input_port();
  if (input_port->is_input_port())
    input_port->get_port()->set_transcript_port(NULL);

  Cell *output_port = vm.core.get_interaction_output_port();
  if (output_port->is_output_port())
    output_port->get_port()->set_transcript_port(NULL);

  if (prev_port != NULL)
    delete prev_port;

  return Cell::unspecified();
}

}
