/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_io_procedure.h,v 1.14 2009/01/01 07:47:00 fuyu Exp $
 */

#ifndef _LIBTSCORE_R5RS_IO_PROCEDURE_H_
#define _LIBTSCORE_R5RS_IO_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/port.h"
#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{
  /*
   * R5RSIOProcedure implements I/O related procedures defined in R5RS.
   */
  class R5RSIOProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];

    TranscriptPort *transcript_port;


  public:
    R5RSIOProcedure(TSCore &tscore)
      : NativeProcedure(tscore), transcript_port(NULL) {}
    ~R5RSIOProcedure();

    Cell *assign_procedures(Cell *environment, Cell *frame);


    // 6.6.1 Ports
    static Cell *call_with_input_file(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);
    static Cell *call_with_output_file(VirtualMachine &vm, Register &context,
				       unsigned long argument_pointer,
				       unsigned long nargs, void *data);
    static Cell *is_input_port(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_output_port(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);

    static Cell *current_input_port(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *current_output_port(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data);
    static Cell *with_input_from_file(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);
    static Cell *with_output_to_file(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data);
    static Cell *open_input_file(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *open_output_file(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *close_input_port(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *close_output_port(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data);

    // 6.6.2 Input
    static Cell *read(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *read_char(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *peek_char(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *is_eof_object(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *is_char_ready(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);

    // 6.6.3 Output
    static Cell *write(VirtualMachine &vm, Register &context,
		       unsigned long argument_pointer,
		       unsigned long nargs, void *data);
    static Cell *display(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *newline(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *write_char(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    // 6.6.4 System interface
    static Cell *load(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *transcript_on(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *transcript_off(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
  };


}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_R5RS_IO_PROCEDURE_H_ */
