/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_list_procedure.h,v 1.18 2009/01/01 07:47:00 fuyu Exp $
 */

#ifndef _LIBTSCORE_R5RS_LIST_PROCEDURE_H_
#define _LIBTSCORE_R5RS_LIST_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{

  class MemorySystem;
  struct BytecodeProcedureInfo;

  /*
   * R5RSListProcedure implements list/vector related procedures defined
   * in R5RS.
   */
  class R5RSListProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];

    void assign_bytecode_procedure(Cell *environment, Cell *frame,
				   const BytecodeProcedureInfo *info);


  public:
    R5RSListProcedure(TSCore &tscore) : NativeProcedure(tscore) {}
    Cell *assign_procedures(Cell *environment, Cell *frame);


    static bool is_list(Cell *cell);
    static long list_length(Cell *list);
    static Cell *list_append(MemorySystem &memory_system, Cell *a, Cell *b);
    static Cell *list_reverse(MemorySystem &memory_system, Cell *&list);
    static Cell *list_reverse_in_place(MemorySystem &memory_system,
				       Cell *term, Cell *list,
				       bool set_immutable);


    // 6.3.2 Pairs and lists
    static Cell *is_pair(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *is_null(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *is_list(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);

    static Cell *car(VirtualMachine &vm, Register &context,
		     unsigned long argument_pointer,
		     unsigned long nargs, void *data);
    static Cell *cdr(VirtualMachine &vm, Register &context,
		     unsigned long argument_pointer,
		     unsigned long nargs, void *data);
    static Cell *cons(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *set_car(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *set_cdr(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);

    static Cell *list(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *length(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *append(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *reverse(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *list_tail(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *list_ref(VirtualMachine &vm, Register &context,
			  unsigned long argument_pointer,
			  unsigned long nargs, void *data);

    static Cell *memq(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *memv(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *member(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);

    static Cell *assq(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *assv(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *assoc(VirtualMachine &vm, Register &context,
		       unsigned long argument_pointer,
		       unsigned long nargs, void *data);

    // 6.3.6 Vectors
    static Cell *is_vector(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *make_vector(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *vector(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *vector_length(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *vector_ref(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *vector_set(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *vector2list(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *list2vector(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *vector_fill(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
  };


}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_R5RS_LIST_PROCEDURE_H_ */
