/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_macro_syntax.h,v 1.11 2009/01/01 07:47:00 fuyu Exp $
 */

#ifdef _SYNTAX_DEFINED_
#ifndef _LIBTSCORE_R5RS_MACRO_SYNTAX_H_
#define _LIBTSCORE_R5RS_MACRO_SYNTAX_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/syntax.h"

#include <vector>


using namespace std;
namespace LibTSCore
{
  class DefineSyntax : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    DefineSyntax() {}
  };


  class LetSyntax : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    LetSyntax() {}
  };


  class LetrecSyntax : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    LetrecSyntax() {}
  };


  class SyntaxRules
  {
  private:
    static Cell *parse_pattern0(VirtualMachine &vm, Cell *pattern,
				Cell *&pattern_variables,
				Cell *&non_pattern_variables);
    static Cell *parse_pattern(VirtualMachine &vm, Cell *pattern,
			       Cell *keyword,
			       Cell *&pattern_variables,
			       Cell *&non_pattern_variables);

    static Cell *alist_cons(VirtualMachine &vm,
			    Cell *key, Cell *datum, Cell *alist);
    static Cell *find_inner_most_sub_pattern(VirtualMachine &vm,
					     Cell *templ, Cell *pattern,
					     bool with_ellipsis,
					     Cell *&template_alist);
    static Cell *find_common_sub_pattern(VirtualMachine &vm,
					 Cell *sub_pattern0,
					 Cell *sub_pattern1,
					 Cell *pattern);
    static Cell *parse_template_datum(VirtualMachine &vm,
				      Cell *templ);
    static Cell *parse_template(VirtualMachine &vm, Cell *templ,
				Cell *&template_alist,
				Cell *pattern, Cell *&pattern_variables);

    static Cell *parse_rule(VirtualMachine &vm, Cell *rule, Cell *keyword,
			    Cell *&non_pattern_variables);


  public:
    SyntaxRules() {}

    /*
     * Parses `expression' as a <transformer spec>.
     *
     * Returns a macro transformer if successfully parsed, otherwise
     * returns NULL.
     */
    static const Cell *parse(VirtualMachine &vm, Cell *keyword,
			     Cell *expression, Cell *environment);
  };
}

#endif

#endif	/* _LIBTSCORE_R5RS_MACRO_SYNTAX_H_ */
#endif	/* _SYNTAX_DEFINED_ */
