/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_math_procedure.h,v 1.12 2009/01/01 07:47:00 fuyu Exp $
 */

#ifndef _LIBTSCORE_R5RS_MATH_PROCEDURE_H_
#define _LIBTSCORE_R5RS_MATH_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{
  /*
   * R5RSMathProcedure implements numerics/mathematics related procedures
   * defined in R5RS.
   */
  class R5RSMathProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];


  public:
    R5RSMathProcedure(TSCore &tscore) : NativeProcedure(tscore) {}

    Cell *assign_procedures(Cell *environment, Cell *frame);


    // 6.2.5 Numerical operations
    static Cell *is_number(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *is_complex(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_real(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *is_rational(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *is_integer(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_exact(VirtualMachine &vm, Register &context,
			  unsigned long argument_pointer,
			  unsigned long nargs, void *data);
    static Cell *is_inexact(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    static Cell *number_eq(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *number_lt(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *number_gt(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *number_le(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *number_ge(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);

    static Cell *is_zero(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *is_positive(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *is_negative(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *is_odd(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *is_even(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);

    static Cell *number_max(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_min(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    static Cell *number_add(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_mul(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_sub(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_div(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    static Cell *number_abs(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    static Cell *number_quotient(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *number_remainder(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *number_modulo(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);

    static Cell *number_gcd(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_lcm(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

#if 0
    static Cell *number_numerator(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *number_denominator(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
#endif

    static Cell *number_floor(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *number_ceiling(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
    static Cell *number_truncate(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *number_round(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);

#if 0
    static Cell *number_rationalize(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
#endif

    static Cell *number_exp(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_log(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_sin(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_cos(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_tan(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *number_asin(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *number_acos(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *number_atan(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);

    static Cell *number_sqrt(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *number_expt(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);

#if 0
    static Cell *make_rectangular(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *make_polar(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *real_part(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *imag_part(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *magnitude(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *angle(VirtualMachine &vm, Register &context,
		       unsigned long argument_pointer,
		       unsigned long nargs, void *data);
#endif

    static Cell *exact2inexact(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *inexact2exact(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);


    // 6.2.6 Numerical input and output
    static Cell *number2string(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *string2number(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
  };
}

#endif

#endif	/* _LIBTSCORE_R5RS_MATH_PROCEDURE_H_ */
