/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_misc_procedure.h,v 1.15 2009/01/01 07:47:01 fuyu Exp $
 */

#ifndef _LIBTSCORE_R5RS_MISC_PROCEDURE_H_
#define _LIBTSCORE_R5RS_MISC_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{

  struct BytecodeProcedureInfo;

  /*
   * R5RSMiscProcedure implements miscellaneous procedures defined in R5RS.
   */
  class R5RSMiscProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];

    void assign_bytecode_procedure(Cell *environment, Cell *frame,
				   const BytecodeProcedureInfo *info);


  public:
    R5RSMiscProcedure(TSCore &tscore) : NativeProcedure(tscore) {}

    Cell *assign_procedures(Cell *environment, Cell *frame);


    static bool eqv(const Cell *x, const Cell *y);
    static bool equal(const Cell *x, const Cell *y);


    // 6.1 Equivalence predicates
    static Cell *eq(VirtualMachine &vm, Register &context,
		    unsigned long argument_pointer,
		    unsigned long nargs, void *data);
    static Cell *eqv(VirtualMachine &vm, Register &context,
		     unsigned long argument_pointer,
		     unsigned long nargs, void *data);
    static Cell *equal(VirtualMachine &vm, Register &context,
		       unsigned long argument_pointer,
		       unsigned long nargs, void *data);

    // 6.3.1 Booleans
    static Cell *is_not(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *is_boolean(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    // 6.3.3 Symbols
    static Cell *is_symbol(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *symbol2string(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *string2symbol(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);

    // 6.4 Control features
    static Cell *is_procedure(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    /*
     * At this time, these are just an enumeration of missing features
     * of libtscore.
     */
    static Cell *apply(VirtualMachine &vm, Register &context,
		       unsigned long argument_pointer,
		       unsigned long nargs, void *data);
    static Cell *map(VirtualMachine &vm, Register &context,
		     unsigned long argument_pointer,
		     unsigned long nargs, void *data);
    static Cell *for_each(VirtualMachine &vm, Register &context,
			  unsigned long argument_pointer,
			  unsigned long nargs, void *data);
    static Cell *call_cc(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *values(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *call_with_values(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *dynamic_wind(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);

    // 6.5 Eval
    static Cell *eval(VirtualMachine &vm, Register &context,
		      unsigned long argument_pointer,
		      unsigned long nargs, void *data);
    static Cell *scheme_report_environment(VirtualMachine &vm,
					   Register &context,
					   unsigned long argument_pointer,
					   unsigned long nargs, void *data);
    static Cell *null_environment(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *interaction_environment(VirtualMachine &vm, Register &context,
					 unsigned long argument_pointer,
					 unsigned long nargs, void *data);
  };


}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_R5RS_MISC_PROCEDURE_H_ */
