/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: r5rs_primitive_syntax.h,v 1.11 2009/01/01 07:47:01 fuyu Exp $
 */

#ifdef _SYNTAX_DEFINED_
#ifndef _LIBTSCORE_R5RS_PRIMITIVE_SYNTAX_H_
#define _LIBTSCORE_R5RS_PRIMITIVE_SYNTAX_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/syntax.h"


using namespace std;
namespace LibTSCore
{

  class Define : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Define() {}
  };


  class Quote : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Quote() {}
  };


  class Lambda : public Syntax
  {
    friend class Define;
    friend class Delay;
    friend class VirtualMachine;

  private:
    static Cell *closure_template_new(VirtualMachine &vm, Register &context,
				      Cell *expression, Cell *environment,
				      Cell *constant_pool, Cell *hint,
				      Cell *formal_args, Cell *procedure_body);

    static size_t emit_lambda(VirtualMachine &vm, Register &context,
			      BytecodeBuffer *buffer,
			      Cell *expression, Cell *environment,
			      Cell *constant_pool, Cell *hint,
			      Cell *formal, Cell *body);


  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Lambda() {}
  };


  class If : public Syntax
  {
  private:
    static size_t emit_if(VirtualMachine &vm, Register &context,
			  BytecodeBuffer *buffer,
			  Cell *expression,
			  Cell *environment, Cell *constant_pool, Cell *hint,
			  Cell *test, Cell *consequent,
			  bool is_tail_context);

    static size_t emit_if(VirtualMachine &vm, Register &context,
			  BytecodeBuffer *buffer,
			  Cell *expression,
			  Cell *environment, Cell *constant_pool, Cell *hint,
			  Cell *test, Cell *consequent, Cell *alternate,
			  bool is_tail_context);


  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    If() {}
  };


  class Assignment : public Syntax
  {
  protected:
    size_t emit_bytecode(VirtualMachine &vm, Register &context,
			 BytecodeBuffer *buffer,
			 Cell *expression, Cell *environment,
			 Cell *constant_pool, Cell *hint,
			 bool is_tail_context);


  public:
    Assignment() {}
  };

}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_R5RS_PRIMITIVE_SYNTAX_H_ */
#endif	/* _SYNTAX_DEFINED_ */
