/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: srfi19_procedure.cc,v 1.6 2009/01/01 07:47:01 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/srfi19_procedure.h"

#include "libtscore/tscore.h"
#include "libtscore/virtual_machine.h"
#include "libtscore/port.h"


#ifdef HAVE_TIME_H
#include <time.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif


namespace LibTSCore
{

NativeProcedure::Info SRFI19Procedure::procedures[] = {
  { &SRFI19Procedure::current_date,
    "current-date", TST_INTEGER, 0, 1 },
  { &SRFI19Procedure::current_julian_day,
    "current-julian-day", TST_NONE, 0, 0 },
  { &SRFI19Procedure::current_modified_julian_day,
    "current-modified-julian-day", TST_NONE, 0, 0 },
  { &SRFI19Procedure::current_time,
    "current-time", TST_SYMBOL, 0, 1 },
  { &SRFI19Procedure::time_resolution,
    "time-resolution", TST_SYMBOL, 0, 1 },
  { &SRFI19Procedure::make_time,
    "make-time", TST_SYMBOL TST_INTEGER TST_INTEGER, 3, 3 },
  { &SRFI19Procedure::is_time,
    "time?", TST_ANY, 1, 1 },
  { &SRFI19Procedure::time_type,
    "time-type", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_nanosecond,
    "time-nanosecond", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_second,
    "time-second", TST_TIME, 1, 1 },
  { &SRFI19Procedure::set_time_type,
    "set-time-type!", TST_TIME TST_SYMBOL, 2, 2 },
  { &SRFI19Procedure::set_time_nanosecond,
    "set-time-nanosecond!", TST_TIME TST_INTEGER, 2, 2 },
  { &SRFI19Procedure::set_time_second,
    "set-time-second!", TST_TIME TST_INTEGER, 2, 2 },
  { &SRFI19Procedure::copy_time,
    "copy-time", TST_TIME, 1, 1 },
  { &SRFI19Procedure::is_time_le,
    "time<=?", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::is_time_lt,
    "time<?", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::is_time_eq,
    "time=?", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::is_time_ge,
    "time>=?", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::is_time_gt,
    "time>?", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::time_difference,
    "time-difference", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::time_difference_ex,
    "time-difference!", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::add_duration,
    "add-duration", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::add_duration_ex,
    "add-duration!", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::subtract_duration,
    "subtract-duration", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::subtract_duration_ex,
    "subtract-duration!", TST_TIME TST_TIME, 2, 2 },
  { &SRFI19Procedure::make_date,
    "make-date", TST_INTEGER TST_INTEGER TST_INTEGER TST_INTEGER TST_INTEGER TST_INTEGER TST_INTEGER TST_INTEGER, 8, 8 },
  { &SRFI19Procedure::is_date,
    "date?", TST_ANY, 1, 1 },
  { &SRFI19Procedure::date_nanosecond,
    "date-nanosecond", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_second,
    "date-second", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_minute,
    "date-minute", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_hour,
    "date-hour", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_day,
    "date-day", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_month,
    "date-month", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_year,
    "date-year", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_zone_offset,
    "date-zone-offset", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_year_day,
    "date-year-day", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_week_day,
    "date-week-day", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_week_number,
    "date-week-number", TST_DATE TST_INTEGER, 2, 2 },
  { &SRFI19Procedure::date_to_julian_day,
    "date->julian-day", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_to_modified_julian_day,
    "date->modified-julian-day", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_to_time_monotonic,
    "date->time-monotonic", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_to_time_tai,
    "date->time-tai", TST_DATE, 1, 1 },
  { &SRFI19Procedure::date_to_time_utc,
    "date->time-utc", TST_DATE, 1, 1 },
  { &SRFI19Procedure::julian_day_to_date,
    "julian-day->date", TST_REAL TST_INTEGER, 1, 2 },
  { &SRFI19Procedure::julian_day_to_time_monotonic,
    "julian-day->time-monotonic", TST_REAL, 1, 1 },
  { &SRFI19Procedure::julian_day_to_time_tai,
    "julian-day->time-tai", TST_REAL, 1, 1 },
  { &SRFI19Procedure::julian_day_to_time_utc,
    "julian-day->time-utc", TST_REAL, 1, 1 },
  { &SRFI19Procedure::modified_julian_day_to_date,
    "modified-julian-day->date", TST_REAL TST_INTEGER, 1, 2 },
  { &SRFI19Procedure::modified_julian_day_to_time_monotonic,
    "modified-julian-day->time-monotonic", TST_REAL, 1, 1 },
  { &SRFI19Procedure::modified_julian_day_to_time_tai,
    "modified-julian-day->time-tai", TST_REAL, 1, 1 },
  { &SRFI19Procedure::modified_julian_day_to_time_utc,
    "modified-julian-day->time-utc", TST_REAL, 1, 1 },
  { &SRFI19Procedure::time_monotonic_to_date,
    "time-monotonic->date", TST_TIME TST_INTEGER, 1, 2 },
  { &SRFI19Procedure::time_monotonic_to_julian_day,
    "time-monotonic->julian-day", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_monotonic_to_modified_julian_day,
    "time-monotonic->modified-julian-day", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_monotonic_to_time_tai,
    "time-monotonic->time-tai", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_monotonic_to_time_tai_ex,
    "time-monotonic->time-tai!", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_monotonic_to_time_utc,
    "time-monotonic->time-utc", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_monotonic_to_time_utc_ex,
    "time-monotonic->time-utc!", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_tai_to_date,
    "time-tai->date", TST_TIME TST_INTEGER, 1, 2 },
  { &SRFI19Procedure::time_tai_to_julian_day,
    "time-tai->julian-day", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_tai_to_modified_julian_day,
    "time-tai->modified-julian-day", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_tai_to_time_monotonic,
    "time-tai->time-monotonic", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_tai_to_time_monotonic_ex,
    "time-tai->time-monotonic!", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_tai_to_time_utc,
    "time-tai->time-utc", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_tai_to_time_utc_ex,
    "time-tai->time-utc!", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_utc_to_date,
    "time-utc->date", TST_TIME TST_INTEGER, 1, 2 },
  { &SRFI19Procedure::time_utc_to_julian_day,
    "time-utc->julian-day", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_utc_to_modified_julian_day,
    "time-utc->modified-julian-day", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_utc_to_time_monotonic,
    "time-utc->time-monotonic", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_utc_to_time_monotonic_ex,
    "time-utc->time-monotonic!", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_utc_to_time_tai,
    "time-utc->time-tai", TST_TIME, 1, 1 },
  { &SRFI19Procedure::time_utc_to_time_tai_ex,
    "time-utc->time-tai!", TST_TIME, 1, 1 },
  { &SRFI19Procedure::date_to_string,
    "date->string", TST_DATE TST_STRING, 1, 2 },
  { &SRFI19Procedure::string_to_date,
    "string->date", TST_STRING TST_STRING, 2, 2 }
};


Cell *
SRFI19Procedure::assign_procedures(Cell *environment, Cell *frame)
{
  TSStackRoot env(core, environment);
  TSStackRoot frm(core, frame);
  size_t n_procs = sizeof(procedures) / sizeof(Info);
  for (size_t i = 0; i < n_procs; i++)
    assign_procedure(env, frm, &procedures[i], this);

  core.define_global(type_table[TIME_TYPE_TIME_DURATION],
		     type_table[TIME_TYPE_TIME_DURATION]);
  core.define_global(type_table[TIME_TYPE_TIME_MONOTONIC],
		     type_table[TIME_TYPE_TIME_MONOTONIC]);
  core.define_global(type_table[TIME_TYPE_TIME_PROCESS],
		     type_table[TIME_TYPE_TIME_PROCESS]);
  core.define_global(type_table[TIME_TYPE_TIME_TAI],
		     type_table[TIME_TYPE_TIME_TAI]);
  core.define_global(type_table[TIME_TYPE_TIME_THREAD],
		     type_table[TIME_TYPE_TIME_THREAD]);
  core.define_global(type_table[TIME_TYPE_TIME_UTC],
		     type_table[TIME_TYPE_TIME_UTC]);

  return env;
}


#define LOCAL_TZ_OFFSET_UNKNOWN	0x7fffffff
static long local_tz_offset = LOCAL_TZ_OFFSET_UNKNOWN;


#if defined(HAVE_SYSCONF) && defined(_SC_CLK_TCK)
static long sc_clk_tck = 0;
static long usec_per_clk_tck = 0;
#endif


struct LeapSecondInfo
{
  long utc_second;
  long delta;
};


static const LeapSecondInfo leap_second_table[] = {
#include "tai-utc.inc"
  { 0, 0 }	// just for making compiler quiet, we should clean tai-utc.inc.
};


static long
leap_second_delta(long utc_second)
{
  if (utc_second >= (1972 - 1970) * 365 * 86400)
    {
      for (size_t i = (sizeof(leap_second_table) / sizeof(LeapSecondInfo)) - 1;
	   --i > 0;)
	if (leap_second_table[i - 1].utc_second >= utc_second)
	  return leap_second_table[i - 1].delta;
    }

  return 0;
}


static long
leap_second_neg_delta(long tai_second)
{
  if (tai_second >= (1972 - 1970) * 365 * 86400)
    {
      for (size_t i = (sizeof(leap_second_table) / sizeof(LeapSecondInfo)) - 1;
	   --i > 0;)
	if (leap_second_table[i - 1].delta
	    <= tai_second - leap_second_table[i - 1].utc_second)
	  return leap_second_table[i - 1].delta;
    }

  return 0;
}


SRFI19Procedure::SRFI19Procedure(TSCore &core)
  : NativeProcedure(core)
{
  type_table[TIME_TYPE_TIME_DURATION] = core.mk_symbol("time-duration");
  core.register_external_root(&type_table[TIME_TYPE_TIME_DURATION]);
  type_table[TIME_TYPE_TIME_MONOTONIC] = core.mk_symbol("time-monotonic");
  core.register_external_root(&type_table[TIME_TYPE_TIME_MONOTONIC]);
  type_table[TIME_TYPE_TIME_PROCESS] = core.mk_symbol("time-process");
  core.register_external_root(&type_table[TIME_TYPE_TIME_PROCESS]);
  type_table[TIME_TYPE_TIME_TAI] = core.mk_symbol("time-tai");
  core.register_external_root(&type_table[TIME_TYPE_TIME_TAI]);
  type_table[TIME_TYPE_TIME_THREAD] = core.mk_symbol("time-thread");
  core.register_external_root(&type_table[TIME_TYPE_TIME_THREAD]);
  type_table[TIME_TYPE_TIME_UTC] = core.mk_symbol("time-utc");
  core.register_external_root(&type_table[TIME_TYPE_TIME_UTC]);

#ifdef HAVE_TIME_H
  if (local_tz_offset == LOCAL_TZ_OFFSET_UNKNOWN)
    {
      time_t utc = time(NULL);
      assert(utc != static_cast<time_t>(-1));

      struct tm *date_p;
#ifdef HAVE_GMTIME_R
      struct tm date;
      memset(&date, 0, sizeof(struct tm));
      date_p = gmtime_r(&utc, &date);
#else
      date_p = gmtime(&utc);
#endif
      if (date_p != NULL)
	{
	  time_t fake_utc = mktime(date_p);
	  if (fake_utc != static_cast<time_t>(-1))
	    local_tz_offset = static_cast<long>(fake_utc)
	      - static_cast<long>(utc);
	  else
	    local_tz_offset = 0;// cannot determine local timezone offset!
	}
      else
	local_tz_offset = 0;	// cannot determine local timezone offset!
    }
#else
  local_tz_offset = 0;		// cannot determine local timezone offset!
#endif

#if defined(HAVE_SYSCONF) && defined(_SC_CLK_TCK)
  if (sc_clk_tck == 0)
    {
      sc_clk_tck = sysconf(_SC_CLK_TCK);
      if (sc_clk_tck > 0)
	usec_per_clk_tck = 1000000 / sc_clk_tck;
    }
#endif
}


SRFI19Procedure::~SRFI19Procedure()
{
  type_table[TIME_TYPE_TIME_DURATION] = Cell::nil();
  core.unregister_external_root(&type_table[TIME_TYPE_TIME_DURATION]);

  type_table[TIME_TYPE_TIME_MONOTONIC] = Cell::nil();
  core.unregister_external_root(&type_table[TIME_TYPE_TIME_MONOTONIC]);

  type_table[TIME_TYPE_TIME_PROCESS] = Cell::nil();
  core.unregister_external_root(&type_table[TIME_TYPE_TIME_PROCESS]);

  type_table[TIME_TYPE_TIME_TAI] = Cell::nil();
  core.unregister_external_root(&type_table[TIME_TYPE_TIME_TAI]);

  type_table[TIME_TYPE_TIME_THREAD] = Cell::nil();
  core.unregister_external_root(&type_table[TIME_TYPE_TIME_THREAD]);

  type_table[TIME_TYPE_TIME_UTC] = Cell::nil();
  core.unregister_external_root(&type_table[TIME_TYPE_TIME_UTC]);
}


#define ARG(n)	context.frame->load_variable(argument_pointer + n)


Cell *
SRFI19Procedure::current_date(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  time_t utc = time(NULL);
  assert(utc != static_cast<time_t>(-1));

  struct tm *date_p;
#if defined(HAVE_GMTIME_R) || defined(HAVE_LOCALTIME_R)
  struct tm value;
#endif

  long zone_offset;
  if (nargs == 0)
    {
      zone_offset = local_tz_offset;

#ifdef HAVE_LOCALTIME_R
      memset(&value, 0, sizeof(struct tm));
      date_p = localtime_r(&utc, &value);
#else
      date_p = localtime(&utc);
#endif
      if (date_p == NULL)
	return signal_error(vm, "current-date: unexpected error: ", Cell::f());
    }
  else
    {
      zone_offset = ARG(0)->get_integer0();
      utc += static_cast<time_t>(zone_offset);
#ifdef HAVE_GMTIME_R
      memset(&value, 0, sizeof(struct tm));
      date_p = gmtime_r(&utc, &value);
#else
      date_p = gmtime(&utc);
#endif
    }

  if (date_p != NULL)
    return vm.mk_date(date_p, 0, zone_offset);

  return signal_error(vm, "current-date: unexpected error: ", Cell::f());
#else
  return signal_error(vm, "current-date: not supported: ", Cell::f());
#endif
}


#ifdef HAVE_TIME_H
static inline double
utc_to_julian_day(long nanosecond, long second)
{
  return ((second + (nanosecond / 1000000000)) / 86400.0) + (4881175 / 2.0);
}
#endif


Cell *
SRFI19Procedure::current_julian_day(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  time_t utc = time(NULL);
  assert(utc != static_cast<time_t>(-1));

  return vm.mk_real(utc_to_julian_day(0, static_cast<long>(utc)));
#else
  return signal_error(vm, "current-julian-day: not supported: ", Cell::f());
#endif
}


#ifdef HAVE_TIME_H
static double
utc_to_modified_julian_day(VirtualMachine &vm, time_t utc)
{
  return (utc / 86400.0) + (4881175 / 2.0) - (4800001 / 2.0);
}
#endif


Cell *
SRFI19Procedure::current_modified_julian_day(VirtualMachine &vm,
					     Register &context,
					     unsigned long argument_pointer,
					     unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  time_t utc = time(NULL);
  assert(utc != static_cast<time_t>(-1));

  return vm.mk_real(utc_to_modified_julian_day(vm, utc));
#else
  return signal_error(vm, "current-modified-julian-day: not supported: ", Cell::f());
#endif
}


#ifdef HAVE_SYS_TIME_H
static struct timeval *
get_current_process_time(struct timeval &value)
{
#ifdef HAVE_GETRUSAGE
  struct rusage usage;
  if (getrusage(RUSAGE_SELF, &usage) == 0)
    {
      value.tv_sec = usage.ru_utime.tv_sec + usage.ru_stime.tv_sec;
      suseconds_t tv_usec = usage.ru_utime.tv_usec + usage.ru_stime.tv_usec;
      value.tv_sec += tv_usec / 1000000;
      value.tv_usec = tv_usec % 1000000;
      return &value;
    }

  return NULL;
#else
#if defined(HAVE_TIMES) && defined(HAVE_SYSCONF) && defined(_SC_CLK_TCK)
  struct tms clocks;
  if (sc_clk_tck > 0 && times(&clock) != static_cast<clock_t>(-1))
    {
      clock_t tms_time = clocks.tms_utime + clocks.tms_stime;
      value.tv_sec = tms_time / sc_clk_tck;
      value.tv_usec = tms_time % sc_clk_tck * usec_per_clk_tck;
      return &value;
    }

  return NULL;
#endif
#endif
  return NULL;
}
#endif


long
SRFI19Procedure::determine_time_type(Cell *symbol)
{
  long i;
  for (i = 0; i < NUMBER_OF_TIME_TYPES; i++)
    if (type_table[i] == symbol)
      break;

  return i;
}


Cell *
SRFI19Procedure::current_time(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
#ifdef HAVE_SYS_TIME_H
  long time_type;
  if (nargs == 0)
    time_type = TIME_TYPE_TIME_UTC;
  else
    {
      time_type
	= static_cast<SRFI19Procedure *>(data)->determine_time_type(ARG(0));
      if (time_type >= NUMBER_OF_TIME_TYPES)
	return signal_error(vm, "current-time: unsupported time-type: ", ARG(0));
    }

  struct timeval value;
  struct timeval *value_p;
  switch (time_type)
    {
    case TIME_TYPE_TIME_MONOTONIC:
    case TIME_TYPE_TIME_TAI:
#ifdef HAVE_GETTIMEOFDAY
      memset(&value, 0, sizeof(struct timeval));
      if (gettimeofday(&value, NULL) == 0)
	{
	  value.tv_sec += leap_second_delta(value.tv_sec);
	  value_p = &value;
	}
      else
	value_p = NULL;

      break;
#else
#ifdef HAVE_TIME_H
      memset(&value, 0, sizeof(struct timeval));
      value.tv_sec = time(NULL);
      if (value.tv_sec != static_cast<time_t>(-1))
	{
	  value.tv_sec += leap_second_delta(value.tv_sec);
	  value_p = &value;
	}
      else
	value_p = NULL;

      break;
#endif
#endif


    case TIME_TYPE_TIME_PROCESS:
    case TIME_TYPE_TIME_THREAD:
      value_p = get_current_process_time(value);
      break;


    case TIME_TYPE_TIME_UTC:
#ifdef HAVE_GETTIMEOFDAY
      memset(&value, 0, sizeof(struct timeval));
      if (gettimeofday(&value, NULL) == 0)
	value_p = &value;
      else
	value_p = NULL;

      break;
#else
#ifdef HAVE_TIME_H
      memset(&value, 0, sizeof(struct timeval));
      value.tv_sec = time(NULL);
      if (value.tv_sec != static_cast<time_t>(-1))
        value_p = &value;
      else
	value_p = NULL;

      break;
#endif
#endif


    default:
      assert(nargs == 1);

      return signal_error(vm, "current-time: unsupported time-type: ", ARG(0));
    }

  if (value_p != NULL)
    return vm.mk_time(time_type, value_p->tv_usec * 1000, value_p->tv_sec);

  return signal_error(vm, "current-time: unexpected error: ", Cell::f());
#else
  return signal_error(vm, "current-time: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::time_resolution(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  long time_type;
  if (nargs == 0)
    time_type = TIME_TYPE_TIME_UTC;
  else
    time_type
      = static_cast<SRFI19Procedure *>(data)->determine_time_type(ARG(0));

  switch (time_type)
    {
    case TIME_TYPE_TIME_TAI:
    case TIME_TYPE_TIME_UTC:
    case TIME_TYPE_TIME_MONOTONIC:
    case TIME_TYPE_TIME_THREAD:
    case TIME_TYPE_TIME_PROCESS:
      return vm.mk_integer(10000);


    default:
      assert(nargs == 1);

      return signal_error(vm,
			  "time-resolution: unsupported time-type: ", ARG(0));
    }
}


Cell *
SRFI19Procedure::make_time(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
  long time_type
    = static_cast<SRFI19Procedure *>(data)->determine_time_type(ARG(0));

  if (time_type < NUMBER_OF_TIME_TYPES)
    return vm.mk_time(time_type,
		      ARG(1)->get_integer0(), ARG(2)->get_integer0());

  return signal_error(vm, "make-time: unsupported time-type: ", ARG(0));
}


Cell *
SRFI19Procedure::is_time(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data)
{
  return Cell::ts_bool(ARG(0)->is_time());
}


Cell *
SRFI19Procedure::time_type(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
  return static_cast<SRFI19Procedure *>(data)
    ->type_table[ARG(0)->get_time_type()];
}


Cell *
SRFI19Procedure::time_nanosecond(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  return vm.mk_integer(ARG(0)->get_time_nanosecond());
}


Cell *
SRFI19Procedure::time_second(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data)
{
  return vm.mk_integer(ARG(0)->get_time_second());
}


Cell *
SRFI19Procedure::set_time_type(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
  long time_type
    = static_cast<SRFI19Procedure *>(data)->determine_time_type(ARG(1));
  if (time_type < NUMBER_OF_TIME_TYPES)
    return ARG(0)->set_time_type(time_type);

  return signal_error(vm, "set-time-type!: unsupported time-type: ", ARG(1));
}


Cell *
SRFI19Procedure::set_time_nanosecond(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data)
{
  return ARG(0)->set_time_nanosecond(ARG(1)->get_integer0());
}


Cell *
SRFI19Procedure::set_time_second(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  return ARG(0)->set_time_second(ARG(1)->get_integer0());
}


Cell *
SRFI19Procedure::copy_time(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  return vm.mk_time(arg->get_time_type(),
		    arg->get_time_nanosecond(), arg->get_time_second());
}


Cell *
SRFI19Procedure::is_time_le(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    {
      long arg0_sec = arg0->get_time_second();
      long arg0_nanosec = arg0->get_time_nanosecond();
      long arg1_sec = arg1->get_time_second();
      long arg1_nanosec = arg1->get_time_nanosecond();

      return Cell::ts_bool(arg0_sec < arg1_sec
			   || (arg0_sec == arg1_sec
			       && arg0_nanosec <= arg1_nanosec));
    }

  return signal_error(vm, "time<=?: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::is_time_lt(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    {
      long arg0_sec = arg0->get_time_second();
      long arg0_nanosec = arg0->get_time_nanosecond();
      long arg1_sec = arg1->get_time_second();
      long arg1_nanosec = arg1->get_time_nanosecond();

      return Cell::ts_bool(arg0_sec < arg1_sec
			   || (arg0_sec == arg1_sec
			       && arg0_nanosec < arg1_nanosec));
    }

  return signal_error(vm, "time<?: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::is_time_eq(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    return Cell::ts_bool(arg0->get_time_second() == arg1->get_time_second()
		&& arg0->get_time_nanosecond() == arg1->get_time_nanosecond());

  return signal_error(vm, "time=?: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::is_time_ge(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    {
      long arg0_sec = arg0->get_time_second();
      long arg0_nanosec = arg0->get_time_nanosecond();
      long arg1_sec = arg1->get_time_second();
      long arg1_nanosec = arg1->get_time_nanosecond();

      return Cell::ts_bool(arg0_sec > arg1_sec
			   || (arg0_sec == arg1_sec
			       && arg0_nanosec >= arg1_nanosec));
    }

  return signal_error(vm, "time>=?: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::is_time_gt(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    {
      long arg0_sec = arg0->get_time_second();
      long arg0_nanosec = arg0->get_time_nanosecond();
      long arg1_sec = arg1->get_time_second();
      long arg1_nanosec = arg1->get_time_nanosecond();

      return Cell::ts_bool(arg0_sec > arg1_sec
			   || (arg0_sec == arg1_sec
			       && arg0_nanosec > arg1_nanosec));
    }

  return signal_error(vm, "time>?: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::time_difference(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    {
      long nanosecond = arg0->get_time_nanosecond()
	- arg1->get_time_nanosecond();
      long second = arg0->get_time_second() - arg1->get_time_second();

      if (nanosecond < 0)
	{
	  --second;
	  nanosecond += 1000000000;
	}

      return vm.mk_time(TIME_TYPE_TIME_DURATION, nanosecond, second);
    }

  return signal_error(vm, "time-difference: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::time_difference_ex(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg0->get_time_type() == arg1->get_time_type())
    {
      long nanosecond = arg0->get_time_nanosecond()
	- arg1->get_time_nanosecond();
      long second = arg0->get_time_second() - arg1->get_time_second();

      while (nanosecond < 0)
	{
	  --second;
	  nanosecond += 1000000000;
	}

      arg0->set_time_type(TIME_TYPE_TIME_DURATION);
      arg0->set_time_nanosecond(nanosecond);
      arg0->set_time_second(second);

      return arg0;
    }

  return signal_error(vm, "time-difference: incompatible time types: ",
		      vm.mk_cons(arg0, arg1, true));
}


Cell *
SRFI19Procedure::add_duration(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg1->get_time_type() == TIME_TYPE_TIME_DURATION)
    {
      long nanosecond = arg0->get_time_nanosecond()
	+ arg1->get_time_nanosecond();
      long second = arg0->get_time_second() + arg1->get_time_second();

      while (nanosecond < 0)
	{
	  --second;
	  nanosecond += 1000000000;
	}

      return vm.mk_time(arg0->get_time_type(), nanosecond, second);
    }

  return signal_error(vm,
		      "add-duration: argument #2 must be of 'time-duration: ",
		      arg1);
}


Cell *
SRFI19Procedure::add_duration_ex(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg1->get_time_type() == TIME_TYPE_TIME_DURATION)
    {
      long nanosecond = arg0->get_time_nanosecond()
	+ arg1->get_time_nanosecond();
      long second = arg0->get_time_second() + arg1->get_time_second();

      while (nanosecond < 0)
	{
	  --second;
	  nanosecond += 1000000000;
	}

      arg0->set_time_nanosecond(nanosecond);
      arg0->set_time_second(second);

      return arg0;
    }

  return signal_error(vm,
		      "add-duration!: argument #2 must be of 'time-duration: ",
		      arg1);
}


Cell *
SRFI19Procedure::subtract_duration(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg1->get_time_type() == TIME_TYPE_TIME_DURATION)
    {
      long nanosecond = arg0->get_time_nanosecond()
	- arg1->get_time_nanosecond();
      long second = arg0->get_time_second() - arg1->get_time_second();

      while (nanosecond < 0)
	{
	  --second;
	  nanosecond += 1000000000;
	}

      return vm.mk_time(arg0->get_time_type(), nanosecond, second);
    }

  return signal_error(vm,
		      "subtract-duration: argument #2 must be of 'time-duration: ",
		      arg1);
}


Cell *
SRFI19Procedure::subtract_duration_ex(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  Cell *arg0 = ARG(0);
  Cell *arg1 = ARG(1);
  if (arg1->get_time_type() == TIME_TYPE_TIME_DURATION)
    {
      long nanosecond = arg0->get_time_nanosecond()
	- arg1->get_time_nanosecond();
      long second = arg0->get_time_second() - arg1->get_time_second();

      while (nanosecond < 0)
	{
	  --second;
	  nanosecond += 1000000000;
	}

      arg0->set_time_nanosecond(nanosecond);
      arg0->set_time_second(second);

      return arg0;
    }

  return signal_error(vm,
		      "subtract-duration!: argument #2 must be of 'time-duration: ",
		      arg1);
}


Cell *
SRFI19Procedure::make_date(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  struct tm date;
  memset(&date, 0, sizeof(struct tm));

  date.tm_sec = ARG(1)->get_integer0();
  date.tm_min = ARG(2)->get_integer0();
  date.tm_hour = ARG(3)->get_integer0();
  date.tm_mday = ARG(4)->get_integer0();
  date.tm_mon = ARG(5)->get_integer0() - 1;
  date.tm_year = ARG(6)->get_integer0() - 1900;

  time_t fake_utc = mktime(&date);
  if (fake_utc == static_cast<time_t>(-1))
    return signal_error(vm, "make-date: unexpected error: ", Cell::f());

#ifdef HAVE_LOCALTIME_R
  struct tm *date_p = localtime_r(&fake_utc, &date);
#else
  struct tm *date_p = localtime(&fake_utc);
#endif
  if (date_p == NULL)
    return signal_error(vm, "make-date: unexpected error: ", Cell::f());

  return vm.mk_date(date_p, ARG(0)->get_integer0(), ARG(7)->get_integer0());
#else
  return signal_error(vm, "make-date: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::is_date(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data)
{
  return Cell::ts_bool(ARG(0)->is_date());
}


Cell *
SRFI19Procedure::date_nanosecond(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_nanosecond());
#else
  return signal_error(vm, "date-nanosecond: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_second(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_second());
#else
  return signal_error(vm, "date-second: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_minute(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_minute());
#else
  return signal_error(vm, "date-minute: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_hour(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_hour());
#else
  return signal_error(vm, "date-hour: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_day(VirtualMachine &vm, Register &context,
			  unsigned long argument_pointer,
			  unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_day());
#else
  return signal_error(vm, "date-day: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_month(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_month());
#else
  return signal_error(vm, "date-month: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_year(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_year());
#else
  return signal_error(vm, "date-year: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_zone_offset(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_zone_offset());
#else
  return signal_error(vm, "date-zone-offset: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_year_day(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_year_day());
#else
  return signal_error(vm, "date-year-day: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_week_day(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  return vm.mk_integer(ARG(0)->get_date_week_day());
#else
  return signal_error(vm, "date-week-day: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_week_number(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
#warning "Implement This!: date-week-number"
  return signal_error(vm, "date-week-number: not implemented yet: ", Cell::f());
}


#ifdef HAVE_TIME_H
static double
encode_julian_day_number(long day, long month, long year)
{
  long a = (14 - month) / 12;
  long y = ((year + 4800) - a) + (year > 0 ? 0 : 1);
  long m = month + 12 * a - 3;

  return day + (((153 * m) + 2) / 5) + (365 * y) + (y / 4) - (y / 100)
    + (y / 400) - 32045;
}
#endif


Cell *
SRFI19Procedure::date_to_julian_day(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  double jd = encode_julian_day_number(arg->get_date_day(),
				       arg->get_date_month(),
				       arg->get_date_year()) - 0.5
    + (((arg->get_date_hour() * 60 * 60) + (arg->get_date_minute() * 60)
	+ arg->get_date_second() + (arg->get_date_nanosecond() / 1000000000))
       / 86400) / (-arg->get_date_zone_offset());
  return vm.mk_real(jd);
#else
  return signal_error(vm, "date->julian-day: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_to_modified_julian_day(VirtualMachine &vm,
					     Register &context,
					     unsigned long argument_pointer,
					     unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  double jd = encode_julian_day_number(arg->get_date_day(),
				       arg->get_date_month(),
				       arg->get_date_year()) - 0.5
    + (((arg->get_date_hour() * 60 * 60) + (arg->get_date_minute() * 60)
	+ arg->get_date_second() + (arg->get_date_nanosecond() / 1000000000))
       / 86400) / (-arg->get_date_zone_offset()) - (4800001 / 2.0);
  return vm.mk_real(jd);
#else
  return signal_error(vm, "date->modified-julian-day: not supported: ",
		      Cell::f());
#endif
}


#ifdef HAVE_TIME_H
static long
date_to_utc_second(Cell *date)
{
  double jd = encode_julian_day_number(date->get_date_day(),
				       date->get_date_month(),
				       date->get_date_year())
    - 4881175 / 2.0;
  return (static_cast<long>(jd - (1 / 2.0)) * 24 * 60 * 60)
    + (date->get_date_hour() * 60 * 60)
    + (date->get_date_minute() * 60)
    + date->get_date_second()
    - date->get_date_zone_offset();
}
#endif


Cell *
SRFI19Procedure::date_to_time_monotonic(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  long utc_second = date_to_utc_second(arg);
  return vm.mk_time(TIME_TYPE_TIME_MONOTONIC, arg->get_date_nanosecond(),
		    utc_second + leap_second_delta(utc_second));
#else
  return signal_error(vm, "date->time-monotonic: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_to_time_tai(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  long utc_second = date_to_utc_second(arg);
  return vm.mk_time(TIME_TYPE_TIME_TAI, arg->get_date_nanosecond(),
		    utc_second + leap_second_delta(utc_second));
#else
  return signal_error(vm, "date->time-tai: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::date_to_time_utc(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  return vm.mk_time(TIME_TYPE_TIME_UTC, arg->get_date_nanosecond(),
		    date_to_utc_second(arg));
#else
  return signal_error(vm, "date->time-utc: not supported: ", Cell::f());
#endif
}


static inline void
julian_day_to_utc(double jd, long *nanosecond_p, long *second_p)
{
  assert(nanosecond_p != NULL);
  assert(second_p != NULL);

  uintmax_t nanosecs
    = static_cast<uintmax_t>((jd - (4881175 / 2.0))) * 1000000000 * 86400;
  *nanosecond_p = static_cast<long>(nanosecs % 1000000000);
  *second_p = static_cast<long>(nanosecs / 1000000000);
}


Cell *
SRFI19Procedure::utc_to_date(VirtualMachine &vm, long nanosecond, long second,
			     long zone_offset)
{
  struct tm *date_p;
#ifdef HAVE_GMTIME_R
  struct tm date;
  memset(&date, 0, sizeof(struct tm));
  date_p = gmtime_r(static_cast<time_t *>(&second), &date);
#else
  date_p = gmtime(static_cast<time_t *>(&second));
#endif
  if (date_p != NULL)
    return vm.mk_date(date_p, nanosecond, zone_offset);

  return signal_error(vm, "utc-to-date: unexpected error: ", Cell::f());
}


Cell *
SRFI19Procedure::julian_day_to_date(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  long nanosecond;
  long second;

  julian_day_to_utc(ARG(0)->get_real(), &nanosecond, &second);

  long zone_offset;
  if (nargs == 1)
    zone_offset = local_tz_offset;
  else
    zone_offset = ARG(1)->get_integer0();

  return utc_to_date(vm, nanosecond, second, zone_offset);
#else
  return signal_error(vm, "julian-day->date: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::julian_day_to_time_monotonic(VirtualMachine &vm,
					      Register &context,
					      unsigned long argument_pointer,
					      unsigned long nargs, void *data)
{
  long nanosecond;
  long second;

  julian_day_to_utc(ARG(0)->get_real(), &nanosecond, &second);

  second += leap_second_delta(second);
  return vm.mk_time(TIME_TYPE_TIME_MONOTONIC, nanosecond, second);
}


Cell *
SRFI19Procedure::julian_day_to_time_tai(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data)
{
  long nanosecond;
  long second;
  julian_day_to_utc(ARG(0)->get_real(), &nanosecond, &second);
  second += leap_second_delta(second);
  return vm.mk_time(TIME_TYPE_TIME_TAI, nanosecond, second);
}


Cell *
SRFI19Procedure::julian_day_to_time_utc(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data)
{
  long nanosecond;
  long second;
  julian_day_to_utc(ARG(0)->get_real(), &nanosecond, &second);
  return vm.mk_time(TIME_TYPE_TIME_UTC, nanosecond, second);
}


Cell *
SRFI19Procedure::modified_julian_day_to_date(VirtualMachine &vm,
					     Register &context,
					     unsigned long argument_pointer,
					     unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  long nanosecond;
  long second;
  julian_day_to_utc(ARG(0)->get_real() + (4800001 / 2.0), &nanosecond, &second);
  return utc_to_date(vm, nanosecond, second,
		     nargs == 1 ? local_tz_offset : ARG(1)->get_integer0());
#else
  return signal_error(vm, "modified-julian-day->date: not implemented yet: ",
		      Cell::f());
#endif
}


Cell *
SRFI19Procedure::modified_julian_day_to_time_monotonic(VirtualMachine &vm,
						Register &context,
						unsigned long argument_pointer,
						unsigned long nargs,
						void *data)
{
  long nanosecond;
  long second;
  julian_day_to_utc(ARG(0)->get_real() + (4800001 / 2.0), &nanosecond, &second);
  second += leap_second_delta(second);
  return vm.mk_time(TIME_TYPE_TIME_MONOTONIC, nanosecond, second);
}


Cell *
SRFI19Procedure::modified_julian_day_to_time_tai(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data)
{
  long nanosecond;
  long second;
  julian_day_to_utc(ARG(0)->get_real() + (4800001 / 2.0), &nanosecond, &second);
  second += leap_second_delta(second);
  return vm.mk_time(TIME_TYPE_TIME_TAI, nanosecond, second);
}


Cell *
SRFI19Procedure::modified_julian_day_to_time_utc(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data)
{
  long nanosecond;
  long second;
  julian_day_to_utc(ARG(0)->get_real() + (4800001 / 2.0), &nanosecond, &second);
  return vm.mk_time(TIME_TYPE_TIME_UTC, nanosecond, second);
}


Cell *
SRFI19Procedure::time_monotonic_to_date(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    {
      long zone_offset = nargs == 1 ? local_tz_offset : ARG(1)->get_integer0();
      long second = arg->get_time_second();
      return utc_to_date(vm, arg->get_time_nanosecond(),
			 second - leap_second_neg_delta(second), zone_offset);
    }

  return signal_error(vm, "time-monotonic->date: invalid time type: ", arg);
#else
  return signal_error(vm, "time-monotonic->date: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::time_monotonic_to_julian_day(VirtualMachine &vm,
					      Register &context,
					      unsigned long argument_pointer,
					      unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    {
      long second = arg->get_time_second();
      return vm.mk_real(utc_to_julian_day(arg->get_time_nanosecond(),
					  second
					  - leap_second_neg_delta(second)));
    }

  return signal_error(vm, "time-monotonic->julian-day: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_monotonic_to_modified_julian_day(VirtualMachine &vm,
						Register &context,
						unsigned long argument_pointer,
						unsigned long nargs,
						void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    {
      long second = arg->get_time_second();
      return vm.mk_real(utc_to_julian_day(arg->get_time_nanosecond(),
					  second
					  - leap_second_neg_delta(second))
			- (4800001 / 2.0));
    }

  return signal_error(vm,
		      "time-monotonic->modified-julian-day: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_monotonic_to_time_tai(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    return vm.mk_time(TIME_TYPE_TIME_TAI,
		      arg->get_time_nanosecond(), arg->get_time_second());

  return signal_error(vm, "time-monotonic->time-tai: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_monotonic_to_time_tai_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    return arg->set_time_type(TIME_TYPE_TIME_TAI);

  return signal_error(vm, "time-monotonic->time-tai!: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_monotonic_to_time_utc(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    {
      long second = arg->get_time_second();
      return vm.mk_time(TIME_TYPE_TIME_UTC, arg->get_time_nanosecond(),
			second - leap_second_neg_delta(second));
    }

  return signal_error(vm, "time-monotonic->time-utc: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_monotonic_to_time_utc_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_MONOTONIC)
    {
      arg->set_time_type(TIME_TYPE_TIME_UTC);
      long second = arg->get_time_second();
      return arg->set_time_second(second - leap_second_neg_delta(second));
    }

  return signal_error(vm, "time-monotonic->time-utc!: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_tai_to_date(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    {
      long zone_offset = nargs == 1 ? local_tz_offset : ARG(1)->get_integer0();
      long second = arg->get_time_second();
      return utc_to_date(vm, arg->get_time_nanosecond(),
			 second - leap_second_neg_delta(second), zone_offset);
    }

  return signal_error(vm, "time-tai->date: invalid time type: ", arg);
#else
  return signal_error(vm, "time-tai->date: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::time_tai_to_julian_day(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    {
      long second = arg->get_time_second();
      return vm.mk_real(utc_to_julian_day(arg->get_time_nanosecond(),
					  second
					  - leap_second_neg_delta(second)));
    }

  return signal_error(vm, "time-tai->julian-day: invalid time type: ", arg);
}


Cell *
SRFI19Procedure::time_tai_to_modified_julian_day(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    {
      long second = arg->get_time_second();
      return vm.mk_real(utc_to_julian_day(arg->get_time_nanosecond(),
					  second
					  - leap_second_neg_delta(second))
			- (4800001 / 2.0));
    }

  return signal_error(vm, "time-tai->modified-julian-day: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_tai_to_time_monotonic(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    return vm.mk_time(TIME_TYPE_TIME_MONOTONIC,
		      arg->get_time_nanosecond(), arg->get_time_second());

  return signal_error(vm, "time-tai->time-monotonic: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_tai_to_time_monotonic_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    return arg->set_time_type(TIME_TYPE_TIME_MONOTONIC);

  return signal_error(vm, "time-tai->time-monotonic!: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_tai_to_time_utc(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    {
      long second = arg->get_time_second();
      return vm.mk_time(TIME_TYPE_TIME_UTC, arg->get_time_nanosecond(),
			second - leap_second_neg_delta(second));
    }

  return signal_error(vm, "time-tai->time-utc: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_tai_to_time_utc_ex(VirtualMachine &vm, Register &context,
					 unsigned long argument_pointer,
					 unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_TAI)
    {
      arg->set_time_type(TIME_TYPE_TIME_UTC);
      long second = arg->get_time_second();
      return arg->set_time_second(second - leap_second_neg_delta(second));
    }

  return signal_error(vm, "time-tai->time-utc!: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_utc_to_date(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
#ifdef HAVE_TIME_H
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    {
      long zone_offset = nargs == 1 ? local_tz_offset : ARG(1)->get_integer0();
      return utc_to_date(vm, arg->get_time_nanosecond(), arg->get_time_second(),
			 zone_offset);
    }

  return signal_error(vm, "time-utc->date: invalid time type: ", arg);
#else
  return signal_error(vm, "time-utc->date: not supported: ", Cell::f());
#endif
}


Cell *
SRFI19Procedure::time_utc_to_julian_day(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    return vm.mk_real(utc_to_julian_day(arg->get_time_nanosecond(),
					arg->get_time_second()));

  return signal_error(vm, "time-utc->julian-day: invalid time type: ", arg);
}


Cell *
SRFI19Procedure::time_utc_to_modified_julian_day(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    return vm.mk_real(utc_to_julian_day(arg->get_time_nanosecond(),
					arg->get_time_second())
		      - (4800001 / 2.0));

  return signal_error(vm, "time-utc->modified-julian-day: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_utc_to_time_monotonic(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    {
      long second = arg->get_time_second();
      return vm.mk_time(TIME_TYPE_TIME_MONOTONIC,
			arg->get_time_nanosecond(),
			second + leap_second_delta(second));
    }

  return signal_error(vm, "time-utc->time-monotonic: invalid time type: ", arg);
}


Cell *
SRFI19Procedure::time_utc_to_time_monotonic_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    {
      arg->set_time_type(TIME_TYPE_TIME_MONOTONIC);
      long second = arg->get_time_second();
      return arg->set_time_second(second + leap_second_delta(second));
    }

  return signal_error(vm, "time-utc->time-monotonic!: invalid time type: ",
		      arg);
}


Cell *
SRFI19Procedure::time_utc_to_time_tai(VirtualMachine &vm,
				      Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    {
      long second = arg->get_time_second();
      return vm.mk_time(TIME_TYPE_TIME_TAI,
			arg->get_time_nanosecond(),
			second + leap_second_delta(second));
    }

  return signal_error(vm, "time-utc->time-tai: invalid time type: ", arg);
}


Cell *
SRFI19Procedure::time_utc_to_time_tai_ex(VirtualMachine &vm,
					 Register &context,
					 unsigned long argument_pointer,
					 unsigned long nargs, void *data)
{
  Cell *arg = ARG(0);
  if (arg->get_time_type() == TIME_TYPE_TIME_UTC)
    {
      arg->set_time_type(TIME_TYPE_TIME_TAI);
      long second = arg->get_time_second();
      return arg->set_time_second(second + leap_second_delta(second));
    }

  return signal_error(vm, "time-utc->time-tai!: invalid time type: ", arg);
}


Cell *
SRFI19Procedure::date_to_string(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data)
{
  // ~~ a literal ~
  // ~a locale's abbreviated weekday name (Sun...Sat)
  // ~A locale's full weekday name (Sunday...Saturday)
  // ~b locale's abbreviate month name (Jan...Dec)
  // ~B locale's full month day (January...December)
  // ~c locale's date and time (e.g., "Fri Jul 14 20:28:42-0400 2000")
  // ~d day of month, zero padded (01...31)
  // ~D date (mm/dd/yy)
  // ~e day of month, blank padded ( 1...31)
  // ~f seconds+fractional seconds, using locale's decimal separator (e.g.
  //    5.2).
  // ~h same as ~b
  // ~H hour, zero padded, 24-hour clock (00...23)
  // ~I hour, zero padded, 12-hour clock (01...12)
  // ~j day of year, zero padded
  // ~k hour, blank padded, 24-hour clock (00...23)
  // ~l hour, blank padded, 12-hour clock (01...12)
  // ~m month, zero padded (01...12)
  // ~M minute, zero padded (00...59)
  // ~n new line
  // ~N nanosecond, zero padded
  // ~p locale's AM or PM
  // ~r time, 12 hour clock, same as "~I:~M:~S ~p"
  // ~s number of full seconds since "the epoch" (in UTC)
  // ~S second, zero padded (00...60)
  // ~t horizontal tab
  // ~T time, 24 hour clock, same as "~H:~M:~S"
  // ~U week number of year with Sunday as first day of week (00...53)
  // ~V week number of year with Monday as first day of week (01...52) !!!!
  // ~w day of week (0...6)
  // ~W week number of year with Monday as first day of week (01...52)
  // ~x week number of year with Monday as first day of week (00...53) !!!!
  // ~X locale's date representation, for example: "07/31/00"
  // ~y last two digits of year (00...99)
  // ~Y year
  // ~z time zone in RFC-822 style
  // ~Z symbol time zone (not-implemented)
  // ~1 ISO-8601 year-month-day format
  // ~2 ISO-8601 hour-minute-second-timezone format
  // ~3 ISO-8601 hour-minute-second format
  // ~4 ISO-8601 year-month-day-hour-minute-second-timezone format
  // ~5 ISO-8601 year-month-day-hour-minute-second format
  


#warning "Implement This!: date->string"
  return signal_error(vm, "date->string: not implemented yet: ", Cell::f());
}


Cell *
SRFI19Procedure::string_to_date(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data)
{
#warning "Implement This!: string->date"
  return signal_error(vm, "string->date: not implemented yet: ", Cell::f());
}


}
