/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: srfi19_procedure.h,v 1.5 2009/01/01 07:47:01 fuyu Exp $
 */

#ifndef _LIBTSCORE_SRFI19_PROCEDURE_H_
#define _LIBTSCORE_SRFI19_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{

  /*
   * SRFI19Procedure implements Time Data Type and Procedures defined
   * in SRFI-19.
   */
  class SRFI19Procedure : public NativeProcedure
  {
  private:
    static Info procedures[];


    enum {
      TIME_TYPE_TIME_DURATION = 0,
      TIME_TYPE_TIME_MONOTONIC,
      TIME_TYPE_TIME_PROCESS,
      TIME_TYPE_TIME_TAI,
      TIME_TYPE_TIME_THREAD,
      TIME_TYPE_TIME_UTC,
      NUMBER_OF_TIME_TYPES
    };


    Cell *type_table[NUMBER_OF_TIME_TYPES];

    long determine_time_type(Cell *symbol);


    static Cell *utc_to_date(VirtualMachine &vm, long nanosecond, long second,
			     long zone_offset);


  public:
    SRFI19Procedure(TSCore &core);
    virtual ~SRFI19Procedure();

    virtual Cell *assign_procedures(Cell *environment, Cell *frame);


    // Current time and clock resolution
    static Cell *current_date(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *current_julian_day(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *current_modified_julian_day(VirtualMachine &vm,
					     Register &context,
					     unsigned long argument_pointer,
					     unsigned long nargs, void *data);
    static Cell *current_time(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *time_resolution(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);

    // Time object and accessors
    static Cell *make_time(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *is_time(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *time_type(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *time_nanosecond(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *time_second(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *set_time_type(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *set_time_nanosecond(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data);
    static Cell *set_time_second(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *copy_time(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);

    // Time comparison procedures
    static Cell *is_time_le(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_time_lt(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_time_eq(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_time_ge(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_time_gt(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);

    // Time arithmetic procedures
    static Cell *time_difference(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *time_difference_ex(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *add_duration(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *add_duration_ex(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *subtract_duration(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data);
    static Cell *subtract_duration_ex(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);

    // Date object and accessors
    static Cell *make_date(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *is_date(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *date_nanosecond(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *date_second(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *date_minute(VirtualMachine &vm, Register &context,
			     unsigned long argument_pointer,
			     unsigned long nargs, void *data);
    static Cell *date_hour(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *date_day(VirtualMachine &vm, Register &context,
			  unsigned long argument_pointer,
			  unsigned long nargs, void *data);
    static Cell *date_month(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *date_year(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *date_zone_offset(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *date_year_day(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *date_week_day(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *date_week_number(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);

    // Time/Date/Julian Day/Modified Julian Day Converters
    static Cell *date_to_julian_day(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *date_to_modified_julian_day(VirtualMachine &vm,
					     Register &context,
					     unsigned long argument_pointer,
					     unsigned long nargs, void *data);
    static Cell *date_to_time_monotonic(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *date_to_time_tai(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *date_to_time_utc(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *julian_day_to_date(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *julian_day_to_time_monotonic(VirtualMachine &vm,
					      Register &context,
					      unsigned long argument_pointer,
					      unsigned long nargs, void *data);
    static Cell *julian_day_to_time_tai(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *julian_day_to_time_utc(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *modified_julian_day_to_date(VirtualMachine &vm,
					     Register &context,
					     unsigned long argument_pointer,
					     unsigned long nargs, void *data);
    static Cell *modified_julian_day_to_time_monotonic(VirtualMachine &vm,
						Register &context,
						unsigned long argument_pointer,
						unsigned long nargs,
						void *data);
    static Cell *modified_julian_day_to_time_tai(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data);
    static Cell *modified_julian_day_to_time_utc(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data);
    static Cell *time_monotonic_to_date(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *time_monotonic_to_julian_day(VirtualMachine &vm,
					      Register &context,
					      unsigned long argument_pointer,
					      unsigned long nargs, void *data);
    static Cell *time_monotonic_to_modified_julian_day(VirtualMachine &vm,
						Register &context,
						unsigned long argument_pointer,
						unsigned long nargs,
						void *data);
    static Cell *time_monotonic_to_time_tai(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data);
    static Cell *time_monotonic_to_time_tai_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data);
    static Cell *time_monotonic_to_time_utc(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data);
    static Cell *time_monotonic_to_time_utc_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data);
    static Cell *time_tai_to_date(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *time_tai_to_julian_day(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *time_tai_to_modified_julian_day(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data);
    static Cell *time_tai_to_time_monotonic(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data);
    static Cell *time_tai_to_time_monotonic_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data);
    static Cell *time_tai_to_time_utc(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);
    static Cell *time_tai_to_time_utc_ex(VirtualMachine &vm, Register &context,
					 unsigned long argument_pointer,
					 unsigned long nargs, void *data);
    static Cell *time_utc_to_date(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *time_utc_to_julian_day(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *time_utc_to_modified_julian_day(VirtualMachine &vm,
						 Register &context,
						 unsigned long argument_pointer,
						 unsigned long nargs,
						 void *data);
    static Cell *time_utc_to_time_monotonic(VirtualMachine &vm,
					    Register &context,
					    unsigned long argument_pointer,
					    unsigned long nargs, void *data);
    static Cell *time_utc_to_time_monotonic_ex(VirtualMachine &vm,
					       Register &context,
					       unsigned long argument_pointer,
					       unsigned long nargs, void *data);
    static Cell *time_utc_to_time_tai(VirtualMachine &vm,
				      Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);
    static Cell *time_utc_to_time_tai_ex(VirtualMachine &vm,
					 Register &context,
					 unsigned long argument_pointer,
					 unsigned long nargs, void *data);

    // Date to String/String to Date Converters
    static Cell *date_to_string(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
    static Cell *string_to_date(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
  };


}


#endif	/* __cplusplus */

#endif	/* _LIBTSCORE_SRFI19_PROCEDURE_H_ */
