/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: srfi6_procedure.cc,v 1.14 2009/01/01 07:47:01 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/srfi6_procedure.h"

#include "libtscore/tscore.h"
#include "libtscore/virtual_machine.h"


namespace LibTSCore
{

NativeProcedure::Info SRFI6Procedure::procedures[] = {
  { &SRFI6Procedure::open_input_string,
    "open-input-string", TST_STRING, 1, 1 },
  { &SRFI6Procedure::open_output_string,
    "open-output-string", TST_NONE, 0, 0 },
  { &SRFI6Procedure::get_output_string,
    "get-output-string", TST_STR_OUTPORT, 1, 1 },
  { &SRFI6Procedure::get_remaining_input_string,
    "get-remaining-input-string", TST_STR_INPORT, 1, 1 },
  { &SRFI6Procedure::clear_output_string,
    "clear-output-string", TST_STR_OUTPORT, 1, 1 }
};


Cell *
SRFI6Procedure::assign_procedures(Cell *environment, Cell *frame)
{
  TSStackRoot env(core, environment);
  TSStackRoot frm(core, frame);
  size_t n_procs = sizeof(procedures) / sizeof(Info);
  for (size_t i = 0; i < n_procs; i++)
    assign_procedure(env, frm, &procedures[i], this);

  return env;
}


#define ARG(n)	context.frame->load_variable(argument_pointer + (n))


Cell *
SRFI6Procedure::open_input_string(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
  Cell *src_str = ARG(0);
  Cell *port = vm.mk_input_string_port(src_str);
  if (port == Cell::nil())
    return Cell::f();

  return port;
}


Cell *
SRFI6Procedure::open_output_string(VirtualMachine &vm, Register &context,
				   unsigned long argument_pointer,
				   unsigned long nargs, void *data)
{
  Cell *port = vm.mk_output_string_port();
  if (port == Cell::nil())
    return Cell::f();
  return port;
}


Cell *
SRFI6Procedure::get_output_string(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data)
{
  Port *port = ARG(0)->get_port();
  return vm.mk_counted_string(port->get_output_string(),
			      port->get_output_strlen());
}


Cell *
SRFI6Procedure::get_remaining_input_string(VirtualMachine &vm,
					   Register &context,
					   unsigned long argument_pointer,
					   unsigned long nargs, void *data)
{
  Port *port = ARG(0)->get_port();
  return vm.mk_counted_string(port->get_remaining_string(),
			      port->get_remaining_strlen());
}


Cell *
SRFI6Procedure::clear_output_string(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data)
{
  Cell *port_cell = ARG(0);
  dynamic_cast<OutputStringPort *>(port_cell->get_port())->clear();
  return port_cell;
}

}
