/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tscore_procedure.h,v 1.14 2009/01/01 07:47:01 fuyu Exp $
 */

#ifndef _LIBTSCORE_TSCORE_PROCEDURE_H_
#define _LIBTSCORE_TSCORE_PROCEDURE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/procedure.h"


using namespace std;
namespace LibTSCore
{
  class MemorySystem;


  /*
   * TSCoreProcedure implements miscellaneous procedures not defined in
   * any standards.
   */
  class TSCoreProcedure : public NativeProcedure
  {
  private:
    static Info procedures[];

    long gensym_count;

    static OnigRegion *regexp_match_region(Cell *regexp, Cell *string,
					   long start_off, long end_off);
    static const char *reflect_match_result(OnigRegion *region,
					    OutputStringPort &string_buf,
					    const char *string,
					    Cell *replacement);
    static Cell *string_split_characters(VirtualMachine &vm, Cell *string);


  public:
    static Cell *cons_asterisk(MemorySystem &memory_system, Cell *&d);


    TSCoreProcedure(TSCore &tscore)
      : NativeProcedure(tscore), gensym_count(0) {}

    Cell *assign_procedures(Cell *environment, Cell *frame);


    static Cell *gc(VirtualMachine &vm, Register &context,
		    unsigned long argument_pointer,
		    unsigned long nargs, void *data);
    static Cell *gc_verbose(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *gensym(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);
    static Cell *get_closure_code(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
    static Cell *is_closure(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_defined(VirtualMachine &vm, Register &context,
			    unsigned long argument_pointer,
			    unsigned long nargs, void *data);
    static Cell *is_environment(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
    static Cell *oblist(VirtualMachine &vm, Register &context,
			unsigned long argument_pointer,
			unsigned long nargs, void *data);

    static Cell *set_input_port(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
    static Cell *set_output_port(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *tracing(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);
    static Cell *bytecompile_tracing(VirtualMachine &vm, Register &context,
				     unsigned long argument_pointer,
				     unsigned long nargs, void *data);
#ifndef NDEBUG
    static Cell *bytecode_tracing(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);
#endif
    static Cell *inspect(VirtualMachine &vm, Register &context,
			 unsigned long argument_pointer,
			 unsigned long nargs, void *data);

    static Cell *current_seconds(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
    static Cell *current_milliseconds(VirtualMachine &vm, Register &context,
				      unsigned long argument_pointer,
				      unsigned long nargs, void *data);

#if 0
    // These should be moved to srfi1_procedure.{h,cc} once we implement that.
    static Cell *cons_asterisk(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *append_in_place(VirtualMachine &vm, Register &context,
				 unsigned long argument_pointer,
				 unsigned long nargs, void *data);
#endif
    static Cell *reverse_in_place(VirtualMachine &vm, Register &context,
				  unsigned long argument_pointer,
				  unsigned long nargs, void *data);

    static Cell *is_regexp(VirtualMachine &vm, Register &context,
			   unsigned long argument_pointer,
			   unsigned long nargs, void *data);
    static Cell *string2regexp(VirtualMachine &vm, Register &context,
			       unsigned long argument_pointer,
			       unsigned long nargs, void *data);
    static Cell *regexp_match_positions(VirtualMachine &vm, Register &context,
					unsigned long argument_pointer,
					unsigned long nargs, void *data);
    static Cell *regexp_match(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *regexp_split(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
    static Cell *regexp_replace(VirtualMachine &vm, Register &context,
				unsigned long argument_pointer,
				unsigned long nargs, void *data);
    static Cell *regexp_replace_all(VirtualMachine &vm, Register &context,
				    unsigned long argument_pointer,
				    unsigned long nargs, void *data);
    static Cell *regexp_quote(VirtualMachine &vm, Register &context,
			      unsigned long argument_pointer,
			      unsigned long nargs, void *data);
  };
}

#endif

#endif	/* _LIBTSCORE_TSCORE_PROCEDURE_H_ */
