/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tscore_syntax.cc,v 1.9 2009/01/01 07:47:01 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "libtscore/tscore_syntax.h"

#include "libtscore/bytecode_buffer.h"
#include "libtscore/syntax.h"
#include "libtscore/virtual_machine.h"
#include "libtscore/writer.h"


#include <cassert>


namespace LibTSCore
{

size_t
SyntaxObject::emit_bytecode(VirtualMachine &vm, Register &context,
			    BytecodeBuffer *buffer,
			    Cell *expression, Cell *environment,
			    Cell *constant_pool, Cell *hint,
			    bool is_tail_context)
{
  assert(expression->is_pair());

  if (CDR(expression) == Cell::nil())
    {
      signal_error(vm, "no syntactic keyword");
      return signal_error(vm, "syntax-object: syntax error: ", expression);
    }

  if (!CADR(expression)->is_symbol())
    {
      signal_error(vm, "not a symbol: ", CADR(expression));
      return signal_error(vm, "syntax-object: syntax error: ", expression);
    }

  if (CDDR(expression) != Cell::nil())
    {
      signal_error(vm, "unexpected form: ", CDDR(expression));
      return signal_error(vm, "syntax-object: syntax error: ", expression);
    }

  const Cell *syntax = environment->lookup(CADR(expression), true);
  if (!syntax->is_syntax())
    {
      signal_error(vm, "not a syntactic keyword: ", CADR(expression));
      return signal_error(vm, "syntax-object: syntax error: ", expression);
    }

  size_t len = emit_literal_expression(vm, context, buffer, syntax,
				       constant_pool);
  assert(len > 0);

  if (!is_tail_context)
    return len;
  return len + buffer->append(OP_APPLY_CONTINUATION0);
}

}
