/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: writer.h,v 1.10 2009/01/01 07:47:01 fuyu Exp $
 */

#ifndef _LIBTSCORE_WRITER_H_
#define _LIBTSCORE_WRITER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/cell.h"
#include "libtscore/number.h"
#include "libtscore/port.h"


using namespace std;
namespace LibTSCore
{
  /*
   * Writer is a S expression writer that write S expression into
   * specified Port in specified manner, i.e. human readable representation
   * or machine readable representation.
   *
   * This is just a building block of TSCore not for users.
   */
  class Writer
  {
  private:
    static void write_atom(const Cell *atom, Port *port,
			   bool external, bool inspection);


  public:
    /*
     * Writes an S expression to given output_port.
     *
     * If `external' is true, the output forms external representation,
     * for `write', it forms human readable representation otherwise, for
     * `display'.
     *
     * This returns Cell::nil() if output of `datum' succeeded,
     * otherwise returns `datum'.  Some Cells cannot be represented
     * as external form at all, e.g. closures and continuations, we
     * want to signal it to users.
     *
     * If external is false and inspection is true, this outputs
     * datum in most unfriendly manner ;-).
     */
    static const Cell *write_datum(const Cell *datum, Port *output_port,
				   bool external, bool inspection = false);


    static const Cell *write_number(const Cell *datum,
				    Port *output_port, long radix);
  };
}

#endif

#endif	/* _LIBTSCORE_WRITER_H_ */
