/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: a_bone_image_db.cc,v 1.5 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "a_bone_image_db.h"

#include "ui_constants.h"

#include "libochusha/misc_utils.h"
#include "libochusha/repository.h"
#include "libochusha/sqlite_utils.h"

#include <cassert>
#include <climits>
#include <cstddef>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <vector>


namespace Ochusha
{

// Ĥϰ(ǡΰѤǤΤ߻)
#define OCHUSHA_A_BONE_IMAGE_TXT	"a-bone-image.txt"
#define OCHUSHA_A_BONE_SERVER_TXT	"a-bone-server.txt"
// Ĥϰ


#define OCHUSHA_A_BONE_IMAGE_URLS_TABLE_DEFINITION "CREATE TABLE IF NOT EXISTS ochusha_a_bone_image_urls (id INTEGER PRIMARY KEY, url TEXT, lastAccessed INTEGER);"
#define OCHUSHA_A_BONE_IMAGE_DOMAINS_TABLE_DEFINITION "CREATE TABLE IF NOT EXISTS ochusha_a_bone_image_domains (id INTEGER PRIMARY KEY, domain TEXT, lastAccessed INTEGER);"

#define COLUMN_ID		0
#define COLUMN_URL		1
#define COLUMN_DOMAIN		1
#define COLUMN_LAST_ACCESSED	2

#define OCHUSHA_A_BONE_IMAGE_URLS_DELETE_ALL	"DELETE FROM ochusha_a_bone_image_urls;"
#define OCHUSHA_A_BONE_IMAGE_DOMAINS_DELETE_ALL	"DELETE FROM ochusha_a_bone_image_domains;"

#define INDEX_ID		1
#define INDEX_URL		2
#define INDEX_DOMAIN		2
#define INDEX_LAST_ACCESSED	3


#define OCHUSHA_A_BONE_IMAGE_URLS_SELECT_ALL			"SELECT * FROM ochusha_a_bone_image_urls;"
#define OCHUSHA_A_BONE_IMAGE_URLS_SELECT_MATCHED_URL		"SELECT * FROM ochusha_a_bone_image_urls WHERE url = ?2;"
#define OCHUSHA_A_BONE_IMAGE_URLS_UPDATE_MATCHED_URL		"UPDATE ochusha_a_bone_image_urls SET lastAccessed = ?3 WHERE id = ?1;"
#define OCHUSHA_A_BONE_IMAGE_URLS_INSERT_NEW_URL		"INSERT INTO ochusha_a_bone_image_urls (url, lastAccessed) VALUES (?2, ?3);"
#define OCHUSHA_A_BONE_IMAGE_URLS_DELETE_MATCHED_URL		"DELETE FROM ochusha_a_bone_image_urls WHERE id = ?1;"


#define OCHUSHA_A_BONE_IMAGE_DOMAINS_SELECT_ALL			"SELECT * FROM ochusha_a_bone_image_domains;"
#define OCHUSHA_A_BONE_IMAGE_DOMAINS_SELECT_MATCHED_DOMAIN	"SELECT * FROM ochusha_a_bone_image_domains WHERE domain = ?2;"
#define OCHUSHA_A_BONE_IMAGE_DOMAINS_UPDATE_MATCHED_DOMAIN	"UPDATE ochusha_a_bone_image_domains SET lastAccessed = ?3 WHERE id = ?1;"
#define OCHUSHA_A_BONE_IMAGE_DOMAINS_INSERT_NEW_DOMAIN		"INSERT INTO ochusha_a_bone_image_domains (domain, lastAccessed) VALUES(?2, ?3);"
#define OCHUSHA_A_BONE_IMAGE_DOMAINS_DELETE_MATCHED_DOMAIN	"DELETE FROM ochusha_a_bone_image_domains WHERE id = ?1;"



ABoneImageDB::ABoneImageDB(Repository &repository)
  : lock("ABoneImageDB")
{
  char pathname[PATH_MAX];
  bool a_bone_image_sqlite_exist
    = repository.file_exist(OCHUSHA_A_BONE_IMAGES_SQLITE);

  if (repository.expand_path(OCHUSHA_A_BONE_IMAGES_SQLITE, pathname, PATH_MAX))
    db_handle = new SQLite3(pathname);
  else
    db_handle = new SQLite3("");

  // table
  db_handle->get_table(OCHUSHA_A_BONE_IMAGE_URLS_TABLE_DEFINITION);
  db_handle->get_table(OCHUSHA_A_BONE_IMAGE_DOMAINS_TABLE_DEFINITION);

  // prepared statementν
  select_all_urls
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_URLS_SELECT_ALL);

  select_matched_url
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_URLS_SELECT_MATCHED_URL);

  insert_new_url
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_URLS_INSERT_NEW_URL);

  update_matched_url_id
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_URLS_UPDATE_MATCHED_URL);

  delete_matched_url_id
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_URLS_DELETE_MATCHED_URL);


  select_all_domains
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_DOMAINS_SELECT_ALL);

  select_matched_domain
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_DOMAINS_SELECT_MATCHED_DOMAIN);

  insert_new_domain
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_DOMAINS_INSERT_NEW_DOMAIN);

  update_matched_domain_id
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_DOMAINS_UPDATE_MATCHED_DOMAIN);

  delete_matched_domain_id
    = db_handle->statement_new(OCHUSHA_A_BONE_IMAGE_DOMAINS_DELETE_MATCHED_DOMAIN);


  if (!a_bone_image_sqlite_exist)
    {
      // dbեƺäΤߡηΥǡɤ߹
      register_a_bone_image_url_from_file(repository);
      register_a_bone_image_domain_from_file(repository);
    }
}


ABoneImageDB::~ABoneImageDB()
{
  delete delete_matched_domain_id;
  delete update_matched_domain_id;
  delete insert_new_domain;
  delete select_matched_domain;
  delete select_all_domains;

  delete delete_matched_url_id;
  delete update_matched_url_id;
  delete insert_new_url;
  delete select_matched_url;
  delete select_all_urls;

  delete db_handle;
}


void
ABoneImageDB::register_a_bone_image_url_from_file(Repository &repository)
{
  // 켰a-bone-image.txtɤSQLite3dbϿ
  int fd = repository.open_file(OCHUSHA_A_BONE_IMAGE_TXT, O_RDONLY);
  if (fd >= 0)
    {
      FILE *file = fdopen(fd, "r");
      if (file != NULL)
	{
	  time_t now = time(NULL);
	  char line_buf[PATH_MAX];
	  line_buf[PATH_MAX - 1] = '\0';
	  while (!feof(file))
	    {
	      char *tmp_str = fgets(line_buf, PATH_MAX - 1, file);
	      if (tmp_str != NULL)
		{
		  tmp_str = strpbrk(line_buf, "\r\n");
		  if (tmp_str != NULL)
		    *tmp_str = '\0';
		  register_a_bone_image_url(line_buf, now);
		}
	    }
	  fclose(file);
	}
      else
	close(fd);
    }
}


void
ABoneImageDB::register_a_bone_image_domain_from_file(Repository &repository)
{
  // 켰a-bone-server.txtɤSQLite3dbϿ
  int fd = repository.open_file(OCHUSHA_A_BONE_SERVER_TXT, O_RDONLY);
  if (fd >= 0)
    {
      FILE *file = fdopen(fd, "r");
      if (file != NULL)
	{
	  time_t now = time(NULL);
	  char line_buf[PATH_MAX];
	  line_buf[PATH_MAX - 1] = '\0';
	  while (!feof(file))
	    {
	      char *tmp_str = fgets(line_buf, PATH_MAX - 1, file);
	      if (tmp_str != NULL)
		{
		  tmp_str = strpbrk(line_buf, "\r\n");
		  if (tmp_str != NULL)
		    *tmp_str = '\0';
		  register_a_bone_image_domain(line_buf, now);
		}
	    }
	  fclose(file);
	}
      else
	close(fd);
    }
}


void
ABoneImageDB::register_a_bone_image_url(const char *url, time_t now)
{
  assert(url != NULL);

  vector<sqlite3_int64> duplicated_entries;

  LockBlock lock_obj(lock);

  select_matched_url->clear_bindings();
  select_matched_url->reset();

  sqlite3_int64 row_id = 0;
  int result = SQLITE_DONE;
  if (select_matched_url->bind_static(INDEX_URL, url))
    {
      while ((result = select_matched_url->step()) == SQLITE_ROW)
	{
	  if (row_id != 0)
	    duplicated_entries.push_back(row_id);
	  row_id = select_matched_url->get_column_int64(COLUMN_ID);
	}
    }

  select_matched_url->clear_bindings();
  select_matched_url->reset();

  if (!duplicated_entries.empty())
    {
      duplicated_entries.push_back(row_id);
      row_id = 0;

      while (!duplicated_entries.empty())
	{
	  delete_matched_url_id->clear_bindings();
	  delete_matched_url_id->reset();

	  if (delete_matched_url_id->bind(INDEX_ID, duplicated_entries.back()))
	    {
	      int del_result;
	      do
		{
		  del_result = delete_matched_url_id->step();
		}
	      while (del_result == SQLITE_ROW);
	    }

	  duplicated_entries.pop_back();
	}

      delete_matched_url_id->clear_bindings();
      delete_matched_url_id->reset();
    }

  if (result == SQLITE_DONE)
    {
      SQLite3Statement *stmt;
      bool bindings_ok = true;

      if (row_id == 0)
	{
	  // INSERT
	  stmt = insert_new_url;
	  stmt->clear_bindings();
	  stmt->reset();

	  bindings_ok = stmt->bind_static(INDEX_URL, url);
	}
      else
	{
	  // UPDATE
	  stmt = update_matched_url_id;
	  stmt->clear_bindings();
	  stmt->reset();

	  bindings_ok = stmt->bind(INDEX_ID, row_id);
	}

      if (bindings_ok
	  && stmt->bind(INDEX_LAST_ACCESSED, static_cast<sqlite3_int64>(now)))
	stmt->step();

      stmt->clear_bindings();
      stmt->reset();
    }
}


void
ABoneImageDB::register_a_bone_image_domain(const char *domain, time_t now)
{
  assert(domain != NULL);

  vector<sqlite3_int64> duplicated_entries;

  LockBlock lock_obj(lock);

  select_matched_domain->reset();
  select_matched_domain->clear_bindings();

  sqlite3_int64 row_id = 0;
  int result = SQLITE_DONE;
  if (select_matched_domain->bind_static(INDEX_DOMAIN, domain))
    {
      while ((result = select_matched_domain->step()) == SQLITE_ROW)
	{
	  if (row_id != 0)
	    duplicated_entries.push_back(row_id);
	  row_id = select_matched_domain->get_column_int64(COLUMN_ID);
	}
    }

  select_matched_domain->clear_bindings();
  select_matched_domain->reset();

  if (!duplicated_entries.empty())
    {
      duplicated_entries.push_back(row_id);
      row_id = 0;

      while (!duplicated_entries.empty())
	{
	  delete_matched_domain_id->clear_bindings();
	  delete_matched_domain_id->reset();

	  if (delete_matched_domain_id->bind(INDEX_ID,
					     duplicated_entries.back()))
	    {
	      int del_result;
	      do
		{
		  del_result = delete_matched_domain_id->step();
		}
	      while (del_result == SQLITE_ROW);
	    }

	  duplicated_entries.pop_back();
	}

      delete_matched_domain_id->clear_bindings();
      delete_matched_domain_id->reset();
    }

  if (result == SQLITE_DONE)
    {
      SQLite3Statement *stmt;
      bool bindings_ok = true;

      if (row_id == 0)
	{
	  // INSERT
	  stmt = insert_new_domain;
	  stmt->clear_bindings();
	  stmt->reset();

	  bindings_ok = stmt->bind_static(INDEX_URL, domain);
	}
      else
	{
	  // UPDATE
	  stmt = update_matched_domain_id;
	  stmt->clear_bindings();
	  stmt->reset();

	  bindings_ok = stmt->bind(INDEX_ID, row_id);
	}

      if (bindings_ok
	  && stmt->bind(INDEX_LAST_ACCESSED, static_cast<sqlite3_int64>(now)))
	stmt->step();

      stmt->clear_bindings();
      stmt->reset();
    }
}


bool
ABoneImageDB::is_a_boned_url(const char *url)
{
  LockBlock lock_obj(lock);

  select_matched_url->reset();
  select_matched_url->clear_bindings();

  sqlite3_int64 row_id = 0;
  int result;
  if (select_matched_url->bind_static(INDEX_URL, url))
    {
      while ((result = select_matched_url->step()) == SQLITE_ROW)
	{
	  assert(row_id == 0);

	  row_id = select_matched_url->get_column_int64(COLUMN_ID);

	  update_matched_url_id->reset();
	  update_matched_url_id->clear_bindings();

	  if (update_matched_url_id->bind(INDEX_ID, row_id)
	      && update_matched_url_id->bind(INDEX_LAST_ACCESSED,
					     static_cast<sqlite3_int64>(time(NULL))))
	    update_matched_url_id->step();

	  update_matched_url_id->reset();
	  update_matched_url_id->clear_bindings();
	}
    }
  select_matched_url->clear_bindings();
  select_matched_url->reset();

  return row_id != 0;
}


bool
ABoneImageDB::is_a_boned_domain(const char *domain)
{
  LockBlock lock_obj(lock);

  select_matched_domain->reset();
  select_matched_domain->clear_bindings();

  sqlite3_int64 row_id = 0;
  int result;
  if (select_matched_domain->bind_static(INDEX_DOMAIN, domain))
    {
      while ((result = select_matched_domain->step()) == SQLITE_ROW)
	{
	  assert(row_id == 0);

	  row_id = select_matched_domain->get_column_int64(COLUMN_ID);

	  update_matched_domain_id->reset();
	  update_matched_domain_id->clear_bindings();

	  if (update_matched_domain_id->bind(INDEX_ID, row_id)
	      && update_matched_domain_id->bind(INDEX_LAST_ACCESSED,
						static_cast<sqlite3_int64>(time(NULL))))
	    update_matched_domain_id->step();

	  update_matched_domain_id->reset();
	  update_matched_domain_id->clear_bindings();
	}
    }
  select_matched_domain->clear_bindings();
  select_matched_domain->reset();

  return row_id != 0;
}


}
