/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: a_bone_image_db.h,v 1.4 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _A_BONE_IMAGE_DB_H_
#define _A_BONE_IMAGE_DB_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/lock.h"

#ifdef HAVE_TIME_H
#include <time.h>
#endif


using namespace std;
namespace Ochusha
{

  class Repository;
  class SQLite3;
  class SQLite3Statement;

  class ABoneImageDB
  {
  private:
    Lock lock;

    SQLite3 *db_handle;

    SQLite3Statement *select_all_urls;
    SQLite3Statement *select_matched_url;
    SQLite3Statement *insert_new_url;
    SQLite3Statement *update_matched_url_id;
    SQLite3Statement *delete_matched_url_id;

    SQLite3Statement *select_all_domains;
    SQLite3Statement *select_matched_domain;
    SQLite3Statement *insert_new_domain;
    SQLite3Statement *update_matched_domain_id;
    SQLite3Statement *delete_matched_domain_id;

    void register_a_bone_image_url_from_file(Repository &repository);
    void register_a_bone_image_domain_from_file(Repository &repository);


  public:
    ABoneImageDB(Repository &repository);
    ~ABoneImageDB();

    void register_a_bone_image_url(const char *url, time_t now);
    void register_a_bone_image_domain(const char *domain, time_t now);

    bool is_a_boned_url(const char *url);
    bool is_a_boned_domain(const char *url);

  };

}


#endif	/* __cplusplus */

#endif	/* _A_BONE_IMAGE_DB_H_ */
