/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: a_bone_ui.cc,v 1.7 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "a_bone_ui.h"

#include "focus_utils.h"

#include <gtk/gtk.h>

#include <cassert>
#include <cstdio>
#include <cstdlib>


namespace Ochusha
{


ABoneSettings::ABoneSettings(ABone &a_bone, Controller &controller)
  : a_bone(a_bone),
    a_bone_by_name_pattern_current(NULL),
    a_bone_by_mail_pattern_current(NULL),
    a_bone_by_id_pattern_current(NULL),
    a_bone_by_content_pattern_current(NULL)
{
  a_bone_by_name_initial = a_bone.get_a_bone_by_name();
  const char *pattern = a_bone.get_name_pattern();
  a_bone_by_name_pattern_initial = pattern != NULL ? strdup(pattern) : NULL;

  a_bone_by_mail_initial = a_bone.get_a_bone_by_mail();
  pattern = a_bone.get_mail_pattern();
  a_bone_by_mail_pattern_initial = pattern != NULL ? strdup(pattern) : NULL;

  a_bone_by_id_initial = a_bone.get_a_bone_by_id();
  pattern = a_bone.get_id_pattern();
  a_bone_by_id_pattern_initial = pattern != NULL ? strdup(pattern) : NULL;

  a_bone_by_content_initial = a_bone.get_a_bone_by_content();
  pattern = a_bone.get_content_pattern();
  a_bone_by_content_pattern_initial = pattern != NULL ? strdup(pattern) : NULL;


  widget = gtk_vbox_new(FALSE, 0);
  GtkBox *vbox = GTK_BOX(widget);


  GtkWidget *button
    = gtk_check_button_new_with_mnemonic(_("A Bone by _Name Field"));
  gtk_widget_show(button);
  gtk_box_pack_start(vbox, button, FALSE, FALSE, 0);
  a_bone_by_name_button = GTK_TOGGLE_BUTTON(button);

  g_signal_connect(button, "toggled",
		   reinterpret_cast<GCallback>(a_bone_by_name_button_toggled_cb),
		   this);

  GtkWidget *editor = ochusha_regex_editor_new(NULL);
  gtk_widget_show(editor);
  gtk_box_pack_start(vbox, editor, TRUE, TRUE, 0);
  a_bone_by_name_pattern = OCHUSHA_REGEX_EDITOR(editor);

  new TextViewFocusNotifier(controller, a_bone_by_name_pattern->regex_view, true);
  g_signal_connect(a_bone_by_name_pattern->regex_buffer, "changed",
		   reinterpret_cast<GCallback>(a_bone_by_name_pattern_changed_cb),
		   this);



  button = gtk_check_button_new_with_mnemonic(_("A Bone by _E-mail"));
  gtk_widget_show(button);
  gtk_box_pack_start(vbox, button, FALSE, FALSE, 0);
  a_bone_by_mail_button = GTK_TOGGLE_BUTTON(button);

  g_signal_connect(button, "toggled",
		   reinterpret_cast<GCallback>(a_bone_by_mail_button_toggled_cb),
		   this);

  editor = ochusha_regex_editor_new(NULL);
  gtk_widget_show(editor);
  gtk_box_pack_start(vbox, editor, TRUE, TRUE, 0);
  a_bone_by_mail_pattern = OCHUSHA_REGEX_EDITOR(editor);

  new TextViewFocusNotifier(controller, a_bone_by_mail_pattern->regex_view, true);
  g_signal_connect(a_bone_by_mail_pattern->regex_buffer, "changed",
		   reinterpret_cast<GCallback>(a_bone_by_mail_pattern_changed_cb),
		   this);



  button = gtk_check_button_new_with_mnemonic(_("A Bone by _ID"));
  gtk_widget_show(button);
  gtk_box_pack_start(vbox, button, FALSE, FALSE, 0);
  a_bone_by_id_button = GTK_TOGGLE_BUTTON(button);

  g_signal_connect(button, "toggled",
		   reinterpret_cast<GCallback>(a_bone_by_id_button_toggled_cb),
		   this);

  editor = ochusha_regex_editor_new(NULL);
  gtk_widget_show(editor);
  gtk_box_pack_start(vbox, editor, TRUE, TRUE, 0);
  a_bone_by_id_pattern = OCHUSHA_REGEX_EDITOR(editor);

  new TextViewFocusNotifier(controller, a_bone_by_id_pattern->regex_view, true);
  g_signal_connect(a_bone_by_id_pattern->regex_buffer, "changed",
		   reinterpret_cast<GCallback>(a_bone_by_id_pattern_changed_cb),
		   this);



  button = gtk_check_button_new_with_mnemonic(_("A Bone by _Message"));
  gtk_widget_show(button);
  gtk_box_pack_start(vbox, button, FALSE, FALSE, 0);
  a_bone_by_content_button = GTK_TOGGLE_BUTTON(button);

  g_signal_connect(button, "toggled",
		   reinterpret_cast<GCallback>(a_bone_by_content_button_toggled_cb),
		   this);

  editor = ochusha_regex_editor_new(NULL);
  gtk_widget_show(editor);
  gtk_box_pack_start(vbox, editor, TRUE, TRUE, 0);
  a_bone_by_content_pattern = OCHUSHA_REGEX_EDITOR(editor);

  new TextViewFocusNotifier(controller, a_bone_by_content_pattern->regex_view, true);
  g_signal_connect(a_bone_by_content_pattern->regex_buffer, "changed",
		   reinterpret_cast<GCallback>(a_bone_by_content_pattern_changed_cb),
		   this);



  gtk_toggle_button_set_active(a_bone_by_name_button, a_bone_by_name_initial);
  gtk_toggle_button_toggled(a_bone_by_name_button);
  ochusha_regex_editor_set_regex_text(a_bone_by_name_pattern,
				      a_bone_by_name_pattern_initial);
  g_signal_emit_by_name(a_bone_by_name_pattern->regex_buffer, "changed");

  gtk_toggle_button_set_active(a_bone_by_mail_button, a_bone_by_mail_initial);
  gtk_toggle_button_toggled(a_bone_by_mail_button);
  ochusha_regex_editor_set_regex_text(a_bone_by_mail_pattern,
				      a_bone_by_mail_pattern_initial);
  g_signal_emit_by_name(a_bone_by_mail_pattern->regex_buffer, "changed");

  gtk_toggle_button_set_active(a_bone_by_id_button, a_bone_by_id_initial);
  gtk_toggle_button_toggled(a_bone_by_id_button);
  ochusha_regex_editor_set_regex_text(a_bone_by_id_pattern,
				      a_bone_by_id_pattern_initial);
  g_signal_emit_by_name(a_bone_by_id_pattern->regex_buffer, "changed");

  gtk_toggle_button_set_active(a_bone_by_content_button,
			       a_bone_by_content_initial);
  gtk_toggle_button_toggled(a_bone_by_content_button);
  ochusha_regex_editor_set_regex_text(a_bone_by_content_pattern,
				      a_bone_by_content_pattern_initial);
  g_signal_emit_by_name(a_bone_by_content_pattern->regex_buffer, "changed");
}


ABoneSettings::~ABoneSettings()
{
  if (a_bone_by_name_pattern_initial != NULL)
    free(a_bone_by_name_pattern_initial);

  if (a_bone_by_name_pattern_current != NULL)
    free(a_bone_by_name_pattern_current);

  if (a_bone_by_mail_pattern_initial != NULL)
    free(a_bone_by_mail_pattern_initial);

  if (a_bone_by_mail_pattern_current != NULL)
    free(a_bone_by_mail_pattern_current);

  if (a_bone_by_id_pattern_initial != NULL)
    free(a_bone_by_id_pattern_initial);

  if (a_bone_by_id_pattern_current != NULL)
    free(a_bone_by_id_pattern_current);

  if (a_bone_by_content_pattern_initial != NULL)
    free(a_bone_by_content_pattern_initial);

  if (a_bone_by_content_pattern_current != NULL)
    free(a_bone_by_content_pattern_current);
}


void
ABoneSettings::a_bone_by_name_button_toggled_cb(GtkToggleButton *button,
						ABoneSettings *settings)
{
  gboolean active = gtk_toggle_button_get_active(button);
  settings->a_bone.set_a_bone_by_name(active);
  gtk_widget_set_sensitive(GTK_WIDGET(settings->a_bone_by_name_pattern),
			   active);
}


void
ABoneSettings::a_bone_by_name_pattern_changed_cb(GtkTextBuffer *buffer,
						 ABoneSettings *settings)
{
  OchushaRegexEditor *editor = settings->a_bone_by_name_pattern;
  char *pattern = ochusha_regex_editor_get_regex_text(editor);
  if (settings->a_bone_by_name_pattern_current == NULL
      && pattern == NULL)
    return;

  if (pattern == NULL)
    {
      free(settings->a_bone_by_name_pattern_current);
      settings->a_bone_by_name_pattern_current = NULL;
      settings->a_bone.set_name_pattern(pattern);
      return;
    }

  if (settings->a_bone_by_name_pattern_current == NULL)
    {
      settings->a_bone_by_name_pattern_current = pattern;
      settings->a_bone.set_name_pattern(pattern);
      return;
    }

  if (strcmp(settings->a_bone_by_name_pattern_current, pattern) == 0)
    {
      free(pattern);
      return;
    }

  free(settings->a_bone_by_name_pattern_current);
  settings->a_bone_by_name_pattern_current = pattern;
  settings->a_bone.set_name_pattern(pattern);
}


void
ABoneSettings::a_bone_by_mail_button_toggled_cb(GtkToggleButton *button,
						ABoneSettings *settings)
{
  gboolean active = gtk_toggle_button_get_active(button);
  settings->a_bone.set_a_bone_by_mail(active);
  gtk_widget_set_sensitive(GTK_WIDGET(settings->a_bone_by_mail_pattern),
			   active);
}


void
ABoneSettings::a_bone_by_mail_pattern_changed_cb(GtkTextBuffer *buffer,
						 ABoneSettings *settings)
{
  OchushaRegexEditor *editor = settings->a_bone_by_mail_pattern;
  char *pattern = ochusha_regex_editor_get_regex_text(editor);
  if (settings->a_bone_by_mail_pattern_current == NULL
      && pattern == NULL)
    return;

  if (pattern == NULL)
    {
      free(settings->a_bone_by_mail_pattern_current);
      settings->a_bone_by_mail_pattern_current = NULL;
      settings->a_bone.set_mail_pattern(pattern);
      return;
    }

  if (settings->a_bone_by_mail_pattern_current == NULL)
    {
      settings->a_bone_by_mail_pattern_current = pattern;
      settings->a_bone.set_mail_pattern(pattern);
      return;
    }

  if (strcmp(settings->a_bone_by_mail_pattern_current, pattern) == 0)
    {
      free(pattern);
      return;
    }

  free(settings->a_bone_by_mail_pattern_current);
  settings->a_bone_by_mail_pattern_current = pattern;
  settings->a_bone.set_mail_pattern(pattern);
}


void
ABoneSettings::a_bone_by_id_button_toggled_cb(GtkToggleButton *button,
					      ABoneSettings *settings)
{
  gboolean active = gtk_toggle_button_get_active(button);
  settings->a_bone.set_a_bone_by_id(active);
  gtk_widget_set_sensitive(GTK_WIDGET(settings->a_bone_by_id_pattern), active);
}


void
ABoneSettings::a_bone_by_id_pattern_changed_cb(GtkTextBuffer *buffer,
					       ABoneSettings *settings)
{
  OchushaRegexEditor *editor = settings->a_bone_by_id_pattern;
  char *pattern = ochusha_regex_editor_get_regex_text(editor);
  if (settings->a_bone_by_id_pattern_current == NULL && pattern == NULL)
    return;

  if (pattern == NULL)
    {
      free(settings->a_bone_by_id_pattern_current);
      settings->a_bone_by_id_pattern_current = NULL;
      settings->a_bone.set_id_pattern(pattern);
      return;
    }

  if (settings->a_bone_by_id_pattern_current == NULL)
    {
      settings->a_bone_by_id_pattern_current = pattern;
      settings->a_bone.set_id_pattern(pattern);
      return;
    }

  if (strcmp(settings->a_bone_by_id_pattern_current, pattern) == 0)
    {
      free(pattern);
      return;
    }

  free(settings->a_bone_by_id_pattern_current);
  settings->a_bone_by_id_pattern_current = pattern;
  settings->a_bone.set_id_pattern(pattern);
}


void
ABoneSettings::a_bone_by_content_button_toggled_cb(GtkToggleButton *button,
						   ABoneSettings *settings)
{
  gboolean active = gtk_toggle_button_get_active(button);
  settings->a_bone.set_a_bone_by_content(active);
  gtk_widget_set_sensitive(GTK_WIDGET(settings->a_bone_by_content_pattern),
			   active);
}


void
ABoneSettings::a_bone_by_content_pattern_changed_cb(GtkTextBuffer *buffer,
						    ABoneSettings *settings)
{
  OchushaRegexEditor *editor = settings->a_bone_by_content_pattern;
  char *pattern = ochusha_regex_editor_get_regex_text(editor);
  if (settings->a_bone_by_content_pattern_current == NULL && pattern == NULL)
    return;

  if (pattern == NULL)
    {
      free(settings->a_bone_by_content_pattern_current);
      settings->a_bone_by_content_pattern_current = NULL;
      settings->a_bone.set_content_pattern(pattern);
      return;
    }

  if (settings->a_bone_by_content_pattern_current == NULL)
    {
      settings->a_bone_by_content_pattern_current = pattern;
      settings->a_bone.set_content_pattern(pattern);
      return;
    }

  if (strcmp(settings->a_bone_by_content_pattern_current, pattern) == 0)
    {
      free(pattern);
      return;
    }

  free(settings->a_bone_by_content_pattern_current);
  settings->a_bone_by_content_pattern_current = pattern;
  settings->a_bone.set_content_pattern(pattern);
}


void
ABoneSettings::cancel_settings()
{
  a_bone.set_a_bone_by_name(a_bone_by_name_initial);
  gtk_toggle_button_set_active(a_bone_by_name_button, a_bone_by_name_initial);

  //a_bone.set_name_pattern(a_bone_by_name_pattern_initial);
  ochusha_regex_editor_set_regex_text(a_bone_by_name_pattern,
				      a_bone_by_name_pattern_initial);


  a_bone.set_a_bone_by_mail(a_bone_by_mail_initial);
  gtk_toggle_button_set_active(a_bone_by_mail_button, a_bone_by_mail_initial);

  //a_bone.set_mail_pattern(a_bone_by_mail_pattern_initial);
  ochusha_regex_editor_set_regex_text(a_bone_by_mail_pattern,
				      a_bone_by_mail_pattern_initial);


  a_bone.set_a_bone_by_id(a_bone_by_id_initial);
  gtk_toggle_button_set_active(a_bone_by_id_button, a_bone_by_id_initial);

  //a_bone.set_id_pattern(a_bone_by_id_pattern_initial);
  ochusha_regex_editor_set_regex_text(a_bone_by_id_pattern,
				      a_bone_by_id_pattern_initial);


  a_bone.set_a_bone_by_content(a_bone_by_content_initial);
  gtk_toggle_button_set_active(a_bone_by_content_button,
			       a_bone_by_content_initial);

  //a_bone.set_content_pattern(a_bone_by_content_pattern_initial);
  ochusha_regex_editor_set_regex_text(a_bone_by_content_pattern,
				      a_bone_by_content_pattern_initial);
}


void
ABoneSettings::append_name_to_be_a_boned(const char *name)
{
  gtk_toggle_button_set_active(a_bone_by_name_button, TRUE);
  ochusha_regex_editor_append_pattern_escaped(a_bone_by_name_pattern, name);
}


void
ABoneSettings::append_mail_to_be_a_boned(const char *mail)
{
  gtk_toggle_button_set_active(a_bone_by_mail_button, TRUE);
  ochusha_regex_editor_append_pattern_escaped(a_bone_by_mail_pattern, mail);
}


void
ABoneSettings::append_id_to_be_a_boned(const char *id)
{
  gtk_toggle_button_set_active(a_bone_by_id_button, TRUE);
  ochusha_regex_editor_append_pattern_escaped(a_bone_by_id_pattern, id);
}


void
ABoneSettings::append_content_to_be_a_boned(const char *content)
{
  gtk_toggle_button_set_active(a_bone_by_content_button, TRUE);
  ochusha_regex_editor_append_pattern_escaped(a_bone_by_content_pattern,
					      content);
}


}
