/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: a_bone_ui.h,v 1.7 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _A_BONE_UI_H_
#define _A_BONE_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/a_bone.h"
#include "libochushagtk/regex_editor.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  class Controller;

  class ABoneSettings
  {
  private:
    ABone &a_bone;

    GtkWidget *widget;

    GtkToggleButton *a_bone_by_name_button;
    OchushaRegexEditor *a_bone_by_name_pattern;

    GtkToggleButton *a_bone_by_mail_button;
    OchushaRegexEditor *a_bone_by_mail_pattern;

    GtkToggleButton *a_bone_by_id_button;
    OchushaRegexEditor *a_bone_by_id_pattern;

    GtkToggleButton *a_bone_by_content_button;
    OchushaRegexEditor *a_bone_by_content_pattern;

    bool a_bone_by_name_initial;
    char *a_bone_by_name_pattern_initial;
    char *a_bone_by_name_pattern_current;
    
    bool a_bone_by_mail_initial;
    char *a_bone_by_mail_pattern_initial;
    char *a_bone_by_mail_pattern_current;
    
    bool a_bone_by_id_initial;
    char *a_bone_by_id_pattern_initial;
    char *a_bone_by_id_pattern_current;

    bool a_bone_by_content_initial;
    char *a_bone_by_content_pattern_initial;
    char *a_bone_by_content_pattern_current;


    static void a_bone_by_name_button_toggled_cb(GtkToggleButton *button,
						 ABoneSettings *settings);
    static void a_bone_by_name_pattern_changed_cb(GtkTextBuffer *buffer,
						  ABoneSettings *settings);
    static void a_bone_by_mail_button_toggled_cb(GtkToggleButton *button,
						 ABoneSettings *settings);
    static void a_bone_by_mail_pattern_changed_cb(GtkTextBuffer *buffer,
						  ABoneSettings *settings);
    static void a_bone_by_id_button_toggled_cb(GtkToggleButton *button,
					       ABoneSettings *settings);
    static void a_bone_by_id_pattern_changed_cb(GtkTextBuffer *buffer,
						ABoneSettings *settings);
    static void a_bone_by_content_button_toggled_cb(GtkToggleButton *button,
						    ABoneSettings *settings);
    static void a_bone_by_content_pattern_changed_cb(GtkTextBuffer *buffer,
						     ABoneSettings *settings);


  public:
    ABoneSettings(ABone &a_bone, Controller &controller);
    ~ABoneSettings();

    inline GtkWidget *get_widget() { return widget; }
    void cancel_settings();

    void append_name_to_be_a_boned(const char *id);
    void append_mail_to_be_a_boned(const char *id);
    void append_id_to_be_a_boned(const char *id);
    void append_content_to_be_a_boned(const char *id);
  };

}


#endif	/* __cplusplus */

#endif	/* _A_BONE_UI_H_ */
