/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: about_ui.cc,v 1.20 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "about_ui.h"

#include "controller.h"

#include <gtk/gtk.h>


#ifdef HAVE_TIME_H
#include <time.h>
#endif


#undef NEW_YEAR_SPECIAL


namespace Ochusha
{

AboutUI::AboutUI(Controller &controller)
  : controller(controller)
{
}


AboutUI::~AboutUI()
{
}


static const char *copyright = "Copyright (c) 2003-2009 The Ochusha Project";
static const char *license =
  "Copyright (c) 2003-2009 The Ochusha Project.\n"
  "All rights reserved.\n"
  "\n"
  "Redistribution and use in source and binary forms, with or without\n"
  "modification, are permitted provided that the following conditions\n"
  "are met:\n"
  "1. Redistributions of source code must retain the above copyright\n"
  "   notice, this list of conditions and the following disclaimer.\n"
  "2. Redistributions in binary form must reproduce the above copyright\n"
  "   notice, this list of conditions and the following disclaimer in the\n"
  "   documentation and/or other materials provided with the distribution.\n"
  "\n"
  "THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND\n"
  "ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\n"
  "IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE\n"
  "ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE\n"
  "FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\n"
  "DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS\n"
  "OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)\n"
  "HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT\n"
  "LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY\n"
  "OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF\n"
  "SUCH DAMAGE."
  "\n\n\n"
  "This product includes OniGuruma.  (http://www.geocities.jp/kosako3/oniguruma/)\n"
  "Copyright (c) 2002-2007  K.Kosako <sndgk393 AT ybb DOT ne DOT jp>\n";


#include "ochusha-banner.h"

#ifdef NEW_YEAR_SPECIAL
#include "ochusha-banner1.h"
#endif

#include "ochusha-banner2.h"


void
AboutUI::link_activated(GtkAboutDialog *about, const gchar *link,
			AboutUI *ui)
{
  ui->controller.open(link, false, true, false);
}


void
AboutUI::show()
{
  struct tm *tm_p;
  time_t utc = time(NULL);
#ifdef HAVE_LOCALTIME_R
  struct tm tm;
  tm_p = localtime_r(&utc, &tm);
#else
  tm_p = localtime(&utc);
#endif

  const guint8 *data;
  if (tm_p->tm_yday == 44
      || (tm_p->tm_mon == 11 && tm_p->tm_mday >= 24 && tm_p->tm_mday <= 25))
    data = ochusha_banner2_inline;
#ifdef NEW_YEAR_SPECIAL
  else if (tm_p->tm_year == 104 && tm_p->tm_yday < 3)
    data = ochusha_banner1_inline;
#endif
  else
    data = ochusha_banner_inline;

  GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline(-1, data, FALSE, NULL);
  gtk_about_dialog_set_url_hook(reinterpret_cast<GtkAboutDialogActivateLinkFunc>(link_activated),
				this, NULL);
  gtk_show_about_dialog(controller.get_toplevel(),
#if GTK_CHECK_VERSION(2,12,0)
			"program-name", N_("Ochusha"),
#else
			"name", N_("Ochusha"),
#endif
			"version", VERSION,
			"copyright", copyright,
			"license", license,
			"website", "http://ochusha.sourceforge.jp",
			"logo", pixbuf,
			"title", _("About Ochusha"),
			NULL);

  g_object_unref(pixbuf);
}

}
