/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_2ch_helper.cc,v 1.19 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "bbs_2ch_helper.h"

#include "libochusha/bbs_2ch_utils.h"
#include "libochusha/misc_utils.h"

#include "controller.h"
#include "focus_utils.h"
#include "session_client.h"
#include "ui_constants.h"


#include <gtk/gtk.h>

#include <cstring>


namespace Ochusha
{

static void
button_toggled_cb(GtkToggleButton *button, GtkToggleAction *action)
{
  gtk_toggle_action_set_active(action, gtk_toggle_button_get_active(button));
}


GtkWidget *
BBS2chHelper::create_auth_dialog(const gchar *title, const char *user_id,
				 const char *passwd, GtkWindow *toplevel,
				 GtkToggleAction *action)
{
  GtkWidget *dialog
    = gtk_dialog_new_with_buttons(title, toplevel,
				  GTK_DIALOG_DESTROY_WITH_PARENT,
				  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				  GTK_STOCK_OK, GTK_RESPONSE_OK,
				  NULL);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

  GtkWidget *outer_hbox = gtk_hbox_new(FALSE, 0);
  GtkWidget *icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_AUTHENTICATION,
					     GTK_ICON_SIZE_DIALOG);
  gtk_widget_show(icon);
  gtk_box_pack_start(GTK_BOX(outer_hbox), icon, FALSE, FALSE, 5);

  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);

  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);

  GtkWidget *label = gtk_label_new_with_mnemonic(_("User _ID:"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);

  GtkWidget *entry = EntryFocusNotifier::gtk_entry_new(controller);
  if (user_id != NULL)
    gtk_entry_set_text(GTK_ENTRY(entry), user_id);
  gtk_widget_show(entry);
  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 5);
  gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 32);
  gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
  g_object_set_data(G_OBJECT(dialog), "user_id", entry);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 5);


  hbox = gtk_hbox_new(FALSE, 0);

  label = gtk_label_new_with_mnemonic(_("_Password:"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);

  entry = EntryFocusNotifier::gtk_entry_new(controller);
  if (passwd != NULL)
    gtk_entry_set_text(GTK_ENTRY(entry), passwd);
  gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
  gtk_widget_show(entry);
  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 5);
  gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
  gtk_entry_set_width_chars(GTK_ENTRY(entry), 32);
  gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
  g_object_set_data(G_OBJECT(dialog), "passwd", entry);

  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(outer_vbox), hbox, FALSE, FALSE, 5);

  GtkWidget *button
    = gtk_check_button_new_with_mnemonic(_("_Remember Password"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),
			       gtk_toggle_action_get_active(action));
  g_signal_connect(button, "toggled",
		   reinterpret_cast<GCallback>(button_toggled_cb), action);
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(outer_vbox), button, FALSE, FALSE, 5);

  gtk_widget_show(outer_vbox);
  gtk_box_pack_start(GTK_BOX(outer_hbox), outer_vbox, TRUE, TRUE, 5);
  gtk_widget_show(outer_hbox);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     outer_hbox, TRUE, TRUE, 0);

  return dialog;
}


BBS2chHelper::BBS2chHelper(Controller &controller)
  : bbs2ch(new BBS2ch(controller.get_config())),
    controller(controller),
    action_group(create_action_group()),
    be_id(NULL),
    be_passwd(NULL),
    viewer_id(NULL),
    viewer_passwd(NULL),
    auth_be_dialog(NULL),
    auth_viewer_dialog(NULL)
{
  g_signal_connect(controller.get_prefs().save_2ch_be_passwd, "toggled",
		   reinterpret_cast<GCallback>(save_2ch_be_passwd_toggled_cb),
		   this);
  g_signal_connect(controller.get_prefs().save_2ch_viewer_passwd, "toggled",
		   reinterpret_cast<GCallback>(save_2ch_viewer_passwd_toggled_cb), this);

  SessionClient &session = controller.get_session();
  set_be_id(session.get_state("BBS2chHelper::be_id"));
  set_be_passwd(session.get_state("BBS2chHelper::be_passwd"));
  set_viewer_id(session.get_state("BBS2chHelper::viewer_id"));
  set_viewer_passwd(session.get_state("BBS2chHelper::viewer_passwd"));


  GDK_THREADS_LEAVE();

  bbs2ch->login_2ch_be_failed
    .connect(this, &BBS2chHelper::login_2ch_be_failed_cb);
  bbs2ch->login_2ch_be_finished
    .connect(this, &BBS2chHelper::login_2ch_be_finished_cb);
  bbs2ch->login_2ch_viewer_failed
    .connect(this, &BBS2chHelper::login_2ch_viewer_failed_cb);
  bbs2ch->login_2ch_viewer_finished
    .connect(this, &BBS2chHelper::login_2ch_viewer_finished_cb);

  GDK_THREADS_ENTER();
}


BBS2chHelper::~BBS2chHelper()
{
  assert(OCHUSHA_IN_GDK_THREADS());

  g_object_unref(action_group);


  GDK_THREADS_LEAVE();

  delete bbs2ch;

  GDK_THREADS_ENTER();
}


GtkActionGroup *
BBS2chHelper::create_action_group()
{
  static GtkToggleActionEntry entries[] = {
    {
      "Login2chBe", NULL,
      N_("Login 2ch _Be"), NULL,
      N_("Login 2ch Be"),
      reinterpret_cast<GCallback>(login_2ch_be_toggled_cb),
      FALSE
    },
    {
      "Login2chViewer", NULL,
      N_("Login 2ch _Viewer"), NULL,
      N_("Login 2ch Viewer"),
      reinterpret_cast<GCallback>(login_2ch_viewer_toggled_cb),
      FALSE
    }
  };

  action_group = gtk_action_group_new("BBS2chHelperActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_toggle_actions(action_group, entries,
				      G_N_ELEMENTS(entries), this);

  login_2ch_be_action
    = GTK_TOGGLE_ACTION(gtk_action_group_get_action(action_group,
						    "Login2chBe"));
  login_2ch_viewer_action
    = GTK_TOGGLE_ACTION(gtk_action_group_get_action(action_group,
						    "Login2chViewer"));

  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  return action_group;
}


void
BBS2chHelper::set_be_id(const char *id)
{
  if (be_id != NULL)
    free(be_id);

  if (id != NULL && *id != '\0')
    {
      be_id = strdup(id);
      controller.get_session().set_state("BBS2chHelper::be_id", be_id);
    }
  else
    {
      be_id = NULL;
      controller.get_session().remove_state("BBS2chHelper::be_id");
    }
}


void
BBS2chHelper::set_be_passwd(const char *passwd)
{
  if (be_passwd != NULL)
    free(be_passwd);

  if (passwd != NULL && *passwd != '\0')
    be_passwd = strdup(passwd);
  else
    be_passwd = NULL;

  if (be_passwd != NULL && controller.get_prefs().get_save_2ch_be_passwd())
    controller.get_session().set_state("BBS2chHelper::be_passwd", be_passwd);
  else
    controller.get_session().remove_state("BBS2chHelper::be_passwd");
}


void
BBS2chHelper::set_viewer_id(const char *id)
{
  if (viewer_id != NULL)
    free(viewer_id);

  if (id != NULL && *id != '\0')
    {
      viewer_id = strdup(id);
      controller.get_session().set_state("BBS2chHelper::viewer_id", viewer_id);
    }
  else
    {
      viewer_id = NULL;
      controller.get_session().remove_state("BBS2chHelper::viewer_id");
    }
}


void
BBS2chHelper::set_viewer_passwd(const char *passwd)
{
  if (viewer_passwd != NULL)
    free(viewer_passwd);

  if (passwd != NULL && *passwd != '\0')
    viewer_passwd = strdup(passwd);
  else
    viewer_passwd = NULL;

  if (viewer_passwd != NULL
      && controller.get_prefs().get_save_2ch_viewer_passwd())
    controller.get_session().set_state("BBS2chHelper::viewer_passwd",
				       viewer_passwd);
  else
    controller.get_session().remove_state("BBS2chHelper::viewer_passwd");
}


void
BBS2chHelper::save_2ch_be_passwd_toggled_cb(GtkToggleAction *action,
					    BBS2chHelper *helper)
{
  if (helper->be_passwd != NULL && gtk_toggle_action_get_active(action))
    helper->controller.get_session().set_state("BBS2chHelper::be_passwd",
					       helper->be_passwd);
  else
    helper->controller.get_session().remove_state("BBS2chHelper::be_passwd");
}


void
BBS2chHelper::save_2ch_viewer_passwd_toggled_cb(GtkToggleAction *action,
						BBS2chHelper *helper)
{
  if (helper->viewer_passwd != NULL && gtk_toggle_action_get_active(action))
    helper->controller.get_session().set_state("BBS2chHelper::viewer_passwd",
					       helper->viewer_passwd);
  else
    helper->controller.get_session().remove_state("BBS2chHelper::viewer_passwd");
}


void
BBS2chHelper::login_2ch_be_toggled_cb(GtkToggleAction *action,
				      BBS2chHelper *helper)
{
  if (gtk_toggle_action_get_active(action))
    helper->login_2ch_be();
  else
    helper->logout_2ch_be();
}


void
BBS2chHelper::login_2ch_be_failed_cb(const char *reason)
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  GtkWidget *dialog = gtk_message_dialog_new(controller.get_toplevel(),
					     GTK_DIALOG_DESTROY_WITH_PARENT,
					     GTK_MESSAGE_WARNING,
					     GTK_BUTTONS_OK_CANCEL,
					     _("2ch Be Login failed"));
  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", reason);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(login_2ch_be_failure_info_response_cb),
		   this);

  gtk_widget_show(dialog);

  GDK_THREADS_LEAVE();
}


void
BBS2chHelper::login_2ch_be_failure_info_response_cb(GtkWidget *widget, gint id,
						    BBS2chHelper *helper)
{
  if (id == GTK_RESPONSE_OK)
    helper->login_2ch_be();

  gtk_widget_destroy(widget);
}


void
BBS2chHelper::login_2ch_be_finished_cb(const char *session_id)
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  gtk_toggle_action_set_active(login_2ch_be_action, session_id != NULL);

  GDK_THREADS_LEAVE();
}


void
BBS2chHelper::auth_be_response_cb(GtkWidget *widget, gint id,
				  BBS2chHelper *helper)
{
  switch (id)
    {
    case GTK_RESPONSE_OK:
      {
	GtkEntry *entry
	  = GTK_ENTRY(g_object_get_data(G_OBJECT(widget), "user_id"));
	helper->set_be_id(gtk_entry_get_text(entry));
	entry = GTK_ENTRY(g_object_get_data(G_OBJECT(widget), "passwd"));
	helper->set_be_passwd(gtk_entry_get_text(entry));
	gtk_widget_hide(widget);
	return;
      }
    }
  gtk_widget_hide(widget);
  gtk_toggle_action_set_active(helper->login_2ch_be_action, FALSE);
}


bool
BBS2chHelper::login_2ch_be()
{
  if (bbs2ch->get_session_id_2ch_be() != NULL)
    return true;

  if (controller.get_config().get_offline())
    return false;

  if (auth_be_dialog == NULL)
    {
      auth_be_dialog
	= create_auth_dialog(_("Login 2ch Be"), be_id, be_passwd,
			     controller.get_toplevel(),
			     controller.get_prefs().save_2ch_be_passwd);
      g_signal_connect(auth_be_dialog, "response",
		       reinterpret_cast<GCallback>(auth_be_response_cb), this);
      g_signal_connect(auth_be_dialog, "destroy",
		       reinterpret_cast<GCallback>(gtk_widget_destroyed),
		       &auth_be_dialog);
    }

  gint response = gtk_dialog_run(GTK_DIALOG(auth_be_dialog));

  if (response == GTK_RESPONSE_OK && be_id != NULL && be_passwd != NULL)
    {
      GDK_THREADS_LEAVE();

      bbs2ch->login_2ch_be(be_id, be_passwd,
			   controller.get_thread_pool(),
			   controller.get_broker());

      GDK_THREADS_ENTER();
      return true;
    }

  return false;
}


void
BBS2chHelper::logout_2ch_be()
{
  GDK_THREADS_LEAVE();

  bbs2ch->logout_2ch_be();

  GDK_THREADS_ENTER();
}


void
BBS2chHelper::login_2ch_viewer_toggled_cb(GtkToggleAction *action,
					  BBS2chHelper *helper)
{
  if (gtk_toggle_action_get_active(action))
    helper->login_2ch_viewer();
  else
    helper->logout_2ch_viewer();
}


void
BBS2chHelper::login_2ch_viewer_failed_cb(const char *reason)
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  // reasonhtmlä⤷줺ġ
  GtkWidget *dialog = gtk_message_dialog_new(controller.get_toplevel(),
					     GTK_DIALOG_DESTROY_WITH_PARENT,
					     GTK_MESSAGE_WARNING,
					     GTK_BUTTONS_OK_CANCEL,
					     _("2ch Viewer Login failed"));
  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), "%s", reason);
  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);

  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(login_2ch_viewer_failure_info_response_cb),
		   this);

  gtk_widget_show(dialog);

  GDK_THREADS_LEAVE();
}


void
BBS2chHelper::login_2ch_viewer_failure_info_response_cb(GtkWidget *widget,
							gint id,
							BBS2chHelper *helper)
{
  if (id == GTK_RESPONSE_OK)
    helper->login_2ch_viewer();

  gtk_widget_destroy(widget);
}


void
BBS2chHelper::login_2ch_viewer_finished_cb(const char *session_id)
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  gtk_toggle_action_set_active(login_2ch_viewer_action, session_id != NULL);

  GDK_THREADS_LEAVE();
}


void
BBS2chHelper::auth_viewer_response_cb(GtkWidget *widget, gint id,
				      BBS2chHelper *helper)
{
  switch (id)
    {
    case GTK_RESPONSE_OK:
      {
	GtkEntry *entry
	  = GTK_ENTRY(g_object_get_data(G_OBJECT(widget), "user_id"));
	helper->set_viewer_id(gtk_entry_get_text(entry));
	entry = GTK_ENTRY(g_object_get_data(G_OBJECT(widget), "passwd"));
	helper->set_viewer_passwd(gtk_entry_get_text(entry));
	gtk_widget_hide(widget);
	return;
      }
    }
  gtk_widget_hide(widget);
  gtk_toggle_action_set_active(helper->login_2ch_viewer_action, FALSE);
}


bool
BBS2chHelper::login_2ch_viewer()
{
  if (bbs2ch->get_session_id_2ch_viewer() != NULL)
    return true;

  if (controller.get_config().get_offline())
    return false;

  if (auth_viewer_dialog == NULL)
    {
      auth_viewer_dialog
	= create_auth_dialog(_("Login 2ch Viewer"), viewer_id, viewer_passwd,
			     controller.get_toplevel(),
			     controller.get_prefs().save_2ch_viewer_passwd);

#if GTK_CHECK_VERSION(2,14,0)
      GtkWidget *content_area
	= gtk_dialog_get_content_area(GTK_DIALOG(auth_viewer_dialog));
#else
      GtkWidget *content_area = GTK_DIALOG(auth_viewer_dialog)->vbox;
#endif

      GtkWidget *label = gtk_label_new(_("WARNING: Viewer related functions are preliminary implementation."));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(content_area), label, FALSE, FALSE, 4);

      label = gtk_label_new(_("WARNING: It is almost entirely non-sense but you may login/logout."));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(content_area), label, FALSE, FALSE, 4);

      label = gtk_label_new(_("WARNING: If you can logged in, you may be able to use the viewer"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(content_area), label, FALSE, FALSE, 4);

      label = gtk_label_new(_("WARNING: to post responses and to create a new threads."));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(content_area), label, FALSE, FALSE, 4);

      g_signal_connect(auth_viewer_dialog, "response",
		       reinterpret_cast<GCallback>(auth_viewer_response_cb),
		       this);
      g_signal_connect(auth_viewer_dialog, "destroy",
		       reinterpret_cast<GCallback>(gtk_widget_destroyed),
		       &auth_viewer_dialog);
    }

  gint response = gtk_dialog_run(GTK_DIALOG(auth_viewer_dialog));

  if (response == GTK_RESPONSE_OK && viewer_id != NULL && viewer_passwd != NULL)
    {
      GDK_THREADS_LEAVE();

      bbs2ch->login_2ch_viewer(viewer_id, viewer_passwd,
			       controller.get_thread_pool(),
			       controller.get_broker());

      GDK_THREADS_ENTER();

      return true;
    }

  return false;
}


void
BBS2chHelper::logout_2ch_viewer()
{
  GDK_THREADS_LEAVE();

  bbs2ch->logout_2ch_viewer();

  GDK_THREADS_ENTER();
}

}
