/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_2ch_helper.h,v 1.12 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BBS_2CH_HELPER_H_
#define _BBS_2CH_HELPER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bbs_2ch_utils.h"
#include "libochusha/sigslot.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class Controller;

  class BBS2chHelper : public has_slots
  {
  private:
    BBS2ch *bbs2ch;
    Controller &controller;
    GtkActionGroup *action_group;
    char *be_id;
    char *be_passwd;
    char *viewer_id;
    char *viewer_passwd;

    GtkToggleAction *login_2ch_be_action;
    GtkToggleAction *login_2ch_viewer_action;

    GtkWidget *auth_be_dialog;
    GtkWidget *auth_viewer_dialog;

    GtkActionGroup *create_action_group();

    GtkWidget *create_auth_dialog(const gchar *title, const char *user_id,
				  const char *passwd, GtkWindow *toplevel,
				  GtkToggleAction *action);

    static void login_2ch_be_toggled_cb(GtkToggleAction *action,
					BBS2chHelper *helper);
    static void login_2ch_viewer_toggled_cb(GtkToggleAction *action,
					    BBS2chHelper *helper);
    static void save_2ch_be_passwd_toggled_cb(GtkToggleAction *action,
					      BBS2chHelper *helper);
    static void save_2ch_viewer_passwd_toggled_cb(GtkToggleAction *action,
						  BBS2chHelper *helper);

    static void auth_be_response_cb(GtkWidget *widget, gint id,
				    BBS2chHelper *helper);

    static void login_2ch_be_failure_info_response_cb(GtkWidget *widget,
						      gint id,
						      BBS2chHelper *helper);
    void login_2ch_be_failed_cb(const char *reason);
    void login_2ch_be_finished_cb(const char *session_id);

    static void auth_viewer_response_cb(GtkWidget *widget, gint id,
				    BBS2chHelper *helper);
    static void login_2ch_viewer_failure_info_response_cb(GtkWidget *widget,
							  gint id,
							  BBS2chHelper *helper);
    void login_2ch_viewer_failed_cb(const char *reason);
    void login_2ch_viewer_finished_cb(const char *session_id);


  public:
    BBS2chHelper(Controller &controller);
    ~BBS2chHelper();

    bool login_2ch_be();
    void logout_2ch_be();
    bool login_2ch_viewer();
    void logout_2ch_viewer();

    inline const char *get_be_id() const { return be_id; }
    void set_be_id(const char *id);
    inline const char *get_be_passwd() const { return be_passwd; }
    void set_be_passwd(const char *passwd);
    inline const char *get_viewer_id() const { return viewer_id; }
    void set_viewer_id(const char *id);
    inline const char *get_viewer_passwd() const { return viewer_passwd; }
    void set_viewer_passwd(const char *passwd);

    inline const char *get_session_id_2ch_be() const
    { return bbs2ch->get_session_id_2ch_be(); }
    inline const char *get_session_id_2ch_viewer() const
    { return bbs2ch->get_session_id_2ch_viewer(); }
  };


}


#endif	/* __cplusplus */

#endif	/* _BBS_2CH_HELPER_H_ */
