/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_thread_renderer.h,v 1.23 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BBS_THREAD_RENDERER_H_
#define _BBS_THREAD_RENDERER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bulletin_board.h"
#include "libochusha/hashtable.h"
#include "libochusha/html_utils.h"
#include "libochusha/regexp.h"
#include "libochusha/sigslot.h"

#include "html_renderer.h"

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/hypertext_view.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class Controller;
  class Preferences;

  class BBSThreadRenderer : public HTMLRenderer
  {
  protected:
    BBSThreadHandle thread;

    Regexp skin_keyword;
    Regexp digits;
    Regexp be_id;
    Regexp id_id;
    Regexp be_icon;

    guint number_of_responses_got;

    virtual void start_element(const char *name, const char *const attrs[]);
    virtual bool is_visited_link(const char *link);

    void render_skin_fragment(const char *skin_fragment, const char *title,
			      guint res_num, const char *name,
			      const char *mailto, const char *date,
			      const char *id,
			      const char *message, size_t message_len);

    void render_response(guint res_num, Response &response,
			 const char *skin_fragment);

    virtual gboolean link_mouse_over(OchushaHypertextView *widget,
				     GdkEventMotion *event,
				     const gchar *link);
    virtual gboolean link_mouse_out(OchushaHypertextView *widget,
				    GdkEventMotion *event);
    virtual gboolean link_mouse_press(OchushaHypertextView *widget,
				      GdkEventButton *event,
				      const gchar *link);
    virtual gboolean link_mouse_release(OchushaHypertextView *widget,
					GdkEventButton *event,
					const gchar *link);


  public:
    BBSThreadRenderer(Controller &controller,
		      BBSThread *thread,
		      OchushaHypertextBuffer *buffer,
		      OchushaHypertextView *view);

    void render_header(const char *title);
    void render_a_bone(guint res_num);
    void render_broken(guint res_num);
    void render_response(guint res_num, Response &response);
    void render_footer();

    // ݥåץå
    void prepare_for_popup();
    void render_popup_response(guint res_num, Response &response);
    void render_popup_broken(guint res_num);
    void render_popup_title(const gchar *title);

    // IDݥåץå
    void prepare_for_popup_id(const gchar *title, const gchar *id);
    void render_popup_id(guint res_num, Response &response, int count);

    // ݥåץå
    void render_image(GtkWidget *image, const gchar *link);

    static char *canon_link(BBSThread *thread, const char *link);
  };


}


#endif	/* __cplusplus */

#endif	/* _BBS_THREAD_RENDERER_H_ */
