/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_thread_ui.cc,v 1.54 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "bbs_thread_ui.h"

#include "libochusha/bulletin_board.h"
#include "libochusha/bbs_thread.h"
#include "libochusha/misc_utils.h"
#include "libochusha/ochusha_procedure.h"
#include "libochusha/text_utils.h"
#include "libochusha/uri_utils.h"

#include "a_bone_ui.h"
#include "bbs_thread_view.h"
#include "controller.h"
#include "image_manager.h"

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/menu_action.h"
#include "libochushagtk/regex_editor.h"
#include "libochushagtk/stock.h"

#include <gtk/gtk.h>

#include <cassert>
#include <cstdlib>
#include <cstring>

#include <string>

#ifdef HAVE_TIME_H
#include <time.h>
#endif


namespace Ochusha
{


BBSThreadUI::BBSThreadUI(Controller &controller)
  : controller(controller),
    thread_view_action_group(create_thread_view_action_group()),
    image_link_action_group(create_image_link_action_group()),
    res_link_action_group(create_res_link_action_group()),
    id_link_action_group(create_id_link_action_group())
{
  g_signal_connect(controller.get_clipboard_internal(), "owner-change",
		   reinterpret_cast<GCallback>(owner_change_cb), this);
}


BBSThreadUI::~BBSThreadUI()
{
  g_object_unref(id_link_action_group);
  g_object_unref(res_link_action_group);
  g_object_unref(image_link_action_group);
  g_object_unref(thread_view_action_group);
}


GtkActionGroup *
BBSThreadUI::create_thread_view_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "WriteResponse", OCHUSHA_STOCK_POST,
      N_("Write _Response"), NULL,
      N_("Write Response"),
      reinterpret_cast<GCallback>(write_response_cb)
    },
    {
      "JumpToBookmark", GTK_STOCK_JUMP_TO,
      N_("_Jump to Bookmark"), "",
      N_("Jump to Bookmark"),
      reinterpret_cast<GCallback>(jump_to_bookmark_cb)
    },
    {
      "GotoBottom", GTK_STOCK_GOTO_BOTTOM,
      N_("Go to _Bottom"), "",
      N_("Go to Bottom"),
      reinterpret_cast<GCallback>(goto_bottom_cb)
    },
    {
      "GotoTop", GTK_STOCK_GOTO_TOP,
      N_("Go to _Top"), "",
      N_("Go to Top"),
      reinterpret_cast<GCallback>(goto_top_cb)
    },
    {
      "GotoNext", GTK_STOCK_GO_FORWARD,
      N_("Go to _Next"), "",
      N_("Go to Next"),
      reinterpret_cast<GCallback>(goto_next_cb)
    },
    {
      "GotoPrev", GTK_STOCK_GO_BACK,
      N_("Go to _Previous"), "",
      N_("Go to Previous"),
      reinterpret_cast<GCallback>(goto_prev_cb)
    },
    {
      "ResetCurrentThread", GTK_STOCK_DELETE,
      N_("Reset Current Thread"), "",
      N_("Reset Current Thread"),
      reinterpret_cast<GCallback>(reset_thread_cb)
    }
  };

  GtkActionGroup *action_group = gtk_action_group_new("ThreadViewActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  write_response = gtk_action_group_get_action(action_group, "WriteResponse");
  gtk_action_set_sensitive(write_response, FALSE);

  jump_to_bookmark = gtk_action_group_get_action(action_group,
						 "JumpToBookmark");
  gtk_action_set_sensitive(jump_to_bookmark, FALSE);

  goto_bottom = gtk_action_group_get_action(action_group, "GotoBottom");
  gtk_action_set_sensitive(goto_bottom, FALSE);

  goto_top = gtk_action_group_get_action(action_group, "GotoTop");
  gtk_action_set_sensitive(goto_top, FALSE);

  goto_next = gtk_action_group_get_action(action_group, "GotoNext");
  gtk_action_set_sensitive(goto_next, FALSE);

  goto_prev = gtk_action_group_get_action(action_group, "GotoPrev");
  gtk_action_set_sensitive(goto_prev, FALSE);

  reset_thread
    = gtk_action_group_get_action(action_group, "ResetCurrentThread");
  gtk_action_set_sensitive(reset_thread, FALSE);


  gtk_action_group_set_sensitive(action_group, FALSE);

  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  return action_group;
}


GtkActionGroup *
BBSThreadUI::create_image_link_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "CopyFileLocation", GTK_STOCK_COPY,
      N_("Copy _File Location"), "",
      N_("Copy File Location"),
      reinterpret_cast<GCallback>(copy_file_location_cb)
    },
    {
      "TenureLinkImage", GTK_STOCK_SAVE,
      N_("Tenure Link Image"), "",
      NULL,
      reinterpret_cast<GCallback>(tenure_link_image_cb)
    },
    {
      "ABoneLinkImage", GTK_STOCK_DELETE,
      N_("A Bone Link Image"), "",
      NULL,
      reinterpret_cast<GCallback>(a_bone_link_image_cb)
    },
    {
      "ABoneLinkServer", GTK_STOCK_DELETE,
      N_("A Bone Link Server"), "",
      NULL,
      reinterpret_cast<GCallback>(a_bone_link_server_cb)
    }
  };

  GtkActionGroup *action_group = gtk_action_group_new("ImageLinkActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  copy_file_location = gtk_action_group_get_action(action_group,
						   "CopyFileLocation");
  gtk_action_set_sensitive(copy_file_location, FALSE);

  tenure_link_image = gtk_action_group_get_action(action_group,
						  "TenureLinkImage");
  gtk_action_set_sensitive(tenure_link_image, FALSE);


  gtk_action_group_set_sensitive(action_group, FALSE);


  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  return action_group;
}


GtkActionGroup *
BBSThreadUI::create_res_link_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "WriteResponseToThisResponse", OCHUSHA_STOCK_POST,
      N_("_Write Response to This Response"), NULL,
      N_("Write Reponse to This Response"),
      reinterpret_cast<GCallback>(write_response_to_this_response_cb)
    },
    {
      "InsertBookmark", NULL,
      N_("Insert Bookmark"), NULL,
      NULL,
      reinterpret_cast<GCallback>(insert_bookmark_cb)
    },
    {
      "ABoneResponse", GTK_STOCK_DELETE,
      N_("A Bone Response"), "",
      NULL,
      reinterpret_cast<GCallback>(a_bone_response_cb)
    },
    {
      "UnABoneResponse", GTK_STOCK_UNDELETE,
      N_("Un-A-Bone Response"), "",
      NULL,
      reinterpret_cast<GCallback>(un_a_bone_response_cb)
    }
  };

  GtkActionGroup *action_group = gtk_action_group_new("ResLinkActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  a_bone_response = gtk_action_group_get_action(action_group,
						"ABoneResponse");
  gtk_action_set_sensitive(a_bone_response, FALSE);

  un_a_bone_response = gtk_action_group_get_action(action_group,
						   "UnABoneResponse");
  gtk_action_set_sensitive(un_a_bone_response, FALSE);

  gtk_action_group_set_sensitive(action_group, FALSE);


  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  return action_group;
}


GtkActionGroup *
BBSThreadUI::create_id_link_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "ABoneIDInThisThread", GTK_STOCK_DELETE,
      N_("A Bone ID in This Thread"), "",
      NULL,
      reinterpret_cast<GCallback>(a_bone_id_in_this_thread_cb)
    },
    {
      "ABoneIDInThisBoard", GTK_STOCK_DELETE,
      N_("A Bone ID in This Board"), "",
      NULL,
      reinterpret_cast<GCallback>(a_bone_id_in_this_board_cb)
    }
  };

  GtkActionGroup *action_group = gtk_action_group_new("IDLinkActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  gtk_action_group_set_sensitive(action_group, FALSE);


  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  return action_group;
}


void
BBSThreadUI::owner_change_cb(GtkClipboard *clipboard, GdkEvent *event,
			     BBSThreadUI *ui)
{
  if (!BBSThreadViewBase::internal_decorated_link_available(clipboard))
    {
      gtk_action_group_set_sensitive(ui->image_link_action_group, FALSE);
      gtk_action_group_set_sensitive(ui->res_link_action_group, FALSE);
      gtk_action_group_set_sensitive(ui->id_link_action_group, FALSE);

      return;
    }

  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);

  if (strncmp(link, "<RES>", 5) == 0)
    {
      gtk_action_group_set_sensitive(ui->res_link_action_group, TRUE);

      // ܡ󤵤Ƥ뤫å
      // a_bone_responseun_a_bone_responseξ֤򹹿
      guint res_num;
      if (sscanf(link + 5, "%u", &res_num) == 1)
	{
	  bool a_boned = thread->is_a_boned(res_num);
	  gtk_action_set_sensitive(ui->a_bone_response,
				   a_boned ? FALSE : TRUE);
	  gtk_action_set_sensitive(ui->un_a_bone_response,
				   a_boned ? TRUE : FALSE);
	}
      else
	{
	  gtk_action_set_sensitive(ui->a_bone_response, FALSE);
	  gtk_action_set_sensitive(ui->un_a_bone_response, FALSE);
	}

      gtk_action_group_set_sensitive(ui->image_link_action_group, FALSE);
      gtk_action_group_set_sensitive(ui->id_link_action_group, FALSE);

      return;
    }

  if (strncmp(link, "<ID>", 4) == 0)
    {
      gtk_action_group_set_sensitive(ui->image_link_action_group, FALSE);
      gtk_action_group_set_sensitive(ui->res_link_action_group, FALSE);
      gtk_action_group_set_sensitive(ui->id_link_action_group, TRUE);

      return;
    }

  gtk_action_group_set_sensitive(ui->res_link_action_group, FALSE);
  gtk_action_group_set_sensitive(ui->id_link_action_group, FALSE);

  ImageManager &image_manager = ui->controller.get_image_manager();
  if (!image_manager.is_image_link(link))
    {
      gtk_action_group_set_sensitive(ui->image_link_action_group, FALSE);
      return;
    }

  gtk_action_group_set_sensitive(ui->image_link_action_group, TRUE);

  gtk_action_set_sensitive(ui->tenure_link_image,
			   !image_manager.is_tenured(link));
  bool cache_exist = image_manager.cache_exist(link);
  gtk_action_set_sensitive(ui->copy_file_location, cache_exist);
  ui->controller.get_scheme_manager().image_tools_set_sensitive(cache_exist);
}


class BBSThreadPropertiesDialog
{
private:
  BBSThreadUI &ui;
  BBSThreadHandle thread;
  ABoneSettings a_bone_settings;

  GtkWidget *dialog;

  GtkNotebook *notebook;

  GtkWidget *general_info_page;
  GtkWidget *thread_view_page;


  // 󥿥


  // ɽϢ


  GtkWidget *create_general_info_page();
  GtkWidget *create_thread_view_page();
  static void response_cb(GtkWidget *widget, int response_id,
			  BBSThreadPropertiesDialog *dialog);
  static void dialog_destroyed_cb(GtkWidget *widget,
				  BBSThreadPropertiesDialog *dialog);
  void reflect_settings();
  ~BBSThreadPropertiesDialog();


public:
  BBSThreadPropertiesDialog(BBSThreadUI &ui, BBSThread *thread);

  inline BBSThread *get_thread() { return thread; }
  void show();
  void append_name_to_be_a_boned(const char *name);
  void append_id_to_be_a_boned(const char *id);
  void append_content_to_be_a_boned(const char *content);
};


#define TABLE_ATTACH_DEFAULTS(table, widget, l, r, t, b)		\
  gtk_table_attach(table, widget, l, r, t, b,				\
		   static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL),	\
		   static_cast<GtkAttachOptions>(GTK_EXPAND|GTK_FILL), 3, 3)
  

GtkWidget *
BBSThreadPropertiesDialog::create_general_info_page()
{
  GtkWidget *container = gtk_vbox_new(FALSE, 0);
  GtkBox *vbox = GTK_BOX(container);


  GtkWidget *widget = gtk_table_new(7, 2, FALSE);
  gtk_widget_show(widget);
  gtk_box_pack_start(vbox, widget, FALSE, FALSE, 3);

  GtkTable *table = GTK_TABLE(widget);



  GtkWidget *label = gtk_label_new(_("Thread Title:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 0, 1);

  const char *title = thread->get_title();
  if (title == NULL)
    title = _("Anonymous Thread");
  label = gtk_label_new(title);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 0, 1);



  label = gtk_label_new(_("Board:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 1, 2);

  const char *name = thread->get_board()->get_name();
  if (name == NULL)
    name = _("Anonymous Board");
  label = gtk_label_new(name);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 1, 2);



  label = gtk_label_new(_("URL:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 2, 3);

  char *url_copy = NULL;
  const char *url = thread->get_kako_html();
  if (url == NULL)
    {
      url = thread->get_grave();
      if (url == NULL)
	{
	  url_copy = thread->get_url();
	  url = url_copy;
	}
    }
  label = gtk_label_new(url);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 2, 3);



  label = gtk_label_new(_("Thread Created:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 3, 4);

  char text_buf[128];
  time_t utc = thread->get_birthtime();
  if (utc != -1)
    {
#ifdef HAVE_LOCALTIME_R
      struct tm tm;
      struct tm *tm_p = localtime_r(&utc, &tm);
#else
      struct tm *tm_p = localtime(&utc);
#endif
      size_t result;
      if (tm_p != NULL)
	result = strftime(text_buf, 128, "%Y-%m-%d %T", tm_p);
      else
	result = 0;
      if (result == 0)
	snprintf(text_buf, 128, _("Unknown"));
    }
  else
    snprintf(text_buf, 128, _("Unknown"));

  label = gtk_label_new(text_buf);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 3, 4);



  label = gtk_label_new(_("States:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 4, 5);

  bool first_item = true;
  string str_buf;

  if (!thread->is_alive())
    {
      str_buf.append(_("DAT dropped"));
      first_item = false;
    }

  if (thread->is_stopped())
    {
      if (!first_item)
	str_buf.append(", ");
      else
	first_item = false;
      str_buf.append(_("Thread stopped"));
    }

  if (thread->is_kako_thread())
    {
      if (!first_item)
	str_buf.append(", ");
      else
	first_item = false;
      str_buf.append(_("Past Thread"));
    }

  if (thread->is_viewer_used())
    {
      if (!first_item)
	str_buf.append(", ");
      str_buf.append(_("Viewer used"));
    }



  label = gtk_label_new(str_buf.c_str());
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 4, 5);

  label = gtk_label_new(_("DAT filesize:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 5, 6);

  size_t dat_filesize
    = thread->get_dat_filesize(ui.controller.get_repository());
  snprintf(text_buf, 128, "%.1f KB (%lu bytes)",
	   static_cast<double>(dat_filesize) / 1024.0,
	   static_cast<unsigned long>(dat_filesize));
  label = gtk_label_new(text_buf);
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 5, 6);



  label = gtk_label_new(_("A Boned Responses:"));
  gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 0, 1, 6, 7);

  list<unsigned int>::const_iterator iter = thread->a_bone_list_begin();
  list<unsigned int>::const_iterator end_iter = thread->a_bone_list_end();
  string res_num_buf;
  char res_num[16];
  while (iter != end_iter)
    {
      snprintf(res_num, 16, "%u,", *iter++);
      res_num_buf.append(res_num);
    }
  label = gtk_label_new(res_num_buf.c_str());
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_label_set_selectable(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  TABLE_ATTACH_DEFAULTS(table, label, 1, 2, 6, 7);

  return container;
}


GtkWidget *
BBSThreadPropertiesDialog::create_thread_view_page()
{
  GtkWidget *container = gtk_vbox_new(FALSE, 0);
  GtkBox *vbox = GTK_BOX(container);

  GtkWidget *settings = a_bone_settings.get_widget();
  gtk_widget_show(settings);
  gtk_box_pack_start(vbox, settings, TRUE, TRUE, 0);

  return container;
}


BBSThreadPropertiesDialog::BBSThreadPropertiesDialog(BBSThreadUI &ui,
						     BBSThread *thread)
  : ui(ui),
    thread(thread),
    a_bone_settings(thread->get_a_bone(), ui.controller),
    general_info_page(create_general_info_page()),
    thread_view_page(create_thread_view_page())
{
  const char *name = thread->get_title();
  if (name == NULL)
    name = _("Anonymous Thread");
  gchar *title = g_strdup_printf(_("Properties for \"%s\" thread"), name);

  dialog = gtk_dialog_new_with_buttons(title,
				       ui.controller.get_toplevel(),
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_OK, GTK_RESPONSE_OK,
				       NULL);
  g_free(title);

  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(response_cb), this);
  g_signal_connect(dialog, "destroy",
		   reinterpret_cast<GCallback>(dialog_destroyed_cb), this);

  GtkWidget *widget = gtk_notebook_new();
  gtk_widget_show(widget);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), widget, TRUE, TRUE, 0);
  notebook = GTK_NOTEBOOK(widget);


  gtk_widget_show(general_info_page);
  gtk_notebook_append_page(notebook,
			   general_info_page, gtk_label_new(_("General")));


  gtk_widget_show(thread_view_page);
  gtk_notebook_append_page(notebook,
			   thread_view_page,
			   gtk_label_new(_("Thread View Settings")));
}


BBSThreadPropertiesDialog::~BBSThreadPropertiesDialog()
{
  ui.dialog_destroyed(this);
}


void
BBSThreadPropertiesDialog::reflect_settings()
{
}


void
BBSThreadPropertiesDialog::response_cb(GtkWidget *widget, int response_id,
				       BBSThreadPropertiesDialog *dialog)
{
  if (response_id == GTK_RESPONSE_OK)
    dialog->reflect_settings();
  else
    dialog->a_bone_settings.cancel_settings();

  gtk_widget_hide(widget);
  gtk_widget_unrealize(widget);
  gtk_widget_destroy(widget);
}


void
BBSThreadPropertiesDialog::dialog_destroyed_cb(GtkWidget *widget,
					       BBSThreadPropertiesDialog *dialog)
{
  delete dialog;
}


void
BBSThreadPropertiesDialog::show()
{
  gtk_widget_show(dialog);
  gtk_window_present(GTK_WINDOW(dialog));
}


void
BBSThreadPropertiesDialog::append_name_to_be_a_boned(const char *name)
{
  a_bone_settings.append_name_to_be_a_boned(name);
  gint page = gtk_notebook_page_num(notebook, thread_view_page);
  g_return_if_fail(page != -1);
  gtk_notebook_set_current_page(notebook, page);
}


void
BBSThreadPropertiesDialog::append_id_to_be_a_boned(const char *id)
{
  a_bone_settings.append_id_to_be_a_boned(id);
  gint page = gtk_notebook_page_num(notebook, thread_view_page);
  g_return_if_fail(page != -1);
  gtk_notebook_set_current_page(notebook, page);
}


void
BBSThreadPropertiesDialog::append_content_to_be_a_boned(const char *content)
{
  a_bone_settings.append_content_to_be_a_boned(content);
  gint page = gtk_notebook_page_num(notebook, thread_view_page);
  g_return_if_fail(page != -1);
  gtk_notebook_set_current_page(notebook, page);
}


BBSThreadPropertiesDialog *
BBSThreadUI::lookup_dialog(BBSThread *thread)
{
  list<BBSThreadPropertiesDialog *>::iterator iter = dialogs.begin();
  list<BBSThreadPropertiesDialog *>::iterator end_iter = dialogs.end();

  while (iter != end_iter)
    {
      BBSThreadPropertiesDialog *dialog = *iter++;
      if (dialog->get_thread() == thread)
	return dialog;
    }

  BBSThreadPropertiesDialog *dialog
    = new BBSThreadPropertiesDialog(*this, thread);
  if (dialog != NULL)
    dialogs.push_front(dialog);

  return dialog;
}


void
BBSThreadUI::dialog_destroyed(BBSThreadPropertiesDialog *dialog)
{
  dialogs.erase(find(dialogs.begin(), dialogs.end(), dialog));
}


void
BBSThreadUI::open_properties(BBSThread *thread)
{
  BBSThreadPropertiesDialog *dialog = lookup_dialog(thread);
  if (dialog != NULL)
    dialog->show();
}


void
BBSThreadUI::prepare_session()
{
}


bool
BBSThreadUI::show_mailto(BBSThread *thread)
{
  g_return_val_if_fail(thread != NULL, false);

  MailtoMode mode = thread->get_board()->get_mailto_mode();
  if (mode == MAILTO_MODE_DEFAULT)
    return controller.get_prefs().get_show_mailto();
  return mode == MAILTO_MODE_SHOW;
}


BBSThreadView *
BBSThreadUI::get_current_thread_view()
{
  GtkWidget *widget = controller.get_view_manager().get_current_thread_widget();
  g_return_val_if_fail(widget != NULL, NULL);

  View *view = View::get_view(widget);
  g_return_val_if_fail(view != NULL, NULL);

  return dynamic_cast<BBSThreadView *>(view);
}


BBSThread *
BBSThreadUI::get_current_thread()
{
  return controller.get_view_manager().get_current_thread();
}


void
BBSThreadUI::write_response_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadView *view = ui->get_current_thread_view();
  g_return_if_fail(view != NULL);

  view->show_post_window();
}


void
BBSThreadUI::jump_to_bookmark_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadView *view = ui->get_current_thread_view();
  g_return_if_fail(view != NULL);

  view->jump_to_bookmark();
}


void
BBSThreadUI::goto_bottom_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadView *view = ui->get_current_thread_view();
  g_return_if_fail(view != NULL);

  view->goto_bottom();
}


void
BBSThreadUI::goto_top_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadView *view = ui->get_current_thread_view();
  g_return_if_fail(view != NULL);

  view->goto_top();
}


void
BBSThreadUI::goto_next_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadView *view = ui->get_current_thread_view();
  g_return_if_fail(view != NULL);

  view->go_forward_view();
}


void
BBSThreadUI::goto_prev_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadView *view = ui->get_current_thread_view();
  g_return_if_fail(view != NULL);

  view->go_back_view();
}


void
BBSThreadUI::reset_thread_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThread *thread = ui->get_current_thread();
  g_return_if_fail(thread != NULL);

  thread->reset(ui->controller.get_repository());
  gtk_widget_queue_draw(GTK_WIDGET(ui->controller.get_toplevel()));
}


void
BBSThreadUI::copy_file_location_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);


  char *filename = ui->controller.get_image_manager().find_cache(link);
  if (filename == NULL)
    return;
  ui->controller.copy(filename);
  free(filename);
}


void
BBSThreadUI::tenure_link_image_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);

  ui->controller.get_image_manager().tenure(link);
}


void
BBSThreadUI::a_bone_link_image_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);

  ui->controller.get_image_manager().a_bone(link);
}


void
BBSThreadUI::a_bone_link_server_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);

  ui->controller.get_image_manager().a_bone_server(link);
}


ResponseEditor *
BBSThreadUI::get_prefered_response_editor(BBSThreadView *view)
{
  ResponseEditor *editor = view->get_post_window();
  if (editor != NULL)
    return editor;

  if (controller.get_prefs().get_enable_embeded_response_editor())
    return &controller.get_view_manager().get_embeded_response_editor();

  if (controller.get_prefs().get_prefer_embeded_response_editor())
    {
      controller.get_prefs().set_enable_embeded_response_editor(true);
      return &controller.get_view_manager().get_embeded_response_editor();
    }

  view->show_post_window();
  return view->get_post_window();
}


void
BBSThreadUI::write_response_to_this_response_cb(GtkAction *action,
						BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);
  g_return_if_fail(strncmp("<RES>", link, 5) == 0);

  BBSThreadView *view
    = dynamic_cast<BBSThreadView *>(view_manager.get_current_thread_view());
  g_return_if_fail(view != NULL);

  guint res_num;
  if (sscanf(link + 5, "%u", &res_num) == 1)
    {
      ResponseEditor *editor = ui->get_prefered_response_editor(view);
      g_return_if_fail(editor != NULL);

      gchar message[256];
      snprintf(message, 256, ">>%u\n", res_num);
      editor->append_message(message);
      editor->grab_focus_into_editor();
    }
}


void
BBSThreadUI::insert_bookmark_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);
  g_return_if_fail(strncmp("<RES>", link, 5) == 0);

  BBSThreadView *view
    = dynamic_cast<BBSThreadView *>(view_manager.get_current_thread_view());
  g_return_if_fail(view != NULL);

  guint res_num;
  if (sscanf(link + 5, "%u", &res_num) == 1)
    {
      view->get_thread()->set_bookmark_position(res_num);
      view->insert_my_own_bookmark(res_num, true);

      GtkWidget *board_widget = view_manager.get_current_board_widget();
      if (board_widget != NULL)
	gtk_widget_queue_draw(board_widget);
    }
}


void
BBSThreadUI::a_bone_response_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);
  g_return_if_fail(strncmp("<RES>", link, 5) == 0);

  guint res_num;
  if (sscanf(link + 5, "%u", &res_num) == 1)
    {
      thread->register_a_bone(res_num);
      // Ūˤ¨¤viewȿǤ٤
    }
}


void
BBSThreadUI::un_a_bone_response_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);
  g_return_if_fail(strncmp("<RES>", link, 5) == 0);

  guint res_num;
  if (sscanf(link + 5, "%u", &res_num) == 1)
    {
      thread->unregister_a_bone(res_num);
      // Ūˤ¨¤viewȿǤ٤
    }
}


void
BBSThreadUI::a_bone_id_in_this_thread_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);
  g_return_if_fail(strncmp("<ID>", link, 4) == 0);

  const char *id = link + 4;
  while (*id != '\0' && *id++ != ':')
    ;
  if (*id == '\0')
    return;

  BBSThreadPropertiesDialog *dialog = ui->lookup_dialog(thread);
  if (dialog != NULL)
    {
      dialog->append_id_to_be_a_boned(id);
      dialog->show();
    }
}


void
BBSThreadUI::a_bone_id_in_this_board_cb(GtkAction *action, BBSThreadUI *ui)
{
  BBSThreadViewSelectionData *selection_data
    = BBSThreadViewBase::get_bbs_thread_view_selection_data_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection_data != NULL);

  BBSThread *thread = selection_data->get_thread();
  g_return_if_fail(thread != NULL);

  ViewManager &view_manager = ui->controller.get_view_manager();
  g_return_if_fail(view_manager.get_current_thread() == thread);

  const char *link = selection_data->get_link();
  g_return_if_fail(link != NULL);
  g_return_if_fail(strncmp("<ID>", link, 4) == 0);

  const char *id = link + 4;
  while (*id != '\0' && *id++ != ':')
    ;
  if (*id == '\0')
    return;

  ui->controller.get_board_properties_ui()
    .append_id_to_be_a_boned(thread->get_board(), id);
}


bool
BBSThreadUI::is_link_in_thread(BBSThread *thread, const char *link)
{
  if (strstr(link, thread->get_id()) != NULL)
    {
      BBSThread *dummy = thread->get_board()->lookup_bbs_thread_by_url(link);
#if 1
      if (dummy == NULL)
	fprintf(stderr, "Couldn't parse link URL: \"%s\"\n", link);
#endif
      return dummy != NULL;
    }
  int value = parse_int(link, strlen(link));
  return value > 0;
}


BBSThreadView *
BBSThreadUI::create_thread_view(BBSThread *thread, TabLabel *label,
				int initial_pos, bool refresh)
{
  return BBSThreadView::create(controller, thread, label, initial_pos, refresh);
}


void
BBSThreadUI::specify_thread_view_position(BBSThread *thread, int position)
{
  View *view = controller.get_view_manager().find_thread_view(thread, false);
  g_return_if_fail(view != NULL);

  BBSThreadView *thread_view = dynamic_cast<BBSThreadView *>(view);
  g_return_if_fail(thread_view != NULL);

  thread_view->specify_view_position(position);
}


void
BBSThreadUI::notify_current_thread_view(View *view, BBSThread *thread)
{
  if (thread == NULL)
    {
      gtk_action_group_set_sensitive(thread_view_action_group, FALSE);
    }
  else
    {
      gtk_action_group_set_sensitive(thread_view_action_group, TRUE);

      gtk_action_set_sensitive(write_response, TRUE);
      gtk_action_set_sensitive(goto_bottom, TRUE);
      gtk_action_set_sensitive(goto_top, TRUE);
      gtk_action_set_sensitive(reset_thread, TRUE);

      BBSThreadView *thread_view = dynamic_cast<BBSThreadView *>(view);
      g_return_if_fail(thread_view != NULL);

      notify_thread_view_state(thread_view, thread);
    }
}


void
BBSThreadUI::notify_thread_view_state(BBSThreadView *view, BBSThread *thread)
{
  g_return_if_fail(view != NULL);

  if (controller.get_view_manager().get_current_thread() != thread)
    return;

  gtk_action_set_sensitive(jump_to_bookmark, view->bookmark_exist());
  gtk_action_set_sensitive(goto_next, view->has_forward_view());
  gtk_action_set_sensitive(goto_prev, view->has_back_view());
}

}
