/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_thread_ui.h,v 1.32 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BBS_THREAD_UI_H_
#define _BBS_THREAD_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "libochusha/sigslot.h"

#include "html_style.h"

#include <list>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BBSThread;
  class BBSThreadPropertiesDialog;
  class BBSThreadView;
  class Controller;
  class ExternalTextToolsMenuHelper;
  class ResponseEditor;
  class TabLabel;
  class View;

  class BBSThreadUI
  {
    friend class BBSThreadPropertiesDialog;
    friend class BBSThreadView;
    friend class BBSThreadViewBase;
    friend class ExternalTextToolsMenuHelper;

  private:
    Controller &controller;

    GtkActionGroup *thread_view_action_group;
    GtkActionGroup *image_link_action_group;
    GtkActionGroup *res_link_action_group;
    GtkActionGroup *id_link_action_group;

    GtkAction *write_response;
    GtkAction *jump_to_bookmark;
    GtkAction *goto_bottom;
    GtkAction *goto_top;
    GtkAction *goto_next;
    GtkAction *goto_prev;
    GtkAction *reset_thread;

    GtkAction *copy_file_location;
    GtkAction *tenure_link_image;

    GtkAction *a_bone_response;
    GtkAction *un_a_bone_response;


    list<BBSThreadPropertiesDialog *> dialogs;


    GtkActionGroup *create_thread_view_action_group();
    GtkActionGroup *create_image_link_action_group();
    GtkActionGroup *create_res_link_action_group();
    GtkActionGroup *create_id_link_action_group();

    static void owner_change_cb(GtkClipboard *clipboard, GdkEvent *event,
				BBSThreadUI *ui);


    // ɽGtkActionϢ
    static void write_response_cb(GtkAction *action, BBSThreadUI *ui);
    static void jump_to_bookmark_cb(GtkAction *action, BBSThreadUI *ui);
    static void goto_bottom_cb(GtkAction *action, BBSThreadUI *ui);
    static void goto_top_cb(GtkAction *action, BBSThreadUI *ui);
    static void goto_next_cb(GtkAction *action, BBSThreadUI *ui);
    static void goto_prev_cb(GtkAction *action, BBSThreadUI *ui);
    static void reset_thread_cb(GtkAction *action, BBSThreadUI *ui);


    // ѥݥåץåץ˥塼Ϣ
    static void copy_file_location_cb(GtkAction *action, BBSThreadUI *ui);
    static void tenure_link_image_cb(GtkAction *action, BBSThreadUI *ui);
    static void a_bone_link_image_cb(GtkAction *action, BBSThreadUI *ui);
    static void a_bone_link_server_cb(GtkAction *action, BBSThreadUI *ui);
    static void write_response_to_this_response_cb(GtkAction *action,
						   BBSThreadUI *ui);
    static void insert_bookmark_cb(GtkAction *action, BBSThreadUI *ui);
    static void a_bone_response_cb(GtkAction *action, BBSThreadUI *ui);
    static void un_a_bone_response_cb(GtkAction *action, BBSThreadUI *ui);
    static void a_bone_id_in_this_thread_cb(GtkAction *action, BBSThreadUI *ui);
    static void a_bone_id_in_this_board_cb(GtkAction *action, BBSThreadUI *ui);


    BBSThreadView *get_current_thread_view();
    BBSThread *get_current_thread();

    BBSThreadPropertiesDialog *lookup_dialog(BBSThread *thread);
    void dialog_destroyed(BBSThreadPropertiesDialog *dialog);

    ResponseEditor *get_prefered_response_editor(BBSThreadView *view);


  public:
    BBSThreadUI(Controller &controller);
    ~BBSThreadUI();

    void open_properties(BBSThread *thread);

    void prepare_session();

    bool show_mailto(BBSThread *thread);

    bool is_link_in_thread(BBSThread *thread, const char *link);

    BBSThreadView *create_thread_view(BBSThread *thread, TabLabel *label,
				      int initial_pos, bool refresh);
    void specify_thread_view_position(BBSThread *thread, int position);
    void notify_current_thread_view(View *view, BBSThread *thread);
    void notify_thread_view_state(BBSThreadView *view, BBSThread *thread);
  };


}


#endif	/* __cplusplus */

#endif	/* _BBS_THREAD_UI_H_ */
