/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_thread_view.h,v 1.36 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BBS_THREAD_VIEW_H_
#define _BBS_THREAD_VIEW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bulletin_board.h"
#include "libochusha/bbs_thread.h"
#include "libochusha/html_utils.h"
#include "libochusha/regexp.h"
#include "libochusha/sigslot.h"

#include "bbs_thread_renderer.h"
#include "bbs_thread_ui.h"
#include "focus_utils.h"
#include "selection_utils.h"
#include "view.h"

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/hypertext_view.h"

#include <list>
#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BBSThreadPopupView;
  class Controller;
  class ResponseEditorWindow;
  class TabLabel;


  class BBSThreadViewSelectionData : public TextViewSelectionData
  {
  private:
    BBSThread *thread;
    gchar *link;
    char *canonical_link;


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);


  public:
    BBSThreadViewSelectionData(BBSThread *thread, GtkTextView *view,
			       bool is_static);
    virtual ~BBSThreadViewSelectionData();

    virtual bool deletable();
    virtual bool cuttable();
    virtual bool copiable();
    virtual bool pastable(GtkClipboard *clipboard);

    virtual bool do_delete();
    virtual bool do_paste(GtkClipboard *clipboard);

    // ¸thisΥԡäݤΤ֤
    virtual SelectionData *snapshot();

    void set_link(const gchar *value);
    const char *get_canonical_link();

    inline BBSThread *get_thread() { return thread; }
    inline const gchar *get_link() const { return link; }
  };


  class BBSThreadViewBase;

  class BBSThreadViewFocusNotifier : public FocusNotifier
  {
  private:
    BBSThreadViewBase &view;


  protected:
    virtual gboolean notify_focus_in();
    virtual gboolean notify_enter();
    virtual gboolean notify_leave();
    virtual void notify_unmap();


  public:
    BBSThreadViewFocusNotifier(BBSThreadViewBase &view, GtkWidget *widget);
    virtual ~BBSThreadViewFocusNotifier();
  };


  struct ResponseRecord
  {
    gint offset;
    size_t length;
    ResponseRecord() : offset(0), length(0) {}
    ResponseRecord(int offset, size_t length)
      : offset(offset), length(length) {}
    ResponseRecord(const ResponseRecord &record)
      : offset(record.offset), length(record.length) {}
  };


  class BBSThreadViewBase : public View, public has_slots
  {
    friend class BBSThreadUI;
    friend class BBSThreadViewFocusNotifier;
    friend class BBSThreadPopupView;

  protected:
    Controller &controller;
    BBSThreadHandle thread;

    OchushaHypertextBuffer *buffer;
    OchushaHypertextView *view;

    BBSThreadRenderer renderer;


  private:
    BBSThreadViewBase *parent_view;
    gchar *window_title;
    gchar *combo_title;
    char *thread_url;

    guint message_id;
    gulong selection_done_id;

    BBSThreadViewSelectionData selection_data;
    BBSThreadViewFocusNotifier focus_notifier;

    gchar *popup_link;
    BBSThreadPopupView *popup_view;
    gulong popup_view_destroy_id;


    static gboolean internal_decorated_link_available(GtkClipboard *clipboard);
    static BBSThreadViewSelectionData *get_bbs_thread_view_selection_data_from_clipboard(GtkClipboard *clipboard);

    static void widget_destroy_cb(GtkWidget *widget, BBSThreadViewBase *view);

    static void popup_destroy_cb(GtkWidget *widget, BBSThreadViewBase *view);
    static gboolean link_mouse_over_cb(OchushaHypertextView *widget,
				       GdkEventMotion *event,
				       const gchar *link,
				       BBSThreadViewBase *view);
    static gboolean link_mouse_out_cb(OchushaHypertextView *widget,
				      GdkEventMotion *event,
				      BBSThreadViewBase *view);

    gboolean popup_context_menu(const gchar *link,
				guint button, guint32 event_time);
    static gboolean popup_menu_cb(GtkWidget *widget, BBSThreadViewBase *view);

    static void populate_popup_cb(GtkTextView *text_view, GtkMenu *popup_menu,
				  BBSThreadViewBase *view);
    static void selection_done_cb(GtkWidget *widget, BBSThreadViewBase *view);

    static gboolean link_mouse_press_cb(OchushaHypertextView *widget,
					GdkEventButton *event,
					const gchar *link,
					BBSThreadViewBase *view);
    static gboolean link_mouse_release_cb(OchushaHypertextView *widget,
					  GdkEventButton *event,
					  const gchar *link,
					  BBSThreadViewBase *view);
    static void copy_clipboard_cb(OchushaHypertextView *widget,
				  BBSThreadViewBase *view);
    static gboolean start_interactive_search_cb(OchushaHypertextView *widget,
						OchushaSearchActionType action,
						BBSThreadViewBase *view);


  protected:
    BBSThreadViewBase(Controller &controller, TabLabel *tab_label,
		      BBSThread *thread, GtkWidget *window,
		      BBSThreadViewBase *parent_view);
    virtual ~BBSThreadViewBase();

    GtkWindow *get_window();

    virtual void notify_unmap();
    virtual gboolean notify_enter();
    virtual gboolean notify_leave();

    virtual ResponseCursor *get_response_cursor() = 0;
    inline bool has_popup() const { return popup_view != NULL; }
    inline bool has_popup_menu() const { return selection_done_id != 0; }

    virtual void show() = 0;
    virtual void hide(bool now) = 0;


  public:
    inline BBSThread *get_thread() { return thread; }
    inline const char *get_last_event_link() const
    { return selection_data.get_link(); }

    inline BBSThreadViewBase *get_parent_view() { return parent_view; }
    BBSThreadViewBase *get_primogenitor_view();


    // Viewvirtualμ
    virtual const gchar *get_combo_title();
    virtual const gchar *get_menu_title();
    virtual const gchar *get_window_title();
    virtual const gchar *get_url();
    virtual void grab_focus();
    virtual bool contains(GtkWidget *child);
    virtual const gchar *get_search_title();
    virtual OchushaSearchable *get_search_target();
    virtual void notify_closed();

    virtual bool bookmarkable();
    virtual bool bookmark();
    virtual bool refreshable();
    virtual bool refresh();

    virtual bool has_info();
    virtual bool show_info();

    // ư˻Ȥפˤʤäcallerfree()͡
    virtual char *get_current_view_url();
    // Viewvirtualμ
  };


  class BBSThreadPopupView : public BBSThreadViewBase
  {
  private:
    ResponseCursorHandle response_cursor;

    guint popup_id;
    guint popdown_id;

    bool pointed;

    void initialize_common();

    static void size_request_cb(GtkWidget *widget, GtkRequisition *requisition,
				BBSThreadPopupView *view);

    static gboolean show_now(BBSThreadPopupView *view);
    static gboolean hide_now(BBSThreadPopupView *view);


  protected:
    virtual ~BBSThreadPopupView();

    virtual gboolean notify_enter();
    virtual gboolean notify_leave();

    virtual ResponseCursor *get_response_cursor();

    virtual void show();
    virtual void hide(bool now);


  public:
    BBSThreadPopupView(Controller &controller, BBSThreadViewBase *parent_view,
		       guint from, guint to);
    BBSThreadPopupView(Controller &controller, BBSThreadViewBase *parent_view,
		       const gchar *id);
    BBSThreadPopupView(Controller &controller, BBSThreadViewBase *parent_view,
		       GtkWidget *image, const gchar *link);
    BBSThreadPopupView(Controller &controller, BBSThreadViewBase *parent_view,
		       const gchar *title, const gchar *link);
  };


  class BBSThreadView : public BBSThreadViewBase
  {
  private:
    ResponseCursorHandle response_cursor;

    int number_of_responses_shown;
    guint append_responses_id;

    vector<ResponseRecord> records;

    int initial_view_position;
    GtkWidget *my_own_bookmark;
    int my_own_bookmark_position;

    GtkWidget *post_window;
    gulong post_window_destroy_id;

    GdkColor *tab_label_normal_color;
    guint auto_refresher_id;
    guint previous_interval;
    int previous_number_of_responses_got;
    bool enable_transparent_a_bone;
    bool disable_a_bone;
    bool enable_chain_a_bone;


    BBSThreadView(Controller &controller, TabLabel *tab_label,
		  BBSThread *thread);

    virtual ~BBSThreadView();

    virtual ResponseCursor *get_response_cursor();

    virtual void show();
    virtual void hide(bool now);


    // 鲶ѤϢ

    // OchushaHypertextBuffer/Viewoffset֤widget
    // records򹹿
    void insert_widget_at_offset(GtkWidget *widget, gint offset);

    // OchushaHypertextBuffer/Viewwidgetrecords
    // 
    void remove_widget(GtkWidget *widget);

    GtkWidget *create_bookmark();
    static void bookmark_clicked_cb(GtkWidget *widget, BBSThreadView *view);

    // ޤǲѤϢ


    void append_response(guint res_num, Response &response);
    static gboolean append_responses_on_idle(BBSThreadView *view);
    void state_changed_cb();


    static void post_window_destroy_cb(GtkWidget *widget, BBSThreadView *view);
    static gboolean auto_refresher_timeout_cb(BBSThreadView *view);


  protected:
    // Viewvirtualμ
    virtual bool move_view(const char *target_url);
    // Viewvirtualμ


  public:
    // BBSThreadView֥Ȥget_widget()֤GtkWidgetȰϡ
    static BBSThreadView *create(Controller &controller, BBSThread *thread,
				 TabLabel *label, int initial_pos,
				 bool refresh);
    void update(bool refresh, bool automatic);

    bool is_head_visible();
    bool is_tail_visible();

    bool advance_viewport();
    bool retreat_viewport();

    void show_prefered_response_editor();

    void show_post_window();
    ResponseEditorWindow *get_post_window();
    void jump_to_bookmark();
    void goto_bottom();
    void goto_top();

    // 쥹ֹres_numΥ쥹ľ˲ѤΤ
    void insert_my_own_bookmark(guint res_num, bool interactive);

    void specify_view_position(int position);
    bool bookmark_exist() { return my_own_bookmark != NULL; }

    int estimate_current_view_position();
    void remember_current_view_position();
  };


}


#endif	/* __cplusplus */

#endif	/* _BBS_THREAD_VIEW_H_ */
