/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: board_properties_ui.h,v 1.8 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BOARD_PROPERTIES_UI_H_
#define _BOARD_PROPERTIES_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/sigslot.h"


#include <list>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BoardProperties;
  class BulletinBoard;
  class Controller;
  class Preferences;

  class BoardPropertiesUI
  {
    friend class BoardProperties;

  private:
    Controller &controller;
    bool global_show_mailto_mode;

    list<BoardProperties *> properties_list;

    static void show_mailto_toggled_cb(GtkToggleAction *action,
				       BoardPropertiesUI *ui);

    BoardProperties *lookup_properties(BulletinBoard *board);
    void confirm_addition_of_board(BulletinBoard *board);


  public:
    BoardPropertiesUI(Controller &controller, Preferences &prefs);
    ~BoardPropertiesUI();

    void process_response(GtkDialog *dialog, int response_id,
			  BulletinBoard *board);
    void construct_properties_contents(GtkDialog *dialog, BulletinBoard *board);


    inline Controller &get_controller() const { return controller; }
    inline bool get_global_show_mailto_mode() const
    { return global_show_mailto_mode; }

    void add_board(BulletinBoard *board);
    void append_id_to_be_a_boned(BulletinBoard *board, const char *id);

    void properties_destroyed(BoardProperties *properties);
  };


}


#endif	/* __cplusplus */

#endif	/* _BOARD_PROPERTIES_UI_H_ */
