/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: board_ui.h,v 1.17 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BOARD_UI_H_
#define _BOARD_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochushagtk/combo_action.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BoardRecoveryManager;
  class BulletinBoard;
  class Controller;
  class TabLabel;
  class View;

  class BoardUI
  {
    friend class BoardRecoveryManager;
    friend class PreferencesUI;

  private:
    Controller &controller;

    BoardRecoveryManager *recovery_manager;

    GtkActionGroup *action_group;
    GtkAction *go_forward;
    GtkAction *go_back;
    GtkAction *new_thread;
    GtkAction *board_properties;
    GtkAction *default_threadlist_rule;
    GtkAction *threadlist_rule;

    GtkTreeModel *default_threadlist_rule_model;
    GtkTreeModel *threadlist_rule_model;

    GtkActionGroup *create_action_group();


    static void go_forward_cb(GtkAction *action, BoardUI *ui);
    static void go_back_cb(GtkAction *action, BoardUI *ui);
    static void new_thread_cb(GtkAction *action, BoardUI *ui);
    static void board_properties_cb(GtkAction *action, BoardUI *ui);
    static void restore_all_boards_cb(GtkAction *action, BoardUI *ui);

    static void default_threadlist_rule_changed_cb(OchushaComboAction *action,
						   GtkTreeModel *model,
						   GtkTreeIter *iter,
						   GtkTreePath *path,
						   BoardUI *ui);
    static void threadlist_rule_changed_cb(OchushaComboAction *action,
					   GtkTreeModel *model,
					   GtkTreeIter *iter,
					   GtkTreePath *path,
					   BoardUI *ui);

    void set_current_threadlist_rule(const char *name);
    void set_default_threadlist_rule(const char *name);

    static void show_hidden_threads_toggled_cb(GtkAction *action, BoardUI *ui);


  public:
    BoardUI(Controller &controller);
    ~BoardUI();

    void prepare_session();

    View *create_board_view(BulletinBoard *board, TabLabel *label,
			    bool refresh);
    void notify_current_board_view(View *view, BulletinBoard *board);

    void update_threadlist_rules();

    void excavate_dat_files(BulletinBoard *board);
    void restore_all_boards();

    void notify_board_moved(BulletinBoard *board);
  };


}


#endif	/* __cplusplus */

#endif	/* _BOARD_UI_H_ */
