/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: boardlist_ui.h,v 1.30 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BOARDLIST_UI_H_
#define _BOARDLIST_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "tree_ui_impl.h"

#include "libochusha/bulletin_board.h"
#include "libochusha/tree.h"

#include "libochushagtk/tree_selector_view.h"

#include <list>
#include <map>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BBSTable;
  class BoardPropertiesUI;
  class BoardUI;
  class Controller;
  class TreeView;

  struct DirtyBoard
  {
    BulletinBoard *board;
    int dirtyness;

    DirtyBoard(BulletinBoard *board)
      : board(board), dirtyness(board->get_dirtyness()) {}
    DirtyBoard(const DirtyBoard &orig)
      : board(orig.board), dirtyness(orig.dirtyness) {}
  };


  typedef map<BulletinBoard *, int> DirtynessMap;


  class BoardlistUI : public TreeUIImpl
  {
    friend class AddBoardHelper;
    friend class BoardPropertiesUI;
    friend class BoardUI;
    friend class ThreadlistView;
    friend class ViewManager;

  private:
    GtkActionGroup *action_group;
    BBSTable *table;
    FolderHandle oabf_folder;


    // threadlist.xmlɬפ˱¸뤿˻Ȥ
    // ΥեɤGDK_THREADS_ENTER()Ķѹ
    guint periodic_save_id;
    guint save_on_idle_id;

    DirtynessMap dirtyness_map;

    GtkActionGroup *create_action_group();


    void prepare_for_oabf(Folder *folder);

    void save_tree_xml();
    void save_threadlist(BulletinBoard *board);
    static gboolean periodic_save_timeout_cb(BoardlistUI *ui);
    static gboolean save_on_idle_cb(BoardlistUI *ui);

    static void search_boardlist_cb(GtkAction *action, BoardlistUI *ui);
    static void refresh_boardlist_cb(GtkAction *action, BoardlistUI *ui);
    static void add_board_cb(GtkAction *action, BoardlistUI *ui);
    static void manage_boardlist_cb(GtkAction *action, BoardlistUI *ui);

    static void show_hidden_boardlist_items_toggled_cb(GtkAction *action,
						       BoardlistUI *ui);


  protected:
    virtual const gchar *get_element_icon_stock_id(TreeElement *element,
						   gboolean expanded);

    virtual void reconstruct();
    virtual const char *get_popup_menu_root();
    virtual const char *get_folder_tree_popup_menu_root();
    virtual GtkWidget *get_menu_widget();
    virtual void open(TreeElement *element, bool in_tab, bool with_browser,
		      bool select_page);
    virtual void bookmark(TreeElement *element);
    virtual const char *get_url(TreeElement *element);
    virtual const char *get_ui_name();
    virtual const gchar *get_search_title();

    virtual void process_response(GtkDialog *dialog, int response_id,
				  TreeElement *element);
    virtual bool construct_properties_contents(GtkDialog *dialog,
					       TreeElement *element);
    virtual void open_properties(TreeElement *element);

    virtual bool show_all() const;


    virtual bool insert_element(TreeElement *src_element,
				Folder *dest_folder, TreeElement *dest_element,
				GtkTreeViewDropPosition position);

    // src_folder/src_elementפ
    // drop_folder/drop_elementפΰpositionmoveͤΰ٤drop줿
    // Ȼפäн衣moveͤtrueϡְưסfalseϡ֥ԡפ̣롣
    // ˤtrue֤
    virtual bool process_drop_data(Folder *src_folder, TreeElement *src_element,
				   Folder *drop_folder,
				   TreeElement *drop_element,
				   GtkTreeViewDropPosition position,
				   bool move);
    // uri
    // drop_folder/drop_elementפΰpositiondrop줿Ȼפäн衣
    // ˤtrue֤
    virtual bool process_drop_data(const char *uri, Folder *drop_folder,
				   TreeElement *drop_element,
				   GtkTreeViewDropPosition position);


  public:
    BoardlistUI(Controller &controller);
    virtual ~BoardlistUI();

    BulletinBoard *lookup_active_folder(const char *url);

    void refresh_boardlist();
    void save_boardlist();
  };


}


#endif	/* __cplusplus */

#endif	/* _BOARDLIST_UI_H_ */
