/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bookmarks_ui.h,v 1.27 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _BOOKMARKS_UI_H_
#define _BOOKMARKS_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "tree_ui_impl.h"

#include "libochusha/tree.h"

#include "libochushagtk/tree_selector_view.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BBSThread;
  class Bookmarks;
  class Controller;
  class TreeView;

  class BookmarksUI : public TreeUIImpl
  {
    friend class Controller;
    friend class BoardlistUI;

  private:
    GtkActionGroup *action_group;
    Bookmarks *bookmarks;
    FolderHandle my_bookmark_folder;

    GtkAction *open_folder_as_threadlist_action;
    GtkAction *open_folder_as_threadlist_in_tab_action;

    GtkActionGroup *create_action_group();

    static void owner_change_cb(GtkClipboard *clipboard, GdkEvent *event,
				BookmarksUI *ui);

    static void bookmark_this_page_cb(GtkAction *action, BookmarksUI *ui);
    static void manage_bookmarks_cb(GtkAction *action, BookmarksUI *ui);
    static void open_folder_as_threadlist_cb(GtkAction *action,
					     BookmarksUI *ui);
    static void open_folder_as_threadlist_in_tab_cb(GtkAction *action,
						    BookmarksUI *ui);
    static void open_folder_as_threadlist(GtkTreeModel *model,
					  GtkTreePath *path, GtkTreeIter *iter,
					  gpointer data);

    void open_selection_as_threadlist(bool in_tab);

    Folder *folder_new(const char *name);
    void bookmark(Folder *folder, const char *url, const char *name);


  protected:
    virtual void save_tree_xml();
    virtual const gchar *get_element_icon_stock_id(TreeElement *element,
						   gboolean expanded);

    virtual void reconstruct();
    virtual const char *get_popup_menu_root();
    virtual const char *get_folder_tree_popup_menu_root();
    virtual GtkWidget *get_menu_widget();
    virtual void open(TreeElement *element, bool in_tab, bool with_browser,
		      bool select_page);
    virtual void bookmark(TreeElement *element);
    virtual const char *get_url(TreeElement *element);
    virtual const char *get_ui_name();
    virtual const gchar *get_search_title();

    virtual void process_response(GtkDialog *dialog, int response_id,
				  TreeElement *element);
    virtual bool construct_properties_contents(GtkDialog *dialog,
					       TreeElement *element);
    virtual void open_properties(TreeElement *element);

    virtual bool show_all() const;

    virtual void notify_selection_changed(GtkTreeSelection *selection,
					  bool has_folder, bool has_item);

    // src_folder/src_elementפ
    // drop_folder/drop_elementפΰpositionmoveͤΰ٤drop줿
    // Ȼפäн衣moveͤtrueϡְưסfalseϡ֥ԡפ̣롣
    // ˤtrue֤
    virtual bool process_drop_data(Folder *src_folder, TreeElement *src_element,
				   Folder *drop_folder,
				   TreeElement *drop_element,
				   GtkTreeViewDropPosition position,
				   bool move);
    // uri
    // drop_folder/drop_elementפΰpositiondrop줿Ȼפäн衣
    // ˤtrue֤
    virtual bool process_drop_data(const char *uri, Folder *drop_folder,
				   TreeElement *drop_element,
				   GtkTreeViewDropPosition position);


  public:
    BookmarksUI(Controller &controller);
    virtual ~BookmarksUI();

    void bookmark(const char *url, const char *name);

    // ֲѤפб֥åޡ
    void insert_my_bookmark(BBSThread *thread, int res_num, const char *name);
    void remove_my_bookmark(BBSThread *thread);

    Folder *lookup_folder(unsigned int serial_number);

    void save_bookmarks();
  };


}


#endif	/* __cplusplus */

#endif	/* _BOOKMARKS_UI_H_ */
