/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: composite_icon.cc,v 1.5 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "composite_icon.h"

#include "libochusha/misc_utils.h"
#include "libochushagtk/stock.h"

#include "libochushagtk_lgpl/mod_pixbuf_simple_anim.h"

#include <gtk/gtk.h>

#include <cassert>
#include <cstdio>
#include <cstring>
#include <cstdlib>

#include <map>


namespace Ochusha
{

//#define ENABLE_OUTPUT_STATISTICS	1


// GtkWidgetCompositeIcon֥ȤФȤ˻Ȥ
static GQuark icon_id = g_quark_from_static_string("CompositeIcon::icon");


class AddonIconSetCache;

class AddonIconSet
{
  friend class AddonIconSetCache;

private:
  GtkStyle *style;
  GtkTextDirection direction;
  GtkIconSize size;

  GdkPixbuf *done_addon;
  GdkPixbuf *error_addon;
  GdkPixbuf *notice_addon;
  GdkPixbuf *preparation_addon;
  GdkPixbuf *wait_addon;
  GdkPixbuf *progress_addons[OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS];


  AddonIconSet *next_entry;
  inline AddonIconSet *get_next_entry() const { return next_entry; }


public:
  AddonIconSet(GtkWidget *widget, GtkStyle *style, GtkTextDirection direction,
	       GtkIconSize size, AddonIconSet *entry);
  ~AddonIconSet();

  inline bool match(GtkStyle *st, GtkTextDirection dir, GtkIconSize sz) const
  { return st == style && dir == direction && sz == size; }

  inline GtkStyle *get_style() const { return style; }
  inline GtkTextDirection get_direction() const { return direction; }
  inline GtkIconSize get_size() const { return size; }

  inline GdkPixbuf *get_done_addon() const { return done_addon; }
  inline GdkPixbuf *get_error_addon() const { return error_addon; }
  inline GdkPixbuf *get_notice_addon() const { return notice_addon; }
  inline GdkPixbuf *get_preparation_addon() const { return preparation_addon; }
  inline GdkPixbuf *get_wait_addon() const { return wait_addon; }
  inline GdkPixbuf *get_progress_addon(unsigned int n) const
  { return progress_addons[n % OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS]; }

  GdkPixbuf *pickup_icon(GtkWidget *widget, const gchar *stock_id);
};


AddonIconSet::AddonIconSet(GtkWidget *widget, GtkStyle *style,
			   GtkTextDirection direction, GtkIconSize size,
			   AddonIconSet *entry)
  : style(style),
    direction(direction),
    size(size),
    next_entry(entry)
{
  assert(style != NULL);

  g_object_ref(style);

  done_addon = pickup_icon(widget, OCHUSHA_STOCK_NET_DONE_ADDON);
  error_addon = pickup_icon(widget, OCHUSHA_STOCK_NET_ERROR_ADDON);
  notice_addon = pickup_icon(widget, OCHUSHA_STOCK_NET_NOTICE_ADDON);
  preparation_addon = pickup_icon(widget, OCHUSHA_STOCK_NET_PREPARATION_ADDON);
  wait_addon = pickup_icon(widget, OCHUSHA_STOCK_NET_WAIT_ADDON);

  progress_addons[0] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON0);
  progress_addons[1] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON1);
  progress_addons[2] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON2);
  progress_addons[3] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON3);
  progress_addons[4] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON4);
  progress_addons[5] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON5);
  progress_addons[6] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON6);
  progress_addons[7] = pickup_icon(widget, OCHUSHA_STOCK_NET_PROGRESS_ADDON7);

  g_return_if_fail(done_addon != NULL);
  g_return_if_fail(error_addon != NULL);
  g_return_if_fail(notice_addon != NULL);
  g_return_if_fail(preparation_addon != NULL);
  g_return_if_fail(wait_addon != NULL);
  g_return_if_fail(progress_addons[0] != NULL);
  g_return_if_fail(progress_addons[1] != NULL);
  g_return_if_fail(progress_addons[2] != NULL);
  g_return_if_fail(progress_addons[3] != NULL);
  g_return_if_fail(progress_addons[4] != NULL);
  g_return_if_fail(progress_addons[5] != NULL);
  g_return_if_fail(progress_addons[6] != NULL);
  g_return_if_fail(progress_addons[7] != NULL);
}


AddonIconSet::~AddonIconSet()
{
  for (int i = 0; i < OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS; i++)
    if (progress_addons[i] != NULL)
      g_object_unref(progress_addons[i]);

  if (done_addon != NULL)
    g_object_unref(done_addon);

  if (error_addon != NULL)
    g_object_unref(error_addon);

  if (notice_addon != NULL)
    g_object_unref(notice_addon);

  if (preparation_addon != NULL)
    g_object_unref(preparation_addon);

  if (wait_addon != NULL)
    g_object_unref(wait_addon);

  g_object_unref(style);

  if (next_entry != NULL)
    delete next_entry;
}


GdkPixbuf *
AddonIconSet::pickup_icon(GtkWidget *widget, const gchar *stock_id)
{
  GtkIconSet *icon_set = gtk_style_lookup_icon_set(style, stock_id);
  if (icon_set == NULL)
    return NULL;

  return gtk_icon_set_render_icon(icon_set, style, direction, GTK_STATE_NORMAL,
				  size, widget, NULL);
}


class AddonIconSetCache
{
private:
  // ΤǤ
  // styleȤƷɻȤʤǥեstyleȻȤstyleࡣ
  // directionϤɤܸʤΤLTRΤߡ
  // sizeGTK_ICON_SIZE_MENUGTK_ICON_SIZE_DIALOGࡣ
  // ȤȤ롣stylesizeΤɤ򥭡ˤƤƱ褦ʤ
  // Ʊsize³ƻȤȤθsize̯
  // ɤΤsize򥭡ˤƤ
  map<GtkIconSize, AddonIconSet *> cache;

#ifdef ENABLE_OUTPUT_STATISTICS
  long n_entries;
  long n_references;
#endif


public:
#ifdef ENABLE_OUTPUT_STATISTICS
  AddonIconSetCache() : n_entries(0), n_references(0) {}
#endif
  ~AddonIconSetCache();

  AddonIconSet *lookup(GtkWidget *widget, GtkStyle *style, GtkIconSize size);
};


AddonIconSetCache::~AddonIconSetCache()
{
  map<GtkIconSize, AddonIconSet *>::iterator iter = cache.begin();
  map<GtkIconSize, AddonIconSet *>::iterator end_iter = cache.end();
  while (iter != end_iter)
    {
      AddonIconSet *icon_set = (*iter++).second;
      delete icon_set;
    }

#ifdef ENABLE_OUTPUT_STATISTICS
  fprintf(stderr,
	  "~AddonIconSetCache(): n_entries=%ld, n_references=%ld\n",
	  n_entries, n_references);
#endif
}


AddonIconSet *
AddonIconSetCache::lookup(GtkWidget *widget, GtkStyle *style, GtkIconSize size)
{
  assert(style != NULL);

#ifdef ENABLE_OUTPUT_STATISTICS
  ++n_references;
#endif

  GtkTextDirection direction = gtk_widget_get_direction(widget);

  map<GtkIconSize, AddonIconSet *>::iterator iter = cache.find(size);
  AddonIconSet *entry = iter != cache.end() ? (*iter).second : NULL;
  AddonIconSet *first_entry = entry;
  while (entry != NULL)
    {
      // styledirectionsizeƥޥåΤå夵Ƥ
      // ֤
      if (entry->match(style, direction, size))
	return entry;
      entry = entry->get_next_entry();
    }

  // ̵пäƥå夷Ĥġ֤
  entry = new AddonIconSet(widget, style, direction, size, first_entry);

#ifdef ENABLE_OUTPUT_STATISTICS
  ++n_entries;
#endif

  if (iter != cache.end())
    cache.erase(iter);
  cache[size] = entry;

  return entry;
}


static AddonIconSetCache addon_icon_set_cache;


static GdkPixbuf *
composite_pixbufs(GdkPixbuf *base, GdkPixbuf *addon)
{
  if (base == NULL)
    {
      fprintf(stderr, "composite_pixbufs(): base is NULL\n");

      if (addon != NULL)
	g_object_ref(addon);
      return addon;
    }

  if (addon == NULL)
    {
      fprintf(stderr, "composite_pixbufs(): addon is NULL\n");

      g_object_ref(base);
      return base;
    }

  if (gdk_pixbuf_get_colorspace(base) != GDK_COLORSPACE_RGB
      || gdk_pixbuf_get_bits_per_sample(base) != 8
      || !gdk_pixbuf_get_has_alpha(base)
      || gdk_pixbuf_get_n_channels(base) != 4)
    {
      fprintf(stderr, "composite_pixbufs(): Unsupported type of GdkPixbuf given as the base!\n");

      g_object_ref(base);
      return base;
    }

  if (gdk_pixbuf_get_colorspace(addon) != GDK_COLORSPACE_RGB
      || gdk_pixbuf_get_bits_per_sample(addon) != 8
      || !gdk_pixbuf_get_has_alpha(addon)
      || gdk_pixbuf_get_n_channels(addon) != 4)
    {
      fprintf(stderr, "composite_pixbufs(): Unsupported type of GdkPixbuf given as the addon!\n");

      g_object_ref(base);
      return base;
    }

  GdkPixbuf *result_pixbuf = gdk_pixbuf_copy(base);
  if (result_pixbuf == NULL)
    {
      fprintf(stderr, "composite_pixbufs(): Out of memory");

      g_object_ref(base);
      return base;
    }


  gint base_width = gdk_pixbuf_get_width(result_pixbuf);
  gint base_height = gdk_pixbuf_get_height(result_pixbuf);
  gint addon_width = gdk_pixbuf_get_width(addon);
  gint addon_height = gdk_pixbuf_get_height(addon);

  int base_rowstride = gdk_pixbuf_get_rowstride(result_pixbuf);
  guchar *base_pixels = gdk_pixbuf_get_pixels(result_pixbuf);

  int addon_rowstride = gdk_pixbuf_get_rowstride(addon);
  guchar *addon_pixels = gdk_pixbuf_get_pixels(addon);

  for (int y = 0; y < base_height && y < addon_height; y++)
    {
      for (int x = 0; x < base_width && x < addon_width; x++)
	{
	  /* 4 is the constant for n_channels we support */
	  guchar *addon_pixel = addon_pixels + y * addon_rowstride + x * 4;
	  guchar addon_r = addon_pixel[0];
	  guchar addon_g = addon_pixel[1];
	  guchar addon_b = addon_pixel[2];
	  guchar addon_a = addon_pixel[3];

	  guchar *base_pixel = base_pixels + y * base_rowstride + x * 4;
	  guchar base_r = base_pixel[0];
	  guchar base_g = base_pixel[1];
	  guchar base_b = base_pixel[2];
	  guchar base_a = base_pixel[3];

	  guchar base_o = 255 - addon_a;
	  base_pixel[0]
	    = ((base_r * base_o) / 256) + ((addon_r * addon_a) / 256);
	  base_pixel[1]
	    = ((base_g * base_o) / 256) + ((addon_g * addon_a) / 256);
	  base_pixel[2]
	    = ((base_b * base_o) / 256) + ((addon_b * addon_a) / 256);
	  base_pixel[3] = base_a > addon_a ? base_a : addon_a;
	}
    }

  return result_pixbuf;
}


class NetworkStateIconSetCache;

class NetworkStateIconSet
{
  friend class NetworkStateIconSetCache;

private:
  AddonIconSet *icon_set;
  string stock_id;

  GdkPixbufAnimation *wait_animation;
  GdkPixbufAnimation *preparation_animation;
  GdkPixbufAnimation *progress_animation;
  GdkPixbuf *done_icon;
  GdkPixbuf *error_icon;
  GdkPixbuf *notice_icon;

  NetworkStateIconSet *next_entry;
  inline NetworkStateIconSet *get_next_entry() { return next_entry; }


public:
  NetworkStateIconSet(const gchar *stock_id, GtkWidget *widget, GtkStyle *style,
		      GtkTextDirection direction, GtkIconSize size,
		      NetworkStateIconSet *entry);
  ~NetworkStateIconSet();

  inline bool match(const string &id, GtkStyle *style,
		    GtkTextDirection direction, GtkIconSize size) const
  { return icon_set->match(style, direction, size) && stock_id == id; }

  inline const AddonIconSet *get_icon_set() const { return icon_set; }
  inline const string &get_stock_id() const { return stock_id; }

  inline GdkPixbufAnimation *get_wait_animation() const
  { return wait_animation; }
  inline GdkPixbufAnimation *get_preparation_animation() const
  { return preparation_animation; }
  inline GdkPixbufAnimation *get_progress_animation() const
  { return progress_animation; }
  inline GdkPixbuf *get_done_icon() const { return done_icon; }
  inline GdkPixbuf *get_error_icon() const { return error_icon; }
  inline GdkPixbuf *get_notice_icon() const { return notice_icon; }
};


NetworkStateIconSet::NetworkStateIconSet(const gchar *stock_id,
					 GtkWidget *widget, GtkStyle *style,
					 GtkTextDirection direction,
					 GtkIconSize size,
					 NetworkStateIconSet *entry)
  : icon_set(addon_icon_set_cache.lookup(widget, style, size)),
    stock_id(strdup(stock_id)),
    wait_animation(NULL),
    preparation_animation(NULL),
    progress_animation(NULL),
    done_icon(NULL),
    error_icon(NULL),
    notice_icon(NULL),
    next_entry(entry)
{
  GdkPixbuf *base = icon_set->pickup_icon(widget, stock_id);
  g_return_if_fail(base != NULL);

  gint icon_width = gdk_pixbuf_get_width(base);
  gint icon_height = gdk_pixbuf_get_height(base);

  ModPixbufSimpleAnim *anim
    = mod_pixbuf_simple_anim_new(icon_width, icon_height, 2.0);
  mod_pixbuf_simple_anim_set_loop(anim, TRUE);
  mod_pixbuf_simple_anim_add_frame(anim, base);
  GdkPixbuf *frame = composite_pixbufs(base, icon_set->get_wait_addon());
  if (frame != NULL)
    {
      mod_pixbuf_simple_anim_add_frame(anim, frame);
      g_object_unref(frame);
    }
  wait_animation = GDK_PIXBUF_ANIMATION(anim);


  anim = mod_pixbuf_simple_anim_new(icon_width, icon_height, 2.0);
  mod_pixbuf_simple_anim_set_loop(anim, TRUE);
  mod_pixbuf_simple_anim_add_frame(anim, base);
  frame = composite_pixbufs(base, icon_set->get_preparation_addon());
  if (frame != NULL)
    {
      mod_pixbuf_simple_anim_add_frame(anim, frame);
      g_object_unref(frame);
    }
  preparation_animation = GDK_PIXBUF_ANIMATION(anim);


  done_icon = composite_pixbufs(base, icon_set->get_done_addon());
  error_icon = composite_pixbufs(base, icon_set->get_error_addon());
  notice_icon = composite_pixbufs(base, icon_set->get_notice_addon());


  anim = mod_pixbuf_simple_anim_new(icon_width, icon_height, 16.0);
  mod_pixbuf_simple_anim_set_loop(anim, TRUE);

  for (int i = 0; i < OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS; i++)
    {
      frame = composite_pixbufs(base, icon_set->get_progress_addon(i));
      if (frame != NULL)
	{
	  mod_pixbuf_simple_anim_add_frame(anim, frame);
	  g_object_unref(frame);
	}
    }

  progress_animation = GDK_PIXBUF_ANIMATION(anim);

  g_object_unref(base);
}


NetworkStateIconSet::~NetworkStateIconSet()
{
  if (next_entry != NULL)
    delete next_entry;
}


class NetworkStateIconSetCache
{
private:
  map<string, NetworkStateIconSet *> cache;

#ifdef ENABLE_OUTPUT_STATISTICS
  long n_entries;
  long n_references;
#endif


public:
#ifdef ENABLE_OUTPUT_STATISTICS
  NetworkStateIconSetCache() : n_entries(0), n_references(0) {}
#endif
  ~NetworkStateIconSetCache();

  NetworkStateIconSet *lookup(const gchar *stock_id, GtkWidget *widget,
			      GtkStyle *style, GtkIconSize size);
};


NetworkStateIconSetCache::~NetworkStateIconSetCache()
{
  map<string, NetworkStateIconSet *>::iterator iter = cache.begin();
  map<string, NetworkStateIconSet *>::iterator end_iter = cache.end();
  while (iter != end_iter)
    {
      NetworkStateIconSet *icon_set = (*iter++).second;
      delete icon_set;
    }

#ifdef ENABLE_OUTPUT_STATISTICS
  fprintf(stderr,
	  "~NetworkStateIconSetCache(): n_entries=%ld, n_references=%ld\n",
	  n_entries, n_references);
#endif
}


NetworkStateIconSet *
NetworkStateIconSetCache::lookup(const gchar *stock_id, GtkWidget *widget,
				 GtkStyle *style, GtkIconSize size)
{
  g_return_val_if_fail(stock_id != NULL, NULL);

  if (style == NULL)
    {
      gtk_widget_ensure_style(widget);
      style = gtk_widget_get_style(widget);
    }
  g_return_val_if_fail(style != NULL, NULL);

#ifdef ENABLE_OUTPUT_STATISTICS
  ++n_references;
#endif

  GtkTextDirection direction = gtk_widget_get_direction(widget);

  string id(stock_id);

  map<string, NetworkStateIconSet *>::iterator iter = cache.find(id);
  NetworkStateIconSet *entry = iter != cache.end() ? (*iter).second : NULL;
  NetworkStateIconSet *first_entry = entry;
  while (entry != NULL)
    {
      if (entry->match(id, style, direction, size))
	return entry;
      entry = entry->get_next_entry();
    }

  entry = new NetworkStateIconSet(stock_id, widget, style, direction, size,
				  first_entry);

#ifdef ENABLE_OUTPUT_STATISTICS
  ++n_entries;
#endif

  if (iter != cache.end())
    cache.erase(iter);
  cache[id] = entry;

  return entry;
}


static NetworkStateIconSetCache network_state_icon_set_cache;


CompositeIcon::CompositeIcon(GtkWidget *widget, const gchar *stock_id,
			     GtkIconSize size)
  : stock_id(strdup(stock_id)),
    icon_set(network_state_icon_set_cache.lookup(stock_id, widget, NULL, size))
{
  g_object_set_qdata_full(G_OBJECT(widget), icon_id, this,
			  reinterpret_cast<GDestroyNotify>(destroy));
  g_signal_connect(widget, "style-set",
		   reinterpret_cast<GCallback>(style_set_cb), this);
}


CompositeIcon::~CompositeIcon()
{
  free(stock_id);
}


void
CompositeIcon::style_set_cb(GtkWidget *widget, GtkStyle *previous_style,
			    CompositeIcon *icon)
{
  NetworkStateIconSet *icon_set = icon->icon_set;
  g_return_if_fail(icon_set != NULL);
  g_return_if_fail(previous_style == NULL
		   || icon_set->get_icon_set()->get_style() == previous_style);

  GtkStyle *style = gtk_widget_get_style(widget);
  g_return_if_fail(style != NULL);

  if (icon_set->get_icon_set()->get_style() == style)
    return;

  icon->icon_set
    = network_state_icon_set_cache.lookup(icon->stock_id, widget,
					  style,
					  icon_set->get_icon_set()->get_size());
}


void
CompositeIcon::destroy(CompositeIcon *icon)
{
  delete icon;
}


CompositeIcon *
CompositeIcon::get_composite_icon(GtkWidget *widget)
{
  return static_cast<CompositeIcon *>(g_object_get_qdata(G_OBJECT(widget),
							 icon_id));
}


GdkPixbufAnimation *
CompositeIcon::get_wait_animation() const
{
  return icon_set->get_wait_animation();
}


GdkPixbufAnimation *
CompositeIcon::get_preparation_animation() const
{
  return icon_set->get_preparation_animation();
}


GdkPixbufAnimation *
CompositeIcon::get_progress_animation() const
{
  return icon_set->get_progress_animation();
}


GdkPixbuf *
CompositeIcon::get_done_icon() const
{
  return icon_set->get_done_icon();
}


GdkPixbuf *
CompositeIcon::get_error_icon() const
{
  return icon_set->get_error_icon();
}


GdkPixbuf *
CompositeIcon::get_notice_icon() const
{
  return icon_set->get_notice_icon();
}


}
