/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: controller.h,v 1.58 2009/01/01 11:02:28 fuyu Exp $
 */

#ifndef _CONTROLLER_H_
#define _CONTROLLER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bbs_table.h"
#include "libochusha/cookie_manager.h"
#include "libochusha/configuration.h"
#include "libochusha/history_manager.h"
#include "libochusha/network_agent.h"
#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"
#include "libochusha/thread_pool.h"

#include "about_ui.h"
#include "bbs_2ch_helper.h"
#include "bbs_thread_ui.h"
#include "board_properties_ui.h"
#include "board_ui.h"
#include "boardlist_ui.h"
#include "bookmarks_ui.h"
#include "console.h"
#include "fonts_ui.h"
#include "html_renderer.h"
#include "link_ui.h"
#include "main_window.h"
#include "preferences.h"
#include "preferences_ui.h"
#include "scheme_manager.h"
#include "tab_label.h"
#include "threadlist_ui.h"
#include "tree_ui.h"

#include "libochushagtk/entry_action.h"

#include <list>
#include <string>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BoardMovementWatcher;
  class CommandPipe;
  class ImageManager;
  class ImageManagerWatcher;
  class NetworkStatusWatcher;
  class Repository;
  class SelectionData;
  class SessionClient;


  struct PendingLog
  {
    LogCategory category;
    string message;

    PendingLog(LogCategory category, const char *message)
      : category(category), message(message) {}
  };
  

  class Controller
  {
    friend class BoardMovementWatcher;
    friend class BoardUI;
    friend class DATFileExplorerWatcher;
    friend class CommandPipe;
    friend class ImageManagerWatcher;
    friend class MainWindow;
    friend class NetworkStatusWatcher;
    friend class ReadThreadlistJob;

  private:
    Repository &repository;
    CommandPipe &command_pipe;
    SessionClient &session;
    Configuration config;
    smart_ptr<BBSTable> bbs_table;

    CookieManager &cookie_manager;
    PollEventManager poll_manager;
    smart_ptr<ThreadPool> thread_pool;
    NetworkAgentBroker *broker;

    HistoryManager &history_manager;
    HistoryManager &post_history_manager;

    GtkClipboard *clipboard;
    GtkClipboard *clipboard_primary;
    GtkClipboard *clipboard_internal;

    GtkUIManager *ui_manager;
    GtkActionGroup *action_group;
    Preferences prefs;

    SchemeManager scheme_manager;

    HTMLRendererTagManager tag_manager;

    BBS2chHelper bbs2ch;

    BookmarksUI bookmarks_ui;
    BBSThreadUI thread_ui;
    BoardPropertiesUI board_properties_ui;
    BoardUI board_ui;
    BoardlistUI boardlist_ui;
    LinkUI link_ui;
    ThreadlistUI threadlist_ui;
    TreeUI tree_ui;

    MainWindow main_window;
    ViewManager &view_manager;

    PreferencesUI prefs_ui;
    FontsUI fonts_ui;
    AboutUI about;

    Console notification;

    Console debug_console;
    list<PendingLog> pending_logs;

    ImageManager *image_manager;

    BoardMovementWatcher *board_movement_watcher;
    ImageManagerWatcher *image_manager_watcher;
    NetworkStatusWatcher *network_status_watcher;

    GtkWidget *open_url_dialog;


    guint process_pending_logs_on_idle_id;

    // ǥХå󥽡ؤνϴϢ
    void process_pending_logs();
    static gboolean process_pending_logs_on_idle_cb(Controller *controller);



    // config.xmlɤ˸߻ͤɤؤ롣
    void restore_configuration();

    // ǤbookmarkѴ롣
    void convert_virtual_board(BulletinBoard *board);
    void convert_virtual_boards();

    // ƤĤˤĤơϿƤ륹ɤ߹ࡣ
    void read_all_threadlists();


    GtkWidget *create_open_url_dialog();
    static void open_url_dialog_response_cb(GtkWidget *dialog, int response_id,
					    Controller *controller);
    void show_open_url_dialog();


    void notify_board_moved(BBSTable *table, BulletinBoard *board,
			    const char *new_url);


    void output_log(LogCategory category, const char *message);
    void pre_prepare_request(NetworkAgent *agent);
    void post_prepare_request(NetworkAgent *agent);


    static void owner_change_cb(GtkClipboard *clip, GdkEvent *event,
				Controller *controller);

    void notify_clipboard_changed(GtkClipboard *clip);
    void notify_clipboard_primary_changed(GtkClipboard *clip);
    void notify_clipboard_internal_changed(GtkClipboard *clip);


    // GtkActionϢʥϥɥ
    static void open_url_cb(GtkAction *action, Controller *controller);
    static void open_url_action_activate_cb(OchushaEntryAction *action,
					    const gchar *url,
					    Controller *controller);
    static void clear_cookies_cb(GtkAction *action, Controller *controller);
    static void open_file_cb(GtkAction *action, Controller *controller);
    static void quit_cb(GtkAction *action, Controller *controller);
    static void cut_cb(GtkAction *action, Controller *controller);
    static void copy_cb(GtkAction *action, Controller *controller);
    static void delete_cb(GtkAction *action, Controller *controller);
    static void paste_cb(GtkAction *action, Controller *controller);
    static void stop_cb(GtkAction *action, Controller *controller);
    static void notification_cb(GtkAction *action, Controller *controller);
    static void debug_console_cb(GtkAction *action, Controller *controller);
    static void scheme_console_cb(GtkAction *action, Controller *controller);
    static void preferences_cb(GtkAction *action, Controller *controller);
    static void select_list_fonts_cb(GtkAction *action, Controller *controller);
    static void select_thread_fonts_cb(GtkAction *action,
				       Controller *controller);
    static void about_cb(GtkAction *action, Controller *controller);
    static void help_cb(GtkAction *action, Controller *controller);
    static void work_offline_toggled_cb(GtkToggleAction *action,
					Controller *controller);


    GtkActionGroup *create_action_group();

    GtkToggleAction *work_offline_action;
    GtkAction *cut_action;
    GtkAction *copy_action;
    GtkAction *delete_action;
    GtkAction *paste_action;
    GtkAction *stop_action;
    OchushaEntryAction *open_url_action;


  public:
    Controller(Repository &repository, CommandPipe &command_pipe,
	       SessionClient &session, CookieManager &cookie_manager,
	       HistoryManager &history_manager,
	       HistoryManager &post_history_manager);
    virtual ~Controller();

    inline Repository &get_repository() { return repository; }
    inline CommandPipe &get_command_pipe() { return command_pipe; }
    inline SessionClient &get_session() { return session; }
    inline Configuration &get_config() { return config; }
    inline BBSTable &get_bbs_table() { return *bbs_table; }
    inline GtkUIManager *get_ui_manager() { return ui_manager; }
    inline Preferences &get_prefs() { return prefs; }
    inline HTMLRendererTagManager &get_tag_manager() { return tag_manager; }
    inline BBS2chHelper &get_bbs2ch() { return bbs2ch; }
    inline BBSThreadUI &get_thread_ui() { return thread_ui; }
    inline BoardPropertiesUI &get_board_properties_ui()
    { return board_properties_ui; }
    inline BoardUI &get_board_ui() { return board_ui; }
    inline BoardlistUI &get_boardlist_ui() { return boardlist_ui; }
    inline BookmarksUI &get_bookmarks_ui() { return bookmarks_ui; }
    inline LinkUI &get_link_ui() { return link_ui; }
    inline ThreadlistUI &get_threadlist_ui() { return threadlist_ui; }
    inline TreeUI &get_tree_ui() { return tree_ui; }
    inline ViewManager &get_view_manager() { return view_manager; }
    inline SchemeManager &get_scheme_manager() { return scheme_manager; }
    inline ThreadPool &get_thread_pool()
    { return *(static_cast<ThreadPool *>(thread_pool)); }
    inline CookieManager &get_cookie_manager() { return cookie_manager; }
    inline NetworkAgentBroker &get_broker() { return *broker; }
    inline ImageManager &get_image_manager() { return *image_manager; }

    inline HistoryManager &get_history_manager() { return history_manager; }
    inline HistoryManager &get_post_history_manager()
    { return post_history_manager; }

    inline GtkWindow *get_toplevel() { return main_window.toplevel; }
    inline GtkWidget *get_searchbar() { return main_window.searchbar; }
    inline guint get_searchbar_context_id()
    { return main_window.searchbar_context_id; }
    inline guint get_boardlist_context_id()
    { return main_window.boardlist_context_id; }
    inline guint get_link_context_id()
    { return main_window.link_context_id; }
    inline GtkStatusbar *get_statusbar() { return main_window.statusbar; }


    void spawn_command(const char *command_line);

    void save_yourself();
    void quit();

    void prepare_shutdown();

    bool is_bbs_link(const char *link);
    bool is_board_url(const char *url);
    bool is_thread_url(const char *url);

    void open(const char *url, bool refresh, bool in_tab, bool with_browser,
	      bool select_tab = false);
    void open(BulletinBoard *board, bool refresh, bool in_tab,
	      bool with_browser, bool select_tab = false);
    void open(BBSThread *thread, bool refresh, int initial_pos, bool in_tab,
	      bool with_browser, bool select_tab = false);

    void refresh(BulletinBoard *board, bool reload);
    void refresh(BBSThread *thread, bool automatic = false);

    gchar *get_bookmark_title(const char *url);

    void bookmark(const char *url, const char *name);
    void bookmark(BulletinBoard *board);
    void bookmark(BBSThread *thread);

    void copy(const char *text);


    bool update_clipboard(SelectionData *data);
    void clear_clipboard(SelectionData *data);
    inline GtkClipboard *get_clipboard() { return clipboard; }

    bool update_clipboard_primary(SelectionData *data);
    void clear_clipboard_primary(SelectionData *data);
    inline GtkClipboard *get_clipboard_primary() { return clipboard_primary; }

    bool update_clipboard_internal(SelectionData *data);
    void clear_clipboard_internal(SelectionData *data);
    inline GtkClipboard *get_clipboard_internal() { return clipboard_internal; }

    void treat_board_movement(BulletinBoard *board, const char *new_url,
			      bool show_notification);
  };


}


#endif	/* __cplusplus */

#endif	/* _CONTROLLER_H_ */
