/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: focus_utils.h,v 1.4 2010/08/15 15:42:20 fuyu Exp $
 */

#ifndef _FOCUS_UTILS_H_
#define _FOCUS_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "selection_utils.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  class Controller;

  class FocusNotifier
  {
  private:
    Controller &controller;
    GtkWidget *widget;
    gulong focus_in_event_id;
    gulong focus_out_event_id;
    gulong enter_notify_event_id;
    gulong leave_notify_event_id;
    gulong realize_id;
    gulong unmap_id;
    gulong destroy_id;
    gulong hide_id;
    gulong state_changed_id;

    bool in_focus;
    bool with_pointer;

    bool delete_when_widget_destroyed;

    void disconnect_all();
    void set_events_mask(GdkWindow *window);

    static gboolean focus_in_event_cb(GtkWidget *widget, GdkEventFocus *event,
				      FocusNotifier *notifier);
    static gboolean focus_out_event_cb(GtkWidget *widget, GdkEventFocus *event,
				       FocusNotifier *notifier);
    static gboolean enter_notify_event_cb(GtkWidget *widget,
					  GdkEventCrossing *event,
					  FocusNotifier *notifier);
    static gboolean leave_notify_event_cb(GtkWidget *widget,
					  GdkEventCrossing *event,
					  FocusNotifier *notifier);
    static void realize_cb(GtkWidget *widget, FocusNotifier *notifier);
    static void unmap_cb(GtkWidget *widget, FocusNotifier *notifier);
    static void destroy_cb(GtkWidget *widget, FocusNotifier *notifier);
    static void hide_cb(GtkWidget *widget, FocusNotifier *notifier);
    static void state_changed_cb(GtkWidget *widget, GtkStateType state,
				 FocusNotifier *notifier);


  protected:
    inline Controller &get_controller() { return controller; }

    virtual GdkWindow *get_window();

    virtual gboolean notify_focus_in();
    virtual gboolean notify_focus_out();
    virtual gboolean notify_enter();
    virtual gboolean notify_leave();
    virtual void notify_hide();
    virtual void notify_unmap();
    virtual void notify_realize();
    virtual void notify_state_changed(GtkStateType state);


  public:
    FocusNotifier(Controller &controller, GtkWidget *widget,
		  bool delete_when_widget_destroyed);
    virtual ~FocusNotifier();

    void set_widget(GtkWidget *widget);
    inline GtkWidget *get_widget() { return widget; }

    inline bool has_focus() const { return in_focus; }
    inline bool has_pointer() const { return with_pointer; }
  };


  class TextViewFocusNotifier : public FocusNotifier
  {
  private:
    TextViewSelectionData selection_data;

    gboolean has_selection;

    static void notify_has_selection_cb(GtkTextBuffer *buffer,
					GParamSpec *pspec,
					TextViewFocusNotifier *notifier);
    static void populate_popup_cb(GtkTextView *view, GtkMenu *popup_menu,
				  TextViewFocusNotifier *notifier);


  protected:
    virtual gboolean notify_focus_in();
    virtual void notify_hide();
    virtual void notify_unmap();
    virtual void notify_state_changed(GtkStateType state);


  public:
    TextViewFocusNotifier(Controller &controller, GtkTextView *view,
			  bool delete_when_widget_destroyed);
    virtual ~TextViewFocusNotifier();

    static GtkWidget *gtk_text_view_new_with_buffer(Controller &controller,
						    GtkTextBuffer *buffer);
  };


  class EntryFocusNotifier : public FocusNotifier
  {
  private:
    EditableSelectionData selection_data;

    gboolean has_selection;

    static void notify_selection_bound_cb(GtkEntry *entry,
					  GParamSpec *pspec,
					  EntryFocusNotifier *notifier);
    static void populate_popup_cb(GtkEntry *entry, GtkMenu *popup_menu,
				  EntryFocusNotifier *notifier);


  protected:
    virtual gboolean notify_focus_in();
    virtual gboolean notify_enter();
    virtual void notify_hide();
    virtual void notify_unmap();
    virtual void notify_state_changed(GtkStateType state);


  public:
    EntryFocusNotifier(Controller &controller, GtkEntry *entry,
		       bool delete_when_widget_destroyed);
    virtual ~EntryFocusNotifier();

    static GtkWidget *gtk_entry_new(Controller &controller);
  };


}


#endif	/* __cplusplus */

#endif	/* _FOCUS_UTILS_H_ */
