/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: fonts_ui.cc,v 1.16 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "fonts_ui.h"

#include "libochusha/repository.h"

#include "controller.h"
#include "preferences.h"
#include "tree_view.h"
#include "ui_constants.h"

#include <gtk/gtk.h>

#include <cstdio>
#include <cstdlib>
#include <cstring>


namespace Ochusha
{


FontsUI::FontsUI(Controller &controller)
  : controller(controller),
    tree_view_font_selection_dialog(NULL),
    thread_view_font_selection_dialog(NULL)
{
  controller.get_prefs().prefs_changed.connect(this,
					       &FontsUI::prefs_changed_cb);
}


FontsUI::~FontsUI()
{
  assert(OCHUSHA_IN_GDK_THREADS());

  if (tree_view_font_selection_dialog != NULL)
    gtk_widget_destroy(tree_view_font_selection_dialog);

  if (thread_view_font_selection_dialog != NULL)
    gtk_widget_destroy(thread_view_font_selection_dialog);
}


void
FontsUI::prepare_shutdown()
{
  if (tree_view_font_selection_dialog != NULL)
    gtk_widget_destroy(tree_view_font_selection_dialog);

  if (thread_view_font_selection_dialog != NULL)
    gtk_widget_destroy(thread_view_font_selection_dialog);

  g_return_if_fail(tree_view_font_selection_dialog == NULL);
  g_return_if_fail(thread_view_font_selection_dialog == NULL);
}


void
FontsUI::prefs_changed_cb(Preferences &prefs)
{
  assert(OCHUSHA_IN_GDK_THREADS());

  // ɽѥեѹ񤭹ߥǥȲѤ
  // եȿǤ롣
  gchar *font_name = get_current_tree_view_font();
  write_prefs_gtkrc(font_name);
  if (font_name != NULL)
    g_free(font_name);

  reload_prefs_gtkrc();
}


void
FontsUI::write_prefs_gtkrc(const gchar *tree_view_font_name)
{
  int fd = controller.get_repository().open_file(OCHUSHA_PREFS_GTKRC,
						 O_WRONLY | O_TRUNC | O_CREAT);
  if (fd >= 0)
    {
      FILE *file = fdopen(fd, "w");
      if (file != NULL)
	{
	  fprintf(file, "# " OCHUSHA_PREFS_GTKRC " generated by "
		  PACKAGE_STRING ".\n");
	  fprintf(file, "# Do NOT Edit!\n\n");
	  fprintf(file, "style \"ochusha-tree-view-style\"\n{\n");
	  if (tree_view_font_name != NULL)
	    fprintf(file, "  font_name=\"%s\"\n", tree_view_font_name);
	  else
	    fprintf(file, "  font_name=\"\"\n");
	  fprintf(file, "}\n\n");
	  fprintf(file, "class \"OchushaTreeSelectorView\""\
		  " style \"ochusha-tree-view-style\"\n");

	  const char *thread_view_font_name
	    = controller.get_prefs().get_thread_view_font_name();
	  if (thread_view_font_name != NULL)
	    {
	      fprintf(file, "\n\nstyle \"ochusha-thread-view-style\"\n{\n");
	      fprintf(file, "  font_name=\"%s\"\n", thread_view_font_name);
	      fprintf(file, "}\n\n");
	      fprintf(file, "widget \"*.mybookmark.GtkLabel\""
		      " style \"ochusha-thread-view-style\"\n");
	      fprintf(file, "widget \"*.mybookmarkeditor\""
		      " style \"ochusha-thread-view-style\"\n");
	      fprintf(file, "widget \"*.responseeditor\""
		      " style \"ochusha-thread-view-style\"\n");
	      fprintf(file, "class \"OchushaHypertextView\""
		      " style \"ochusha-thread-view-style\"\n");
	    }
	  fclose(file);
	}
      else
	close(fd);
    }
}


void
FontsUI::reload_prefs_gtkrc()
{
  char *rc_file = controller.get_repository().find_file(OCHUSHA_PREFS_GTKRC);

  if (rc_file != NULL)
    {
      gtk_rc_parse(rc_file);
      gtk_rc_add_default_file(rc_file);
      free(rc_file);
    }

  gtk_widget_reset_rc_styles(GTK_WIDGET(controller.get_toplevel()));
}


gchar *
FontsUI::get_current_tree_view_font()
{
  TreeView *view
    = dynamic_cast<TreeView *>(controller.get_boardlist_ui().get_master_view());
  g_return_val_if_fail(view != NULL, NULL);

  GtkWidget *tree_view = view->get_tree_view();
  if (tree_view != NULL)
    {
      PangoContext *context = gtk_widget_get_pango_context(tree_view);
      PangoFontDescription *font_desc
	= pango_context_get_font_description(context);

      return pango_font_description_to_string(font_desc);
    }

  return NULL;
}


GtkWidget *
FontsUI::setup_tree_view_font_selection_dialog()
{
  g_return_val_if_fail(tree_view_font_selection_dialog == NULL,
		       tree_view_font_selection_dialog);

  GtkWidget *dialog = gtk_font_selection_dialog_new(_("Boardlist, Bookmarks and Threadlist View Font Selection"));
  gchar *font_name = get_current_tree_view_font();

  if (font_name != NULL)
    {
      gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dialog),
					      font_name);
      g_free(font_name);
    }

  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(tree_view_font_selection_dialog_response_cb),
		   this);
  g_signal_connect(dialog, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed),
		   &tree_view_font_selection_dialog);
  gtk_widget_show(GTK_FONT_SELECTION_DIALOG(dialog)->apply_button);

  return dialog;
}


void
FontsUI::tree_view_font_selection_dialog_response_cb(GtkWidget *dialog,
						     int response_id,
						     FontsUI *ui)
{
  g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(dialog));

  GtkFontSelectionDialog *fsd = GTK_FONT_SELECTION_DIALOG(dialog);
  switch (response_id)
    {
    case GTK_RESPONSE_APPLY:
    case GTK_RESPONSE_OK:
      {
	gchar *font_name = gtk_font_selection_dialog_get_font_name(fsd);
	ui->write_prefs_gtkrc(font_name);
	ui->reload_prefs_gtkrc();
	if (font_name != NULL)
	  g_free(font_name);
	if (response_id == GTK_RESPONSE_APPLY)
	  return;
	break;
      }

    default:
      {
	gchar *font_name = ui->get_current_tree_view_font();
	gtk_font_selection_dialog_set_font_name(fsd, font_name);
	if (font_name != NULL)
	  g_free(font_name);
	break;
      }
    }

  gtk_widget_hide(dialog);
}


void
FontsUI::show_tree_view_font_selection_dialog()
{
  if (tree_view_font_selection_dialog == NULL)
    tree_view_font_selection_dialog = setup_tree_view_font_selection_dialog();

  gtk_widget_show(tree_view_font_selection_dialog);
}


GtkWidget *
FontsUI::setup_thread_view_font_selection_dialog()
{
  g_return_val_if_fail(thread_view_font_selection_dialog == NULL,
		       thread_view_font_selection_dialog);

  GtkWidget *dialog = gtk_font_selection_dialog_new(_("Thread View Font Selection"));
  const gchar *font_name = controller.get_prefs().get_thread_view_font_name();

  if (font_name != NULL)
    gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dialog),
					    font_name);

  g_signal_connect(dialog, "response",
		   reinterpret_cast<GCallback>(thread_view_font_selection_dialog_response_cb),
		   this);
  g_signal_connect(dialog, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed),
		   &thread_view_font_selection_dialog);
  gtk_widget_show(GTK_FONT_SELECTION_DIALOG(dialog)->apply_button);

  return dialog;
}


void
FontsUI::thread_view_font_selection_dialog_response_cb(GtkWidget *dialog,
						       int response_id,
						       FontsUI *ui)
{
  g_return_if_fail(GTK_IS_FONT_SELECTION_DIALOG(dialog));

  GtkFontSelectionDialog *fsd = GTK_FONT_SELECTION_DIALOG(dialog);
  switch (response_id)
    {
    case GTK_RESPONSE_APPLY:
    case GTK_RESPONSE_OK:
      {
	gchar *font_name = gtk_font_selection_dialog_get_font_name(fsd);
	ui->controller.get_prefs().set_thread_view_font_name(font_name);
	if (font_name != NULL)
	  g_free(font_name);
	if (response_id == GTK_RESPONSE_APPLY)
	  return;
	break;
      }

    default:
      {
	const gchar *font_name
	  = ui->controller.get_prefs().get_thread_view_font_name();
	gtk_font_selection_dialog_set_font_name(fsd, font_name);
	break;
      }
    }

  gtk_widget_hide(dialog);
}


void
FontsUI::show_thread_view_font_selection_dialog()
{
  if (thread_view_font_selection_dialog == NULL)
    thread_view_font_selection_dialog
      = setup_thread_view_font_selection_dialog();

  gtk_widget_show(thread_view_font_selection_dialog);
}


}
