/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: gtk_utils.cc,v 1.2 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "gtk_utils.h"

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>


//#define VERBOSE_DESTRUCTOR	1


namespace Ochusha
{


TreePathHandle &
TreePathHandle::operator=(GtkTreePath *value)
{
  if (path != NULL)
    gtk_tree_path_free(path);
  path = value;
  return *this;
}


TreePathHandle &
TreePathHandle::operator=(const TreePathHandle &tph)
{
  if (path != NULL)
    gtk_tree_path_free(path);
  path = tph.path;
  return *this;
}


static GQuark handler_id =
  g_quark_from_static_string("KeyPressHandler::handler");


KeyPressHandler::KeyPressHandler(GtkWidget *widget)
  : widget(widget)
{
  g_signal_connect(widget, "key_press_event",
		   reinterpret_cast<GCallback>(key_press_event_cb), this);
  g_object_set_qdata_full(G_OBJECT(widget), handler_id, this,
			  reinterpret_cast<GDestroyNotify>(destroy));
}


KeyPressHandler::~KeyPressHandler()
{
#ifdef VERBOSE_DESTRUCTOR
  fprintf(stderr, "KeyPressHandler::~KeyPressHandler(): this=%p\n", this);
#endif
}


void
KeyPressHandler::destroy(KeyPressHandler *handler)
{
  delete handler;
}


gboolean
KeyPressHandler::key_press_event_cb(GtkWidget *widget, GdkEventKey *event,
				    KeyPressHandler *handler)
{
  g_return_val_if_fail(widget == handler->widget, FALSE);

  return handler->key_press_event(event);
}


gboolean
KeyPressHandler::key_press_event(GdkEventKey *event)
{
  return FALSE;
}


TextViewKeyPressHandler::TextViewKeyPressHandler(GtkTextView *text_view,
						 Controller &controller)
  : KeyPressHandler(GTK_WIDGET(text_view)),
    controller(controller)
{
}


TextViewKeyPressHandler::~TextViewKeyPressHandler()
{
#ifdef VERBOSE_DESTRUCTOR
  fprintf(stderr, "TextViewKeyPressHandler::~TextViewKeyPressHandler(): this=%p\n", this);
#endif
}


gboolean
TextViewKeyPressHandler::key_press_event(GdkEventKey *event)
{
  /*
   * OchushaHypertextView˰¸ߤ
   * ochusha_hypertext_view_key_press_event(...)饳Ȥȴ衣
   *
   * ȴ褷
   * MEMO#1: gtk+-2.10.xgtk+-2.12.xΤɤǡ!editableGtkTextView
   * ˤkey_press_eventȿѤꡢ٥ȥϥɥǽʤ
   * ٥(editableʤ餽ʸϤ褦ʥ٥)ξˡ
   * view-portʤinsertion point˰ư(!editableȤٹ
   * ΰ̣äݤ)٥뤬ĤȤưˤʤä񤭹ʤȤϺǽ餫
   * ȽäƤΤ顢Ʊ褦̵뤷ߤΤġġ
   *
   * MEMO#2: gtk+-2.14.xǤˤʤޤġġ
   * !editableGtkTextViewˤkey_press_eventȿƤѤꡢ
   * Ǥ⤫Ǥ̵뤵褦ˤʤäƤ롽PageUp/PageDownȤ
   * binding_setʬ̵뤵Ƥ롽ꤨʤ͡
   *
   * ŪˤϡμťΤϥϡɥǥ󥰤ΤǤϤʤ
   * scheme٥˲widgetȤƤϤ˾٤ǤϤʤ
   * Ū˰ڤ̵뤹褦ˤƤŪˤ
   * key_press_event˥ϥɥꤹɤ
   * ȴ褷ȡ
   *
   *
   * Ȥǡ̤˥ϥɥꤹ뤳ȤˤΥǥե
   * ĤǡGtkTextViewΥǥեȤΥХǥ󥰤⡢ڡ
   * ư˴ؤΤäĤǤSpace/BackSpace˸
   * ؤΥڡưХɤƤ롣
   *
   * ΤȤschemeؿؤβˤϽФƤʤ줽ˤꡢ
   * 桼ΥХǥ󥰤¸Ǥ븫ߡ
   *
   * ȼ餷ochusha_hypertext_view_key_press_event(...)
   * Ϻ
   */

  GtkTextView *view = GTK_TEXT_VIEW(get_widget());
  g_return_val_if_fail(view != NULL, FALSE);

  GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
  if (buffer == NULL)
    return FALSE;

  GtkTextMark *insert_mark = gtk_text_buffer_get_insert(buffer);
  GtkTextIter insert_iter;
  gtk_text_buffer_get_iter_at_mark(buffer, &insert_iter, insert_mark);
  if (gtk_text_iter_can_insert(&insert_iter, gtk_text_view_get_editable(view)))
    return FALSE;	// editableʤʪˤǤ

  GtkMovementStep step;
  gint count;
  gboolean extend_selection;

  guint keyval = event->keyval;
  guint state = event->state;

  switch (keyval)
    {
    case GDK_Right:
    case GDK_KP_Right:
    case GDK_Left:
    case GDK_KP_Left:
      {
	if (keyval == GDK_Right || keyval == GDK_KP_Right)
	  count = 1;
	else
	  count = -1;

	if (state == 0)
	  {
	    step = GTK_MOVEMENT_VISUAL_POSITIONS;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT(state))
	  {
	    step = GTK_MOVEMENT_VISUAL_POSITIONS;
	    extend_selection = TRUE;
	  }
	else if (EVENT_KEY_WITH_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_WORDS;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_WORDS;
	    extend_selection = TRUE;
	  }
	else
	  return FALSE;
	break;
      }

    case GDK_Up:
    case GDK_KP_Up:
    case GDK_Down:
    case GDK_KP_Down:
      {
	if (keyval == GDK_Up || keyval == GDK_KP_Up)
	  count = -1;
	else
	  count = 1;

	if (state == 0)
	  {
	    step = GTK_MOVEMENT_DISPLAY_LINES;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT(state))
	  {
	    step = GTK_MOVEMENT_DISPLAY_LINES;
	    extend_selection = TRUE;
	  }
	else if (EVENT_KEY_WITH_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_PARAGRAPHS;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_PARAGRAPHS;
	    extend_selection = TRUE;
	  }
	else
	  return FALSE;
	break;
      }

    case GDK_Home:
    case GDK_KP_Home:
    case GDK_End:
    case GDK_KP_End:
      {
	if (keyval == GDK_Home || keyval == GDK_KP_Home)
	  count = -1;
	else
	  count = 1;

	if (state == 0)
	  {
	    step = GTK_MOVEMENT_DISPLAY_LINE_ENDS;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT(state))
	  {
	    step = GTK_MOVEMENT_DISPLAY_LINE_ENDS;
	    extend_selection = TRUE;
	  }
	else if (EVENT_KEY_WITH_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_BUFFER_ENDS;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_BUFFER_ENDS;
	    extend_selection = TRUE;
	  }
	else
	  return FALSE;
	break;
      }

    case GDK_space:
    case GDK_KP_Space:
    case GDK_BackSpace:
      {
	if (state != 0)
	  return FALSE;

	if (keyval == GDK_BackSpace)
	  count = -1;
	else
	  count = 1;

	step = GTK_MOVEMENT_PAGES;
	extend_selection = FALSE;
	break;
      }

    case GDK_Page_Up:
    case GDK_KP_Page_Up:
    case GDK_Page_Down:
    case GDK_KP_Page_Down:
      {
	if (keyval == GDK_Page_Up || keyval == GDK_KP_Page_Up)
	  count = -1;
	else
	  count = 1;

	if (state == 0)
	  {
	    step = GTK_MOVEMENT_PAGES;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT(state))
	  {
	    step = GTK_MOVEMENT_PAGES;
	    extend_selection = TRUE;
	  }
	else if (EVENT_KEY_WITH_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_HORIZONTAL_PAGES;
	    extend_selection = FALSE;
	  }
	else if (EVENT_KEY_WITH_SHIFT_CONTROL(state))
	  {
	    step = GTK_MOVEMENT_HORIZONTAL_PAGES;
	    extend_selection = TRUE;
	  }
	else
	  return FALSE;
	break;
      }

    case GDK_a:
      if (EVENT_KEY_WITH_CONTROL(state))
	{
	  g_signal_emit_by_name(view, "select-all", TRUE);
	  return TRUE;
	}
      if (EVENT_KEY_WITH_SHIFT_CONTROL(state))
	{
	  g_signal_emit_by_name(view, "select-all", FALSE);
	  return TRUE;
	}
      return FALSE;

    case GDK_slash:
      if (EVENT_KEY_WITH_CONTROL(state))
	{
	  g_signal_emit_by_name(view, "select-all", TRUE);
	  return TRUE;
	}
      return FALSE;
      
    case GDK_backslash:
      if (EVENT_KEY_WITH_CONTROL(state))
	{
	  g_signal_emit_by_name(view, "select-all", FALSE);
	  return TRUE;
	}
      return FALSE;

    case GDK_F7:
      g_signal_emit_by_name(view, "toggle-cursor-visible");
      return TRUE;

    default:
      return FALSE;
    }

  g_signal_emit_by_name(view, "move-cursor", step, count, extend_selection);

  return TRUE;
}


}
