/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: gtk_utils.h,v 1.2 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _GTK_UTILS_H_
#define _GTK_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{


  class TreePathHandle
  {
  private:
    GtkTreePath *path;


  public:
    inline TreePathHandle(GtkTreePath *path = NULL) : path(path) {}
    inline ~TreePathHandle() { if (path != NULL) gtk_tree_path_free(path); }

    inline GtkTreePath *get_path() const { return path; }
    inline operator GtkTreePath *() const { return path; }
    inline GtkTreePath *operator->() const { return path; }
    TreePathHandle &operator=(GtkTreePath *value);
    TreePathHandle &operator=(const TreePathHandle &tph);
  };


#define EVENT_KEY_WITH_SHIFT(state)		(state == GDK_SHIFT_MASK)
#define EVENT_KEY_WITH_CONTROL(state)		(state == GDK_CONTROL_MASK)
#define EVENT_KEY_WITH_SHIFT_CONTROL(state)	(state == (static_cast<guint>(GDK_SHIFT_MASK) | static_cast<guint>(GDK_CONTROL_MASK)))

  class KeyPressHandler
  {
  private:
    GtkWidget *widget;


    static void destroy(KeyPressHandler *handler);
    static gboolean key_press_event_cb(GtkWidget *widget,
				       GdkEventKey *event,
				       KeyPressHandler *handler);


  protected:
    virtual ~KeyPressHandler();

    inline GtkWidget *get_widget() { return widget; }
    virtual gboolean key_press_event(GdkEventKey *event);


  public:
    KeyPressHandler(GtkWidget *widget);
  };


  class Controller;


  /*
   * GTK+-2.14.xǤeditableGtkTextViewΥХǥ󥰤
   * ̵뤵ƤޤäݤΤǡкͤơ
   */
  class TextViewKeyPressHandler : public KeyPressHandler
  {
  protected:
    Controller &controller;


    virtual ~TextViewKeyPressHandler();
    virtual gboolean key_press_event(GdkEventKey *event);


  public:
    TextViewKeyPressHandler(GtkTextView *text_view, Controller &controller);
  };


}


#endif	/* __cplusplus */

#endif	/* _GTK_UTILS_H_ */
