/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: histories_ui.h,v 1.3 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _HISTORIES_UI_H_
#define _HISTORIES_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/history_manager.h"
#include "libochusha/tree.h"


using namespace std;
namespace Ochusha
{

  class HistoryEntry : public TreeElement
  {
  private:
    History *history;


  public:
    HistoryEntry(const char *name = NULL, const char *url = NULL,
		 time_t tc = 0, time_t last_mod = 0);
    HistoryEntry(History &history, time_t tc, time_t last_mod);
    virtual ~HistoryEntry();

    inline const char *get_url() const { return history->get_url(); }

    virtual bool is_folder() const;
    virtual const char *get_element_name() const;
    virtual void read_extended_tree_element(AttributeTable &attributes);
    virtual void write_extended_tree_element(WriteTree &args);
  };


  class HistoriesRoot : public RootFolder
  {
  protected:
    virtual Folder *folder_new(time_t tc, time_t last_mod, unsigned int sn);
    virtual Folder *folder_new(const Folder *folder, unsigned int sn);
    virtual TreeElement *leaf_element_new(AttributeTable &attributes,
					  time_t tc, time_t last_mod);


  public:
    HistoriesRoot();

    virtual TreeElement *copy(const TreeElement *element);

    bool read_histories(Repository &repository);
    bool write_histories(Repository &repository);

    virtual const char *get_root_element_name() const;
    virtual const char *get_leaf_element_name() const;
  };

}


#endif	/* __cplusplus */

#endif	/* _HISTORIES_UI_H_ */
