/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: html_document.h,v 1.3 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _HTML_DOCUMENT_H_
#define _HTML_DOCUMENT_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"


using namespace std;
namespace Ochusha
{


  class HTMLElement
  {
  protected:
    char *name;
    bool inline_element;


  public:
    HTMLElement(const char *name, bool inline_element);
    virtual ~HTMLElement();

    inline const char *get_name() const { return name; }
    inline bool is_inline_element() const { return inline_element; }
    inline bool is_block_element() const { return !inline_element; }

    virtual bool is_containable(const HTMLElement *content) const = 0;
    virtual bool is_empty_element() const = 0;
  };


  class HTMLElementManager : public hashtable<const char *, HTMLElement *>
  {
  private:
    void insert(HTMLElement *element);


  protected:
    struct Entry : public HashtableEntry
    {
      Entry(unsigned int hash, const char *key, HTMLElement *value,
	    HashtableEntry *next);
      virtual ~Entry();
    };

    virtual unsigned int calculate_hash(const char *key) const;
    virtual bool equals(const char *key1, const char *key2) const;
    virtual HashtableEntry *entry_new(unsigned int hash, const char *key,
				      HTMLElement *value, HashtableEntry *next);


  public:
    HTMLElementManager(int initial_capacity = 101);
  };


}


#endif /* __cplusplus */

#endif	/* _HTML_DOCUMENT_H_ */
