/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: html_renderer.h,v 1.22 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _HTML_RENDERER_H_
#define _HTML_RENDERER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "libochusha/html_utils.h"
#include "libochusha/sigslot.h"

#include "html_document.h"
#include "html_style.h"

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/hypertext_view.h"

#include <string>
#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class Controller;
  class Preferences;
  class Repository;

  // Ƕ̾ʥˤʤäƤʡġ
  class HTMLRendererTagManager : public has_slots
  {
  protected:
    Controller &controller;

    CStringToPointerHashtable tag_table;

    GtkTextTag *link_tag;

    GtkTextTag *my_bookmark_tag;


    HTMLStyleManager html_styles;


    vector<GtkTextTag *> title_color_tags;
    vector<GtkTextTag *> name_color_tags;
    vector<GtkTextTag *> link_color_tags;
    vector<GtkTextTag *> visited_color_tags;


    // sigslotʥϥɥ
    void prefs_changed_cb(Preferences &prefs);

    void set_colors(Preferences &prefs);


  public:
    HTMLRendererTagManager(Controller &controller);
    ~HTMLRendererTagManager();

    inline HTMLStyleManager &get_html_styles() { return html_styles; }
    GtkTextTag *create_tag(const char *name);

    inline GtkTextTag *get_link_tag() { return link_tag; }
    inline GtkTextTag *get_my_bookmark_tag() { return my_bookmark_tag; }

    inline GtkTextTag *lookup_tag(const char *name)
    { return static_cast<GtkTextTag *>(tag_table.lookup(name)); }
    void insert_tag(const char *name, GtkTextTag *tag);

    void write_prefs_css(Repository &repository);
    void read_prefs_css(Repository &repository);

    bool parse_color(const CSSValue &value, GdkColor *color);

    GtkTextTag *set_foreground(GtkTextTag *tag, const CSSValue &value);
    GtkTextTag *set_background(GtkTextTag *tag, const CSSValue &value);
    GtkTextTag *set_paragraph_background(GtkTextTag *tag,
					 const CSSValue &value);
  };


  class HTMLRenderer : public HTMLSAXParser
  {
  private:
    static gboolean link_mouse_over_cb(OchushaHypertextView *widget,
				       GdkEventMotion *event,
				       const gchar *link,
				       HTMLRenderer *renderer);
    static gboolean link_mouse_out_cb(OchushaHypertextView *widget,
				      GdkEventMotion *event,
				      HTMLRenderer *renderer);
    static gboolean link_mouse_press_cb(OchushaHypertextView *widget,
					GdkEventButton *event,
					const gchar *link,
					HTMLRenderer *renderer);
    static gboolean link_mouse_release_cb(OchushaHypertextView *widget,
					  GdkEventButton *event,
					  const gchar *link,
					  HTMLRenderer *renderer);


  protected:
    Controller &controller;
    HTMLRendererTagManager &tag_manager;
    HTMLStyleManager &html_styles;

    OchushaHypertextBuffer *buffer;
    OchushaHypertextView *view;

    CStringBuffer canon_buf;
    TextCanonicalizer canonicalizer;

    string title_text;

    GtkTextTag *link_tag;

    guint message_id;
    char *last_event_link;

    bool at_beginning_of_line;
    bool has_leading_space;

    string pcdata;

    vector<const HTMLStyle *> context_stack;


    static HTMLElementManager html_elements;


    virtual void flush_pcdata(bool with_newline);

    virtual void start_element(const char *name, const char *const attrs[]);
    virtual void end_element(const char *name);
    virtual void characters(const char *ch, size_t len);

    virtual bool is_visited_link(const char *link);

    virtual gboolean link_mouse_over(OchushaHypertextView *widget,
				     GdkEventMotion *event,
				     const gchar *link);
    virtual gboolean link_mouse_out(OchushaHypertextView *widget,
				    GdkEventMotion *event);
    virtual gboolean link_mouse_press(OchushaHypertextView *widget,
				      GdkEventButton *event,
				      const gchar *link);
    virtual gboolean link_mouse_release(OchushaHypertextView *widget,
					GdkEventButton *event,
					const gchar *link);


  public:
    HTMLRenderer(Controller &controller,
		 OchushaHypertextBuffer *buffer,
		 OchushaHypertextView *view);
    virtual ~HTMLRenderer();

    void render_html(const char *html);
  };


}


#endif	/* __cplusplus */

#endif	/* _HTML_RENDERER_H_ */
