/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: html_style.cc,v 1.13 2009/01/01 07:47:06 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "html_style.h"

#include "html_document.h"
#include "html_renderer.h"

#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>


#ifdef HAVE_STDINT_H
#include <stdint.h>
#else
# ifdef HAVE_INTTYPES_H
# include <inttypes.h>
# endif
#endif


#include <gtk/gtk.h>


// #define DEBUG_TEXT_TAG	1


namespace Ochusha
{


HTMLStyle::HTMLStyle(const HTMLElement *element, const StyleMatcher *matcher)
  : context(NULL),
    element(element),
    matcher(matcher),
    tag(NULL)
{
}


HTMLStyle::HTMLStyle(const HTMLStyle *context, const HTMLElement *element,
		     const StyleMatcher *matcher)
  : context(context),
    element(element),
    matcher(matcher),
    tag(NULL)
{
}


HTMLStyle::~HTMLStyle()
{
}


static double
get_resolution()
{
#if 0
// 
// 衢ɤscreenɽ뤫ռʤз׻Ǥʤ
  static double resolution = 0.0;
  static gint width_px = 0;
  static gint width_mm = 0;

  if (width_px != 0)
    return resolution;

  GdkScreen *screen = gdk_screen_get_default();
  width_px = gdk_screen_get_width(screen);
  width_mm = gdk_screen_get_width_mm(screen);

  if (width_px == 0 || width_mm == 0)
    {
      // ˥Ƹⲽ
      width_px = 1;
      width_mm = 1;
      resolution = 96.0;	// ¿ʬGdkΥǥե
      return resolution;
    }
  resolution = width_px * 25.4 / width_mm;

  return resolution;
#else
  // Ǥ⡢GTK+ǤϤäȥƤ뤫ġ
  return 96.0;
#endif
}


static int
get_pixels(const CSSValue &value, int font_px)
{
  if (!value.is_value_set() || !value.is_dimension())
    return 0;

  CSSKeyword unit = value.get_dimension_unit();
  if (unit == css_em)
    return static_cast<int>(font_px * value.get_dimension_value());

  if (unit == css_px)
    return static_cast<int>(value.get_dimension_value());

  if (unit == css_pt)
    return static_cast<int>(value.get_dimension_value() * get_resolution()
			    / 72.0);

  return 0;	// ¾̵
}


GtkTextTag *
HTMLStyle::create_gtk_text_tag(HTMLRendererTagManager &manager,
			       GtkWidget *widget)
{
  char name_buffer[64];
  snprintf(name_buffer, 64, "GtkTextTag4HTMLStyle@%p", this);
  tag = manager.create_tag(name_buffer);

#ifdef DEBUG_TEXT_TAG
  fprintf(stderr, "HTMLStyle::create_gtk_text_tag(): %s {\n", name_buffer);
#endif

  const CSSStyle &style = matcher->get_style();

  PangoContext *ctx = gtk_widget_get_pango_context(widget);
  PangoFontDescription *desc = pango_context_get_font_description(ctx);
  gint pango_font_size = pango_font_description_get_size(desc);
  double font_pt;
  if (pango_font_size > 0)
    font_pt = static_cast<double>(pango_font_size) / PANGO_SCALE;
  else
    font_pt = 12.0;	// 

  int font_px = static_cast<int>(font_pt * get_resolution() / 72.0);

  if (style.get_text_align().is_value_set())
    {
      const CSSValue &val = style.get_text_align();
      if (val.is_keyword())
	{
	  gboolean is_set = FALSE;
	  CSSKeyword keyword = val.get_keyword();
	  GtkJustification justification = GTK_JUSTIFY_LEFT;
	  if (keyword == css_left)
	    {
	      justification = GTK_JUSTIFY_LEFT;
	      is_set = TRUE;
	    }
	  else if (keyword == css_right)
	    {
	      justification = GTK_JUSTIFY_RIGHT;
	      is_set = TRUE;
	    }
	  else if (keyword == css_center)
	    {
	      justification = GTK_JUSTIFY_CENTER;
	      is_set = TRUE;
	    }
	  else if (keyword == css_justify)
	    {
	      justification = GTK_JUSTIFY_FILL;
	      is_set = TRUE;
	    }

	  g_object_set(tag,
		       "justification", justification,
		       "justification-set", is_set,
		       NULL);
#ifdef DEBUG_TEXT_TAG
	  fprintf(stderr, "  justification: %d;\n", justification);
#endif
	}
    }

  if (style.get_background_color().is_value_set())
    {
      const CSSValue &val = style.get_background_color();
      bool is_bg_set = false;
      if (!val.is_keyword()
	  || (val.get_keyword() != css_transparent
	      && val.get_keyword() != css_inherit))
	{
	  if (element->is_block_element())
	    {
	      manager.set_paragraph_background(tag, val);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  paragraph-background: XXX;\n");
#endif
	      is_bg_set = true;
	    }
	  else
	    {
	      manager.set_background(tag, val);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  background: XXX;\n");
#endif
	    }
	}

      if (!is_bg_set)
	{
	  const HTMLStyle *ancestor = context;
	  while (ancestor != NULL)
	    {
	      if (ancestor->get_element()->is_inline_element())
		{
		  ancestor = ancestor->context;
		  continue;
		}

	      const CSSStyle &anc_style = ancestor->matcher->get_style();
	      const CSSValue &bg = anc_style.get_background_color();
	      if (bg.is_value_set()
		  && (!bg.is_keyword()
		      || (bg.get_keyword() != css_transparent
			  && bg.get_keyword() != css_inherit)))
		{
		  manager.set_paragraph_background(tag, bg);
#ifdef DEBUG_TEXT_TAG
		  fprintf(stderr, "  paragraph-background: XXX;\n");
#endif
		  break;
		}
	    }
	}
    }

  if (style.get_text_indent().is_value_set())
    {
      const CSSValue &val = style.get_text_indent();
      if (val.is_dimension() && val.get_dimension_value() > 0.0)
	{
	  int px = get_pixels(style.get_text_indent(), font_px);
	  if (px > 0)
	    {
	      g_object_set(tag, "indent", px, "indent-set", TRUE, NULL);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  indent: %d;\n", px);
#endif
	    }
	}
    }

  if (style.get_margin_left().is_value_set())
    {
      const CSSValue &val = style.get_margin_left();
      if (val.is_dimension() && val.get_dimension_value() > 0.0)
	{
	  int px = get_pixels(val, font_px);
	  if (px > 0)
	    {
	      g_object_set(tag,
			   "left-margin", px,
			   "left-margin-set", TRUE,
			   NULL);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  left-margin: %d;\n", px);
#endif
	    }
	}
    }

  if (style.get_margin_right().is_value_set())
    {
      const CSSValue &val = style.get_margin_right();
      if (val.is_dimension() && val.get_dimension_value() > 0.0)
	{
	  int px = get_pixels(val, font_px);
	  if (px > 0)
	    {
	      g_object_set(tag,
			   "right-margin", px,
			   "right-margin-set", TRUE,
			   NULL);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  right-margin: %d;\n", px);
#endif
	    }
	}
    }

  if (style.get_vertical_align().is_value_set())
    {
      const CSSValue &val = style.get_vertical_align();
      if (val.is_keyword())
	{
	  CSSKeyword keyword = val.get_keyword();
	  if (keyword == css_super || keyword == css_sub)
	    {
	      gint rise_value = static_cast<gint>(pango_font_size * 0.3);
	      if (keyword == css_sub)
		rise_value = -rise_value;
	      if (rise_value != 0)
		{
		  g_object_set(tag,
			       "rise", rise_value,
			       "rise-set", TRUE,
			       NULL);
#ifdef DEBUG_TEXT_TAG
		  fprintf(stderr, "  rise: %d;\n", rise_value);
#endif
		}
	    }
	}
    }

  if (style.get_font_size().is_value_set())
    {
      const CSSValue &val = style.get_font_size();
      if (val.is_dimension())
	{
	  CSSKeyword unit = val.get_dimension_unit();
	  double dim_val = val.get_dimension_value();
	  if (unit == css_em)
	    {
	      if (dim_val != 1.0)
		g_object_set(tag, "scale", dim_val, "scale-set", TRUE, NULL);
	    }
	  else if (unit == css_pt || unit == css_px)
	    {
	      if (unit == css_px)
		dim_val = dim_val * 72.0 / get_resolution();
	      g_object_set(tag, "size-points", dim_val, "size-set", TRUE, NULL);
	    }
	}
    }

  vector<CSSValue *>::const_iterator f_iter = style.get_font_family_begin();
  vector<CSSValue *>::const_iterator f_end_iter = style.get_font_family_end();
  string f_buf;
  while (f_iter != f_end_iter)
    {
      const CSSValue *f_val = *f_iter++;
      if (f_val->is_font_string())
	{
	  if (!f_buf.empty())
	    f_buf.push_back(',');
	  f_buf.append(f_val->get_font_string());
	}
      else if (f_val->is_keyword())
	{
	  CSSKeyword keyword = f_val->get_keyword();
	  const char *fn = NULL;
	  if (keyword == css_serif)
	    fn = "Serif";
	  else if (keyword == css_sans_serif)
	    fn = "Sans";
	  else if (keyword == css_monospace)
	    fn = "Monospace";
	  // ¾Υɤ̵
	  if (fn != NULL)
	    {
	      if (!f_buf.empty())
		f_buf.push_back(',');
	      f_buf.append(fn);
	    }
	}
    }
  if (!f_buf.empty())
    {
      g_object_set(tag,
		   "family", f_buf.c_str(),
		   "family-set", TRUE,
		   NULL);
#ifdef DEBUG_TEXT_TAG
      fprintf(stderr, "  family: %s;\n", f_buf.c_str());
#endif
    }

  if (style.get_font_style().is_value_set())
    {
      const CSSValue &val = style.get_font_style();
      if (val.is_keyword())
	{
	  CSSKeyword keyword = val.get_keyword();
	  PangoStyle font_style = PANGO_STYLE_NORMAL;
	  gboolean is_set = FALSE;
	  if (keyword == css_normal)
	    {
	      font_style = PANGO_STYLE_NORMAL;
	      is_set = TRUE;
	    }
	  else if (keyword == css_italic)
	    {
	      font_style = PANGO_STYLE_ITALIC;
	      is_set = TRUE;
	    }
	  else if (keyword == css_oblique)
	    {
	      font_style = PANGO_STYLE_OBLIQUE;
	      is_set = TRUE;
	    }
	  g_object_set(tag, "style", font_style, "style-set", is_set, NULL);
#ifdef DEBUG_TEXT_TAG
	  fprintf(stderr, "  style: %d;\n", font_style);
#endif
	}
    }

  if (style.get_font_variant().is_value_set())
    {
      const CSSValue &val = style.get_font_variant();
      if (val.is_keyword())
	{
	  CSSKeyword keyword = val.get_keyword();
	  PangoVariant variant = PANGO_VARIANT_NORMAL;
	  gboolean is_set = FALSE;
	  if (keyword == css_normal)
	    {
	      variant = PANGO_VARIANT_NORMAL;
	      is_set = TRUE;
	    }
	  else if (keyword == css_small_caps)
	    {
	      variant = PANGO_VARIANT_SMALL_CAPS;
	      is_set = TRUE;
	    }
	  g_object_set(tag, "variant", variant, "variant-set", is_set, NULL);
#ifdef DEBUG_TEXT_TAG
	  fprintf(stderr, "  variant: %d;\n", variant);
#endif
	}
    }

  if (style.get_font_weight().is_value_set())
    {
      const CSSValue &val = style.get_font_weight();
      if (val.is_integer())
	{
	  g_object_set(tag,
		       "weight", val.get_integer(),
		       "weight-set", TRUE,
		       NULL);
#ifdef DEBUG_TEXT_TAG
	  fprintf(stderr, "  weight: %d;\n", val.get_integer());
#endif
	}
    }

  if (style.get_line_height().is_value_set())
    {
      const CSSValue &val = style.get_line_height();
      if (val.is_dimension())
	{
	  int lh = get_pixels(val, font_px);
	  if (lh > font_px)
	    {
	      g_object_set(tag,
			   "pixels-above-lines", lh - font_px,
			   "pixels-above-lines-set", TRUE,
			   NULL);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  pixels-above-lines: %d;\n", lh - font_px);
#endif
	    }
	}
    }

  if (style.get_color().is_value_set())
    {
      manager.set_foreground(tag, style.get_color());
#ifdef DEBUG_TEXT_TAG
      fprintf(stderr, "  foreground: XXX;\n");
#endif
    }

  if (style.get_text_decoration().is_value_set())
    {
      const CSSValue &val = style.get_text_decoration();
      if (val.is_text_decoration())
	{
	  if (val.get_underline())
	    {
	      g_object_set(tag,
			   "underline", PANGO_UNDERLINE_SINGLE,
			   "underline-set", TRUE,
			   NULL);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  underline: TRUE;\n");
#endif
	    }
	  if (val.get_line_through())
	    {
	      g_object_set(tag,
			   "strikethrough", TRUE,
			   "strikethrough-set", TRUE,
			   NULL);
#ifdef DEBUG_TEXT_TAG
	      fprintf(stderr, "  strikethrough: TRUE;\n");
#endif
	    }
	}
    }

#ifdef DEBUG_TEXT_TAG
  fprintf(stderr, "}\n");
#endif


  return tag;
}


HTMLStyleManager::HTMLStyleManager()
{
  initialize_styles();
}


HTMLStyleManager::~HTMLStyleManager()
{
}


#include "html40_css.inc"


void
HTMLStyleManager::initialize_styles()
{
  define_styles(default_style_sheet_html40, sizeof(default_style_sheet_html40));
}


void
HTMLStyleManager::insert(HTMLStyle *style)
{
  hashtable<const StyleMatcher *, HTMLStyle *>::insert(style->get_matcher(),
						       style);
}


HTMLStyleManager::Entry::Entry(unsigned int hash, const StyleMatcher *key,
			       HTMLStyle *value,
			       HTMLStyleManager::HashtableEntry *next)
  : HTMLStyleManager::HashtableEntry(hash, key, value, next)
{
}


HTMLStyleManager::Entry::~Entry()
{
  delete value;
}


unsigned int
HTMLStyleManager::calculate_hash(const StyleMatcher *key) const
{
  return static_cast<unsigned int>(reinterpret_cast<intptr_t>(const_cast<StyleMatcher *>(key)));
}


bool
HTMLStyleManager::equals(const StyleMatcher *key1, const StyleMatcher *key2) const
{
  return key1 == key2;
}


HTMLStyleManager::HashtableEntry *
HTMLStyleManager::entry_new(unsigned int hash, const StyleMatcher *key,
			    HTMLStyle *value,
			    HTMLStyleManager::HashtableEntry *next)
{
  return new Entry(hash, key, value, next);
}


void
HTMLStyleManager::reset_styles()
{
  clear();
  style_sheet.reset_rulesets();
  initialize_styles();
}


bool
HTMLStyleManager::define_styles(const char *style_sheet_src, int length)
{
  style_sheet.parse_style_sheet(style_sheet_src, length);
  return true;
}


const HTMLStyle *
HTMLStyleManager::lookup_style(const HTMLStyle *context,
			       const char *element_name,
			       const HTMLElement *element,
			       const char *const attrs[],
			       CSSPseudoClass pseudo_class,
			       CSSPseudoElement pseudo_element)
{
  const StyleMatcher *matcher
    = style_sheet.lookup_style(context != NULL ? context->get_matcher() : NULL,
			       element_name, attrs,
			       pseudo_class, pseudo_element);
  g_return_val_if_fail(matcher != NULL, NULL);

  HTMLStyle *style = lookup(matcher);
  if (style == NULL)
    {
      if (context != NULL)
	style = new HTMLStyle(context, element, matcher);
      else
	style = new HTMLStyle(element, matcher);
      insert(style);
    }

  return style;
}


}
