/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: html_style.h,v 1.11 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _HTML_STYLE_H_
#define _HTML_STYLE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"

#include "style_sheet.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class HTMLElement;
  class HTMLRendererTagManager;


  // HTMLStyleHTMLʸǤб륹ξȤ
  // ߥ졼ȤGtkTextTagޤȤ᤿Ρ
  class HTMLStyle
  {
  private:
    const HTMLStyle *context;
    const HTMLElement *element;
    const StyleMatcher *matcher;

    GtkTextTag *tag;


    GtkTextTag *create_gtk_text_tag(HTMLRendererTagManager &manager,
				    GtkWidget *widget);


  public:
    HTMLStyle(const HTMLElement *element, const StyleMatcher *matcher);
    HTMLStyle(const HTMLStyle *context, const HTMLElement *element,
	      const StyleMatcher *matcher);
    ~HTMLStyle();

    inline GtkTextTag *get_gtk_text_tag(HTMLRendererTagManager &manager,
					GtkWidget *widget)
    { return tag != NULL ? tag : create_gtk_text_tag(manager, widget); }


    inline const HTMLElement *get_element() const { return element; }
    inline const HTMLStyle *get_context() const { return context; }
    inline const StyleMatcher *get_matcher() const { return matcher; }
  };


  class HTMLStyleManager : public hashtable<const StyleMatcher *, HTMLStyle *>
  {
  private:
    StyleSheet style_sheet;


    void initialize_styles();
    void insert(HTMLStyle *style);


  protected:
    struct Entry : public HashtableEntry
    {
      Entry(unsigned int hash, const StyleMatcher *key, HTMLStyle *value,
	    HashtableEntry *next);
      virtual ~Entry();
    };

    virtual unsigned int calculate_hash(const StyleMatcher *key) const;
    virtual bool equals(const StyleMatcher *key1, const StyleMatcher *key2) const;
    virtual HashtableEntry *entry_new(unsigned int hash,
				      const StyleMatcher *key,
				      HTMLStyle *value,
				      HashtableEntry *next);


  public:
    HTMLStyleManager();
    ~HTMLStyleManager();
    void reset_styles();
    bool define_styles(const char *style_sheet_src, int length);
    inline StyleSheet &get_style_sheet() { return style_sheet; }

    const HTMLStyle *lookup_style(const HTMLStyle *context,
				  const char *element_name,
				  const HTMLElement *element,
				  const char *const attrs[],
				  CSSPseudoClass pseudo_class,
				  CSSPseudoElement pseudo_element);
  };


}


#endif	/* __cplusplus */

#endif	/* _HTML_STYLE_H_ */
