/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: image_access_db.h,v 1.2 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _IMAGE_ACCESS_DB_H_
#define _IMAGE_ACCESS_DB_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/lock.h"

#ifdef HAVE_TIME_H
#include <time.h>
#endif


using namespace std;
namespace Ochusha
{

  class Repository;
  class SQLite3;
  class SQLite3Statement;

  class AccessResult
  {
    friend class ImageAccessDB;

  public:
    enum ResultType {
      UNKNOWN = -1,
      GOOD,
      BROKEN_IMAGE,
      ACCESS_ERROR
    };


  private:
    char *url;
    ResultType result;
    int status_code;
    time_t last_accessed;

    void set_url(const char *value);
    inline void set_result(ResultType value) { result = value; }
    inline void set_status_code(int value) { status_code = value; }
    inline void set_last_accessed(time_t value) { last_accessed = value; }


  public:
    AccessResult(const char *url, ResultType result, int status_code,
		 time_t access_time);
    ~AccessResult();

    inline const char *get_url() const { return url; }
    inline ResultType get_result() const { return result; }
    inline int get_status_code() const { return status_code; }
    inline time_t get_last_accessed() const { return last_accessed; }
  };


  class ImageAccessDB
  {
  private:
    Lock lock;

    SQLite3 *db_handle;

    SQLite3Statement *select_all_urls;
    SQLite3Statement *select_matched_url;
    SQLite3Statement *insert_new_url;
    SQLite3Statement *update_matched_id;
    SQLite3Statement *delete_matched_id;


  public:
    ImageAccessDB(Repository &repository);
    ~ImageAccessDB();

    void register_access_result(const char *url,
				AccessResult::ResultType result,
				int status_code, time_t access_time);

    // ̤ϰȤͿ줿result񤭴֤
    // DBϿƤˤresultؤΥݥ󥿤
    // DBϿƤʤäˤNULL֤
    AccessResult *query_access_result(const char *url,
				      AccessResult &result);
  };


}


#endif	/* __cplusplus */

#endif	/* _IMAGE_ACCESS_DB_H_ */
