/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: image_loader.h,v 1.6 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _IMAGE_LOADER_H_
#define _IMAGE_LOADER_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/ref_count.h"

#include <list>
#include <vector>

#ifdef HAVE_TIME_H
#include <time.h>
#endif

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  class ImageLoader;
  class ImageLoaderContext;
  class ImageManager;

  class ImageLoaderInfo : public RefCount
  {
    friend class ImageLoader;
    friend class ImageLoaderContext;
    friend class ImageManager;

  private:
    ImageManager &manager;

    char *url;
    char *referer_url;
    char *redirected_url;

    int status_code;
    time_t last_accessed;

    char *description;


    // ImageManagerΥ塼θΨ
    enum {
      FREE_ELEMENT = 0,
      IN_LOADER_QUEUE,
      IN_BUSY_QUEUE
    } state;

    inline void set_free_element() { state = FREE_ELEMENT; }
    inline bool is_free_element() const { return state == FREE_ELEMENT; }
    inline void set_in_loader_queue() { state = IN_LOADER_QUEUE; }
    inline bool is_in_loader_queue() const { return state == IN_LOADER_QUEUE; }
    inline void set_in_busy_queue() { state = IN_BUSY_QUEUE; }
    inline bool is_in_busy_queue() const { return state == IN_BUSY_QUEUE; }
    // ImageManagerΥ塼θΨ


    vector<GdkPixbufLoader *> active_loaders;

    // ʲΥꥹȤGDK_THREADS_ENTER()ʴĶ롣
    // ɤ߹楹åɤоݤȤʤImageLoaderΥꥹ
    typedef list<ImageLoader *> ImageLoaderList;
    ImageLoaderList active_loader_list;

    // ɤ߹ߤγϤ˴֤˹ʤäImageLoaderΥꥹȡĤ
    // ɤ߹ߤδλˤޤȤƽ
    ImageLoaderList pending_loader_list;

    // ɤ߹ߤδλImageLoaderΥꥹȡ
    ImageLoaderList done_loader_list;


    // ImageLoaderθƤӽФ
    void enqueue(ImageLoader *loader, bool interactive);
    void dequeue(ImageLoader *loader);
    bool is_alive();


    // ImageLoaderؤΥ˻Ȥ
    bool prepare_loading();
    bool append_data(const char *data, size_t length);
    bool close_loaders(bool error_exit);


    // ImageLoaderContextθƤӽФ
    void notify_access_started();
    void notify_access_finished(int status_code);
    bool check_redirection(const char *source);


  public:
    ImageLoaderInfo(ImageManager &manager, const char *url);
    ~ImageLoaderInfo();

    const char *to_string();	// ǥХå

    inline const char *get_url() const { return url; }

    void set_referer_url(const char *value);
    inline const char *get_referer_url() const
    { return redirected_url != NULL ? url : referer_url; }

    void set_redirected_url(const char *value);
    inline const char *get_redirected_url() const { return redirected_url; }

    inline int get_status_code() const { return status_code; }
    inline void set_status_code(int value) { status_code = value; }

    inline time_t get_last_accessed() const { return last_accessed; }
    inline void set_last_accessed(time_t value) { last_accessed = value; }

    const char *get_effective_url() const;
    const char *get_effective_referer() const;

    bool hungry();

    void notify_a_boned();

    GtkWidget *get_widget(int width, int height, bool interactive);
    bool download_image();


    // : ĤGDK_THREADS_ENTER()ǤʤĶƤ֤
    bool load_cache(int fd);
  };


  typedef smart_ptr<ImageLoaderInfo> ImageLoaderInfoHandle;

}


#endif	/* __cplusplus */

#endif	/* _IMAGE_LOADER_H_ */
