/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: image_manager.h,v 1.26 2009/01/01 07:47:06 fuyu Exp $
 */

#ifndef _IMAGE_MANAGER_H_
#define _IMAGE_MANAGER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/cache_manager.h"
#include "libochusha/hashtable.h"
#include "libochusha/lock.h"
#include "libochusha/network_agent.h"
#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"
#include "libochusha/thread_pool.h"

#include "a_bone_image_db.h"
#include "controller.h"
#include "image_access_db.h"
#include "image_loader.h"

#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  class ImageManager : public CacheManager, public has_slots
  {
    friend class ImageLoaderContext;
    friend class ImageLoaderInfo;
    friend class ImageLoaderJob;

  private:
    Controller &controller;

    ABoneImageDB a_bone_image_db;

    GSList *supported_format_list;
    vector<char *> supported_format_mime_types;

    char *user_agent;
    Monitor queue_monitor;		// åGDK => queue_monitor

    ImageAccessDB image_access_db;
    RefCountHashtable loader_table;	// URL => ImageLoaderInfo *ʥơ֥

    typedef list<ImageLoaderInfoHandle> ImageLoaderInfoList;
    ImageLoaderInfoList loader_queue;
    ImageLoaderInfoList busy_queue;

    unsigned int number_of_downloaders_active;
    unsigned int number_of_workers_available;
    bool terminate_workers;

    bool process(const WorkerThread *employee);
    void activate_downloaders();

    inline bool is_alive() const { return !terminate_workers; }

    void enqueue(ImageLoaderInfo *info, bool interactive);
    void dequeue(ImageLoaderInfo *info);

    void notify_access_finished(ImageLoaderInfo *info);

    void prefs_changed_cb(Preferences &prefs);


  public:
    ImageManager(Controller &controller);
    virtual ~ImageManager();

    // urlǻꤵ줿width  height˼ޤ礭˽̤᤿
    // ɽ뤿GtkWidget(GtkImage)ؤΥݥ󥿤֤
    //
    // ܡ󤵤ƤˤNULL֤
    //
    // online⡼ɤǥåˤʤϡ̥åɤǥͥåȥ
    // 򳫻ϤǽŪˡꤵ줿⤷
    // GTK_STOCK_MISSING_IMAGEɽ褦GtkWidget(GtkImage)ؤ
    // ݥ󥿤֤
    //
    // offline⡼ɤǥåˤʤϡGTK_STOCK_MISSING_IMAGE
    // ɽƤGtkWidget(GtkImage)ؤΥݥ󥿤֤
    //
    // ˤ衢ƤӽФϤĤ֤GtkWidgetŪβǤ
    // ȻפäƤɤ
    //
    // interactivetrueξ硢׵ᤵ줿ͥŪɤ߹ࡣ
    //
    GtkWidget *get_widget(const char *url, int width, int height,
			  bool interactive = false);

    void prepare_shutdown();

    void a_bone(const char *url);
    void a_bone_server(const char *url);

    bool is_a_boned(const char *url);
    bool is_image_link(const char *url);

    char *check_redirection(const char *url, const char *source);

    void set_user_agent(const char *value);
    inline const char *get_user_agent() { return user_agent; }
    bool is_content_type_supported(const char *content_type);

    virtual void invalidate(const char *url);


    // ͥåȥβǡɤ߹߻ȯ롣
    // void output_log(LogCategory category, const char *message);
    signal2<LogCategory, const char *> output_log;
  };


}


#endif	/* __cplusplus */

#endif	/* _IMAGE_MANAGER_H_ */
