/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: main_window.h,v 1.26 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _MAIN_WINDOW_H_
#define _MAIN_WINDOW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/network_agent.h"
#include "libochusha/sigslot.h"

#include "bbs_thread_ui.h"
#include "board_ui.h"
#include "boardlist_ui.h"
#include "bookmarks_ui.h"
#include "threadlist_ui.h"
#include "view_manager.h"

#include <list>
#include <string>
#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  struct AccessMessage
  {
    string url;
    guint message_id;

    AccessMessage(const char *url, guint message_id)
      : url(url), message_id(message_id) {}
  };


  struct PendingStatus
  {
    string url;
    NetworkAccessStatus status;

    PendingStatus(const char *url, NetworkAccessStatus status)
      : url(url), status(status) {}
  };


  class Controller;
  class View;

  class MainWindow : public has_slots
  {
    friend class CommandPipe;
    friend class Controller;
    friend class View;
    friend class ViewManager;

  private:
    Controller &controller;

    GtkActionGroup *menubar_action_group;
    GtkActionGroup *current_view_action_group;

    GtkAction *bookmark_this_page;
    GtkAction *close_this_page;
    GtkAction *find_in_this_page;
    GtkAction *go_back;
    GtkAction *go_forward;
    GtkAction *page_info;
    GtkAction *refresh_this_page;

    GtkWindow *toplevel;
    GtkBox *main_box;			// GtkVBox

    GtkWidget *menubar;
    GtkWidget *navigation_toolbar;

    GtkEntry *url_entry;		// NavigationToolbar椫õ
					// URLGtkEntry

    // ɥС()ڥȥڥʥƥ
    GtkWidget *main_container;		// GtkHPaned


    // ɥС
    ModNotebook *sidebar_book;

    View *boardlist_view;
    View *bookmarks_view;
    View *history_view;			// ͽ
    View *post_history_view;		// ͽ

#define BOARDLIST_SIDEBAR	(1 << 0)
#define BOOKMARKS_SIDEBAR	(1 << 1)
#define HISTORY_SIDEBAR		(1 << 2)
#define POST_HISTORY_SIDEBAR	(1 << 3)
#define SHOWN_SIDEBAR_MASK	(BOARDLIST_SIDEBAR | BOOKMARKS_SIDEBAR \
				 | HISTORY_SIDEBAR | POST_HISTORY_SIDEBAR)
    unsigned int shown_sidebar_items;


    // ()ڥȥڥƤĤˤޤ
    ViewManager view_manager;


    // СϢ
    GtkWidget *searchbar;		// OchushaSearchbar

    View *search_target;
    guint search_message_id;
    gulong search_response_id;
    gulong search_request_id;

    void setup_search(View *target);
    static void search_response_cb(GtkWidget *widget, int response_id,
				   MainWindow *window);


    // ơС
    GtkBox *status_box;			// GtkHBox
    GtkStatusbar *statusbar;
    GtkWidget *connection_image;	// GtkImage
    GtkWidget *edit_image;		// GtkImage


    // ܡɥեϢ
    View *current_focus_view;
    View *current_main_view;


    // statusbarcontext ID
    guint searchbar_context_id;		// Сcontext
    guint boardlist_context_id;		// İcontext
    guint link_context_id;		// ɽcontext
    guint network_context_id;		// ͥåȥcontext

    // ͥåȥ֤ɽ
    list<AccessMessage> access_message_list;
    list<PendingStatus> pending_status;
    guint process_pending_status_on_idle_id;


    GdkPixbuf *stable_icon;
    GdkPixbufAnimation *progress_icon;
    GdkPixbufAnimationIter *progress_icon_iter;
    guint progress_icon_frame_timeout_id;

    void progress_animation_disable();
    void progress_animation_enable();
    static gboolean menubar_expose_event_cb(GtkWidget *widget,
					    GdkEventExpose *event,
					    MainWindow *window);
    static gboolean progress_animation_timeout_cb(MainWindow *window);

    static void area_prepared_cb(GdkPixbufLoader *loader, MainWindow *window);
    static void area_updated_cb(GdkPixbufLoader *loader, gint x, gint y,
				gint width, gint height, MainWindow *window);
    static void closed_cb(GdkPixbufLoader *loader, MainWindow *window);


    // 
    vector<char *> opened_tab_urls;
    char *active_board_url;
    char *active_thread_url;


    // ɥ粽Ƥ뤫ɤϿ롣
    bool maximized;


    GtkActionGroup *create_menubar_action_group();
    GtkActionGroup *create_current_view_action_group();
    void setup_main_ui();

    static void bookmark_this_page_cb(GtkAction *action, MainWindow *window);
    static void close_this_page_cb(GtkAction *action, MainWindow *window);
    static void find_in_this_page_cb(GtkAction *action, MainWindow *window);
    static void go_back_cb(GtkAction *action, MainWindow *window);
    static void go_forward_cb(GtkAction *action, MainWindow *window);
    static void page_info_cb(GtkAction *action, MainWindow *window);
    static void refresh_this_page_cb(GtkAction *action, MainWindow *window);

    static void notify_position_cb(GtkPaned *paned, GParamSpec *arg1,
				   MainWindow *window);
    static void notify_position_set_cb(GtkPaned *paned, GParamSpec *arg1,
				       MainWindow *window);
    static void match_case_toggled_cb(GtkToggleButton *button,
				      MainWindow *window);
    static void use_regexp_toggled_cb(GtkToggleButton *button,
				      MainWindow *window);
    static void delete_event_cb(GtkWidget *widget, GdkEventAny *event,
				MainWindow *window);
    static gboolean window_state_event_cb(GtkWidget *widget,
					  GdkEventWindowState *event,
					  MainWindow *window);

    static void show_navigation_toolbar_toggled_cb(GtkToggleAction *action,
						   MainWindow *window);

    static gboolean connection_image_pressed_cb(GtkWidget *event_box,
						GdkEventButton *event,
						Controller *controller);
    static void work_offline_toggled_cb(GtkToggleAction *action,
					MainWindow *window);

    static gboolean edit_image_pressed_cb(GtkWidget *event_box,
					  GdkEventButton *event,
					  GtkToggleAction *action);
    static void embeded_response_editor_toggled_cb(GtkToggleAction *action,
						   MainWindow *window);


    // ɥСϢ
    static gboolean sidebar_button_press_event_cb(ModNotebook *notebook,
						  GdkEventButton *event,
						  MainWindow *window);
    static void show_boardlist_sidebar_toggled_cb(GtkToggleAction *action,
						  MainWindow *window);
    static void show_bookmarks_sidebar_toggled_cb(GtkToggleAction *action,
						  MainWindow *window);
    static void show_history_sidebar_toggled_cb(GtkToggleAction *action,
						MainWindow *window);
    static void show_post_history_sidebar_toggled_cb(GtkToggleAction *action,
						     MainWindow *window);

    static void url_entry_activate_cb(GtkEntry *entry, MainWindow *window);

    // ܡɥեϢ
    void notify_current_main_view(View *view);
    inline View *get_current_main_view() { return current_main_view; }
    void notify_current_focus_view(View *view);
    inline View *get_current_focus_view() { return current_focus_view; }
    void notify_view_vanished(View *view);

    // current_focus_viewθ˹碌ViewϢGtkActionξ֤򹹿롣
    void update_actions();

    // ߤViewϢGtkActionξ֤ɬפ˱ƹ롣
    void notify_state_changed(View *view);


    void update_sidebar_view();


    // ͥåȥsigslotʥϥɥ
    void notify_access_status_cb(const char *url, NetworkAccessStatus status);

    void process_pending_status();
    void notify_access_status(const char *url, NetworkAccessStatus status);

    static gboolean process_pending_status_on_idle_cb(MainWindow *window);

    void clear_tab_related_session_data();
    void prepare_tab_related_session();
    void restore_tab_related_session();


  public:
    MainWindow(Controller &controller);
    ~MainWindow();

    bool url_entry_grab_focus();

    void show();
    void save_states();
    void prepare_session();
    void restore_session();

    void prepare_shutdown();
  };


}


#endif	/* __cplusplus */

#endif	/* _MAIN_WINDOW_H_ */
