/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: post_ui.h,v 1.24 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _POST_UI_H_
#define _POST_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/bulletin_board.h"
#include "libochusha/sigslot.h"

#include "libochushagtk/response_editor.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class Controller;

  class ResponseEditor : public has_slots
  {
  protected:
    Controller &controller;

    GtkAction *post_action;
    GtkAction *clear_action;

    GtkWidget *sage_button;
    GtkWidget *be_button;
    GtkWidget *viewer_button;

    OchushaResponseEditor *editor;

    BulletinBoardHandle board;
    BBSThreadHandle thread;
    smart_ptr<PostResponseAgent> agent;

    GtkWidget *result_window;
    gulong result_window_destroy_id;
    guint ensure_sage_on_idle_id;


  private:
    void remember_current_entries();
    void update_current_entries();

    void update_actions(GtkTextBuffer *buffer);

    static void title_changed_cb(GtkEntry *widget, ResponseEditor *editor);
    static void name_changed_cb(GtkComboBox *widget, ResponseEditor *editor);

    static gboolean ensure_sage_on_idle_cb(ResponseEditor *editor);
    static void mail_changed_cb(GtkComboBox *widget, ResponseEditor *editor);
    static void message_changed_cb(GtkTextBuffer *buffer,
				   ResponseEditor *editor);
    static void post_activate_cb(GObject *object, ResponseEditor *editor);
    static void clear_activate_cb(GObject *object, ResponseEditor *editor);
    static void sage_button_toggled_cb(GtkToggleButton *button,
				       ResponseEditor *editor);
    static void be_button_toggled_cb(GtkToggleButton *button,
				     ResponseEditor *editor);
    static void viewer_button_toggled_cb(GtkToggleButton *button,
					 ResponseEditor *editor);
    static void result_window_destroy_cb(GtkWidget *widget,
					 ResponseEditor *editor);


    // 񤭹߽̤˱ȯԤsigslotʥΥϥɥ
    void post_finished_cb(PostResponseAgent *agent);
    void post_failed_cb(PostResponseAgent *agent, const char *reason);
    void check_cookie_cb(PostResponseAgent *agent, const char *message);


  protected:
    void update_preferences();
    virtual void notify_post_finished();


  public:
    ResponseEditor(Controller &controller, BulletinBoard *board);
    ResponseEditor(Controller &controller, BBSThread *thread = NULL);
    virtual ~ResponseEditor();

    virtual void set_thread(BBSThread *thread);

    void post();
    void clear();
    void reload_aa();

    void append_message(const gchar *message);
    void insert_message(const gchar *message);
    void set_message(const gchar *message);
    void grab_focus_into_editor();

    virtual GtkWidget *get_widget();
    virtual bool is_embeded();
  };


  class EmbededResponseEditor : public ResponseEditor
  {
  private:
    static void close_cb(OchushaResponseEditor *widget,
			 EmbededResponseEditor *editor);


  protected:
    GtkWidget *frame;
    GtkWidget *hbox;
    GtkWidget *button_box;
    GtkWidget *post_button;
    GtkWidget *delete_button;


  public:
    EmbededResponseEditor(Controller &controller);
    virtual ~EmbededResponseEditor();

    virtual void set_thread(BBSThread *thread);

    virtual GtkWidget *get_widget();
    virtual bool is_embeded();
  };


  class ResponseEditorWindow : public ResponseEditor
  {
  protected:
    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *title_hbox;
    GtkWidget *title_label;
    GtkWidget *board_label;
    GtkWidget *url_hbox;
    GtkWidget *url_entry;
    GtkWidget *keep_button;

    guint destroy_on_idle_id;

    static gboolean destroy_on_idle_cb(ResponseEditorWindow *editor);

    static void notify_sensitive_cb(GtkAction *action, GParamSpec *arg1,
				    ResponseEditorWindow *window);
    static void response_cb(GtkWidget *dialog, int response_id,
			    ResponseEditorWindow *window);

    virtual void notify_post_finished();


  public:
    ResponseEditorWindow(Controller &controller, BBSThread *thread);
    virtual ~ResponseEditorWindow();

    virtual void set_thread(BBSThread *thread);

    virtual GtkWidget *get_widget();

    static ResponseEditorWindow *get_window(GtkWidget *widget);
  };


  class NewThreadWindow : public ResponseEditor
  {
  protected:
    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *board_hbox;
    GtkWidget *board_label;

    guint destroy_on_idle_id;

    static gboolean destroy_on_idle_cb(NewThreadWindow *editor);

    static void notify_sensitive_cb(GtkAction *action, GParamSpec *arg1,
				    NewThreadWindow *window);
    static void response_cb(GtkWidget *dialog, int response_id,
			    NewThreadWindow *window);

    virtual void notify_post_finished();


  public:
    NewThreadWindow(Controller &controller, BulletinBoard *board);
    virtual ~NewThreadWindow();

    virtual void set_thread(BBSThread *thread);

    virtual GtkWidget *get_widget();
    static NewThreadWindow *get_window(GtkWidget *widget);
  };


}


#endif	/* __cplusplus */

#endif	/* _POST_UI_H_ */
