/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: preferences.h,v 1.32 2010/08/15 15:42:20 fuyu Exp $
 */

#ifndef _PREFERENCES_H_
#define _PREFERENCES_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/constants.h"
#include "libochusha/sigslot.h"
#include "libochusha/uri_utils.h"


#include <list>

#include <gtk/gtk.h>

#if defined(HAVE_STDINT_H) || !defined(HAVE_CONFIG_H)
#include <stdint.h>
#else
#ifdef HAVE_INTTYPES_H
#include <inttypes.h>
#endif
#endif

#include <sys/types.h>


using namespace std;
namespace Ochusha
{
  class Controller;
  class HTMLRendererTagManager;
  class Repository;
  class URI;

  enum ViewPosition {
    MY_BOOKMARK = 0,
    LAST_VIEW_POSITION = 1,
    THREAD_TOP = 2
  };

  class Preferences
  {
    friend class BBS2chHelper;
    friend class BBSThreadUI;
    friend class BoardlistUI;
    friend class BoardPropertiesUI;
    friend class BoardUI;
    friend class Controller;
    friend class EmbededResponseEditor;
    friend class FontsUI;
    friend class HTMLRendererTagManager;
    friend class MainWindow;	// Sidebar
    friend class PreferencesUI;
    friend class ResponseEditor;
    friend class SchemeManager;
    friend class ViewManager;

  private:
    Controller &controller;
    GtkActionGroup *action_group;
    bool changed;


    // ġСϢ
    GtkToggleAction *show_navigation_toolbar;	// ͽ
    GtkToggleAction *show_boardlist_toolbar;	// ͽ
    GtkToggleAction *show_bookmarks_toolbar;	// ͽ
    GtkToggleAction *show_threadlist_toolbar;
    GtkToggleAction *show_thread_toolbar;


    // ɥСϢ
    GtkToggleAction *show_boardlist_sidebar;
    GtkToggleAction *show_bookmarks_sidebar;
    GtkToggleAction *show_history_sidebar;	// ͽ
    GtkToggleAction *show_post_history_sidebar;	// ͽ


    // ꥹɽϢ
    GtkToggleAction *show_hidden_boardlist_items;
    GtkToggleAction *show_hidden_threads;

    GdkColor fg_normal_color;
    GdkColor *fg_normal;

    GdkColor bg_normal_color;
    GdkColor *bg_normal;

    GdkColor fg_emph_color;
    GdkColor *fg_emph;

    GdkColor bg_emph_color;
    GdkColor *bg_emph;

    GdkColor fg_strong_color;
    GdkColor *fg_strong;

    GdkColor bg_strong_color;
    GdkColor *bg_strong;

    GdkColor fg_hidden_color;
    GdkColor *fg_hidden;

    GdkColor bg_hidden_color;
    GdkColor *bg_hidden;


    // 
    GtkToggleAction *vertical_placement;
    GtkToggleAction *default_open_in_tab;
    GtkToggleAction *select_tab_when_opened;
    GtkToggleAction *focus_follow_mouse;
    GtkToggleAction *focus_follow_mouse_modestly;
    GtkToggleAction *enable_embeded_response_editor;
    GtkToggleAction *prefer_embeded_response_editor;


    // ()ڥϢ
    GtkToggleAction *board_tab_shrinkable;
    unsigned int board_tab_minimum_size;
    GtkToggleAction *board_tab_enable_tooltips;
    GtkToggleAction *board_tab_always_show;
    GtkToggleAction *board_pane_always_show;


    // Ϣ
    char *threadlist_view_contents;
    char *default_threadlist_rule;


    // ڥϢ
    GtkToggleAction *thread_tab_shrinkable;
    unsigned int thread_tab_minimum_size;
    GtkToggleAction *thread_tab_enable_tooltips;
    GtkToggleAction *thread_tab_always_show;
    GtkToggleAction *thread_pane_always_show;


    // ɽϢ
    char *thread_view_font_name;
    GtkToggleAction *enable_transparent_a_bone;
    GtkToggleAction *disable_popup_a_bone;
    GtkToggleAction *disable_a_bone;
    GtkToggleAction *enable_chain_a_bone;
    GtkToggleAction *show_mailto;
    GtkToggleAction *enable_be_icon;
    GtkToggleAction *enable_inline_image;

    ViewPosition initial_view_position_of_active_thread;
    ViewPosition initial_view_position_of_inactive_thread;

    GdkColor auto_refresh_go_color;
    GdkColor *auto_refresh_go;
    GdkColor auto_refresh_wait_color;
    GdkColor *auto_refresh_wait;

    GdkColor thread_title_color;
    GdkColor *thread_title;
    GdkColor name_color;
    GdkColor *name;

    GdkColor link_color;
    GdkColor *link;
    GdkColor visited_color;
    GdkColor *visited;


    // ѤϢ
    GtkToggleAction *hide_bookmark;
    char *bookmark_text;


    // ݥåץå״Ϣ
    GtkToggleAction *enable_popup_response;
    GtkToggleAction *enable_popup_image;
    GtkToggleAction *enable_popup_title;
    GtkToggleAction *enable_popup_id;

    unsigned int popup_delay;
    unsigned int popdown_delay;

    int maximum_number_of_popup_responses;


    // Ϣ
    int inline_image_width;
    int inline_image_height;
    int popup_image_width;
    int popup_image_height;


    // åϢ
    uintmax_t capacity_of_image_cache;
    int maximum_number_of_image_cache_files;

    GtkToggleAction *download_shown_images_only;


    // ֥֥饦Ϣ
    char *web_browser_template;


    // proxyϢ
    GtkToggleAction *enable_proxy_to_read;
    GtkToggleAction *enable_proxy_to_post;


    GtkToggleAction *enable_proxy_for_images;
    URI proxy_uri_for_images;


    // ǥХåϢ
    GtkToggleAction *debug_console;
    GtkToggleAction *scheme_console;


    // 񤭹ߴϢ
    list<char *> recent_name_list;
    list<char *> recent_mail_list;


    // ͤ
    GtkToggleAction *login_2ch_be_at_startup;
    GtkToggleAction *save_2ch_be_passwd;

    GtkToggleAction *login_2ch_viewer_at_startup;
    GtkToggleAction *save_2ch_viewer_passwd;


    // åط
    GtkToggleAction *restore_opened_tabs;


    static void prefs_changed_cb(gpointer unused, Preferences *prefs);


  public:
    Preferences(Controller &controller);
    virtual ~Preferences();


    // GLib/GTK+ȤߤѲǧΤǤʤ(intchar *)
    // ѲȯԤsigslotΥʥ롣ѲäƤ
    // ɤΤGtkToggleActionξˤϡGObjectΥʥ뤬
    // Τǥ֤äΤ̵̤ˤƤ
    // void prefs_changed(Preferences &prefs)
    signal1<Preferences &> prefs_changed;


  private:
    void set_show_navigation_toolbar(bool val)
    { gtk_toggle_action_set_active(show_navigation_toolbar, val); }
    void set_show_boardlist_toolbar(bool val)
    { gtk_toggle_action_set_active(show_boardlist_toolbar, val); }
    void set_show_bookmarks_toolbar(bool val)
    { gtk_toggle_action_set_active(show_bookmarks_toolbar, val); }
    void set_show_threadlist_toolbar(bool val)
    { gtk_toggle_action_set_active(show_threadlist_toolbar, val); }
    void set_show_thread_toolbar(bool val)
    { gtk_toggle_action_set_active(show_thread_toolbar, val); }

    void set_show_boardlist_sidebar(bool val)
    { gtk_toggle_action_set_active(show_boardlist_sidebar, val); }
    void set_show_bookmarks_sidebar(bool val)
    { gtk_toggle_action_set_active(show_bookmarks_sidebar, val); }
    void set_show_history_sidebar(bool val)
    { gtk_toggle_action_set_active(show_history_sidebar, val); }
    void set_show_post_history_sidebar(bool val)
    { gtk_toggle_action_set_active(show_post_history_sidebar, val); }

    void set_show_hidden_boardlist_items(bool val)
    { gtk_toggle_action_set_active(show_hidden_boardlist_items, val); }
    void set_show_hidden_threads(bool val)
    { gtk_toggle_action_set_active(show_hidden_threads, val); }

    void set_fg_normal_color_set(bool val);
    void set_bg_normal_color_set(bool val);
    void set_fg_emph_color_set(bool val);
    void set_bg_emph_color_set(bool val);
    void set_fg_strong_color_set(bool val);
    void set_bg_strong_color_set(bool val);
    void set_fg_hidden_color_set(bool val);
    void set_bg_hidden_color_set(bool val);


    void set_vertical_placement(bool val)
    { gtk_toggle_action_set_active(vertical_placement, val); }
    void set_default_open_in_tab(bool val)
    { gtk_toggle_action_set_active(default_open_in_tab, val); }
    void set_select_tab_when_opened(bool val)
    { gtk_toggle_action_set_active(select_tab_when_opened, val); }
    void set_focus_follow_mouse(bool val)
    { gtk_toggle_action_set_active(focus_follow_mouse, val); }
    void set_focus_follow_mouse_modestly(bool val)
    { gtk_toggle_action_set_active(focus_follow_mouse_modestly, val); }
    void set_enable_embeded_response_editor(bool val)
    { gtk_toggle_action_set_active(enable_embeded_response_editor, val); }
    void set_prefer_embeded_response_editor(bool val)
    { gtk_toggle_action_set_active(prefer_embeded_response_editor, val); }

    void set_board_tab_shrinkable(bool val)
    { gtk_toggle_action_set_active(board_tab_shrinkable, val); }
    void set_board_tab_minimum_size(unsigned int val);
    void set_board_tab_enable_tooltips(bool val)
    { gtk_toggle_action_set_active(board_tab_enable_tooltips, val); }
    void set_board_tab_always_show(bool val)
    { gtk_toggle_action_set_active(board_tab_always_show, val); }
    void set_board_pane_always_show(bool val)
    { gtk_toggle_action_set_active(board_pane_always_show, val); }

    void set_threadlist_view_contents(const char *val);
    void set_default_threadlist_rule(const char *val);

    void set_thread_tab_shrinkable(bool val)
    { gtk_toggle_action_set_active(thread_tab_shrinkable, val); }
    void set_thread_tab_minimum_size(unsigned int val);
    void set_thread_tab_enable_tooltips(bool val)
    { gtk_toggle_action_set_active(thread_tab_enable_tooltips, val); }
    void set_thread_tab_always_show(bool val)
    { gtk_toggle_action_set_active(thread_tab_always_show, val); }
    void set_thread_pane_always_show(bool val)
    { gtk_toggle_action_set_active(thread_pane_always_show, val); }

    void set_thread_view_font_name(const char *val);
    void set_enable_transparent_a_bone(bool val)
    { gtk_toggle_action_set_active(enable_transparent_a_bone, val); }
    void set_disable_popup_a_bone(bool val)
    { gtk_toggle_action_set_active(disable_popup_a_bone, val); }
    void set_disable_a_bone(bool val)
    { gtk_toggle_action_set_active(disable_a_bone, val); }
    void set_enable_chain_a_bone(bool val)
    { gtk_toggle_action_set_active(enable_chain_a_bone, val); }
    void set_show_mailto(bool val)
    { gtk_toggle_action_set_active(show_mailto, val); }
    void set_show_mailto_mode(MailtoMode val)
    { set_show_mailto(val == MAILTO_MODE_SHOW); }
    void set_initial_view_position_of_active_thread(ViewPosition position)
    { initial_view_position_of_active_thread = position; }
    void set_initial_view_position_of_inactive_thread(ViewPosition position)
    { initial_view_position_of_inactive_thread = position; }

    void set_hide_bookmark(bool val)
    { gtk_toggle_action_set_active(hide_bookmark, val); }
    void set_bookmark_text(const char *val);

    void set_enable_popup_response(bool val)
    { gtk_toggle_action_set_active(enable_popup_response, val); }
    void set_enable_be_icon(bool val)
    { gtk_toggle_action_set_active(enable_be_icon, val); }
    void set_enable_inline_image(bool val)
    { gtk_toggle_action_set_active(enable_inline_image, val); }
    void set_download_shown_images_only(bool val)
    { gtk_toggle_action_set_active(download_shown_images_only, val); }
    void set_enable_popup_image(bool val)
    { gtk_toggle_action_set_active(enable_popup_image, val); }
    void set_enable_popup_title(bool val)
    { gtk_toggle_action_set_active(enable_popup_title, val); }
    void set_enable_popup_id(bool val)
    { gtk_toggle_action_set_active(enable_popup_id, val); }

    void set_popup_delay(unsigned int val);
    void set_popdown_delay(unsigned int val);

    void set_maximum_number_of_popup_responses(int val);

    void set_inline_image_width(int val);
    void set_inline_image_height(int val);
    void set_popup_image_width(int val);
    void set_popup_image_height(int val);

    void set_capacity_of_image_cache(uintmax_t val);
    void set_maximum_number_of_image_cache_files(int val);

    void set_web_browser_template(const char *val);

    static void enable_proxy_to_read_toggled_cb(GtkToggleAction *action,
						Preferences *prefs);
    void set_enable_proxy_to_read(bool val)
    { gtk_toggle_action_set_active(enable_proxy_to_read, val); }
    void set_proxy_to_read(const char *server, int port,
			   const char *user, const char *passwd);


    static void enable_proxy_to_post_toggled_cb(GtkToggleAction *action,
						Preferences *prefs);
    void set_enable_proxy_to_post(bool val)
    { gtk_toggle_action_set_active(enable_proxy_to_post, val); }
    void set_proxy_to_post(const char *server, int port,
			   const char *user, const char *passwd);


    void set_enable_proxy_for_images(bool val)
    { gtk_toggle_action_set_active(enable_proxy_for_images, val); }
    void set_proxy_for_images(const char *server, int port,
			      const char *user, const char *passwd);


    void set_debug_console(bool val)
    { gtk_toggle_action_set_active(debug_console, val); }
    void set_scheme_console(bool val)
    { gtk_toggle_action_set_active(scheme_console, val); }

    void register_recent_name(const char *val);
    void register_recent_mail(const char *val);

    void set_login_2ch_be_at_startup(bool val)
    { gtk_toggle_action_set_active(login_2ch_be_at_startup, val); }
    void set_save_2ch_be_passwd(bool val)
    { gtk_toggle_action_set_active(save_2ch_be_passwd, val); }
    void set_login_2ch_viewer_at_startup(bool val)
    { gtk_toggle_action_set_active(login_2ch_viewer_at_startup, val); }
    void set_save_2ch_viewer_passwd(bool val)
    { gtk_toggle_action_set_active(save_2ch_viewer_passwd, val); }

    void set_restore_opened_tabs(bool val)
    { gtk_toggle_action_set_active(restore_opened_tabs, val); }


  public:
    bool get_show_navigation_toolbar()
    { return gtk_toggle_action_get_active(show_navigation_toolbar); }
    bool get_show_threadlist_toolbar()
    { return gtk_toggle_action_get_active(show_threadlist_toolbar); }
    bool get_show_thread_toolbar()
    { return gtk_toggle_action_get_active(show_thread_toolbar); }
    bool get_show_boardlist_toolbar()
    { return gtk_toggle_action_get_active(show_boardlist_toolbar); }
    bool get_show_bookmarks_toolbar()
    { return gtk_toggle_action_get_active(show_bookmarks_toolbar); }

    bool get_show_boardlist_sidebar()
    { return gtk_toggle_action_get_active(show_boardlist_sidebar); }
    bool get_show_bookmarks_sidebar()
    { return gtk_toggle_action_get_active(show_bookmarks_sidebar); }
    bool get_show_history_sidebar()
    { return gtk_toggle_action_get_active(show_history_sidebar); }
    bool get_show_post_history_sidebar()
    { return gtk_toggle_action_get_active(show_post_history_sidebar); }

    bool get_show_hidden_boardlist_items() const
    { return gtk_toggle_action_get_active(show_hidden_boardlist_items); }
    bool get_show_hidden_threads() const
    { return gtk_toggle_action_get_active(show_hidden_threads); }

    GdkColor *get_fg_normal() const { return fg_normal; }
    GdkColor *get_bg_normal() const { return bg_normal; }
    GdkColor *get_fg_emph() const { return fg_emph; }
    GdkColor *get_bg_emph() const { return bg_emph; }
    GdkColor *get_fg_strong() const { return fg_strong; }
    GdkColor *get_bg_strong() const { return bg_strong; }
    GdkColor *get_fg_hidden() const { return fg_hidden; }
    GdkColor *get_bg_hidden() const { return bg_hidden; }

    int get_vertical_placement() const
    { return gtk_toggle_action_get_active(vertical_placement); }
    bool get_default_open_in_tab() const
    { return gtk_toggle_action_get_active(default_open_in_tab); }
    bool get_select_tab_when_opened() const
    { return gtk_toggle_action_get_active(select_tab_when_opened); }
    bool get_focus_follow_mouse() const
    { return gtk_toggle_action_get_active(focus_follow_mouse); }
    bool get_focus_follow_mouse_modestly() const
    { return gtk_toggle_action_get_active(focus_follow_mouse_modestly); }
    bool get_enable_embeded_response_editor() const
    { return gtk_toggle_action_get_active(enable_embeded_response_editor); }
    bool get_prefer_embeded_response_editor() const
    { return gtk_toggle_action_get_active(prefer_embeded_response_editor); }

    bool get_board_tab_shrinkable() const
    { return gtk_toggle_action_get_active(board_tab_shrinkable); }
    unsigned int get_board_tab_minimum_size() const
    { return board_tab_minimum_size; }
    bool get_board_tab_enable_tooltips() const
    { return gtk_toggle_action_get_active(board_tab_enable_tooltips); }
    bool get_board_tab_always_show() const
    { return gtk_toggle_action_get_active(board_tab_always_show); }
    bool get_board_pane_always_show() const
    { return gtk_toggle_action_get_active(board_pane_always_show); }

    const char *get_threadlist_view_contents() const
    { return threadlist_view_contents; }
    const char *get_default_threadlist_rule() const
    { return default_threadlist_rule; }

    bool get_thread_tab_shrinkable() const
    { return gtk_toggle_action_get_active(thread_tab_shrinkable); }
    unsigned int get_thread_tab_minimum_size() const
    { return thread_tab_minimum_size; }
    bool get_thread_tab_enable_tooltips() const
    { return gtk_toggle_action_get_active(thread_tab_enable_tooltips); }
    bool get_thread_tab_always_show() const
    { return gtk_toggle_action_get_active(thread_tab_always_show); }
    bool get_thread_pane_always_show() const
    { return gtk_toggle_action_get_active(thread_pane_always_show); }

    const char *get_thread_view_font_name() const
    { return thread_view_font_name; }
    bool get_enable_transparent_a_bone() const
    { return gtk_toggle_action_get_active(enable_transparent_a_bone); }
    bool get_disable_popup_a_bone() const
    { return gtk_toggle_action_get_active(disable_popup_a_bone); }
    bool get_disable_a_bone() const
    { return gtk_toggle_action_get_active(disable_a_bone); }
    bool get_enable_chain_a_bone() const
    { return gtk_toggle_action_get_active(enable_chain_a_bone); }
    bool get_show_mailto() const
    { return gtk_toggle_action_get_active(show_mailto); }
    MailtoMode get_show_mailto_mode() const
    {
      return gtk_toggle_action_get_active(show_mailto)
	? MAILTO_MODE_SHOW : MAILTO_MODE_HIDE;
    }
    ViewPosition get_initial_view_position_of_active_thread()
    { return initial_view_position_of_active_thread; }
    ViewPosition get_initial_view_position_of_inactive_thread()
    { return initial_view_position_of_inactive_thread; }

    GdkColor *get_auto_refresh_go() const { return auto_refresh_go; }
    GdkColor *get_auto_refresh_wait() const { return auto_refresh_wait; }

    GdkColor *get_thread_title() const { return thread_title; }
    GdkColor *get_name() const { return name; }
    GdkColor *get_link() const { return link; }
    GdkColor *get_visited() const { return visited; }

    bool get_hide_bookmark() const
    { return gtk_toggle_action_get_active(hide_bookmark); }
    const char *get_bookmark_text() const { return bookmark_text; }

    bool get_enable_popup_response() const
    { return gtk_toggle_action_get_active(enable_popup_response); }
    bool get_enable_be_icon() const
    { return gtk_toggle_action_get_active(enable_be_icon); }
    bool get_enable_inline_image() const
    { return gtk_toggle_action_get_active(enable_inline_image); }
    bool get_download_shown_images_only() const
    { return gtk_toggle_action_get_active(download_shown_images_only); }
    bool get_enable_popup_image() const
    { return gtk_toggle_action_get_active(enable_popup_image); }
    bool get_enable_popup_title() const
    { return gtk_toggle_action_get_active(enable_popup_title); }
    bool get_enable_popup_id() const
    { return gtk_toggle_action_get_active(enable_popup_id); }

    unsigned int get_popup_delay() const { return popup_delay; }
    unsigned int get_popdown_delay() const { return popdown_delay; }

    int get_maximum_number_of_popup_responses() const
    { return maximum_number_of_popup_responses; }

    int get_inline_image_width() const { return inline_image_width; }
    int get_inline_image_height() const
    { return inline_image_height; }
    int get_popup_image_width() const { return popup_image_width; }
    int get_popup_image_height() const { return popup_image_height; }

    uintmax_t get_capacity_of_image_cache() const
    { return capacity_of_image_cache; }
    int get_maximum_number_of_image_cache_files() const
    { return maximum_number_of_image_cache_files; }

    const char *get_web_browser_template() const
    { return web_browser_template; }

    bool get_enable_proxy_to_read() const
    { return gtk_toggle_action_get_active(enable_proxy_to_read); }
    const URI &get_proxy_uri_to_read() const;


    bool get_enable_proxy_to_post() const
    { return gtk_toggle_action_get_active(enable_proxy_to_post); }
    const URI &get_proxy_uri_to_post() const;


    bool get_enable_proxy_for_images() const
    { return gtk_toggle_action_get_active(enable_proxy_for_images); }
    const URI &get_proxy_uri_for_images() const;


    bool get_debug_console() const
    { return gtk_toggle_action_get_active(debug_console); }
    bool get_scheme_console() const
    { return gtk_toggle_action_get_active(scheme_console); }

    list<char *>::const_iterator get_recent_name_list_begin() const
    { return recent_name_list.begin(); }
    list<char *>::const_iterator get_recent_name_list_end() const
    { return recent_name_list.end(); }

    list<char *>::const_iterator get_recent_mail_list_begin() const
    { return recent_mail_list.begin(); }
    list<char *>::const_iterator get_recent_mail_list_end() const
    { return recent_mail_list.end(); }

    bool get_login_2ch_be_at_startup() const
    { return gtk_toggle_action_get_active(login_2ch_be_at_startup); }
    bool get_save_2ch_be_passwd() const
    { return gtk_toggle_action_get_active(save_2ch_be_passwd); }
    bool get_login_2ch_viewer_at_startup() const
    { return gtk_toggle_action_get_active(login_2ch_viewer_at_startup); }
    bool get_save_2ch_viewer_passwd() const
    { return gtk_toggle_action_get_active(save_2ch_viewer_passwd); }

    bool get_restore_opened_tabs() const
    { return gtk_toggle_action_get_active(restore_opened_tabs); }

    bool write_prefs_xml(Repository &repository);
    bool read_prefs_xml(Repository &repository);
  };


}


#endif	/* __cplusplus */

#endif	/* _PREFERENCES_H_ */
