/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: preferences_ui.h,v 1.24 2010/08/15 15:42:20 fuyu Exp $
 */

#ifndef _PREFERENCES_UI_H_
#define _PREFERENCES_UI_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "a_bone_ui.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class Configuration;
  class Controller;
  class Preferences;

  class PreferencesUI
  {
  private:
    Controller &controller;
    Configuration &config;
    Preferences &prefs;
    ABoneSettings *a_bone_settings;


    GtkWidget *dialog;


    GtkWidget *web_browser_template_entry;

    GtkWidget *board_tab_minimum_width_spin_button;
    GtkWidget *thread_tab_minimum_width_spin_button;

    GtkWidget *default_threadlist_rule_combo;
    GtkWidget *default_threadlist_column_entry;

    GtkWidget *proxy_server_to_read_entry;
    GtkWidget *proxy_port_to_read_entry;
    GtkWidget *proxy_user_to_read_entry;
    GtkWidget *proxy_passwd_to_read_entry;

    GtkWidget *proxy_server_to_post_entry;
    GtkWidget *proxy_port_to_post_entry;
    GtkWidget *proxy_user_to_post_entry;
    GtkWidget *proxy_passwd_to_post_entry;

    GtkWidget *proxy_server_for_images_entry;
    GtkWidget *proxy_port_for_images_entry;
    GtkWidget *proxy_user_for_images_entry;
    GtkWidget *proxy_passwd_for_images_entry;

    GtkWidget *popup_delay_spin_button;
    GtkWidget *popdown_delay_spin_button;

    GtkWidget *inline_image_width_spin_button;
    GtkWidget *inline_image_height_spin_button;
    GtkWidget *popup_image_width_spin_button;
    GtkWidget *popup_image_height_spin_button;

    GtkWidget *maximum_number_of_image_cache_files_spin_button;
    GtkWidget *current_number_of_image_cache_files;
    GtkWidget *capacity_of_image_cache_spin_button;
    GtkWidget *current_image_cache_size;

    GtkTextBuffer *bookmark_text_buffer;

    GtkWidget *fg_normal_color_button;
    GtkWidget *fg_normal_color_set_button;
    GtkWidget *fg_emph_color_button;
    GtkWidget *fg_emph_color_set_button;
    GtkWidget *fg_strong_color_button;
    GtkWidget *fg_strong_color_set_button;
    GtkWidget *fg_hidden_color_button;
    GtkWidget *fg_hidden_color_set_button;

    GtkWidget *bg_normal_color_button;
    GtkWidget *bg_normal_color_set_button;
    GtkWidget *bg_emph_color_button;
    GtkWidget *bg_emph_color_set_button;
    GtkWidget *bg_strong_color_button;
    GtkWidget *bg_strong_color_set_button;
    GtkWidget *bg_hidden_color_button;
    GtkWidget *bg_hidden_color_set_button;


    GtkWidget *auto_refresh_go_color_button;
    GtkWidget *auto_refresh_wait_color_button;
    GtkWidget *thread_title_color_button;
    GtkWidget *name_color_button;
    GtkWidget *link_color_button;
    GtkWidget *visited_color_button;


    GtkWidget *fg_normal_state_color_button;
    GtkWidget *fg_prelight_state_color_button;
    GtkWidget *fg_active_state_color_button;
    GtkWidget *fg_selected_state_color_button;
    GtkWidget *fg_insensitive_state_color_button;

    GtkWidget *bg_normal_state_color_button;
    GtkWidget *bg_prelight_state_color_button;
    GtkWidget *bg_active_state_color_button;
    GtkWidget *bg_selected_state_color_button;
    GtkWidget *bg_insensitive_state_color_button;

    GtkWidget *text_normal_state_color_button;
    GtkWidget *text_prelight_state_color_button;
    GtkWidget *text_active_state_color_button;
    GtkWidget *text_selected_state_color_button;
    GtkWidget *text_insensitive_state_color_button;

    GtkWidget *base_normal_state_color_button;
    GtkWidget *base_prelight_state_color_button;
    GtkWidget *base_active_state_color_button;
    GtkWidget *base_selected_state_color_button;
    GtkWidget *base_insensitive_state_color_button;


    GtkWidget *focus_follow_mouse_button;
    GtkWidget *focus_follow_mouse_modestly_button;
    GtkWidget *default_open_in_tab_button;
    GtkWidget *restore_opened_tabs_button;
    GtkWidget *enable_popup_id_button;


    GtkWidget *a_bone_settings_frame;


    GtkWidget *auto_refresh_go;
    GtkWidget *auto_refresh_wait;

    GtkWidget *state_color_settings_frame;

    guint check_cache_status_timeout_id;
    guint ensure_5digits_only_on_idle_id;

    GtkWidget *create_general_settings_frame();
    GtkWidget *create_list_view_settings_frame();
    GtkWidget *create_thread_view_settings_frame();
    GtkWidget *create_a_bone_settings_frame();
    GtkWidget *create_image_view_settings_frame();
    GtkWidget *create_my_bookmark_settings_frame();
    GtkWidget *create_network_settings_frame();
    GtkWidget *create_state_color_settings_frame();

    void setup_dialog();
    void write_anarchy_gtkrc();
    void reload_anarchy_gtkrc();

    static void entry_activate_cb(GtkEntry *entry, PreferencesUI *ui);

    static gboolean check_cache_status_timeout_cb(PreferencesUI *ui);
    static void initial_view_position_of_active_thread_changed_cb(GtkToggleButton *button,
								  PreferencesUI *ui);
    static void initial_view_position_of_inactive_thread_changed_cb(GtkToggleButton *button,
								    PreferencesUI *ui);
    static void color_set_cb(GtkColorButton *button, PreferencesUI *ui);
    static void color_set_toggled_cb(GtkToggleButton *button,
				     PreferencesUI *ui);
    static void state_color_set_cb(GtkColorButton *button, PreferencesUI *ui);
    static void changed_cb(GObject *object, PreferencesUI *ui);
    static void response_cb(GtkWidget *dialog, int response_id,
			    PreferencesUI *ui);
    static void show_all_toggled_cb(GtkToggleButton *button,
				    PreferencesUI *ui);
    static gboolean ensure_5digits_only_on_idle(PreferencesUI *ui);
    static void port_entry_changed_cb(GtkEntry *entry, PreferencesUI *ui);


  public:
    PreferencesUI(Controller &controller);
    ~PreferencesUI();

    void show_dialog();

    void prepare_shutdown();
  };


}


#endif	/* __cplusplus */

#endif	/* _PREFERENCES_UI_H_ */
