/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: scheme_manager.h,v 1.24 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _SCHEME_MANAGER_H_
#define _SCHEME_MANAGER_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "console.h"

#include "libochusha/network_agent.h"
#include "libochusha/sigslot.h"
#include "libtscore/tscore.h"

#include "libochushagtk/menu_action.h"

#include <gtk/gtk.h>


using namespace LibTSCore;
using namespace std;
namespace Ochusha
{
  class BoardUI;
  class Controller;
  class OchushaProcedure;
  class Preferences;
  class SelectionData;

  class SchemeManager : public has_slots
  {
    friend class BBSThreadRenderer;
    friend class BBSThreadViewBase;
    friend class Controller;
    friend class ImageManager;
    friend class PreferencesUI;
    friend class ThreadlistView;

  private:
    Controller &controller;
    Console console;
    TSCore core;
    const OchushaProcedure *ochusha_procedure;
    int comm[2];


    // 
    CellHandle decorate_list_entry;
    CellHandle is_interest_thread;


    // 
    CellHandle abbreviate_thread_title;
    CellHandle rewrite_link;
    CellHandle rewrite_message;


    // 
    CellHandle is_image_link;
    CellHandle rewrite_image_link;
    CellHandle redirect_image_link;
    CellHandle prepare_image_request;


    // UIĥ
    GtkActionGroup *action_group;
    GtkListStore *external_tools_store;

    GtkTreeModel *link_tools_model;
    GtkAction *link_tools_action;

    GtkTreeModel *image_tools_model;
    GtkAction *image_tools_action;

    GtkTreeModel *text_tools_model;
    GtkAction *text_tools_action;


    int get_output_fd();
    int get_input_fd();

    GtkActionGroup *create_action_group();
    void update_tools_models();
    void define_constants();


    // NetworkAgentBrokersigslotʥΥϥɥ
    void pre_prepare_request_cb(NetworkAgent *agent);
    void post_prepare_request_cb(NetworkAgent *agent);


    // ġϢ
    static void menu_activate_cb(OchushaMenuAction *action,
				 GtkTreeModel *model,
				 GtkTreeIter *iter, GtkTreePath *path,
				 SchemeManager *manager);

    static void owner_change_cb(GtkClipboard *clipboard, GdkEvent *event,
				SchemeManager *manager);

    static void file_chooser_response_cb(GtkWidget *widget,
					 gint response_id,
					 SchemeManager *manager);
    void invoke_external_tool_helper(const char *name, int tool_type);
    void tools_set_sensitive(int tool_type, bool value);


  public:
    SchemeManager(Controller &controller);
    ~SchemeManager();

    inline TSCore &get_core() { return core; }
    inline const OchushaProcedure &get_ochusha_procedure() const
    { return *ochusha_procedure; }

    void invoke_external_tool(const char *name, const char *argument);
    void attach_external_text_tools_menu(GtkMenu *popup_menu,
					 SelectionData *data);

    void link_tools_set_sensitive(bool value);
    void image_tools_set_sensitive(bool value);
    void text_tools_set_sensitive(bool value);

    void prepare_shutdown();
  };


}


#endif	/* __cplusplus */

#endif	/* _SCHEME_MANAGER_H_ */
