/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: selection_utils.h,v 1.7 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _SELECTION_UTILS_H_
#define _SELECTION_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{


  class SelectionDataProxy;

  class SelectionData
  {
    friend class SelectionDataProxy;

  private:
    static void get_data(GtkClipboard *clipboard,
			 GtkSelectionData *selection_data,
			 guint info, gpointer data);
    static void clear_data(GtkClipboard *clipboard, gpointer data);


  protected:
    virtual const GtkTargetEntry *get_targets() = 0;
    virtual guint get_n_targets() = 0;

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info) = 0;
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard) = 0;


  public:
    virtual ~SelectionData();

    virtual bool deletable() = 0;
    virtual bool cuttable();
    virtual bool copiable() = 0;
    virtual bool pastable(GtkClipboard *clipboard) = 0;

    virtual bool do_delete() = 0;
    virtual bool do_copy(GtkClipboard *clipboard);
    virtual bool do_cut(GtkClipboard *clipboard);
    virtual bool do_paste(GtkClipboard *clipboard) = 0;


    // ¸thisΥԡäݤΤ֤
    virtual SelectionData *snapshot() = 0;

    gboolean set_to(GtkClipboard *clipboard);
    static SelectionData *get_from(GtkClipboard *clipboard);

    static gboolean set_uris(GtkSelectionData *data,
			     const vector<const char *> &uris);
    static gboolean set_uris(GtkSelectionData *data,
			     const vector<char *> &uris);
  };


  class TextViewSelectionData : public SelectionData
  {
  private:
    GtkTextView *view;
    GtkTextBuffer *buffer;
    gchar *text;

    gulong destroy_id;
    bool frozen;
    bool is_static;

    static void destroy_cb(GtkWidget *widget, TextViewSelectionData *data);


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);

    inline GtkTextView *get_view() const { return view; }
    inline GtkTextBuffer *get_buffer() const { return buffer; }


  public:
    TextViewSelectionData(GtkTextView *view, bool is_static);
    virtual ~TextViewSelectionData();

    virtual bool deletable();
    virtual bool cuttable();
    virtual bool copiable();
    virtual bool pastable(GtkClipboard *clipboard);

    virtual bool do_delete();
    virtual bool do_paste(GtkClipboard *clipboard);

    // ¸thisΥԡäݤΤ֤
    virtual SelectionData *snapshot();
  };


  class EditableSelectionData : public SelectionData
  {
  private:
    GtkWidget *widget;
    GtkEditable *editable;
    gchar *text;

    gulong destroy_id;
    bool frozen;
    bool is_static;

    static void destroy_cb(GtkWidget *widget, EditableSelectionData *data);


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    EditableSelectionData(GtkWidget *widget, bool is_static);
    virtual ~EditableSelectionData();

    virtual bool deletable();
    virtual bool cuttable();
    virtual bool copiable();
    virtual bool pastable(GtkClipboard *clipboard);

    virtual bool do_delete();
    virtual bool do_paste(GtkClipboard *clipboard);

    // ¸thisΥԡäݤΤ֤
    virtual SelectionData *snapshot();
  };


  class SelectionDataProxy : public SelectionData
  {
  private:
    SelectionData *data;
    GtkWidget *widget;
    gulong destroy_id;

    static void destroy_cb(GtkWidget *widget, SelectionDataProxy *proxy);


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    SelectionDataProxy(SelectionData *data, GtkWidget *widget);
    virtual ~SelectionDataProxy();

    virtual bool deletable();
    virtual bool cuttable();
    virtual bool copiable();
    virtual bool pastable(GtkClipboard *clipboard);

    virtual bool do_delete();
    virtual bool do_paste(GtkClipboard *clipboard);

    // ¸thisΥԡäݤΤ֤
    virtual SelectionData *snapshot();
  };


}


#endif	/* __cplusplus */

#endif	/* _SELECTION_UTILS_H_ */
