/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: session_client.h,v 1.9 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _SESSION_CLIENT_H_
#define _SESSION_CLIENT_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"
#include "xsmp_handler.h"


using namespace std;
namespace Ochusha
{
  class Repository;
  class Controller;

  class SessionClient : public XSMPHandler
  {
  private:
    Repository &repository;
    Controller *controller;

    AttributeTable states;


  public:
    SessionClient(Repository &repository);
    virtual ~SessionClient();

    void open_session(Controller *controller, int argc, char **argv);
    void close_session();

    const char *get_state(const char *name);
    void set_state(const char *name, const char *value);
    void remove_state(const char *name);

    void read_session_xml(const char *smc_id = NULL);
    void write_session_xml(const char *smc_id = NULL);

    void discard_session();


  protected:
    virtual void save_yourself(const char *smc_id);
    virtual void die(const char *smc_id);
    virtual void save_complete(const char *smc_id);
    virtual void shutdown_cancelled(const char *smc_id);

    virtual void restore(const char *smc_id);
    virtual void discard(const char *smc_id);
  };


}


#endif	/* __cplusplus */

#endif	/* _SESSION_CLIENT_H_ */
