/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: style_sheet.cc,v 1.12 2009/01/01 07:47:07 fuyu Exp $
 */

#include "config.h"

#define _STYLE_SHEET_CC_	1

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "style_sheet.h"

#include "libochusha/html_utils.h"
#include "libochusha/misc_utils.h"
#include "libochusha/text_utils.h"


#include <cassert>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>


namespace Ochusha
{


// #undef DEBUG_CSS_PARSER


CSSKeyword css_auto = "auto";
CSSKeyword css_none = "none";
CSSKeyword css_normal = "normal";
CSSKeyword css_inherit = "inherit";

CSSKeyword css_em = "em";
CSSKeyword css_ex = "ex";
CSSKeyword css_px = "px";
CSSKeyword css_in = "in";
CSSKeyword css_cm = "cm";
CSSKeyword css_mm = "mm";
CSSKeyword css_pt = "pt";
CSSKeyword css_pc = "pc";
CSSKeyword css_ms = "ms";
CSSKeyword css_s = "s";

CSSKeyword css_maroon = "maroon";
CSSKeyword css_red = "red";
CSSKeyword css_orange = "orange";
CSSKeyword css_yellow = "yellow";
CSSKeyword css_olive = "olive";
CSSKeyword css_purple = "purple";
CSSKeyword css_fuchsia = "fuchsia";
CSSKeyword css_white = "white";
CSSKeyword css_lime = "lime";
CSSKeyword css_green = "green";
CSSKeyword css_navy = "navy";
CSSKeyword css_blue = "blue";
CSSKeyword css_aqua = "aqua";
CSSKeyword css_teal = "teal";
CSSKeyword css_black = "black";
CSSKeyword css_silver = "silver";
CSSKeyword css_gray = "gray";

CSSKeyword css_transparent = "transparent";

CSSKeyword css_repeat = "repeat";
CSSKeyword css_repeat_x = "repeat-x";
CSSKeyword css_repeat_y = "repeat-y";
CSSKeyword css_no_repeat = "no-repeat";

CSSKeyword css_scroll = "scroll";
CSSKeyword css_fixed = "fixed";

CSSKeyword css_open_quote = "open-quote";
CSSKeyword css_close_quote = "close-quote";
CSSKeyword css_no_open_quote = "no-open-quote";
CSSKeyword css_no_close_quote = "no-close-quote";

CSSKeyword css_ltr = "ltr";
CSSKeyword css_rtl = "rtl";

CSSKeyword css_inline = "inline";
CSSKeyword css_block = "block";
CSSKeyword css_list_item = "list-item";
CSSKeyword css_run_in = "run-in";
CSSKeyword css_inline_block = "inline-block";
CSSKeyword css_table = "table";
CSSKeyword css_inline_table = "inline-table";
CSSKeyword css_table_row_group = "table-row-group";
CSSKeyword css_table_header_group = "table-header-group";
CSSKeyword css_table_footer_group = "table-footer-group";
CSSKeyword css_table_row = "table-row";
CSSKeyword css_table_column_group = "table-column-group";
CSSKeyword css_table_column = "table-column";
CSSKeyword css_table_cell = "table-cell";
CSSKeyword css_table_caption = "table-caption";

CSSKeyword css_serif = "serif";
CSSKeyword css_sans_serif = "sans-serif";
CSSKeyword css_cursive = "cursive";
CSSKeyword css_fantasy = "fantasy";
CSSKeyword css_monospace = "monospace";

CSSKeyword css_caption = "caption";
CSSKeyword css_icon = "icon";
CSSKeyword css_menu = "menu";
CSSKeyword css_message_box = "message-box";
CSSKeyword css_small_caption = "small-caption";
CSSKeyword css_status_bar = "status-bar";

CSSKeyword css_xx_small = "xx-small";
CSSKeyword css_x_small = "x-small";
CSSKeyword css_small = "small";
CSSKeyword css_medium = "medium";
CSSKeyword css_large = "large";
CSSKeyword css_x_large = "x-large";
CSSKeyword css_xx_large = "xx-large";
CSSKeyword css_smaller = "smaller";
CSSKeyword css_larger = "larger";

CSSKeyword css_italic = "italic";
CSSKeyword css_oblique = "oblique";

CSSKeyword css_small_caps = "small-caps";

CSSKeyword css_bold = "bold";
CSSKeyword css_bolder = "bolder";
CSSKeyword css_lighter = "lighter";

CSSKeyword css_inside = "inside";
CSSKeyword css_outside = "outside";

CSSKeyword css_disc = "disc";
CSSKeyword css_circle = "circle";
CSSKeyword css_square = "square";
CSSKeyword css_decimal = "decimal";
CSSKeyword css_decimal_leading_zero = "decimal-leading-zero";
CSSKeyword css_lower_roman = "lower-roman";
CSSKeyword css_upper_roman = "upper-roman";
CSSKeyword css_lower_greek = "lower-greek";
CSSKeyword css_lower_latin = "lower-latin";
CSSKeyword css_upper_latin = "upper-latin";
CSSKeyword css_armenian = "armenian";
CSSKeyword css_georgian = "georgian";
CSSKeyword css_lower_alpha = "lower-alpha";
CSSKeyword css_upper_alpha = "upper-alpha";

CSSKeyword css_left = "left";
CSSKeyword css_right = "right";
CSSKeyword css_center = "center";
CSSKeyword css_justify = "justify";

CSSKeyword css_underline = "underline";
CSSKeyword css_overline = "overline";
CSSKeyword css_line_through = "line-through";
CSSKeyword css_blink = "blink";

CSSKeyword css_capitalize = "capitalize";
CSSKeyword css_uppercase = "uppercase";
CSSKeyword css_lowercase = "lowercase";

CSSKeyword css_embed = "embed";
CSSKeyword css_bidi_override = "bidi-override";

CSSKeyword css_baseline = "baseline";
CSSKeyword css_sub = "sub";
CSSKeyword css_super = "super";
CSSKeyword css_top = "top";
CSSKeyword css_text_top = "text-top";
CSSKeyword css_middle = "middle";
CSSKeyword css_bottom = "bottom";
CSSKeyword css_text_bottom = "text-bottom";

CSSKeyword css_pre = "pre";
CSSKeyword css_nowrap = "nowrap";
CSSKeyword css_pre_wrap = "pre-wrap";
CSSKeyword css_pre_line = "pre-line";


enum CSSTokenType {
  CSS_TOKEN_TYPE_IDENT = 0,	// {ident}
  CSS_TOKEN_TYPE_ATKEYWORD,	// @{ident}
  CSS_TOKEN_TYPE_STRING,	// {string}
  CSS_TOKEN_TYPE_INVALID,	// {invalid}
  CSS_TOKEN_TYPE_HASH,		// #{name}
  CSS_TOKEN_TYPE_NUMBER,	// {num}
  CSS_TOKEN_TYPE_PERCENTAGE,	// {num}%
  CSS_TOKEN_TYPE_DIMENSION,	// {num}{ident}
  CSS_TOKEN_TYPE_URI,	// url\({w}{string}{w}\)
  			// url\({w}([!#$%&*-~]|{nonascii}|{escape})*{w}\)
  CSS_TOKEN_TYPE_UNICODE_RANGE,	// U\+[0-9A-F?]{1,6}(-[0-9A-F]{1,6})?
  CSS_TOKEN_TYPE_CDO,		// <!--
  CSS_TOKEN_TYPE_CDC,		// -->
  CSS_TOKEN_TYPE_SEMICOLON,	// ;
  CSS_TOKEN_TYPE_LBRACE,	// \{
  CSS_TOKEN_TYPE_RBRACE,	// \}
  CSS_TOKEN_TYPE_LPAREN,	// \(
  CSS_TOKEN_TYPE_RPAREN,	// \)
  CSS_TOKEN_TYPE_LBRACKET,	// \[
  CSS_TOKEN_TYPE_RBRACKET,	// \]
  CSS_TOKEN_TYPE_SPACE,		// [ \t\r\n\f]+
  CSS_TOKEN_TYPE_COMMENT,	// \/\*[^*]*\*+([^/*][^*]*\*+)*\/
  CSS_TOKEN_TYPE_FUNCTION,	// {ident}\(
  CSS_TOKEN_TYPE_INCLUDES,	// ~=
  CSS_TOKEN_TYPE_DASHMATCH,	// |=
  CSS_TOKEN_TYPE_DELIM		// any other character not matched by the
  				// above rules, and neither a single nor
  				// a double quote.
};


/*
 * ΥʬǻȤƤmacro
 *
 * ident	[-]?{nmstart}{nmchar}*
 * name		{nmchar}+
 * nmstart	{_a-zA-Z]|{nonascii}|{escape}
 * nonascii 	[^\0-\177]
 * unicode	\\[0-9a-f]{1,6}(\r\n|[ \n\r\t\f])?
 * escape	{unicode}|\\[^\n\r\f0-9a-f]
 * nmchar	[_a-zA-Z0-9-]|{nonascii}|{escape}
 * num		[0-9]+|[0-9]*\.[0-9]+
 * string	{string1}|{string2}
 * string1	\"([^\n\r\f\\"]|\\{nl}|{escape})*\"
 * string2	\'([^\n\r\f\\']|\\{nl}|{escape})*\'
 * invalid	{invalid1}|{invalid2}
 * invalid1	\"([^\n\r\f\\"]|\\{nl}|{escape})*
 * invalid2	\'([^\n\r\f\\']|\\{nl}|{escape})*
 * nl		\n|\r\n|\r|\f
 * w		[ \t\r\n\f]*
 */


class CSSToken
{
private:
  const char *head;
  int length;
  CSSTokenType type;

  char delim;
  char *string_value;
  int int_value;
  double number_value;


public:
  CSSToken(const char *head, int length, CSSTokenType type,
	   const char *ident_buf, const char *value_buf);
  ~CSSToken();
  inline const char *get_head() const { return head; }
  inline int get_length() const { return length; }
  inline CSSTokenType get_type() const { return type; }
  void append_as_string(string &buf) const;
  char *to_string() const;

  // դƻʺŬ;Ϥʤꤢġġ
  inline bool is_ident_inherit() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && strcmp(string_value, css_inherit) == 0;
  }
  inline bool is_ident_auto() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && strcmp(string_value, css_auto) == 0;
  }
  inline bool is_ident_background_repeat() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_repeat) == 0
	  || strcmp(string_value, css_repeat_x) == 0
	  || strcmp(string_value, css_repeat_y) == 0
	  || strcmp(string_value, css_no_repeat) == 0);
  }
  inline bool is_ident_background_attachment() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_scroll) == 0
	  || strcmp(string_value, css_fixed) == 0);
  }
  inline bool is_ident_background_position() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_left) == 0
	  || strcmp(string_value, css_center) == 0
	  || strcmp(string_value, css_right) == 0
	  || strcmp(string_value, css_top) == 0
	  || strcmp(string_value, css_bottom) == 0);
  }
  inline bool is_ident_font_style() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_normal) == 0
	  || strcmp(string_value, css_italic) == 0
	  || strcmp(string_value, css_oblique) == 0);
  }
  inline bool is_ident_font_variant() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_normal) == 0
	  || strcmp(string_value, css_small_caps) == 0);
  }
  inline bool is_ident_font_weight() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_normal) == 0
	  || strcmp(string_value, css_bold) == 0
	  || strcmp(string_value, css_bolder) == 0
	  || strcmp(string_value, css_lighter) == 0);
  }
  inline bool is_ident_font_size() const
  {
    return type == CSS_TOKEN_TYPE_IDENT
      && (strcmp(string_value, css_xx_small) == 0
	  || strcmp(string_value, css_x_small) == 0
	  || strcmp(string_value, css_small) == 0
	  || strcmp(string_value, css_medium) == 0
	  || strcmp(string_value, css_large) == 0
	  || strcmp(string_value, css_x_large) == 0
	  || strcmp(string_value, css_xx_large) == 0
	  || strcmp(string_value, css_smaller) == 0
	  || strcmp(string_value, css_larger) == 0);
  }
  inline bool is_length() const
  {
    return type == CSS_TOKEN_TYPE_DIMENSION
      && (strcmp(string_value, css_em) == 0
	  || strcmp(string_value, css_pt) == 0
	  || strcmp(string_value, css_px) == 0
	  || strcmp(string_value, css_ex) == 0
	  || strcmp(string_value, css_in) == 0
	  || strcmp(string_value, css_cm) == 0
	  || strcmp(string_value, css_pc) == 0);
  }

  CSSKeyword get_generic_family() const
  {
    if (type == CSS_TOKEN_TYPE_IDENT)
      {
	if (strcmp(string_value, css_serif) == 0)
	  return css_serif;
	if (strcmp(string_value, css_sans_serif) == 0)
	  return css_sans_serif;
	if (strcmp(string_value, css_cursive) == 0)
	  return css_cursive;
	if (strcmp(string_value, css_fantasy) == 0)
	  return css_fantasy;
	if (strcmp(string_value, css_monospace) == 0)
	  return css_monospace;
	if (strcmp(string_value, css_caption) == 0)
	  return css_caption;
	if (strcmp(string_value, css_icon) == 0)
	  return css_icon;
	if (strcmp(string_value, css_menu) == 0)
	  return css_menu;
	if (strcmp(string_value, css_message_box) == 0)
	  return css_message_box;
	if (strcmp(string_value, css_small_caption) == 0)
	  return css_small_caption;
	if (strcmp(string_value, css_status_bar) == 0)
	  return css_status_bar;
	if (strcmp(string_value, css_inherit) == 0)
	  return css_inherit;
      }
    return NULL;
  }

  CSSKeyword get_list_style_type() const
  {
    if (type == CSS_TOKEN_TYPE_IDENT)
      {
	if (strcmp(string_value, css_disc) == 0)
	  return css_disc;
	if (strcmp(string_value, css_circle) == 0)
	  return css_circle;
	if (strcmp(string_value, css_square) == 0)
	  return css_square;
	if (strcmp(string_value, css_decimal) == 0)
	  return css_decimal;
	if (strcmp(string_value, css_decimal_leading_zero) == 0)
	  return css_decimal_leading_zero;
	if (strcmp(string_value, css_lower_roman) == 0)
	  return css_lower_roman;
	if (strcmp(string_value, css_upper_roman) == 0)
	  return css_upper_roman;
	if (strcmp(string_value, css_lower_greek) == 0)
	  return css_lower_greek;
	if (strcmp(string_value, css_lower_latin) == 0)
	  return css_lower_latin;
	if (strcmp(string_value, css_upper_latin) == 0)
	  return css_upper_latin;
	if (strcmp(string_value, css_armenian) == 0)
	  return css_armenian;
	if (strcmp(string_value, css_georgian) == 0)
	  return css_georgian;
	if (strcmp(string_value, css_lower_alpha) == 0)
	  return css_lower_alpha;
	if (strcmp(string_value, css_upper_alpha) == 0)
	  return css_upper_alpha;
	if (strcmp(string_value, css_none) == 0)
	  return css_none;
	if (strcmp(string_value, css_inherit) == 0)
	  return css_inherit;
      }
    return NULL;
  }

  inline bool is_ident() const { return type == CSS_TOKEN_TYPE_IDENT; }
  inline const char *get_ident() const
  { assert(type == CSS_TOKEN_TYPE_IDENT); return string_value; }

  inline bool is_atkeyword() const
  { return type == CSS_TOKEN_TYPE_ATKEYWORD; }
  inline const char *get_atkeyword() const
  { assert(type == CSS_TOKEN_TYPE_ATKEYWORD); return string_value; }

  inline bool is_string() const { return type == CSS_TOKEN_TYPE_STRING; }
  inline const char *get_string() const
  { assert(type == CSS_TOKEN_TYPE_STRING); return string_value; }

  inline bool is_hash() const { return type == CSS_TOKEN_TYPE_HASH; }
  inline const char *get_hash() const
  { assert(type == CSS_TOKEN_TYPE_HASH); return string_value; }

  inline bool is_number() const { return type == CSS_TOKEN_TYPE_NUMBER; }
  inline int get_int_number() const
  { assert(type == CSS_TOKEN_TYPE_NUMBER); return int_value; }
  inline double get_number() const
  { assert(type == CSS_TOKEN_TYPE_NUMBER); return number_value; }

  inline bool is_percentage() const
  { return type == CSS_TOKEN_TYPE_PERCENTAGE; }
  inline double get_percentage() const
  { assert(type == CSS_TOKEN_TYPE_PERCENTAGE); return number_value; }

  inline bool is_dimension() const
  { return type == CSS_TOKEN_TYPE_DIMENSION; }
  inline double get_dimension_value() const
  { assert(type == CSS_TOKEN_TYPE_DIMENSION); return number_value; }
  inline const char *get_dimension_unit() const
  { assert(type == CSS_TOKEN_TYPE_DIMENSION); return string_value; }

  inline bool is_uri() const { return type == CSS_TOKEN_TYPE_URI; }
  inline const char *get_uri() const
  { assert(type == CSS_TOKEN_TYPE_URI); return string_value; }

  inline bool is_unicode_range() const
  { return type == CSS_TOKEN_TYPE_UNICODE_RANGE; }
  inline const char *get_unicode_range() const
  { assert(type == CSS_TOKEN_TYPE_UNICODE_RANGE); return string_value; }

  inline bool is_delim() const { return type == CSS_TOKEN_TYPE_DELIM; }
  inline char get_delim() const
  { assert(type == CSS_TOKEN_TYPE_DELIM); return delim; }

  inline bool is_function() const { return type == CSS_TOKEN_TYPE_FUNCTION; }
  inline const char *get_function() const
  { assert(type == CSS_TOKEN_TYPE_FUNCTION); return string_value; }

  // tokenizeϤsrcͭʴָ
  inline char *get_entire_token() const { return strndup(head, length); }

  static CSSToken *tokenize(const char *src, int length);
};


class CSSNodeAny;


class CSSNode
{
private:
  int lineno;


protected:
  CSSNodeAny *new_CSSNodeAny(CSSToken *token, int lineno);


public:
  CSSNode(int lineno) : lineno(lineno) {}
  virtual ~CSSNode() {}
  inline int get_lineno() const { return lineno; }
  virtual char *to_string() const = 0;

  virtual bool is_term() const { return false; }
  virtual bool is_any() const { return false; }
  virtual const CSSToken *get_term() const { return NULL; }
  virtual bool is_statement() const { return false; }
  virtual bool is_at_rule() const { return false; }
  virtual bool is_block() const { return false; }
  virtual bool is_ruleset() const { return false; }
  virtual bool is_declaration() const { return false; }
  virtual bool eat(CSSToken *token, int lineno) = 0;
};


// ̾ʥ
// ºݤˤ
// [ ATKEYWORD S* ] | [ ';' S* ] | [ '}' S* ]
// ѡ
class CSSNodeTerm : public CSSNode
{
private:
  CSSToken *term;


public:
  CSSNodeTerm(CSSToken *token, int lineno);
  virtual ~CSSNodeTerm();
  virtual char *to_string() const;

  virtual bool is_term() const { return true; }
  virtual const CSSToken *get_term() const { return term; }
  virtual bool eat(CSSToken *token, int lineno);
};


class CSSNodeAny : public CSSNode
{
private:
  CSSToken *leader;
  enum {
    TERM,
    FUNCTION_COMPLETE,
    PAREN_COMPLETE,
    BRACKET_COMPLETE,
    FUNCTION_INCOMPLETE,
    PAREN_INCOMPLETE,
    BRACKET_INCOMPLETE
  } state;
  vector<CSSNodeAny *> any;

  CSSNodeAny *next;


public:
  CSSNodeAny(CSSToken *token, int lineno);
  virtual ~CSSNodeAny();
  virtual char *to_string() const;
  virtual bool is_term() const;
  virtual bool is_any() const { return true; }
  virtual const CSSToken *get_term() const { return leader; }
  virtual bool eat(CSSToken *token, int lineno);

  inline const CSSToken *get_leader() const { return leader; }
  inline const vector<CSSNodeAny *> &get_any() const { return any; }
  inline bool is_completed() const { return state <= BRACKET_COMPLETE; }
};


class CSSNodeBlock : public CSSNode
{
public:
  enum BlockType {
    GENERIC_BLOCK,
    DECLARATION_BLOCK,
    RULESET_BLOCK
  };


private:
  BlockType block_type;
  vector<CSSNode *> elements;

  CSSNode *next;


public:
  CSSNodeBlock(int lineno, BlockType block_type)
    : CSSNode(lineno), block_type(block_type), next(NULL) {}
  virtual ~CSSNodeBlock();
  virtual char *to_string() const;
  virtual bool is_block() const { return true; }
  virtual bool eat(CSSToken *token, int lineno);
  inline bool is_declaration_block() const
  { return block_type == DECLARATION_BLOCK; }
  inline bool is_ruleset_block() const
  { return block_type == RULESET_BLOCK; }
  inline const vector<CSSNode *> &get_elements() const { return elements; }
};


class CSSNodeValue : public CSSNode
{
private:
  vector<CSSNode *> values;

  CSSNode *next;
  bool important;


public:
  CSSNodeValue(int lineno) : CSSNode(lineno), next(NULL), important(false) {}
  virtual ~CSSNodeValue();
  virtual char *to_string() const;
  virtual bool eat(CSSToken *token, int lineno);

  inline const vector<CSSNode *> &get_values() const { return values; }
};


class CSSNodeDeclaration : public CSSNode
{
private:
  CSSToken *delim;
  CSSToken *property;
  CSSNodeValue *value;
  bool colon;


public:
  CSSNodeDeclaration(CSSToken *token, int lineno);
  virtual ~CSSNodeDeclaration();
  virtual char *to_string() const;
  virtual bool is_declaration() const { return true; }
  virtual bool eat(CSSToken *token, int lineno);

  inline const char *get_property_name() const
  { return property != NULL ? property->get_ident() : NULL; }
  inline const CSSNodeValue *get_property_value() const { return value; }
};


class CSSNodeRuleset : public CSSNode
{
private:
  vector<CSSNodeAny *> selector;
  CSSNodeAny *next;
  CSSNodeBlock *declarations;


public:
  CSSNodeRuleset(int lineno);
  virtual ~CSSNodeRuleset();
  virtual char *to_string() const;
  virtual bool is_statement() const { return true; }
  virtual bool is_ruleset() const { return true; }
  virtual bool eat(CSSToken *token, int lineno);

  inline const vector<CSSNodeAny *> &get_selector() const { return selector; }
  inline const CSSNodeBlock *get_declarations() const { return declarations; }
};


class CSSNodeAtRule : public CSSNode
{
private:
  CSSToken *atkeyword;
  vector<CSSNodeAny *> any;
  CSSNode *tail;


public:
  CSSNodeAtRule(CSSToken *atkeyword, int lineno);
  virtual ~CSSNodeAtRule();
  virtual char *to_string() const;
  virtual bool is_statement() const { return true; }
  virtual bool is_at_rule() const { return true; }
  virtual bool eat(CSSToken *token, int lineno);
};


CSSToken::CSSToken(const char *head, int length, CSSTokenType type,
		   const char *ident_buf, const char *value_buf)
  : delim('\0'),
    string_value(NULL),
    int_value(0),
    number_value(0.0)
{
  this->head = head;
  this->length = length;
  this->type = type;

  if (string_value != NULL)
    {
      free(string_value);
      string_value = NULL;
    }

  switch (type)
    {
    case CSS_TOKEN_TYPE_DELIM:
      delim = *head;
      break;

    case CSS_TOKEN_TYPE_NUMBER:
    case CSS_TOKEN_TYPE_PERCENTAGE:
      {
	int result = sscanf(value_buf, "%lf", &number_value);
	if (*value_buf != '.')
	  result = sscanf(value_buf, "%d", &int_value);
	assert(result == 1);
	break;
      }

    case CSS_TOKEN_TYPE_DIMENSION:
      {
#ifndef NDEBUG
	int result = sscanf(value_buf, "%lf", &number_value);
	assert(result == 1);
#else
	sscanf(value_buf, "%lf", &number_value);
#endif
      }
      /* FALLTHROUGH */

    case CSS_TOKEN_TYPE_IDENT:
    case CSS_TOKEN_TYPE_ATKEYWORD:
    case CSS_TOKEN_TYPE_HASH:
    case CSS_TOKEN_TYPE_FUNCTION:
      string_value = strdup(ident_buf);
      break;

    case CSS_TOKEN_TYPE_UNICODE_RANGE:
    case CSS_TOKEN_TYPE_STRING:
    case CSS_TOKEN_TYPE_URI:
      string_value = strdup(value_buf);
      break;

    default:
      break;
    }
}


CSSToken::~CSSToken()
{
  if (string_value != NULL)
    free(string_value);
}


char *
CSSToken::to_string() const
{
  static const char * const type_string[] = {
    "IDENT",
    "ATKEYWORD",
    "STRING",
    "INVALID",
    "HASH",
    "NUMBER",
    "PERCENTAGE",
    "DIMENSION",
    "URI",
    "UNICODE-RANGE",
    "CDO",
    "CDC",
    "SEMICOLON",
    "LBRACE",
    "RBRACE",
    "LPAREN",
    "RPAREN",
    "LBRACKET",
    "RBRACKET",
    "SPACE",
    "COMMENT",
    "FUNCTION",
    "INCLUDES",
    "DASHMATCH",
    "DELIM"
  };
  assert(type <= CSS_TOKEN_TYPE_DELIM);

  string buf;
#ifdef EXTRA_RICH_TOKEN_STRING
  buf.append("CSSToken {\n").append("  entire_token: \"")
    .append(head, length).append("\"\n  length: ");
  char tmp_buf[64];
  snprintf(tmp_buf, 64, "%d\n", length);
  buf.append(tmp_buf).append("  type: ").append(type_string[type])
    .append("\n");
  if (string_value != NULL)
    buf.append("  string_value: \"").append(string_value).append("\"\n");
  buf.append("  number_value: ");
  snprintf(tmp_buf, 64, "%lf\n", number_value);
  buf.append(tmp_buf).append("}");
#else
  buf.append(type_string[type]);
  switch (type)
    {
    case CSS_TOKEN_TYPE_IDENT:
    case CSS_TOKEN_TYPE_ATKEYWORD:
    case CSS_TOKEN_TYPE_STRING:
    case CSS_TOKEN_TYPE_INVALID:
    case CSS_TOKEN_TYPE_HASH:
    case CSS_TOKEN_TYPE_URI:
    case CSS_TOKEN_TYPE_UNICODE_RANGE:
    case CSS_TOKEN_TYPE_FUNCTION:
      buf.append(1, '(').append(string_value).append(1, ')');
      break;

    case CSS_TOKEN_TYPE_NUMBER:
    case CSS_TOKEN_TYPE_PERCENTAGE:
    case CSS_TOKEN_TYPE_DIMENSION:
      {
	char num_buf[64];
	snprintf(num_buf, 64, "(%lf", number_value);
	buf.append(num_buf);
	if (type == CSS_TOKEN_TYPE_NUMBER)
	  buf.append(1, ')');
	else if (type == CSS_TOKEN_TYPE_PERCENTAGE)
	  buf.append("%)");
	else
	  buf.append(string_value).append(1, ')');
	break;
      }

    case CSS_TOKEN_TYPE_DELIM:
      buf.append("('").append(1, delim).append("')");
      break;

    case CSS_TOKEN_TYPE_COMMENT:
    case CSS_TOKEN_TYPE_CDO:
    case CSS_TOKEN_TYPE_CDC:
    case CSS_TOKEN_TYPE_SEMICOLON:
    case CSS_TOKEN_TYPE_LBRACE:
    case CSS_TOKEN_TYPE_RBRACE:
    case CSS_TOKEN_TYPE_LPAREN:
    case CSS_TOKEN_TYPE_RPAREN:
    case CSS_TOKEN_TYPE_LBRACKET:
    case CSS_TOKEN_TYPE_RBRACKET:
    case CSS_TOKEN_TYPE_SPACE:
    case CSS_TOKEN_TYPE_INCLUDES:
    case CSS_TOKEN_TYPE_DASHMATCH:
      break;
    }
#endif
  return strdup(buf.c_str());
}


void
CSSToken::append_as_string(string &buf) const
{
  switch (type)
    {
    case CSS_TOKEN_TYPE_ATKEYWORD:
      buf.append(1, '@').append(string_value);
      return;

    case CSS_TOKEN_TYPE_IDENT:
    case CSS_TOKEN_TYPE_STRING:
    case CSS_TOKEN_TYPE_UNICODE_RANGE:
      buf.append(string_value);
      return;

    case CSS_TOKEN_TYPE_HASH:
      buf.append(1, '#').append(string_value);
      return;

    case CSS_TOKEN_TYPE_URI:
      buf.append("url(\"").append(string_value).append(")\"");
      return;

    case CSS_TOKEN_TYPE_FUNCTION:
      buf.append(string_value).append(1, '(');
      return;

    case CSS_TOKEN_TYPE_NUMBER:
    case CSS_TOKEN_TYPE_PERCENTAGE:
    case CSS_TOKEN_TYPE_DIMENSION:
      {
	char num_buf[64];
	snprintf(num_buf, 64, "%lf", number_value);
	buf.append(num_buf);
	if (type == CSS_TOKEN_TYPE_PERCENTAGE)
	  buf.append("%");
	else if (type == CSS_TOKEN_TYPE_DIMENSION)
	  buf.append(string_value);
	return;
      }

    case CSS_TOKEN_TYPE_DELIM:
      buf.append(1, delim);
      return;

    case CSS_TOKEN_TYPE_CDO:
      buf.append("<!--");
      return;

    case CSS_TOKEN_TYPE_CDC:
      buf.append("-->");
      return;

    case CSS_TOKEN_TYPE_SEMICOLON:
      buf.append(1, ';');
      return;

    case CSS_TOKEN_TYPE_LBRACE:
      buf.append(1, '{');
      return;

    case CSS_TOKEN_TYPE_RBRACE:
      buf.append(1, '}');
      return;

    case CSS_TOKEN_TYPE_LPAREN:
      buf.append(1, '(');
      return;

    case CSS_TOKEN_TYPE_RPAREN:
      buf.append(1, ')');
      return;

    case CSS_TOKEN_TYPE_LBRACKET:
      buf.append(1, '[');
      return;

    case CSS_TOKEN_TYPE_RBRACKET:
      buf.append(1, ']');
      return;

    case CSS_TOKEN_TYPE_SPACE:
      buf.append(1, ' ');
      return;

    case CSS_TOKEN_TYPE_INCLUDES:
      buf.append("~=");
      return;

    case CSS_TOKEN_TYPE_DASHMATCH:
      buf.append("|=");
      return;

    case CSS_TOKEN_TYPE_INVALID:
    case CSS_TOKEN_TYPE_COMMENT:
      return;
    }
}


static inline bool
is_digit(char c)
{
  return c >= '0' && c <= '9';
}


static inline bool
is_alpha(char c)
{
  return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z');
}


static inline bool
is_alnum(char c)
{
  return is_alpha(c) || is_digit(c);
}


static inline bool
is_xdigit(char c)
{
  return is_digit(c) || (c >= 'A' && c <= 'F') || (c >= 'a' && c <= 'f');
}


static bool
is_escape(const char *src, int length)
{
  if (length < 2)
    return false;

  int c = reinterpret_cast<const unsigned char *>(src)[1];
  return *src == '\\'
    ? (is_xdigit(c) || (c != '\n' && c != '\r' && c != '\f'))
    : false;
}


static int
eat_unicode(const char *src, int length, string &buf)
{
  const char *cur_pos = src + 1;
  if (length < 2 || *src != '\\' || !is_xdigit(*cur_pos))
    return 0;

  unsigned int c = *reinterpret_cast<const unsigned char *>(cur_pos++);
  unsigned int uc;
  if (c <= '9')
    uc = c - '0';
  else if (c <= 'F')
    uc = c - 'A' + 10;
  else
    uc = c - 'a' + 10;
  for (int i = 0; i < 5; i++)
    {
      c = *reinterpret_cast<const unsigned char *>(cur_pos);
      if (!is_xdigit(c))
	break;
      uc = (uc << 4) | (c <= '9'
			? c - '0'
			: (c <= 'F' ? c - 'A' + 10 : c - 'a' + 10));
      ++cur_pos;
    }

  char utf8_buf[5];
  char *utf8_string = encode_unicode_to_utf8(uc, utf8_buf);
  if (utf8_string != NULL)
    buf.append(utf8_string);
  return cur_pos - src;
}


// src
// {unicode}|\\[^\n\r\f0-9a-f]
// ˥ޥåʸɤ߼ꡢCSSͤǲᤷbufappend롣
// ޥåʸĹ֤
static int
eat_escape(const char *src, int length, string &buf)
{
  if (!is_escape(src, length))
    return 0;

  assert(length >= 2);
  char c = src[1];
  if (is_xdigit(c))
    return eat_unicode(src, length, buf);

  assert(c != '\n' && c != '\r' && c != '\f');
  buf.append(1, src[1]);
  return 1;
}


// src
// ([_a-z0-9-]|{nonascii}|{escape})*
// ˥ޥåʸĹlengthޤɤ߼ꡢbufappend롣
// ޥåʸĹ֤
static int
eat_nmchars(const char *src, int length, string &buf)
{
  if (length == 0)
    return 0;

  const char *tail_pos = src + length;
  const char *cur_pos = src;
  const char *tmp_pos = cur_pos;
  while (true)
    {
      for (char c = *tmp_pos;
	   c == '_' || c == '-' || is_alnum(c) || (c & 0x80) != 0;
	   c = *tmp_pos)
	++tmp_pos;
      if (tmp_pos - cur_pos > 0)
	buf.append(cur_pos, tmp_pos - cur_pos);
      tmp_pos += eat_escape(tmp_pos, tail_pos - tmp_pos, buf);
      if (tmp_pos == cur_pos)
	break;
      cur_pos = tmp_pos;
    }

  return cur_pos - src;
}


static bool
is_nmstart(const char *src, int length)
{
  assert(length > 0);
  int c = *src;
  return c == '_' || (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')
    || (c & 0x80) || is_escape(src, length);
}


// src
// -?([_a-z]|{nonascii}|{escape})([_a-z0-9-]|{nonascii}|{escape})*
// ˥ޥåʸĹlengthޤɤ߼ꡢbufappend롣
// ޥåʸĹ֤
static int
eat_ident(const char *src, int length, string &buf)
{
  if (length == 0)
    return 0;

  if (is_nmstart(src, length))
    return eat_nmchars(src, length, buf);

  if (*src != '-')
    return 0;

  if (length < 2 || !is_nmstart(src + 1, length - 1))
    return 0;

  buf.append(1, '-');
  return 1 + eat_nmchars(src + 1, length - 1, buf);
}


static int
eat_hash(const char *src, int length, string &buf)
{
  if (length == 0)
    return 0;

  if (is_nmstart(src, length) || is_xdigit(*src))
    return eat_nmchars(src, length, buf);

  if (*src != '-')
    return 0;

  if (length < 2 || !is_nmstart(src + 1, length - 1))
    return 0;

  buf.append(1, '-');
  return 1 + eat_nmchars(src + 1, length - 1, buf);
}


// src
// [0-9]*
// ˥ޥåʸĹlengthޤɤ߼ꡢbufappend롣
// ޥåʸĹ֤
static int
eat_number(const char *src, int length, string &buf)
{
  if (length == 0)
    return 0;

  const char *cur_pos = src;
  const char *tail_pos = src + length;
  while (cur_pos < tail_pos)
    {
      char c = *cur_pos;
      if (!is_digit(c))
	break;
      buf.append(1, *cur_pos++);
    }
  return cur_pos - src;
}


static int
eat_string_char(const char *src, int length, string &buf)
{
  if (length == 0)
    return 0;

  char c = *src;
  if (c != '\\')
    {
      buf.append(1, c);
      return 1;
    }

  if (length < 2)
    return 0;

  int len = eat_escape(src, length, buf);
  if (len > 0)
    return len;

  c = src[1];
  assert(c == '\n' || c == '\r' || c == '\f');
  if (c == '\n' || c == '\f')
    return 2;

  if (length < 3 || src[2] != '\n')
    return 2;
  return 3;
}


static int
eat_url(const char *src, int length, string &buf)
{
  if (length == 0)
    return 0;

  const char *cur_pos = src;
  const char *tail_pos = src + length;
  while (cur_pos < tail_pos)
    {
      char c = *cur_pos;
      if ((c & 0x80) != 0
	  || c == '!' || c == '#' || c == '$' || c == '%'
	  || c == '&' || c == '*' || c == '-' || c == '~')
	{
	  buf.append(1, c);
	  ++cur_pos;
	  continue;
	}

      int len = eat_escape(cur_pos, tail_pos - cur_pos, buf);
      if (len > 0)
	{
	  cur_pos += len;
	  continue;
	}
      break;
    }

  return cur_pos - src;
}


// src
// [ \n\t\r\f]*
// ˥ޥåʸĹ֤
static int
count_white_space(const char *src, int length)
{
  const char *cur_pos = src;
  const char *tail_pos = src + length;
  while (cur_pos < tail_pos)
    {
      int c = *cur_pos;
      if (c != ' ' && c != '\n' && c != '\t' && c != '\r' && c != '\f')
	break;
      ++cur_pos;
    }
  return cur_pos - src;
}


CSSToken *
CSSToken::tokenize(const char *src, int length)
{
  if (*src == '\0')
    return NULL;

  string ident_buf;
  string value_buf;

  CSSTokenType type;
  const char *tail_pos = src + length;
  const char *cur_pos = src;

  char c = *cur_pos;
  int len;

  // lexer_state_initial:
  if (c == '"')
    {
      ++cur_pos;
      while (*cur_pos != '"')
	{
	  len = eat_string_char(cur_pos, tail_pos - cur_pos, value_buf);
	  if (len > 0)
	    {
	      cur_pos += len;
	      continue;
	    }
	  break;
	}
      if (*cur_pos == '"')
	{
	  ++cur_pos;
	  type = CSS_TOKEN_TYPE_STRING;
	}
      else
	type = CSS_TOKEN_TYPE_INVALID;
      goto accepted;
    }

  if (c == '\'')
    {
      ++cur_pos;
      while (*cur_pos != '\'')
	{
	  len = eat_string_char(cur_pos, tail_pos - cur_pos, value_buf);
	  if (len > 0)
	    {
	      cur_pos += len;
	      continue;
	    }
	  break;
	}
      if (*cur_pos == '\'')
	{
	  ++cur_pos;
	  type = CSS_TOKEN_TYPE_STRING;
	}
      else
	type = CSS_TOKEN_TYPE_INVALID;
      goto accepted;
    }

  if (c == '/')
    {
      ++cur_pos;
      if (cur_pos < tail_pos && *cur_pos == '*')
	{
	  ++cur_pos;
	  bool escape = false;
	  while (cur_pos < tail_pos)
	    {
	      if (!escape)
		{
		  if (*cur_pos == '\\')
		    escape = true;
		  else if (*cur_pos == '*'
			   && cur_pos + 1 < tail_pos
			   && cur_pos[1] == '/')
		    {
		      cur_pos += 2;
		      break;
		    }
		}
	      else
		escape = true;
	      ++cur_pos;
	    }
	  type = CSS_TOKEN_TYPE_COMMENT;
	}
      else
	type = CSS_TOKEN_TYPE_DELIM;
      goto accepted;
    }

  if (c == '#')
    {
      ++cur_pos;
      len = eat_hash(cur_pos, tail_pos - cur_pos, ident_buf);
      if (len > 0)
	{
	  type = CSS_TOKEN_TYPE_HASH;
	  cur_pos += len;
	}
      else
	type = CSS_TOKEN_TYPE_DELIM;
      goto accepted;
    }

  if (c == '@')
    {
      ++cur_pos;
      len = eat_ident(cur_pos, tail_pos - cur_pos, ident_buf);
      if (len > 0)
	{
	  type = CSS_TOKEN_TYPE_ATKEYWORD;
	  cur_pos += len;
	}
      else
	type = CSS_TOKEN_TYPE_DELIM;
      goto accepted;
    }

  if (c == '.')
    {
      ++cur_pos;
      value_buf.append(1, '.');
      len = eat_number(cur_pos, tail_pos - cur_pos, value_buf);
      if (len > 0)
	{
	  cur_pos += len;
	  c = *cur_pos;
	  goto lexer_state_number_period;
	}
      type = CSS_TOKEN_TYPE_DELIM;
      goto accepted;
    }

  if (c == 'U' && cur_pos + 2 < tail_pos && cur_pos[1] == '\\'
      && is_xdigit(cur_pos[2]))
    {
      ident_buf.append(cur_pos, 3);
      cur_pos += 3;
      for (int hex_len = 1; hex_len <= 6; ++hex_len)
	{
	  if (cur_pos - src >= length || !is_xdigit(*cur_pos))
	    break;
	  ident_buf.append(1, *cur_pos++);
	}
      if (cur_pos + 1 < tail_pos && *cur_pos == '-'
	  && is_xdigit(cur_pos[1]))
	{
	  ident_buf.append(cur_pos, 2);
	  cur_pos += 2;
	  for (int hex_len = 1; hex_len <= 6; ++hex_len)
	    {
	      if (cur_pos - src >= length || !is_xdigit(*cur_pos))
		break;
	      ident_buf.append(1, *cur_pos++);
	    }
	}
      type = CSS_TOKEN_TYPE_UNICODE_RANGE;
      goto accepted;
    }

  len = eat_ident(cur_pos, tail_pos - cur_pos, ident_buf);
  if (len > 0)
    {
      cur_pos += len;
      c = *cur_pos;
      goto lexer_state_ident;
    }

  len = eat_number(cur_pos, tail_pos - cur_pos, value_buf);
  if (len > 0)
    {
      cur_pos += len;
      c = *cur_pos;
      goto lexer_state_number;
    }

  len = count_white_space(cur_pos, tail_pos - cur_pos);
  if (len > 0)
    {
      type = CSS_TOKEN_TYPE_SPACE;
      cur_pos += len;
      goto accepted;
    }

  ++cur_pos;
  if (c == ';')
    type = CSS_TOKEN_TYPE_SEMICOLON;
  else if (c == '{')
    type = CSS_TOKEN_TYPE_LBRACE;
  else if (c == '}')
    type = CSS_TOKEN_TYPE_RBRACE;
  else if (c == '(')
    type = CSS_TOKEN_TYPE_LPAREN;
  else if (c == ')')
    type = CSS_TOKEN_TYPE_RPAREN;
  else if (c == '[')
    type = CSS_TOKEN_TYPE_LBRACKET;
  else if (c == ']')
    type = CSS_TOKEN_TYPE_RBRACKET;
  else if (tail_pos - cur_pos >= 1 && *cur_pos == '=')
    {
      if (c == '~')
	{
	  type = CSS_TOKEN_TYPE_INCLUDES;
	  ++cur_pos;
	}
      else if (c == '|')
	{
	  type = CSS_TOKEN_TYPE_DASHMATCH;
	  ++cur_pos;
	}
      else
	type = CSS_TOKEN_TYPE_DELIM;
    }
  else if (c == '<' && tail_pos - cur_pos >= 3
	   && strncmp(cur_pos, "!--", 3) == 0)
    {
      type = CSS_TOKEN_TYPE_CDO;
      cur_pos += 3;
    }
  else if (c == '-' && tail_pos - cur_pos >= 2
	   && strncmp(cur_pos, "->", 2) == 0)
    {
      type = CSS_TOKEN_TYPE_CDC;
      cur_pos += 2;
    }
  else
    type = CSS_TOKEN_TYPE_DELIM;
  goto accepted;



 lexer_state_ident:
  if (c == '(')
    {
      ++cur_pos;
      if (strcasecmp(ident_buf.c_str(), "uri") == 0)
	{
	  c = *cur_pos;
	  goto lexer_state_uri;
	}
      type = CSS_TOKEN_TYPE_FUNCTION;
    }
  else
    type = CSS_TOKEN_TYPE_IDENT;
  goto accepted;



 lexer_state_number:
  if (c == '.' && cur_pos + 1 < tail_pos)
    {
      c = cur_pos[1];
      if (is_digit(c))
	{
	  ++cur_pos;
	  value_buf.append(1, '.');
	  len = eat_number(cur_pos, tail_pos - cur_pos, value_buf);
	  cur_pos += len;
	}
      if (cur_pos >= tail_pos)
	{
	  type = CSS_TOKEN_TYPE_NUMBER;
	  goto accepted;
	}
      c = *cur_pos;
    }



 lexer_state_number_period:
  if (c == '%')
    {
      ++cur_pos;
      type = CSS_TOKEN_TYPE_PERCENTAGE;
    }
  else
    {
      len = eat_ident(cur_pos, tail_pos - cur_pos, ident_buf);
      if (len > 0)
	{
	  type = CSS_TOKEN_TYPE_DIMENSION;
	  cur_pos += len;
	}
      else
	type = CSS_TOKEN_TYPE_NUMBER;
    }
  goto accepted;



 lexer_state_uri:
  cur_pos += count_white_space(cur_pos, tail_pos - cur_pos);
  if (cur_pos < tail_pos)
    {
      c = *cur_pos;
      if (c == '"')
	{
	  ++cur_pos;
	  while (*cur_pos != '"')
	    {
	      len = eat_string_char(cur_pos, tail_pos - cur_pos, value_buf);
	      if (len > 0)
		{
		  cur_pos += len;
		  continue;
		}
	      break;
	    }
	  if (cur_pos >= tail_pos || *cur_pos != '"')
	    goto lexer_state_uri_invalid;
	  ++cur_pos;
	}
      else if (c == '\'')
	{
	  ++cur_pos;
	  while (*cur_pos != '\'')
	    {
	      len = eat_string_char(cur_pos, tail_pos - cur_pos, value_buf);
	      if (len > 0)
		{
		  cur_pos += len;
		  continue;
		}
	      break;
	    }
	  if (cur_pos >= tail_pos || *cur_pos != '\'')
	    goto lexer_state_uri_invalid;
	  ++cur_pos;
	}
      else
	{
	  len = eat_url(cur_pos, tail_pos - cur_pos, value_buf);
	  if (len == 0)
	    goto lexer_state_uri_invalid;
	  cur_pos += len;
	}
      cur_pos += count_white_space(cur_pos, tail_pos - cur_pos);
      if (cur_pos < tail_pos && *cur_pos == ')')
	{
	  type = CSS_TOKEN_TYPE_URI;
	  ++cur_pos;
	  goto accepted;
	}
    }



 lexer_state_uri_invalid:
  type = CSS_TOKEN_TYPE_FUNCTION;
  cur_pos = src + 4;
  goto accepted;



 accepted:
  return new CSSToken(src, cur_pos - src, type,
		      ident_buf.c_str(), value_buf.c_str());
}


CSSValue::CSSValue(AcceptableValueType acceptable_value_type)
  : value_type(KEYWORD_VALUE),
    acceptable_value_type(acceptable_value_type)
{
  set_initial();
  value_set = false;
}


CSSValue::CSSValue(const CSSValue *src)
  : value_type(KEYWORD_VALUE)
{
  value_set = false;

  assert(src != NULL);
  acceptable_value_type = src->acceptable_value_type;
  set_value(*src);
}


CSSValue::~CSSValue()
{
  free_value();
}


void
CSSValue::free_value()
{
  switch (value_type)
    {
    case URI_VALUE:
    case FONT_STRING_VALUE:
    case COLOR_STRING_VALUE:
    case STRING_VALUE:
    case ATTR_REF_VALUE:
      free(value.string);
      break;

    case COUNTER_DEF_VALUE:
      free(value.counter_def.id);
      break;

    case COUNTER_REF_VALUE:
      free(value.counter_ref.id);
      if (value.counter_ref.string != NULL)
	free(value.counter_ref.string);
      break;

    default:
      break;
    }

  value_type = KEYWORD_VALUE;
}


void
CSSValue::set_initial()
{
  free_value();
  value_type = KEYWORD_VALUE;
  value_set = true;
  switch (acceptable_value_type)
    {
    case CSS_BACKGROUND_COLOR:
      value.keyword = css_transparent;
      return;

    case CSS_BACKGROUND_IMAGE:
      value.keyword = css_none;
      return;

    case CSS_BACKGROUND_REPEAT:
      value.keyword = css_repeat;
      return;

    case CSS_BACKGROUND_ATTACHMENT:
      value.keyword = css_scroll;
      return;

    case CSS_BACKGROUND_POSITION_H:
    case CSS_BACKGROUND_POSITION_V:
      value_type = PERCENTAGE_VALUE;
      value.number = 0.0;
      return;

    case CSS_COLOR:
      value.keyword = css_black;
      return;

    case CSS_CONTENT:
      value.keyword = css_normal;
      return;

    case CSS_COUNTER_RESET:
    case CSS_COUNTER_INCREMENT:
      value.keyword = css_none;
      return;

    case CSS_DIRECTION:
      value.keyword = css_ltr;
      return;

    case CSS_DISPLAY:
      value.keyword = css_inline;
      return;

    case CSS_FONT_FAMILY:
      value.keyword = css_sans_serif;
      return;

    case CSS_FONT_SIZE:
      value.keyword = css_medium;
      return;

    case CSS_FONT_STYLE:
    case CSS_FONT_VARIANT:
    case CSS_FONT_WEIGHT:
    case CSS_LINE_HEIGHT:
      value.keyword = css_normal;
      return;

    case CSS_LIST_STYLE_IMAGE:
      value.keyword = css_none;

    case CSS_LIST_STYLE_POSITION:
      value.keyword = css_outside;
      return;

    case CSS_LIST_STYLE_TYPE:
      value.keyword = css_disc;
      return;

    case CSS_MARGIN_WIDTH:
      value_type = DIMENSION_VALUE;
      value.dimension.value = 0.0;
      value.dimension.unit = css_px;
      return;

    case CSS_TEXT_ALIGN:
      value.keyword = css_left;
      return;

    case CSS_TEXT_DECORATION:
      value.keyword = css_none;
      return;

    case CSS_TEXT_INDENT:
      value_type = DIMENSION_VALUE;
      value.dimension.value = 0.0;
      value.dimension.unit = css_px;
      return;

    case CSS_TEXT_TRANSFORM:
      value.keyword = css_none;
      return;

    case CSS_UNICODE_BIDI:
      value.keyword = css_normal;
      return;

    case CSS_VERTICAL_ALIGN:
      value.keyword = css_baseline;
      return;

    case CSS_WHITE_SPACE:
      value.keyword = css_normal;
      return;
    }
}


void
CSSValue::unset_value()
{
  set_initial();
  value_set = 0;
}


char *
CSSValue::to_string() const
{
  string buf;
  append_string(buf);
  return strdup(buf.c_str());
}


void
CSSValue::append_string(string &buf) const
{
  char tmp_buf[64];
  switch (value_type)
    {
    case INTEGER_VALUE:
      snprintf(tmp_buf, 64, "%d", value.integer);
      buf.append(tmp_buf);
      return;

    case NUMBER_VALUE:
      snprintf(tmp_buf, 64, "%lf", value.number);
      buf.append(tmp_buf);
      return;

    case PERCENTAGE_VALUE:
      snprintf(tmp_buf, 64, "%lf%%", value.number);
      buf.append(tmp_buf);
      return;

    case DIMENSION_VALUE:
      snprintf(tmp_buf, 64, "%lf%s",
	       value.dimension.value, value.dimension.unit);
      buf.append(tmp_buf);
      return;

    case URI_VALUE:
      buf.append("url(").append(value.string).append(")");
      return;

    case KEYWORD_VALUE:
      buf.append(value.keyword);
      return;

    case COLOR_STRING_VALUE:
      buf.append(value.string);
      return;

    case COLOR_RGB_VALUE:
      snprintf(tmp_buf, 64, "#%02x%02x%02x",
	       value.rgb.r, value.rgb.g, value.rgb.b);
      buf.append(tmp_buf);
      return;

    case FONT_STRING_VALUE:
    case STRING_VALUE:
      buf.append("\"").append(value.string).append("\"");
      return;

    case TEXT_DECORATION_VALUE:
      {
	bool first = true;
	if (value.decoration.underline)
	  {
	    buf.append(css_underline);
	    first = false;
	  }
	if (value.decoration.overline)
	  {
	    if (!first)
	      buf.append(" ");
	    buf.append(css_overline);
	    first = false;
	  }
	if (value.decoration.line_through)
	  {
	    if (!first)
	      buf.append(" ");
	    buf.append(css_line_through);
	    first = false;
	  }
	if (value.decoration.blink)
	  {
	    if (!first)
	      buf.append(" ");
	    buf.append(css_blink);
	  }
	return;
      }

    case ATTR_REF_VALUE:
      buf.append("attr(").append(value.string).append(")");
      return;

    case COUNTER_DEF_VALUE:
      buf.append(value.counter_def.id).append(" ");
      snprintf(tmp_buf, 64, "%d", value.counter_def.value);
      buf.append(tmp_buf);
      return;

    case COUNTER_REF_VALUE:
      buf.append("counter(").append(value.counter_ref.id);
      if (value.counter_ref.string != NULL)
	buf.append(", ").append(value.counter_ref.string);
      if (value.counter_ref.style != NULL)
	buf.append(", ").append(value.counter_ref.style);
      buf.append(")");
      return;
    }
}


enum ExpectedValueType {
  E_AUTO = 1 << 0,
  E_NONE = 1 << 1,
  E_NORMAL = 1 << 2,
  E_LENGTH = 1 << 3,
  E_TIME = 1 << 5,
  E_PERCENTAGE = 1 << 7,
  E_NUMBER = 1 << 8,
  E_INTEGER = 1 << 10,
  E_STRING = 1 << 11,
  E_URI = 1 << 12,
  E_COLOR = 1 << 13,
  E_COUNTER_REF = 1 << 14,
  E_COUNTER_DEF = 1 << 15
};


static int expected_value_types[] = {
  E_COLOR,					// CSS_BACKGROUND_COLOR
  E_NONE|E_URI,					// CSS_BACKGROUND_IMAGE
  0,						// CSS_BACKGROUND_REPEAT
  0,						// CSS_BACKGROUND_ATTACHMENT
  E_LENGTH|E_PERCENTAGE,			// CSS_BACKGROUND_POSITION_H
  E_LENGTH|E_PERCENTAGE,			// CSS_BACKGROUND_POSITION_V
  E_COLOR,					// CSS_COLOR
  E_NORMAL|E_NONE|E_STRING|E_URI|E_COUNTER_REF,	// CSS_CONTENT
  E_COUNTER_DEF|E_NONE,				// CSS_COUNTER_RESET
  E_COUNTER_DEF|E_NONE,				// CSS_COUNTER_INCREMENT
  0,						// CSS_DIRECTION
  E_NONE,					// CSS_DISPLAY
  E_STRING,					// CSS_FONT_FAMILY
  E_LENGTH|E_PERCENTAGE,			// CSS_FONT_SIZE
  E_NORMAL,					// CSS_FONT_STYLE
  E_NORMAL,					// CSS_FONT_VARIANT
  E_NORMAL,					// CSS_FONT_WEIGHT
  E_NORMAL|E_NUMBER|E_LENGTH|E_PERCENTAGE,	// CSS_LINE_HEIGHT
  E_URI|E_NONE,					// CSS_LIST_STYLE_IMAGE
  0,						// CSS_LIST_STYLE_POSITION
  E_NONE,					// CSS_LIST_STYLE_TYPE
  E_LENGTH|E_PERCENTAGE|E_AUTO,			// CSS_MARGIN_WIDTH
  0,						// CSS_TEXT_ALIGN
  E_NONE,					// CSS_TEXT_DECORATION
  E_LENGTH|E_PERCENTAGE,			// CSS_TEXT_INDENT
  E_NONE,					// CSS_TEXT_TRANSFORM
  E_NORMAL,					// CSS_UNICODE_BIDI
  E_LENGTH|E_PERCENTAGE,			// CSS_VERTICAL_ALIGN
  E_NORMAL					// CSS_WHITE_SPACE
};


static unsigned char
clip_rgb_value(bool minus, const CSSToken *token)
{
  if (minus)
    return 0;

  unsigned int value;
  if (token->is_percentage())
    {
      double percentage = token->get_percentage();
      value = static_cast<unsigned int>(255 * (percentage / 100.0));
    }
  else
    {
      assert(token->is_number());
      value = token->get_int_number();
    }
  if (value > 255)
    value = 255;

  return static_cast<unsigned char>(value);
}


bool
CSSValue::set_value(const CSSNode *src)
{
  if (!src->is_any())
    return false;

  const CSSNodeAny *any = dynamic_cast<const CSSNodeAny *>(src);
  assert(any != NULL);

  const CSSToken *token = any->get_leader();
  CSSTokenType type = token->get_type();
  int expected_value_type = expected_value_types[acceptable_value_type];

  if (type == CSS_TOKEN_TYPE_PERCENTAGE)
    {
      if ((expected_value_type & E_PERCENTAGE) != 0)
	{
	  free_value();
	  value_type = PERCENTAGE_VALUE;
	  value.number = token->get_percentage();
	  value_set = true;
	  return true;
	}
      return false;
    }

  if (type == CSS_TOKEN_TYPE_DIMENSION)
    {
      const char *token_unit = token->get_dimension_unit();
      const char *unit = NULL;
      if ((expected_value_type & E_LENGTH) != 0)
	{
	  if (strcmp(token_unit, css_em) == 0)
	    unit = css_em;
	  else if (strcmp(token_unit, css_ex) == 0)
	    unit = css_ex;
	  else if (strcmp(token_unit, css_px) == 0)
	    unit = css_px;
	  else if (strcmp(token_unit, css_in) == 0)
	    unit = css_in;
	  else if (strcmp(token_unit, css_cm) == 0)
	    unit = css_cm;
	  else if (strcmp(token_unit, css_mm) == 0)
	    unit = css_mm;
	  else if (strcmp(token_unit, css_pt) == 0)
	    unit = css_pt;
	  else if (strcmp(token_unit, css_pc) == 0)
	    unit = css_pc;
	  if (unit != NULL)
	    {
	      free_value();
	      value_type = DIMENSION_VALUE;
	      value.dimension.value = token->get_dimension_value();
	      value.dimension.unit = unit;
	      value_set = true;
	      return true;
	    }
	}
      if ((expected_value_type & E_TIME) != 0)
	{
	  if (strcmp(token_unit, css_ms) == 0)
	    unit = css_ms;
	  else if (strcmp(token_unit, css_s) == 0)
	    unit = css_s;
	  if (unit != NULL)
	    {
	      free_value();
	      value_type = DIMENSION_VALUE;
	      value.dimension.value = token->get_dimension_value();
	      value.dimension.unit = unit;
	      value_set = true;
	      return true;
	    }
	}
      return false;
    }

  if (type == CSS_TOKEN_TYPE_NUMBER)
    {
      if ((expected_value_type & E_LENGTH) != 0
	  && token->get_number() == 0.0)
	{
	  free_value();
	  value_type = DIMENSION_VALUE;
	  value.dimension.value = 0.0;
	  value.dimension.unit = css_px;
	  value_set = true;
	  return true;
	}
      if ((expected_value_type & E_INTEGER) != 0)
	{
	  free_value();
	  value_type = INTEGER_VALUE;
	  value.integer = token->get_int_number();
	  value_set = true;
	  return true;
	}
      if (acceptable_value_type == CSS_FONT_WEIGHT)
	{
	  int weight = token->get_int_number();
	  switch (weight)
	    {
	    case 100: case 200: case 300: case 400: case 500:
	    case 600: case 700: case 800: case 900:
	      break;

	    default:
	      weight = 0;
	    }
	  if (weight != 0)
	    {
	      free_value();
	      value_type = INTEGER_VALUE;
	      value.integer = weight;
	      value_set = true;
	      return true;
	    }
	  return false;
	}
      if ((expected_value_type & E_NUMBER) != 0)
	{
	  free_value();
	  value_type = NUMBER_VALUE;
	  value.number = token->get_number();
	  value_set = true;
	  return true;
	}
    }

  if (type == CSS_TOKEN_TYPE_HASH)
    {
      if ((expected_value_type & E_COLOR) != 0)
	{
	  const char *hash = token->get_hash();
	  int len = strlen(hash);
	  if (len == 3 || len == 6)
	    {
	      int i;
	      for (i = 0; i < len; i++)
		if (!is_xdigit(hash[i]))
		  break;
	      if (i == len)
		{
		  string buf("#");
		  buf.append(hash);
		  free_value();
		  value_type = COLOR_STRING_VALUE;
		  value.string = strdup(buf.c_str());
		  value_set = true;
		  return true;
		}
	    }
	}
      return false;
    }

  if (type == CSS_TOKEN_TYPE_STRING)
    {
      if ((expected_value_type & E_STRING) != 0)
	{
	  free_value();
	  value_type = STRING_VALUE;
	  value.string = strdup(token->get_string());
	  value_set = true;
	  return true;
	}
      return false;
    }

  if (type == CSS_TOKEN_TYPE_URI)
    {
      if ((expected_value_type & E_URI) != 0)
	{
	  free_value();
	  value_type = URI_VALUE;
	  value.string = strdup(token->get_string());
	  value_set = true;
	  return true;
	}
      return false;
    }

  if (type == CSS_TOKEN_TYPE_FUNCTION)
    {
      switch (acceptable_value_type)
	{
	case CSS_BACKGROUND_COLOR:
	case CSS_COLOR:
	  if (strcmp(token->get_function(), "rgb") == 0)
	    {
	      const vector<CSSNodeAny *> &any_any = any->get_any();
	      bool minus_sign[3];
	      const CSSToken *rgb[3];
	      int i = 0;
	      bool expect_comma = false;
	      bool minus = false;
	      for (size_t j = 0; i < 3 && j < any_any.size(); j++)
		{
		  const CSSNodeAny *val = any_any[j];
		  token = val->get_leader();
		  type = token->get_type();
		  if (!expect_comma)
		    {
		      if (type == CSS_TOKEN_TYPE_NUMBER
			  || type == CSS_TOKEN_TYPE_PERCENTAGE)
			{
			  minus_sign[i] = minus;
			  rgb[i++] = val->get_leader();
			  expect_comma = true;
			  minus = false;
			  continue;
			}
		      if (type == CSS_TOKEN_TYPE_DELIM)
			{
			  if (token->get_delim() == '-')
			    minus = !minus;
			  else if (token->get_delim() != '+')
			    return false;	// ʥǥߥ
			}
		    }
		  else
		    {
		      if (type == CSS_TOKEN_TYPE_DELIM
			  && token->get_delim() == ',')
			{
			  expect_comma = false;
			  continue;
			}
		    }
		  return false;	// rgb
		}
	      if (i == 3)
		{
		  free_value();
		  value_type = COLOR_RGB_VALUE;
		  value.rgb.r = clip_rgb_value(minus_sign[0], rgb[0]);
		  value.rgb.g = clip_rgb_value(minus_sign[1], rgb[1]);
		  value.rgb.b = clip_rgb_value(minus_sign[2], rgb[2]);
		  return true;
		}
	    }
	  return false;	// ͽʤfunction

	case CSS_CONTENT:
	  if (strcmp(token->get_function(), "attr") == 0)
	    {
	      const vector<CSSNodeAny *> &any_any = any->get_any();
	      if (any_any.size() == 1)
		{
		  const CSSNodeAny *id_node = any_any[0];
		  if (id_node->is_term())
		    {
		      const CSSToken *id = id_node->get_term();
		      if (id->is_ident())
			{
			  free_value();
			  value_type = ATTR_REF_VALUE;
			  value.string = strdup(id->get_ident());
			  return true;
			}
		    }
		}
	      return false;
	    }
	  if (strcmp(token->get_function(), "counter") == 0)
	    {
	      const vector<CSSNodeAny *> &any_any = any->get_any();
	      const char *id = NULL;
	      const char *style;
	      if (any_any.size() == 0)
		return false;

	      const CSSNodeAny *id_node = any_any[0];
	      if (id_node->is_term())
		{
		  token = id_node->get_leader();
		  if (token->is_ident())
		    id = token->get_ident();
		  else
		    return false;
		}
	      else
		return false;

	      if (any_any.size() == 1)
		style = css_decimal;
	      else if (any_any.size() == 3)
		{
		  token = any_any[1]->get_leader();
		  if (token->is_delim() && token->get_delim() == ',')
		    {
		      token = any_any[2]->get_leader();
		      if (token->is_ident())
			{
			  style = token->get_list_style_type();
			  if (style == NULL)
			    return false;
			}
		      else
			return false;
		    }
		  else
		    return false;
		}
	      else
		return false;

	      free_value();
	      value_type = COUNTER_REF_VALUE;
	      value.counter_ref.id = strdup(id);
	      value.counter_ref.string = NULL;
	      value.counter_ref.style = style;
	      return true;
	    }
	  if (strcmp(token->get_function(), "counters") == 0)
	    {
	      const vector<CSSNodeAny *> &any_any = any->get_any();
	      const char *id = NULL;
	      const char *str = NULL;
	      const char *style;
	      if (any_any.size() < 3)
		return false;

	      const CSSNodeAny *tmp_node = any_any[0];
	      if (tmp_node->is_term())
		{
		  token = tmp_node->get_leader();
		  if (token->is_ident())
		    id = token->get_ident();
		  else
		    return false;
		}
	      else
		return false;

	      tmp_node = any_any[1];
	      if (tmp_node->is_term())
		{
		  token = tmp_node->get_leader();
		  if (!token->is_delim() || token->get_delim() != ',')
		    return false;
		}
	      else
		return false;

	      tmp_node = any_any[2];
	      if (tmp_node->is_term())
		{
		  token = tmp_node->get_leader();
		  if (token->is_string())
		    str = token->get_string();
		  else
		    return false;
		}
	      else
		return false;

	      if (any_any.size() == 3)
		style = css_decimal;
	      else if (any_any.size() == 5)
		{
		  token = any_any[3]->get_leader();
		  if (token->is_delim() && token->get_delim() == ',')
		    {
		      token = any_any[4]->get_leader();
		      if (token->is_ident())
			{
			  style = token->get_list_style_type();
			  if (style == NULL)
			    return false;
			}
		      else
			return false;
		    }
		  else
		    return false;
		}
	      else
		return false;

	      free_value();
	      value_type = COUNTER_REF_VALUE;
	      value.counter_ref.id = strdup(id);
	      value.counter_ref.string = strdup(str);
	      value.counter_ref.style = style;
	      return true;
	    }

	default:
	  break;
	}
      return false;
    }

  if (type != CSS_TOKEN_TYPE_IDENT)
    return false;

  const char *ident = token->get_ident();
  const char *keyword = NULL;
  if (strcmp(ident, css_inherit) == 0)
    keyword = css_inherit;
  else if (strcmp(ident, css_auto) == 0)
    {
      if ((expected_value_type & E_AUTO) != 0)
	keyword = css_auto;
    }
  else if (strcmp(ident, css_none) == 0)
    {
      if ((expected_value_type & E_NONE) != 0)
	keyword = css_none;
    }
  else if (strcmp(ident, css_normal) == 0)
    {
      if ((expected_value_type & E_NORMAL) != 0)
	keyword = css_normal;
    }
  else
    {
      switch (acceptable_value_type)
	{
	case CSS_BACKGROUND_REPEAT:
	  if (strcmp(ident, css_repeat) == 0)
	    keyword = css_repeat;
	  else if (strcmp(ident, css_repeat_x) == 0)
	    keyword = css_repeat_x;
	  else if (strcmp(ident, css_repeat_y) == 0)
	    keyword = css_repeat_y;
	  else if (strcmp(ident, css_no_repeat) == 0)
	    keyword = css_no_repeat;
	  break;

	case CSS_BACKGROUND_ATTACHMENT:
	  if (strcmp(ident, css_scroll) == 0)
	    keyword = css_scroll;
	  else if (strcmp(ident, css_fixed) == 0)
	    keyword = css_fixed;
	  break;

	case CSS_BACKGROUND_POSITION_H:
	  if (strcmp(ident, css_left) == 0)
	    keyword = css_left;
	  else if (strcmp(ident, css_center) == 0)
	    keyword = css_center;
	  else if (strcmp(ident, css_right) == 0)
	    keyword = css_right;
	  break;

	case CSS_BACKGROUND_POSITION_V:
	  if (strcmp(ident, css_top) == 0)
	    keyword = css_top;
	  else if (strcmp(ident, css_center) == 0)
	    keyword = css_center;
	  else if (strcmp(ident, css_bottom) == 0)
	    keyword = css_bottom;
	  break;

	case CSS_BACKGROUND_COLOR:
	  if (strcmp(ident, css_transparent) == 0)
	    {
	      keyword = css_transparent;
	      break;
	    }
	  /* FALLTHROUGH */
	case CSS_COLOR:
	  if (strcmp(ident, css_maroon) == 0)
	    keyword = css_maroon;
	  else if (strcmp(ident, css_red) == 0)
	    keyword = css_red;
	  else if (strcmp(ident, css_orange) == 0)
	    keyword = css_orange;
	  else if (strcmp(ident, css_yellow) == 0)
	    keyword = css_yellow;
	  else if (strcmp(ident, css_olive) == 0)
	    keyword = css_olive;
	  else if (strcmp(ident, css_purple) == 0)
	    keyword = css_purple;
	  else if (strcmp(ident, css_fuchsia) == 0)
	    keyword = css_fuchsia;
	  else if (strcmp(ident, css_white) == 0)
	    keyword = css_white;
	  else if (strcmp(ident, css_lime) == 0)
	    keyword = css_lime;
	  else if (strcmp(ident, css_green) == 0)
	    keyword = css_green;
	  else if (strcmp(ident, css_navy) == 0)
	    keyword = css_navy;
	  else if (strcmp(ident, css_blue) == 0)
	    keyword = css_blue;
	  else if (strcmp(ident, css_aqua) == 0)
	    keyword = css_aqua;
	  else if (strcmp(ident, css_teal) == 0)
	    keyword = css_teal;
	  else if (strcmp(ident, css_black) == 0)
	    keyword = css_black;
	  else if (strcmp(ident, css_silver) == 0)
	    keyword = css_silver;
	  else if (strcmp(ident, css_gray) == 0)
	    keyword = css_gray;
	  else
	    {
	      free_value();
	      value_type = COLOR_STRING_VALUE;
	      value.string = strdup(ident);
	      value_set = true;
	      return true;
	    }
	  break;

	case CSS_CONTENT:
	  // contentץѥƥλϰ̤ʣͤϢ뤷ΤȤʤꡢ
	  // ġͤñΥȡɽȤϸ¤ʤǤϡ
	  // ñȡɽΤ᤹롣
	  if (strcmp(ident, css_open_quote) == 0)
	    keyword = css_open_quote;
	  else if (strcmp(ident, css_close_quote) == 0)
	    keyword = css_close_quote;
	  else if (strcmp(ident, css_no_open_quote) == 0)
	    keyword = css_no_open_quote;
	  else if (strcmp(ident, css_no_close_quote) == 0)
	    keyword = css_no_close_quote;
	  break;

	case CSS_DIRECTION:
	  if (strcmp(ident, css_ltr) == 0)
	    keyword = css_ltr;
	  else if (strcmp(ident, css_rtl) == 0)
	    keyword = css_rtl;
	  break;

	case CSS_DISPLAY:
	  if (strcmp(ident, css_inline) == 0)
	    keyword = css_inline;
	  else if (strcmp(ident, css_block) == 0)
	    keyword = css_block;
	  else if (strcmp(ident, css_list_item) == 0)
	    keyword = css_list_item;
	  else if (strcmp(ident, css_run_in) == 0)
	    keyword = css_run_in;
	  else if (strcmp(ident, css_inline_block) == 0)
	    keyword = css_inline_block;
	  else if (strcmp(ident, css_table) == 0)
	    keyword = css_table;
	  else if (strcmp(ident, css_inline_table) == 0)
	    keyword = css_inline_table;
	  else if (strcmp(ident, css_table_row_group) == 0)
	    keyword = css_table_row_group;
	  else if (strcmp(ident, css_table_header_group) == 0)
	    keyword = css_table_header_group;
	  else if (strcmp(ident, css_table_footer_group) == 0)
	    keyword = css_table_footer_group;
	  else if (strcmp(ident, css_table_row) == 0)
	    keyword = css_table_row;
	  else if (strcmp(ident, css_table_column_group) == 0)
	    keyword = css_table_column_group;
	  else if (strcmp(ident, css_table_column) == 0)
	    keyword = css_table_column;
	  else if (strcmp(ident, css_table_cell) == 0)
	    keyword = css_table_cell;
	  else if (strcmp(ident, css_table_caption) == 0)
	    keyword = css_table_caption;
	  break;

	case CSS_FONT_FAMILY:
	  // font-familyλϥȡĤȤϸ¤ʤΤա
	  // () { font-family: serif monospace, fantasy }
	  // ȤĤΥեȤfont-family˻ꤹ롣
	  // ʶ路ܤserifmonospaceȤĤident
	  // ñǷҤ"serif monospace"Ȥե̾Ȥ
	  // ᤷʤФʤʤܤfantasyľCSS
	  // generic-familyfantasyȤƼ갷ɬפ롣
	  // ãΤñȡǰĤfont-family
	  // 뤳Ȥ餫ˤʤäǤ뤳Ȥcallerݾڤ뤳ȡ
	  keyword = token->get_generic_family();
	  if (keyword == NULL)
	    {
	      free_value();
	      value_type = FONT_STRING_VALUE;
	      value.string = strdup(ident);
	      value_set = true;
	      return true;
	    }
	  break;

	case CSS_FONT_SIZE:
	  if (strcmp(ident, css_xx_small) == 0)
	    keyword = css_xx_small;
	  else if (strcmp(ident, css_x_small) == 0)
	    keyword = css_x_small;
	  else if (strcmp(ident, css_small) == 0)
	    keyword = css_small;
	  else if (strcmp(ident, css_medium) == 0)
	    keyword = css_medium;
	  else if (strcmp(ident, css_large) == 0)
	    keyword = css_large;
	  else if (strcmp(ident, css_x_large) == 0)
	    keyword = css_x_large;
	  else if (strcmp(ident, css_xx_large) == 0)
	    keyword = css_xx_large;
	  else if (strcmp(ident, css_smaller) == 0)
	    keyword = css_smaller;
	  else if (strcmp(ident, css_larger) == 0)
	    keyword = css_larger;
	  break;

	case CSS_FONT_STYLE:
	  if (strcmp(ident, css_italic) == 0)
	    keyword = css_italic;
	  else if (strcmp(ident, css_oblique) == 0)
	    keyword = css_oblique;
	  break;

	case CSS_FONT_VARIANT:
	  if (strcmp(ident, css_small_caps) == 0)
	    keyword = css_small_caps;
	  break;

	case CSS_FONT_WEIGHT:
	  if (strcmp(ident, css_bold) == 0)
	    keyword = css_bold;
	  else if (strcmp(ident, css_bolder) == 0)
	    keyword = css_bolder;
	  else if (strcmp(ident, css_lighter) == 0)
	    keyword = css_lighter;
	  break;

	case CSS_LIST_STYLE_POSITION:
	  if (strcmp(ident, css_inside) == 0)
	    keyword = css_inside;
	  else if (strcmp(ident, css_outside) == 0)
	    keyword = css_outside;
	  break;

	case CSS_LIST_STYLE_TYPE:
	  keyword = token->get_list_style_type();
	  break;

	case CSS_TEXT_ALIGN:
	  if (strcmp(ident, css_left) == 0)
	    keyword = css_left;
	  else if (strcmp(ident, css_right) == 0)
	    keyword = css_right;
	  else if (strcmp(ident, css_center) == 0)
	    keyword = css_center;
	  else if (strcmp(ident, css_justify) == 0)
	    keyword = css_justify;
	  break;

	case CSS_TEXT_TRANSFORM:
	  if (strcmp(ident, css_capitalize) == 0)
	    keyword = css_capitalize;
	  else if (strcmp(ident, css_uppercase) == 0)
	    keyword = css_uppercase;
	  else if (strcmp(ident, css_lowercase) == 0)
	    keyword = css_lowercase;
	  break;

	case CSS_UNICODE_BIDI:
	  if (strcmp(ident, css_embed) == 0)
	    keyword = css_embed;
	  else if (strcmp(ident, css_bidi_override) == 0)
	    keyword = css_bidi_override;
	  break;

	case CSS_VERTICAL_ALIGN:
	  if (strcmp(ident, css_baseline) == 0)
	    keyword = css_baseline;
	  else if (strcmp(ident, css_sub) == 0)
	    keyword = css_sub;
	  else if (strcmp(ident, css_super) == 0)
	    keyword = css_super;
	  else if (strcmp(ident, css_top) == 0)
	    keyword = css_top;
	  else if (strcmp(ident, css_text_top) == 0)
	    keyword = css_text_top;
	  else if (strcmp(ident, css_middle) == 0)
	    keyword = css_middle;
	  else if (strcmp(ident, css_bottom) == 0)
	    keyword = css_bottom;
	  else if (strcmp(ident, css_text_bottom) == 0)
	    keyword = css_text_bottom;
	  break;

	case CSS_WHITE_SPACE:
	  if (strcmp(ident, css_pre) == 0)
	    keyword = css_pre;
	  else if (strcmp(ident, css_nowrap) == 0)
	    keyword = css_nowrap;
	  else if (strcmp(ident, css_pre_wrap) == 0)
	    keyword = css_pre_wrap;
	  else if (strcmp(ident, css_pre_line) == 0)
	    keyword = css_pre_line;
	  break;

	default:
	  break;
	}
    }

  if (keyword != NULL)
    {
      free_value();
      value_type = KEYWORD_VALUE;
      value.keyword = keyword;
      value_set = true;
      return true;
    }
  return false;
}


bool
CSSValue::set_value(const CSSValue &src)
{
  if (acceptable_value_type != src.acceptable_value_type)
    return false;

  free_value();

  value_type = src.value_type;
  value_set = src.value_set;

  if (!value_set)
    return false;

  switch (value_type)
    {
    case INTEGER_VALUE:
      value.integer = src.value.integer;
      break;

    case NUMBER_VALUE:
    case PERCENTAGE_VALUE:
      value.number = src.value.number;
      break;

    case DIMENSION_VALUE:
      value.dimension.value = src.value.dimension.value;
      value.dimension.unit = src.value.dimension.unit;
      break;

    case KEYWORD_VALUE:
      value.keyword = src.value.keyword;
      break;

    case URI_VALUE:
    case FONT_STRING_VALUE:
    case COLOR_STRING_VALUE:
    case STRING_VALUE:
    case ATTR_REF_VALUE:
      value.string = strdup(src.value.string);
      break;

    case COLOR_RGB_VALUE:
      value.rgb.r = src.value.rgb.r;
      value.rgb.g = src.value.rgb.g;
      value.rgb.b = src.value.rgb.b;
      break;

    case TEXT_DECORATION_VALUE:
      value.decoration.underline = src.value.decoration.underline;
      value.decoration.overline = src.value.decoration.overline;
      value.decoration.line_through = src.value.decoration.line_through;
      value.decoration.blink = src.value.decoration.blink;
      break;

    case COUNTER_DEF_VALUE:
      value.counter_def.id = strdup(src.value.counter_def.id);
      value.counter_def.value = src.value.counter_def.value;
      break;

    case COUNTER_REF_VALUE:
      value.counter_ref.id = strdup(src.value.counter_ref.id);
      if (src.value.counter_ref.string != NULL)
	value.counter_ref.string = strdup(src.value.counter_ref.string);
      else
	value.counter_ref.string = NULL;
      value.counter_ref.style = src.value.counter_ref.style;
      break;
    }

  return true;
}


bool
CSSValue::set_value(vector<CSSNode *>::const_iterator &iter,
		    vector<CSSNode *>::const_iterator &end_iter)
{
  // Ǥ
  // font-family
  // counter-increment
  //   <identifier><integer>?
  // counter-reset
  //   <identifier><integer>?
  // text-decoration
  // ʤɤʬɤߡͤ⤷ޤiterʤ롣
  // ʾʳξreturn set_value(*iter);롣
  assert(iter != end_iter);

  const CSSNode *node = *iter++;
  if (!node->is_any())
    return false;	// ͽʤ

  if (acceptable_value_type == CSS_COUNTER_RESET
      || acceptable_value_type == CSS_COUNTER_INCREMENT)
    {
      const CSSNodeAny *any = dynamic_cast<const CSSNodeAny *>(node);
      assert(any != NULL);

      const CSSToken *token = any->get_leader();
      if (token->is_ident())
	{
	  const char *id = token->get_ident();
	  bool has_counter_value = false;
	  int counter_value = 0;
	  if (iter != end_iter)
	    {
	      node = *iter;
	      if (node->is_any())
		{
		  any = dynamic_cast<const CSSNodeAny *>(node);
		  assert(any != NULL);

		  token = any->get_leader();
		  bool minus = false;
		  if (token->is_delim() && token->get_delim() == '-')
		    {
		      minus = true;
		      if (iter + 1 != end_iter)
			{
			  node = iter[1];
			  if (node->is_any())
			    {
			      any = dynamic_cast<const CSSNodeAny *>(node);
			      assert(any != NULL);
			      token = any->get_leader();
			    }
			}
		    }
		  if (token->is_number())
		    {
		      has_counter_value = true;
		      if (!minus)
			{
			  counter_value = token->get_int_number();
			  ++iter;
			}
		      else
			{
			  counter_value = -(token->get_int_number());
			  iter += 2;
			}
		    }
		}
	    }
	  if (!has_counter_value
	      && acceptable_value_type == CSS_COUNTER_INCREMENT)
	    counter_value = 1;

	  free_value();
	  value_type = COUNTER_DEF_VALUE;
	  value.counter_def.id = strdup(id);
	  value.counter_def.value = counter_value;
	  value_set = true;
	  return true;
	}
      return false;	// ͽʤ
    }
  else if (acceptable_value_type == CSS_FONT_FAMILY)
    {
      // nodeӥǥߥ','ޤǤΥȡҤʸ
      // Ĥfont-familyˤʤ롣[ident|string][ident|string]
      // Ϣ³' '³롣
      vector<const CSSToken *> words;

      const CSSNodeAny *any = dynamic_cast<const CSSNodeAny *>(node);
      assert(any != NULL);
      
      const CSSToken *token = any->get_leader();
      while (!token->is_delim() || token->get_delim() != ',')
	{
	  words.push_back(token);

	  if (iter == end_iter)
	    break;

	  node = *iter++;
	  if (!node->is_any())
	    return false;	// ͽʤ
	  any = dynamic_cast<const CSSNodeAny *>(node);
	  assert(any != NULL);
	  token = any->get_leader();
	}

      if (words.size() == 0)
	return false;

      if (words.size() == 1)
	{
	  token = words[0];
	  const char *generic_family = token->get_generic_family();
	  if (generic_family != NULL)
	    {
	      free_value();
	      value_type = KEYWORD_VALUE;
	      value.keyword = generic_family;
	      value_set = true;
	      return true;
	    }
	}

      string buf;
      for (size_t i = 0; i < words.size(); i++)
	words[i]->append_as_string(buf);

      if (strlen(buf.c_str()) == 0)
	return false;

      free_value();
      value_type = FONT_STRING_VALUE;
      value.string = strdup(buf.c_str());
      value_set = true;
      return true;
    }
  else if (acceptable_value_type == CSS_TEXT_DECORATION)
    {
      const CSSNodeAny *any = dynamic_cast<const CSSNodeAny *>(node);
      assert(any != NULL);
      
      const CSSToken *token = any->get_leader();
      if (!token->is_ident())
	return false;	// ͽʤ

      const char *ident = token->get_ident();
      if (strcmp(ident, css_inherit) == 0
	  || strcmp(ident, css_none) == 0)
	return set_value(node);

      bool underline = false;
      bool overline = false;
      bool line_through = false;
      bool blink = false;

      while (true)
	{
	  if (strcmp(ident, css_underline) == 0)
	    underline = true;
	  else if (strcmp(ident, css_overline) == 0)
	    overline = true;
	  else if (strcmp(ident, css_line_through) == 0)
	    line_through = true;
	  else if (strcmp(ident, css_blink) == 0)
	    blink = true;
	  
	  if (iter == end_iter)
	    break;
	  node = *iter++;
	  if (!node->is_any())
	    return false;	// ͽʤ
	  any = dynamic_cast<const CSSNodeAny *>(node);
	  token = any->get_leader();
	  if (!token->is_ident())
	    return false;	// ͽʤ
	  ident = token->get_ident();
	};

      if (!underline && !overline && !line_through && !blink)
	{
	  set_initial();
	  return true;
	}

      free_value();
      value_type = TEXT_DECORATION_VALUE;
      value.decoration.underline = underline;
      value.decoration.overline = overline;
      value.decoration.line_through = line_through;
      value.decoration.blink = blink;
      value_set = true;
      return true;
    }

  return set_value(node);
}


bool
CSSValue::set_keyword(CSSKeyword val)
{
  free_value();
  value_type = KEYWORD_VALUE;
  value.keyword = val;
  value_set = true;
  return true;
}


bool
CSSValue::set_integer(int val)
{
  if ((expected_value_types[acceptable_value_type] & E_INTEGER) != 0)
    {
      free_value();
      value_type = INTEGER_VALUE;
      value.integer = val;
      value_set = true;
      return true;
    }
  if (acceptable_value_type == CSS_FONT_WEIGHT)
    {
      int weight = val;
      switch (weight)
	{
	case 100: case 200: case 300: case 400: case 500:
	case 600: case 700: case 800: case 900:
	  break;

	default:
	  weight = 0;
	}
      if (weight != 0)
	{
	  free_value();
	  value_type = INTEGER_VALUE;
	  value.integer = weight;
	  value_set = true;
	  return true;
	}
    }
  return false;
}


bool
CSSValue::set_dimension(double val, const char *unit)
{
  if ((expected_value_types[acceptable_value_type] & (E_LENGTH|E_TIME)) != 0)
    {
      free_value();
      value_type = DIMENSION_VALUE;
      value.dimension.value = val;
      value.dimension.unit = unit;
      value_set = true;
      return true;
    }
  return false;
}


bool
CSSValue::set_percentage(double val)
{
  if ((expected_value_types[acceptable_value_type] & E_PERCENTAGE) != 0)
    {
      free_value();
      value_type = PERCENTAGE_VALUE;
      value.number = val;
      value_set = true;
      return true;
    }
  return false;
}


bool
CSSValue::set_color_string(const char *val)
{
  if ((expected_value_types[acceptable_value_type] & E_COLOR) != 0)
    {
      free_value();
      value_type = COLOR_STRING_VALUE;
      value.string = strdup(val);
      value_set = true;
      return true;
    }
  return false;
}


CSSStyle::CSSStyle(const CSSNodeBlock *declaration_block, bool important)
  : margin_top(CSSValue::CSS_MARGIN_WIDTH),
    margin_right(CSSValue::CSS_MARGIN_WIDTH),
    margin_bottom(CSSValue::CSS_MARGIN_WIDTH),
    margin_left(CSSValue::CSS_MARGIN_WIDTH),
    display(CSSValue::CSS_DISPLAY),
    direction(CSSValue::CSS_DIRECTION),
    line_height(CSSValue::CSS_LINE_HEIGHT),
    vertical_align(CSSValue::CSS_VERTICAL_ALIGN),
    list_style_image(CSSValue::CSS_LIST_STYLE_IMAGE),
    list_style_position(CSSValue::CSS_LIST_STYLE_POSITION),
    list_style_type(CSSValue::CSS_LIST_STYLE_TYPE),
    color(CSSValue::CSS_COLOR),
    background_color(CSSValue::CSS_BACKGROUND_COLOR),
    background_image(CSSValue::CSS_BACKGROUND_IMAGE),
    background_repeat(CSSValue::CSS_BACKGROUND_REPEAT),
    background_attachment(CSSValue::CSS_BACKGROUND_ATTACHMENT),
    background_position_h(CSSValue::CSS_BACKGROUND_POSITION_H),
    background_position_v(CSSValue::CSS_BACKGROUND_POSITION_V),
    font_style(CSSValue::CSS_FONT_STYLE),
    font_variant(CSSValue::CSS_FONT_VARIANT),
    font_weight(CSSValue::CSS_FONT_WEIGHT),
    font_size(CSSValue::CSS_FONT_SIZE),
    text_indent(CSSValue::CSS_TEXT_INDENT),
    text_align(CSSValue::CSS_TEXT_ALIGN),
    text_decoration(CSSValue::CSS_TEXT_DECORATION),
    text_transform(CSSValue::CSS_TEXT_TRANSFORM),
    unicode_bidi(CSSValue::CSS_UNICODE_BIDI),
    white_space(CSSValue::CSS_WHITE_SPACE),
    important_style(NULL)
{
  assert(declaration_block->is_declaration_block());
  assert(important);
  initialize_style(declaration_block, true);
}


CSSStyle::CSSStyle(const CSSNodeBlock *declaration_block)
  : margin_top(CSSValue::CSS_MARGIN_WIDTH),
    margin_right(CSSValue::CSS_MARGIN_WIDTH),
    margin_bottom(CSSValue::CSS_MARGIN_WIDTH),
    margin_left(CSSValue::CSS_MARGIN_WIDTH),
    display(CSSValue::CSS_DISPLAY),
    direction(CSSValue::CSS_DIRECTION),
    line_height(CSSValue::CSS_LINE_HEIGHT),
    vertical_align(CSSValue::CSS_VERTICAL_ALIGN),
    list_style_image(CSSValue::CSS_LIST_STYLE_IMAGE),
    list_style_position(CSSValue::CSS_LIST_STYLE_POSITION),
    list_style_type(CSSValue::CSS_LIST_STYLE_TYPE),
    color(CSSValue::CSS_COLOR),
    background_color(CSSValue::CSS_BACKGROUND_COLOR),
    background_image(CSSValue::CSS_BACKGROUND_IMAGE),
    background_repeat(CSSValue::CSS_BACKGROUND_REPEAT),
    background_attachment(CSSValue::CSS_BACKGROUND_ATTACHMENT),
    background_position_h(CSSValue::CSS_BACKGROUND_POSITION_H),
    background_position_v(CSSValue::CSS_BACKGROUND_POSITION_V),
    font_style(CSSValue::CSS_FONT_STYLE),
    font_variant(CSSValue::CSS_FONT_VARIANT),
    font_weight(CSSValue::CSS_FONT_WEIGHT),
    font_size(CSSValue::CSS_FONT_SIZE),
    text_indent(CSSValue::CSS_TEXT_INDENT),
    text_align(CSSValue::CSS_TEXT_ALIGN),
    text_decoration(CSSValue::CSS_TEXT_DECORATION),
    text_transform(CSSValue::CSS_TEXT_TRANSFORM),
    unicode_bidi(CSSValue::CSS_UNICODE_BIDI),
    white_space(CSSValue::CSS_WHITE_SPACE),
    important_style(NULL)
{
  assert(declaration_block->is_declaration_block());
  initialize_style(declaration_block, false);
}


CSSStyle::CSSStyle()
  : margin_top(CSSValue::CSS_MARGIN_WIDTH),
    margin_right(CSSValue::CSS_MARGIN_WIDTH),
    margin_bottom(CSSValue::CSS_MARGIN_WIDTH),
    margin_left(CSSValue::CSS_MARGIN_WIDTH),
    display(CSSValue::CSS_DISPLAY),
    direction(CSSValue::CSS_DIRECTION),
    line_height(CSSValue::CSS_LINE_HEIGHT),
    vertical_align(CSSValue::CSS_VERTICAL_ALIGN),
    list_style_image(CSSValue::CSS_LIST_STYLE_IMAGE),
    list_style_position(CSSValue::CSS_LIST_STYLE_POSITION),
    list_style_type(CSSValue::CSS_LIST_STYLE_TYPE),
    color(CSSValue::CSS_COLOR),
    background_color(CSSValue::CSS_BACKGROUND_COLOR),
    background_image(CSSValue::CSS_BACKGROUND_IMAGE),
    background_repeat(CSSValue::CSS_BACKGROUND_REPEAT),
    background_attachment(CSSValue::CSS_BACKGROUND_ATTACHMENT),
    background_position_h(CSSValue::CSS_BACKGROUND_POSITION_H),
    background_position_v(CSSValue::CSS_BACKGROUND_POSITION_V),
    font_style(CSSValue::CSS_FONT_STYLE),
    font_variant(CSSValue::CSS_FONT_VARIANT),
    font_weight(CSSValue::CSS_FONT_WEIGHT),
    font_size(CSSValue::CSS_FONT_SIZE),
    text_indent(CSSValue::CSS_TEXT_INDENT),
    text_align(CSSValue::CSS_TEXT_ALIGN),
    text_decoration(CSSValue::CSS_TEXT_DECORATION),
    text_transform(CSSValue::CSS_TEXT_TRANSFORM),
    unicode_bidi(CSSValue::CSS_UNICODE_BIDI),
    white_space(CSSValue::CSS_WHITE_SPACE),
    important_style(NULL)
{
}


CSSStyle::~CSSStyle()
{
  while (!content.empty())
    {
      CSSValue *val = content.back();
      delete val;
      content.pop_back();
    }

  while (!font_family.empty())
    {
      CSSValue *val = font_family.back();
      delete val;
      font_family.pop_back();
    }

  if (important_style != NULL)
    delete important_style;
}


void
CSSStyle::parse_margin(const vector<CSSNode *> &values)
{
  if (values.size() == 1)
    {
      const CSSNode *node = values[0];
      margin_top.set_value(node);
      margin_right.set_value(node);
      margin_bottom.set_value(node);
      margin_left.set_value(node);
      return;
    }
  if (values.size() == 2)
    {
      const CSSNode *node = values[0];
      margin_top.set_value(node);
      margin_bottom.set_value(node);

      node = values[1];
      margin_right.set_value(node);
      margin_left.set_value(node);
      return;
    }
  if (values.size() == 3)
    {
      const CSSNode *node = values[0];
      margin_top.set_value(node);

      node = values[1];
      margin_bottom.set_value(node);

      node = values[2];
      margin_right.set_value(node);
      margin_left.set_value(node);
      return;
    }
  if (values.size() == 4)
    {
      const CSSNode *node = values[0];
      margin_top.set_value(node);

      node = values[1];
      margin_right.set_value(node);

      node = values[2];
      margin_bottom.set_value(node);

      node = values[3];
      margin_left.set_value(node);
      return;
    }
}


static inline bool
is_special_font_ident(const char *ident)
{
  return strcmp(ident, css_inherit) == 0
    || strcmp(ident, css_caption) == 0
    || strcmp(ident, css_icon) == 0
    || strcmp(ident, css_menu) == 0
    || strcmp(ident, css_message_box) == 0
    || strcmp(ident, css_small_caption) == 0
    || strcmp(ident, css_status_bar) == 0;
}


void
CSSStyle::parse_font(const vector<CSSNode *> &values)
{
  while (!font_family.empty())
    {
      CSSValue *val = font_family.back();
      delete val;
      font_family.pop_back();
    }
  line_height.set_initial();
  font_style.set_initial();
  font_variant.set_initial();
  font_weight.set_initial();
  font_size.set_initial();

  vector<CSSNode *>::const_iterator iter = values.begin();
  vector<CSSNode *>::const_iterator end_iter = values.end();

  bool non_special = false;
  bool style = false;
  bool variant = false;
  bool weight = false;
  bool size = false;
  while (iter != end_iter)
    {
      const CSSNode *node = *iter++;
      if (!node->is_term())
	return;	// ͽʤ

      const CSSToken *token = node->get_term();
      if (!non_special)
	{
	  if (token->is_ident())
	    {
	      const char *ident = token->get_ident();

	      if (strcmp(ident, css_inherit) == 0)
		{
		  line_height.set_value(node);
		  font_style.set_value(node);
		  font_variant.set_value(node);
		  font_weight.set_value(node);
		  font_size.set_value(node);
		  CSSValue *val = new CSSValue(CSSValue::CSS_FONT_FAMILY);
		  val->set_value(node);
		  font_family.push_back(val);
		  return;
		}
	      if (strcmp(ident, css_caption) == 0
		  || strcmp(ident, css_icon) == 0
		  || strcmp(ident, css_menu) == 0
		  || strcmp(ident, css_message_box) == 0
		  || strcmp(ident, css_small_caption) == 0
		  || strcmp(ident, css_status_bar) == 0)
		{
		  CSSValue *val = new CSSValue(CSSValue::CSS_FONT_FAMILY);
		  val->set_value(node);
		  font_family.push_back(val);
		  return;
		}
	    }
	}
      if (!size)
	{
	  if (token->is_ident() && is_special_font_ident(token->get_ident()))
	    return;	// 顼
	  if (!style && font_style.set_value(node))
	    {
	      style = true;
	      non_special = true;
	      continue;
	    }
	  if (!variant && font_variant.set_value(node))
	    {
	      variant = true;
	      non_special = true;
	      continue;
	    }
	  if (!weight && font_weight.set_value(node))
	    {
	      weight = true;
	      non_special = true;
	      continue;
	    }
	  if (font_size.set_value(node))
	    {
	      style = true;
	      variant = true;
	      weight = true;
	      size = true;
	      non_special = true;
	      continue;
	    }
	  return;	// 顼
	}
      if (token->is_delim())
	{
	  if (token->get_delim() == '/')
	    {
	      if (iter == end_iter || !line_height.set_value(*iter++))
		return; // 顼
	    }
	  else
	    return;	// 顼
	}
      // Ĥfont-family
      while (iter != end_iter)
	{
	  CSSValue *val = new CSSValue(CSSValue::CSS_FONT_FAMILY);
	  if (val->set_value(iter, end_iter))
	    font_family.push_back(val);
	  else
	    {
	      delete val;
	      return;	// 顼
	    }
	}
      return;	// λ
    }
}


void
CSSStyle::parse_background(const vector<CSSNode *> &values)
{
  background_color.set_initial();
  background_image.set_initial();
  background_repeat.set_initial();
  background_attachment.set_initial();
  background_position_h.set_initial();
  background_position_v.set_initial();

  vector<CSSNode *>::const_iterator iter = values.begin();
  vector<CSSNode *>::const_iterator end_iter = values.end();

  bool non_inherit = false;
  bool bg_color = false;
  bool bg_image = false;
  bool bg_repeat = false;
  bool bg_attachment = false;
  bool bg_position = false;

  while (iter != end_iter)
    {
      const CSSNode *node = *iter++;
      if (!node->is_any())
	return;	// ͽʤ
      const CSSNodeAny *any = dynamic_cast<const CSSNodeAny *>(node);
      assert(any != NULL);
      const CSSToken *token = any->get_leader();

      if (token->is_ident_inherit())
	{
	  if (!non_inherit)
	    {
	      background_color.set_value(node);
	      background_image.set_value(node);
	      background_repeat.set_value(node);
	      background_attachment.set_value(node);
	      background_position_h.set_value(node);
	      background_position_v.set_value(node);
	      return;
	    }
	  return;	// 顼
	}

      if (!bg_color && background_color.set_value(node))
	{
	  bg_color = true;
	  non_inherit = true;
	  continue;
	}
      if (!bg_image && background_image.set_value(node))
	{
	  bg_image = true;
	  non_inherit = true;
	  continue;
	}
      if (!bg_repeat && background_repeat.set_value(node))
	{
	  bg_repeat = true;
	  non_inherit = true;
	  continue;
	}
      if (!bg_attachment && background_attachment.set_value(node))
	{
	  bg_attachment = true;
	  non_inherit = true;
	  continue;
	}
      if (!bg_position)
	{
	  if (iter != end_iter)
	    {
	      if (background_position_h.set_value(node)
		  && background_position_v.set_value(*iter))
		{
		  bg_position = true;
		  non_inherit = true;
		  ++iter;
		  continue;
		}
	      if (background_position_v.set_value(node)
		  && background_position_h.set_value(*iter))
		{
		  bg_position = true;
		  non_inherit = true;
		  ++iter;
		  continue;
		}
	    }
	  if (background_position_h.set_value(node))
	    {
	      bg_position = true;
	      non_inherit = true;
	      background_position_v.set_percentage(50.0);
	      continue;
	    }
	  if (background_position_v.set_value(node))
	    {
	      bg_position = true;
	      non_inherit = true;
	      background_position_h.set_percentage(50.0);
	      continue;
	    }
	}
    }
}


void
CSSStyle::set_property(const char *name, const CSSNodeValue *value)
{
  const vector<CSSNode *> &values = value->get_values();
  if (values.empty())
    return;	// ͤʤΤ

  if (strcmp(name, "margin") == 0)
    {
      parse_margin(values);
      return;
    }

  if (strcmp(name, "font") == 0)
    {
      parse_font(values);
      return;
    }

  if (strcmp(name, "background") == 0)
    {
      parse_background(values);
      return;
    }

  if (strcmp(name, "color") == 0)
    {
      vector<CSSNode *>::const_iterator iter = values.begin();
      vector<CSSNode *>::const_iterator end_iter = values.end();
      color.set_value(iter, end_iter);
      return;
    }

  if (strcmp(name, "background-color") == 0)
    {
      vector<CSSNode *>::const_iterator iter = values.begin();
      vector<CSSNode *>::const_iterator end_iter = values.end();
      background_color.set_value(iter, end_iter);
      return;
    }

  if (strcmp(name, "background-position") == 0)
    {
      if (values.size() == 1)
	{
	  if (background_position_h.set_value(values[0]))
	    {
	      background_position_v.set_percentage(50.0);
	      return;
	    }
	  if (background_position_v.set_value(values[0]))
	    {
	      background_position_h.set_percentage(50.0);
	      return;
	    }
	  return;
	}
      else if (values.size() == 2)
	{
	  if (background_position_h.set_value(values[0])
	      && background_position_v.set_value(values[1]))
	    return;
	  // 顼ο񤤤ʥΤ
	  background_position_h.set_value(values[1]);
	  background_position_v.set_value(values[0]);
	  return;
	}
      return;
    }

  if (strcmp(name, "content") == 0)
    {
      while (!content.empty())
	{
	  CSSValue *val = content.back();
	  delete val;
	  content.pop_back();
	}
      vector<CSSNode *>::const_iterator iter = values.begin();
      vector<CSSNode *>::const_iterator end_iter = values.end();
      while (iter != end_iter)
	{
	  CSSValue *val = new CSSValue(CSSValue::CSS_CONTENT);
	  if (val->set_value(iter, end_iter))
	    content.push_back(val);
	  else
	    {
	      delete val;
	      return;	// 顼
	    }
	}
      return;
    }

  if (strcmp(name, "font-family") == 0)
    {
      while (!font_family.empty())
	{
	  CSSValue *val = font_family.back();
	  delete val;
	  font_family.pop_back();
	}
      vector<CSSNode *>::const_iterator iter = values.begin();
      vector<CSSNode *>::const_iterator end_iter = values.end();
      while (iter != end_iter)
	{
	  CSSValue *val = new CSSValue(CSSValue::CSS_FONT_FAMILY);
	  if (val->set_value(iter, end_iter))
	    font_family.push_back(val);
	  else
	    {
	      delete val;
	      return;	// 顼
	    }
	}
      return;
    }

  if (strcmp(name, "text-decoration") == 0)
    {
      text_decoration.set_initial();
      vector<CSSNode *>::const_iterator iter = values.begin();
      vector<CSSNode *>::const_iterator end_iter = values.end();
      text_decoration.set_value(iter, end_iter);
      return;
    }


  // ñproperty
  if (values.size() != 1)
    return;

  const CSSNode *node = values.front();

  if (strcmp(name, "margin-top") == 0)
    {
      margin_top.set_value(node);
      return;
    }

  if (strcmp(name, "margin-right") == 0)
    {
      margin_right.set_value(node);
      return;
    }

  if (strcmp(name, "margin-bottom") == 0)
    {
      margin_bottom.set_value(node);
      return;
    }

  if (strcmp(name, "margin-left") == 0)
    {
      margin_left.set_value(node);
      return;
    }

  if (strcmp(name, "display") == 0)
    {
      display.set_value(node);
      return;
    }

  if (strcmp(name, "direction") == 0)
    {
      direction.set_value(node);
      return;
    }

  if (strcmp(name, "line-height") == 0)
    {
      line_height.set_value(node);
      return;
    }

  if (strcmp(name, "vertical-align") == 0)
    {
      vertical_align.set_value(node);
      return;
    }

  if (strcmp(name, "list-style-type") == 0)
    {
      list_style_type.set_value(node);
      return;
    }

  if (strcmp(name, "list-style-image") == 0)
    {
      list_style_image.set_value(node);
      return;
    }

  if (strcmp(name, "list-style-position") == 0)
    {
      list_style_position.set_value(node);
      return;
    }

  if (strcmp(name, "background-image") == 0)
    {
      background_image.set_value(node);
      return;
    }

  if (strcmp(name, "background-repeat") == 0)
    {
      background_repeat.set_value(node);
      return;
    }

  if (strcmp(name, "background-attachment") == 0)
    {
      background_attachment.set_value(node);
      return;
    }

  if (strcmp(name, "font-style") == 0)
    {
      font_style.set_value(node);
      return;
    }

  if (strcmp(name, "font-variant") == 0)
    {
      font_variant.set_value(node);
      return;
    }

  if (strcmp(name, "font-weight") == 0)
    {
      font_weight.set_value(node);
      return;
    }

  if (strcmp(name, "font-size") == 0)
    {
      font_size.set_value(node);
      return;
    }

  if (strcmp(name, "text-indent") == 0)
    {
      text_indent.set_value(node);
      return;
    }

  if (strcmp(name, "text-align") == 0)
    {
      text_align.set_value(node);
      return;
    }

  if (strcmp(name, "text-transform") == 0)
    {
      text_transform.set_value(node);
      return;
    }

  if (strcmp(name, "unicode-bidi") == 0)
    {
      unicode_bidi.set_value(node);
      return;
    }

  if (strcmp(name, "white-space") == 0)
    {
      white_space.set_value(node);
      return;
    }
}


void
CSSStyle::initialize_style(const CSSNodeBlock *declaration_block,
			   bool important)
{
  bool important_found = false;

  // declaration_blockᤷthisΥեɤ롣
  vector<CSSNode *>::const_iterator iter
    = declaration_block->get_elements().begin();
  vector<CSSNode *>::const_iterator end_iter
    = declaration_block->get_elements().end();

  while (iter != end_iter)
    {
      assert((*iter)->is_declaration());
      CSSNodeDeclaration *decl = dynamic_cast<CSSNodeDeclaration *>(*iter++);
      assert(decl != NULL);
      const char *prop = decl->get_property_name();
      const CSSNodeValue *value = decl->get_property_value();
      if (prop != NULL && value != NULL)
	set_property(prop, value);
    }

  if (!important && important_found)
    important = new CSSStyle(declaration_block, true);
}


void
CSSStyle::composite(const CSSStyle &style)
{
  // font_family
  vector<CSSValue *>::const_iterator ff_iter = style.get_font_family_begin();
  vector<CSSValue *>::const_iterator ff_end_iter = style.get_font_family_end();
  if (ff_iter != ff_end_iter)
    {
      while (!font_family.empty())
	{
	  delete font_family.back();
	  font_family.pop_back();
	}

      while (ff_iter != ff_end_iter)
	font_family.push_back(new CSSValue(*ff_iter++));
    }

  // font_style
  if (style.get_font_style().is_value_set())
    font_style.set_value(style.get_font_style());

  // font_variant
  if (style.get_font_variant().is_value_set())
    font_variant.set_value(style.get_font_variant());

  // font_weight
  if (style.get_font_weight().is_value_set())
    {
      int cur_val = 0;
      if (font_weight.is_value_set())
	{
	  if (font_weight.is_keyword())
	    {
	      CSSKeyword keyword = font_weight.get_keyword();
	      if (keyword == css_bold)
		cur_val = 700;
	    }
	  else if (font_weight.is_integer())
	    cur_val = font_weight.get_integer();
	}
      if (cur_val == 0)
	cur_val = 400;	// normal400Ƥ롩

      const CSSValue &weight = style.get_font_weight();
      int val = 0;
      if (weight.is_keyword())
	{
	  CSSKeyword keyword = weight.get_keyword();
	  if (keyword == css_normal)
	    val = 400;
	  else if (keyword == css_bold)
	    val = 700;
	  else if (keyword == css_bolder)
	    val = cur_val + 200;
	  else if (keyword == css_lighter)
	    val = cur_val - 200;
	}
      else if (weight.is_integer())
	val = weight.get_integer();
      else
	val = cur_val;

      val = ((val + 99) / 100) * 100;
      if (val < 100)
	val = 100;
      else if (val > 900)
	val = 900;

      font_weight.set_integer(val);
    }

  // font_size
  if (style.get_font_size().is_value_set())
    font_size.set_value(style.get_font_size());

  // margin_top
  if (style.get_margin_top().is_value_set())
    {
      if (!style.get_margin_top().is_keyword())
	margin_top.set_value(style.get_margin_top());
    }

  // margin_right
  if (style.get_margin_right().is_value_set())
    {
      const CSSValue &val = style.get_margin_right();
      if (!val.is_keyword())
	margin_right.set_value(val);
    }

  // margin_bottom
  if (style.get_margin_bottom().is_value_set())
    {
      const CSSValue &val = style.get_margin_bottom();
      if (!val.is_keyword())
	margin_bottom.set_value(val);
    }

  // margin_left
  if (style.get_margin_left().is_value_set())
    {
      const CSSValue &val = style.get_margin_left();
      if (!val.is_keyword())
	margin_left.set_value(val);
    }

  // display
  if (style.get_display().is_value_set())
    {
      const CSSValue &val = style.get_display();
      assert(val.is_keyword());
      display.set_value(val);
    }

  // direction
  if (style.get_direction().is_value_set())
    {
      const CSSValue &val = style.get_direction();
      assert(val.is_keyword());
      direction.set_value(val);
    }

  // line_height
  if (style.get_line_height().is_value_set())
    {
      const CSSValue &val = style.get_line_height();
      line_height.set_value(val);
    }

  // vertical_align
  if (style.get_vertical_align().is_value_set())
    {
      const CSSValue &val = style.get_vertical_align();
      vertical_align.set_value(val);
    }
  else
    vertical_align.unset_value();

  // list_style_image
  if (style.get_list_style_image().is_value_set())
    {
      const CSSValue &val = style.get_list_style_image();
      list_style_image.set_value(val);
    }

  // list_style_position
  if (style.get_list_style_position().is_value_set())
    {
      const CSSValue &val = style.get_list_style_position();
      assert(val.is_keyword());
      list_style_position.set_value(val);
    }

  // list_style_type
  if (style.get_list_style_type().is_value_set())
    {
      const CSSValue &val = style.get_list_style_type();
      assert(val.is_keyword());
      list_style_type.set_value(val);
    }

  // color
  if (style.get_color().is_value_set())
    {
      const CSSValue &val = style.get_color();
      color.set_value(val);
    }

  // background_color
  if (style.get_background_color().is_value_set())
    {
      const CSSValue &val = style.get_background_color();
      background_color.set_value(val);
    }

  // background_image
  if (style.get_background_image().is_value_set())
    {
      const CSSValue &val = style.get_background_image();
      background_image.set_value(val);
    }
  else
    background_image.unset_value();

  // background_repeat
  if (style.get_background_repeat().is_value_set())
    {
      const CSSValue &val = style.get_background_repeat();
      assert(val.is_keyword());
      background_repeat.set_value(val);
    }

  // background_attachment
  if (style.get_background_attachment().is_value_set())
    {
      const CSSValue &val = style.get_background_attachment();
      assert(val.is_keyword());
      background_attachment.set_value(val);
    }

  // background_position_h
  if (style.get_background_position_h().is_value_set())
    {
      const CSSValue &val = style.get_background_position_h();
      background_position_h.set_value(val);
    }

  // background_position_v
  if (style.get_background_position_v().is_value_set())
    {
      const CSSValue &val = style.get_background_position_v();
      background_position_v.set_value(val);
    }

  // content
  vector<CSSValue *>::const_iterator c_iter = style.get_content_begin();
  vector<CSSValue *>::const_iterator c_end_iter = style.get_content_end();
  if (c_iter != c_end_iter)
    {
      while (!content.empty())
	{
	  delete content.back();
	  content.pop_back();
	}

      while (c_iter != c_end_iter)
	{
	  const CSSValue *val = *c_iter++;
	  content.push_back(new CSSValue(*val));
	}
    }

  // text_indent
  if (style.get_text_indent().is_value_set())
    {
      const CSSValue &val = style.get_text_indent();
      text_indent.set_value(val);
    }

  // text_align
  if (style.get_text_align().is_value_set())
    {
      const CSSValue &val = style.get_text_align();
      text_align.set_value(val);
    }

  // text_decoration
  if (style.get_text_decoration().is_value_set())
    {
      const CSSValue &val = style.get_text_decoration();
      text_decoration.set_value(val);
    }

  // text_transform
  if (style.get_text_transform().is_value_set())
    {
      const CSSValue &val = style.get_text_transform();
      assert(val.is_keyword());
      text_transform.set_value(val);
    }

  // unicode_bidi
  if (style.get_unicode_bidi().is_value_set())
    {
      const CSSValue &val = style.get_unicode_bidi();
      assert(val.is_keyword());
      unicode_bidi.set_value(val);
    }

  // white_space
  if (style.get_white_space().is_value_set())
    {
      const CSSValue &val = style.get_white_space();
      assert(val.is_keyword());
      white_space.set_value(val);
    }
}


char *
CSSStyle::to_string() const
{
  string buf("CSSStyle {");

  if (margin_top.is_value_set())
    {
      buf.append("\n    margin-top: ");
      margin_top.append_string(buf);
    }

  if (margin_right.is_value_set())
    {
      buf.append("\n    margin-right: ");
      margin_right.append_string(buf);
    }

  if (margin_bottom.is_value_set())
    {
      buf.append("\n    margin-bottom: ");
      margin_bottom.append_string(buf);
    }

  if (margin_left.is_value_set())
    {
      buf.append("\n    margin-left: ");
      margin_left.append_string(buf);
    }

  if (display.is_value_set())
    {
      buf.append("\n    display: ");
      display.append_string(buf);
    }

  if (direction.is_value_set())
    {
      buf.append("\n    direction: ");
      direction.append_string(buf);
    }

  if (line_height.is_value_set())
    {
      buf.append("\n    line-height: ");
      line_height.append_string(buf);
    }

  if (vertical_align.is_value_set())
    {
      buf.append("\n    vertical-align: ");
      vertical_align.append_string(buf);
    }

  if (list_style_type.is_value_set())
    {
      buf.append("\n    list-style-type: ");
      list_style_type.append_string(buf);
    }

  if (list_style_image.is_value_set())
    {
      buf.append("\n    list-style-image: ");
      list_style_image.append_string(buf);
    }

  if (list_style_position.is_value_set())
    {
      buf.append("\n    list-style-position: ");
      list_style_position.append_string(buf);
    }

  if (color.is_value_set())
    {
      buf.append("\n    color: ");
      color.append_string(buf);
    }

  if (background_color.is_value_set())
    {
      buf.append("\n    background_color: ");
      background_color.append_string(buf);
    }

  if (background_image.is_value_set())
    {
      buf.append("\n    background-image: ");
      background_image.append_string(buf);
    }

  if (background_repeat.is_value_set())
    {
      buf.append("\n    background-repeat: ");
      background_repeat.append_string(buf);
    }

  if (background_attachment.is_value_set())
    {
      buf.append("\n    background-attachment: ");
      background_attachment.append_string(buf);
    }

  if (background_position_h.is_value_set())
    {
      buf.append("\n    background-position: ");
      background_position_h.append_string(buf);
      buf.append(1, ' ');
      background_position_v.append_string(buf);
    }

  if (!content.empty())
    {
      buf.append("\n    content: ");
      vector<CSSValue *>::const_iterator iter = content.begin();
      vector<CSSValue *>::const_iterator end_iter = content.end();
      while (iter != end_iter)
	{
	  (*iter++)->append_string(buf);
	  if (iter != end_iter)
	    buf.append(1, ' ');
	}
    }

  if (!font_family.empty())
    {
      buf.append("\n    font-family: ");
      vector<CSSValue *>::const_iterator iter = font_family.begin();
      vector<CSSValue *>::const_iterator end_iter = font_family.end();
      while (iter != end_iter)
	{
	  (*iter++)->append_string(buf);
	  if (iter != end_iter)
	    buf.append(", ");
	}
    }

  if (font_style.is_value_set())
    {
      buf.append("\n    font-style: ");
      font_style.append_string(buf);
    }

  if (font_variant.is_value_set())
    {
      buf.append("\n    font-variant: ");
      font_variant.append_string(buf);
    }

  if (font_weight.is_value_set())
    {
      buf.append("\n    font-weight: ");
      font_weight.append_string(buf);
    }

  if (font_size.is_value_set())
    {
      buf.append("\n    font-size: ");
      font_size.append_string(buf);
    }

  if (text_indent.is_value_set())
    {
      buf.append("\n    text-indent: ");
      text_indent.append_string(buf);
    }

  if (text_align.is_value_set())
    {
      buf.append("\n    text-align: ");
      text_align.append_string(buf);
    }

  if (text_decoration.is_value_set())
    {
      buf.append("\n    text-decoration: ");
      text_decoration.append_string(buf);
    }

  if (text_transform.is_value_set())
    {
      buf.append("\n    text-transform: ");
      text_transform.append_string(buf);
    }

  if (unicode_bidi.is_value_set())
    {
      buf.append("\n    unicode-bidi: ");
      unicode_bidi.append_string(buf);
    }

  if (white_space.is_value_set())
    {
      buf.append("\n    white-space: ");
      white_space.append_string(buf);
    }

  if (important_style != NULL)
    {
      buf.append("\n\n*** IMPORTANT PROPERTIES ***\n");
      char *tmp_str = important_style->to_string();
      buf.append(tmp_str);
    }

  buf.append("\n  }");

  return strdup(buf.c_str());
}


CSSNodeAny *
CSSNode::new_CSSNodeAny(CSSToken *token, int lineno)
{
  switch (token->get_type())
    {
    case CSS_TOKEN_TYPE_IDENT:
    case CSS_TOKEN_TYPE_NUMBER:
    case CSS_TOKEN_TYPE_PERCENTAGE:
    case CSS_TOKEN_TYPE_DIMENSION:
    case CSS_TOKEN_TYPE_STRING:
    case CSS_TOKEN_TYPE_DELIM:
    case CSS_TOKEN_TYPE_URI:
    case CSS_TOKEN_TYPE_HASH:
    case CSS_TOKEN_TYPE_UNICODE_RANGE:
    case CSS_TOKEN_TYPE_INCLUDES:
    case CSS_TOKEN_TYPE_DASHMATCH:
    case CSS_TOKEN_TYPE_FUNCTION:
    case CSS_TOKEN_TYPE_LPAREN:
    case CSS_TOKEN_TYPE_LBRACKET:
    case CSS_TOKEN_TYPE_SPACE:
      return new CSSNodeAny(token, lineno);

    default:
      return NULL;
    }
}


CSSNodeTerm::CSSNodeTerm(CSSToken *token, int lineno)
  : CSSNode(lineno),
    term(token)
{
  assert(token->get_type() == CSS_TOKEN_TYPE_ATKEYWORD
	 || token->get_type() == CSS_TOKEN_TYPE_SEMICOLON
	 || token->get_type() == CSS_TOKEN_TYPE_RBRACE);
}


CSSNodeTerm::~CSSNodeTerm()
{
  delete term;
}


char *
CSSNodeTerm::to_string() const
{
  string buf("CSSNodeTerm<");
  switch (term->get_type())
    {
    case CSS_TOKEN_TYPE_ATKEYWORD:
      {
	buf.append(1, '@').append(term->get_atkeyword()).append(1, '>');
	return strdup(buf.c_str());
      }

    case CSS_TOKEN_TYPE_SEMICOLON:
      buf.append(";>");
      return strdup(buf.c_str());

    case CSS_TOKEN_TYPE_RBRACE:
      buf.append("}>");
      return strdup(buf.c_str());

    default:
      break;
    }

  buf.append("INVALID Token: ");
  char *tmp = term->to_string();
  buf.append(tmp).append(1, '>');
  free(tmp);
  return strdup(buf.c_str());
}


bool
CSSNodeTerm::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();
  if (type == CSS_TOKEN_TYPE_SPACE || type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }
  return false;
}


char *
CSSNodeAny::to_string() const
{
  if (state == TERM)
    {
      string buf("CSSNodeAny<");
      switch (leader->get_type())
	{
	case CSS_TOKEN_TYPE_IDENT:
	case CSS_TOKEN_TYPE_NUMBER:
	case CSS_TOKEN_TYPE_PERCENTAGE:
	case CSS_TOKEN_TYPE_DIMENSION:
	case CSS_TOKEN_TYPE_STRING:
	case CSS_TOKEN_TYPE_DELIM:
	case CSS_TOKEN_TYPE_URI:
	case CSS_TOKEN_TYPE_HASH:
	case CSS_TOKEN_TYPE_UNICODE_RANGE:
	case CSS_TOKEN_TYPE_INCLUDES:
	case CSS_TOKEN_TYPE_DASHMATCH:
	case CSS_TOKEN_TYPE_SPACE:
	  break;

	default:
	  buf.append("INVALID Token: ");
	}
      char *tmp_str = leader->to_string();
      buf.append(tmp_str);
      free(tmp_str);
      buf.append(1, '>');
      return strdup(buf.c_str());
    }

  string buf("CSSNodeAny<");
  char *tmp_str = leader->to_string();
  buf.append(tmp_str);
  free(tmp_str);

  if (state == BRACKET_COMPLETE || state == BRACKET_INCOMPLETE)
    buf.append(" [");
  else
    buf.append(" (");

  vector<CSSNodeAny *>::const_iterator iter = any.begin();
  vector<CSSNodeAny *>::const_iterator end_iter = any.end();
  while (iter != end_iter)
    {
      buf.append(1, ' ');
      tmp_str = (*iter++)->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  if (state == BRACKET_COMPLETE)
    buf.append(" ]");
  else if (state == FUNCTION_COMPLETE || state == PAREN_COMPLETE)
    buf.append(" )");
  buf.append(1, '>');
  return strdup(buf.c_str());
}


CSSNodeAny::CSSNodeAny(CSSToken *token, int lineno)
  : CSSNode(lineno),
    leader(token),
    next(NULL)
{
  switch (token->get_type())
    {
    case CSS_TOKEN_TYPE_FUNCTION:
      state = FUNCTION_INCOMPLETE;
      break;

    case CSS_TOKEN_TYPE_LPAREN:
      state = PAREN_INCOMPLETE;
      break;

    case CSS_TOKEN_TYPE_LBRACKET:
      state = BRACKET_INCOMPLETE;
      break;

    default:
      state = TERM;
    }
}


CSSNodeAny::~CSSNodeAny()
{
  if (next != NULL)
    delete next;
  while (!any.empty())
    {
      CSSNodeAny *node = any.back();
      delete node;
      any.pop_back();
    }
  delete leader;
}


bool
CSSNodeAny::is_term() const
{
  return state == TERM;
}


bool
CSSNodeAny::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();

  if (type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }

  if (state <= BRACKET_COMPLETE)
    {
      if (type == CSS_TOKEN_TYPE_SPACE)
	{
	  delete token;
	  return true;
	}
      return false;
    }

  if (next == NULL && type == CSS_TOKEN_TYPE_SPACE)
    {
      delete token;
      return true;
    }

  if (next != NULL)
    {
      if (next->eat(token, lineno))
	return true;
      any.push_back(next);
      next = NULL;
    }

  switch (type)
    {
    case CSS_TOKEN_TYPE_IDENT:
    case CSS_TOKEN_TYPE_NUMBER:
    case CSS_TOKEN_TYPE_PERCENTAGE:
    case CSS_TOKEN_TYPE_DIMENSION:
    case CSS_TOKEN_TYPE_STRING:
    case CSS_TOKEN_TYPE_DELIM:
    case CSS_TOKEN_TYPE_URI:
    case CSS_TOKEN_TYPE_HASH:
    case CSS_TOKEN_TYPE_UNICODE_RANGE:
    case CSS_TOKEN_TYPE_INCLUDES:
    case CSS_TOKEN_TYPE_DASHMATCH:
    case CSS_TOKEN_TYPE_FUNCTION:
    case CSS_TOKEN_TYPE_LPAREN:
    case CSS_TOKEN_TYPE_LBRACKET:
      next = new CSSNodeAny(token, lineno);
      return true;

    case CSS_TOKEN_TYPE_RPAREN:
      if (state == FUNCTION_INCOMPLETE)
	{
	  state = FUNCTION_COMPLETE;
	  delete token;
	  return true;
	}
      if (state == PAREN_INCOMPLETE)
	{
	  state = PAREN_COMPLETE;
	  delete token;
	  return true;
	}
      break;

    case CSS_TOKEN_TYPE_RBRACKET:
      if (state == BRACKET_INCOMPLETE)
	{
	  state = BRACKET_COMPLETE;
	  delete token;
	  return true;
	}
      break;

    default:
      break;
    }

  // 顼
  char *token_string = token->to_string();
  fprintf(stderr, "Unexpected Token: %s at line %d\n",
	  token_string, get_lineno());
  free(token_string);
  return false;
}


CSSNodeBlock::~CSSNodeBlock()
{
  while (!elements.empty())
    {
      CSSNode *node = elements.back();
      delete node;
      elements.pop_back();
    }
  if (next != NULL)
    delete next;
}


char *
CSSNodeBlock::to_string() const
{
  string buf("CSSNodeBlock<{");

  vector<CSSNode *>::const_iterator iter = elements.begin();
  vector<CSSNode *>::const_iterator end_iter = elements.end();
  while (iter != end_iter)
    {
      buf.append(block_type == GENERIC_BLOCK ? " " : "\n    ");
      char *tmp_str = (*iter++)->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }
  if (next != NULL)
    {
      buf.append(block_type == GENERIC_BLOCK ? " " : "\n    ");
      char *tmp_str = next->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }
  buf.append(1, '>');
  return strdup(buf.c_str());
}


bool
CSSNodeBlock::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();
  if (type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }

  if (next != NULL)
    {
      if (next->eat(token, lineno))
	return true;
      if (next->is_term()
	  && next->get_term()->get_type() == CSS_TOKEN_TYPE_RBRACE)
	return false;
      elements.push_back(next);
      next = NULL;
    }
  else if (type == CSS_TOKEN_TYPE_SPACE)
    {
      delete token;
      return true;
    }

  if (type != CSS_TOKEN_TYPE_RBRACE && block_type == RULESET_BLOCK)
    {
      next = new CSSNodeRuleset(lineno);
      if (!next->eat(token, lineno))
	{
	  char *tmp_str = token->to_string();
	  fprintf(stderr, "Unexpected Token at line %d: %s\n",
		  lineno, tmp_str);
	  free(tmp_str);
	  delete next;
	      next = NULL;
	      return false;
	}
      return true;
    }

  switch (type)
    {
    case CSS_TOKEN_TYPE_LBRACE:
      next = new CSSNodeBlock(lineno, CSSNodeBlock::GENERIC_BLOCK);
      delete token;
      return true;

    case CSS_TOKEN_TYPE_ATKEYWORD:
    case CSS_TOKEN_TYPE_SEMICOLON:
    case CSS_TOKEN_TYPE_RBRACE:
      if (type == CSS_TOKEN_TYPE_SEMICOLON && block_type == DECLARATION_BLOCK)
	{
	  delete token;
	  return true;
	}
      next = new CSSNodeTerm(token, lineno);
      return true;

    case CSS_TOKEN_TYPE_DELIM:
    case CSS_TOKEN_TYPE_IDENT:
      if (block_type == DECLARATION_BLOCK)
	{
	  next = new CSSNodeDeclaration(token, lineno);
	  return true;
	}
      break;

    default:
      break;
    }

  next = new_CSSNodeAny(token, lineno);
  return next != NULL;
}


CSSNodeValue::~CSSNodeValue()
{
  if (next != NULL)
    delete next;

  while (!values.empty())
    {
      CSSNode *node = values.back();
      delete node;
      values.pop_back();
    }
}


char *
CSSNodeValue::to_string() const
{
  string buf("CSSNodeValue<");

  vector<CSSNode *>::const_iterator iter = values.begin();
  vector<CSSNode *>::const_iterator end_iter = values.end();
  while (iter != end_iter)
    {
      buf.append(1, ' ');
      char *tmp_str = (*iter++)->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  if (!important)
    buf.append(" >");
  else
    buf.append(" !important>");
  return strdup(buf.c_str());
}


bool
CSSNodeValue::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();
  if (type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }

  if (important)
    {
      if (type == CSS_TOKEN_TYPE_SPACE)
	{
	  delete token;
	  return true;
	}
      return false;
    }

  if (next != NULL)
    {
      if (next->eat(token, lineno))
	return true;
      values.push_back(next);
      next = NULL;
    }

  if (type == CSS_TOKEN_TYPE_LBRACE)
    {
      next = new CSSNodeBlock(lineno, CSSNodeBlock::GENERIC_BLOCK);
      delete token;
      return true;
    }

  if (type == CSS_TOKEN_TYPE_ATKEYWORD)
    {
      next = new CSSNodeTerm(token, lineno);
      delete token;
      return true;
    }

  if (type == CSS_TOKEN_TYPE_IDENT
       && !values.empty() && strcasecmp(token->get_ident(), "important") == 0)
    {
      CSSNode *prev = values.back();
      if (prev->is_term())
	{
	  const CSSToken *prev_token = prev->get_term();
	  if (prev_token->get_type() == CSS_TOKEN_TYPE_DELIM
	      && prev_token->get_delim() == '!')
	    {
	      important = true;
	      delete token;
	      delete prev;
	      values.pop_back();
	      return true;
	    }
	}
    }

  next = new_CSSNodeAny(token, lineno);
  return next != NULL;
}


CSSNodeDeclaration::CSSNodeDeclaration(CSSToken *token, int lineno)
  : CSSNode(lineno),
    value(NULL),
    colon(false)
{
  if (token->get_type() == CSS_TOKEN_TYPE_IDENT)
    {
      delim = NULL;
      property = token;
    }
  else
    {
      assert(token->get_type() == CSS_TOKEN_TYPE_DELIM);
      delim = token;
      property = NULL;
    }
}


CSSNodeDeclaration::~CSSNodeDeclaration()
{
  if (delim != NULL)
    delete delim;
  if (property != NULL)
    delete property;
  if (value != NULL)
    delete value;
}


char *
CSSNodeDeclaration::to_string() const
{
  string buf("CSSNodeDeclaration<");

  if (delim != NULL)
    {
      char *tmp_str = delim->to_string();
      buf.append(tmp_str);
      free(tmp_str);
      buf.append(1, ' ');
    }

  if (property != NULL)
    {
      char *tmp_str = property->to_string();
      buf.append(tmp_str);
      free(tmp_str);
      buf.append(": ");
    }

  if (value != NULL)
    {
      char *tmp_str = value->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  buf.append(1, '>');
  return strdup(buf.c_str());
}


bool
CSSNodeDeclaration::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();
  if (type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }

  if (value != NULL)
    return value->eat(token, lineno);

  if (property != NULL)
    {
      if (type == CSS_TOKEN_TYPE_SPACE)
	{
	  delete token;
	  return true;
	}

      if (type == CSS_TOKEN_TYPE_DELIM && token->get_delim() == ':' && !colon)
	{
	  colon = true;
	  delete token;
	  return true;
	}

      value = new CSSNodeValue(lineno);
      if (value->eat(token, lineno))
	return true;
    }
  else if (type == CSS_TOKEN_TYPE_IDENT)
    {
      property = token;
      return true;
    }

  // 顼
  char *token_string = token->to_string();
  fprintf(stderr, "Unexpected Token: %s at line %d\n",
	  token_string, get_lineno());
  free(token_string);
  return false;
}


CSSNodeRuleset::CSSNodeRuleset(int lineno)
  : CSSNode(lineno),
    next(NULL),
    declarations(NULL)
{
}


CSSNodeRuleset::~CSSNodeRuleset()
{
  while (!selector.empty())
    {
      CSSNodeAny *sel = selector.back();
      delete sel;
      selector.pop_back();
    }

  if (declarations != NULL)
    delete declarations;

  if (next != NULL)
    delete next;
}


char *
CSSNodeRuleset::to_string() const
{
  string buf("CSSNodeRuleset<\n  selector:");

  vector<CSSNodeAny *>::const_iterator sel_iter = selector.begin();
  vector<CSSNodeAny *>::const_iterator sel_end = selector.end();
  while (sel_iter != sel_end)
    {
      buf.append(1, ' ');
      char *tmp_str = (*sel_iter++)->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  buf.append("\n  declaration: ");

  if (declarations != NULL)
    {
      char *tmp_str = declarations->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }
  else if (next != NULL)
    {
      char *tmp_str = next->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }
  buf.append("\n>");
  return strdup(buf.c_str());
}


bool
CSSNodeRuleset::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();
  if (type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }

  if (declarations != NULL)
    return declarations->eat(token, lineno);

  if (next != NULL)
    {
      if (next->eat(token, lineno))
	{
	  if (next->is_completed())
	    {
	      selector.push_back(next);
	      next = NULL;
	    }
	  return true;
	}
      // Τϥ顼ġġ
      selector.push_back(next);
      next = NULL;
    }

  if (type == CSS_TOKEN_TYPE_LBRACE)
    {
      delete token;
      declarations = new CSSNodeBlock(lineno, CSSNodeBlock::DECLARATION_BLOCK);
      return true;
    }

  next = new_CSSNodeAny(token, lineno);
  if (next != NULL)
    {
      if (next->is_completed())
	{
	  selector.push_back(next);
	  next = NULL;
	}
      return true;
    }

  // 顼
  char *token_string = token->to_string();
  fprintf(stderr, "Unexpected Token: %s at line %d\n",
	  token_string, get_lineno());
  free(token_string);
  return false;
}


CSSNodeAtRule::CSSNodeAtRule(CSSToken *atkeyword, int lineno)
  : CSSNode(lineno),
    atkeyword(atkeyword),
    tail(NULL)
{
}


CSSNodeAtRule::~CSSNodeAtRule()
{
  delete atkeyword;
  while (!any.empty())
    {
      CSSNodeAny *a = any.back();
      delete a;
      any.pop_back();
    }
  if (tail != NULL)
    delete tail;
}


char *
CSSNodeAtRule::to_string() const
{
  string buf("CSSNodeAtRule<");

  char *tmp_str = atkeyword->to_string();
  buf.append(tmp_str);
  free(tmp_str);

  vector<CSSNodeAny *>::const_iterator iter = any.begin();
  vector<CSSNodeAny *>::const_iterator end_iter = any.end();
  while (iter != end_iter)
    {
      buf.append(1, ' ');
      tmp_str = (*iter++)->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  if (tail != NULL)
    {
      buf.append(1, ' ');
      tmp_str = tail->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  buf.append(1, '>');
  return strdup(buf.c_str());
}


bool
CSSNodeAtRule::eat(CSSToken *token, int lineno)
{
  CSSTokenType type = token->get_type();
  if (type == CSS_TOKEN_TYPE_COMMENT)
    {
      delete token;
      return true;
    }

  if (tail != NULL)
    {
      if (tail->eat(token, lineno))
	return true;

      if (tail->is_block()
	  || (tail->is_term()
	      && tail->get_term()->get_type() == CSS_TOKEN_TYPE_SEMICOLON))
	return false;

      any.push_back(dynamic_cast<CSSNodeAny *>(tail));
      assert(any.back() != NULL);
      tail = NULL;
    }
  else if (type == CSS_TOKEN_TYPE_SPACE)
    {
      delete token;
      return true;
    }

  if (type == CSS_TOKEN_TYPE_LBRACE)
    {
      delete token;
      if (strcmp(atkeyword->get_atkeyword(), "media") == 0)
	tail = new CSSNodeBlock(lineno, CSSNodeBlock::RULESET_BLOCK);
      else if (strcmp(atkeyword->get_atkeyword(), "page") == 0)
	tail = new CSSNodeBlock(lineno, CSSNodeBlock::DECLARATION_BLOCK);
      else
	tail = new CSSNodeBlock(lineno, CSSNodeBlock::GENERIC_BLOCK);
	
      return true;
    }
  if (type == CSS_TOKEN_TYPE_SEMICOLON)
    {
      tail = new CSSNodeTerm(token, lineno);
      return true;
    }
  tail = new_CSSNodeAny(token, lineno);
  if (tail != NULL)
    return true;

  // 顼
  char *token_string = token->to_string();
  fprintf(stderr, "Unexpected Token: %s at line %d\n",
	  token_string, get_lineno());
  free(token_string);
  return false;
}


class CSSAttrSelector
{
public:
  enum MatchType {
    EQUALS = 0,
    INCLUDES,
    DASHMATCH
  };


private:
  char *name;
  char *value;
  MatchType type;

public:
  CSSAttrSelector(const char *name);
  CSSAttrSelector(const char *name, MatchType type, const char *value);
  ~CSSAttrSelector();
  void append_string(string &buf) const;

  inline const char *get_name() const { return name; }
  inline const char *get_value() const { return value; }
  inline MatchType get_type() const { return type; }
};


CSSAttrSelector::CSSAttrSelector(const char *name)
  : name(strdup(name)),
    value(NULL),
    type(EQUALS)
{
}


CSSAttrSelector::CSSAttrSelector(const char *name, MatchType type,
				 const char *value)
  : name(strdup(name)),
    value(strdup(value)),
    type(type)
{
}


CSSAttrSelector::~CSSAttrSelector()
{
  if (name != NULL)
    free(name);

  if (value != NULL)
    free(value);
}


void
CSSAttrSelector::append_string(string &buf) const
{
  buf.append(1, '[');
  buf.append(name);
  if (value != NULL)
    {
      static const char * const type_string[] = {
	"=",
	"~=",
	"|="
      };
      buf.append(type_string[type]);
      buf.append(value);
    }
  buf.append(1, ']');
}


class CSSSelector
{
public:
  enum SelectorType {
    SINGLE,
    DESCENDANT,
    CHILD,
    ADJACENT
  };


private:
  char *name;			// element̾(NULLϥ˥С)

  char *hash;			// #IDIDʬCSSʸˡǤʣΥϥå
  				// Ĥsimple_selector˻Ǥ
				// ѾϰĤǽʬ
  char *klass;			// HTMLŪˤϡ<elm class="a b">Τ褦˰
				// ǤʣΥ饹ꤹ뤳ȤǤ
				// 뤷cssŪˤelm.a.bΤ褦˥饹a
				// ĥ饹belmȤ쥯
				// Ǥ뤬ѾϰĤǽʬ

  vector<CSSAttrSelector *> attrs;	// ݡȤʤäƼ⤢

  CSSSelector *descendant;

  int specificity;

  SelectorType type : 2;

  CSSPseudoClass link_pseudo : 2;

  CSSPseudoElement before_after_pseudo : 2;

  bool never_matched_selector : 1;


public:
  CSSSelector()
    : name(NULL), hash(NULL), klass(NULL), descendant(NULL),
      specificity(-2), type(SINGLE),
      link_pseudo(CSS_PSEUDO_CLASS_NO),
      before_after_pseudo(CSS_PSEUDO_ELEMENT_NO),
      never_matched_selector(false) {}
  ~CSSSelector();
  char *to_string() const;
  inline void set_name(const char *value)
  { assert(name == NULL); name = strdup(value); }
  void set_hash(const char *hash);
  void set_class(const char *klass);
  void add_attr(CSSAttrSelector *attr);
  void add_pseudo(const char *pseudo);
  inline void set_descendant(CSSSelector *desc, SelectorType t)
  { descendant = desc; type = t; }
  bool is_universal() const;

  // Υ쥯ޥåspecificity֤
  // ޥåʤ쥯ξ(-1)֤롣
  // CSS216.4.3 Calculating a selector's specificity˷׻뤬
  // style°̸ǴΤǹθʤspecificityϥ쥯
  // ʤΤǡ2ܰʹߤϥå夷ƤͤȤ
  // CSS216.4.3Υ르ꥺǤϷ׻̤ϻͤ(style°ʬ
  // Ƥ⻰)ˤʤ뤬δؿint32bitȲꤷ
  // 1ɤΥͤˤޤȤ֤ͤˤʤǽϤ뤬
  // ѾϤʤ
  int calculate_specificity();
  int get_specificity() const { return specificity; }

  const StyleMatcher *match(const StyleMatcher *matcher) const;
};


char *
CSSSelector::to_string() const
{
  string buf;
  if (name != NULL)
    buf.append(name);

  if (hash != NULL)
    buf.append(1, '#').append(hash);

  if (klass != NULL)
    buf.append(1, '.').append(klass);

  if (!attrs.empty())
    {
      vector<CSSAttrSelector *>::const_iterator iter = attrs.begin();
      vector<CSSAttrSelector *>::const_iterator end_iter = attrs.end();
      while (iter != end_iter)
	(*iter++)->append_string(buf);
    }

  if (link_pseudo == CSS_PSEUDO_CLASS_LINK)
    buf.append(":link");
  else if (link_pseudo == CSS_PSEUDO_CLASS_VISITED)
    buf.append(":visited");
  
  if (before_after_pseudo == CSS_PSEUDO_ELEMENT_BEFORE)
    buf.append(":before");
  else if (before_after_pseudo == CSS_PSEUDO_ELEMENT_AFTER)
    buf.append(":after");

  if (never_matched_selector)
    buf.append("#<NOT SUPPORTED OR INVALID SELECTOR>");

  if (descendant != NULL)
    {
      static const char * const type_string[] = {
	" *INVALID* ",
	" ",
	" > ",
	" + "
      };
      buf.append(type_string[type]);
      char *tmp_str = descendant->to_string();
      buf.append(tmp_str);
      free(tmp_str);
    }

  return strdup(buf.c_str());
}


CSSSelector::~CSSSelector()
{
  if (name != NULL)
    free(name);

  if (hash != NULL)
    free(hash);

  if (klass != NULL)
    free(klass);

  while (!attrs.empty())
    {
      CSSAttrSelector *attr = attrs.back();
      delete attr;
      attrs.pop_back();
    }

  if (descendant != NULL)
    delete descendant;
}


void
CSSSelector::set_hash(const char *value)
{
  if (hash == NULL)
    {
      hash = strdup(value);
      return;
    }

  never_matched_selector = true;	// ʣϥåϥݡȤʤ
  return;
}


void
CSSSelector::set_class(const char *value)
{
  if (klass == NULL)
    {
      klass = strdup(value);
      return;
    }

  never_matched_selector = true;		// ʣ饹ϥݡȤʤ
  return;
}


void
CSSSelector::add_attr(CSSAttrSelector *attr)
{
  if (strcmp(attr->get_name(), "class") == 0
      && attr->get_type() == CSSAttrSelector::INCLUDES)
    {
      // [class~=饹̾]ʤΤ.饹̾ƱʤΤ̰
      set_class(attr->get_value());
      delete attr;
      return;
    }
  attrs.push_back(attr);
}


void
CSSSelector::add_pseudo(const char *pseudo)
{
  if (strcmp(pseudo, "link") == 0 && link_pseudo == CSS_PSEUDO_CLASS_NO)
    {
      link_pseudo = CSS_PSEUDO_CLASS_LINK;
      return;
    }
  if (strcmp(pseudo, "visited") == 0 && link_pseudo == CSS_PSEUDO_CLASS_NO)
    {
      link_pseudo = CSS_PSEUDO_CLASS_VISITED;
      return;
    }
  if (strcmp(pseudo, "before") == 0
      && before_after_pseudo == CSS_PSEUDO_ELEMENT_NO)
    {
      before_after_pseudo = CSS_PSEUDO_ELEMENT_BEFORE;
      return;
    }
  if (strcmp(pseudo, "after") == 0
      && before_after_pseudo == CSS_PSEUDO_ELEMENT_NO)
    {
      before_after_pseudo = CSS_PSEUDO_ELEMENT_AFTER;
      return;
    }
  // :link:visited:before:afterͭʤ
  // ݡȤʤpseudo classpseudo element쥯ΰ
  // ޤޤ줿쥯ϥޥåɬפʤ
  never_matched_selector = true;
}


// CSS216.4.3
// - count 1 if the selector is a `style' attribute rather than a
//   selector, 0 otherwise (= a).
// - count the number of ID attributes in the selector (= b)
// - count the number of other attributes and pseudo-classes
//   in the selector (= c)
// - count the number of element names and pseudo-elements in
//   the selector (= d)
// Ĥϥ륷ͳΥ쥯specificity׻Τa̵ط
//
// int calculate_specificity()Ǥϡ
// (b << 24) | (c << 12) | d
// specificityȤƻȤb7bit(ͤϥޥåʤʤΤ7bit)
// cdˤ12bitʬ;͵롣쥯ĹȥСե뤬
// ѾʤСե褦Ĺ쥯ˤϤɤޥå
// ʤ̵롣
// () elm0#ID0 elm1#ID1  elm127#ID127ǥСե롣
int
CSSSelector::calculate_specificity()
{
  // specificityʤΤǥå夹롣-2ϥåʤΥޥå
  if (specificity != -2)
    return specificity;

  if (never_matched_selector)
    {
      specificity = -1;
      return -1;
    }

  int spec;
  if (descendant == NULL)
    spec = 0;
  else if (type != ADJACENT)
    spec = descendant->calculate_specificity();
  else
    spec = -1;	// adjacent selectorϥݡȤʤ

  if (spec != -1)
    {
      if (hash != NULL)
	spec += 1 << 24;

      spec += ((link_pseudo != CSS_PSEUDO_CLASS_NO ? 1 : 0)
	       + (klass != NULL ? 1 : 0 ) + attrs.size()) * (1 << 12)
	+ (before_after_pseudo != CSS_PSEUDO_ELEMENT_NO ? 1 : 0)
	+ (name != NULL ? 1 : 0);
    }

  specificity = spec;
  return spec;
}


bool
CSSSelector::is_universal() const
{
  return name == NULL && hash == NULL && klass == NULL && attrs.empty()
    && link_pseudo == CSS_PSEUDO_CLASS_NO
    && before_after_pseudo == CSS_PSEUDO_ELEMENT_NO;
}


const StyleMatcher *
CSSSelector::match(const StyleMatcher *matcher) const
{
  if (never_matched_selector)
    return NULL;

  if (descendant != NULL)
    {
      if (descendant->match(matcher) != NULL)
	matcher = matcher->get_context();
    }

  while (matcher != NULL)
    {
      // °쥯ϼ괺ݡȤʤ
      if (attrs.empty()
	  && (name == NULL
	      || strcasecmp(name, matcher->get_element_name()) == 0)
	  && (hash == NULL
	      || (matcher->get_id() != NULL
		  && strcasecmp(hash, matcher->get_id()) == 0))
	  && (klass == NULL
	      || (matcher->get_klass() != NULL
		  && strcasecmp(klass, matcher->get_klass()) == 0))
	  && (link_pseudo == CSS_PSEUDO_CLASS_NO
	      || link_pseudo == matcher->get_pseudo_class())
	  && (before_after_pseudo == CSS_PSEUDO_ELEMENT_NO
	      || before_after_pseudo == matcher->get_pseudo_element()))
	return matcher;	// ޥå

      if (descendant == NULL || !descendant->is_universal())
	return NULL;

      matcher = matcher->get_context();
    }

  return NULL;
}


StyleSelector::StyleSelector(const vector<CSSNodeAny *> &selector)
{
  vector<CSSNodeAny *>::const_iterator iter = selector.begin();
  vector<CSSNodeAny *>::const_iterator end_iter = selector.end();
  CSSSelector *top = NULL;
  CSSSelector *current = NULL;
  CSSSelector::SelectorType type = CSSSelector::SINGLE;
  while (iter != end_iter)
    {
      // ⤷>+˽Ф魯ޤǤϰĤCSSSelector
      // ,˽Ф路餽λǰĤδʥ쥯ν夬ʤΤ
      // selectors.push_back(top)ơ
      // top=NULL; current=NULL; type=CSSSelector::SINGLE;Ǻٽ
      const CSSNodeAny *any = *iter++;
      const CSSToken *token = any->get_leader();
      CSSTokenType token_type = token->get_type();
      if (token_type == CSS_TOKEN_TYPE_SPACE)
	{
	  if (current != NULL)
	    type = CSSSelector::DESCENDANT;
	  continue;	// ɤФ
	}
      if (token_type == CSS_TOKEN_TYPE_IDENT)
	{
	  if (type == CSSSelector::SINGLE)
	    {
	      assert(top == NULL);
	      assert(current == NULL);
	      current = new CSSSelector();
	      top = current;
	    }
	  else
	    {
	      CSSSelector *desc = new CSSSelector();
	      current->set_descendant(desc, type);
	      current = desc;
	    }
	  current->set_name(token->get_ident());
	  continue;
	}
      if (token_type == CSS_TOKEN_TYPE_LBRACKET)
	{
	  const vector<CSSNodeAny *> &any_any = any->get_any();
	  const char *attr_name = NULL;
	  CSSAttrSelector::MatchType attr_type = CSSAttrSelector::EQUALS;
	  const char *attr_value =NULL;
	  if (any_any.size() > 0)
	    {
	      const CSSNodeAny *name_node = any_any[0];
	      if (name_node->is_term())
		{
		  const CSSToken *name_token = name_node->get_term();
		  if (name_token->is_ident())
		    attr_name = name_token->get_ident();
		}
	    }
	  if (any_any.size() == 3)
	    {
	      const CSSNodeAny *tmp_node = any_any[1];
	      if (tmp_node->is_term())
		{
		  const CSSToken *tmp_token = tmp_node->get_term();
		  CSSTokenType tmp_type = tmp_token->get_type();
		  if (tmp_type == CSS_TOKEN_TYPE_DELIM
		      && tmp_token->get_delim() == '=')
		    attr_type = CSSAttrSelector::EQUALS;
		  else if (tmp_type == CSS_TOKEN_TYPE_INCLUDES)
		    attr_type = CSSAttrSelector::INCLUDES;
		  else if (tmp_type == CSS_TOKEN_TYPE_DASHMATCH)
		    attr_type = CSSAttrSelector::DASHMATCH;
		  else
		    tmp_node = NULL;	// 顼
		  if (tmp_node != NULL)
		    {
		      tmp_node = any_any[2];
		      if (tmp_node->is_term())
			{
			  tmp_token = tmp_node->get_term();
			  if (tmp_token->is_ident())
			    attr_value = tmp_token->get_ident();
			  else if (tmp_token->is_string())
			    attr_value = tmp_token->get_string();
			}
		    }
		}
	    }
	  CSSAttrSelector *attr;
	  if (attr_name != NULL && attr_value != NULL)
	    attr = new CSSAttrSelector(attr_name, attr_type, attr_value);
	  else if (attr_name != NULL)
	    attr = new CSSAttrSelector(attr_name);
	  else
	    continue;

	  if (type != CSSSelector::SINGLE)
	    {
	      CSSSelector *desc = new CSSSelector();
	      current->set_descendant(desc, type);
	      current = desc;
	    }
	  else if (current == NULL)
	    {
	      current = new CSSSelector();
	      //current->set_name("*");
	      assert(top == NULL);
	      top = current;
	    }
	  current->add_attr(attr);
	  continue;
	}
      if (token_type == CSS_TOKEN_TYPE_HASH)
	{
	  if (type != CSSSelector::SINGLE)
	    {
	      CSSSelector *desc = new CSSSelector();
	      current->set_descendant(desc, type);
	      current = desc;
	    }
	  else if (current == NULL)
	    {
	      current = new CSSSelector();
	      //current->set_name("*");
	      assert(top == NULL);
	      top = current;
	    }
	  current->set_hash(token->get_hash());
	  continue;
	}
      if (token_type == CSS_TOKEN_TYPE_DELIM)
	{
	  char delim = token->get_delim();
	  switch (delim)
	    {
	    case '*':
	      if (current == NULL)
		{
		  current = new CSSSelector();
		  //current->set_name("*");
		  assert(top == NULL);
		  top = current;
		  type = CSSSelector::SINGLE;
		}
	      continue;

	    case ',':
	      if (top != NULL)
		{
		  top->calculate_specificity();
		  selectors.push_back(top);
		}
	      top = NULL;
	      current = NULL;
	      type = CSSSelector::SINGLE;
	      continue;

	    case '>':
	      type = CSSSelector::CHILD;
	      if (current == NULL)
		{
		  current = new CSSSelector();
		  //current->set_name("*");
		  assert(top == NULL);
		  top = current;
		}
	      continue;

	    case '+':
	      type = CSSSelector::ADJACENT;
	      if (current == NULL)
		{
		  current = new CSSSelector();
		  //current->set_name("*");
		  assert(top == NULL);
		  top = current;
		}
	      continue;

	    case '.':
	      {
		const CSSNodeAny *class_any = *iter++;
		const CSSToken *class_token = class_any->get_leader();
		if (class_token->is_ident())
		  {
		    const char *class_name = class_token->get_ident();
		    if (type != CSSSelector::SINGLE)
		      {
			CSSSelector *desc = new CSSSelector();
			current->set_descendant(desc, type);
			current = desc;
		      }
		    else if (current == NULL)
		      {
			current = new CSSSelector();
			//current->set_name("*");
			assert(top == NULL);
			top = current;
		      }
		    current->set_class(class_name);
		  }
		else
		  {
		    // 顼ʤɤΤġġ
		  }
		continue;
	      }

	    case ':':
	      {
		const CSSNodeAny *name_any = *iter++;
		const CSSToken *name_token = name_any->get_leader();
		if (name_token->is_ident())
		  {
		    const char *name = name_token->get_ident();
		    if (type != CSSSelector::SINGLE)
		      {
			CSSSelector *desc = new CSSSelector();
			current->set_descendant(desc, type);
			current = desc;
			//current->set_name("*");
		      }
		    else if (current == NULL)
		      {
			current = new CSSSelector();
			//current->set_name("*");
			assert(top == NULL);
			top = current;
		      }
		    current->add_pseudo(name);
		  }
		else
		  {
		    // lang()Ȥ뤬ݤʤΤб
		    // ʳʤ饨顼ʤ̵롣
		    current->add_pseudo("NOT-SUPPORTED");
		  }
		continue;
	      }
	    }
	}
    }
  if (top != NULL)
    {
      top->calculate_specificity();
      selectors.push_back(top);
    }
}


StyleSelector::~StyleSelector()
{
  while (!selectors.empty())
    {
      CSSSelector *sel = selectors.back();
      delete sel;
      selectors.pop_back();
    }
}


char *
StyleSelector::to_string() const
{
  char tmp_buf[64];
  string buf;
  vector<CSSSelector *>::const_iterator iter = selectors.begin();
  vector<CSSSelector *>::const_iterator end_iter = selectors.end();
  while (iter != end_iter)
    {
      const CSSSelector *sel = *iter++;
      char *tmp_str = sel->to_string();
      buf.append(tmp_str);
      free(tmp_str);
      snprintf(tmp_buf, 64, "{%d}", sel->get_specificity());
      buf.append(tmp_buf);
      if (iter != end_iter)
	buf.append(", ");
    }
  return strdup(buf.c_str());
}


int
StyleSelector::match(const StyleMatcher *matcher)
{
  int result = -1;
  vector<CSSSelector *>::iterator iter = selectors.begin();
  vector<CSSSelector *>::iterator end_iter = selectors.end();
  while (iter != end_iter)
    {
      CSSSelector *selector = *iter++;
      int specificity = selector->calculate_specificity();
      if (specificity > result && selector->match(matcher) != NULL)
	result = specificity;
    }

  return result;
}


StyleRuleset::StyleRuleset(const CSSNodeRuleset *ruleset)
  : selector(ruleset->get_selector()),
    style(ruleset->get_declarations())
{
}


StyleRuleset::~StyleRuleset()
{
}


char *
StyleRuleset::to_string() const
{
  string buf("StyleRuleset {\n  selector: ");
  char *tmp_str = selector.to_string();
  buf.append(tmp_str);
  free(tmp_str);

  buf.append("\n  style: ");
  tmp_str = style.to_string();
  buf.append(tmp_str);
  free(tmp_str);

  buf.append("\n}\n");
  return strdup(buf.c_str());
}


int
StyleRuleset::calculate_specificity(const StyleMatcher *matcher)
{
  return selector.match(matcher);
}


StyleMatcher::StyleMatcher(StyleMatcher *context,
			   const char *element_name, const char *id,
			   const char *klass, const vector<char *> &attrs,
			   CSSPseudoClass pseudo_class,
			   CSSPseudoElement pseudo_element)
  : context(context),
    element_name(strdup(element_name)),
    id(id != NULL && *id != '\0' ? strdup(id) : NULL),
    klass(klass != NULL && *klass != '\0' ? strdup(klass) : NULL),
    pseudo_class(pseudo_class),
    pseudo_element(pseudo_element),
    next(NULL)
{
  vector<char *>::const_iterator iter = attrs.begin();
  vector<char *>::const_iterator end_iter = attrs.end();
  while (iter != end_iter)
    attributes.push_back(strdup(*iter++));
}


StyleMatcher::~StyleMatcher()
{
  free(element_name);

  if (id != NULL)
    free(id);
  if (klass != NULL)
    free(klass);

  if (next != NULL)
    delete next;

  while (!attributes.empty())
    {
      free(attributes.back());
      attributes.pop_back();
    }
}


StyleMatcher *
StyleMatcher::find(StyleMatcher *context,
		   const char *element_name, const char *id,
		   const char *klass, const vector<char *> &attrs,
		   CSSPseudoClass pseudo_class, CSSPseudoElement pseudo_element)
{
  assert(strcasecmp(element_name, this->element_name) == 0);
  assert((id == NULL && this->id == NULL)
	 || (id != NULL && this->id  != NULL && strcasecmp(id, this->id) == 0));
  assert((klass == NULL && this->klass == NULL)
	 || (klass != NULL && this->klass  != NULL
	     && strcasecmp(klass, this->klass) == 0));
  assert(pseudo_class == this->pseudo_class);
  assert(pseudo_element == this->pseudo_element);

  if (context == this->context)
    {
      if (attrs.empty() && attributes.empty())
	return this;

      if (!attrs.empty() && !attributes.empty())
	{
	  vector<char *>::const_iterator attrs_iter = attrs.begin();
	  vector<char *>::const_iterator attrs_end_iter = attrs.end();
	  vector<char *>::const_iterator this_iter = attributes.begin();
	  vector<char *>::const_iterator this_end_iter = attributes.end();
	  while (attrs_iter != attrs_end_iter && this_iter != this_end_iter)
	    {
	      if (strcmp(*attrs_iter, *this_iter) != 0)
		break;
	      ++attrs_iter;
	      ++this_iter;
	    }

	  if (attrs_iter == attrs_end_iter && this_iter == this_end_iter)
	    return this;
	}
    }

  if (next != NULL)
    return next->find(context, element_name, id, klass, attrs,
		      pseudo_class, pseudo_element);
  return NULL;
}


struct CompareSpecificity
{
  StyleMatcher *matcher;
  CompareSpecificity(StyleMatcher *matcher) : matcher(matcher) {}

  inline bool operator()(StyleRuleset *x, StyleRuleset *y)
  {
    return x->calculate_specificity(matcher)
      < y->calculate_specificity(matcher);
  }
};


void
StyleMatcher::match(StyleRuleset *ruleset)
{
  int specificity = ruleset->calculate_specificity(this);
  if (specificity == -1)
    return;

  list<StyleRuleset *> new_ruleset;
  new_ruleset.push_front(ruleset);

  style_rulesets.merge(new_ruleset, CompareSpecificity(this));
}


void
StyleMatcher::inherit(CSSValue &dst, const CSSValue &src)
{
  if (dst.is_value_set()
      && dst.is_keyword() && dst.get_keyword() == css_inherit)
    {
      if (src.is_value_set())
	dst.set_value(src);
      else
	dst.unset_value();
    }
}


void
StyleMatcher::unset_inherit(CSSValue &value)
{
  if (value.is_value_set()
      && value.is_keyword() && value.get_keyword() == css_inherit)
    value.unset_value();
}


StyleSheet::StyleSheet()
  : hashtable<const char *, StyleMatcher *>(101)
{
}


void
StyleMatcher::composite_style()
{
  if (context != NULL)
    style.composite(context->get_style());

  // 
  CSSValue &font_size = style.get_font_size();
  font_size.unset_value();

  list<StyleRuleset *>::iterator iter = style_rulesets.begin();
  list<StyleRuleset *>::iterator end_iter = style_rulesets.end();

  while (iter != end_iter)
    style.composite((*iter++)->get_style());

  if (!attributes.empty())
    {
      vector<char *>::iterator a_iter = attributes.begin();
      vector<char *>::iterator a_end_iter = attributes.end();
      while (a_iter != a_end_iter)
	{
	  const char *attr_name = *a_iter++;
	  assert(a_iter != a_end_iter);
	  if (strcasecmp(attr_name, "color") == 0)
	    {
	     if (strcasecmp(element_name, "body") == 0
		 || strcasecmp(element_name, "font") == 0)
	       {
		 style.get_color().set_color_string(*a_iter);
	       }
	    }
	  else if (strcasecmp(attr_name, "bgcolor") == 0)
	    {
	      if (strcasecmp(element_name, "body") == 0)
		{
		  style.get_background_color().set_color_string(*a_iter);
		}
	    }
	  else if (strcasecmp(attr_name, "size") == 0)
	    {
	      if (strcasecmp(element_name, "font") == 0)
		{
		  const char *size = *a_iter;
		  double scale = -1.0;
		  if (*size == '+' || *size == '-')
		    {
		      double factor;
		      if (*size == '+')
			factor = 1.2;
		      else
			factor = 1 / 1.2;
		      unsigned int level;
		      if (sscanf(size + 1, "%u", &level) == 1)
			{
			  scale = 1.0;
			  while (level-- > 0)
			    scale *= factor;
			}
		    }
		  else if (*size >= '1' && *size <= '6')
		    {
		      static double scales[] = {
			1 / (1.2 * 1.2),
			1 / 1.2,
			1.0,
			1.2,
			1.2 * 1.2,
			1.2 * 1.2 * 1,2
		      };
		      scale = scales[*size - '1'];
		    }
		  if (scale > 0.0)
		    font_size.set_dimension(scale, css_em);
		}
	    }

	  ++a_iter;
	}
    }

  if (font_size.is_value_set())
    {
      // StyleMatcherλĥäݤޤȤѥǤϡ
      // font-sizeͤ⤷UAɸΥեȥФ
      // Ȥ롣ˤ衢ƥ¸Υȸʤ
      // лξñ̤emƤ
      // лξñ̤pt⤷pxƤ
      if (font_size.is_keyword())
	{
	  CSSKeyword keyword = font_size.get_keyword();
	  if (keyword == css_inherit)
	    font_size.unset_value();
	  else if (keyword == css_xx_small)
	    font_size.set_dimension(1 / (1.2 * 1.2 * 1.2), css_em);
	  else if (keyword == css_x_small)
	    font_size.set_dimension(1 / (1.2 * 1.2), css_em);
	  else if (keyword == css_small)
	    font_size.set_dimension(1 / 1.2, css_em);
	  else if (keyword == css_medium)
	    font_size.set_dimension(1.0, css_em);
	  else if (keyword == css_large)
	    font_size.set_dimension(1.2, css_em);
	  else if (keyword == css_x_large)
	    font_size.set_dimension(1.2 * 1.2, css_em);
	  else if (keyword == css_xx_large)
	    font_size.set_dimension(1.2 * 1.2 * 1.2, css_em);
	  else
	    font_size.unset_value();
	}
      else if (font_size.is_dimension())
	{
	  CSSKeyword unit = font_size.get_dimension_unit();
	  if (unit == css_em || unit == css_ex)
	    {
	      // exΥݡȤϤΥ쥤䡼ǤΤ̵
	      // ơ̥쥤䡼ǤϤݤ򸫤ʤ
	      // emƱ뤷Ƥ
	      if (context != NULL)
		{
		  const CSSValue &parent_font_size
		    = context->get_style().get_font_size();

		  assert(parent_font_size.is_value_set());
		  assert(parent_font_size.is_dimension());

		  double val = font_size.get_dimension_value();
		  double p_val = parent_font_size.get_dimension_value();
		  CSSKeyword p_unit = parent_font_size.get_dimension_unit();
		  assert(p_unit != css_ex);

		  font_size.set_dimension(val * p_val, p_unit);
		}
	      else if (unit == css_ex)
		font_size.set_dimension(font_size.get_dimension_value(),
					css_em);
	    }
	  else if (unit == css_in)
	    {
	      // ptñ̤
	      font_size.set_dimension(font_size.get_dimension_value() * 72.0,
				      css_pt);
	    }
	  else if (unit == css_cm)
	    {
	      // ptñ̤
	      font_size.set_dimension(font_size.get_dimension_value() * 72.0
				      / 2.54, css_pt);
	    }
	  else if (unit == css_mm)
	    {
	      // ptñ̤
	      font_size.set_dimension(font_size.get_dimension_value() * 72.0
				      / 25.4, css_pt);
	    }
	  else if (unit == css_pc)
	    {
	      // ptñ̤
	      font_size.set_dimension(font_size.get_dimension_value() * 12.0,
				      css_pt);
	    }
	  else if (unit != css_pt && unit != css_px)
	    font_size.unset_value();	// ̵äȤˤ롣
	}
      else if (font_size.is_percentage())
	{
	  // emñ̤
	  font_size.set_dimension(font_size.get_percentage() / 100.0, css_em);
	}
      else
	font_size.unset_value();	// ̵äȤˤ롣
    }

  if (!font_size.is_value_set())
    {
      if (context != NULL)
	font_size.set_value(context->get_style().get_font_size());
      else
	font_size.set_dimension(1.0, css_em);
    }

  // inheritͤ
  if (context != NULL)
    {
      inherit(style.get_margin_top(), context->get_style().get_margin_top());
      inherit(style.get_margin_right(),
	      context->get_style().get_margin_right());
      inherit(style.get_margin_bottom(),
	      context->get_style().get_margin_bottom());
      inherit(style.get_margin_left(), context->get_style().get_margin_left());
      inherit(style.get_display(), context->get_style().get_display());
      inherit(style.get_direction(), context->get_style().get_direction());
      inherit(style.get_line_height(), context->get_style().get_line_height());
      inherit(style.get_vertical_align(),
	      context->get_style().get_vertical_align());
      inherit(style.get_list_style_image(),
	      context->get_style().get_list_style_image());
      inherit(style.get_list_style_position(),
	      context->get_style().get_list_style_position());
      inherit(style.get_list_style_type(),
	      context->get_style().get_list_style_type());
      inherit(style.get_color(), context->get_style().get_color());
      inherit(style.get_background_color(),
	      context->get_style().get_background_color());
      inherit(style.get_background_image(),
	      context->get_style().get_background_image());
      inherit(style.get_background_repeat(),
	      context->get_style().get_background_repeat());
      inherit(style.get_background_attachment(),
	      context->get_style().get_background_attachment());
      inherit(style.get_background_position_h(),
	      context->get_style().get_background_position_h());
      inherit(style.get_background_position_v(),
	      context->get_style().get_background_position_v());

      vector<CSSValue *> &content = style.get_content();
      if (!content.empty()
	  && content.front()->is_keyword()
	  && content.front()->get_keyword() == css_inherit)
	{
	  while (!content.empty())
	    {
	      delete content.back();
	      content.pop_back();
	    }

	  vector<CSSValue *>::const_iterator iter
	    = context->get_style().get_content_begin();
	  vector<CSSValue *>::const_iterator end_iter
	    = context->get_style().get_content_end();
	  while (iter != end_iter)
	    content.push_back(new CSSValue(*iter++));
	}

      vector<CSSValue *> &font_family = style.get_font_family();
      if (!font_family.empty()
	  && font_family.front()->is_keyword()
	  && font_family.front()->get_keyword() == css_inherit)
	{
	  while (!font_family.empty())
	    {
	      delete font_family.back();
	      font_family.pop_back();
	    }

	  vector<CSSValue *>::const_iterator iter
	    = context->get_style().get_font_family_begin();
	  vector<CSSValue *>::const_iterator end_iter
	    = context->get_style().get_font_family_end();
	  while (iter != end_iter)
	    font_family.push_back(new CSSValue(*iter++));
	}

      inherit(style.get_font_style(), context->get_style().get_font_style());
      inherit(style.get_font_variant(),
	      context->get_style().get_font_variant());
      inherit(style.get_font_weight(), context->get_style().get_font_weight());

      inherit(style.get_text_indent(), context->get_style().get_text_indent());
      inherit(style.get_text_align(), context->get_style().get_text_align());
      inherit(style.get_text_decoration(),
	      context->get_style().get_text_decoration());
      inherit(style.get_text_transform(),
	      context->get_style().get_text_transform());
      inherit(style.get_unicode_bidi(),
	      context->get_style().get_unicode_bidi());
      inherit(style.get_white_space(), context->get_style().get_white_space());
    }
  else
    {
      unset_inherit(style.get_margin_top());
      unset_inherit(style.get_margin_right());
      unset_inherit(style.get_margin_bottom());
      unset_inherit(style.get_margin_left());
      unset_inherit(style.get_display());
      unset_inherit(style.get_direction());
      unset_inherit(style.get_line_height());
      unset_inherit(style.get_vertical_align());
      unset_inherit(style.get_list_style_image());
      unset_inherit(style.get_list_style_position());
      unset_inherit(style.get_list_style_type());
      unset_inherit(style.get_color());
      unset_inherit(style.get_background_color());
      unset_inherit(style.get_background_image());
      unset_inherit(style.get_background_repeat());
      unset_inherit(style.get_background_attachment());
      unset_inherit(style.get_background_position_h());
      unset_inherit(style.get_background_position_v());

      vector<CSSValue *> &content = style.get_content();
      if (!content.empty()
	  && content.front()->is_keyword()
	  && content.front()->get_keyword() == css_inherit)
	{
	  while (!content.empty())
	    {
	      delete content.back();
	      content.pop_back();
	    }
	}

      vector<CSSValue *> &font_family = style.get_font_family();
      if (!font_family.empty()
	  && font_family.front()->is_keyword()
	  && font_family.front()->get_keyword() == css_inherit)
	{
	  while (!font_family.empty())
	    {
	      delete font_family.back();
	      font_family.pop_back();
	    }
	}

      unset_inherit(style.get_font_style());
      unset_inherit(style.get_font_variant());
      unset_inherit(style.get_font_weight());
      unset_inherit(style.get_text_indent());
      unset_inherit(style.get_text_align());
      unset_inherit(style.get_text_decoration());
      unset_inherit(style.get_text_transform());
      unset_inherit(style.get_unicode_bidi());
      unset_inherit(style.get_white_space());
    }
}


StyleSheet::~StyleSheet()
{
  reset_rulesets();
}


StyleSheet::Entry::Entry(unsigned int hash, const char *key,
			 StyleMatcher *value, HashtableEntry *next)
  : HashtableEntry(hash, strdup(key), value, next)
{
}


StyleSheet::Entry::~Entry()
{
  if (value != NULL)
    delete value;

  free(const_cast<char *>(key));
}


StyleMatcher *
StyleSheet::Entry::set_value(StyleMatcher *value)
{
  StyleMatcher *prev = HashtableEntry::set_value(value);
  assert(value->get_next() == NULL);
  value->set_next(prev);
  return value;
}


unsigned int
StyleSheet::calculate_hash(const char *key) const
{
  return cstring_case_insensitive_hash(key);
}


bool
StyleSheet::equals(const char *key1, const char *key2) const
{
  return strcasecmp(key1, key2) == 0;
}


StyleSheet::HashtableEntry *
StyleSheet::entry_new(unsigned int hash, const char *key,
		      StyleMatcher *value,
		      StyleSheet::HashtableEntry *next)
{
  return new Entry(hash, key, value, next);
}


void
StyleSheet::reset_rulesets()
{
  while (!rulesets.empty())
    {
      StyleRuleset *ruleset = rulesets.back();
      delete ruleset;
      rulesets.pop_back();
    }

  clear();
}


static int
count_newline(const char *src, int length)
{
  const char *cur_pos = src;
  const char *tail_pos = src + length;
  int count = 0;
  while (cur_pos < tail_pos)
    {
      char c = *cur_pos;
      if (c == '\n' || c == '\f')
	++count;
      else if (c == '\r')
	{
	  ++count;
	  if (cur_pos + 1 && cur_pos[1] == '\n')
	    ++cur_pos;
	}
      ++cur_pos;
    }
  return count;
}


void
StyleSheet::parse_style_sheet(const char *style_sheet, int length)
{
  assert(style_sheet != NULL);

  const char *cur_pos = style_sheet;
  vector<CSSNode *> statements;
  CSSNode *statement = NULL;
  int lineno = 1;
  while (length > 0)
    {
      CSSToken *token = CSSToken::tokenize(cur_pos, length);
      if (token == NULL)
	break;

      int len = token->get_length();
      int n_nl = count_newline(cur_pos, len);
      cur_pos += len;
      length -= len;
      CSSTokenType type = token->get_type();
      if (type == CSS_TOKEN_TYPE_COMMENT)
	{
	  delete token;
	  continue;
	}

      if (statement != NULL)
	{
	  if (!statement->eat(token, lineno))
	    {
	      statements.push_back(statement);
	      statement = NULL;
	    }
	}

      if (statement == NULL)
	{
	  switch (type)
	    {
	    case CSS_TOKEN_TYPE_CDO:
	    case CSS_TOKEN_TYPE_CDC:
	    case CSS_TOKEN_TYPE_SPACE:
	      delete token;
	      break;

	    case CSS_TOKEN_TYPE_ATKEYWORD:
	      statement = new CSSNodeAtRule(token, lineno);
	      break;

	    default:
	      statement = new CSSNodeRuleset(lineno);
	      if (!statement->eat(token, lineno))
		{
		  char *tmp_str = token->to_string();
		  fprintf(stderr, "Unexpected Token at line %d: %s\n",
			  lineno, tmp_str);
		  free(tmp_str);
		  delete token;
		  delete statement;
		  statement = NULL;
		}
	    }
	}
      lineno += n_nl;
    }
  if (statement != NULL)
    statements.push_back(statement);
  statement = NULL;

  vector<CSSNode *>::iterator iter = statements.begin();
  vector<CSSNode *>::iterator end_iter = statements.end();
  while (iter != end_iter)
    {
      CSSNode *s = (*iter++);
      if (s->is_ruleset())
	{
	  rulesets.push_back(new StyleRuleset(dynamic_cast<CSSNodeRuleset *>(s)));
	  assert(rulesets.back() != NULL);
	}
#ifdef DEBUG_CSS_PARSER
      else
	{
	  char *tmp_str = s->to_string();
	  fprintf(stderr, "*** toplevel statement ***\n%s\n\n", tmp_str);
	  free(tmp_str);
	}
#endif
      delete s;
    }

#ifdef DEBUG_CSS_PARSER
  vector<StyleRuleset *>::iterator rule_iter = rulesets.begin();
  vector<StyleRuleset *>::iterator rule_end_iter = rulesets.end();
  while (rule_iter != rule_end_iter)
    {
      char *tmp_str = (*rule_iter++)->to_string();
      fprintf(stderr, "*** TOPLEVEL Ruleset ***\n%s\n", tmp_str);
      free(tmp_str);
    }
#endif
#if 0
  fprintf(stderr, "sizeof(CSSValue)=%d, sizeof(CSSStyle)=%d\n",
	  sizeof(CSSValue), sizeof(CSSStyle));
#endif
}


const StyleMatcher *
StyleSheet::lookup_style(const StyleMatcher *context,
			 const char *element_name,
			 const char *const attrs[],
			 CSSPseudoClass pseudo_class,
			 CSSPseudoElement pseudo_element)
{
  if (element_name == NULL)
    return NULL;

  vector<char *> attributes;
  const char *klass = "";
  const char *id = "";
  if (attrs != NULL)
    {
      const char *const *cur_attr = attrs;
      while (*cur_attr != NULL && cur_attr[1] != NULL)
	{
	  if (strcasecmp(*cur_attr, "class") == 0)
	    klass = cur_attr[1];
	  else if (strcasecmp(*cur_attr, "id") == 0
		   || strcasecmp(*cur_attr, "name") == 0)
	    id = cur_attr[1];
	  else if (strcasecmp(*cur_attr, "href") != 0
		   && strcasecmp(*cur_attr, "src") != 0)
	    {
	      // href°src°̵롣
	      attributes.push_back(const_cast<char *>(*cur_attr));
	      attributes.push_back(const_cast<char *>(cur_attr[1]));
	    }
	  cur_attr += 2;
	}
    }

  char key_buf[256];
  if (snprintf(key_buf, 256, "%d%d%s.%s#%s",
	       pseudo_class, pseudo_element, element_name, klass, id) < 256)
    {
      if (*id == '\0')
	id = NULL;
      if (*klass == '\0')
	klass = NULL;

      StyleMatcher *matcher = lookup(key_buf);
      if (matcher != NULL)
	matcher = matcher->find(const_cast<StyleMatcher *>(context),
				element_name, id, klass, attributes,
				pseudo_class, pseudo_element);
      if (matcher != NULL)
	return matcher;

      matcher = new StyleMatcher(const_cast<StyleMatcher *>(context),
				 element_name, id, klass, attributes,
				 pseudo_class, pseudo_element);

      vector<StyleRuleset *>::const_iterator iter = rulesets.begin();
      vector<StyleRuleset *>::const_iterator end_iter = rulesets.end();
      while (iter != end_iter)
	matcher->match(*iter++);
      matcher->composite_style();

#if 0
      string buf;
      buf.append("StyleMatcher for \"%s\"");
      vector<char *>::const_iterator a_iter = attributes.begin();
      vector<char *>::const_iterator a_end_iter = attributes.end();
      while (a_iter != a_end_iter)
	{
	  buf.push_back(' ');
	  buf.append(*a_iter++);
	  if (a_iter != a_end_iter)
	    buf.append("=\"").append(*a_iter++).push_back('"');
	}
      buf.push_back('\n');
      char *style = matcher->get_style().to_string();
      if (style != NULL)
	{
	  buf.append(style);
	  free(style);
	}
      fprintf(stderr, buf.c_str(), key_buf);
      fprintf(stderr, "\n");
#endif

      insert(key_buf, matcher);
      return matcher;
    }

  return NULL;
}

}
