/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: style_sheet.h,v 1.10 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _STYLE_SHEET_H_
#define _STYLE_SHEET_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/hashtable.h"

#include <cassert>
#include <list>
#include <string>
#include <vector>


using namespace std;
namespace Ochusha
{
  typedef const char * CSSKeyword;	// Ѥ뤫⤷ʤΤ
#ifndef _STYLE_SHEET_CC_
  extern CSSKeyword css_auto;
  extern CSSKeyword css_none;
  extern CSSKeyword css_normal;
  extern CSSKeyword css_inherit;

  extern CSSKeyword css_em;
  extern CSSKeyword css_ex;
  extern CSSKeyword css_px;
  extern CSSKeyword css_in;
  extern CSSKeyword css_cm;
  extern CSSKeyword css_mm;
  extern CSSKeyword css_pt;
  extern CSSKeyword css_pc;
  extern CSSKeyword css_ms;
  extern CSSKeyword css_s;

  extern CSSKeyword css_maroon;
  extern CSSKeyword css_red;
  extern CSSKeyword css_orange;
  extern CSSKeyword css_yellow;
  extern CSSKeyword css_olive;
  extern CSSKeyword css_purple;
  extern CSSKeyword css_fuchsia;
  extern CSSKeyword css_white;
  extern CSSKeyword css_lime;
  extern CSSKeyword css_green;
  extern CSSKeyword css_navy;
  extern CSSKeyword css_blue;
  extern CSSKeyword css_aqua;
  extern CSSKeyword css_teal;
  extern CSSKeyword css_black;
  extern CSSKeyword css_silver;
  extern CSSKeyword css_gray;

  extern CSSKeyword css_transparent;

  extern CSSKeyword css_repeat;
  extern CSSKeyword css_repeat_x;
  extern CSSKeyword css_repeat_y;
  extern CSSKeyword css_no_repeat;

  extern CSSKeyword css_scroll;
  extern CSSKeyword css_fixed;

  extern CSSKeyword css_open_quote;
  extern CSSKeyword css_close_quote;
  extern CSSKeyword css_no_open_quote;
  extern CSSKeyword css_no_close_quote;

  extern CSSKeyword css_ltr;
  extern CSSKeyword css_rtl;

  extern CSSKeyword css_inline;
  extern CSSKeyword css_block;
  extern CSSKeyword css_list_item;
  extern CSSKeyword css_run_in;
  extern CSSKeyword css_inline_block;
  extern CSSKeyword css_table;
  extern CSSKeyword css_inline_table;
  extern CSSKeyword css_table_row_group;
  extern CSSKeyword css_table_header_group;
  extern CSSKeyword css_table_footer_group;
  extern CSSKeyword css_table_row;
  extern CSSKeyword css_table_column_group;
  extern CSSKeyword css_table_column;
  extern CSSKeyword css_table_cell;
  extern CSSKeyword css_table_caption;

  extern CSSKeyword css_serif;
  extern CSSKeyword css_sans_serif;
  extern CSSKeyword css_cursive;
  extern CSSKeyword css_fantasy;
  extern CSSKeyword css_monospace;

  extern CSSKeyword css_caption;
  extern CSSKeyword css_icon;
  extern CSSKeyword css_menu;
  extern CSSKeyword css_message_box;
  extern CSSKeyword css_small_caption;
  extern CSSKeyword css_status_bar;

  extern CSSKeyword css_xx_small;
  extern CSSKeyword css_x_small;
  extern CSSKeyword css_small;
  extern CSSKeyword css_medium;
  extern CSSKeyword css_large;
  extern CSSKeyword css_x_large;
  extern CSSKeyword css_xx_large;
  extern CSSKeyword css_smaller;
  extern CSSKeyword css_larger;

  extern CSSKeyword css_italic;
  extern CSSKeyword css_oblique;

  extern CSSKeyword css_small_caps;

  extern CSSKeyword css_bold;
  extern CSSKeyword css_bolder;
  extern CSSKeyword css_lighter;

  extern CSSKeyword css_inside;
  extern CSSKeyword css_outside;

  extern CSSKeyword css_disc;
  extern CSSKeyword css_circle;
  extern CSSKeyword css_square;
  extern CSSKeyword css_decimal;
  extern CSSKeyword css_decimal_leading_zero;
  extern CSSKeyword css_lower_roman;
  extern CSSKeyword css_upper_roman;
  extern CSSKeyword css_lower_greek;
  extern CSSKeyword css_lower_latin;
  extern CSSKeyword css_upper_latin;
  extern CSSKeyword css_armenian;
  extern CSSKeyword css_georgian;
  extern CSSKeyword css_lower_alpha;
  extern CSSKeyword css_upper_alpha;

  extern CSSKeyword css_left;
  extern CSSKeyword css_right;
  extern CSSKeyword css_center;
  extern CSSKeyword css_justify;

  extern CSSKeyword css_underline;
  extern CSSKeyword css_overline;
  extern CSSKeyword css_line_through;
  extern CSSKeyword css_blink;

  extern CSSKeyword css_capitalize;
  extern CSSKeyword css_uppercase;
  extern CSSKeyword css_lowercase;

  extern CSSKeyword css_embed;
  extern CSSKeyword css_bidi_override;

  extern CSSKeyword css_baseline;
  extern CSSKeyword css_sub;
  extern CSSKeyword css_super;
  extern CSSKeyword css_top;
  extern CSSKeyword css_text_top;
  extern CSSKeyword css_middle;
  extern CSSKeyword css_bottom;
  extern CSSKeyword css_text_bottom;

  extern CSSKeyword css_pre;
  extern CSSKeyword css_nowrap;
  extern CSSKeyword css_pre_wrap;
  extern CSSKeyword css_pre_line;
#endif


  class CSSNode;
  class CSSNodeBlock;
  class CSSNodeRuleset;
  class CSSNodeValue;

  class CSSValue
  {
  public:
    enum AcceptableValueType {
      CSS_BACKGROUND_COLOR = 0,
      CSS_BACKGROUND_IMAGE,
      CSS_BACKGROUND_REPEAT,
      CSS_BACKGROUND_ATTACHMENT,
      CSS_BACKGROUND_POSITION_H,
      CSS_BACKGROUND_POSITION_V,
      CSS_COLOR,
      CSS_CONTENT,
      CSS_COUNTER_RESET,
      CSS_COUNTER_INCREMENT,
      CSS_DIRECTION,
      CSS_DISPLAY,
      CSS_FONT_FAMILY,
      CSS_FONT_SIZE,
      CSS_FONT_STYLE,
      CSS_FONT_VARIANT,
      CSS_FONT_WEIGHT,
      CSS_LINE_HEIGHT,
      CSS_LIST_STYLE_IMAGE,
      CSS_LIST_STYLE_POSITION,
      CSS_LIST_STYLE_TYPE,
      CSS_MARGIN_WIDTH,
      CSS_TEXT_ALIGN,
      CSS_TEXT_DECORATION,
      CSS_TEXT_INDENT,
      CSS_TEXT_TRANSFORM,
      CSS_UNICODE_BIDI,
      CSS_VERTICAL_ALIGN,
      CSS_WHITE_SPACE
    };


  private:
    union {
      struct { double value; CSSKeyword unit; } dimension;
      double number;
      int integer;
      CSSKeyword keyword;
      char *string;	// char *uri; char *color; char *attr_ref;
      struct { unsigned char r; unsigned char g; unsigned char b; } rgb;
      struct { char *id; int value; } counter_def;
      struct { char *id; char *string; CSSKeyword style; } counter_ref;
      struct { unsigned int underline : 1; unsigned int overline : 1;
	unsigned int line_through : 1; unsigned int blink : 1; } decoration;
    } value;

    enum ValueType {
      INTEGER_VALUE = 0,
      NUMBER_VALUE,
      PERCENTAGE_VALUE,
      DIMENSION_VALUE,
      URI_VALUE,
      KEYWORD_VALUE,
      FONT_STRING_VALUE,
      COLOR_RGB_VALUE,
      COLOR_STRING_VALUE,
      STRING_VALUE,
      TEXT_DECORATION_VALUE,
      ATTR_REF_VALUE,
      COUNTER_DEF_VALUE,
      COUNTER_REF_VALUE
    } value_type : 7;
    AcceptableValueType acceptable_value_type : 24;
    unsigned int value_set : 1;

    void free_value();


  public:
    CSSValue(AcceptableValueType acceptable_value_type);
    CSSValue(const CSSValue *src);
    ~CSSValue();
    inline bool is_value_set() const { return value_set != 0; }
    void set_initial();
    void unset_value();

    char *to_string() const;
    void append_string(string &buf) const;

    bool set_value(const CSSNode *src);
    bool set_value(const CSSValue &src);
    bool set_value(vector<CSSNode *>::const_iterator &iter,
		   vector<CSSNode *>::const_iterator &end_iter);
    bool set_keyword(CSSKeyword val);
    bool set_integer(int val);
    bool set_dimension(double val, CSSKeyword unit);
    bool set_percentage(double val);
    bool set_color_string(const char *val);

    inline bool is_integer() const
    { return value_set != 0 && value_type == INTEGER_VALUE; }
    inline int get_integer() const
    { return is_integer() ? value.integer : 0; }

    inline bool is_number() const
    { return value_set != 0 && value_type == NUMBER_VALUE; }
    inline double get_number() const
    { return is_number() ? value.number : 0.0; }

    inline bool is_percentage() const
    { return value_set != 0 && value_type == PERCENTAGE_VALUE; }
    inline double get_percentage() const
    { return is_percentage() ? value.number : 0.0; }

    inline bool is_dimension() const
    { return value_set != 0 && value_type == DIMENSION_VALUE; }
    inline double get_dimension_value() const
    { return is_dimension() ? value.dimension.value : 0.0; }
    inline CSSKeyword get_dimension_unit() const
    { return is_dimension() ? value.dimension.unit : NULL; }

    inline bool is_uri() const
    { return value_set != 0 && value_type == URI_VALUE; }
    inline const char *get_uri() const
    { return is_uri() ? value.string : NULL; }

    inline bool is_keyword() const
    { return value_set != 0 && value_type == KEYWORD_VALUE; }
    inline CSSKeyword get_keyword() const
    { return is_keyword() ? value.keyword : NULL; }

    inline bool is_font_string() const
    { return value_set != 0 && value_type == FONT_STRING_VALUE; }
    inline const char *get_font_string() const
    { return is_font_string() ? value.string : NULL; }

    inline bool is_color_rgb() const
    { return value_set != 0 && value_type == COLOR_RGB_VALUE; }
    inline unsigned char get_color_rgb_red() const
    { return is_color_rgb() ? value.rgb.r : 0; }
    inline unsigned char get_color_rgb_green() const
    { return is_color_rgb() ? value.rgb.g : 0; }
    inline unsigned char get_color_rgb_blue() const
    { return is_color_rgb() ? value.rgb.b : 0; }

    inline bool is_color_string() const
    { return value_set != 0 && value_type == COLOR_STRING_VALUE; }
    inline const char *get_color_string() const
    { return is_color_string() ? value.string : NULL; }

    inline bool is_text_decoration() const
    { return value_set != 0 && value_type == TEXT_DECORATION_VALUE; }
    inline bool get_underline() const
    { return is_text_decoration() ? value.decoration.underline != 0 : false; }
    inline bool get_overline() const
    { return is_text_decoration() ? value.decoration.overline != 0 : false; }
    inline bool get_line_through() const
    { return is_text_decoration()
	? value.decoration.line_through != 0 : false; }
    inline bool get_blink() const
    { return is_text_decoration() ? value.decoration.blink != 0 : false; }

    inline bool is_attr_ref() const
    { return value_set != 0 && value_type == ATTR_REF_VALUE; }
    inline const char *get_attr_ref() const
    { return is_attr_ref() ? value.string : NULL; }

    inline bool is_counter_def() const
    { return value_set != 0 && value_type == COUNTER_DEF_VALUE; }
    inline const char *get_counter_def_id() const
    { return is_counter_def() ? value.counter_def.id : NULL; }
    inline int get_counter_def_value() const
    { return is_counter_def() ? value.counter_def.value : 0; }

    inline bool is_counter_ref() const
    { return value_set != 0 && value_type == COUNTER_REF_VALUE; }
    inline const char *get_counter_ref_id() const
    { return is_counter_ref() ? value.counter_ref.id : NULL; }
    inline const char *get_counter_ref_string() const
    { return is_counter_ref() ? value.counter_ref.string : NULL; }
    inline CSSKeyword get_counter_ref_style() const
    { return is_counter_ref() ? value.counter_ref.style : NULL; }
  };


  // style sheetβη̡ġΥ쥯ˤ򤵤륹
  // (specified values)ɽ롣
  class CSSStyle
  {
  private:
    CSSValue margin_top;
    CSSValue margin_right;
    CSSValue margin_bottom;
    CSSValue margin_left;

    CSSValue display;
    CSSValue direction;

    CSSValue line_height;

    CSSValue vertical_align;

    CSSValue list_style_image;
    CSSValue list_style_position;
    CSSValue list_style_type;

    CSSValue color;
    CSSValue background_color;
    CSSValue background_image;
    CSSValue background_repeat;
    CSSValue background_attachment;
    CSSValue background_position_h;
    CSSValue background_position_v;

    vector<CSSValue *> content;

    vector<CSSValue *> font_family;
    CSSValue font_style;
    CSSValue font_variant;
    CSSValue font_weight;
    CSSValue font_size;

    CSSValue text_indent;

    CSSValue text_align;
    CSSValue text_decoration;
    CSSValue text_transform;

    CSSValue unicode_bidi;
    CSSValue white_space;

    CSSStyle *important_style;

    void parse_margin(const vector<CSSNode *> &values);
    void parse_font(const vector<CSSNode *> &values);
    void parse_background(const vector<CSSNode *> &values);

    void set_property(const char *name, const CSSNodeValue *value);
    void initialize_style(const CSSNodeBlock *declaration_block,
			  bool important);
    CSSStyle(const CSSNodeBlock *declaration_block, bool important);


  public:
    CSSStyle(const CSSNodeBlock *declaration_block);
    CSSStyle();
    ~CSSStyle();

    void composite(const CSSStyle &style);
    char *to_string() const;

    inline CSSValue &get_margin_top() { return margin_top; }
    inline CSSValue &get_margin_right() { return margin_right; }
    inline CSSValue &get_margin_bottom() { return margin_bottom; }
    inline CSSValue &get_margin_left() { return margin_left; }

    inline CSSValue &get_display() { return display; }
    inline CSSValue &get_direction() { return direction; }

    inline CSSValue &get_line_height() { return line_height; }

    inline CSSValue &get_vertical_align() { return vertical_align; }

    inline CSSValue &get_list_style_image() { return list_style_image; }
    inline CSSValue &get_list_style_position() { return list_style_position; }
    inline CSSValue &get_list_style_type() { return list_style_type; }

    inline CSSValue &get_color() { return color; }
    inline CSSValue &get_background_color() { return background_color; }
    inline CSSValue &get_background_image() { return background_image; }
    inline CSSValue &get_background_repeat() { return background_repeat; }
    inline CSSValue &get_background_attachment()
    { return background_attachment; }
    inline CSSValue &get_background_position_h()
    { return background_position_h; }
    inline CSSValue &get_background_position_v()
    { return background_position_v; }

    inline vector<CSSValue *> &get_content() { return content; }
    inline vector<CSSValue *>::iterator get_content_begin()
    { return content.begin(); }
    inline vector<CSSValue *>::iterator get_content_end()
    { return content.end(); }

    inline vector<CSSValue *> &get_font_family() { return font_family; }
    inline vector<CSSValue *>::iterator get_font_family_begin()
    { return font_family.begin(); }
    inline vector<CSSValue *>::iterator get_font_family_end()
    { return font_family.end(); }
    inline CSSValue &get_font_style() { return font_style; }
    inline CSSValue &get_font_variant() { return font_variant; }
    inline CSSValue &get_font_weight() { return font_weight; }
    inline CSSValue &get_font_size() { return font_size; }

    inline CSSValue &get_text_indent() { return text_indent; }

    inline CSSValue &get_text_align() { return text_align; }
    inline CSSValue &get_text_decoration() { return text_decoration; }
    inline CSSValue &get_text_transform() { return text_transform; }

    inline CSSValue &get_unicode_bidi() { return unicode_bidi; }
    inline CSSValue &get_white_space() { return white_space; }

    inline CSSStyle *get_important_style() { return important_style; }


    inline const CSSValue &get_margin_top() const { return margin_top; }
    inline const CSSValue &get_margin_right() const { return margin_right; }
    inline const CSSValue &get_margin_bottom() const { return margin_bottom; }
    inline const CSSValue &get_margin_left() const { return margin_left; }

    inline const CSSValue &get_display() const { return display; }
    inline const CSSValue &get_direction() const { return direction; }

    inline const CSSValue &get_line_height() const { return line_height; }

    inline const CSSValue &get_vertical_align() const { return vertical_align; }

    inline const CSSValue &get_list_style_image() const
    { return list_style_image; }
    inline const CSSValue &get_list_style_position() const
    { return list_style_position; }
    inline const CSSValue &get_list_style_type() const
    { return list_style_type; }

    inline const CSSValue &get_color() const { return color; }
    inline const CSSValue &get_background_color() const
    { return background_color; }
    inline const CSSValue &get_background_image() const
    { return background_image; }
    inline const CSSValue &get_background_repeat() const
    { return background_repeat; }
    inline const CSSValue &get_background_attachment() const
    { return background_attachment; }
    inline const CSSValue &get_background_position_h() const
    { return background_position_h; }
    inline const CSSValue &get_background_position_v() const
    { return background_position_v; }

    inline vector<CSSValue *>::const_iterator get_content_begin() const
    { return content.begin(); }
    inline vector<CSSValue *>::const_iterator get_content_end() const
    { return content.end(); }

    inline vector<CSSValue *>::const_iterator get_font_family_begin() const
    { return font_family.begin(); }
    inline vector<CSSValue *>::const_iterator get_font_family_end() const
    { return font_family.end(); }
    inline const CSSValue &get_font_style() const { return font_style; }
    inline const CSSValue &get_font_variant() const { return font_variant; }
    inline const CSSValue &get_font_weight() const { return font_weight; }
    inline const CSSValue &get_font_size() const { return font_size; }

    inline const CSSValue &get_text_indent() const { return text_indent; }

    inline const CSSValue &get_text_align() const { return text_align; }
    inline const CSSValue &get_text_decoration() const
    { return text_decoration; }
    inline const CSSValue &get_text_transform() const { return text_transform; }

    inline const CSSValue &get_unicode_bidi() const { return unicode_bidi; }
    inline const CSSValue &get_white_space() const { return white_space; }

    inline const CSSStyle *get_important_style() const
    { return important_style; }
  };


  enum CSSPseudoClass {
    CSS_PSEUDO_CLASS_NO = 0,
    CSS_PSEUDO_CLASS_LINK,
    CSS_PSEUDO_CLASS_VISITED
  };


  enum CSSPseudoElement {
    CSS_PSEUDO_ELEMENT_NO = 0,
    CSS_PSEUDO_ELEMENT_BEFORE,
    CSS_PSEUDO_ELEMENT_AFTER
  };


  class StyleRuleset;


  class StyleMatcher
  {
  private:
    StyleMatcher *context;
    char *element_name;
    char *id;
    char *klass;
    CSSPseudoClass pseudo_class;
    CSSPseudoElement pseudo_element;

    vector<char *> attributes;

    // ޥå륹specificityξ¤٤ꥹ
    list<StyleRuleset *> style_rulesets;
    CSSStyle style;	// ޥå륹ŬѤ̤Υ


    // singly linked listˤƥϥåơ֥ϿƤ
    StyleMatcher *next;


    void inherit(CSSValue &dst, const CSSValue &src);
    void unset_inherit(CSSValue &value);


  public:
    StyleMatcher(StyleMatcher *context, const char *element_name,
		 const char *id, const char *klass,
		 const vector<char *> &attrs, CSSPseudoClass pseudo_class,
		 CSSPseudoElement pseudo_element);
    ~StyleMatcher();

    inline const StyleMatcher *get_context() const { return context; }
    inline const char *get_element_name() const { return element_name; }
    inline const char *get_id() const { return id; }
    inline const char *get_klass() const { return klass; }
    inline CSSPseudoClass get_pseudo_class() const { return pseudo_class; }
    inline CSSPseudoElement get_pseudo_element() const { return pseudo_element; }
    inline vector<char *>::const_iterator get_attributes_begin() const
    { return attributes.begin(); }
    inline vector<char *>::const_iterator get_attributes_end() const
    { return attributes.end(); }

    inline const CSSStyle &get_style() const { return style; }


    // StyleSheetμ
    inline StyleMatcher *get_next() { return next; }
    inline void set_next(StyleMatcher *matcher) { next = matcher; }
    StyleMatcher *find(StyleMatcher *context,
		       const char *element_name, const char *id,
		       const char *klass, const vector<char *> &attrs,
		       CSSPseudoClass pseudo_class,
		       CSSPseudoElement pseudo_element);
    void match(StyleRuleset *ruleset);
    void composite_style();
  };


  class CSSNodeAny;
  class CSSSelector;


  class StyleSelector
  {
  private:
    vector<CSSSelector *> selectors;


  public:
    StyleSelector(const vector<CSSNodeAny *> &selector);
    ~StyleSelector();
    char *to_string() const;
    int match(const StyleMatcher *matcher);
  };


  class StyleRuleset
  {
  private:
    StyleSelector selector;
    CSSStyle style;		// cascadespecified valuesɽ


  public:
    StyleRuleset(const CSSNodeRuleset *ruleset);
    ~StyleRuleset();
    char *to_string() const;
    inline const CSSStyle &get_style() const { return style; }

    // matcherФspecificity׻֤
    // ʣselector˥ޥåˤϺspecificity֤
    // matchʤ-1֤
    int calculate_specificity(const StyleMatcher *matcher);
  };


  class StyleSheet : public hashtable<const char *, StyleMatcher *>
  {
  private:
    vector<StyleRuleset *> rulesets;


  protected:
    struct Entry : public HashtableEntry
    {
      Entry(unsigned int hash, const char *key, StyleMatcher *value,
	    HashtableEntry *next);
      virtual ~Entry();
      virtual StyleMatcher *set_value(StyleMatcher *value);
    };


    virtual unsigned int calculate_hash(const char *key) const;
    virtual bool equals(const char *key1, const char *key2) const;
    virtual HashtableEntry *entry_new(unsigned int hash, const char *key,
				      StyleMatcher *value,
				      HashtableEntry *next);


  public:
    StyleSheet();
    ~StyleSheet();
    void reset_rulesets();
    void parse_style_sheet(const char *style_sheet, int length);

    const StyleMatcher *lookup_style(const StyleMatcher *context,
				     const char *element_name,
				     const char *const attrs[],
				     CSSPseudoClass pseudo_class,
				     CSSPseudoElement pseudo_element);
  };


}


#endif	/* __cplusplus */

#endif	/* _STYLE_SHEET_H_ */
