/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: threadlist_ui.cc,v 1.20 2009/01/01 07:47:07 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "threadlist_ui.h"

#include "controller.h"
#include "threadlist_view.h"


namespace Ochusha
{

ThreadlistUI::ThreadlistUI(Controller &controller)
  : controller(controller),
    action_group(create_action_group())
{
}


ThreadlistUI::~ThreadlistUI()
{
  g_object_unref(action_group);
}


GtkActionGroup *
ThreadlistUI::create_action_group()
{
  static GtkActionEntry entries[] = {
    {
      "OpenTrackbackWithWebBrowser", GTK_STOCK_EXECUTE,
      N_("Open Track_back Page with Web Browser"), "",
      N_("Open Trackback Page with Web Browser"),
      reinterpret_cast<GCallback>(open_trackback_with_web_browser_cb)
    },
    {
      "MarkThread", NULL,
      N_("_Mark Thread"), NULL,
      N_("Mark Thread"),
      reinterpret_cast<GCallback>(mark_thread_cb)
    },
    {
      "UnmarkThread", NULL,
      N_("_Unmark Thread"), NULL,
      N_("Unmark Thread"),
      reinterpret_cast<GCallback>(unmark_thread_cb)
    },
    {
      "HideThread", NULL,
      N_("_Hide"), NULL,
      N_("Hide"),
      reinterpret_cast<GCallback>(hide_thread_cb)
    },
    {
      "ToggleThreadHiddenState", NULL,
      N_("Toggle _Hide"), NULL,
      N_("Toggle Hide"),
      reinterpret_cast<GCallback>(toggle_hide_thread_cb)
    },
    {
      "ResetThread", GTK_STOCK_DELETE,
      N_("_Reset"), "",
      N_("Reset"),
      reinterpret_cast<GCallback>(reset_thread_cb)
    },
    {
      "CopyThreadURL", GTK_STOCK_COPY,
      N_("_Copy URL"), "",
      N_("Copy URL"),
      reinterpret_cast<GCallback>(copy_url_cb)
    },
    {
      "CopyThreadTitle", GTK_STOCK_COPY,
      N_("Copy Title"), "",
      NULL,
      reinterpret_cast<GCallback>(copy_title_cb)
    },
    {
      "CopyThreadTitleURL", GTK_STOCK_COPY,
      N_("Copy Title _and URL"), "",
      N_("Copy Title and URL"),
      reinterpret_cast<GCallback>(copy_title_url_cb)
    },
    {
      "ThreadProperties", GTK_STOCK_PROPERTIES,
      N_("_Properties"), "",
      N_("Properties"),
      reinterpret_cast<GCallback>(properties_cb)
    }
  };

  action_group = gtk_action_group_new("ThreadlistActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  open_trackback_with_web_browser
    = gtk_action_group_get_action(action_group, "OpenTrackbackWithWebBrowser");
  mark_thread = gtk_action_group_get_action(action_group, "MarkThread");
  unmark_thread = gtk_action_group_get_action(action_group, "UnmarkThread");
  hide_thread = gtk_action_group_get_action(action_group, "HideThread");
  toggle_hide_thread
    = gtk_action_group_get_action(action_group, "ToggleThreadHiddenState");
  reset_thread = gtk_action_group_get_action(action_group, "ResetThread");
  copy_url = gtk_action_group_get_action(action_group, "CopyThreadURL");
  copy_title = gtk_action_group_get_action(action_group, "CopyThreadTitle");
  copy_url = gtk_action_group_get_action(action_group, "CopyThreadTitleURL");
  properties = gtk_action_group_get_action(action_group, "ThreadProperties");


  gtk_action_group_set_sensitive(action_group, FALSE);

  g_signal_connect(controller.get_clipboard_internal(), "owner-change",
		   reinterpret_cast<GCallback>(owner_change_cb), this);

  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  return action_group;
}


void
ThreadlistUI::owner_change_cb(GtkClipboard *clipboard, GdkEvent *event,
			      ThreadlistUI *ui)
{
  gboolean selection_available
    = ThreadlistView::internal_list_of_threads_available(clipboard);

  gtk_action_group_set_sensitive(ui->action_group, selection_available);

  bool show_hidden = ui->controller.get_prefs().get_show_hidden_threads();
  gtk_action_set_sensitive(ui->hide_thread,
			   !show_hidden && selection_available);
  gtk_action_set_sensitive(ui->toggle_hide_thread,
			   show_hidden && selection_available);
}


void
ThreadlistUI::tree_selection_open_trackback_with_web_browser(GtkTreeModel *model,
							     GtkTreePath *path,
							     GtkTreeIter *iter,
							     gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  ThreadlistUI *ui = static_cast<ThreadlistUI *>(data);
  if (!thread->is_trackback_supported())
    return;

  char *url = thread->get_trackback_url();
  ui->controller.open(url, true, true, true);
  free(url);
}


void
ThreadlistUI::open_trackback_with_web_browser_cb(GtkAction *action,
						 ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_open_trackback_with_web_browser, ui);
}


void
ThreadlistUI::tree_selection_mark(GtkTreeModel *model, GtkTreePath *path,
				  GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  thread->set_marked(true);
  gtk_tree_model_row_changed(model, path, iter);
}


void
ThreadlistUI::mark_thread_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_mark, ui);
}


void
ThreadlistUI::tree_selection_unmark(GtkTreeModel *model, GtkTreePath *path,
				    GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  thread->set_marked(false);
  gtk_tree_model_row_changed(model, path, iter);
}


void
ThreadlistUI::unmark_thread_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_unmark, ui);
}


struct RemoveArgs
{
  list<GtkTreeRowReference *> rows_to_be_removed;
};


void
ThreadlistUI::tree_selection_hide(GtkTreeModel *model, GtkTreePath *path,
				    GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  thread->set_hidden(true);
  RemoveArgs *args = static_cast<RemoveArgs *>(data);
  args->rows_to_be_removed.push_front(gtk_tree_row_reference_new(model, path));
}


void
ThreadlistUI::hide_thread_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  RemoveArgs args;
  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_hide, &args);

  list<GtkTreeRowReference *>::iterator iter = args.rows_to_be_removed.begin();
  list<GtkTreeRowReference *>::iterator end_iter = args.rows_to_be_removed.end();
  GtkTreeView *tree_view = gtk_tree_selection_get_tree_view(selection);
  GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
  GtkListStore *store = GTK_LIST_STORE(model);
  while (iter != end_iter)
    {
      GtkTreeRowReference *ref = *iter;
      GtkTreePath *path = gtk_tree_row_reference_get_path(ref);
      if (path != NULL)
	{
	  GtkTreeIter tree_iter;
	  gtk_tree_model_get_iter(model, &tree_iter, path);
	  gtk_list_store_remove(store, &tree_iter);
	  gtk_tree_path_free(path);
	}
      gtk_tree_row_reference_free(ref);
      iter++;
    }
}


void
ThreadlistUI::tree_selection_toggle_hide(GtkTreeModel *model, GtkTreePath *path,
					 GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  thread->set_hidden(!thread->is_hidden());
  gtk_tree_model_row_changed(model, path, iter);
}


void
ThreadlistUI::toggle_hide_thread_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_toggle_hide, ui);
}


void
ThreadlistUI::tree_selection_reset(GtkTreeModel *model, GtkTreePath *path,
				   GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  ThreadlistUI *ui = static_cast<ThreadlistUI *>(data);
  thread->reset(ui->controller.get_repository());
  gtk_tree_model_row_changed(model, path, iter);
}


void
ThreadlistUI::reset_thread_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  gtk_tree_selection_selected_foreach(selection, tree_selection_reset, ui);
}


struct CopyArgs
{
  ThreadlistUI *ui;
  string text;

  CopyArgs(ThreadlistUI *ui) : ui(ui) {}
};


void
ThreadlistUI::tree_selection_copy_url(GtkTreeModel *model, GtkTreePath *path,
				      GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  CopyArgs *args = static_cast<CopyArgs *>(data);
  if (args->text.size() > 0)
    args->text.append("\n");
  char *url = thread->get_url();
  args->text.append(url);
  free(url);
}


void
ThreadlistUI::copy_url_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  CopyArgs args(ui);
  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_copy_url, &args);
  ui->controller.copy(args.text.c_str());
}


void
ThreadlistUI::tree_selection_copy_title(GtkTreeModel *model, GtkTreePath *path,
					GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  CopyArgs *args = static_cast<CopyArgs *>(data);
  if (args->text.size() > 0)
    args->text.append("\n");
  args->text.append(thread->get_title());
}


void
ThreadlistUI::copy_title_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);

  CopyArgs args(ui);
  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_copy_title, &args);
  ui->controller.copy(args.text.c_str());
}


void
ThreadlistUI::tree_selection_copy_title_url(GtkTreeModel *model,
					    GtkTreePath *path,
					    GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  CopyArgs *args = static_cast<CopyArgs *>(data);
  if (args->text.size() > 0)
    args->text.append("\n");
  args->text.append(thread->get_title()).append("\n");

  char *url = thread->get_url();
  args->text.append(url);
  free(url);
}


void
ThreadlistUI::copy_title_url_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);


  CopyArgs args(ui);
  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_copy_title_url, &args);
  ui->controller.copy(args.text.c_str());
}


void
ThreadlistUI::tree_selection_properties(GtkTreeModel *model, GtkTreePath *path,
					GtkTreeIter *iter, gpointer data)
{
  BBSThread *thread = ThreadlistView::get_thread(model, iter);
  g_return_if_fail(thread != NULL);

  ThreadlistUI *ui = static_cast<ThreadlistUI *>(data);
  ui->open_properties(thread);
}


void
ThreadlistUI::properties_cb(GtkAction *action, ThreadlistUI *ui)
{
  GtkTreeSelection *selection
    = ThreadlistView::get_gtk_tree_selection_from_clipboard(ui->controller.get_clipboard_internal());
  g_return_if_fail(selection != NULL);


  gtk_tree_selection_selected_foreach(selection,
				      tree_selection_properties, ui);
}


void
ThreadlistUI::open_properties(BBSThread *thread)
{
  controller.get_thread_ui().open_properties(thread);
}


void
ThreadlistUI::prepare_session()
{
}

}
