/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree_view.h,v 1.15 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _TREE_VIEW_H_
#define _TREE_VIEW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "focus_utils.h"
#include "gtk_utils.h"
#include "selection_utils.h"
#include "tree_ui_impl.h"
#include "view.h"

#include "libochushagtk/searchable.h"
#include "libochushagtk/tree_selector_view.h"

#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{


  class SelectedRow
  {
  private:
    FolderHandle folder;
    TreeElementHandle element;
    TreePathHandle path;


  public:
    SelectedRow(Folder *folder, TreeElement *element, GtkTreePath *path)
      : folder(folder),	element(element), path(gtk_tree_path_copy(path))
    {}
    SelectedRow(const SelectedRow &src)
      : folder(src.folder),
	element(src.element),
	path(gtk_tree_path_copy(src.path))
    {}
    inline Folder *get_folder() const { return folder; }
    inline TreeElement *get_element() const { return element; }
    inline GtkTreePath *get_path() const { return path; }
  };


  class TreeViewSelectionData : public SelectionData
  {
  private:
    TreeUIImpl &impl;
    GtkTreeSelection *selection;
    bool is_static;

    vector<SelectedRow> rows;

    vector<TreeElement *> non_folders;

    static void accumulate_dnd_data(GtkTreeModel *model, GtkTreePath *path,
				    GtkTreeIter *iter, gpointer data);
    static void accumulate_elements(GtkTreeModel *model, GtkTreePath *path,
				    GtkTreeIter *iter, gpointer data);
    static void changed_cb(GtkTreeSelection *selection,
			   TreeViewSelectionData *data);


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    TreeViewSelectionData(TreeUIImpl &impl, GtkTreeSelection *selection,
			  bool is_static);
    virtual ~TreeViewSelectionData();

    virtual bool deletable();
    virtual bool cuttable();
    virtual bool copiable();
    virtual bool pastable(GtkClipboard *clipboard);

    virtual bool do_delete();
    virtual bool do_paste(GtkClipboard *clipboard);

    // ¸thisΥԡäݤΤ֤
    virtual SelectionData *snapshot();

    bool set_data(GtkSelectionData *data, guint info, GdkAtom type);
    bool is_selected(GtkTreePath *path);

    inline TreeUIImpl &get_impl() { return impl; }
    inline vector<SelectedRow>::const_iterator rows_begin() const
    { return rows.begin(); }
    inline vector<SelectedRow>::const_iterator rows_end() const
    { return rows.end(); }

    SelectionData *create_proxy();

    void toggle_hide();
    void hide();
    void open_properties();
  };


  class TreeView;

  class TreeViewFocusNotifier : public FocusNotifier
  {
  private:
    TreeView &view;


  protected:
    virtual gboolean notify_focus_in();
    virtual void notify_unmap();


  public:
    TreeViewFocusNotifier(TreeView &view, GtkWidget *widget);
    virtual ~TreeViewFocusNotifier();
  };


  class SessionClient;

  class TreeView : public View
  {
    friend class BookmarksUI;
    friend class BoardlistUI;
    friend class TreeUI;
    friend class TreeUIImpl;
    friend class TreeViewFocusNotifier;
    friend class TreeViewSelectionData;

  protected:
    TreeUIImpl &impl;

    gchar *title;
    GtkWidget *tree_view;

    TreeViewSelectionData selection_data;


  private:
    TreeViewFocusNotifier focus_notifier;
    TreePathHandle last_event_path;
    gint press_start_x;
    gint press_start_y;
    guint press_start_state;
    guint press_start_button;
    GdkEventType press_start_type;

    guint popup_delay_timeout_id;

    bool is_master_view;


    static gboolean internal_list_of_elements_available(GtkClipboard *clipboard);
    static TreeUIImpl *get_tree_ui_impl_from_clipboard(GtkClipboard *clipboard);

    static void selection_changed_cb(GtkTreeSelection *selection,
				     TreeView *view);

    static void row_changed_cb(GtkTreeModel *model, GtkTreePath *path,
			       GtkTreeIter *iter, TreeView *view);

    static gboolean start_interactive_search_cb(OchushaTreeSelectorView *view,
						TreeView *tree_view);

    View *activate_view_at_cursor();

    static gboolean key_press_event_cb(OchushaTreeSelectorView *widget,
				       GdkEventKey *event, TreeView *view);

    static gboolean popup_menu_cb(GtkWidget *widget, TreeView *view);

    static void row_expanded_cb(GtkTreeView *tree_view, GtkTreeIter *iter,
				GtkTreePath *path, TreeView *view);
    static void row_collapsed_cb(GtkTreeView *tree_view, GtkTreeIter *iter,
				 GtkTreePath *path, TreeView *view);
    static gboolean row_button_press_cb(OchushaTreeSelectorView *tree_view,
					GtkTreePath *path,
					GtkTreeViewColumn *column,
					GdkEventButton *event,
					TreeView *view);
    static gboolean motion_notify_event_cb(GtkWidget *widget,
					   GdkEventMotion *event,
					   TreeView *view);
    static gboolean popup_delay_timeout_cb(TreeView *view);
    static gboolean row_button_release_cb(OchushaTreeSelectorView *tree_view,
					  GtkTreePath *path,
					  GtkTreeViewColumn *column,
					  GdkEventButton *event,
					  TreeView *view);

    static void column_set_icon(GtkTreeViewColumn *column,
				GtkCellRenderer *renderer,
				GtkTreeModel *model, GtkTreeIter *iter,
				gpointer data);
    static void column_set_name(GtkTreeViewColumn *column,
				GtkCellRenderer *renderer,
				GtkTreeModel *model, GtkTreeIter *iter,
				gpointer data);


    // DnDݡȤμ¸Ƥ
    void initialize_for_drag_source();

    static void drag_data_get_cb(GtkWidget *widget,
				 GdkDragContext *context,
				 GtkSelectionData *data,
				 guint info, guint time,
				 TreeView *view);
    static void drag_data_delete_cb(GtkWidget *widget, GdkDragContext *context,
				    TreeView *view);
    static void drag_end_cb(GtkWidget *widget, GdkDragContext *context,
			    TreeView *view);

    void initialize_for_drag_dest();
    static gboolean drag_drop_cb(GtkWidget *widget, GdkDragContext *context,
				 gint x, gint y, guint time, TreeView *view);
    static void drag_data_received_cb(GtkWidget *widget,
				      GdkDragContext *context,
				      gint x, gint y, GtkSelectionData *data,
				      guint info, guint time,
				      TreeView *view);
    // ޤDnDϢ


  protected:
    virtual void popup_context_menu(GtkTreeSelection *selection, guint button,
				    guint32 event_time);
    virtual bool activate_element(TreeElement *element, GdkEventType event_type,
				  guint button);

    virtual void column_set_icon(GtkCellRendererPixbuf *renderer,
				 TreeElement *element, gboolean expanded);
    virtual void column_set_name(GtkCellRendererText *renderer,
				 TreeElement *element);


  public:
    TreeView(GtkTreeModel *model, const gchar *title, TreeUIImpl &impl,
	     bool is_master_view);
    virtual ~TreeView();

    inline GtkWidget *get_tree_view() const { return tree_view; }
    inline GtkTreeSelection *get_selection() const
    { return gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)); }

    void restore_session(const char *name, SessionClient &session);
    void prepare_session(const char *name, SessionClient &session);


    // Viewvirtualμ
    virtual const gchar *get_combo_title();
    virtual const gchar *get_menu_title();
    virtual const gchar *get_window_title();
    virtual const gchar *get_url();
    virtual void grab_focus();
    virtual bool contains(GtkWidget *child);
    virtual const gchar *get_search_title();
    virtual OchushaSearchable *get_search_target();
    virtual void notify_closed();
    // Viewvirtualμ
  };


  class FolderTreeView : public TreeView
  {
  protected:
    virtual void popup_context_menu(GtkTreeSelection *selection, guint button,
				    guint32 event_time);


  public:
    FolderTreeView(GtkTreeModel *model, const gchar *title, TreeUIImpl &impl);
  };


}


#endif	/* __cplusplus */

#endif	/* _TREE_VIEW_H_ */
