/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: view.cc,v 1.4 2009/01/01 07:47:07 fuyu Exp $
 */

#include "config.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "view.h"

#include "controller.h"
#include "main_window.h"
#include "tab_label.h"

#include <cassert>

#include <gtk/gtk.h>


//#define VERBOSE_DESTRUCTOR	1


namespace Ochusha
{


static GQuark view_id = g_quark_from_static_string("View::view");


View::View(TabLabel *tab_label, GtkWidget *widget)
  : tab_label(tab_label),
    label(tab_label != NULL ? tab_label->get_widget() : NULL),
    widget(widget),
    main_window(NULL),
    state(NORMAL)
{
  assert(GTK_IS_WIDGET(widget));

  g_signal_connect(widget, "destroy",
		   reinterpret_cast<GCallback>(destroy_cb), this);
  g_object_set_qdata(G_OBJECT(widget), view_id, this);
}



View::View(GtkWidget *label, GtkWidget *widget)
  : tab_label(NULL),
    label(label),
    widget(widget),
    main_window(NULL),
    state(NORMAL)
{
  assert(GTK_IS_WIDGET(widget));

  g_signal_connect(widget, "destroy",
		   reinterpret_cast<GCallback>(destroy_cb), this);
  g_object_set_qdata(G_OBJECT(widget), view_id, this);
}



View::~View()
{
#ifdef VERBOSE_DESTRUCTOR
  fprintf(stderr, "~View(): %p\n", this);
#endif

  backward_history_clear();
  forward_history_clear();

  assert(state == NORMAL);
}


void
View::destroy_cb(GtkWidget *widget, View *view)
{
  assert(widget == view->widget);

  if (view->main_window != NULL)
    view->main_window->notify_view_vanished(view);

  delete view;
}


void
View::request_search()
{
  g_return_if_fail(main_window != NULL);

  main_window->setup_search(this);
}


void
View::notify_attached(MainWindow *window)
{
  assert(main_window == NULL);

  main_window = window;
}


void
View::notify_detached(MainWindow *window)
{
  assert(main_window == window);

  main_window = NULL;
}


void
View::notify_focus_in()
{
#if 0
  g_return_if_fail(main_window != NULL);
#else
  // notify_attached()θƤ˺Ǥʤ¤ꡢNULLʤ̵ɤ
  // ơΤȤƤ˺ϤʤϤġġ
  if (main_window == NULL)
    return;
#endif

  // main_windowthisViewܡɥեȤ롣
  // η̡main_windowϥȥСNavigationToolbarURL
  // thisView褦ꤹ롣
  main_window->notify_current_focus_view(this);
}


const gchar *
View::get_window_title()
{
  return NULL;
}


const gchar *
View::get_url()
{
  return NULL;
}


bool
View::bookmarkable()
{
  return false;
}


bool
View::bookmark()
{
  g_return_val_if_fail(!bookmarkable(), false);

  return false;
}


bool
View::refreshable()
{
  return false;
}


bool
View::refresh()
{
  g_return_val_if_fail(!refreshable(), false);

  return false;
}


bool
View::has_info()
{
  return false;
}


bool
View::show_info()
{
  g_return_val_if_fail(!has_info(), false);

  return false;
}


char *
View::get_current_view_url()
{
  const gchar *url = get_url();
  return url != NULL ? strdup(reinterpret_cast<const char *>(url)) : NULL;
}


void
View::backward_history_push(char *url)
{
  g_return_if_fail(url != NULL);

  if (!backward_history.empty() && strcmp(backward_history.front(), url) == 0)
    {
      free(url);	// ݤưʤ̵롣
      return;
    }

  backward_history.push_front(url);

  if (main_window != NULL)
    main_window->notify_state_changed(this);
}


char *
View::backward_history_pop()
{
  g_return_val_if_fail(!backward_history.empty(), NULL);

  char *url = backward_history.front();
  backward_history.pop_front();

  if (main_window != NULL)
    main_window->notify_state_changed(this);

  return url;
}


void
View::backward_history_clear()
{
  while (!backward_history.empty())
    {
      free(backward_history.front());
      backward_history.pop_front();
    }

  if (main_window != NULL)
    main_window->notify_state_changed(this);
}


void
View::snatch_backward_history(View *view)
{
  assert(backward_history.empty());
  backward_history = view->backward_history;
  view->backward_history.clear();

  if (main_window != NULL)
    main_window->notify_state_changed(this);

  if (main_window != NULL)
    main_window->notify_state_changed(view);
}


void
View::forward_history_push(char *url)
{
  g_return_if_fail(url != NULL);

  if (!forward_history.empty() && strcmp(forward_history.front(), url) == 0)
    {
      free(url);	// ݤưʤ̵롣
      return;
    }

  forward_history.push_front(url);

  if (main_window != NULL)
    main_window->notify_state_changed(this);
}


char *
View::forward_history_pop()
{
  g_return_val_if_fail(!forward_history.empty(), NULL);

  char *url = forward_history.front();
  forward_history.pop_front();

  if (main_window != NULL)
    main_window->notify_state_changed(this);

  return url;
}


void
View::forward_history_clear()
{
  while (!forward_history.empty())
    {
      free(forward_history.front());
      forward_history.pop_front();
    }

  if (main_window != NULL)
    main_window->notify_state_changed(this);
}


void
View::snatch_forward_history(View *view)
{
  assert(forward_history.empty());
  forward_history = view->forward_history;
  view->forward_history.clear();

  if (main_window != NULL)
    main_window->notify_state_changed(this);

  if (main_window != NULL)
    main_window->notify_state_changed(view);
}


void
View::go_back_view()
{
  g_return_if_fail(has_back_view());

  forward_history_push(get_current_view_url());

  char *target_url = backward_history_pop();
  g_return_if_fail(target_url != NULL);

  if (move_view(target_url))
    {
      // ɽViewǤΰưξ
      free(target_url);
      return;
    }

  // ɽViewؤΰưξ
  assert(main_window != NULL);
  Controller &controller = main_window->controller;

  state = GOING_BACK;
  controller.open(target_url, false, false, false, true);

  // λthisdeleteƤǽ⤤

  free(target_url);
  return;
}


void
View::go_forward_view()
{
  g_return_if_fail(has_forward_view());

  backward_history_push(get_current_view_url());

  char *target_url = forward_history_pop();
  g_return_if_fail(target_url != NULL);

  if (move_view(target_url))
    {
      // ɽViewǤΰưξ
      free(target_url);
      return;
    }

  // ɽViewؤΰưξ
  assert(main_window != NULL);
  Controller &controller = main_window->controller;

  state = GOING_FORWARD;
  controller.open(target_url, false, false, false, true);

  // λthisdeleteƤǽ⤤

  free(target_url);
  return;
}


bool
View::move_view(const char *target_url)
{
  return false;
}


View *
View::get_view(GtkWidget *widget)
{
  View *view = static_cast<View *>(g_object_get_qdata(G_OBJECT(widget),
						      view_id));
  g_return_val_if_fail(view != NULL, NULL);
  g_return_val_if_fail(view->widget == widget, NULL);

  return view;
}


}
