/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: view.h,v 1.3 2009/01/01 07:47:07 fuyu Exp $
 */

#ifndef _VIEW_H_
#define _VIEW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochushagtk/searchable.h"

#include <list>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  class MainWindow;
  class TabLabel;

  class View
  {
    friend class ViewManager;

  private:
    TabLabel *tab_label;
    GtkWidget *label;
    GtkWidget *widget;

    // ViewɽƤ륦ɥMainWindowؤΥݥ
    MainWindow *main_window;

    list<char *> backward_history;	// 뤿URL
    list<char *> forward_history;	// ʤिURL

    enum {
      NORMAL = 0,
      GOING_BACK,
      GOING_FORWARD
    } state;


    // View֥ȤwidgetȰϡ
    static void destroy_cb(GtkWidget *widget, View *view);


  protected:
    virtual ~View();


    void backward_history_push(char *url);
    char *backward_history_pop();
    void backward_history_clear();

    // viewbackward_historythiscopyơviewbackward_history򥯥ꥢ
    void snatch_backward_history(View *view);


    void forward_history_push(char *url);
    char *forward_history_pop();
    void forward_history_clear();

    // viewforward_historythiscopyơviewforward_history򥯥ꥢ
    void snatch_forward_history(View *view);


    // target_urlViewURLξ礽URLɽ֤ư
    // true֤ViewURLǤνcallerǤ롣
    virtual bool move_view(const char *target_url);


  public:
    // View֥ȤwidgetȰϡwidgetdestroyʥ
    // delete뤳Ȥա
    View(TabLabel *tab_label, GtkWidget *widget);
    View(GtkWidget *label, GtkWidget *widget);

    inline TabLabel *get_tab_label() const { return tab_label; }
    inline GtkWidget *get_label() const { return label; }
    inline GtkWidget *get_widget() const { return widget; }

    void request_search();

    void notify_attached(MainWindow *window);
    void notify_detached(MainWindow *window);


    // MEMO: GTK+ǥܡɥեμƻ뤹Τ
    // ̤񤷤ǡViewΥ֥饹ˤϼʬδˤ
    // GtkWidgetܡɥեˤΥ᥽åɤ
    // ƤӽФȤְۤε̳פݤ
    void notify_focus_in();


    // View֥ڡȤɽˡGtkComboBoxɽ
    // ٤ʸ֤Хå
    virtual const gchar *get_combo_title() = 0;


    // View֥ڡȤɽˡ˥塼ɽ
    // ٤ʸ֤Хå
    virtual const gchar *get_menu_title() = 0;


    // ViewܡɥեƤʤɡ
    // Τ濴ŪǤˤʤäƤ˥ȥС˷Ǽ
    // ٤ʸꤹ뤿˸ƤФ륳Хå
    // μNULL֤Υ᥽åɤΥ桼ǤMainWindow
    // NULLξˤϥȥСѹʤ
    virtual const gchar *get_window_title();


    // ViewܡɥեƤʤɡ
    // Τ濴ŪǤˤʤäƤNavigationToolbar
    // URLȥ˷Ǽ٤URLꤹ뤿˸ƤФ륳Хå
    // μNULL֤Υ᥽åɤΥ桼ǤMainWindow
    // NULLξˤURLȥѹʤ
    virtual const gchar *get_url();


    // ViewɽGtkWidget⡢ViewܡɥեȤ
    // ˤơ֤ˤäߤGtkWidgetˤĤgtk_grab_focus()
    // 륳Хå

    // MEMO: ŵŪˤget_widget()֤GtkWidgetGtkScrolledWindowä
    // ꤹ뤻Ǥ˥ܡɥեͿƤUIλȤ꤬ɤ
    // ʤȤ顢GtkScrolledWindowʤ餽λҶġŪad-hoc
    // ʼ򤷤Ƥʬ뤿ѰդΡ
    virtual void grab_focus() = 0;


    // childǻؤ줿GtkWidgetView񤹤ΤǤ뤫ɤȽꤹ
    // Хå

    // MEMO: ochusha_find_focus_widget()μäƤ褦ˡGtkWidget
    // οƻҴطõȤ⤢뤬餯ViewΤΤȤμ
    // μѤȽꤹΨɤ˲äݥåץåɽ
    // ξʤɡGTK+Ūʰ̣ǤοƻҴطƤؤнɬס
    virtual bool contains(GtkWidget *child) = 0;


    // Viewؤʸ󸡺򤹤ݤ˥ơСɽ٤о
    // ɽʸ֤ХåNULL֤ViewϸоݤȤʤʤ
    virtual const gchar *get_search_title() = 0;


    // Viewؤʸ󸡺򤹤ݤѤ륳Хå
    virtual OchushaSearchable *get_search_target() = 0;


    // Viewget_widget()֤GtkWidgetõ˸ƤФ륳Хå
    // Ĵ档
    virtual void notify_closed() = 0;


    // ƼGtkActionμ᥽å
    virtual bool bookmarkable();
    virtual bool bookmark();
    virtual bool refreshable();
    virtual bool refresh();

    virtual bool has_info();
    virtual bool show_info();


    // ViewɽƤΤξ֤ɽURL֤
    //
    // ǤviewportΥ쥹URLʳĤURLʤɤ
    // Τޤ֤
    //
    // URLϿ˳Ƥ줿ʸȤ֤פˤʤäcaller
    // free()͡
    virtual char *get_current_view_url();


    // ءפоݤȤʤ򤬤true֤
    inline bool has_back_view() const { return !backward_history.empty(); }

    void go_back_view();

    // ּءפоݤȤʤ򤬤true֤
    inline bool has_forward_view() const { return !forward_history.empty(); }

    void go_forward_view();


    // widget˴Ϣդ줿View֥ȤؤΥݥ󥿤֤
    // ϢդƤʤˤNULL֤
    static View *get_view(GtkWidget *widget);
  };


}


#endif	/* __cplusplus */

#endif	/* _VIEW_H_ */
