/* -*- c++ -*-
 * Copyright (c) 2003-2009 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: view_manager.cc,v 1.53 2010/08/15 15:42:20 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "view_manager.h"

#include "libochusha/bbs_thread.h"
#include "libochusha/bulletin_board.h"

#include "bbs_thread_view.h"
#include "controller.h"
#include "gtk_utils.h"
#include "preferences.h"
#include "session_client.h"
#include "ui_constants.h"
#include "view.h"

#include "libochushagtk/arrow_action.h"
#include "libochushagtk/combo_action.h"
#if 0	// OchushaHVPanedϰͽ
#include "libochushagtk/hvpaned.h"
#endif
#include "libochushagtk/icon_label.h"
#include "libochushagtk/stock.h"


#include <gtk/gtk.h>
#include <glib.h>

#include <cassert>
#include <cstdlib>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif


// ܥܥåɽ륢Ѳؤб򥳡Хå̩
// ȤʤݤΤǡХå椵줿ݡ󥰤н衣
// ݡ󥰤ΤߤѲ̵Ĺ³褦ʾˤʤ̵̤ˤʤ롣
#define ICON_POLLING_INTERVAL	50	// 50ms¸


namespace Ochusha
{

GQuark ViewManager::bulletin_board_id
  = g_quark_from_static_string("ViewManager::BulletinBoard");

GQuark ViewManager::bbs_thread_id
  = g_quark_from_static_string("ViewManager::BBSThread");


enum {
  VIEW_WIDGET = 0,
  VIEW_OBJECT,
  NOTEBOOK_ITEM_N_COLUMNS
};


static void
combo_changed_cb(OchushaComboAction *action,
		 GtkTreeModel *model, GtkTreeIter *iter, GtkTreePath *path,
		 ModNotebook *notebook)
{
  GtkWidget *widget = NULL;
  gtk_tree_model_get(model, iter, VIEW_WIDGET, &widget, -1);
  g_return_if_fail(widget != NULL);

  gint page = mod_notebook_page_num(notebook, widget);
  g_return_if_fail(page != -1);

  mod_notebook_set_current_page(notebook, page);
}


static void
set_label_text(GtkMenuItem *item, GtkLabel *label,
	       GtkTreeModel *model, GtkTreeIter *iter,
	       gpointer unused)
{
  View *view = NULL;
  gtk_tree_model_get(model, iter, VIEW_OBJECT, &view, -1);

  const gchar *menu_title = NULL;
  if (view != NULL)
    menu_title = view->get_menu_title();

  if (menu_title == NULL)
    menu_title = _("Anonymous");

  gtk_label_set_text(label, menu_title);
}


static void
set_combo_text(GtkCellLayout *layout, GtkCellRenderer *renderer,
	       GtkTreeModel *model, GtkTreeIter *iter, gpointer unused)
{
  View *view = NULL;
  gtk_tree_model_get(model, iter, VIEW_OBJECT, &view, -1);

  const gchar *combo_title = NULL;
  if (view != NULL)
    combo_title = view->get_combo_title();

  if (combo_title == NULL)
    combo_title = _("Anonymous");

  g_object_set(renderer, "text", combo_title, NULL);
}


static void
set_combo_pixbuf(GtkCellLayout *layout, GtkCellRenderer *renderer,
		 GtkTreeModel *model, GtkTreeIter *iter, gpointer unused)
{
  View *view = NULL;

  gtk_tree_model_get(model, iter, VIEW_OBJECT, &view, -1);

  GdkPixbuf *pixbuf = NULL;
  if (view != NULL)
    {
      TabLabel *tab_label = view->get_tab_label();
      if (tab_label != NULL)
	{
	  GtkWidget *label_widget = tab_label->get_widget();
	  if (label_widget != NULL && OCHUSHA_IS_ICON_LABEL(label_widget))
	    {
	      OchushaIconLabel *icon_label = OCHUSHA_ICON_LABEL(label_widget);
	      pixbuf = ochusha_icon_label_get_current_pixbuf(icon_label);
	    }
	}
    }
  else
    pixbuf = NULL;

  g_object_set(renderer, "pixbuf", pixbuf, NULL);
}


ViewManager::ViewManager(Controller &controller, MainWindow &main_window)
  : controller(controller),
    main_window(main_window),
    prefs(controller.get_prefs()),
    contents_pane(NULL),
    number_of_boards(0),
    number_of_threads(0),
    editor(controller),
    board_combo_widget(NULL),
    thread_combo_widget(NULL),
    icon_polling_timeout_id(0),
    selection_done_id(0),
    last_event_widget(NULL)
{
  static GtkActionEntry entries[] = {
    {
      "BookmarkTab", OCHUSHA_STOCK_ADD_BOOKMARK,
      N_("Bookmark This _Tab"), NULL,
      N_("Bookmark This Tab"),
      reinterpret_cast<GCallback>(bookmark_tab_cb)
    },
    {
      "RefreshTab", GTK_STOCK_REFRESH,
      N_("_Refresh This Tab"), "",
      N_("Refresh This Tab"),
      reinterpret_cast<GCallback>(refresh_tab_cb)
    },
    {
      "CloseTab", GTK_STOCK_CLOSE,
      N_("_Close This Tab"), "",
      N_("Close This Tab"),
      reinterpret_cast<GCallback>(close_tab_cb)
    },
    {
      "AutoRefreshThreadTab", NULL,
      N_("Auto Refresh This Tab"), NULL,
      NULL,
      reinterpret_cast<GCallback>(auto_refresh_thread_tab_cb)
    }
  };
  action_group = gtk_action_group_new("ViewManagerActions");
  gtk_action_group_set_translation_domain(action_group, PACKAGE_NAME);
  gtk_action_group_add_actions(action_group, entries,
			       G_N_ELEMENTS(entries), this);

  board_left
    = GTK_ACTION(ochusha_arrow_action_new("BoardArrowLeft",
					  _("Left Board Page"),
					  _("Go to the Left Board Page"),
					  GTK_ARROW_LEFT,
					  GTK_SHADOW_ETCHED_OUT));
  gtk_action_group_add_action_with_accel(action_group, board_left, NULL);

  board_right
    = GTK_ACTION(ochusha_arrow_action_new("BoardArrowRight",
					  _("Right Board Page"),
					  _("Go to the Right Board Page"),
					  GTK_ARROW_RIGHT,
					  GTK_SHADOW_ETCHED_OUT));
  gtk_action_group_add_action_with_accel(action_group, board_right, NULL);

  thread_left
    = GTK_ACTION(ochusha_arrow_action_new("ThreadArrowLeft",
					  _("Left Thread Page"),
					  _("Go to the Left Thread Page"),
					  GTK_ARROW_LEFT,
					  GTK_SHADOW_ETCHED_OUT));
  gtk_action_group_add_action_with_accel(action_group, thread_left, NULL);

  thread_right
    = GTK_ACTION(ochusha_arrow_action_new("ThreadArrowRight",
					  _("Right Thread Page"),
					  _("Go to the Right Thread Page"),
					  GTK_ARROW_RIGHT,
					  GTK_SHADOW_ETCHED_OUT));
  gtk_action_group_add_action_with_accel(action_group, thread_right, NULL);


  board_entries = gtk_list_store_new(NOTEBOOK_ITEM_N_COLUMNS,
				     G_TYPE_POINTER,		// GtkWidget *
				     G_TYPE_POINTER);		// View *

  OchushaComboAction *combo_action
    = ochusha_combo_action_new_with_model("ThreadlistTabs",
					  _("Thread_list Tabs"),
					  _("Threadlist Tabs"),
					  NULL,
					  GTK_TREE_MODEL(board_entries));
  ochusha_combo_action_set_menu_data_func(combo_action,
					  set_label_text, this, NULL);
  ochusha_combo_action_set_cell_data_func(combo_action,
					  set_combo_text, this, NULL);

  board_combo = GTK_ACTION(combo_action);
  gtk_action_group_add_action_with_accel(action_group, board_combo, NULL);


  thread_entries = gtk_list_store_new(NOTEBOOK_ITEM_N_COLUMNS,
				      G_TYPE_POINTER,		// GtkWidget *
				      G_TYPE_POINTER);		// View *

  combo_action
    = ochusha_combo_action_new_with_model("ThreadTabs",
					  _("_Thread Tabs"),
					  _("Thread Tabs"),
					  NULL,
					  GTK_TREE_MODEL(thread_entries));
  ochusha_combo_action_set_menu_data_func(combo_action,
					  set_label_text, this, NULL);
  ochusha_combo_action_set_cell_data_func(combo_action,
					  set_combo_text, this, NULL);

  thread_combo = GTK_ACTION(combo_action);
  gtk_action_group_add_action_with_accel(action_group, thread_combo, NULL);


  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     action_group, 0);

  static GtkActionEntry board_view_action_entries[] = {
    {
      "CloseCurrentThreadlist", GTK_STOCK_CLOSE,
      N_("Close Current Thread_list"), "",
      N_("Close Current Threadlist"),
      reinterpret_cast<GCallback>(close_board_cb)
    },
    {
      "SearchThreadlist", GTK_STOCK_FIND,
      N_("Search Thread_list"), "",
      N_("Search Threadlist"),
      reinterpret_cast<GCallback>(search_board_cb)
    },
    {
      "RefreshThreadlist", GTK_STOCK_REFRESH,
      N_("Refresh Thread_list"), "",
      N_("Refresh Threadlist"),
      reinterpret_cast<GCallback>(refresh_board_cb)
    },
    {
      "BookmarkCurrentBoard", OCHUSHA_STOCK_ADD_BOOKMARK,
      N_("Bookmark Current Boar_d"), NULL,
      N_("Bookmark Current Board"),
      reinterpret_cast<GCallback>(bookmark_board_cb)
    },
    {
#ifdef GTK_STOCK_INFO
      "BoardInfo", GTK_STOCK_INFO,
#else
      "BoardInfo", GTK_STOCK_PROPERTIES,
#endif
      N_("_Board Information"), "",
      N_("Board Information"),
      reinterpret_cast<GCallback>(board_info_cb)
    }
  };

  board_view_action_group = gtk_action_group_new("BoardViewManagerActions");
  gtk_action_group_set_translation_domain(board_view_action_group,
					  PACKAGE_NAME);
  gtk_action_group_add_actions(board_view_action_group,
			       board_view_action_entries,
			       G_N_ELEMENTS(board_view_action_entries), this);
  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     board_view_action_group, 0);


  close_board = gtk_action_group_get_action(board_view_action_group,
					    "CloseCurrentThreadlist");


  static GtkActionEntry thread_view_action_entries[] = {
    {
      "CloseCurrentThread", GTK_STOCK_CLOSE,
      N_("Close Current _Thread"), "",
      N_("Close Current Thread"),
      reinterpret_cast<GCallback>(close_thread_cb)
    },
    {
      "SearchThread", GTK_STOCK_FIND,
      N_("Search _Thread"), "",
      N_("Search Thread"),
      reinterpret_cast<GCallback>(search_thread_cb)
    },
    {
      "RefreshThread", GTK_STOCK_REFRESH,
      N_("Refresh _Thread"), "",
      N_("Refresh Thread"),
      reinterpret_cast<GCallback>(refresh_thread_cb)
    },
    {
      "AutoRefreshThread", NULL,
      N_("Auto Refresh Thread"), NULL,
      NULL,
      reinterpret_cast<GCallback>(auto_refresh_thread_cb)
    },
    {
      "ResetThreadClose", GTK_STOCK_DELETE,
      N_("_Reset"), "",
      N_("Drop DAT file and Close Thread"),
      reinterpret_cast<GCallback>(reset_thread_cb)
    },
    {
      "BookmarkCurrentThread", OCHUSHA_STOCK_ADD_BOOKMARK,
      N_("Bookmark Current _Thread"), NULL,
      N_("Bookmark Current Thread"),
      reinterpret_cast<GCallback>(bookmark_thread_cb)
    },
    {
#ifdef GTK_STOCK_INFO
      "ThreadInfo", GTK_STOCK_INFO,
#else
      "ThreadInfo", GTK_STOCK_PROPERTIES,
#endif
      N_("_Thread Information"), "",
      N_("Thread Information"),
      reinterpret_cast<GCallback>(thread_info_cb)
    },
    {
      "OpenBoardCM", GTK_STOCK_OPEN,
      N_("Open Boar_d"), "",
      N_("Open the Board this Thread belongs"),
      reinterpret_cast<GCallback>(open_board_cb)
    },
    {
#ifdef GTK_STOCK_INFO
      "ThreadInfoCM", GTK_STOCK_INFO,
#else
      "ThreadInfoCM", GTK_STOCK_PROPERTIES,
#endif
      N_("Thread _Information"), "",
      N_("Thread Information"),
      reinterpret_cast<GCallback>(thread_info_cb)
    },
    {
      "BookmarkThreadCM", OCHUSHA_STOCK_ADD_BOOKMARK,
      N_("Book_mark This Thread"), NULL,
      N_("Bookmark This Thread"),
      reinterpret_cast<GCallback>(bookmark_thread_cb)
    },
    {
      "AutoRefreshThreadCM", NULL,
      N_("Auto Refresh Thread"), NULL,
      NULL,
      reinterpret_cast<GCallback>(auto_refresh_thread_cb)
    },
    {
      "RefreshThreadCM", GTK_STOCK_REFRESH,
      N_("_Refresh Thread"), "",
      N_("Refresh Thread"),
      reinterpret_cast<GCallback>(refresh_thread_cb)
    },
    {
      "SearchThreadCM", GTK_STOCK_FIND,
      N_("_Search Thread"), "",
      N_("Search Thread"),
      reinterpret_cast<GCallback>(search_thread_cb)
    },
    {
      "ResetThreadCloseCM", GTK_STOCK_DELETE,
      N_("Reset Thread"), "",
      N_("Drop DAT file and Close Thread"),
      reinterpret_cast<GCallback>(reset_thread_cb)
    }
  };
  thread_view_action_group = gtk_action_group_new("ThreadViewManagerActions");
  gtk_action_group_set_translation_domain(thread_view_action_group,
					  PACKAGE_NAME);
  gtk_action_group_add_actions(thread_view_action_group,
			       thread_view_action_entries,
			       G_N_ELEMENTS(thread_view_action_entries), this);
  gtk_ui_manager_insert_action_group(controller.get_ui_manager(),
				     thread_view_action_group, 0);


  close_thread = gtk_action_group_get_action(thread_view_action_group,
					     "CloseCurrentThread");

  cm_thread_info = gtk_action_group_get_action(thread_view_action_group,
					       "ThreadInfoCM");
  cm_bookmark_thread = gtk_action_group_get_action(thread_view_action_group,
						   "BookmarkThreadCM");
  cm_open_board = gtk_action_group_get_action(thread_view_action_group,
					      "OpenBoardCM");
  cm_auto_refresh_thread = gtk_action_group_get_action(thread_view_action_group,
						       "AutoRefreshThreadCM");
  cm_refresh_thread = gtk_action_group_get_action(thread_view_action_group,
						  "RefreshThreadCM");
  cm_search_thread = gtk_action_group_get_action(thread_view_action_group,
						 "SearchThreadCM");
  cm_reset_thread = gtk_action_group_get_action(thread_view_action_group,
						"ResetThreadCloseCM");
}


gboolean
ViewManager::icon_polling_timeout_cb(ViewManager *view_manager)
{
  // g_timeout_add()ΥХåGLibɳʤΤGDK_THREADS_ENTER()Ǥʤ

  GDK_THREADS_ENTER();

  view_manager->icon_polling_timeout_id = 0;

  if (view_manager->board_combo_widget != NULL)
    gtk_widget_queue_draw(view_manager->board_combo_widget);

  if (view_manager->thread_combo_widget != NULL)
    gtk_widget_queue_draw(view_manager->thread_combo_widget);

  GDK_THREADS_LEAVE();


  return FALSE;	// ǽλ
}


void
ViewManager::notify_current_pixbuf_cb(OchushaIconLabel *label,
				      GParamSpec *arg,
				      ViewManager *view_manager)
{
  if (view_manager->icon_polling_timeout_id != 0)
    return;

  view_manager->icon_polling_timeout_id
    = g_timeout_add(ICON_POLLING_INTERVAL,
		    reinterpret_cast<GSourceFunc>(icon_polling_timeout_cb),
		    view_manager);
}


ViewManager::~ViewManager()
{
  if (icon_polling_timeout_id != 0)
    g_source_remove(icon_polling_timeout_id);

  g_object_unref(thread_combo);
  g_object_unref(thread_entries);

  g_object_unref(board_combo);
  g_object_unref(board_entries);

  g_object_unref(thread_right);
  g_object_unref(thread_left);
  g_object_unref(board_right);
  g_object_unref(board_left);

  g_object_unref(thread_view_action_group);
  g_object_unref(board_view_action_group);
  g_object_unref(action_group);
}


void
ViewManager::notify_position_set_cb(GtkPaned *paned, GParamSpec *arg,
				    ViewManager *view_manager)
{
  char value[64];
  snprintf(value, 64, "%d", gtk_paned_get_position(paned));
  SessionClient &session = view_manager->controller.get_session();
  if (static_cast<gpointer>(paned) == static_cast<gpointer>(view_manager->contents_pane))
    session.set_state("ViewManager::board_pane_size", value);
  else
    session.set_state("ViewManager::thread_pane_size", value);
}


GtkWidget *
ViewManager::create_contents_pane()
{
  g_return_val_if_fail(contents_pane == NULL, contents_pane);


#if 0	// OchushaHVPanedϰͽ
  contents_pane = ochusha_hvpaned_new(prefs.get_vertical_placement()
				      ? OCHUSHA_VPANED_STYLE
				      : OCHUSHA_HPANED_STYLE);
#else
  contents_pane
    = prefs.get_vertical_placement() ? gtk_vpaned_new() : gtk_hpaned_new();
#endif
  g_signal_connect(contents_pane, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed),
		   &contents_pane);
  g_signal_connect(contents_pane, "notify::position-set",
		   reinterpret_cast<GCallback>(notify_position_set_cb), this);


  // ()ڥν
  board_box = gtk_vbox_new(FALSE, 0);
  // gtk_container_set_border_width(GTK_CONTAINER(board_box), 0);

  board_toolbar = gtk_ui_manager_get_widget(controller.get_ui_manager(),
					    "/BoardViewToolbar");
  if (board_toolbar != NULL)
    {
      GtkToolbar *toolbar = GTK_TOOLBAR(board_toolbar);
      gtk_toolbar_set_style(toolbar, GTK_TOOLBAR_ICONS);

      gtk_box_pack_start(GTK_BOX(board_box), board_toolbar, FALSE, TRUE, 0);


      GtkWidget *tabs_widget
	= gtk_ui_manager_get_widget(controller.get_ui_manager(),
				    "/BoardViewToolbar/ThreadlistTabs");
      if (tabs_widget != NULL && GTK_IS_TOOL_ITEM(tabs_widget))
	{
	  GtkWidget *combo_widget = gtk_bin_get_child(GTK_BIN(tabs_widget));
	  if (GTK_IS_COMBO_BOX(combo_widget))
	    {
	      GtkCellLayout *layout = GTK_CELL_LAYOUT(combo_widget);
	      GtkCellRenderer *renderer	= gtk_cell_renderer_pixbuf_new();
	      gtk_cell_layout_pack_start(layout, renderer, FALSE);
	      gtk_cell_layout_set_cell_data_func(layout, renderer,
						 set_combo_pixbuf, this, NULL);
	      gtk_cell_layout_reorder(layout, renderer, 0);

	      board_combo_widget = combo_widget;
	    }
	}
    }


  GtkWidget *tmp_widget = mod_notebook_new();
  g_signal_connect(tmp_widget, "switch_page",
		   reinterpret_cast<GCallback>(board_switch_page_cb), this);
  g_signal_connect(tmp_widget, "notify::page",
		   reinterpret_cast<GCallback>(notify_page_cb), this);
  g_signal_connect(tmp_widget, "button_press_event",
		   reinterpret_cast<GCallback>(board_button_press_event_cb),
		   this);
  g_signal_connect(tmp_widget, "close_button_clicked",
		   reinterpret_cast<GCallback>(close_board_cb),
		   this);
  board_book = MOD_NOTEBOOK(tmp_widget);
  mod_notebook_set_scrollable(board_book, TRUE);
  if (board_toolbar == NULL
      || (gtk_ui_manager_get_widget(controller.get_ui_manager(),
				    "/BoardViewToolbar/CloseCurrentThreadlist")
	  == NULL))
    mod_notebook_close_button_enable(board_book);
  else
    mod_notebook_close_button_disable(board_book);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(board_box), tmp_widget, TRUE, TRUE, 0);
  g_signal_connect(board_book, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed),
		   &board_book);


  g_signal_connect_swapped(board_left, "activate",
			   reinterpret_cast<GCallback>(mod_notebook_prev_page),
			   board_book);
  g_signal_connect_swapped(board_right, "activate",
			   reinterpret_cast<GCallback>(mod_notebook_next_page),
			   board_book);

  g_signal_connect(board_combo, "changed",
		   reinterpret_cast<GCallback>(combo_changed_cb), board_book);

  gtk_widget_show(board_box);
  gtk_paned_add1(GTK_PANED(contents_pane), board_box);



  /* ɽڥν */
  thread_pane = gtk_vpaned_new();
  gtk_container_set_border_width(GTK_CONTAINER(thread_pane), 0);

  g_signal_connect(thread_pane, "notify::position-set",
		   reinterpret_cast<GCallback>(notify_position_set_cb), this);

  thread_box = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(thread_box), 0);

  thread_toolbar = gtk_ui_manager_get_widget(controller.get_ui_manager(),
					     "/ThreadViewToolbar");
  if (thread_toolbar != NULL)
    {
      gtk_container_set_border_width(GTK_CONTAINER(thread_toolbar), 0);
      GtkToolbar *toolbar = GTK_TOOLBAR(thread_toolbar);
      gtk_toolbar_set_style(toolbar, GTK_TOOLBAR_ICONS);
      gtk_box_pack_start(GTK_BOX(thread_box), thread_toolbar, FALSE, TRUE, 0);


      GtkWidget *tabs_widget
	= gtk_ui_manager_get_widget(controller.get_ui_manager(),
				    "/ThreadViewToolbar/ThreadTabs");
      if (tabs_widget != NULL && GTK_IS_TOOL_ITEM(tabs_widget))
	{
	  GtkWidget *combo_widget = gtk_bin_get_child(GTK_BIN(tabs_widget));
	  if (GTK_IS_COMBO_BOX(combo_widget))
	    {
	      GtkCellLayout *layout = GTK_CELL_LAYOUT(combo_widget);
	      GtkCellRenderer *renderer	= gtk_cell_renderer_pixbuf_new();
	      gtk_cell_layout_pack_start(layout, renderer, FALSE);
	      gtk_cell_layout_set_cell_data_func(layout, renderer,
						 set_combo_pixbuf, this, NULL);
	      gtk_cell_layout_reorder(layout, renderer, 0);

	      thread_combo_widget = combo_widget;
	    }
	}
    }


  tmp_widget = mod_notebook_new();
  g_signal_connect(tmp_widget, "switch_page",
		   reinterpret_cast<GCallback>(thread_switch_page_cb), this);
  g_signal_connect(tmp_widget, "notify::page",
		   reinterpret_cast<GCallback>(notify_page_cb), this);
  g_signal_connect(tmp_widget, "button_press_event",
		   reinterpret_cast<GCallback>(thread_button_press_event_cb),
		   this);
  g_signal_connect(tmp_widget, "close_button_clicked",
		   reinterpret_cast<GCallback>(close_thread_cb),
		   this);
  thread_book = MOD_NOTEBOOK(tmp_widget);
  mod_notebook_set_scrollable(thread_book, TRUE);
  if (thread_toolbar == NULL
      || (gtk_ui_manager_get_widget(controller.get_ui_manager(),
				    "/ThreadViewToolbar/CloseCurrentThread")
	  == NULL))
    mod_notebook_close_button_enable(thread_book);
  else
    mod_notebook_close_button_disable(thread_book);
  gtk_widget_show(tmp_widget);
  gtk_box_pack_start(GTK_BOX(thread_box), tmp_widget, TRUE, TRUE, 0);
  g_signal_connect(thread_book, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed),
		   &thread_book);


  g_signal_connect_swapped(thread_left, "activate",
			   reinterpret_cast<GCallback>(mod_notebook_prev_page),
			   thread_book);
  g_signal_connect_swapped(thread_right, "activate",
			   reinterpret_cast<GCallback>(mod_notebook_next_page),
			   thread_book);

  g_signal_connect(thread_combo, "changed",
		   reinterpret_cast<GCallback>(combo_changed_cb), thread_book);

  gtk_widget_show(thread_box);
  gtk_paned_pack1(GTK_PANED(thread_pane), thread_box, TRUE, TRUE);

  tmp_widget = editor.get_widget();
  if (prefs.get_enable_embeded_response_editor())
    gtk_widget_show(tmp_widget);
  gtk_paned_pack2(GTK_PANED(thread_pane), tmp_widget, FALSE, TRUE);


  gtk_paned_add2(GTK_PANED(contents_pane), thread_pane);


  // 
  mod_notebook_set_tab_shrinkable(board_book, prefs.get_board_tab_shrinkable());
  mod_notebook_set_minimum_tab_label_size(board_book,
					  prefs.get_board_tab_minimum_size());
  if (prefs.get_board_tab_enable_tooltips())
    mod_notebook_tooltips_enable(board_book);
  else
    mod_notebook_tooltips_disable(board_book);

  mod_notebook_set_tab_shrinkable(thread_book,
				  prefs.get_thread_tab_shrinkable());
  mod_notebook_set_minimum_tab_label_size(thread_book,
					  prefs.get_thread_tab_minimum_size());
  if (prefs.get_thread_tab_enable_tooltips())
    mod_notebook_tooltips_enable(thread_book);
  else
    mod_notebook_tooltips_disable(thread_book);

  const char *pane_size
    = controller.get_session().get_state("ViewManager::board_pane_size");
  if (pane_size != NULL)
    {
      int size;
      if (sscanf(pane_size, "%d", &size) == 1)
	gtk_paned_set_position(GTK_PANED(contents_pane), size);
    }
  pane_size
    = controller.get_session().get_state("ViewManager::thread_pane_size");
  if (pane_size != NULL)
    {
      int size;
      if (sscanf(pane_size, "%d", &size) == 1)
	gtk_paned_set_position(GTK_PANED(thread_pane), size);
    }

  g_signal_connect(prefs.vertical_placement, "toggled",
		   reinterpret_cast<GCallback>(vertical_placement_toggled_cb),
		   this);
  g_signal_connect(prefs.board_tab_shrinkable, "toggled",
		   reinterpret_cast<GCallback>(board_tab_shrinkable_toggled_cb),
		   this);
  g_signal_connect(prefs.board_tab_enable_tooltips, "toggled",
		   reinterpret_cast<GCallback>(board_tab_enable_tooltips_toggled_cb), this);
  g_signal_connect(prefs.board_tab_always_show, "toggled",
		   reinterpret_cast<GCallback>(board_tab_always_show_toggled_cb), this);
  g_signal_connect(prefs.board_pane_always_show, "toggled",
		   reinterpret_cast<GCallback>(board_pane_always_show_toggled_cb), this);
  g_signal_connect(prefs.thread_tab_shrinkable, "toggled",
		   reinterpret_cast<GCallback>(thread_tab_shrinkable_toggled_cb), this);
  g_signal_connect(prefs.thread_tab_enable_tooltips, "toggled",
		   reinterpret_cast<GCallback>(thread_tab_enable_tooltips_toggled_cb), this);
  g_signal_connect(prefs.thread_tab_always_show, "toggled",
		   reinterpret_cast<GCallback>(thread_tab_always_show_toggled_cb), this);
  g_signal_connect(prefs.thread_pane_always_show, "toggled",
		   reinterpret_cast<GCallback>(thread_pane_always_show_toggled_cb), this);

  g_signal_connect(prefs.show_threadlist_toolbar, "toggled",
		   reinterpret_cast<GCallback>(show_threadlist_toolbar_toggled_cb), this);
  gtk_toggle_action_toggled(prefs.show_threadlist_toolbar);
  g_signal_connect(prefs.show_thread_toolbar, "toggled",
		   reinterpret_cast<GCallback>(show_thread_toolbar_toggled_cb),
		   this);
  gtk_toggle_action_toggled(prefs.show_thread_toolbar);

  g_signal_connect(prefs.enable_embeded_response_editor, "toggled",
		   reinterpret_cast<GCallback>(enable_embeded_response_editor_toggled_cb), this);


  prefs.prefs_changed.connect(this, &ViewManager::prefs_changed_cb);


  update_board_pane();
  update_board_tab();
  update_board_navigator();
  update_thread_pane();
  update_thread_tab();
  update_thread_navigator();

  return contents_pane;
}


GtkWidget *
ViewManager::get_widget()
{
  return contents_pane != NULL ? contents_pane : create_contents_pane();
}


void
ViewManager::update_board_pane()
{
  if (number_of_boards == 0 && !prefs.get_board_pane_always_show())
    {
      gtk_widget_set_child_visible(thread_pane, TRUE);
      gtk_widget_set_sensitive(board_box, FALSE);
      gtk_widget_hide(board_box);
    }
  else
    {
      gtk_widget_set_sensitive(board_box, TRUE);
      gtk_widget_show(board_box);
    }

  gtk_action_group_set_sensitive(board_view_action_group, number_of_boards > 0);
}


void
ViewManager::update_board_tab()
{
  if (number_of_boards < 2 && !prefs.get_board_tab_always_show())
    mod_notebook_set_show_tabs(board_book, FALSE);
  else
    mod_notebook_set_show_tabs(board_book, TRUE);
}


void
ViewManager::update_board_navigator()
{
  gint page = mod_notebook_get_current_page(board_book);

  gtk_action_set_sensitive(board_left, page > 0);
  gtk_action_set_sensitive(board_right,
			   page != -1
			   && static_cast<guint>(page) < number_of_boards - 1);
}


void
ViewManager::update_thread_pane()
{
  if (number_of_threads == 0 && !prefs.get_thread_pane_always_show())
    {
      gtk_widget_set_child_visible(board_box, TRUE);
      gtk_widget_set_sensitive(thread_pane, FALSE);
      gtk_widget_hide(thread_pane);
    }
  else
    {
      gtk_widget_set_sensitive(thread_pane, TRUE);
      gtk_widget_show(thread_pane);
    }

  if (number_of_threads == 0)
    {
      gtk_action_group_set_sensitive(thread_view_action_group, FALSE);

      editor.set_thread(NULL);
    }
  else
    {
      gtk_action_group_set_sensitive(thread_view_action_group, TRUE);

      if (prefs.get_enable_embeded_response_editor())
	editor.set_thread(get_current_thread());
    }
}


void
ViewManager::update_thread_tab()
{
  if (number_of_threads < 2 && !prefs.get_thread_tab_always_show())
    mod_notebook_set_show_tabs(thread_book, FALSE);
  else
    mod_notebook_set_show_tabs(thread_book, TRUE);
}


void
ViewManager::update_thread_navigator()
{
  gint page = mod_notebook_get_current_page(thread_book);

  gtk_action_set_sensitive(thread_left, page > 0);
  gtk_action_set_sensitive(thread_right,
			   page != -1
			   && static_cast<guint>(page) < number_of_threads - 1);
}


void
ViewManager::close_board_cb(GtkAction *action, ViewManager *view_manager)
{
  gint page = mod_notebook_get_current_page(view_manager->board_book);
  if (page == -1)
    return;

  view_manager->close_nth_board_page(page);

  view_manager->update_board_pane();
  view_manager->update_board_tab();
  view_manager->update_board_navigator();
}


void
ViewManager::close_thread_cb(GtkAction *action, ViewManager *view_manager)
{
  gint page = mod_notebook_get_current_page(view_manager->thread_book);
  if (page == -1)
    return;

  view_manager->close_nth_thread_page(page);

  view_manager->update_thread_pane();
  view_manager->update_thread_tab();
  view_manager->update_thread_navigator();
}


void
ViewManager::search_board_cb(GtkAction *action, ViewManager *view_manager)
{
  GtkWidget *widget = view_manager->get_current_board_widget();
  g_return_if_fail(widget != NULL);

  View *view = View::get_view(widget);
  g_return_if_fail(view != NULL);

  view->request_search();
}


void
ViewManager::search_thread_cb(GtkAction *action, ViewManager *view_manager)
{
  GtkWidget *widget = view_manager->get_current_thread_widget();
  g_return_if_fail(widget != NULL);

  View *view = View::get_view(widget);
  g_return_if_fail(view != NULL);

  view->request_search();
}


void
ViewManager::refresh_board_cb(GtkAction *action, ViewManager *view_manager)
{
  BulletinBoard *board = view_manager->get_current_board();
  if (board == NULL)
    return;

  view_manager->controller.refresh(board, true);
}


void
ViewManager::refresh_thread_cb(GtkAction *action, ViewManager *view_manager)
{
  BBSThread *thread = view_manager->get_current_thread();
  if (thread == NULL)
    return;

  view_manager->controller.refresh(thread);
}


void
ViewManager::auto_refresh_thread_cb(GtkAction *action,
				    ViewManager *view_manager)
{
  BBSThread *thread = view_manager->get_current_thread();
  if (thread == NULL)
    return;

  view_manager->controller.refresh(thread, true);
}


void
ViewManager::reset_thread_cb(GtkAction *action, ViewManager *view_manager)
{
  BBSThread *thread = view_manager->get_current_thread();
  if (thread == NULL)
    return;

  thread->reset(view_manager->controller.get_repository());

  gint page = mod_notebook_get_current_page(view_manager->thread_book);
  if (page == -1)
    return;

  view_manager->close_nth_thread_page(page);

  view_manager->update_thread_pane();
  view_manager->update_thread_tab();
  view_manager->update_thread_navigator();
}


void
ViewManager::open_board_cb(GtkAction *action, ViewManager *view_manager)
{
  BBSThread *thread = view_manager->get_current_thread();
  if (thread == NULL)
    return;

  view_manager->controller.open(thread->get_board(), true, true, false, true);
}


void
ViewManager::bookmark_board_cb(GtkAction *action, ViewManager *view_manager)
{
  BulletinBoard *board = view_manager->get_current_board();
  if (board == NULL)
    return;

  view_manager->controller.bookmark(board);
}


void
ViewManager::bookmark_thread_cb(GtkAction *action, ViewManager *view_manager)
{
  BBSThread *thread = view_manager->get_current_thread();
  if (thread == NULL)
    return;

  view_manager->controller.bookmark(thread);
}


void
ViewManager::board_info_cb(GtkAction *action, ViewManager *view_manager)
{
  BulletinBoard *board = view_manager->get_current_board();
  if (board == NULL)
    return;

  view_manager->controller.get_boardlist_ui().open_properties(board);
}


void
ViewManager::thread_info_cb(GtkAction *action, ViewManager *view_manager)
{
  BBSThread *thread = view_manager->get_current_thread();
  if (thread == NULL)
    return;

  view_manager->controller.get_thread_ui().open_properties(thread);
}


void
ViewManager::bookmark_tab_cb(GtkAction *action, ViewManager *view_manager)
{
  g_return_if_fail(view_manager->last_event_widget != NULL);

  BulletinBoard *board = get_board(view_manager->last_event_widget);
  if (board != NULL)
    {
      view_manager->controller.bookmark(board);
      return;
    }

  BBSThread *thread = get_thread(view_manager->last_event_widget);
  if (thread != NULL)
    {
      view_manager->controller.bookmark(thread);
      return;
    }

  return;
}


void
ViewManager::refresh_tab_cb(GtkAction *action, ViewManager *view_manager)
{
  g_return_if_fail(view_manager->last_event_widget != NULL);

  BulletinBoard *board = get_board(view_manager->last_event_widget);
  if (board != NULL)
    {
      view_manager->controller.refresh(board, true);
      return;
    }

  BBSThread *thread = get_thread(view_manager->last_event_widget);
  if (thread != NULL)
    {
      view_manager->controller.refresh(thread);
      return;
    }

  return;
}


void
ViewManager::close_tab_cb(GtkAction *action, ViewManager *view_manager)
{
  g_return_if_fail(view_manager->last_event_widget != NULL);

  View *view = View::get_view(view_manager->last_event_widget);
  g_return_if_fail(view != NULL);

  view_manager->close_tab(view);
}


void
ViewManager::auto_refresh_thread_tab_cb(GtkAction *action,
					ViewManager *view_manager)
{
  g_return_if_fail(view_manager->last_event_widget != NULL);

  BBSThread *thread = get_thread(view_manager->last_event_widget);
  if (thread != NULL)
    {
      view_manager->controller.refresh(thread, true);
      return;
    }

  return;
}


void
ViewManager::vertical_placement_toggled_cb(GtkToggleAction *action,
					   ViewManager *view_manager)
{
#if 0	// OchushaHVPanedϰͽ
  ochusha_hvpaned_set_pane_style(OCHUSHA_HVPANED(view_manager->contents_pane),
				 gtk_toggle_action_get_active(action)
				 ? OCHUSHA_VPANED_STYLE : OCHUSHA_HPANED_STYLE);
#else
  GtkWidget *old_contents_pane = view_manager->contents_pane;
  GtkWidget *parent_container = gtk_widget_get_parent(old_contents_pane);
  g_return_if_fail(GTK_IS_PANED(parent_container));

  GtkWidget *new_contents_pane
    = gtk_toggle_action_get_active(action)
    ? gtk_vpaned_new() : gtk_hpaned_new();

  gtk_paned_set_position(GTK_PANED(new_contents_pane),
			 gtk_paned_get_position(GTK_PANED(old_contents_pane)));
  g_signal_connect(new_contents_pane, "notify::position-set",
		   reinterpret_cast<GCallback>(notify_position_set_cb),
		   view_manager);


  g_object_ref(view_manager->board_box);
  gtk_container_remove(GTK_CONTAINER(old_contents_pane),
		       view_manager->board_box);
  gtk_paned_add1(GTK_PANED(new_contents_pane), view_manager->board_box);
  g_object_unref(view_manager->board_box);


  g_object_ref(view_manager->thread_pane);
  gtk_container_remove(GTK_CONTAINER(old_contents_pane),
		       view_manager->thread_pane);
  gtk_paned_add2(GTK_PANED(new_contents_pane), view_manager->thread_pane);
  g_object_unref(view_manager->thread_pane);


  gtk_widget_destroy(old_contents_pane);
  assert(view_manager->contents_pane == NULL);


  g_signal_connect(new_contents_pane, "destroy",
		   reinterpret_cast<GCallback>(gtk_widget_destroyed),
		   &view_manager->contents_pane);


  view_manager->contents_pane = new_contents_pane;
  gtk_widget_show(new_contents_pane);
  gtk_paned_add2(GTK_PANED(parent_container), new_contents_pane);
  /*
   * ľcontents_paneοƤGtkPanedǤȲꤷƤꡢ
   * Ǽ꤬pane2ǤȷĤƤꤹΤϥޥ롣
   *
   * OchushaHVPanedΥåȤȤѰդȤΤ
   * ĤμǤϤ뤬¸ȤʤȤΥåȤGtkPaned
   * ѹ򤤤ĤޤǤɤݤ³ɬޤΤǡGTK+¦ѹ
   * ƶϰϤ︺뤿OchushaHVPanedवȤŪȿ롣
   * ɤݤ³ʤ顢̤OchushaHVPanedGTK+Ѳɽ³
   * ˤͭ
   */
#endif
}


void
ViewManager::board_tab_shrinkable_toggled_cb(GtkToggleAction *action,
					     ViewManager *view_manager)
{
  mod_notebook_set_tab_shrinkable(view_manager->board_book,
				  gtk_toggle_action_get_active(action));
}


void
ViewManager::board_tab_enable_tooltips_toggled_cb(GtkToggleAction *action,
						  ViewManager *view_manager)
{
  if (gtk_toggle_action_get_active(action))
    mod_notebook_tooltips_enable(view_manager->board_book);
  else
    mod_notebook_tooltips_disable(view_manager->board_book);
}


void
ViewManager::board_tab_always_show_toggled_cb(GtkToggleAction *action,
					      ViewManager *view_manager)
{
  view_manager->update_board_tab();
}


void
ViewManager::board_pane_always_show_toggled_cb(GtkToggleAction *action,
					       ViewManager *view_manager)
{
  view_manager->update_board_pane();
}


void
ViewManager::thread_tab_shrinkable_toggled_cb(GtkToggleAction *action,
					      ViewManager *view_manager)
{
  mod_notebook_set_tab_shrinkable(view_manager->thread_book,
				  gtk_toggle_action_get_active(action));
}


void
ViewManager::thread_tab_enable_tooltips_toggled_cb(GtkToggleAction *action,
						   ViewManager *view_manager)
{
  if (gtk_toggle_action_get_active(action))
    mod_notebook_tooltips_enable(view_manager->thread_book);
  else
    mod_notebook_tooltips_disable(view_manager->thread_book);
}


void
ViewManager::thread_tab_always_show_toggled_cb(GtkToggleAction *action,
					       ViewManager *view_manager)
{
  view_manager->update_thread_tab();
}


void
ViewManager::thread_pane_always_show_toggled_cb(GtkToggleAction *action,
						ViewManager *view_manager)
{
  view_manager->update_thread_pane();
}


void
ViewManager::show_threadlist_toolbar_toggled_cb(GtkToggleAction *action,
						ViewManager *view_manager)
{
  if (view_manager->board_toolbar == NULL)
    return;

  gboolean active = gtk_toggle_action_get_active(action);
  if (active)
    gtk_widget_show(view_manager->board_toolbar);
  else
    gtk_widget_hide(view_manager->board_toolbar);

  if (!active
      || (gtk_ui_manager_get_widget(view_manager->controller.get_ui_manager(),
				    "/BoardViewToolbar/CloseCurrentThreadlist")
	  == NULL))
    mod_notebook_close_button_enable(view_manager->board_book);
  else
    mod_notebook_close_button_disable(view_manager->board_book);
}


void
ViewManager::show_thread_toolbar_toggled_cb(GtkToggleAction *action,
					    ViewManager *view_manager)
{
  if (view_manager->thread_toolbar == NULL)
    return;

  gboolean active = gtk_toggle_action_get_active(action);
  if (active)
    gtk_widget_show(view_manager->thread_toolbar);
  else
    gtk_widget_hide(view_manager->thread_toolbar);

  if (!active
      || (gtk_ui_manager_get_widget(view_manager->controller.get_ui_manager(),
				    "/ThreadViewToolbar/CloseCurrentThread")
	  == NULL))
    mod_notebook_close_button_enable(view_manager->thread_book);
  else
    mod_notebook_close_button_disable(view_manager->thread_book);
}


void
ViewManager::enable_embeded_response_editor_toggled_cb(GtkToggleAction *action,
						       ViewManager *view_manager)
{
  if (gtk_toggle_action_get_active(action))
    {
      view_manager->editor.set_thread(view_manager->get_current_thread());
      gtk_widget_show(view_manager->editor.get_widget());
    }
  else
    {
      view_manager->editor.set_thread(NULL);
      gtk_widget_hide(view_manager->editor.get_widget());
    }
}


void
ViewManager::prefs_changed_cb(Preferences &prefs)
{
  // sigslotʥϥɥ
  assert(OCHUSHA_IN_GDK_THREADS());

  mod_notebook_set_minimum_tab_label_size(board_book,
					  prefs.get_board_tab_minimum_size());
  mod_notebook_set_minimum_tab_label_size(thread_book,
					  prefs.get_thread_tab_minimum_size());
}


GtkWidget *
ViewManager::get_current_board_widget()
{
  if (board_book == NULL)
    return NULL;

  gint page = mod_notebook_get_current_page(board_book);
  if (page == -1)
    return NULL;

  return mod_notebook_get_nth_page(board_book, page);
}


View *
ViewManager::get_current_board_view()
{
  GtkWidget *widget = get_current_board_widget();

  if (widget == NULL)
    return NULL;

  return View::get_view(widget);
}


BulletinBoard *
ViewManager::get_current_board()
{
  if (board_book == NULL)
    return NULL;

  gint page = mod_notebook_get_current_page(board_book);
  if (page == -1)
    return NULL;

  return get_board(mod_notebook_get_nth_page(board_book, page));
}
  

GtkWidget *
ViewManager::get_current_thread_widget()
{
  if (thread_book == NULL)
    return NULL;

  gint page = mod_notebook_get_current_page(thread_book);
  if (page == -1)
    return NULL;

  return mod_notebook_get_nth_page(thread_book, page);
}


View *
ViewManager::get_current_thread_view()
{
  GtkWidget *widget = get_current_thread_widget();
  if (widget == NULL)
    return NULL;

  return View::get_view(widget);
}


BBSThread *
ViewManager::get_current_thread()
{
  if (thread_book == NULL)
    return NULL;

  gint page = mod_notebook_get_current_page(thread_book);
  if (page == -1)
    return NULL;

  return get_thread(mod_notebook_get_nth_page(thread_book, page));
}


struct RemoveEntryArgs
{
  GtkWidget *widget;
  gboolean result;
};


static gboolean
remove_entry(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
	     RemoveEntryArgs *args)
{
  GtkWidget *widget = NULL;
  gtk_tree_model_get(model, iter, VIEW_WIDGET, &widget, -1);
  if (widget == args->widget)
    {
      assert(!args->result);
      gtk_list_store_remove(GTK_LIST_STORE(model), iter);
      args->result = TRUE;
      return TRUE;
    }

  return FALSE;
}


static void
remove_list_entry(GtkListStore *store, GtkWidget *widget)
{
  RemoveEntryArgs args = { widget, FALSE };
  gtk_tree_model_foreach(GTK_TREE_MODEL(store),
		reinterpret_cast<GtkTreeModelForeachFunc>(remove_entry), &args);
  assert(args.result);
}


struct ActivateComboEntryArgs
{
  GtkWidget *widget;
  OchushaComboAction *combo;
};


static gboolean
activate_entry(GtkTreeModel *model,
	       GtkTreePath *path, GtkTreeIter *iter,
	       ActivateComboEntryArgs *args)
{
  GtkWidget *widget = NULL;
  gtk_tree_model_get(model, iter, VIEW_WIDGET, &widget, -1);
  assert(widget != NULL);

  if (widget == args->widget)
    {
      ochusha_combo_action_set_active_iter(args->combo, iter);
      return TRUE;
    }

  return FALSE;
}


static void
activate_combo_entry(GtkListStore *store,
		     OchushaComboAction *combo, GtkWidget *widget)
{
  ActivateComboEntryArgs args = { widget, combo };
  gtk_tree_model_foreach(GTK_TREE_MODEL(store),
		reinterpret_cast<GtkTreeModelForeachFunc>(activate_entry),
		&args);
}


void
ViewManager::notify_page_cb(ModNotebook *notebook, GParamSpec *arg,
			    ViewManager *view_manager)
{
  g_return_if_fail(MOD_IS_NOTEBOOK(notebook));

  gint page = mod_notebook_get_current_page(notebook);
  if (page == -1)
    return;

  GtkWidget *new_widget = mod_notebook_get_nth_page(notebook, page);
  g_return_if_fail(new_widget != NULL);

  view_manager->notify_view_changed(notebook, new_widget);
}


void
ViewManager::notify_view_changed(ModNotebook *notebook, GtkWidget *new_widget)
{
  // MEMO: GtkNotebookͳ餷֤ڤؤڤؤ줿
  // ڡ蘆focusäƤޤΤǡGtkTreeViewʥڡ
  // 褦Ⱦüʥܡɥե礤(إåʬեꡢ
  // GtkTreeViewŪˤfocus-in٥ȤȯԤƤʤ)ˤʤΤ
  // 򤱤뤿ᡢview->grab_focus()Ƥ
  // ݤäݡǡإåեäƤޤġġ
  View *view = View::get_view(new_widget);
  view->grab_focus();
  main_window.notify_current_focus_view(view);

  BulletinBoard *board = get_board(new_widget);
  if (board != NULL)
    {
      if (get_current_board_widget() == new_widget)
	controller.get_board_ui().notify_current_board_view(view, board);

      return;
    }

  BBSThread *thread = get_thread(new_widget);
  g_return_if_fail(thread != NULL);

  if (get_current_thread_widget() == new_widget)
    {
      controller.get_thread_ui().notify_current_thread_view(view, thread);

      if (prefs.get_enable_embeded_response_editor())
	editor.set_thread(thread);
    }

  // 1ڥ󥵥ݡȤΤ
  if (get_current_board() == NULL)
    controller.get_board_ui().notify_current_board_view(NULL, NULL);
}


void
ViewManager::board_switch_page_cb(ModNotebook *notebook, ModNotebookPage *page,
				  guint page_num, ViewManager *view_manager)
{
  GtkWidget *new_widget = mod_notebook_get_nth_page(notebook, page_num);

  gtk_action_set_sensitive(view_manager->board_left, page_num > 0);
  gtk_action_set_sensitive(view_manager->board_right,
			   page_num < view_manager->number_of_boards - 1);

  activate_combo_entry(view_manager->board_entries,
		       OCHUSHA_COMBO_ACTION(view_manager->board_combo),
		       new_widget);
}


gboolean
ViewManager::board_button_press_event_cb(ModNotebook *notebook,
					 GdkEventButton *event,
					 ViewManager *view_manager)
{
  g_return_val_if_fail(notebook == view_manager->board_book, FALSE);

  if (event->button == 2)
    {
      gint page = mod_notebook_get_mouse_event_page_num(notebook,
					reinterpret_cast<GdkEventAny *>(event));
      if (page != -1)
	{
	  view_manager->close_nth_board_page(page);
	  view_manager->update_board_pane();
	  view_manager->update_board_tab();
	  view_manager->update_board_navigator();
	  return TRUE;
	}
      return FALSE;
    }

  if (event->button == 1)
    {
      if (event->type == GDK_2BUTTON_PRESS)
	{
	  // ֥륯å
	  gint page = mod_notebook_get_mouse_event_page_num(notebook,
					reinterpret_cast<GdkEventAny *>(event));
	  if (page == -1)
	    return FALSE;

	  GtkWidget *view = mod_notebook_get_nth_page(notebook, page);
	  g_return_val_if_fail(view != NULL, FALSE);

	  BulletinBoard *board = get_board(view);
	  if (board != NULL)
	    {
	      view_manager->controller.refresh(board, true);
	      return TRUE;
	    }
	  // 1ڥ󥵥ݡȤΤ
	  BBSThread *thread = get_thread(view);
	  if (thread != NULL)
	    {
	      view_manager->controller.refresh(thread);
	      return TRUE;
	    }
	}
      return FALSE;
    }

  if (event->button != 3)
    return FALSE;

  gint page = mod_notebook_get_mouse_event_page_num(notebook,
					reinterpret_cast<GdkEventAny *>(event));
  if (page == -1)
    return FALSE;

  GtkWidget *view = mod_notebook_get_nth_page(notebook, page);
  g_return_val_if_fail(view != NULL, FALSE);

  if (view_manager->show_popup_menu(view, event))
    return TRUE;

  return FALSE;
}


void
ViewManager::selection_done_cb(GtkWidget *widget, ViewManager *view_manager)
{
  g_signal_handler_disconnect(widget, view_manager->selection_done_id);

  view_manager->selection_done_id = 0;
  view_manager->last_event_widget = NULL;
}


gboolean
ViewManager::show_popup_menu(GtkWidget *target, GdkEventButton *event)
{
  g_return_val_if_fail(selection_done_id == 0, FALSE);

  const gchar *menu_path = NULL;

  BulletinBoard *board = get_board(target);
  if (board != NULL)
    menu_path = "/BoardTabPopupMenu";

  BBSThread *thread = get_thread(target);
  if (thread != NULL)
    menu_path = "/ThreadTabPopupMenu";

  if (menu_path != NULL)
    {
      last_event_widget = target;

      GtkWidget *menu_widget
	= gtk_ui_manager_get_widget(controller.get_ui_manager(), menu_path);
      g_return_val_if_fail(GTK_IS_MENU(menu_widget), FALSE);

      selection_done_id
	= g_signal_connect(menu_widget, "selection-done",
			   reinterpret_cast<GCallback>(selection_done_cb),
			   this);

      gtk_menu_popup(GTK_MENU(menu_widget), NULL, NULL, NULL, NULL,
		     event->button, event->time);

      return TRUE;
    }

  last_event_widget = NULL;
  return FALSE;
}


void
ViewManager::close_nth_board_page(gint page)
{
  g_return_if_fail(page >= 0);

  GtkWidget *widget = mod_notebook_get_nth_page(board_book, page);
  g_return_if_fail(widget != NULL);

  View *view = View::get_view(widget);
  g_return_if_fail(view != NULL);

  view->notify_closed();

  bool view_has_focus = main_window.get_current_focus_view() == view;
  bool view_is_main = main_window.get_current_main_view() == view;

  remove_list_entry(board_entries, widget);
  mod_notebook_remove_page(board_book, page);

  BulletinBoard *board;
  BBSThread *thread;

  if (--number_of_boards > 0)
    {
      page = mod_notebook_get_current_page(board_book);
      assert(page != -1);

      widget = mod_notebook_get_nth_page(board_book, page);
      assert(widget != NULL);

      board = get_board(widget);
      thread = get_thread(widget);
    }
  else
    {
      board = NULL;

      widget = get_current_thread_widget();
      if (widget != NULL)
	thread = get_thread(widget);
      else
	thread = NULL;
    }

  if (widget != NULL)
    {
      view = View::get_view(widget);
      if (view_has_focus)
	{
	  view->grab_focus();
	  main_window.notify_current_focus_view(view);
	}
      else if (view_is_main)
	main_window.notify_current_main_view(view);
    }
  else
    view = NULL;


  if (board != NULL)
    controller.get_board_ui().notify_current_board_view(view, board);
  else
    controller.get_board_ui().notify_current_board_view(NULL, NULL);

  if (thread != NULL)
    controller.get_thread_ui().notify_current_thread_view(view, thread);
  else if (number_of_threads == 0)
    controller.get_thread_ui().notify_current_thread_view(NULL, NULL);
}


void
ViewManager::thread_switch_page_cb(ModNotebook *notebook,
				   ModNotebookPage *page, guint page_num,
				   ViewManager *view_manager)
{
  GtkWidget *new_widget = mod_notebook_get_nth_page(notebook, page_num);

  gtk_action_set_sensitive(view_manager->thread_left, page_num > 0);
  gtk_action_set_sensitive(view_manager->thread_right,
			   page_num < view_manager->number_of_threads - 1);

  activate_combo_entry(view_manager->thread_entries,
		       OCHUSHA_COMBO_ACTION(view_manager->thread_combo),
		       new_widget);
}


gboolean
ViewManager::thread_button_press_event_cb(ModNotebook *notebook,
					  GdkEventButton *event,
					  ViewManager *view_manager)
{
  g_return_val_if_fail(notebook == view_manager->thread_book, FALSE);

  if (event->button == 2)
    {
      gint page = mod_notebook_get_mouse_event_page_num(notebook,
					reinterpret_cast<GdkEventAny *>(event));
      if (page != -1)
	{
	  view_manager->close_nth_thread_page(page);
	  view_manager->update_thread_pane();
	  view_manager->update_thread_tab();
	  view_manager->update_thread_navigator();
	  return TRUE;
	}

      return FALSE;
    }

  if (event->button == 1)
    {
      if (event->type == GDK_2BUTTON_PRESS)
	{
	  // ֥륯å
	  gint page = mod_notebook_get_mouse_event_page_num(notebook,
					reinterpret_cast<GdkEventAny *>(event));
	  if (page == -1)
	    return FALSE;

	  GtkWidget *view = mod_notebook_get_nth_page(notebook, page);
	  g_return_val_if_fail(view != NULL, FALSE);

	  BBSThread *thread = get_thread(view);
	  if (thread != NULL)
	    {
	      view_manager->controller.refresh(thread);
	      return TRUE;
	    }
	}

      return FALSE;
    }

  if (event->button != 3)
    return FALSE;

  gint page = mod_notebook_get_mouse_event_page_num(notebook,
					reinterpret_cast<GdkEventAny *>(event));
  if (page == -1)
    return FALSE;

  GtkWidget *view = mod_notebook_get_nth_page(notebook, page);
  g_return_val_if_fail(view != NULL, FALSE);

  if (view_manager->show_popup_menu(view, event))
    return TRUE;

  return FALSE;
}


void
ViewManager::close_nth_thread_page(gint page)
{
  g_return_if_fail(page >= 0);

  GtkWidget *widget = mod_notebook_get_nth_page(thread_book, page);
  g_return_if_fail(widget != NULL);

  View *view = View::get_view(widget);
  g_return_if_fail(view != NULL);

  view->notify_closed();

  bool view_has_focus = main_window.get_current_focus_view() == view;
  bool view_is_main = main_window.get_current_main_view() == view;

  remove_list_entry(thread_entries, widget);
  mod_notebook_remove_page(thread_book, page);

  BBSThread *thread;

  if (--number_of_threads > 0)
    {
      page = mod_notebook_get_current_page(thread_book);
      assert(page != -1);

      widget = mod_notebook_get_nth_page(thread_book, page);
      assert(widget != NULL);

      thread = get_thread(widget);
    }
  else
    {
      thread = NULL;
      widget = get_current_board_widget();
    }

  if (widget != NULL)
    {
      view = View::get_view(widget);
      if (view_has_focus)
	{
	  view->grab_focus();
	  main_window.notify_current_focus_view(view);
	}
      else if (view_is_main)
	main_window.notify_current_main_view(view);
    }
  else
    view = NULL;

  if (thread != NULL)
    controller.get_thread_ui().notify_current_thread_view(view, thread);
  else
    controller.get_thread_ui().notify_current_thread_view(NULL, NULL);
}


void
ViewManager::append_board_view(BulletinBoard *board, View *view,
			       bool in_tab, bool select_tab)
{
  ModNotebook *notebook = board_book;

  GtkWidget *widget = view->get_widget();
  gtk_widget_show(widget);

  GtkWidget *tab_label = view->get_label();

  g_object_set_qdata(G_OBJECT(widget), bulletin_board_id, board);

  GtkTreeIter dummy_iter;
  gtk_list_store_insert_with_values(board_entries, &dummy_iter, G_MAXINT32,
				    VIEW_WIDGET, widget,
				    VIEW_OBJECT, view,
				    -1);

  if (number_of_boards == 0)
    gtk_widget_show(board_box);
  else
    {
      View *current_view = get_current_board_view();
      assert(current_view != NULL);

      if (current_view->state != View::NORMAL)
	{
	  assert(!in_tab);

	  if (current_view->state == View::GOING_BACK)
	    {
	      view->snatch_backward_history(current_view);
	      view->snatch_forward_history(current_view);
	      view->forward_history_push(current_view->get_current_view_url());
	    }
	  else
	    {
	      assert(current_view->state == View::GOING_FORWARD);

	      view->snatch_backward_history(current_view);
	      view->snatch_forward_history(current_view);
	      view->backward_history_push(current_view->get_current_view_url());
	    }

	  // current_viewϤdeleteΤ~View()Ǥ
	  // assert(state == NORMAL);ˤåŪΤߤΤ
	  current_view->state = View::NORMAL;
	}
      else if (!in_tab)
	{
	  // ʬѤ
	  view->snatch_backward_history(current_view);
	  view->backward_history_push(current_view->get_current_view_url());
	}
    }

  if (number_of_boards == 0 || in_tab)
    {
      mod_notebook_append_page(notebook, widget, tab_label);
      mod_notebook_set_tab_reorderable(notebook, widget, TRUE);
      mod_notebook_set_tab_label_packing(notebook, widget,
					 TRUE, TRUE, GTK_PACK_START);

      if (++number_of_boards > 1 && select_tab)
	{
	  gint page = mod_notebook_page_num(notebook, widget);
	  assert(page != -1);

	  mod_notebook_set_current_page(notebook, page);
	}

      if (number_of_boards == 1)
	{
	  controller.get_board_ui().notify_current_board_view(view, board);
	  if (number_of_threads == 0)
	    main_window.notify_current_main_view(view);
	}
    }
  else
    {
      gint page = mod_notebook_get_current_page(notebook);
      assert(page != -1);

      GtkWidget *prev_widget = mod_notebook_get_nth_page(notebook, page);
      View *prev_view = View::get_view(prev_widget);
      assert(prev_view != NULL);

      View *prev_focus_view = main_window.get_current_focus_view();
      bool view_has_focus = prev_focus_view == prev_view;
      View *prev_main_view = main_window.get_current_main_view();
      bool view_is_main = prev_main_view == prev_view;

      remove_list_entry(board_entries, prev_widget);
      mod_notebook_remove_page(notebook, page);

      mod_notebook_insert_page(notebook, widget, tab_label, page);
      mod_notebook_set_tab_reorderable(notebook, widget, TRUE);
      mod_notebook_set_tab_label_packing(notebook, widget,
					 TRUE, TRUE, GTK_PACK_START);
      mod_notebook_set_current_page(notebook, page);

      if (view_has_focus)
	{
	  view->grab_focus();
	  main_window.notify_current_focus_view(view);
	}
      else if (view_is_main)
	{
	  main_window.notify_current_main_view(view);
	}
      else
	{
	  // MEMO: mode_notebook_set_current_page(...)Ƥ̤Ȥơ
	  // ܡɥե蘆˰žƤޤäƤʬ
	  // 
	  main_window.notify_current_main_view(prev_main_view);
	  if (prev_focus_view != NULL)
	    prev_focus_view->grab_focus();
	  main_window.notify_current_focus_view(prev_focus_view);
	}
    }

  if (select_tab)
    {
      view->grab_focus();
      main_window.notify_current_focus_view(view);
    }

  update_board_pane();
  update_board_tab();
  update_board_navigator();

  if (OCHUSHA_IS_ICON_LABEL(tab_label))
    g_signal_connect(tab_label, "notify::current-pixbuf",
		     reinterpret_cast<GCallback>(notify_current_pixbuf_cb),
		     this);
}


void
ViewManager::append_thread_view(BBSThread *thread, View *view,
				bool in_tab, bool select_tab)
{
  ModNotebook *notebook = thread_book;

  GtkWidget *widget = view->get_widget();
  gtk_widget_show(widget);

  GtkWidget *tab_label = view->get_label();

  g_object_set_qdata(G_OBJECT(widget), bbs_thread_id, thread);

  GtkTreeIter dummy_iter;
  gtk_list_store_insert_with_values(thread_entries, &dummy_iter, G_MAXINT32,
				    VIEW_WIDGET, widget,
				    VIEW_OBJECT, view,
				    -1);

  if (number_of_threads == 0)
    gtk_widget_show(thread_pane);
  else
    {
      View *current_view = get_current_thread_view();
      assert(current_view != NULL);

      if (current_view->state != View::NORMAL)
	{
	  assert(!in_tab);

	  if (current_view->state == View::GOING_BACK)
	    {
	      view->snatch_backward_history(current_view);
	      view->snatch_forward_history(current_view);
	      view->forward_history_push(current_view->get_current_view_url());
	    }
	  else
	    {
	      assert(current_view->state == View::GOING_FORWARD);

	      view->snatch_backward_history(current_view);
	      view->snatch_forward_history(current_view);
	      view->backward_history_push(current_view->get_current_view_url());
	    }

	  // current_viewϤdeleteΤ~View()Ǥ
	  // assert(state == NORMAL);ˤåŪΤߤΤ
	  current_view->state = View::NORMAL;
	}
      else if (!in_tab)
	{
	  // ʬѤ
	  view->snatch_backward_history(current_view);
	  view->backward_history_push(current_view->get_current_view_url());
	}
    }

  if (number_of_threads == 0 || in_tab)
    {
      mod_notebook_append_page(notebook, widget, tab_label);
      mod_notebook_set_tab_reorderable(notebook, widget, TRUE);
      mod_notebook_set_tab_label_packing(notebook, widget,
					 TRUE, TRUE, GTK_PACK_START);

      if (++number_of_threads > 1 && select_tab)
	{
	  gint page = mod_notebook_page_num(notebook, widget);
	  assert(page != -1);

	  mod_notebook_set_current_page(notebook, page);
	}

      if (number_of_threads == 1)
	{
	  controller.get_thread_ui().notify_current_thread_view(view, thread);
	  
	  if (number_of_boards == 0)
	    main_window.notify_current_main_view(view);
	}
    }
  else
    {
      gint page = mod_notebook_get_current_page(notebook);
      assert(page != -1);

      GtkWidget *prev_widget = mod_notebook_get_nth_page(notebook, page);
      View *prev_view = View::get_view(prev_widget);
      assert(prev_view != NULL);

      View *prev_focus_view = main_window.get_current_focus_view();
      bool view_has_focus = prev_focus_view == prev_view;
      View *prev_main_view = main_window.get_current_main_view();
      bool view_is_main = prev_main_view == prev_view;

      remove_list_entry(thread_entries, prev_widget);
      mod_notebook_remove_page(notebook, page);

      mod_notebook_insert_page(notebook, widget, tab_label, page);
      mod_notebook_set_tab_reorderable(notebook, widget, TRUE);
      mod_notebook_set_tab_label_packing(notebook, widget,
					 TRUE, TRUE, GTK_PACK_START);
      mod_notebook_set_current_page(notebook, page);

      if (view_has_focus)
	{
	  view->grab_focus();
	  main_window.notify_current_focus_view(view);
	}
      else if (view_is_main)
	{
	  main_window.notify_current_main_view(view);
	}
      else
	{
	  // MEMO: mode_notebook_set_current_page(...)Ƥ̤Ȥơ
	  // ܡɥե蘆˰žƤޤäƤʬ
	  // 
	  main_window.notify_current_main_view(prev_main_view);
	  if (prev_focus_view != NULL)
	    prev_focus_view->grab_focus();
	  main_window.notify_current_focus_view(prev_focus_view);
	}
    }

  if (select_tab)
    {
      view->grab_focus();
      main_window.notify_current_focus_view(view);
    }

  update_thread_pane();
  update_thread_tab();
  update_thread_navigator();

  if (OCHUSHA_IS_ICON_LABEL(tab_label))
    g_signal_connect(tab_label, "notify::current-pixbuf",
		     reinterpret_cast<GCallback>(notify_current_pixbuf_cb),
		     this);
}


struct FindBoardArgs
{
  const BulletinBoard *board;
  GtkWidget *view;
};


static gboolean
compare_board_entry(GtkTreeModel *model,
		    GtkTreePath *path, GtkTreeIter *iter,
		    FindBoardArgs *args)
{
  GtkWidget *widget = NULL;
  gtk_tree_model_get(model, iter, VIEW_WIDGET, &widget, -1);
  assert(widget != NULL);

  if (ViewManager::get_board(widget) == args->board)
    {
      args->view = widget;
      return TRUE;
    }

  return FALSE;
}


static GtkWidget *
find_board_entry(GtkListStore *store, const BulletinBoard *board)
{
  FindBoardArgs args = { board, NULL };
  gtk_tree_model_foreach(GTK_TREE_MODEL(store),
		reinterpret_cast<GtkTreeModelForeachFunc>(compare_board_entry),
		&args);
  return args.view;
}


View *
ViewManager::find_board_view(const BulletinBoard *board, bool select_tab)
{
  GtkWidget *widget = find_board_entry(board_entries, board);

  if (widget == NULL)
    return NULL;

  View *current_view = get_current_board_view();
  if (current_view->state != View::NORMAL)
    {
      assert(select_tab);

      current_view->state = View::NORMAL;
    }

  if (select_tab && widget != NULL)
    {
      gint page = mod_notebook_page_num(board_book, widget);
      assert(page != -1);

      mod_notebook_set_current_page(board_book, page);
    }

  return View::get_view(widget);
}


struct FindThreadArgs
{
  const BBSThread *thread;
  GtkWidget *view;
};


static gboolean
compare_thread_entry(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter,
		     FindThreadArgs *args)
{
  GtkWidget *widget = NULL;
  gtk_tree_model_get(model, iter, VIEW_WIDGET, &widget, -1);
  assert(widget != NULL);

  if (ViewManager::get_thread(widget) == args->thread)
    {
      args->view = widget;
      return TRUE;
    }

  return FALSE;
}


static GtkWidget *
find_thread_entry(GtkListStore *store, const BBSThread *thread)
{
  FindThreadArgs args = { thread, NULL };
  gtk_tree_model_foreach(GTK_TREE_MODEL(store),
		reinterpret_cast<GtkTreeModelForeachFunc>(compare_thread_entry),
		&args);
  return args.view;
}


View *
ViewManager::find_thread_view(const BBSThread *thread, bool select_tab)
{
  GtkWidget *widget = find_thread_entry(thread_entries, thread);

  if (widget == NULL)
    return NULL;

  View *current_view = get_current_thread_view();
  if (current_view->state != View::NORMAL)
    {
      assert(select_tab);

      current_view->state = View::NORMAL;
    }

  if (select_tab && widget != NULL)
    {
      gint page = mod_notebook_page_num(thread_book, widget);
      assert(page != -1);

      mod_notebook_set_current_page(thread_book, page);
    }

  return View::get_view(widget);
}


void
ViewManager::close_tab(View *view)
{
  g_return_if_fail(view != NULL);

  GtkWidget *widget = view->get_widget();
  g_return_if_fail(widget != NULL);

  gint page = mod_notebook_page_num(board_book, widget);
  if (page != -1)
    {
      close_nth_board_page(page);
      update_board_pane();
      update_board_tab();
      update_board_navigator();
      return;
    }

  page = mod_notebook_page_num(thread_book, widget);
  g_return_if_fail(page != -1);

  close_nth_thread_page(page);
  update_thread_pane();
  update_thread_tab();
  update_thread_navigator();
}


struct ForeachBoardArgs
{
  ViewManager::ForeachBoardFunc callback;
  void *callback_data;
};


static void
foreach_board_helper(GtkWidget *widget, gpointer data)
{
  BulletinBoard *board = ViewManager::get_board(widget);
  if (board != NULL)
    {
      ForeachBoardArgs *args = static_cast<ForeachBoardArgs *>(data);
      (*args->callback)(widget, board, args->callback_data);
    }
}


void
ViewManager::foreach_board(ForeachBoardFunc callback, void *callback_data)
{
  ForeachBoardArgs args = { callback, callback_data };

  gtk_container_foreach(GTK_CONTAINER(board_book), foreach_board_helper,
			&args);
}


struct ForeachThreadArgs
{
  ViewManager::ForeachThreadFunc callback;
  void *callback_data;
};


static void
foreach_thread_helper(GtkWidget *widget, gpointer data)
{
  BBSThread *thread = ViewManager::get_thread(widget);
  if (thread != NULL)
    {
      ForeachThreadArgs *args = static_cast<ForeachThreadArgs *>(data);
      (*args->callback)(widget, thread, args->callback_data);
    }
}


void
ViewManager::foreach_thread(ForeachThreadFunc callback, void *callback_data)
{
  ForeachThreadArgs args = { callback, callback_data };

  gtk_container_foreach(GTK_CONTAINER(thread_book), foreach_thread_helper,
			&args);
}


void
ViewManager::attach_thread_view_menu(GtkMenu *popup_menu)
{
  g_return_if_fail(GTK_IS_MENU_SHELL(popup_menu));

  GtkMenuShell *shell = GTK_MENU_SHELL(popup_menu);

  GtkWidget *separator = gtk_separator_menu_item_new();
  gtk_widget_show(separator);
  gtk_menu_shell_prepend(shell, separator);

  GtkWidget *menu = gtk_action_create_menu_item(cm_thread_info);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);

  menu = gtk_action_create_menu_item(cm_open_board);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);

  menu = gtk_action_create_menu_item(cm_bookmark_thread);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);

  separator = gtk_separator_menu_item_new();
  gtk_widget_show(separator);
  gtk_menu_shell_prepend(shell, separator);

  menu = gtk_action_create_menu_item(cm_auto_refresh_thread);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);

  menu = gtk_action_create_menu_item(cm_refresh_thread);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);

#if 0
  menu = gtk_action_create_menu_item(go_forward_action);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);

  menu = gtk_action_create_menu_item(go_back_action);
  gtk_widget_show(menu);
  gtk_menu_shell_prepend(shell, menu);
#endif

  menu = gtk_action_create_menu_item(cm_search_thread);
  gtk_widget_show(menu);
  gtk_menu_shell_append(shell, menu);

  separator = gtk_separator_menu_item_new();
  gtk_widget_show(separator);
  gtk_menu_shell_append(shell, separator);

  menu = gtk_action_create_menu_item(cm_reset_thread);
  gtk_widget_show(menu);
  gtk_menu_shell_append(shell, menu);
}


BulletinBoard *
ViewManager::get_board(GtkWidget *view)
{
  return static_cast<BulletinBoard *>(g_object_get_qdata(G_OBJECT(view),
							 bulletin_board_id));
}


BBSThread *
ViewManager::get_thread(GtkWidget *view)
{
  return static_cast<BBSThread *>(g_object_get_qdata(G_OBJECT(view),
						     bbs_thread_id));
}

}
